<?php
/**
 * package.xml parsing class, package.xml version 2.0
 *
 * PHP versions 4 and 5
 *
 * @category   pear
 * @package    PEAR
 * @author     Greg Beaver <cellog@php.net>
 * @copyright  1997-2009 The Authors
 * @license    http://opensource.org/licenses/bsd-license.php New BSD License
 * @version    CVS: $Id: v2.php 276385 2009-02-24 23:46:03Z dufuz $
 * @link       http://pear.php.net/package/PEAR
 * @since      File available since Release 1.4.0a1
 */
/**
 * base xml parser class
 */
require_once kPathToWebSite . kPathToPear.'PEAR/XMLParser.php';
require_once kPathToWebSite . kPathToPear.'PEAR/PackageFile/v2.php';
/**
 * Parser for package.xml version 2.0
 * 
 * @category pear
 * @package PEAR
 * @author Greg Beaver <cellog@php.net>
 * @copyright 1997-2009 The Authors
 * @license http://opensource.org/licenses/bsd-license.php New BSD License
 * @version Release: @PEAR-VER@
 * @link http://pear.php.net/package/PEAR
 * @since Class available since Release 1.4.0a1
 */
class PEAR_PackageFile_Parser_v2 extends PEAR_XMLParser {
	var $_config;
	var $_logger;
	var $_registry;
	function setConfig(&$c) {
		$this->_config = &$c;
		$this->_registry = &$c->getRegistry ();
	}
	function setLogger(&$l) {
		$this->_logger = &$l;
	}
	/**
	 * Unindent given string
	 *
	 * @param string $str
	 *        	The string that has to be unindented.
	 * @return string
	 * @access private
	 */
	function _unIndent($str) {
		// remove leading newlines
		$str = preg_replace ( '/^[\r\n]+/', '', $str );
		// find whitespace at the beginning of the first line
		$indent_len = strspn ( $str, " \t" );
		$indent = substr ( $str, 0, $indent_len );
		$data = '';
		// remove the same amount of whitespace from following lines
		foreach ( explode ( "\n", $str ) as $line ) {
			if (substr ( $line, 0, $indent_len ) == $indent) {
				$data .= substr ( $line, $indent_len ) . "\n";
			} else {
				$data .= $line . "\n";
			}
		}
		return $data;
	}
	
	/**
	 * post-process data
	 *
	 * @param string $data        	
	 * @param string $element
	 *        	element name
	 */
	function postProcess($data, $element) {
		if ($element == 'notes') {
			return trim ( $this->_unIndent ( $data ) );
		}
		return trim ( $data );
	}
	
	/**
	 *
	 * @param
	 *        	string
	 * @param
	 *        	string file name of the package.xml
	 * @param
	 *        	string|false name of the archive this package.xml came from,
	 *        	if any
	 * @param
	 *        	string class name to instantiate and return. This must be
	 *        	PEAR_PackageFile_v2 or
	 *        	a subclass
	 * @return PEAR_PackageFile_v2
	 */
	function &parse($data, $file, $archive = false, $class = 'PEAR_PackageFile_v2') {
		if (PEAR::isError ( $err = parent::parse ( $data, $file ) )) {
			return $err;
		}
		
		$ret = new $class ();
		$ret->encoding = $this->encoding;
		$ret->setConfig ( $this->_config );
		if (isset ( $this->_logger )) {
			$ret->setLogger ( $this->_logger );
		}
		
		$ret->fromArray ( $this->_unserializedData );
		$ret->setPackagefile ( $file, $archive );
		return $ret;
	}
}