<?php

/*
 * Script Name: Numeros a Romanos Class
 * Script URI: http://www.mis-algoritmos.com//convertir-numeros-a-romanos/
 * Description: Permite la conversi�n entre n�meros Romanos y Ar�bigos.
 * Script Version: 1
 * Author: Victor De la Rocha
 * Author URI: http://www.mis-algoritmos.com
 * Script Version: 2
 * Description: Agregada la compatibilidad para PHP 5 y niveles de visibilidad
 * funciones est�ticas y la posibilidad de saber si un n�mero es Romano o no mediante
 * expresiones regulares, en el constructor se obtienen ya las dos posibilidades
 * Author: Reynier De la Rosa
 * Author URI: http://scriptinside.blogspot.com/
 */

class DecimalRomano {

    protected $arabigo;
    protected $romano;

    /*
     * A partir del valor, lo convierte a su valor contrario.
     * Romano a Arabigo
     * Arabigo a Romano
     * @param type $valor
     * @return type
     */

    public function process($valor) {
        if (is_numeric($valor)) {
		  $this->arabigo = intval($valor);
		  $this->romano = DecimalRomano::arabigo2romano($this->arabigo);
		  return $this->romano;
        } else {
		  $this->arabigo = DecimalRomano::romano2arabigo($valor);
		  $this->romano = DecimalRomano::arabigo2romano($this->arabigo);
		  return $this->arabigo;
        }
    }

    public static function esRomano($valor) {
        $patron = '/^M{0,4}(CM|CD|D?C{0,3})(XC|XL|L?X{0,3})(IX|IV|V?I{0,3})$/';
        return (preg_match($patron, $valor)) ? true : false;
    }

    /*
     * Convierte de Romano a Arabigo, retorna un entero.
     */

    public static function romano2arabigo($valor) {
        $valromano = array('I' => CLX_NUM_ONE, 'V' => CLX_NUM_FIVE, 'X' => CLX_NUM_TEN, 'L' => CLX_NUM_50, 'C' => CLX_NUM_HUNDRED, 'D' => CLX_NUM_500, 'M' => CLX_NUM_THOUSAND);
        $resultado = CLX_INIT_ZERO;
        if (!empty($valor)) {
		  for ($i = CLX_INIT_ZERO; $i < strlen($valor) - CLX_NUM_ONE; $i++) {
			 $letra1 = substr($valor, $i, CLX_NUM_ONE);
			 $letra2 = substr($valor, $i + CLX_NUM_ONE, CLX_NUM_ONE);
			 if ($valromano[$letra1] >= $valromano[$letra2]) {
			     $resultado+=$valromano[$letra1];
			 } else {
			     $resultado-=$valromano[$letra1];
			 }
		  }
		  $letra1 = substr($valor, strlen($valor) - CLX_NUM_ONE, CLX_NUM_ONE);
		  $resultado+=$valromano[$letra1];
        }
        return $resultado;
    }

    /*
     * Convierte de Arabigo a Romano, retorna una cadena.
     */

    public static function arabigo2romano($valor) {
        $romanos = array('M', 'CM', 'D', 'CD', 'C', 'XC', 'L', 'XL', 'X', 'IX', 'V', 'IV', 'I');
        $valores = array(CLX_NUM_THOUSAND, CLX_NUM_900, CLX_NUM_500, CLX_NUM_400, CLX_NUM_HUNDRED, CLX_NUM_90, CLX_NUM_50, CLX_NUM_40, CLX_NUM_TEN, CLX_NUM_NINE, CLX_NUM_FIVE, CLX_NUM_FOUR, CLX_NUM_ONE);
        $resultado = '';
        $total = count($romanos);
        for ($i = CLX_INIT_ZERO; $i < $total; $i++) {
		  while ($valor >= $valores[$i]) {
			 $resultado.=$romanos[$i];
			 $valor-=$valores[$i];
		  }
        }
        return $resultado;
    }

}
