<?php

/**
 * General Class
 * For more information contact  support@alenet.com
 * @since        $Date: 2018-09-13 12:50:40 -0400 (Thu, 13 Sep 2018) $
 * @version        $Rev: 10060 $
 * @package		Clientix.Classes
 * @abstract		Date and Time Functions. Based on ADOdb Date Library, part of the ADOdb abstraction library Download: http://php.weblogs.com/adodb_date_time_library
 *
 * CHANGE LOG
 * 2009/04/06 Included label
  ADOdb Date Library, part of the ADOdb abstraction library Download: http://php.weblogs.com/adodb_date_time_library
  PHP native date functions use integer timestamps for computations. Because of this, dates are restricted to the years 1901-2038 on Unix and 1970-2038 on Windows due to integer overflow for dates beyond those years. This library overcomes these limitations by replacing the native function's signed integers (normally 32-bits) with PHP floating point numbers (normally 64-bits).  Dates from 100 A.D. to 3000 A.D. and later have been tested. The minimum is 100 A.D. as <100 will invoke the
  2 => 4 digit year conversion. The maximum is billions of years in the
  future, but this is a theoretical limit as the computation of that year would take too long with the current implementation of adodb_mktime().  This library replaces native functions as follows:
  <pre>
  getdate()  with  adodb_getdate() date()     with  adodb_date() gmdate()   with  adodb_gmdate() mktime()   with  adodb_mktime() gmmktime() with  adodb_gmmktime() </pre>
  The parameters are identical, except that adodb_date() accepts a subset of date()'s field formats. Mktime() will convert from local time to GMT, and date() will convert from GMT to local time, but daylight savings is not handled currently.  This library is independant of the rest of ADOdb, and can be used as standalone code.  PERFORMANCE  For high speed, this library uses the native date functions where possible, and only switches to PHP code when the dates fall outside the 32-bit signed integer range.  GREGORIAN CORRECTION  Pope Gregory shortened October of A.D. 1582 by ten days. Thursday, October 4, 1582 (Julian) was followed immediately by Friday, October 15, 1582 (Gregorian).  Since 0.06, we handle this correctly, so:
  adodb_mktime(0, 0, 0,10,15,1582) - adodb_mktime(0, 0, 0,10,4,1582) == 24 * 3600 (1 day)
  =============================================================================  COPYRIGHT  (c) 2003-2004 John Lim and released under BSD-style license except for code by jackbbs, which includes adodb_mktime, adodb_get_gmt_diff, adodb_is_leap_year
  and originally found at http://www.php.net/manual/en/function.mktime.php
  =============================================================================  BUG REPORTS  These should be posted to the ADOdb forums at  http://phplens.com/lens/lensforum/topics.php?id=4
  =============================================================================  FUNCTION DESCRIPTIONS   FUNCTION adodb_getdate($date=false)  Returns an array containing date information, as getdate(), but supports dates greater than 1901 to 2038   FUNCTION adodb_date($fmt, $timestamp = false)  Convert a timestamp to a formatted local date. If $timestamp is not defined, the current timestamp is used. Unlike the function date(), it supports dates outside the 1901 to 2038 range.  The format fields that adodb_date supports:
  <pre>
  a - 'am' or 'pm' A - 'AM' or 'PM' d - day of the month, 2 digits with leading zeros; i.e. '01' to '31'
  D - day of the week, textual, 3 letters; e.g. 'Fri'
  F - month, textual, long; e.g. 'January'
  g - hour, 12-hour format without leading zeros; i.e. '1' to '12'
  G - hour, 24-hour format without leading zeros; i.e. '0' to '23'
  h - hour, 12-hour format; i.e. '01' to '12'
  H - hour, 24-hour format; i.e. '00' to '23'
  i - minutes; i.e. '00' to '59'
  j - day of the month without leading zeros; i.e. '1' to '31'
  l (uppercase 'L') - day of the week, textual, long; e.g. 'Friday'
  L - boolean for whether it is a leap year; i.e. '0' or '1'
  m - month; i.e. '01' to '12'
  M - month, textual, 3 letters; e.g. 'Jan'
  n - month without leading zeros; i.e. '1' to '12'
  O - Difference to Greenwich time in hours; e.g. '+0200'
  Q - Quarter, as in 1, 2, 3, 4 r - RFC 822 formatted date; e.g. 'Thu, 21 Dec 2000 16:01:07 +0200'
  s - seconds; i.e. '00' to '59'
  S - English ordinal suffix for the day of the month, 2 characters;
  i.e. 'st', 'nd', 'rd' or 'th' t - number of days in the given month; i.e. '28' to '31'
  T - Timezone setting of this machine; e.g. 'EST' or 'MDT'
  U - seconds since the Unix Epoch (January 1 1970 00:00:00 GMT)
  w - day of the week, numeric, i.e. '0' (Sunday) to '6' (Saturday) Y - year, 4 digits; e.g. '1999'
  y - year, 2 digits; e.g. '99'
  z - day of the year; i.e. '0' to '365'
  Z - timezone offset in seconds (i.e. '-43200' to '43200'). The offset for timezones west of UTC is always negative, and for those east of UTC is always positive. </pre>
  Unsupported:
  <pre>
  B - Swatch Internet time I (capital i) - '1' if Daylight Savings Time, '0' otherwise.
  W - ISO-8601 week number of year, weeks starting on Monday  </pre>
  FUNCTION adodb_date2($fmt, $isoDateString = false) Same as adodb_date, but 2nd parameter accepts iso date, eg.  adodb_date2('d-M-Y H:i','2003-12-25 13:01:34');
  FUNCTION adodb_gmdate($fmt, $timestamp = false)  Convert a timestamp to a formatted GMT date. If $timestamp is not defined, the current timestamp is used. Unlike the function date(), it supports dates outside the 1901 to 2038 range.   FUNCTION adodb_mktime($hr, $min, $sec[, $month, $day, $year])  Converts a local date to a unix timestamp.  Unlike the function mktime(), it supports dates outside the 1901 to 2038 range. All parameters are optional.  FUNCTION adodb_gmmktime($hr, $min, $sec [, $month, $day, $year])  Converts a gmt date to a unix timestamp.  Unlike the function gmmktime(), it supports dates outside the 1901 to 2038 range. Differs from gmmktime() in that all parameters
  are currently compulsory.  =============================================================================  NOTES  Useful url for generating test timestamps:
  http://www.4webhelp.net/us/timestamp.php
  Possible future optimizations include  a. Using an algorithm similar to Plauger's in 'The Standard C Library' (page 428, xttotm.c _Ttotm() function). Plauger's algorithm will not work outside 32-bit signed range, so i decided not to implement it.  b. Iterate over a block of years (say 12) when searching for the correct year.  c. Implement daylight savings, which looks awfully complicated, see http://webexhibits.org/daylightsaving/
  CHANGELOG  - 21 Dec 2004 017 In adodb_getdate(), the timestamp was accidentally converted to gmt when $is_gmt is false. Also adodb_mktime(0, 0,0) did not work properly. Both fixed thx Mauro.  - 17 Nov 2004 016 Removed intval typecast in adodb_mktime() for secs, allowing:
  adodb_mktime(0, 0,0 + 2236672153,1,1,1934);
  Suggested by Ryan.  - 18 July 2004 015 All params in adodb_mktime were formerly compulsory. Now only the hour, min, secs is compulsory. This brings it more in line with mktime (still not identical).  - 23 June 2004 014  Allow you to define your own daylights savings function, adodb_daylight_sv. If the function is defined (somewhere in an include), then you can correct for daylights savings.  In this example, we apply daylights savings in June or July, adding one hour. This is extremely unrealistic as it does not take into account time-zone, geographic location, current year.  function adodb_daylight_sv(&$arr, $is_gmt) {
  if ($is_gmt=== true) return;
  $m = $arr['mon'];
  if ($m == 6 || $m == 7) $arr['hours'] += 1;
  }
  This is only called by adodb_date() and not by adodb_mktime().  The format of $arr is Array ( [seconds] => 0
  [minutes] => 0
  [hours] => 0
  [mday] => 1      // day of month, eg 1st day of the month
  [mon] => 2       // month (eg. Feb)
  [year] => 2102
  [yday] => 31     // days in current year
  [leap] =>        // true if leap year
  [ndays] => 28    // no of days in current month
  )
  - 28 Apr 2004 013 Fixed adodb_date to properly support $is_gmt. Thx to Dimitar Angelov.  - 20 Mar 2004 012 Fixed month calculation error in adodb_date. 2102-June-01 appeared as 2102-May-32.  - 26 Oct 2003 011 Because of daylight savings problems (some systems apply daylight savings to January!!!), changed adodb_get_gmt_diff() to ignore daylight savings.
  - 9 Aug 2003 010 Fixed bug with dates after 2038 See http://phplens.com/lens/lensforum/msgs.php?id=6980
  - 1 July 2003 009 Added support for Q (Quarter). Added adodb_date2(), which accepts ISO date in 2nd param  - 3 March 2003 008 Added support for 'S' adodb_date() format char. Added constant ADODB_ALLOW_NEGATIVE_TS if you want PHP to handle negative timestamps between 1901 to 1969
  - 27 Feb 2003 007 All negative numbers handled by adodb now because of RH 7.3+ problems. See http://bugs.php.net/bug.php?id=20048&edit=2
  - 4 Feb 2003 006 Fixed a typo, 1852 changed to 1582! This means that pre-1852 dates are now correctly handled.  - 29 Jan 2003 005  Leap year checking differs under Julian calendar (pre 1582). Also
  leap year code optimized by checking for most common case first.  We also handle month overflow correctly in mktime (eg month set to 13).  Day overflow for less than one month's days is supported.  - 28 Jan 2003 0.04  Gregorian correction handled. In PHP5, we might throw an error if
  mktime uses invalid dates around 5-14 Oct 1582 Released with ADOdb 3.10. Added limbo 5-14 Oct 1582 check, when we set to 15 Oct 1582  - 27 Jan 2003 003  Fixed some more month problems due to gmt issues. Added constant ADODB_DATE_VERSION. Fixed calculation of days since start of year for <1970
  - 27 Jan 2003 0.02  Changed _adodb_getdate() to inline leap year checking for better performance. Fixed problem with time-zones west of GMT +0000.  - 24 Jan 2003 0.01  First implementation. */
/* Initialization */
/*
  Version Number */
define('ADODB_DATE_VERSION', CLX_NUM_0_PT_17);
/*
  We check for Windows as only +ve ints are accepted as dates on Windows.  Apparently this problem happens also with Linux, RH 73 and later!  glibc-2.2.5-34 and greater has been changed to return -1 for dates <
  1970  This used to work.  The problem exists with RedHat 73 and 80 echo (mktime(0, 0, 0, 1, 1, 1960));  // prints -1
  References:
  http://bugs.php.net/bug.php?id=20048&edit=2
  http://lists.debian.org/debian-glibc/2002/debian-glibc-200205/msg00010.html
 */
if (!defined('ADODB_ALLOW_NEGATIVE_TS'))
    define('ADODB_NO_NEGATIVE_TS', CLX_NUM_ONE);

function adodb_date_test_date($y1, $m) {
    $t = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $m, CLX_NUM_13, $y1);
    if ($y1 . '-' . $m . '-13 00:00:00' != adodb_date('Y-n-d H:i:s', $t)) {
        $contenidoHTML = '<b>' . $y1 . ' error</b><br>';
        print $contenidoHTML;
        return false;
    }
    return true;
}

/**
  Test Suite */
function adodb_date_test() {
    $contenidoHTML = '';
    error_reporting(E_ALL);
    $contenidoHTML.= '<h4>Testing adodb_date and adodb_mktime. version=' . ADODB_DATE_VERSION . '</h4>';
    UserResponse::setRunTimeLimit(CLX_NUM_ZERO);
    $fail = false;
    // This flag disables calling of PHP native functions, so we can properly test the code
    if (!defined('ADODB_TEST_DATES'))
        define('ADODB_TEST_DATES', CLX_NUM_ONE);
    $t = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO);
    if (!(adodb_date('Y-m-d') == date('Y-m-d')))
        $contenidoHTML.= 'Error in ' . adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO) . '<br>';
    $t = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_SIX, CLX_NUM_ONE, CLX_NUM_2102);
    if (!(adodb_date('Y-m-d', $t) === '2102-06-01'))
        $contenidoHTML.= 'Error in ' . adodb_date('Y-m-d', $t) . '<br>';
    $t = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_TWO, CLX_NUM_ONE, CLX_NUM_2102);
    if (!(adodb_date('Y-m-d', $t) === '2102-02-01'))
        $contenidoHTML.= 'Error in ' . adodb_date('Y-m-d', $t) . '<br>';
    $contenidoHTML.= '<p>Testing gregorian <=> julian conversion<p>';
    $t = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_TEN, CLX_NUM_11, CLX_NUM_1492);
    if (!(adodb_date('D Y-m-d', $t) === 'Fri 1492-10-11'))
        $contenidoHTML.= 'Error in Columbus landing<br>';
    $t = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_TWO, CLX_NUM_29, CLX_NUM_1500);
    if (!(adodb_date('Y-m-d', $t) === '1500-02-29'))
        $contenidoHTML.= 'Error in julian leap years<br>';
    $t = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_TWO, CLX_NUM_29, CLX_NUM_1700);
    if (!(adodb_date('Y-m-d', $t) === '1700-03-01'))
        $contenidoHTML.= 'Error in gregorian leap years<br>';
    $contenidoHTML.= adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_TEN, CLX_NUM_FOUR, CLX_NUM_1582) . ' ';
    $contenidoHTML.= adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_TEN, CLX_NUM_15, CLX_NUM_1582);
    $diff = (adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_TEN, CLX_NUM_15, CLX_NUM_1582) - adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_TEN, CLX_NUM_FOUR, CLX_NUM_1582));
    if ($diff != CLX_NUM_3600 * CLX_NUM_24)
        $contenidoHTML.= ' <b>Error in gregorian correction = ' . ($diff / CLX_360_DAYS0 / CLX_NUM_24) . ' days </b><br>';
    $contenidoHTML.= ' 15 Oct 1582, Fri=' . (adodb_dow(CLX_NUM_1582, CLX_NUM_TEN, CLX_NUM_15) == CLX_NUM_FIVE ? 'Fri' : '<b>Error</b>') . '<br>';
    $contenidoHTML.= ' 4 Oct 1582, Thu=' . (adodb_dow(CLX_NUM_1582, CLX_NUM_TEN, CLX_NUM_FOUR) == CLX_NUM_FOUR ? 'Thu' : '<b>Error</b>') . '<br>';
    $contenidoHTML.= '<p>Testing overflow<p>';
    $t = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_THREE, CLX_NUM_33, CLX_NUM_1965);
    if (!(adodb_date('Y-m-d', $t) === '1965-04-02'))
        $contenidoHTML.= 'Error in day overflow 1 <br>';
    $t = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_FOUR, CLX_NUM_33, CLX_NUM_1971);
    if (!(adodb_date('Y-m-d', $t) === '1971-05-03'))
        $contenidoHTML.= 'Error in day overflow 2 <br>';
    $t = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ONE, CLX_NUM_60, CLX_NUM_1965);
    if (!(adodb_date('Y-m-d', $t) === '1965-03-01'))
        $contenidoHTML.= 'Error in day overflow 3 ' . adodb_date('Y-m-d', $t) . ' <br>';
    $t = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_12, CLX_NUM_32, CLX_NUM_1965);
    if (!(adodb_date('Y-m-d', $t) === '1966-01-01'))
        $contenidoHTML.= 'Error in day overflow 4 ' . adodb_date('Y-m-d', $t) . ' <br>';
    $t = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_12, CLX_NUM_63, CLX_NUM_1965);
    if (!(adodb_date('Y-m-d', $t) === '1966-02-01'))
        $contenidoHTML.= 'Error in day overflow 5 ' . adodb_date('Y-m-d', $t) . ' <br>';
    $t = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_13, CLX_NUM_THREE, CLX_NUM_1965);
    if (!(adodb_date('Y-m-d', $t) === '1966-01-03'))
        $contenidoHTML.= 'Error in mth overflow 1 <br>';
    $contenidoHTML.= 'Testing 2-digit => 4-digit year conversion<p>';
    if (adodb_year_digit_check(CLX_NUM_ZERO) != CLX_NUM_2000)
        $contenidoHTML.= 'Err 2-digit 2000<br>';
    if (adodb_year_digit_check(CLX_NUM_10) != CLX_NUM_2010)
        $contenidoHTML.= 'Err 2-digit 2010<br>';
    if (adodb_year_digit_check(CLX_NUM_20) != CLX_NUM_2020)
        $contenidoHTML.= 'Err 2-digit 2020<br>';
    if (adodb_year_digit_check(CLX_NUM_30) != CLX_NUM_2030)
        $contenidoHTML.= 'Err 2-digit 2030<br>';
    if (adodb_year_digit_check(CLX_NUM_40) != CLX_NUM_1940)
        $contenidoHTML.= 'Err 2-digit 1940<br>';
    if (adodb_year_digit_check(CLX_NUM_50) != CLX_NUM_1950)
        $contenidoHTML.= 'Err 2-digit 1950<br>';
    if (adodb_year_digit_check(CLX_NUM_90) != CLX_NUM_1990)
        $contenidoHTML.= 'Err 2-digit 1990<br>';
    // Test string formating
    $contenidoHTML.= '<p>Testing date formating</p>';
    $fmt = '\d\a\t\e T Y-m-d H:i:s a A d D F g G h H i j l L m M n O \R\F\C822 r s t U w y Y z Z 2003';
    $s1 = date($fmt, CLX_NUM_ZERO);
    $s2 = adodb_date($fmt, CLX_NUM_ZERO);
    if ($s1 != $s2) {
        $contenidoHTML.= ' date() 0 failed<br>' . $s1 . '<br>' . $s2 . '<br>';
    }
    flush();
    for ($i = CLX_NUM_HUNDRED; --$i > CLX_NUM_ZERO;) {
        $ts = CLX_NUM_3600 * ((rand() % CLX_NUM_60000) + (rand() % CLX_NUM_60000)) + (rand() % CLX_NUM_60000);
        $s1 = date($fmt, $ts);
        $s2 = adodb_date($fmt, $ts);
        if (($s1) != ($s2)) {
		  for ($j = CLX_NUM_ZERO, $k = strlen($s1); $j < $k; $j++) {
			 if ($s1[$j] != $s2[$j]) {
			     $contenidoHTML.= substr($s1, $j) . ' ';
			     break;
			 }
		  }
		  $contenidoHTML.= '<b>Error date(): ' . $ts . '<br><pre>&nbsp; ' . singleQuote($s1) . ' (date len=' . singleQuote(strlen($s1)) . ')&nbsp; ' . singleQuote($s2) . ' (adodb_date len=' . strlen($s2) . ')</b></pre><br>';
		  $fail = true;
        }
        $a1 = getdate($ts);
        $a2 = adodb_getdate($ts);
        $rez = array_diff($a1, $a2);
        if (sizeof($rez) > CLX_NUM_ZERO) {
		  $contenidoHTML.= '<b>Error getdate() ' . $ts . '</b><br>';
		  $contenidoHTML.=print_r($a1, true);
		  $contenidoHTML.= '<br>';
		  $contenidoHTML.=print_r($a2, true);
		  $contenidoHTML.= '<p>';
		  $fail = true;
        }
    }
    // Test generation of dates outside 1901-2038
    $contenidoHTML.= '<p>Testing random dates between 100 and 4000</p>';
    ClxFreeReturn( adodb_date_test_date(CLX_NUM_HUNDRED, CLX_NUM_ONE));

    for ($i = CLX_NUM_HUNDRED; --$i >= CLX_INIT_ZERO;) {
        $y1 = CLX_NUM_HUNDRED + rand(CLX_NUM_ZERO, CLX_NUM_1970 - CLX_NUM_HUNDRED);
        $m = rand(CLX_NUM_ONE, CLX_NUM_12);
        ClxFreeReturn(adodb_date_test_date($y1, $m));
        $y1 = CLX_NUM_3000 - rand(CLX_NUM_ZERO, CLX_NUM_3000 - CLX_NUM_1970);
        ClxFreeReturn(adodb_date_test_date($y1, $m));
    }
    $contenidoHTML.= '<p>';
    $start = CLX_NUM_1960 + rand(CLX_NUM_ZERO, CLX_NUM_TEN);
    $yrs = CLX_NUM_12;
    $i = CLX_NUM_365_PT_25 * CLX_NUM_86400 * ($start - CLX_NUM_1970);
    $offset = CLX_NUM_36000 + rand(CLX_NUM_TEN_THOUSAND, CLX_NUM_60000);
    $max = CLX_365_DAYS * $yrs * CLX_NUM_86400;
    $lastyear = CLX_INIT_ZERO;
    // we generate a timestamp, convert it to a date, and convert it back to a timestamp
    // and check if the roundtrip broke the original timestamp value.
    $contenidoHTML.= 'Testing ' . $start . ' to ' . ($start + $yrs) . ', or ' . $max . ' seconds, offset=' . $offset . ': ';
    $cnt = CLX_INIT_ZERO;
    for ($max += $i; $i < $max; $i += $offset) {
        $ret = adodb_date('m,d,Y,H,i,s', $i);
        $arr = explode(',', $ret);
        if ($lastyear != $arr[CLX_NUM_TWO]) {
		  $lastyear = $arr[CLX_NUM_TWO];
		  $contenidoHTML.= ' ' . $lastyear . ' ';
		  flush();
        }
        $newi = adodb_mktime($arr[CLX_NUM_THREE], $arr[CLX_NUM_FOUR], $arr[CLX_NUM_FIVE], $arr[CLX_NUM_ZERO], $arr[CLX_NUM_ONE], $arr[CLX_NUM_TWO]);
        if ($i != $newi) {
		  $contenidoHTML.= 'Error at ' . $i . ', adodb_mktime returned ' . $newi . ' (' . $ret . ')';
		  $fail = true;
		  break;
        }
        $cnt += CLX_NUM_ONE;
    }
    $contenidoHTML.= 'Tested ' . $cnt . ' dates<br>';
    if (empty($fail))
        $contenidoHTML.= '<p>Passed !</p>';
    else
        $contenidoHTML.= '<p><b>Failed</b> :-(</p>';
    echo $contenidoHTML;
}

/**
  Returns day of week, 0 = Sunday,... 6=Saturday. Algorithm from PEAR::Date_Calc
 */
function adodb_dow($year, $month, $day) {
    /*
      Pope Gregory removed 10 days - October 5 to October 14 - from the year 1582 and proclaimed that from that time onwards 3 days would be dropped from the calendar every 400 years.  Thursday, October 4, 1582 (Julian) was followed immediately by Friday, October 15, 1582 (Gregorian). */
    if ($year <= CLX_NUM_1582) {
        if ($year < CLX_NUM_1582 ||
			 ($year == CLX_NUM_1582 && ($month < CLX_NUM_TEN || ($month == CLX_NUM_TEN && $day < CLX_NUM_15))))
		  $greg_correction = CLX_NUM_THREE;
        else
		  $greg_correction = CLX_INIT_ZERO;
    } else
        $greg_correction = CLX_INIT_ZERO;
    if ($month > CLX_NUM_TWO)
        $month -= CLX_NUM_TWO;
    else {
        $month += CLX_NUM_TEN;
        $year--;
    }
    $day = ( floor((CLX_NUM_13 * $month - CLX_NUM_ONE) / CLX_NUM_FIVE) +
		  $day + ($year % CLX_NUM_HUNDRED) + floor(($year % CLX_NUM_HUNDRED) / CLX_NUM_FOUR) +
		  floor(($year / CLX_NUM_HUNDRED) / CLX_NUM_FOUR) - CLX_NUM_TWO *
		  floor($year / CLX_NUM_HUNDRED) + CLX_NUM_77);
    return (($day - CLX_NUM_7 * floor($day / CLX_NUM_7))) + $greg_correction;
}

/**
  Checks for leap year, returns true if it is. No 2-digit year check. Also
  handles julian calendar correctly. */
function _adodb_is_leap_year($year) {
    if ($year % CLX_NUM_FOUR != CLX_NUM_ZERO)
        return false;
    if ($year % CLX_NUM_400 == CLX_NUM_ZERO) {
        return true;
        // if gregorian calendar (>1582), century not-divisible by 400 is not leap
    } else if ($year > CLX_NUM_1582 && $year % CLX_NUM_HUNDRED == CLX_NUM_ZERO) {
        return false;
    }
    return true;
}

/**
  checks for leap year, returns true if it is. Has 2-digit year check
 */
function adodb_is_leap_year($year) {
    return _adodb_is_leap_year(adodb_year_digit_check($year));
}

/**
  Fix 2-digit years. Works for any century. Assumes that if 2-digit is more than 30 years in future, then previous century.
 */
function adodb_year_digit_check($y) {
    if ($y < CLX_NUM_HUNDRED) {
        $yr = (integer) date('Y');
        $century = (integer) ($yr / CLX_NUM_HUNDRED);
        if ($yr % CLX_NUM_HUNDRED > CLX_NUM_50) {
		  $c1 = $century + CLX_NUM_ONE;
		  $c0 = $century;
        } else {
		  $c1 = $century;
		  $c0 = $century - CLX_NUM_ONE;
        }
        $c1 *= CLX_NUM_HUNDRED;
        // if 2-digit year is less than 30 years in future, set it to this century
        // otherwise if more than 30 years in future, then we set 2-digit year to the prev century.
        if (($y + $c1) < $yr + CLX_NUM_30)
		  $y = $y + $c1;
        else
		  $y = $y + $c0 * CLX_NUM_HUNDRED;
    }
    return $y;
}

/**
  get local time zone offset from GMT */
function adodb_get_gmt_diff() {
    static $TZ;
    if (isset($TZ))
        return $TZ;
    if (version_compare(PHP_VERSION, '5.1.0') >= CLX_NUM_ZERO) {
        $TZ = mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ONE, CLX_NUM_TWO, CLX_NUM_1970) - gmmktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ONE, CLX_NUM_TWO, CLX_NUM_1970);
    } else {
        $TZ = mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ONE, CLX_NUM_TWO, CLX_NUM_1970, CLX_NUM_ZERO) - gmmktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ONE, CLX_NUM_TWO, CLX_NUM_1970, CLX_NUM_ZERO);
    }
    return $TZ;
}

/**
  Returns an array with date info. */
function adodb_getdate($d = false, $fast = false) {
    if (empty($d))
        return getdate();
    if (!defined('ADODB_TEST_DATES')) {
        if ((abs($d) <= CLX_NUM_0x7FFFFFFF)) { // check if number in 32-bit signed range
		  if (!defined('ADODB_NO_NEGATIVE_TS') || $d >= CLX_NUM_ZERO) // if windows, must be +ve integer
			 return getdate($d);
        }
    }
    return _adodb_getdate($d);
}

/**
  Low-level function that returns the getdate() array. We have a special $fast flag, which if set to true, will return fewer array values,
  and is much faster as it does not calculate dow, etc. */
function _adodb_getdate($origd = false, $fast = false, $is_gmt = false) {
    $d = $origd - ($is_gmt ? CLX_NUM_ZERO : adodb_get_gmt_diff());
    $_day_power = CLX_NUM_86400;
    $_hour_power = CLX_NUM_3600;
    $_min_power = CLX_NUM_60;
    if ($d < -CLX_NUM_12219321600)
        $d -= CLX_NUM_86400 * CLX_NUM_TEN; // if 15 Oct 1582 or earlier, gregorian correction
    $_month_table_normal = array('', CLX_NUM_31, CLX_NUM_28, CLX_NUM_31, CLX_NUM_30, CLX_NUM_31, CLX_NUM_30, CLX_NUM_31, CLX_NUM_31, CLX_NUM_30, CLX_NUM_31, CLX_NUM_30, CLX_NUM_31);
    $_month_table_leaf = array('', CLX_NUM_31, CLX_NUM_29, CLX_NUM_31, CLX_NUM_30, CLX_NUM_31, CLX_NUM_30, CLX_NUM_31, CLX_NUM_31, CLX_NUM_30, CLX_NUM_31, CLX_NUM_30, CLX_NUM_31);
    $d366 = $_day_power * CLX_366_DAYS;
    $d365 = $_day_power * CLX_365_DAYS;
    if ($d < CLX_NUM_ZERO) {
        if (!empty($is_gmt))
		  $origd = $d;
        // The valid range of a 32bit signed timestamp is typically from
        // Fri, 13 Dec 1901 20:45:54 GMT to Tue, 19 Jan 2038 03:14:07 GMT
        for ($a = CLX_NUM_1970; --$a >= CLX_INIT_ZERO;) {
		  $lastd = $d;
		  if ($leaf = _adodb_is_leap_year($a))
			 $d += $d366;
		  else
			 $d += $d365;
		  if ($d >= CLX_NUM_ZERO) {
			 $year = $a;
			 break;
		  }
        }
        $secsInYear = CLX_NUM_86400 * ($leaf ? CLX_366_DAYS : CLX_365_DAYS) + $lastd;
        $d = $lastd;
        $mtab = ($leaf) ? $_month_table_leaf : $_month_table_normal;
        for ($a = CLX_NUM_13; --$a > CLX_NUM_ZERO;) {
		  $lastd = $d;
		  $d += $mtab[$a] * $_day_power;
		  if ($d >= CLX_NUM_ZERO) {
			 $month = $a;
			 $ndays = $mtab[$a];
			 break;
		  }
        }
        $d = $lastd;
        $day = $ndays + ceil(($d + CLX_NUM_ONE ) / ($_day_power));
        $d += ($ndays - $day + CLX_NUM_ONE) * $_day_power;
        $hour = floor($d / $_hour_power);
    } else {
        for ($a = CLX_NUM_1970;; $a++) {
		  $lastd = $d;
		  if ($leaf = _adodb_is_leap_year($a))
			 $d -= $d366;
		  else
			 $d -= $d365;
		  if ($d < CLX_NUM_ZERO) {
			 $year = $a;
			 break;
		  }
        }
        $secsInYear = $lastd;
        $d = $lastd;
        $mtab = ($leaf) ? $_month_table_leaf : $_month_table_normal;
        for ($a = CLX_NUM_ONE; $a <= CLX_NUM_12; $a++) {
		  $lastd = $d;
		  $d -= $mtab[$a] * $_day_power;
		  if ($d < CLX_NUM_ZERO) {
			 $month = $a;
			 $ndays = $mtab[$a];
			 break;
		  }
        }
        $d = $lastd;
        $day = ceil(($d + CLX_NUM_ONE) / $_day_power);
        $d = $d - ($day - CLX_NUM_ONE) * $_day_power;
        $hour = floor($d / $_hour_power);
    }
    $d -= $hour * $_hour_power;
    $min = floor($d / $_min_power);
    $secs = $d - $min * $_min_power;
    if (!empty($fast)) {
        return array('seconds' => $secs,
		  'minutes' => $min,
		  'hours' => $hour,
		  'mday' => $day,
		  'mon' => $month,
		  'year' => $year,
		  'yday' => floor($secsInYear / $_day_power),
		  'leap' => $leaf,
		  'ndays' => $ndays
        );
    }
    $dow = adodb_dow($year, $month, $day);
    return array('seconds' => $secs,
        'minutes' => $min,
        'hours' => $hour,
        'mday' => $day,
        'wday' => $dow,
        'mon' => $month,
        'year' => $year,
        'yday' => floor($secsInYear / $_day_power),
        'weekday' => gmdate('l', $_day_power * (CLX_NUM_THREE + $dow)),
        'month' => gmdate('F', mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $month, CLX_NUM_TWO, CLX_NUM_1971)),
        CLX_NUM_ZERO => $origd
    );
}

function adodb_gmdate($fmt, $d = false) {
    return adodb_date($fmt, $d, true);
}

// accepts unix timestamp and iso date format in $d
function adodb_date2($fmt, $d = false, $is_gmt = false) {
    if (!empty($d)) {
        if (!preg_match('|^([0-9]{4})[-/\.]?([0-9]{1,2})[-/\.]?([0-9]{1,2})[ -]?(([0-9]{1,2}):?([0-9]{1,2}):?([0-9\.]{1,4}))?|', ($d), $rr))
		  return adodb_date($fmt, false, $is_gmt);
        if ($rr[CLX_NUM_ONE] <= CLX_NUM_HUNDRED && $rr[CLX_NUM_TWO] <= CLX_NUM_ONE)
		  return adodb_date($fmt, false, $is_gmt);
        // h-m-s-MM-DD-YY
        if (!isset($rr[CLX_NUM_FIVE]))
		  $d = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $rr[CLX_NUM_TWO], $rr[CLX_NUM_THREE], $rr[CLX_NUM_ONE]);
        else
		  $d = adodb_mktime($rr[CLX_NUM_FIVE], $rr[CLX_NUM_SIX], $rr[CLX_NUM_7], $rr[CLX_NUM_TWO], $rr[CLX_NUM_THREE], $rr[CLX_NUM_ONE]);
    }
    return adodb_date($fmt, $d, $is_gmt);
}

/**
  Return formatted date based on timestamp $d */
function adodb_date($fmt, $d = false, $is_gmt = false) {
    static $daylight;
    if (empty($d))
        return ($is_gmt) ? gmdate($fmt) : date($fmt);
    if (!defined('ADODB_TEST_DATES')) {
        if ((abs($d) <= CLX_NUM_0x7FFFFFFF)) { // check if number in 32-bit signed range
		  if (!defined('ADODB_NO_NEGATIVE_TS') || $d >= CLX_NUM_ZERO) // if windows, must be +ve integer
			 return ($is_gmt) ? gmdate($fmt, $d) : date($fmt, $d);
        }
    }
    $_day_power = CLX_NUM_86400;
    $arr = _adodb_getdate($d, true, $is_gmt);
    if (!isset($daylight)) {
        $daylight = function_exists('adodb_daylight_sv');
    }
    if (!empty($daylight)) {
        adodb_daylight_sv($arr, $is_gmt);
    }
    $year = $arr['year'];
    $month = $arr['mon'];
    $day = $arr['mday'];
    $hour = $arr['hours'];
    $min = $arr['minutes'];
    $secs = $arr['seconds'];
    $max = strlen($fmt);
    $dates = '';
    /*
      at this point, we have the following integer vars to manipulate:
      $year, $month, $day, $hour, $min, $secs */
    for ($i = CLX_INIT_ZERO; $i < $max; $i++) {
        switch ($fmt[$i]) {
		  case 'T': $dates .= date('T');
			 break;
		  // YEAR
		  case 'L': $dates .= $arr['leap'] ? '1' : '0';
			 break;
		  case 'r': // Thu, 21 Dec 2000 16:01:07 +0200
			 $dates .= gmdate('D', $_day_power * (CLX_NUM_THREE + adodb_dow($year, $month, $day))) . ', ' . ($day < CLX_NUM_TEN ? ' ' . $day : $day) . ' ' . date('M', mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $month, CLX_NUM_TWO, CLX_NUM_1971)) . ' ' . $year . ' ';
			 if ($hour < CLX_NUM_TEN)
			     $dates .= '0' . $hour;
			 else
			     $dates .= $hour;
			 if ($min < CLX_NUM_TEN)
			     $dates .= ':0' . $min;
			 else
			     $dates .= ':' . $min;
			 if ($secs < CLX_NUM_TEN)
			     $dates .= ':0' . $secs;
			 else
			     $dates .= ':' . $secs;
			 $gmt = adodb_get_gmt_diff();
			 $dates .= sprintf(' %s%04d', ($gmt < CLX_NUM_ZERO) ? '+' : '-', abs($gmt) / CLX_NUM_36);
			 break;
		  case 'Y': $dates .= $year;
			 break;
		  case 'y': $dates .= substr($year, strlen($year) - CLX_NUM_TWO, CLX_NUM_TWO);
			 break;
		  // MONTH
		  case 'm': if ($month < CLX_NUM_TEN)
			     $dates .= '0' . $month;
			 else
			     $dates .= $month;
			 break;
		  case 'Q': $dates .= ($month + CLX_NUM_THREE) >> CLX_NUM_TWO;
			 break;
		  case 'n': $dates .= $month;
			 break;
		  case 'M': $dates .= date('M', mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $month, CLX_NUM_TWO, CLX_NUM_1971));
			 break;
		  case 'F': $dates .= date('F', mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $month, CLX_NUM_TWO, CLX_NUM_1971));
			 break;
		  // DAY
		  case 't': $dates .= $arr['ndays'];
			 break;
		  case 'z': $dates .= $arr['yday'];
			 break;
		  case 'w': $dates .= adodb_dow($year, $month, $day);
			 break;
		  case 'l': $dates .= gmdate('l', $_day_power * (CLX_NUM_THREE + adodb_dow($year, $month, $day)));
			 break;
		  case 'D': $dates .= gmdate('D', $_day_power * (CLX_NUM_THREE + adodb_dow($year, $month, $day)));
			 break;
		  case 'j': $dates .= $day;
			 break;
		  case 'd': if ($day < CLX_NUM_TEN)
			     $dates .= '0' . $day;
			 else
			     $dates .= $day;
			 break;
		  case 'S':
			 $d10 = $day % CLX_NUM_TEN;
			 if ($d10 == CLX_NUM_ONE)
			     $dates .= 'st';
			 else if ($d10 == CLX_NUM_TWO && $day != CLX_NUM_12)
			     $dates .= 'nd';
			 else if ($d10 == CLX_NUM_THREE)
			     $dates .= 'rd';
			 else
			     $dates .= 'th';
			 break;
		  // HOUR
		  case 'Z':
			 $dates .= ($is_gmt) ? CLX_NUM_ZERO : -adodb_get_gmt_diff();
			 break;
		  case 'O':
			 $gmt = ($is_gmt) ? CLX_NUM_ZERO : adodb_get_gmt_diff();
			 $dates .= sprintf('%s%04d', ($gmt < CLX_NUM_ZERO) ? '+' : '-', abs($gmt) / CLX_NUM_36);
			 break;
		  case 'H':
			 if ($hour < CLX_NUM_TEN)
			     $dates .= '0' . $hour;
			 else
			     $dates .= $hour;
			 break;
		  case 'h':
			 if ($hour > CLX_NUM_12)
			     $hh = $hour - CLX_NUM_12;
			 else {
			     if ($hour == CLX_NUM_ZERO)
			         $hh = '12';
			     else
			         $hh = $hour;
			 }
			 if ($hh < CLX_NUM_TEN)
			     $dates .= '0' . $hh;
			 else
			     $dates .= $hh;
			 break;
		  case 'G':
			 $dates .= $hour;
			 break;
		  case 'g':
			 if ($hour > CLX_NUM_12)
			     $hh = $hour - CLX_NUM_12;
			 else {
			     if ($hour == CLX_NUM_ZERO)
			         $hh = '12';
			     else
			         $hh = $hour;
			 }
			 $dates .= $hh;
			 break;
		  // MINUTES
		  case 'i': if ($min < CLX_NUM_TEN)
			     $dates .= '0' . $min;
			 else
			     $dates .= $min;
			 break;
		  // SECONDS
		  case 'U': $dates .= $d;
			 break;
		  case 's': if ($secs < CLX_NUM_TEN)
			     $dates .= '0' . $secs;
			 else
			     $dates .= $secs;
			 break;
		  // AM/PM
		  // Note 00:00 to 11:59 is AM, while 12:00 to 23:59 is PM
		  case 'a':
			 if ($hour >= CLX_NUM_12)
			     $dates .= 'pm';
			 else
			     $dates .= 'am';
			 break;
		  case 'A':
			 if ($hour >= CLX_NUM_12) {
			     $dates .= 'PM';
			 } else {
			     $dates .= 'AM';
			 }
			 break;
		  default:
			 $dates .= $fmt[$i];
			 break;
		  // ESCAPE
		  case '\\':
			 if (($i + CLX_NUM_ONE) < $max) {
			     $dates .= $fmt[($i + CLX_NUM_ONE)];
			 }
			 break;
        }
    }
    return $dates;
}

/**
  Returns a timestamp given a GMT/UTC time.
  Note that $is_dst is not implemented and is ignored. */
function adodb_gmmktime($hr, $min, $sec, $mon = false, $day = false, $year = false, $is_dst = false) {
    return adodb_mktime($hr, $min, $sec, $mon, $day, $year, $is_dst, true);
}

/**
  Return a timestamp given a local time. Originally by jackbbs. Note that $is_dst is not implemented and is ignored.  Not a very fast algorithm - O(n) operation. Could be optimized to O(1).   Please note than $hr, $min, $sec was not false review later.*****************
  some missing argument error on clientix. */
function adodb_mktime($hr = false, $min = false, $sec = false, $mon = false, $day = false, $year = false, $is_dst = false, $is_gmt = false) {
    if (!defined('ADODB_TEST_DATES')) {
        if (empty($mon)) {
		  return $is_gmt ? gmmktime($hr, $min, $sec) : mktime($hr, $min, $sec);
        }
        //When interval of months are > 238 then this need test for 2038>
        $test_normal = true;
        if ($mon > CLX_NUM_ONE) {
		  if (($year + floor(($mon / CLX_NUM_12))) >= CLX_NUM_2038) {
			 $test_normal = false;
		  }
        }
        // for windows, we don't check 1970 because with timezone differences,
        // 1 Jan 1970 could generate negative timestamp, which is illegal
        if ((CLX_NUM_1971 < $year && $year < CLX_NUM_2038 || !defined('ADODB_NO_NEGATIVE_TS') && (CLX_NUM_1901 < $year && $year < CLX_NUM_2038)) && $test_normal) {
		  return $is_gmt ? gmmktime($hr, $min, $sec, $mon, $day, $year) :
			     mktime($hr, $min, $sec, $mon, $day, $year);
        }
    }
    $gmt_different = ($is_gmt) ? CLX_NUM_ZERO : adodb_get_gmt_diff();
    /*
      // disabled because some people place large values in $sec.
      // however we need it for $mon because we use an array...
      $hr = intval($hr);
      $min = intval($min);
      $sec = intval($sec);
     */
    $mon = intval($mon);
    $day = intval($day);
    $year = intval($year);
    $year = adodb_year_digit_check($year);
    if ($mon > CLX_NUM_12) {
        $y = floor($mon / CLX_NUM_12);
        $year += $y;
        $mon -= $y * CLX_NUM_12;
    }
    $_day_power = CLX_NUM_86400;
    $_hour_power = CLX_NUM_3600;
    $_min_power = CLX_NUM_60;
    $_month_table_normal = array('', CLX_NUM_31, CLX_NUM_28, CLX_NUM_31, CLX_NUM_30, CLX_NUM_31, CLX_NUM_30, CLX_NUM_31, CLX_NUM_31, CLX_NUM_30, CLX_NUM_31, CLX_NUM_30, CLX_NUM_31);
    $_month_table_leaf = array('', CLX_NUM_31, CLX_NUM_29, CLX_NUM_31, CLX_NUM_30, CLX_NUM_31, CLX_NUM_30, CLX_NUM_31, CLX_NUM_31, CLX_NUM_30, CLX_NUM_31, CLX_NUM_30, CLX_NUM_31);
    $_total_date = CLX_INIT_ZERO;
    if ($year >= CLX_NUM_1970) {
        for ($a = CLX_NUM_1970; $a <= $year; $a++) {
		  $leaf = _adodb_is_leap_year($a);
		  if (!empty($leaf)) {
			 $loop_table = $_month_table_leaf;
			 $_add_date = CLX_366_DAYS;
		  } else {
			 $loop_table = $_month_table_normal;
			 $_add_date = CLX_365_DAYS;
		  }
		  if ($a < $year) {
			 $_total_date += $_add_date;
		  } else {
			 for ($b = CLX_NUM_ONE; $b < $mon; $b++) {
			     $_total_date += $loop_table[$b];
			 }
		  }
        }
        $_total_date +=$day - CLX_NUM_ONE;
        $ret = $_total_date * $_day_power + $hr * $_hour_power + $min * $_min_power + $sec + $gmt_different;
    } else {
        for ($a = CLX_NUM_1969; $a >= $year; $a--) {
		  $leaf = _adodb_is_leap_year($a);
		  if (!empty($leaf)) {
			 $loop_table = $_month_table_leaf;
			 $_add_date = CLX_366_DAYS;
		  } else {
			 $loop_table = $_month_table_normal;
			 $_add_date = CLX_365_DAYS;
		  }
		  if ($a > $year) {
			 $_total_date += $_add_date;
		  } else {
			 for ($b = CLX_NUM_12; $b > $mon; $b--) {
			     $_total_date += $loop_table[$b];
			 }
		  }
        }
        $_total_date += $loop_table[$mon] - $day;
        $_day_time = $hr * $_hour_power + $min * $_min_power + $sec;
        $_day_time = $_day_power - $_day_time;
        $ret = -CLX_NUM_ONE * ( $_total_date * $_day_power + $_day_time - $gmt_different);
        if ($ret < -CLX_NUM_ONE * CLX_NUM_12220185600)
		  $ret += CLX_NUM_TEN * CLX_NUM_86400; // if earlier than 5 Oct 1582 - gregorian correction
        else if ($ret < -CLX_NUM_ONE * CLX_NUM_12219321600)
		  $ret = -CLX_NUM_ONE * CLX_NUM_12219321600; // if in limbo, reset to 15 Oct 1582
    }
    return $ret;
}

function adodb_strtotime($str_date) {
    $stamp = -CLX_NUM_ONE;
    $str_date = lowercase(trim($str_date));
    $str_date_len = strlen($str_date);
    $hour = adodb_date('H');
    $min = adodb_date('i');
    $sec = adodb_date('s');
    $year = adodb_date('Y');
    $mon = adodb_date('m');
    $day = adodb_date('d');
    //Standard Calculations
    if ((stristr($str_date, 'now') && $str_date_len == CLX_NUM_THREE) || (stristr($str_date, 'ahora') && $str_date_len == CLX_NUM_FIVE)) {
        $stamp = adodb_mktime($hour, $min, $sec, $mon, $day, $year);
    } elseif ((stristr($str_date, 'today') && $str_date_len == CLX_NUM_FIVE) || (stristr($str_date, 'hoy') && $str_date_len == CLX_NUM_THREE)) {
        $stamp = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $mon, $day, $year);
    } elseif ((stristr($str_date, 'tomorrow') && $str_date_len == CLX_NUM_8) || (stristr($str_date, 'mañana') && $str_date_len == CLX_NUM_SIX)) {
        $stamp = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $mon, $day + CLX_NUM_ONE, $year);
    } elseif (stristr($str_date, 'day ago') && $str_date_len == CLX_NUM_7) {
        $stamp = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $mon, $day - CLX_NUM_ONE, $year);
    } elseif ((stristr($str_date, 'next year') && $str_date_len == CLX_NUM_NINE) || (stristr($str_date, 'proximo año') && $str_date_len == CLX_NUM_11)) {
        $stamp = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ONE, CLX_NUM_ONE, $year + CLX_NUM_ONE);
    } elseif ((stristr($str_date, 'next month') && $str_date_len == CLX_NUM_TEN) || (stristr($str_date, 'proximo mes') && $str_date_len == CLX_NUM_11)) {
        $stamp = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $mon + CLX_NUM_ONE, CLX_NUM_ONE, $year);
    } elseif (stristr($str_date, 'next day') && $str_date_len == CLX_NUM_8) {
        $stamp = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $mon, $day + CLX_NUM_ONE, $year);
    } elseif ((stristr($str_date, 'yesterday') && $str_date_len == CLX_NUM_NINE) || (stristr($str_date, 'ayer') && $str_date_len == CLX_NUM_FOUR)) {
        $stamp = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $mon, $day - CLX_NUM_ONE, $year);
    } elseif ((stristr($str_date, 'next week') && $str_date_len == CLX_NUM_NINE) || (stristr($str_date, 'proxima semana') && $str_date_len == CLX_NUM_14)) {
        $stamp = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $mon, $day + CLX_NUM_7, $year);
    } elseif ((stristr($str_date, 'last week') && $str_date_len == CLX_NUM_NINE) || (stristr($str_date, 'semana pasada') && $str_date_len == CLX_NUM_13)) {
        $stamp = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $mon, $day - CLX_NUM_7, $year);
    } elseif ((stristr($str_date, 'never') || stristr($str_date, 'nunca')) && $str_date_len == CLX_NUM_FIVE) {
        $stamp = -CLX_NUM_ONE;
    } elseif (preg_match('#([0-9]{1,4})[-|/]([0-9]{1,2})[-|/]([0-9]{1,2}) ([0-9]{1,2}):([0-9]{1,2}):([0-9]{1,2})#', $str_date, $regs)) {
        if ($regs[CLX_NUM_SIX] === 'p')
		  $regs[CLX_NUM_FOUR] = $regs[CLX_NUM_FOUR] + CLX_NUM_12;
        if (empty($regs[CLX_NUM_FOUR]))
		  $regs[CLX_NUM_FOUR] = CLX_INIT_ZERO;
        if (empty($regs[CLX_NUM_FIVE]))
		  $regs[CLX_NUM_FIVE] = CLX_INIT_ZERO;
        if (empty($regs[CLX_NUM_SIX]))
		  $regs[CLX_NUM_SIX] = CLX_INIT_ZERO;
        if (checkdate($regs[CLX_NUM_TWO], $regs[CLX_NUM_THREE], $regs[CLX_NUM_ONE])) {
		  $stamp = adodb_mktime($regs[CLX_NUM_FOUR], $regs[CLX_NUM_FIVE], $regs[CLX_NUM_SIX], $regs[CLX_NUM_TWO], $regs[CLX_NUM_THREE], $regs[CLX_NUM_ONE]);
        }
    } elseif (preg_match('#([0-9]{1,4})[-|/]([0-9]{1,2})[-|/]([0-9]{1,2}) ([0-9]{1,2}):([0-9]{1,2})([ am| pm]{0,1})#', $str_date, $regs)) {
        if ($regs[CLX_NUM_SIX] === 'p')
		  $regs[CLX_NUM_FOUR] = $regs[CLX_NUM_FOUR] + CLX_NUM_12;
        if (empty($regs[CLX_NUM_FOUR]))
		  $regs[CLX_NUM_FOUR] = CLX_INIT_ZERO;
        if (empty($regs[CLX_NUM_FIVE]))
		  $regs[CLX_NUM_FIVE] = CLX_INIT_ZERO;
        if (empty($regs[CLX_NUM_SIX]))
		  $regs[CLX_NUM_SIX] = CLX_INIT_ZERO;
        if (checkdate($regs[CLX_NUM_TWO], $regs[CLX_NUM_THREE], $regs[CLX_NUM_ONE])) {
		  $stamp = adodb_mktime($regs[CLX_NUM_FOUR], $regs[CLX_NUM_FIVE], $regs[CLX_NUM_SIX], $regs[CLX_NUM_TWO], $regs[CLX_NUM_THREE], $regs[CLX_NUM_ONE]);
        }
    } elseif (preg_match('#([0-9]{1,4})[-|/]([0-9]{1,2})[-|/]([0-9]{1,2}) ([0-9]{1,2}):([0-9]{1,2}):([0-9]{1,2})#', $str_date, $regs)) {
        if (empty($regs[CLX_NUM_FOUR]))
		  $regs[CLX_NUM_FOUR] = CLX_INIT_ZERO;
        if (empty($regs[CLX_NUM_FIVE]))
		  $regs[CLX_NUM_FIVE] = CLX_INIT_ZERO;
        if (empty($regs[CLX_NUM_SIX]))
		  $regs[CLX_NUM_SIX] = CLX_INIT_ZERO;
        if (checkdate($regs[CLX_NUM_TWO], $regs[CLX_NUM_THREE], $regs[CLX_NUM_ONE])) {
		  $stamp = adodb_mktime($regs[CLX_NUM_FOUR], $regs[CLX_NUM_FIVE], $regs[CLX_NUM_SIX], $regs[CLX_NUM_TWO], $regs[CLX_NUM_THREE], $regs[CLX_NUM_ONE]);
        }
    } elseif (preg_match('#([0-9]{1,4})[-|/]([0-9]{1,2})[-|/]([0-9]{1,2})#', $str_date, $regs)) {
        if (checkdate($regs[CLX_NUM_TWO], $regs[CLX_NUM_THREE], $regs[CLX_NUM_ONE])) {
		  $stamp = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $regs[CLX_NUM_TWO], $regs[CLX_NUM_THREE], $regs[CLX_NUM_ONE]);
        }
    } elseif (preg_match('#([0-9]{2,4})([0-9]{2})([0-9]{2})#', $str_date, $regs) && ($str_date_len == CLX_NUM_SIX || $str_date_len == CLX_NUM_8)) {
        if (checkdate($regs[CLX_NUM_TWO], $regs[CLX_NUM_THREE], $regs[CLX_NUM_ONE])) {
		  $stamp = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $regs[CLX_NUM_TWO], $regs[CLX_NUM_THREE], $regs[CLX_NUM_ONE]);
        }
    } elseif (preg_match('#([0-9]{2,4})([0-9]{2})([0-9]{2})([0-9]{2})([0-9]{2})([0-9]{2})#', $str_date, $regs) && ($str_date_len == CLX_NUM_14 || $str_date_len == CLX_NUM_12)) {
        if (checkdate($regs[CLX_NUM_TWO], $regs[CLX_NUM_THREE], $regs[CLX_NUM_ONE])) {
		  $stamp = adodb_mktime($regs[CLX_NUM_FOUR], $regs[CLX_NUM_FIVE], $regs[CLX_NUM_SIX], $regs[CLX_NUM_TWO], $regs[CLX_NUM_THREE], $regs[CLX_NUM_ONE]);
        }
    } elseif (preg_match('#([a-zA-Z ]{0,20}){0,1}(\+|\-)([0-9 ]{1,3})([a-zA-Z ]{1,20})#', $str_date, $regs)) {
        if ($regs['2'] === '-')
		  $regs['3'] = - CLX_NUM_ONE * $regs['3'];
        $tmp_stamp = CLX_INIT_ZERO;
        if (!empty($regs['1'])) {// today, now, etc
		  $a = adodb_strtotime($regs['1']);
		  if ($a != - CLX_NUM_ONE)
			 $tmp_stamp = $tmp_stamp + $a;
        }else {
		  $tmp_stamp = adodb_strtotime('now');
        }
        if (!empty($regs['4']) && stristr($regs['4'], 'day')) {
		  $tmp_stamp = $tmp_stamp + adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $mon, $day + $regs['3'], $year);
        } elseif (!empty($regs['4']) && stristr($regs['4'], 'year')) {
		  $tmp_stamp = $tmp_stamp + adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $mon, $day, $year + $regs['3']);
        } elseif (!empty($regs['4']) && stristr($regs['4'], 'month')) {
		  $tmp_stamp = $tmp_stamp + adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $mon + $regs['3'], $day, $year);
        } elseif (!empty($regs['4']) && stristr($regs['4'], 'week')) {
		  $regs['3'] = $regs['3'] * CLX_NUM_7;
		  $tmp_stamp = $tmp_stamp + adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $mon, $day + $regs['3'], $year);
        }
        if ($tmp_stamp == CLX_NUM_ZERO)
		  $stamp = -CLX_NUM_ONE;
        else
		  $stamp = $tmp_stamp - adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $mon, $day, $year);
    }
    elseif (preg_match('#([0-9]{1,2}):([0-9]{1,2})(:([0-9]{1,2})){0,1}([ am| pm]{0,1})#', lowercase($str_date), $regs)) {
        $stamp = strtotime($str_date);
    }
    if ($stamp === '0000-00-00' || $stamp === '0000-00-00 00:00:00')
        $stamp = '';
    return $stamp;
}

/**
 * Return number of days between 2 dates or false on invalid period or date.
 * @param: $edate End Date
 * @param: $sdate Start Date
 */
function day_diff($edate, $sdate) {
    $gd_a = adodb_getdate(adodb_strtotime($edate));
    $gd_b = adodb_getdate(adodb_strtotime($sdate));
    $a_new = adodb_mktime(CLX_NUM_12, CLX_NUM_ZERO, CLX_NUM_ZERO, $gd_a['mon'], $gd_a['mday'], $gd_a['year']);
    $b_new = adodb_mktime(CLX_NUM_12, CLX_NUM_ZERO, CLX_NUM_ZERO, $gd_b['mon'], $gd_b['mday'], $gd_b['year']);
    return round(($a_new - $b_new) / CLX_NUM_86400);
}

/**
 * Return Next Day
 */
function adodb_next_day($date, $format = 'Y-m-d') {
    return adodb_next_date($date, 'D', $format);
}

/**
 * Return previous day
 */
function adodb_prev_dayVoid($date, $format = 'Y-m-d') {
    ClxFreeReturn(adodb_prev_day($date, $format));
}

function adodb_prev_day($date, $format = 'Y-m-d') {
    $last_run_date = adodb_getdate(adodb_strtotime($date));
    return adodb_date($format, adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'], $last_run_date['mday'] - CLX_NUM_ONE, $last_run_date['year']));
}

/**
 * Return next month last day
 */
function adodb_next_month_lastday($date, $format = 'Y-m-d') {
    $last_run_date = adodb_getdate(adodb_strtotime($date));
    return adodb_date($format, adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'] + CLX_NUM_TWO, CLX_NUM_ZERO, $last_run_date['year']));
}

/**
 * Return next month last day
 */
function adodb_prev_month_lastday($date, $format = 'Y-m-d') {
    $last_run_date = adodb_getdate(adodb_strtotime($date));
    return adodb_date($format, adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'], CLX_NUM_ZERO, $last_run_date['year']));
}

function adodb_month_lastday($date, $format = 'Y-m-d') {
    $last_run_date = adodb_getdate(adodb_strtotime($date));
    return adodb_date($format, adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'] + CLX_NUM_ONE, CLX_NUM_ZERO, $last_run_date['year']));
}

/**
 * Returns current month last day
 *
 * @param unknown_type $date
 * @param unknown_type $format
 * @return unknown
 */
function adodb_get_month_lastday($date, $format = 'Y-m-d') {
    $last_run_date = adodb_getdate(adodb_strtotime($date));
    return adodb_date($format, adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'] + CLX_NUM_ONE, CLX_NUM_ZERO, $last_run_date['year']));
}

function adodb_next_date($date, $period, $format = 'Y-m-d') {
    return adodb_period_date($date, $period, CLX_NUM_ONE, $format);
}

/**
 * Period Next Day
 * Return next day in future from Day, Month, Yearly.
 */
function adodb_period_date($date, $period, $no_periods, $format = 'Y-m-d') {
    if ($no_periods === '')
        $no_periods = CLX_INIT_ZERO;
    $next_run_date = '';
    $last_run_date = adodb_getdate(adodb_strtotime($date));
    switch ($period) {
        case '0':
		  $next_run_date = $date;
		  break;
        case 'D'://Daily
		  $next_run_date = adodb_date($format, adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'], $last_run_date['mday'] + (CLX_NUM_ONE * $no_periods), $last_run_date['year']));
		  break;
        case '360'://Daily
		  $next_run_date = adodb_date($format, adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'], $last_run_date['mday'] + (CLX_NUM_ONE * $no_periods), $last_run_date['year']));
		  break;
        case '365'://Daily
		  $next_run_date = adodb_date($format, adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'], $last_run_date['mday'] + (CLX_NUM_ONE * $no_periods), $last_run_date['year']));
		  break;
        case 'W'://Weekly
		  $next_run_date = adodb_date($format, adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'], $last_run_date['mday'] + (CLX_NUM_7 * $no_periods), $last_run_date['year']));
		  break;
        case '1'://biweekly
		  $next_run_date = adodb_date($format, adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'], $last_run_date['mday'] + (CLX_NUM_3_PT_5 * $no_periods), $last_run_date['year']));
		  break;
        case '2'://Monthly
		  $next_run_date = adodb_next_month_lastday($date, $format = 'Y-m-d');
		  break;
        case '3'://bionthly
		  $next_run_date = adodb_date($format, adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'] + (CLX_NUM_TWO * $no_periods), $last_run_date['mday'], $last_run_date['year']));
		  break;
        case 'BMO'://bimonthly
		  $next_run_date = adodb_date($format, adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'] + (CLX_NUM_TWO * $no_periods), $last_run_date['mday'], $last_run_date['year']));
		  break;
        case '4'://quaterly
		  $next_run_date = adodb_date($format, adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'] + (CLX_NUM_THREE * $no_periods), ($last_run_date['mday']), $last_run_date['year']));
		  break;
        case 'F'://Four-monthly
		  $next_run_date = adodb_date($format, adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'] + (CLX_NUM_FOUR * $no_periods), ($last_run_date['mday']), $last_run_date['year']));
		  break;
        case 'Q'://quaterly
		  $next_run_date = adodb_date($format, adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'] + (CLX_NUM_THREE * $no_periods), ($last_run_date['mday']), $last_run_date['year']));
		  break;
        case '5'://semestral
		  $next_run_date = adodb_date($format, adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'] + (CLX_NUM_SIX * $no_periods), $last_run_date['mday'], $last_run_date['year']));
		  break;
        case '6'://Yearly
		  $next_run_date = adodb_date($format, adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'], $last_run_date['mday'], $last_run_date['year'] + (CLX_NUM_ONE * $no_periods)));
		  break;
        case 'M'://Monthly
		  $next_run_date = adodb_date($format, adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'] + (CLX_NUM_ONE * $no_periods), $last_run_date['mday'], $last_run_date['year']));
		  break;
        case 'SE'://Semi Annual Annual
		  $next_run_date = adodb_date($format, adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'] + (CLX_NUM_SIX * $no_periods), $last_run_date['mday'], $last_run_date['year']));
		  break;
        case 'WE'://Wekkly
		  $next_run_date = adodb_date($format, adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'], $last_run_date['mday'] + (CLX_NUM_7 * $no_periods), $last_run_date['year']));
		  break;
        case 'Y'://Annual
		  $next_run_date = adodb_date($format, adodb_mktime($last_run_date['hours'], $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'], $last_run_date['mday'], $last_run_date['year'] + (CLX_NUM_ONE * $no_periods)));
		  break;
        case 'I'://Minutes
		  $next_run_date = adodb_date($format, adodb_mktime($last_run_date['hours'], $last_run_date['minutes'] + (CLX_NUM_ONE * $no_periods), $last_run_date['seconds'], $last_run_date['mon'], $last_run_date['mday'], $last_run_date['year']));
		  break;
        case 'H'://Hour
		  $next_run_date = adodb_date($format, adodb_mktime($last_run_date['hours'] + (CLX_NUM_ONE * $no_periods), $last_run_date['minutes'], $last_run_date['seconds'], $last_run_date['mon'], $last_run_date['mday'], $last_run_date['year']));
		  break;
        case 'NONE':
		  $next_run_date = '';
		  break;
        default://10 Ontime only or IM Investment Maturity
		  $next_run_date = $date;
		  break;
    }
    return $next_run_date;
}

function adodb_valid_date($date) {
    $timestamp = adodb_strtotime($date);
    return $timestamp <> -CLX_NUM_ONE && !empty($date);
}

function adodb_month_last_day($date) {
    //Validate end of month, if not just return
    $return = (adodb_next_day($date, 'm') == adodb_date('m', adodb_strtotime($date))) ? false : true;
    return $return;
}

/**
 * Imports date according to format. Used on sockets to interpret incoming dates
 * formats are defined from list on clx_tools
 * @param string $date
 * @param string $format
 */
function toIsoDate($date, $format) {
    $contenidoHTML = '';
    $date = trim($date);
    if ($date === '')
        return $date;
    $format = str_replace('/', '-', $format);
    switch ($format) {
        case 'd-m-Y':
		  $return = substr($date, CLX_NUM_SIX, CLX_NUM_FOUR) . '-' . substr($date, CLX_NUM_THREE, CLX_NUM_TWO) . '-' . substr($date, CLX_NUM_ZERO, CLX_NUM_TWO);
		  break;
        case 'd-m-y':
		  $return = substr($date, CLX_NUM_SIX, CLX_NUM_TWO) . '-' . substr($date, CLX_NUM_THREE, CLX_NUM_TWO) . '-' . substr($date, CLX_NUM_ZERO, CLX_NUM_TWO);
		  break;
        case 'c':
		  $return = substr($date, CLX_NUM_ZERO, CLX_NUM_TEN) . ' ' . substr($date, CLX_NUM_11, CLX_NUM_8);
		  break;
        case 'mdy':
		  $return = substr($date, CLX_NUM_FOUR, CLX_NUM_TWO) . '-' . substr($date, CLX_NUM_ZERO, CLX_NUM_TWO) . '-' . substr($date, CLX_NUM_TWO, CLX_NUM_TWO);
		  break;
        case 'mdY':
		  $return = substr($date, CLX_NUM_FOUR, CLX_NUM_FOUR) . '-' . substr($date, CLX_NUM_ZERO, CLX_NUM_TWO) . '-' . substr($date, CLX_NUM_TWO, CLX_NUM_TWO);
		  break;
        case 'mdy':
		  $return = substr($date, CLX_NUM_FOUR, CLX_NUM_TWO) . '-' . substr($date, CLX_NUM_TWO, CLX_NUM_TWO) . '-' . substr($date, CLX_NUM_ZERO, CLX_NUM_TWO);
		  break;
        case 'mdY':
		  $return = substr($date, CLX_NUM_FOUR, CLX_NUM_FOUR) . '-' . substr($date, CLX_NUM_TWO, CLX_NUM_TWO) . '-' . substr($date, CLX_NUM_ZERO, CLX_NUM_TWO);
		  break;
        case '0m0d0y':
		  $return = substr($date, CLX_NUM_7, CLX_NUM_TWO) . '-' . substr($date, CLX_NUM_ONE, CLX_NUM_TWO) . '-' . substr($date, CLX_NUM_FOUR, CLX_NUM_TWO);
		  break;
        case '0m0d0Y':
		  $return = substr($date, CLX_NUM_7, CLX_NUM_FOUR) . '-' . substr($date, CLX_NUM_ONE, CLX_NUM_TWO) . '-' . substr($date, CLX_NUM_FOUR, CLX_NUM_TWO);
		  break;
        case 'md Y':
		  $contenidoHTML.= '->' . $date . '<-<br>';
		  $return = trim(substr($date, CLX_NUM_FOUR, CLX_NUM_FIVE)) . '-' . trim(substr($date, CLX_NUM_ZERO, CLX_NUM_TWO)) . '-' . trim(substr($date, CLX_NUM_TWO, CLX_NUM_TWO));
		  break;
        case 'n j y':
		  $date = trim(str_replace('  ', ' ', $date));
		  $dtar = explode(' ', $date);
		  if (count($dtar) == CLX_NUM_THREE) {
			 if (strlen($dtar[CLX_NUM_ZERO]) < CLX_NUM_TWO)
			     $dtar[CLX_NUM_ZERO] = str_repeat('0', (CLX_NUM_TWO - strlen($dtar[CLX_NUM_ZERO]))) . $dtar[CLX_NUM_ZERO];
			 if (strlen($dtar[CLX_NUM_ONE]) < CLX_NUM_TWO)
			     $dtar[CLX_NUM_ONE] = str_repeat('0', CLX_NUM_(CLX_NUM_TWO - strlen($dtar[CLX_NUM_ONE]))) . $dtar[CLX_NUM_ONE];
			 if (strlen($dtar[CLX_NUM_TWO]) < CLX_NUM_TWO)
			     $dtar[CLX_NUM_TWO] = str_repeat('0', (CLX_NUM_TWO - strlen($dtar[CLX_NUM_TWO]))) . $dtar[CLX_NUM_TWO];
			 $return = $dtar[CLX_NUM_TWO] . '-' . $dtar[CLX_NUM_ONE] . '-' . $dtar[CLX_NUM_ZERO];
		  }else {
			 $return = $date;
		  }
		  break;
        case 'n j Y':
		  $date = trim(str_replace('  ', ' ', $date));
		  $dtar = explode(' ', $date);
		  if (count($dtar) == CLX_NUM_THREE) {
			 if (strlen($dtar[CLX_NUM_ZERO]) < CLX_NUM_TWO)
			     $dtar[CLX_NUM_ZERO] = str_repeat('0', (CLX_NUM_TWO - strlen($dtar[CLX_NUM_ZERO]))) . $dtar[CLX_NUM_ZERO];
			 if (strlen($dtar[CLX_NUM_ONE]) < CLX_NUM_TWO)
			     $dtar[CLX_NUM_ONE] = str_repeat('0', (CLX_NUM_TWO - strlen($dtar[CLX_NUM_ONE]))) . $dtar[CLX_NUM_ONE];
			 if (strlen($dtar[CLX_NUM_TWO]) < CLX_NUM_FOUR)
			     $dtar[CLX_NUM_TWO] = str_repeat('0', (CLX_NUM_FOUR - strlen($dtar[CLX_NUM_TWO]))) . $dtar[CLX_NUM_TWO];
			 $return = $dtar[CLX_NUM_TWO] . '-' . $dtar[CLX_NUM_ONE] . '-' . $dtar[CLX_NUM_ZERO];
		  }else {
			 $return = $date;
		  }
		  break;
        case ' n j y':
		  $date = trim(str_replace('  ', ' ', $date));
		  $dtar = explode(' ', $date);
		  if (count($dtar) == CLX_NUM_THREE) {
			 if (strlen($dtar[CLX_NUM_ZERO]) < CLX_NUM_TWO)
			     $dtar[CLX_NUM_ZERO] = str_repeat('0', (CLX_NUM_TWO - strlen($dtar[CLX_NUM_ZERO]))) . $dtar[CLX_NUM_ZERO];
			 if (strlen($dtar[CLX_NUM_ONE]) < CLX_NUM_TWO)
			     $dtar[CLX_NUM_ONE] = str_repeat('0', (CLX_NUM_TWO - strlen($dtar[CLX_NUM_ONE]))) . $dtar[CLX_NUM_ONE];
			 if (strlen($dtar[CLX_NUM_TWO]) < CLX_NUM_TWO)
			     $dtar[CLX_NUM_TWO] = str_repeat('0', (CLX_NUM_TWO - strlen($dtar[CLX_NUM_TWO]))) . $dtar[CLX_NUM_TWO];
			 $return = $dtar[CLX_NUM_TWO] . '-' . $dtar[CLX_NUM_ONE] . '-' . $dtar[CLX_NUM_ZERO];
		  }else {
			 $return = $date;
		  }
		  break;
        case ' n j Y':
		  $date = trim(str_replace('  ', ' ', $date));
		  $dtar = explode(' ', $date);
		  if (count($dtar) == CLX_NUM_THREE) {
			 if (strlen($dtar[CLX_NUM_ZERO]) < CLX_NUM_TWO)
			     $dtar[CLX_NUM_ZERO] = str_repeat('0', (CLX_NUM_TWO - strlen($dtar[CLX_NUM_ZERO]))) . $dtar[CLX_NUM_ZERO];
			 if (strlen($dtar[CLX_NUM_ONE]) < CLX_NUM_TWO)
			     $dtar[CLX_NUM_ONE] = str_repeat('0', (CLX_NUM_TWO - strlen($dtar[CLX_NUM_ONE]))) . $dtar[CLX_NUM_ONE];
			 if (strlen($dtar[CLX_NUM_TWO]) < CLX_NUM_FOUR)
			     $dtar[CLX_NUM_TWO] = str_repeat('0', (CLX_NUM_FOUR - strlen($dtar[CLX_NUM_TWO]))) . $dtar[CLX_NUM_TWO];
			 $return = $dtar[CLX_NUM_TWO] . '-' . $dtar[CLX_NUM_ONE] . '-' . $dtar[CLX_NUM_ZERO];
		  }else {
			 $return = $date;
		  }
		  break;
        case ' j n y':
		  $date = trim(str_replace('  ', ' ', $date));
		  $dtar = explode(' ', $date);
		  if (count($dtar) == CLX_NUM_THREE) {
			 if (strlen($dtar[CLX_NUM_ZERO]) < CLX_NUM_TWO)
			     $dtar[CLX_NUM_ZERO] = str_repeat('0', (CLX_NUM_TWO - strlen($dtar[CLX_NUM_ZERO]))) . $dtar[CLX_NUM_ZERO];
			 if (strlen($dtar[CLX_NUM_ONE]) < CLX_NUM_TWO)
			     $dtar[CLX_NUM_ONE] = str_repeat('0', (CLX_NUM_TWO - strlen($dtar[CLX_NUM_ONE]))) . $dtar[CLX_NUM_ONE];
			 if (strlen($dtar[CLX_NUM_TWO]) < CLX_NUM_TWO)
			     $dtar[CLX_NUM_TWO] = str_repeat('0', (CLX_NUM_TWO - strlen($dtar[CLX_NUM_TWO]))) . $dtar[CLX_NUM_TWO];
			 $return = $dtar[CLX_NUM_TWO] . '-' . $dtar[CLX_NUM_ZERO] . '-' . $dtar[CLX_NUM_ONE];
		  }else {
			 $return = $date;
		  }
		  break;
        case ' j n Y':
		  $date = trim(str_replace('  ', ' ', $date));
		  $dtar = explode(' ', $date);
		  if (count($dtar) == CLX_NUM_THREE) {
			 if (strlen($dtar[CLX_NUM_ZERO]) < CLX_NUM_TWO)
			     $dtar[CLX_NUM_ZERO] = str_repeat('0', (CLX_NUM_TWO - strlen($dtar[CLX_NUM_ZERO]))) . $dtar[CLX_NUM_ZERO];
			 if (strlen($dtar[CLX_NUM_ONE]) < CLX_NUM_TWO)
			     $dtar[CLX_NUM_ONE] = str_repeat('0', (CLX_NUM_TWO - strlen($dtar[CLX_NUM_ONE]))) . $dtar[CLX_NUM_ONE];
			 if (strlen($dtar[CLX_NUM_TWO]) < CLX_NUM_FOUR)
			     $dtar[CLX_NUM_TWO] = str_repeat('0', (CLX_NUM_FOUR - strlen($dtar[CLX_NUM_TWO]))) . $dtar[CLX_NUM_TWO];
			 $return = $dtar[CLX_NUM_TWO] . '-' . $dtar[CLX_NUM_ZERO] . '-' . $dtar[CLX_NUM_ONE];
		  }else {
			 $return = $date;
		  }
		  break;
        case 'YmdHis':
		  $date = substr($date, CLX_NUM_ZERO, CLX_NUM_FOUR) . '-' . substr($date, CLX_NUM_FOUR, CLX_NUM_TWO) . '-' . substr($date, CLX_NUM_SIX, CLX_NUM_TWO) . ' ' . substr($date, CLX_NUM_8, CLX_NUM_TWO) . ':' . substr($date, CLX_NUM_TEN, CLX_NUM_TWO);
		  if (substr($date, CLX_NUM_12, CLX_NUM_TWO) <> '')
			 $date .=':' . substr($date, CLX_NUM_12, CLX_NUM_TWO);
		  if (adodb_valid_date($date)) {
			 $return = adodb_date('Y-m-d H:i:s', adodb_strtotime($date));
		  } else {
			 $return = '';
		  }
		  break;
        case 'Ymd':
		  $date = substr($date, CLX_NUM_ZERO, CLX_NUM_FOUR) . '-' . substr($date, CLX_NUM_FOUR, CLX_NUM_TWO) . '-' . substr($date, CLX_NUM_SIX, CLX_NUM_TWO);
		  if (adodb_valid_date($date)) {
			 $return = adodb_date('Y-m-d H:i:s', adodb_strtotime($date));
		  } else {
			 $return = '';
		  }
		  break;
        case 'ymdHis':
		  $date = '00' . substr($date, CLX_NUM_ZERO, CLX_NUM_TWO) . '-' . substr($date, CLX_NUM_TWO, CLX_NUM_TWO) . '-' . substr($date, CLX_NUM_FOUR, CLX_NUM_TWO) . ' ' . substr($date, CLX_NUM_SIX, CLX_NUM_TWO) . ':' . substr($date, CLX_NUM_8, CLX_NUM_TWO);
		  if (substr($date, CLX_NUM_TEN, CLX_NUM_TWO) <> '')
			 $date .=':' . substr($date, CLX_NUM_TEN, CLX_NUM_TWO);
		  if (adodb_valid_date($date)) {
			 $return = adodb_date('Y-m-d H:i:s', adodb_strtotime($date));
		  } else {
			 $return = '';
		  }
		  break;
        case 'ymd':
		  if (adodb_valid_date($date)) {
			 $return = adodb_date('Y-m-d H:i:s', adodb_strtotime('00' . $date));
		  } else {
			 $return = '';
		  }
		  break;
        default:
		  if (adodb_valid_date($date)) {
			 $return = adodb_date('Y-m-d H:i:s', adodb_strtotime($date));
		  } else {
			 $return = '';
		  }
		  break;
    }
    echo $contenidoHTML;
    return $return;
}

/**
 * Export date according to format. Used on sockets to interpret outgoing dates
 * formats are defined from list on clx_tools. Inverse to toIsoDate.
 * @param string $date
 * @param string $format
 */
function fromIsoDate($date, $format) {
    if($date===CLX_CHAR_EMPTY_STRING){
        return '';
    }
    $format = str_replace('/', '-', $format);
    switch ($format) {
	case 'Y-m-d':
		  $xdate = explode('/', $date);
	    if (count($xdate)>1) {
		$return = $xdate[CLX_NUM_ZERO]."-". $xdate[CLX_NUM_ONE]."-".$xdate[CLX_NUM_TWO];
	    }
	    else
		$return =$date;

	    break;
	case 'd-m-Y':
			$xdate = explode('/', $date);
			if (count($xdate)<=1)
				$xdate = explode('-', $date);
			$return = (empty($xdate[CLX_NUM_TWO])?'':$xdate[CLX_NUM_TWO])."-". (empty($xdate[CLX_NUM_ONE])?'':$xdate[CLX_NUM_ONE])."-".$xdate[CLX_NUM_ZERO];
	   	    break;
        case '0m0d0y':
		  $date = explode('-', $date);
		  $return = stringAddSpacer(intval($date[CLX_NUM_ONE]), CLX_NUM_THREE, true, CLX_NUM_ZERO, false);
		  $return .= stringAddSpacer(intval($date[CLX_NUM_TWO]), CLX_NUM_THREE, true, CLX_NUM_ZERO, false);
		  $return .= stringAddSpacer(intval(substr($date[CLX_NUM_ZERO], - CLX_NUM_TWO)), CLX_NUM_THREE, true, CLX_NUM_ZERO, false);
		  break;
        case 'md Y':
		  $date = explode('-', $date);
		  $return = stringAddSpacer(intval($date[CLX_NUM_ONE]), CLX_NUM_TWO, true, CLX_NUM_ZERO, false);
		  $return .= stringAddSpacer(intval($date[CLX_NUM_TWO]), CLX_NUM_TWO, true, CLX_NUM_ZERO, false);
		  $return .= stringAddSpacer(intval($date[CLX_NUM_ZERO]), CLX_NUM_FIVE, true, CLX_NUM_ZERO, false);
		  break;
        case '0m0d0Y':
		  $date = explode('-', $date);
		  $return = stringAddSpacer(intval($date[CLX_NUM_ONE]), CLX_NUM_THREE, true, CLX_NUM_ZERO, false);
		  $return .= stringAddSpacer(intval($date[CLX_NUM_TWO]), CLX_NUM_THREE, true, CLX_NUM_ZERO, false);
		  $return .= stringAddSpacer(intval($date[CLX_NUM_ZERO]), CLX_NUM_FIVE, true, CLX_NUM_ZERO, false);
		  break;
        case 'n j y':
		  $date = explode('-', $date);
		  $return = stringAddSpacer(intval($date[CLX_NUM_ONE]), CLX_NUM_TWO, true, ' ', false);
		  $return .= stringAddSpacer(intval($date[CLX_NUM_TWO]), CLX_NUM_THREE, true, ' ', false);
		  $return .= stringAddSpacer(intval(substr($date[CLX_NUM_ZERO], -CLX_NUM_TWO)), CLX_NUM_THREE, true, ' ', false);
		  break;
        case 'n j Y':
		  $date = explode('-', $date);
		  $return = stringAddSpacer($date[CLX_NUM_ONE], CLX_NUM_TWO, true, ' ', false);
		  $return .= stringAddSpacer($date[CLX_NUM_TWO], CLX_NUM_THREE, true, ' ', false);
		  $return .= stringAddSpacer($date[CLX_NUM_ZERO], CLX_NUM_FIVE, true, ' ', false);
		  break;
        case ' n j y':
		  $date = explode('-', $date);
		  $return = stringAddSpacer(intval($date[CLX_NUM_ONE]), CLX_NUM_THREE, true, ' ', false);
		  $return .= stringAddSpacer(intval($date[CLX_NUM_TWO]), CLX_NUM_THREE, true, ' ', false);
		  $return .= stringAddSpacer(intval(substr($date[CLX_NUM_ZERO], -CLX_NUM_TWO)), CLX_NUM_THREE, true, ' ', false);
		  break;
        case ' n j Y':
		  $date = explode('-', $date);
		  $return = stringAddSpacer(intval($date[CLX_NUM_ONE]), CLX_NUM_THREE, true, ' ', false);
		  $return .= stringAddSpacer(intval($date[CLX_NUM_TWO]), CLX_NUM_THREE, true, ' ', false);
		  $return .= stringAddSpacer(intval($date[CLX_NUM_ZERO]), CLX_NUM_FIVE, true, ' ', false);
		  break;
        case ' j n y':
		  $date = explode('-', $date);
		  $return = stringAddSpacer(intval($date[CLX_NUM_TWO]), CLX_NUM_THREE, true, ' ', false);
		  $return .= stringAddSpacer(intval($date[CLX_NUM_ONE]), CLX_NUM_THREE, true, ' ', false);
		  $return .= stringAddSpacer(intval(substr($date[CLX_NUM_ZERO], -CLX_NUM_TWO)), CLX_NUM_THREE, true, ' ', false);
		  break;
        case ' j n Y':
		  $date = explode('-', $date);
		  $return = stringAddSpacer(intval($date[CLX_NUM_TWO]), CLX_NUM_THREE, true, ' ', false);
		  $return .= stringAddSpacer(intval($date[CLX_NUM_ONE]), CLX_NUM_THREE, true, ' ', false);
		  $return .= stringAddSpacer(intval($date[CLX_NUM_ZERO]), CLX_NUM_FIVE, true, ' ', false);
		  break;
        default:
		  $return = adodb_date($format, adodb_strtotime($date));
		  break;
    }
    return $return;
}

/**
 * Returns formated array according to input.
 * @param string $date
 * @param string $format
 * @return array
 */
function adodb_parse_date($date, $format = 'Y-m-d H:i:s') {
    $date = toIsoDate($date, $format);
    $d = explode(' ', $date);
    $dt = explode('-', $d[CLX_NUM_ZERO]);
    $hr = explode(':', $d[CLX_NUM_ONE]);
    $r = array_merge($dt, $hr);
    $ret['year'] = $r[CLX_NUM_ZERO];
    $ret['month'] = $r[CLX_NUM_ONE];
    $ret['day'] = $r[CLX_NUM_TWO];
    $ret['hour'] = $r[CLX_NUM_THREE];
    $ret['minute'] = $r[CLX_NUM_FOUR];
    $ret['second'] = $r[CLX_NUM_FIVE];
    return $ret;
}

/**
 * If not a valid date it will clear the value to blank
 *
 * @param unknown_type $date
 */
function adodb_clear_invalid_date(&$date) {
    if (adodb_strtotime($date) == -CLX_NUM_ONE) {
        $date = '';
    }
}

function adodb_period_date_database($date, $period, $no_periods, $format = 'Y-m-d') {
    /*
      MICROSECOND SECOND MINUTE HOUR DAY WEEK MONTH QUARTER YEAR SECOND_MICROSECOND MINUTE_MICROSECOND MINUTE_SECOND HOUR_MICROSECOND HOUR_SECOND HOUR_MINUTE DAY_MICROSECOND DAY_SECOND DAY_MINUTE DAY_HOUR YEAR_MONTH */
    switch ($period) {
        case 'D':
		  $interval = 'DAY';
		  break;
        case 'W'://Weekly
		  $interval = 'WEEK';
		  break;
        case '1'://biweekly
		  $no_periods = CLX_NUM_TWO * $no_periods;
		  $interval = 'WEEK';
		  break;
        case '2'://Monthly
		  $interval = 'MONTH';
		  break;
        case 'BMO'://bionthly
		  $no_periods = CLX_NUM_TWO * $no_periods;
		  $interval = 'MONTH';
		  break;
        case '3'://bionthly
		  $no_periods = CLX_NUM_TWO * $no_periods;
		  $interval = 'MONTH';
		  break;
        case '4'://quaterly
		  $interval = 'QUARTER';
		  break;
        case 'Q'://quaterly
		  $interval = 'QUARTER';
		  break;
        case '5'://semestral
		  $no_periods = CLX_NUM_TWO * $no_periods;
		  $interval = 'QUARTER';
		  break;
        case '6'://Yearly
		  $interval = 'YEAR';
		  break;
        case 'M'://Semi Annual
		  $interval = 'MONTH';
		  break;
        case 'SE'://Semi Annual Annual
		  $no_periods = CLX_NUM_TWO * $no_periods;
		  $interval = 'QUARTER';
		  break;
        case 'WE'://Weekly
		  $interval = 'WEEK';
		  break;
        case 'Y'://Annual
		  $interval = 'YEAR';
		  break;
        case 'NONE':
		  return null;
		  break;
        default://10 Ontime only or IM Investment Maturity
		  $interval = 'DAY';
		  break;
    }
    return xFormat::date(ClxCommon::$db->getOne('SELECT ADDDATE(' . singleQuote($date) . ', INTERVAL ' . $no_periods . ' ' . $interval . ') FROM DUAL'), $format);
}

/**
 * Return previous day
 */
function adodb_prev_day_database($date, $format = 'Y-m-d') {
    return adodb_period_date_database($date, 'D', CLX_NUM_MINUS_1, $format);
}

/**
 * Counts the number or periods of an interval
 * @param string $sdate
 * @param string $edate
 * @param string $period_type
 * @return number
 */
function adodb_number_of_periods($sdate, $edate, $period_type) {
    $number = (int) CLX_NUM_ZERO;
    $date1 = new DateTime($edate);
    $date2 = new DateTime($sdate);
    $interval = date_diff($date2, $date1);
    switch ($period_type) {
        case '360':
		  $number = day_diff($edate, $sdate);
		  break;
        case '365':
		  $number = day_diff($edate, $sdate);
		  break;
        case '366':
		  $number = day_diff($edate, $sdate);
		  break;
        case '12'://Monthly
		  $number = floor((($interval->y * CLX_NUM_12) + $interval->m) / CLX_NUM_ONE);
		  break;
        case '6'://Bimonthly(6)
		  $number = floor((($interval->y * CLX_NUM_12) + $interval->m) / CLX_NUM_SIX);
		  break;
        case '4'://Quarterly(4)
		  $number = floor((($interval->y * CLX_NUM_12) + $interval->m) / CLX_NUM_THREE);
		  break;
        case '3'://Four-monthly 1/3(3)
		  $number = floor((($interval->y * CLX_NUM_12) + $interval->m) / CLX_NUM_FOUR);
		  break;
        case '2'://Semiannually(2)
		  $number = floor((($interval->y * CLX_NUM_12) + $interval->m) / CLX_NUM_SIX);
		  break;
        case '1'://Annually(1)
		  $number = $interval->y;
		  break;
        default:
		  $sdate = CLX_INIT_ZERO;
		  break;
    }
    return $number;
}

/**
 * Counts the number or periods of an interval for days 360
 * @param string $sdate
 * @param string $edate
 * @param string $period_type
 * @return number
 */
function adodb_number_of_periods360($sdate, $edate, $period_type, $method = false) {
    $number = (int) CLX_NUM_ZERO;
    if (!class_exists('Financial'))
        require 'classes/Financial.php';
    $a = new Financial();
    $interval = $a->DAYS360(adodb_strtotime($sdate), adodb_strtotime($edate), $method);
    switch ($period_type) {
        case '360':
		  $number = ($interval / CLX_360_DAYS);
		  break;
        case '365':
		  $number = ($interval / CLX_365_DAYS);
		  break;
        case '366':
		  $number = ($interval / CLX_366_DAYS);
		  break;
        case '12'://Monthly
		  $number = $interval / (CLX_360_DAYS / CLX_NUM_12);
		  break;
        case '6'://Bimonthly(6)
		  $number = $interval / (CLX_360_DAYS / CLX_NUM_SIX);
		  break;
        case '4'://Quarterly(4)
		  $number = $interval / (CLX_360_DAYS / CLX_NUM_FOUR);
		  break;
        case '3'://Four-monthly 1/3(3)
		  $number = $interval / (CLX_360_DAYS / CLX_NUM_THREE);
		  break;
        case '2'://Semiannually(2)
		  $number = $interval / (CLX_360_DAYS / CLX_NUM_TWO);
		  break;
        case '1'://Annually(1)
		  $number = $interval;
		  break;
        default:
		  $sdate = CLX_INIT_ZERO;
		  break;
    }
    return floor($number);
}

/**
 * Calculates dates
 * @param unknown_type $start_date
 * @param unknown_type $end_date
 * @param unknown_type $method
 * @return number
 */
function date_diff360($start_date, $end_date, $method = false) {
    if (!class_exists('Financial'))
        require 'classes/Financial.php';
    $a = new Financial();
    return $a->DAYS360(adodb_strtotime($start_date), adodb_strtotime($end_date), $method);
}

/**
 * Workaround for PHP < 5.3.0
 */
if (!function_exists('date_diff')) {

    class DateInterval {

        public $y;
        public $m;
        public $d;
        public $h;
        public $i;
        public $s;
        public $invert;

        public function format($format) {
		  $format = str_replace('%R%y', ($this->invert ? '-' : '+') . $this->y, $format);
		  $format = str_replace('%R%m', ($this->invert ? '-' : '+') . $this->m, $format);
		  $format = str_replace('%R%d', ($this->invert ? '-' : '+') . $this->d, $format);
		  $format = str_replace('%R%h', ($this->invert ? '-' : '+') . $this->h, $format);
		  $format = str_replace('%R%i', ($this->invert ? '-' : '+') . $this->i, $format);
		  $format = str_replace('%R%s', ($this->invert ? '-' : '+') . $this->s, $format);
		  $format = str_replace('%y', $this->y, $format);
		  $format = str_replace('%m', $this->m, $format);
		  $format = str_replace('%d', $this->d, $format);
		  $format = str_replace('%h', $this->h, $format);
		  $format = str_replace('%i', $this->i, $format);
		  $format = str_replace('%s', $this->s, $format);
		  return $format;
        }

    }

    function date_diff(DateTime $date1, DateTime $date2) {
        $diff = new DateInterval();
        if ($date1 > $date2) {
		  $tmp = $date1;
		  $date1 = $date2;
		  $date2 = $tmp;
		  $diff->invert = true;
        }
        $diff->y = ((int) $date2->format('Y')) - ((int) $date1->format('Y'));
        $diff->m = ((int) $date2->format('n')) - ((int) $date1->format('n'));
        if ($diff->m < CLX_NUM_ZERO) {
		  $diff->y -= CLX_NUM_ONE;
		  $diff->m = $diff->m + CLX_NUM_12;
        }
        $diff->d = ((int) $date2->format('j')) - ((int) $date1->format('j'));
        if ($diff->d < CLX_NUM_ZERO) {
		  $diff->m -= CLX_NUM_ONE;
		  $diff->d = $diff->d + ((int) $date1->format('t'));
        }
        $diff->h = ((int) $date2->format('G')) - ((int) $date1->format('G'));
        if ($diff->h < CLX_NUM_ZERO) {
		  $diff->d -= CLX_NUM_ONE;
		  $diff->h = $diff->h + CLX_NUM_24;
        }
        $diff->i = ((int) $date2->format('i')) - ((int) $date1->format('i'));
        if ($diff->i < CLX_NUM_ZERO) {
		  $diff->h -= CLX_NUM_ONE;
		  $diff->i = $diff->i + CLX_NUM_60;
        }
        $diff->s = ((int) $date2->format('s')) - ((int) $date1->format('s'));
        if ($diff->s < CLX_NUM_ZERO) {
		  $diff->i -= CLX_NUM_ONE;
		  $diff->s = $diff->s + CLX_NUM_60;
        }
        return $diff;
    }

}