<?php

/**
 * General Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-10-09 11:22:02 -0400 (Tue, 09 Oct 2018) $
 * @version        $Rev: 10222 $
 * @package		Clientix.Classes.File
 * @abstract	General Class
 * CHANGE LOG
 *  Included label
 *  CR write() method added
 *  CR readline() method and documentation added.
 *  CR delete_ending_crlf() method added
 */
/*
 * Plain text files management
 */
class clx_file {

    var $status;
    var $handle;
    var $file_name;
    var $file_size;
    var $mode;
    var $eof;
    var $num_lines;
    var $line_num;
    var $error;
    var $errorMsg;
    var $debug;

    public function getError() {
        return $this->error;
    }

    public function hasError() {
        return ($this->error) ? true : false;
    }

    public function setError($error) {
        $this->error = $error;
    }

    public function getErrorMsg() {
        return $this->errorMsg;
    }

    public function setErrorMsg($errorMsg) {
        $this->errorMsg = $errorMsg;
        $this->setError(!empty($errorMsg));
    }

    function __construct($file_name, $mode) {
        $this->status = 'CLOSE';
        $this->file_name = $file_name;
        $this->mode = $mode;
        $this->line_num = CLX_INIT_ZERO;
        $this->num_lines = CLX_INIT_ZERO;
        $this->error = false;
        $this->errorMsg = '';
        $this->debug = false;
		IF (ISSET($GLOBALS['ADDRDEBUGIMPORT']) && (empty($_SERVER['REMOTE_ADDR']) || $GLOBALS['ADDRDEBUGIMPORT']==$_SERVER['REMOTE_ADDR'])) {
			$this->debug=true;
		}	
		else if ( ISSET($GLOBALS['ADDRDEBUGIMPORTREP']) && ($GLOBALS['ADDRDEBUGIMPORTREP']==$_SERVER['REMOTE_ADDR'] or $GLOBALS['ADDRDEBUGIMPORTREP']=='(1)' )){
			$this->debug=true; 
			$this->debugescribir=true;
		}
    }

    /**
     *
     */
    function open() {
    	if ($this->debug) {
			$tta = fopen(kPathToWebSite . "/" . kPathToTemp . "archivo".($this->debug?"_file":"").".txt", "a+");
			fwrite($tta, chr(13) . chr(10) . __LINE__. "(23)-hora:" . date("h:i:s d/m/Y") . chr(13) . chr(10));
		
			fclose($tta);
		}
        $this->error = false;
        if ($this->checkFileAccess()) {
		  $this->handle = fopen($this->file_name, $this->mode);
		  if (!is_resource($this->handle)) {
			 $this->setErrorMsg(dict('GEN_ERR_CANT_OPEN_FILE', $this->fileGetPathFileName() . ' [0]'));
		  } else {
			 $this->status = 'OPEN';
		  }
        } else {
		  $this->setErrorMsg(dict('GEN_ERR_CANT_OPEN_FILE', $this->fileGetPathFileName() . ' [1]'));
        }
    }

    /**
     * Returns type
     * @return type
     */
    function size() {
        $this->file_size = filesize($this->file_name);
        if (empty($this->file_size)) {
		  $this->error = true;
		  $this->errorMsg = dict('GEN_ERR_CANT_GET_FILE_SIZE', $this->fileGetPathFileName());
        }
        return $this->file_size;
    }

    /*
     * close the flat file
     */

    function close() {
        if ($this->status === 'OPEN') {
		  fclose($this->handle);
		  $this->status = 'CLOSE';
        } else {
		  $this->error = true;
		  $this->errorMsg = dict('GEN_LBL_CANT_CLOSE_FILE', $this->fileGetPathFileName());
        }
    }

    /**
     * Test for file status
     * @return boolean
     */
    private function checkFileAccess() {

        $fileinfo = $this->filePathInfo();
        $filefound = file_exists($this->file_name);
        $dirfound = file_exists($fileinfo['dirname']);
        $dirwritable = is_writable($fileinfo['dirname']);
        $readable = is_readable($this->file_name);
        $writable = is_readable($this->file_name);
        $return = false;
        switch ($this->mode) {
		  case 'r':
                $return = ($filefound /*&& $readable*/) ? true : false;
			 break;
		  case 'r+':
			 $return = ($filefound && $readable && $writable) ? true : false;
			 break;
		  case 'w':
		  case 'w+':
            case 'wb':                      
		  case 'a':

		  case 'a+':
                $return =  (($dirfound /*&& $dirwritable && !$filefound*/) || ($filefound /* && $writable*/)) ? true : false;
			 break;
		  case 'x':
			 $return = (($dirfound && $dirwritable && !$filefound)) ? true : false;
			 break;
		  case 'x+':
			 $return = (($dirfound && $dirwritable && !$filefound)) ? true : false;
			 break;
		  case 'c':
			 $return = (($dirfound && $dirwritable && !$filefound) || ($filefound && $writable)) ? true : false;
			 break;
		  case 'c+':
			 $return = (($dirfound && $dirwritable && !$filefound) || ($filefound && $writable)) ? true : false;
			 break;
		  default:
			 $return = false;
			 break;
        }

        //validate file name spacial characters
        if ($return && !xValidate::checkFilenameChars($this->file_name)) {
		  $return = false;
        }
        return $return;
    }

    /**
     * Destructor. Close any file if open.
     */
    function __destruct() {
        if (is_resource($this->handle)) {
		  fclose($this->handle);
        }
    }

    /**
     * Reads entire file (or file portion from 1st chacarter up to the
     * specified lenght) if lenght omited will read entire file
     * @param integer $length
     * @return string
     */
    function read($length = false) {
        if (empty($length)) {
		  $length = $this->size();
        }
        return fread($this->handle, $length);
    }

    /**
     * Writes string to file
     * @param string $s_line_content
     */
    function write($s_line_content) {if ( !empty($_SESSION["_dEBUG"]) && $_SESSION["_dEBUG"]==99 ) {$posx=0;$pos=1627;$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");$te=debug_backtrace(); for ($t=0;$t<12;$t++) {if (isset($te[$t]["file"])){if ($t==$posx)fwrite($tta,chr(13).chr(10).$pos."_".$te[$posx]["function"]."_db_import-hora:".date("h:i:s d/m/Y").chr(13).chr(10));fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));}}fclose($tta);}
        if (fwrite($this->handle, $s_line_content) === false) {
		  $this->error = true;
		  $this->errorMsg = dict('GEN_LBL_CANT_WRITE_FILE');
        }
    }

    /*
     * reads a line from file
     */

    function read_line($length = CLX_NUM_4096) {if ( !empty($_SESSION["_dEBUG"]) && $_SESSION["_dEBUG"]==99 ) {$posx=0;$pos=1627;$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");$te=debug_backtrace(); for ($t=0;$t<12;$t++) {if (isset($te[$t]["file"])){if ($t==$posx)fwrite($tta,chr(13).chr(10).$pos."_".$te[$posx]["function"]."_db_import-hora:".date("h:i:s d/m/Y").chr(13).chr(10));fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));}}fclose($tta);}
        $buffer = null;
		if (!empty($this->handle)){
       		if (!($this->eof = feof($this->handle))) {
		  	$buffer = fgets($this->handle, $length);
        	}
		}
        return $buffer;
    }

    /**
     * Return the number of lines in a file
     * @return integer
     */
    function line_count() {if ( !empty($_SESSION["_dEBUG"]) && $_SESSION["_dEBUG"]==99 ) {$posx=0;$pos=1627;$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");$te=debug_backtrace(); for ($t=0;$t<12;$t++) {if (isset($te[$t]["file"])){if ($t==$posx)fwrite($tta,chr(13).chr(10).$pos."_".$te[$posx]["function"]."_db_import-hora:".date("h:i:s d/m/Y").chr(13).chr(10));fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));}}fclose($tta);}
        $linecount = (int) CLX_NUM_ZERO;
        if (!empty ($this->handle) ) {
 	       rewind($this->handle);
        	while ($this->read_line()) {
		  	$linecount++;
        	}
        	rewind($this->handle);
		}
        return $linecount;
    }

    /**
     * Removes the trailing CR/LFs (line endings)
     */
    function delete_ending_crlf($s_ch) {
        for ($i = CLX_NUM_ZERO; $i < CLX_NUM_TWO; $i++) {
		  if (ord(substr($s_ch, CLX_NUM_MINUS_1)) == CLX_NUM_13 || ord(substr($s_ch, CLX_NUM_MINUS_1)) == CLX_NUM_TEN) {
			 $s_ch = substr($s_ch, CLX_NUM_ZERO, strlen($s_ch) - CLX_NUM_ONE);
		  }
        }
        return $s_ch;
    }

// Method End
    /**
     * Copy file from to
     *
     * @param string $from_filename
     * @param string $to_filename
     * @return boolean
     */
    static function copy($from_filename, $to_filename) {
        return copy($from_filename, $to_filename);
    }
	 /**
     * Copy file from to
     *
     * @param string $from_filename
     * @param string $to_filename
     * @return boolean
     */
    static function copyStatic($from_filename, $to_filename) {
        return copy($from_filename, $to_filename);
    }

    /**
     * Move or rename file
     *
     * @param string $from_filename
     * @param string $to_filename
     * @return boolean
     */
    function rename($from_filename, $to_filename) {
        return rename($from_filename, $to_filename);
    }

    /**
     * Move or rename file verifing if operation was successfully accomplished
     *
     * @param string $from_filename
     * @param string $to_filename
     * @return boolean
     */
    function renameVerify($from_filename, $to_filename) {
        $rename_error = false;
        $rename_text = array();
        if (!$this->rename($from_filename, $to_filename) || $this->fileExists_($from_filename)) {
		  // Rename throws an error
		  $rename_text[] = 'RENAME';
		  // File could not be renamed with rename function, so try copy and delete
		  if (!$this->copy($from_filename, $to_filename) || !$this->fileExists_($to_filename)) {
			 // copy couldn´t be done
			 $rename_error = true;
			 $rename_text[] = 'COPY';
		  } else {
			 // Copy was successfully, so delete source file
			 unlink($from_filename);
			 if ($this->fileExists_($from_filename)) {
			     // deletion couldn´t be done
			     $rename_error = true;
			     $rename_text[] = 'UNLINK';
			 }
		  }
        }
        if ($rename_error === true) {
		  $rename_text = 'DIRECTORY RIGHTS ERROR ON RENAME OF ' . singleQuote($from_filename) . ' to ' . singleQuote($to_filename) . ' (OPERATION: ' . implode(', ', $rename_text) . ')';
        }
        return array('error' => $rename_error,
		  'errormsg' => $rename_text
        );
    }

    function setFilename($filename) {
        if ($this->status === 'OPEN') {
		  $this->close();
        }
        $this->file_name = $filename;
    }

    function fileExists_() {
        return file_exists($this->file_name);
    }

    static function deleteFileByName($filename) {

        if (file_exists($filename) && is_readable($filename)) {
		  unlink($filename);
        }
        return (!file_exists($filename)) ? true : false;
    }

    /**
     * Returns string with mime type
     * @return string
     */
    function getMimeType() {
        $mimes = array();
        $mimes['jpg'] = 'image/jpeg';
        $mimes['bmp'] = 'image/bmp';
        $mimes['gif'] = 'image/gif';
        $mimes['ief'] = 'image/ief';
        $mimes['jpe'] = 'image/jpeg';
        $mimes['jpeg'] = 'image/jpeg';
        $mimes['jpg'] = 'image/jpeg';
        $mimes['png'] = 'image/png';
        $mimes['tiff'] = 'image/tiff';
        $mimes['tif'] = 'image/tiff';
        $mimes['tiff'] = 'image/tiff';
        $mimes['png'] = 'image/x-png';
        $ext = lowercase($this->getFileExtension());
        return (array_key_exists($ext, $mimes)) ? $mimes[$ext] : 'application/octet-stream';
    }

    /**
     * Returns file extension
     * @return string
     */
    function getFileExtension() {
        $f = explode('.', $this->file_name);
        return (!empty($f[CLX_NUM_ONE])) ? $f[CLX_NUM_ONE] : '';
    }

    /**
     * Returns file path information
     * @return mixed
     */
    function filePathInfo($options = 0) {
        $return = ($options === 0) ? pathinfo($this->file_name) : pathinfo($this->file_name, $options);
        return $return;
    }

    /**
     * Returns file path
     * @return type
     */
    function fileGetPathFileName() {
        $filepath = $this->filePathInfo(PATHINFO_FILENAME);
        return (!empty($filepath)) ? $filepath : '';
    }

}

// Class End
/**
 * Log files management
 */
class clx_logfile_mngr {

    var $o_logfile_hndl;
    var $error;
    var $errorMsg;

    /**
     * constructor: opens log file for writting
     */
    function __construct($s_log_filename) {
        $this->o_logfile_hndl = new clx_file($s_log_filename, 'w');
        $this->o_logfile_hndl->open();
        $this->error = $this->o_logfile_hndl->error;
        $this->errorMsg = $this->o_logfile_hndl->errorMsg;
        if (!empty($this->error))
		  $this->errorMsg .= ' (' . $s_log_filename . ' )';
    }

// Method End
    /**
     * Writes contents into log file
     */
    function write($s_response) {
        $this->o_logfile_hndl->write($s_response . CLX_CHAR_NL);
        $this->error = $this->o_logfile_hndl->error;
        $this->errorMsg = $this->o_logfile_hndl->errorMsg;
        return $this->error;
    }

// Method End
    /**
     * close log file
     */
    function close() {
        $this->o_logfile_hndl->close();
        $this->error = $this->o_logfile_hndl->error;
        $this->errorMsg = $this->o_logfile_hndl->errorMsg;
        return $this->error;
    }

// Method End
}
