<?php

/**
 * General Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-02-18 19:07:35 -0400 (lu., 18 feb. 2019) $
 * @version      $Rev: 10552 $
 * @package		 Clientix.Classes.Ldap
 * @abstract	General Ldap Class
 *
 * CHANGE LOG
 *  Relase
 */
class clx_ldap {

    private $cnx = null;
    private $bnd = null;
    private $err = false;
    private $err_no = CLX_INIT_ZERO;
	private $extended_error = '';
    public $debug = false;
    private $active_directory = false;
    private $ldap_host = '';
    private $ldap_port = CLX_NUM_389;   // Default port
    private $ldap_base_dn = ''; // o=University of Virginia,c=US';
    private $ldap_base_dn_ar = array(); // o=University of Virginia,c=US';
    private $ldap_search_fieldname = '';
    private $extended_open_ldap = false;
    private $extended_open_ldap_user = '';

    /**
     * Constructor
     * @param string $ldap_host
     * @param integer $ldap_port
     * @param string $ldap_base_dn
     */
    function __construct($ldap_host, $ldap_port = CLX_NUM_389, $ldap_base_dn = '') {
        $this->sethost($ldap_host);
        $this->setport($ldap_port);
        $this->setbasedn($ldap_base_dn);
    }

    /**
     * Set ldap host
     * @param string $ldap_host
     */
    function sethost($ldap_host) {
        $this->ldap_host = (string) $ldap_host;
    }

	/**
     * Get ldap host
     * @return string
     */
    function gethost() {
        return $this->ldap_host;
    }

    /**
     * Set ldap port
     * @param int $ldap_port
     */
    function setport($ldap_port = CLX_NUM_386) {
        $this->ldap_port = (int) $ldap_port;
    }

    /**
     * Get ldap port
     * @return string
     */
	function getport() {
        return $this->ldap_port;
    }

    /**
     * Set ldap host
     * @param string $ldap_host
     */
    function setbasedn($ldap_base_dn) {
        $this->ldap_base_dn = (string) $ldap_base_dn;
		$this->set_final_base_dn();
    }

    /**
     * Get ldap host
     * @return string
     */
    function getbasedn() {
        return $this->ldap_base_dn;
    }

    /**
     * Sets active directory
     * @param boolean $boolean
     */
    function set_active_directory($boolean = false) {
        $this->active_directory = $boolean;
    }

    /**
     * Sets special Open LDAP
     * @param boolean $boolean
     */
    function set_extended_open_ldap($boolean = false) {
        $this->extended_open_ldap = $boolean;
    }

    /**
     * Sets search_fieldname
     * @param string $ldap_search_fieldname
     */
    function set_search_fieldname($ldap_search_fieldname = '') {
        $this->ldap_search_fieldname = $ldap_search_fieldname;
    }

    /**
     * Return true if service is active directory
     * @return boolean
     */
    function get_active_directory() {
        return $this->active_directory;
    }

    /**
     * Return true if service is active directory
     * @return boolean
     */
    function get_extended_open_ldap() {
        return $this->extended_open_ldap;
    }

    /**
     * Returns search_fieldname
     * @return string
     */
    function get_search_fieldname() {
        return $this->ldap_search_fieldname;
    }

    /**
     * Set true to record error number.
     * @param bool $error_flag
     */
    function seterror($error_flag) {
		if (empty($this->cnx)) {
			$this->err =true;
		}
		else {
	        $this->err_no = (!$error_flag) ? CLX_NUM_ZERO : ldap_errno($this->cnx);
			$this->extended_error = '';
			if (function_exists('ldap_get_option') && ldap_get_option($this->cnx, 0x0032, $extended_error)) {
				$this->extended_error = $extended_error;
			}
			if (!empty($this->debug)) {
				echolog($this->err_no." ".((is_array($extended_error))?print_r($extended_error,true):$extended_error));
			}
			$this->err = ($this->err_no) ? true : false;
		}
    }

    /**
     * Returns error number
     * @return number
     */
    function geterrorno() {
        return (int) $this->err_no;
    }

    /**
     * Return error string
     * @return string
     */
    function geterrormsg() {
        return ldap_err2str($this->geterrorno());
    }

    /**
     * Returns error number
     * @return string <string>
     */
    function geterrorlabel() {
        //Common LDAP error codes
        $err = array(CLX_NUM_0x00 => 'LDAP_SUCCESS',
		  CLX_NUM_0x01 => 'LDAP_OPERATIONS_ERROR',
		  CLX_NUM_0x02 => 'LDAP_PROTOCOL_ERROR',
		  CLX_NUM_0x03 => 'LDAP_TIMELIMIT_EXCEEDED',
		  CLX_NUM_0x04 => 'LDAP_SIZELIMIT_EXCEEDED',
		  CLX_NUM_0x05 => 'LDAP_COMPARE_false',
		  CLX_NUM_0x06 => 'LDAP_COMPARE_true',
		  CLX_NUM_0x07 => 'LDAP_AUTH_METHOD_NOT_SUPPORTED',
		  CLX_NUM_0x08 => 'LDAP_STRONG_AUTH_REQUIRED',
		  CLX_NUM_0x09 => 'LDAP_PARTIAL_RESULTS',
		  CLX_NUM_0x0a => 'LDAP_REFERRAL',
		  CLX_NUM_0x0b => 'LDAP_ADMINLIMIT_EXCEEDED',
		  CLX_NUM_0x0c => 'LDAP_UNAVAILABLE_CRITICAL_EXTENSION',
		  CLX_NUM_0x0d => 'LDAP_CONFIDENTIALITY_REQUIRED',
		  CLX_NUM_0x0e => 'LDAP_SASL_BIND_INPROGRESS',
		  CLX_NUM_0x10 => 'LDAP_NO_SUCH_ATTRIBUTE',
		  CLX_NUM_0x11 => 'LDAP_UNDEFINED_TYPE',
		  CLX_NUM_0x12 => 'LDAP_INAPPROPRIATE_MATCHING',
		  CLX_NUM_0x13 => 'LDAP_CONSTRAINT_VIOLATION',
		  CLX_NUM_0x14 => 'LDAP_TYPE_OR_VALUE_EXISTS',
		  CLX_NUM_0x15 => 'LDAP_INVALID_SYNTAX',
		  CLX_NUM_0x20 => 'LDAP_NO_SUCH_OBJECT',
		  CLX_NUM_0x21 => 'LDAP_ALIAS_PROBLEM',
		  CLX_NUM_0x22 => 'LDAP_INVALID_DN_SYNTAX',
		  CLX_NUM_0x23 => 'LDAP_IS_LEAF',
		  CLX_NUM_0x24 => 'LDAP_ALIAS_DEREF_PROBLEM',
		  CLX_NUM_0x30 => 'LDAP_INAPPROPRIATE_AUTH',
		  CLX_NUM_0x31 => 'LDAP_INVALID_CREDENTIALS',
		  CLX_NUM_0x32 => 'LDAP_INSUFFICIENT_ACCESS',
		  CLX_NUM_0x33 => 'LDAP_BUSY',
		  CLX_NUM_0x34 => 'LDAP_UNAVAILABLE',
		  CLX_NUM_0x35 => 'LDAP_UNWILLING_TO_PERFORM',
		  CLX_NUM_0x36 => 'LDAP_LOOP_DETECT',
		  CLX_NUM_0x3C => 'LDAP_SORT_CONTROL_MISSING',
		  CLX_NUM_0x3D => 'LDAP_INDEX_RANGE_ERROR',
		  CLX_NUM_0x40 => 'LDAP_NAMING_VIOLATION',
		  CLX_NUM_0x41 => 'LDAP_OBJECT_CLASS_VIOLATION',
		  CLX_NUM_0x42 => 'LDAP_NOT_ALLOWED_ON_NONLEAF',
		  CLX_NUM_0x43 => 'LDAP_NOT_ALLOWED_ON_RDN',
		  CLX_NUM_0x44 => 'LDAP_ALREADY_EXISTS',
		  CLX_NUM_0x45 => 'LDAP_NO_OBJECT_CLASS_MODS',
		  CLX_NUM_0x46 => 'LDAP_RESULTS_TOO_LARGE',
		  CLX_NUM_0x47 => 'LDAP_AFFECTS_MULTIPLE_DSAS',
		  CLX_NUM_0x50 => 'LDAP_OTHER',
		  CLX_NUM_0x51 => 'LDAP_SERVER_DOWN',
		  CLX_NUM_0x52 => 'LDAP_LOCAL_ERROR',
		  CLX_NUM_0x53 => 'LDAP_ENCODING_ERROR',
		  CLX_NUM_0x54 => 'LDAP_DECODING_ERROR',
		  CLX_NUM_0x55 => 'LDAP_TIMEOUT',
		  CLX_NUM_0x56 => 'LDAP_AUTH_UNKNOWN',
		  CLX_NUM_0x57 => 'LDAP_FILTER_ERROR',
		  CLX_NUM_0x58 => 'LDAP_USER_CANCELLED',
		  CLX_NUM_0x59 => 'LDAP_PARAM_ERROR',
		  CLX_NUM_0x5a => 'LDAP_NO_MEMORY',
		  CLX_NUM_0x5b => 'LDAP_CONNECT_ERROR',
		  CLX_NUM_0x5c => 'LDAP_NOT_SUPPORTED',
		  CLX_NUM_0x5d => 'LDAP_CONTROL_NOT_FOUND',
		  CLX_NUM_0x5e => 'LDAP_NO_RESULTS_RETURNED',
		  CLX_NUM_0x5f => 'LDAP_MORE_RESULTS_TO_RETURN',
		  CLX_NUM_0x60 => 'LDAP_CLIENT_LOOP',
		  CLX_NUM_0x61 => 'LDAP_REFERRAL_LIMIT_EXCEEDED',
		  -CLX_NUM_ONE => 'LDAP_CONNECT_ERROR',
		  '' => 'LDAP_ERR_UNKNOWN');
        return (string) $err[$this->geterrorno()];
    }

	function getExtendedError() {
		return $this->extended_error;
	}
	
    /**
     * Return falses if error has occured
     * @return boolean
     */
    function haserror() {
        return ($this->err == CLX_NUM_ZERO) ? false : true;
    }

    /**
     * Set ldap option
     * @param integer $opt_name
     * @param mixed $opt_value
     */
    function set_ldap_option($opt_name, $opt_value) {
        ldap_set_option($this->cnx, $opt_name, $opt_value);
    }
    
    /**
     * Verify LDAP PHP extension library availability and then call ldap_connect()
     * @return boolean
     */
    function ldap_verify_and_connect() {
        if(!function_exists('ldap_connect')) {
		  $this->err_no = 0x01;   // Intialization of LDAP library failed.
		  $this->err = true;
		  return false;
        }
        $old_error_reporting = error_reporting(E_ALL);
        if (!empty($this->debug)) {
		  echolog('host: '.$this->ldap_host."<br>");
			echolog('port: '.$this->ldap_port,"<br>");
	    }
        $this->cnx = ldap_connect($this->ldap_host, $this->ldap_port);
        error_reporting($old_error_reporting);
		if (!empty($this->debug)) {
		  echolog('host: '.$this->ldap_host."<br>");
		}
        return true;
    }

    /**
     * Connects to ldap server
     */
    function connect() {
        if(!$this->ldap_verify_and_connect())
		  return;
        if (!empty($this->debug))
		  echoLog('Connecting status: ');
        if (empty($this->cnx)) {
		  $this->seterror(true);
		  if (!empty($this->debug))
			 echoLog('Fail<Br>');
        } else {
		  if (!empty($this->debug))
			 echoLog('Ok<Br>');
        }
    }

    /**
     * Connects to ldap server and binds connection
     * @param string $ldap_rdn
     * @param string $ldap_pass
     */
    function connectandbind($ldap_rdn = null, $ldap_pass = null) {
        if(!$this->ldap_verify_and_connect())
		  return;
        if (!empty($this->debug))
		  echoLog('Connecting status ' . $this->ldap_host . ':' . $this->ldap_port . ': ');
        if (empty($this->cnx)) {
		  $this->seterror(true);
		  if (!empty($this->debug))
			 echoLog('Fail<br>');
        }else {
		  if (!empty($this->debug))
			 echoLog('Ok<br>');
		  $this->bind($ldap_rdn, $ldap_pass);
        }
    }

    /**
     * Bind user and password to ldap server
     * @param string $ldap_rdn
     * @param string $ldap_pass
     */
    function bind($ldap_rdn = null, $ldap_pass = null) {
		foreach($this->ldap_base_dn_ar as $one_ldap_base_dn) {
			$this->extended_open_ldap_user = $ldap_rdn;
			if (!empty($this->debug))
				echoLog('ACTIVE USER:'.$ldap_rdn.' <br>');
			if (!empty($this->debug))
				echoLog('ACTIVE PASS:'.$ldap_pass.' <br>');
			
			if ($this->get_active_directory()) {
				// AD or Active Directory
				if (!empty($this->debug))
					echoLog('ACTIVE DIRECTORY: 1<br>');
				if (!empty($this->debug))
					echoLog('LDAP_OPT_REFERRALS: 0<br>');
				if (!empty($this->debug))
					echoLog('LDAP_OPT_PROTOCOL_VERSION: 3<br>');
				$this->set_ldap_option(LDAP_OPT_REFERRALS, CLX_NUM_ZERO);
				$this->set_ldap_option(LDAP_OPT_PROTOCOL_VERSION, CLX_NUM_THREE);
			} elseif($this->get_extended_open_ldap()) {
				// Extended open LDAP adds 'uid=' and Base DN to user before call ldap_bind PHP function [2016-09-21 CR]
				if (!empty($this->debug))
					echoLog('EXTENDED_OPEN_LDAP: 1<br>');
				if (!empty($this->debug))
					echoLog('LDAP_OPT_REFERRALS: 0<br>');
				if (!empty($this->debug))
					echoLog('LDAP_OPT_PROTOCOL_VERSION: 3<br>');
				$this->set_ldap_option(LDAP_OPT_PROTOCOL_VERSION, CLX_NUM_THREE);
				$ldap_rdn = 'uid=' . $this->extended_open_ldap_user .','.$this->ldap_base_dn;
			}
			$old_error_reporting = error_reporting(E_ALL);
			$this->bnd = ldap_bind($this->cnx, $one_ldap_base_dn, $ldap_pass);
			error_reporting($old_error_reporting);
			if (!empty($this->debug))
				echoLog('Bind status: ');
			if (empty($this->bnd)) {
				if (!empty($this->debug))
					echoLog('Fail<br>');
				$this->seterror(true);
			}else {
				if (!empty($this->debug))
					echoLog('Ok<br>');
				return;
			}
		}
    }

    /**
     * Close connection to ldap server
     */
    function close() {
        ldap_close($this->cnx);
        $this->cnx = null;
        $this->bnd = null;
        $this->seterror(false);
    }

	function set_final_base_dn($dn_to_split = null) {
		$change=false; 
		if(is_null($dn_to_split)) {
			$change=true;
			$dn_to_split = $this->ldap_base_dn;
		}
		
		preg_match_all('/\{([^}]+)\}/', $dn_to_split, $retr);
		$retr=$retr[1];
		if (count($retr)>0){
			if ($change==true)
				$this->ldap_base_dn_ar=$retr;
			return $retr;
		}
		else {
			if ($change==true)
			$this->ldap_base_dn_ar = array($dn_to_split);
			return $dn_to_split;
		}
	}
			
    /**
     * Perform Search
     * @param unknown_type $ldap_filter
     * @param array $ldap_attributes = null
     * @param unknown_type $ldap_attrsonly = null
     * @param unknown_type $ldap_sizelimit = 0
     * @param unknown_type $ldap_timelimit = 0
     * @param unknown_type $ldap_deref
     * @return resource
     */
    function search($ldap_filter, $ldap_attributes = null, $ldap_attrsonly = null, $ldap_sizelimit = CLX_NUM_ZERO, $ldap_timelimit = CLX_NUM_ZERO, $ldap_deref = null) {
        if (!empty($this->debug)) {
			echolog($this->cnx);
			echolog(print_r($this->ldap_base_dn_ar,true));
			echolog($ldap_filter);
			echolog($ldap_attributes);
			echolog($ldap_attrsonly);
			echolog($ldap_sizelimit);
			echolog($ldap_timelimit);
			echolog($ldap_deref);
		}
		
        //$r = ldap_search($this->cnx, $this->ldap_base_dn, $ldap_filter, $ldap_attributes, $ldap_attrsonly, $ldap_sizelimit, $ldap_timelimit, $ldap_deref);
		foreach($this->ldap_base_dn_ar as $one_ldap_base_dn) {
			$r = ldap_search($this->cnx, $one_ldap_base_dn, $ldap_filter, $ldap_attributes, $ldap_attrsonly, $ldap_sizelimit, $ldap_timelimit, $ldap_deref);
			fwrite($tta,$one_ldap_base_dn.chr(13).chr(10));	
							
			if (!$empty($r))
				break;
		}
        if (!empty($this->debug))
		  echoLog('Search Status for (' . $ldap_filter . '): ');
        if (empty($r)) {
		  $this->seterror(true);
		  if (!empty($this->debug)) {
			 echoLog('NO Ok<br>');
		  }
        } else {
		  if (!empty($this->debug)) {
			 echoLog('Ok<br>');
		  }
		  if (!empty($this->debug)) {
			 pre($r);
		  }
        }
        return $r;
    }

    /**
     * Returns array conteining search results
     * @param resouce $search_result
     * @return multitype:
     */
    function get_entries($search_result) {
        return ldap_get_entries($this->cnx, $search_result);
    }

    /**
     * Returns user for conecting user
     * @param string $username
     * @param boolean $search_single_user
     * @param unknown_type $ldap_filter
     * @return multitype:
     */
    function get_user_info($username, $search_single_user = false) {
        //note: Other information regarding active ldap attributes search for 'Active Directory Entry Values'
        $return = array();
        // 2014-11-07 CR Get search field name, if not defined, set default based on LDAP system type
        $ldap_filter = $this->get_search_fieldname();
        if ($ldap_filter !== '') {
		  $ldap_filter = $ldap_filter . '=' . $username;
        } else {
		  if ($this->get_active_directory()) {
			 //  FS
			 $ldap_filter = 'sAMAccountName=' . $username;
		  } else {
			 $ldap_filter = 'cn=' . $username;
		  }
        }
        $ldap_attributes = array();
        $ldap_attrsonly = null;
        $ldap_sizelimit = ($search_single_user) ? CLX_NUM_ONE : CLX_NUM_ZERO;
        $ldap_timelimit = CLX_INIT_ZERO; //search time limit 0 means no limit. will not overrider server's default
        $ldap_deref = null;
		$this->ldap_search_options=array('ldap_base_dn_ar'=> $this->ldap_base_dn_ar, 'ldap_filter'=>$ldap_filter, 'ldap_attributes'=>$ldap_attributes, 'ldap_attrsonly'=>$ldap_attrsonly, 'ldap_sizelimit'=>$ldap_sizelimit, 'ldap_timelimit'=>$ldap_timelimit,'ldap_sizederef'=> $ldap_deref);
        $rs = $this->search($ldap_filter, $ldap_attributes, $ldap_attrsonly, $ldap_sizelimit, $ldap_timelimit, $ldap_deref);
        if ($rs) {
		  $info = $this->get_entries($rs);
		  if (!empty($this->debug))
			 pre($info);
		  if (!empty($search_single_user)) {
			 $return = $this->get_active_user_record($info[CLX_NUM_ZERO]);
		  } else {
			 for ($i = CLX_NUM_ZERO; $i < $info['count']; $i++) {
			     $return[] = $this->get_active_user_record($info[$i]);
			 }
		  }
        }
        return $return;
    }

    /**
     * Returns user formated according user record
     * @param unknown_type $response_item
     * @return multitype:number unknown string
     */
    function get_active_user_record($response_item) {
        $return = array();
        if ($this->get_active_directory()) {
			$single_view_response_item=true;
			for ($rtitem=CLX_NUM_ZERO;$rtitem<1000;$rtitem++) {
				if (!empty($response_item['givenname'][$rtitem])) {
					$return['FIRSTNAME'] = $response_item['givenname'][$rtitem];
					$return['LASTNAME'] = $response_item['sn'][$rtitem];
					$return['EXT_REF_NO'] = $response_item['initials'][$rtitem];
					$return['PHONE'] = $response_item['telephonenumber'][$rtitem];
					$return['EMAIL'] = $response_item['mail'][$rtitem];
					$return['FAX'] = $response_item['facsimiletelephonenumber'][$rtitem];
					$return['USERNAME'] = $response_item['samaccountname'][$rtitem];
					$return['LDAP_RDN_USERNAME'] = $response_item['cn'][$rtitem];
					$return['POSITION'] = $response_item[CLX_CLASS_TITLE][$rtitem];
					$return['COMPANY'] = $response_item['company'][$rtitem];
					$return['MEMBEROF'] = $response_item['memberof'][$rtitem];
					$return['PRIMARYGROUPID'] = $response_item['primarygroupid'][$rtitem];
					$return['PASSWORD'] = substr(clx_mcript(uniqid(rand())),-32);
					$return['STATUS'] = ($response_item['useraccountcontrol'][$rtitem]) ? CLX_NUM_ONE : CLX_NUM_ZERO;
					$single_view_response_item=false;
					
				}
			}
			if($single_view_response_item==true) {
			 $return['FIRSTNAME'] = $response_item['givenname'];
			 $return['LASTNAME'] = $response_item['sn'];
			 $return['EXT_REF_NO'] = $response_item['initials'];
			 $return['PHONE'] = $response_item['telephonenumber'];
			 $return['EMAIL'] = $response_item['mail'];
			 $return['FAX'] = $response_item['facsimiletelephonenumber'];
			 $return['USERNAME'] = $response_item['samaccountname'];
			 $return['LDAP_RDN_USERNAME'] = $response_item['cn'];
			 $return['POSITION'] = $response_item[CLX_CLASS_TITLE];
			 $return['COMPANY'] = $response_item['company'];
			 $return['MEMBEROF'] = $response_item['memberof'];
			 $return['PRIMARYGROUPID'] = $response_item['primarygroupid'];
			 $return['PASSWORD'] = substr(clx_mcript(uniqid(rand())),-32);
			 $return['STATUS'] = ($response_item['useraccountcontrol']) ? CLX_NUM_ONE : CLX_NUM_ZERO;
		  }
        } else {
		  $return['FIRSTNAME'] = $this->get_ldap_field_value($response_item, 'givenname', 'cn');
		  $return['LASTNAME'] = $this->get_ldap_field_value($response_item, 'sn', 'cn');
		  $return['EXT_REF_NO'] = $this->get_ldap_field_value($response_item, 'initials', CLX_CHAR_EMPTY_STRING);
		  $return['PHONE'] = '';
		  $return['EMAIL'] = (!empty($response_item['mail'][CLX_NUM_ZERO])) ? $response_item['mail'][CLX_NUM_ZERO] : CLX_CHAR_EMPTY_STRING;
		  $return['FAX'] = '';
		  $return['USERNAME'] = $this->get_ldap_field_value($response_item, 'uid', CLX_CHAR_EMPTY_STRING);
		  $return['LDAP_RDN_USERNAME'] = $this->get_ldap_field_value($response_item, 'cn', CLX_CHAR_EMPTY_STRING);
		  $return['POSITION'] = ''; //$response_item[CLX_CLASS_TITLE];
		  $return['COMPANY'] = '';
		  $return['MEMBEROF'] = '';
		  $return['PRIMARYGROUPID'] = '';
		  $return['PASSWORD'] = substr(clx_mcript(uniqid(rand())),-32);
		  $return['STATUS'] = (!empty($response_item['useraccountcontrol'][CLX_NUM_ZERO])) ? CLX_NUM_ONE : CLX_NUM_ZERO;
        }
        $return['ISADMIN'] = CLX_INIT_ZERO;
        $return['AUDITU'] = CLX_INIT_ZERO;
        $return['LOGIN_ATTEMPTS'] = CLX_INIT_ZERO;
        $return['PASSEXPIRE'] = CLX_INIT_ZERO;
        $return['PASSREQCHANGE'] = CLX_INIT_ZERO;
        return $return;
    }

    /**
     * Returns LDAP search object entry field value, if it doesn't exist, return alternative field value, if not, returns blank
     * @param array $response_item
     * @param string $ldap_field_name
     * @param string $alt_ldap_field_name
     * @return string
     */
    function get_ldap_field_value($response_item, $ldap_field_name, $alt_ldap_field_name) {
        $response = CLX_CHAR_EMPTY_STRING;
        if (isset($response_item[$ldap_field_name][CLX_NUM_ZERO])) {
		  $response = $response_item[$ldap_field_name][CLX_NUM_ZERO];
        } elseif (isset($response_item[$ldap_field_name])) {
		  $response = $response_item[$ldap_field_name];
        } elseif ($alt_ldap_field_name !== CLX_CHAR_EMPTY_STRING && isset($response_item[$alt_ldap_field_name][CLX_NUM_ZERO])) {
		  $response = $response_item[$alt_ldap_field_name][CLX_NUM_ZERO];
        } elseif ($alt_ldap_field_name !== CLX_CHAR_EMPTY_STRING && isset($response_item[$alt_ldap_field_name])) {
		  $response = $response_item[$alt_ldap_field_name];
        }
        return $response;
    }
	function get_ldap_search_options () {
		return $this->ldap_search_options;
    }
}
