<?php

/**
 * Report Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-08-28 12:32:24 +0000 (Mon, 28 Aug 2017) $
 * @version        $Rev: 9499 $
 * @package		Clientix.Classes.Report
 * @abstract		Abstract Class
 */
class clx_report {

    /**
     * Report Styles
     * @var array
     */
    public $styles;

    /**
     * Report Header
     * @var clx_report_header
     */
    private $header;
    private $pages;

    /**
     * Report Footer
     * @var clx_report_footer
     */
    private $footer;

    /**
     * Page Numbers
     * @var integer
     */
    public $no_pages = CLX_INIT_ZERO;
    private $nl = CLX_CHAR_NL;
    private $orientation;
    private $unit;
    private $format;
    public $repLogo = false;
    private $logo_x = CLX_INIT_ZERO;
    private $logo_y = CLX_INIT_ZERO;
    private $logo_width = CLX_INIT_ZERO;
    private $logo_height = CLX_INIT_ZERO;
    private $logo_img = '';
    var $disable_borders = false;
    var $page_footer_use_def_address = true;
    var $page_footer_notes_msgs;
    var $page_footer_address;
    var $page_footer_total_extra_lines = CLX_NUM_TWO;
    var $page_footer_total_address_lines = CLX_NUM_TWO;
    var $send_pdf = false;
    var $resetMargin = 0;
    /**
     * Constructor
     */
    function __construct($orientation = 'P', $unit = 'mm', $format = 'Letter') {
        $this->pages = array();
        $this->no_pages = CLX_INIT_ZERO;
        $this->orientation = $orientation;
        $this->unit = $unit;
        $this->format = $format;
    }

    /**
     * Set Report Header
     * @param $header
     */
    function setHeader(clx_report_data $header) {
	    $this->header[] = $header;
    }

    /**
     * Set Report Header
     * @return data
     */
    function getHeader() {
        return $this->header;
    }

    /**
     * Sets Report footer
     * @param $footer
     */
    function setFooter($param, clx_report_data $footer) {
        $this->footer[$param] = $footer;
    }

    function footer_start($par_comp_code, $ficorec = null) {
        if (!defined('TOKEN_NO_ADDRESS_IN_FOOTPRINT')) {
		  define('TOKEN_NO_ADDRESS_IN_FOOTPRINT', '%NO_ADDRESS_IN_FOOTPRINT%'); // Para que no ponga la direcci�n por defecto en el pie de pagina
		  define('TOKEN_ADDRESS', '%ADDRESS%'); // Sustituye en el texto donde se ubique el token por la direcci�n por defecto.
		  define('TOKEN_FOOTPRINT_BEGIN', '%FOOTPRINT_BEGIN%');
		  define('TOKEN_FOOTPRINT_END', '%FOOTPRINT_END%'); // Sustituye el pie de pagina standard por este texto entre los tokens
        }
        $this->page_footer_use_def_address = true;
        $this->page_footer_address = $this->footer_get_company_address($par_comp_code);
    }

    function get_orientation() {
        return $this->orientation;
    }

    function get_format() {



        return $this->format;
    }

    function get_unit() {
        return $this->unit;
    }

    /**
     * Company Address for footprint
     * @param array $ficorec
     */
    function footer_get_company_address($par_comp_code, &$colorec = null) {
        $DBLT = &getTool('DBLT');
        multiTableName($this, array('COMP', 'COLO', 'DYCL'));
        $comprec = ClxCommon::$db->getRecord($this->compname, 'IDCODE = ' . singleQuote($par_comp_code));
        $sql = 'SELECT COLO.* FROM ' . $this->coloname . ' COLO, ' . $this->dyclname . ' DYCL  WHERE DYCL.COMP_CODE = ' . singleQuote($comprec['IDCODE']) . ' AND DYCL.COMPLOC_CODE = COLO.IDCODE ORDER BY COLO.ISPRIMARY DESC ';
        $colorec = ClxCommon::$db->getOneRecord($sql);
        if (empty($colorec ['ADDRESS1'])) {
		  $colorec ['ADDRESS1'] = '';
        } else {
		  $colorec ['ADDRESS1'] = $colorec ['ADDRESS1'] . ' ';
        }
        if (empty($colorec ['ADDRESS2'])) {
		  $colorec ['ADDRESS2'] = '';
        } else {
		  $colorec ['ADDRESS2'] = $colorec ['ADDRESS2'];
        }
        if (empty($colorec ['CITY'])) {
		  $colorec ['CITY'] = '';
        } else {
		  $colorec ['CITY'] = $DBLT->getListItem('CITI', $colorec ['CITY'], CLX_NUM_TWO);
        }
        if (empty($colorec ['STATE'])) {
		  $colorec ['STATE'] = '';
        } else {
		  $colorec ['STATE'] = $DBLT->getListItem('STAT', $colorec ['STATE'], CLX_NUM_TWO);
        }
        if (empty($colorec ['PHONE'])) {
		  $colorec ['PHONE'] = '';
        } else {
		  $colorec ['PHONE'] = ' Tel: ' . $colorec ['PHONE'];
        }
        if (empty($colorec ['FAX'])) {
		  $colorec ['FAX'] = '';
        } else {
		  $colorec ['FAX'] = ', Fax: ' . $colorec ['FAX'];
        }
        if (empty($colorec ['COUNTRY'])) {
		  $colorec ['COUNTRY'] = '';
        } else {
		  $colorec ['COUNTRY'] = $DBLT->getListItem('COUN', $colorec ['COUNTRY'], CLX_NUM_TWO);
        }
        $address = $colorec ['CITY'];
        $address .= ($address !== '' ? ', ' : '') . $colorec ['STATE'];
        $address .= ($address !== '' ? ', ' : '') . $colorec ['COUNTRY'];
        $address .= ($address !== '' ? '. ' : '');
        $address .= $colorec ['PHONE'];
        $address .= ($colorec ['PHONE'] !== '' ? '' : '') . $colorec ['FAX'];
		$address .= '.';
        $address = trim($colorec ['ADDRESS1'] . $colorec ['ADDRESS2']) . (!$this->send_pdf ? '<br>' : '. ') . $address;
        return $address;
    }

    /**
     * PDF Footer lines assigment
     */
    function footer_put_pdf_lines($ve_txt_par, $stle) {
        $ve_txt = $this->footer_process_special_tokens($ve_txt_par);
        $ve_txt = strip_tags(str_replace('<br>', CLX_CHAR_NL, $ve_txt));
        $ve_txt = str_replace(CLX_CHAR_NL . CLX_CHAR_NL, CLX_CHAR_NL, $ve_txt);
        $ve_txt = explode(CLX_CHAR_NL, $ve_txt);
        if (!is_array($ve_txt))
		  $ve_txt = array($ve_txt);
        $curr_line_counter = CLX_NUM_ONE;
        $total_footer_lines = $this->page_footer_total_extra_lines + $this->page_footer_total_address_lines;
        $address_line = CLX_NUM_ONE;
        foreach ($ve_txt as $key => $val) {
		  if ($curr_line_counter <= $this->page_footer_total_extra_lines) {
			 $fld_name_sf = 'extra' . $curr_line_counter;
		  } else {
			 $fld_name_sf = 'address' . ($curr_line_counter - $this->page_footer_total_extra_lines);
			 if ($curr_line_counter >= $total_footer_lines) {
			     $this->page_footer_use_def_address = false;
			 } else {
			     $address_line ++;
			 }
		  }
		  if (!empty($val)) {
			 $this->setFooter($fld_name_sf, new clx_report_data(strip_tags($val), $stle));
			 $curr_line_counter ++;
		  }
        }
        if ($this->page_footer_use_def_address === true) {
		  $fld_name_sf = 'address' . ($address_line);
		  $this->setFooter($fld_name_sf, new clx_report_data(strip_tags(str_replace('<br>', CLX_CHAR_NL, $this->page_footer_address)), $stle));
        }
    }

    function footer_process_special_tokens($str_par) {
        $line_comment = ' ' . $str_par; // adds space at the begining to avoid mb_strpos() y str_replace() misbehavior �?
        $token_begin = mb_strpos($line_comment, TOKEN_FOOTPRINT_BEGIN);
        if (false === !$token_begin) {
		  $token_end = mb_strpos($line_comment, TOKEN_FOOTPRINT_END);
		  $total_len = (($token_end + mb_strlen(TOKEN_FOOTPRINT_END)) - $token_begin);
		  $this->page_footer_address = mb_substr($line_comment, $token_begin, $total_len);
		  $line_comment = str_replace($this->page_footer_address, '', $line_comment);
		  $this->page_footer_address = str_replace(TOKEN_FOOTPRINT_BEGIN, '', $this->page_footer_address);
		  $this->page_footer_address = str_replace(TOKEN_FOOTPRINT_END, '', $this->page_footer_address);
		  unset($token_begin, $token_end, $total_len);
        }
        if (!mb_strpos($line_comment, TOKEN_NO_ADDRESS_IN_FOOTPRINT) === false) {
		  $line_comment = str_replace(TOKEN_NO_ADDRESS_IN_FOOTPRINT, '', $line_comment);
		  $this->page_footer_use_def_address = false;
        }
        if (!mb_strpos($line_comment, TOKEN_ADDRESS) === false) {
		  $line_comment = str_replace(TOKEN_ADDRESS, $this->page_footer_address, $line_comment);
        }
        $line_comment = mb_substr($line_comment, CLX_NUM_ONE); // deletes space at the begining to avoid mb_strpos() y str_replace() misbehavior �?
        if ($this->send_pdf === true) {
		  $line_comment = str_replace('<br>', CLX_CHAR_NL, $line_comment);
        } else {
		  $line_comment = str_replace(CLX_CHAR_NL, '<br>', $line_comment);
        }
        return $line_comment;
    }

    /**
     * Add Page
     * @param xReportPage $page
     */
    function addPage(clx_report_page $page) {
        $this->pages[] = $page;
        $this->no_pages = count($this->pages);
    }

    /**
     * Add Styles
     * @param $styles
     */
    function addStyles(clx_style $styles) {
        $this->styles[] = $styles;
    }

    /**
     * Returns Report Logo
     * @return string
     */
    function getReplogo() {
        return $this->logo_img;
    }

    /**
     * Returns Report Logo
     * @param unknown_type $url
     */
    function setReplogo($filename, $x = CLX_NUM_ZERO, $y = CLX_NUM_ZERO, $w = CLX_NUM_ZERO, $h = CLX_NUM_ZERO) {
        $this->logo_x = $x;
        $this->logo_y = $y;
        $this->logo_width = $w;
        $this->logo_height = $h;
        $this->logo_img = $filename;
        $this->repLogo = true;
    }

    /**
     * Return Report Formated
     * @param $format
     */
    function source2($format, $download, $filename = '', $batch = false){
        if (!empty($filename)) {
		  $this->filename = $filename;
        }
        $header = '';
        $return = '';
        $sw = CLX_INIT_ZERO;
        if (empty($this->filename))
		  $this->filename = $format; //(!empty($filename))?$filename:'';
        switch ($format) {
		  case 'text':
			 $header = 'text/plain';
			 $return = $this->sourceText();
			 break;
		  case 'csv':
			 $header = 'text/plain';
			 $return = $this->sourceCsv();
			 break;
		  case 'html':
			 $header = 'text/html';
			 $return = $this->sourceHtml();
			 break;
		  case 'xml':
			 $header = 'text/xml';
			 $return = $this->sourceXml();
			 break;
		  case 'pdf':
			 $this->send_pdf = true;
			 $return = $this->sourcePdf($download,$batch,$filename);
			 $sw = CLX_NUM_ONE;
			 break;
		  case 'msexcel':
			 $header = 'application/vnd.ms-excel';
			 //$this->filename = 'download.xls');
				$return = $this->sourceMsExcel_($download, $batch,$filename.'.xls');
			 
				if (!$batch) {
					UserResponse::setContentType($header);
					UserResponse::setContentDisposition('attachment');
					UserResponse::setFilename($this->filename.'.xls');
					UserResponse::setExpires('1');
					if (true === $download){
						UserResponse::setCacheControl('must-revalidate, post-check=0,pre-check=0');

					}else{
						UserResponse::setContentDisposition('inline');
					}
					UserResponse::setPragma('public');
					$sw = CLX_NUM_ONE;
				}	
			 break;
		  case 'msword':
			 $header = 'text/html';
			 $return = $this->sourceMsWord();
			 break;
		  default:
			 $header = 'text/html';
			 $return = 'Type ' . $format . ' not defined. ' . 'Available types:html, text, xml, pdf, csv, msexcel, msword';
			 break;
        }
        if (true === $download && $sw == CLX_NUM_ZERO) {

		  UserResponse::setContentType($header);
		  UserResponse::setContentDisposition('inline');
		  UserResponse::setFilename(microtime_float());
		  UserResponse::setExpires('1');
		  UserResponse::setCacheControl('must-revalidate, post-check=0,pre-check=0');
		  UserResponse::setPragma('public');
        }

        return $return;
    }

    function sourceText() {
        $contents = '';
        //See alternatives on global
        if ($this->header === true) {
		  $contents .= $this->header->sourceText() . $this->nl;
        }
        //Per each Page
        foreach ($this->pages as $page_no => $page) {
		  //Adding Headers
		  foreach ($page->getHeaders() as $head) {
			 $contents .= $head->sourceText() . $this->nl;
		  }
		  //Adding Content
		  foreach ($page->getPanels() as $panel) {
			 if ($panel->getTitle()) {
			     $contents .= $panel->getTitle()->sourceText() . $this->nl;
			 }
			 $contents .= $panel->getObject()->sourceText() . $this->nl;
		  }//end contents
		  //Adding Footer
		  foreach ($page->getFooters() as $foot) {
			 $contents .= $foot->sourceText() . $this->nl;
		  }
        }//end for pages
        if ($this->footer === true) {
		  $contents .= $this->footer->sourceText() . $this->nl;
        }
        return $contents;
    }

    function sourceCsv() {
        return $this->sourceText();
    }

    /**
     * Parses report into html
     */
    function sourceHtml() {
        $contents = '';
        $ancho = $this->logo_width * CLX_NUM_FIVE;
        $alto = $this->logo_height * CLX_NUM_FIVE;
        //Add a general logo here
        $this->logo_img = str_replace(kPathToWebSite, kWebSite, $this->logo_img);
        if ($this->getRepLogo()) {
		  $contents .= '<IMG SRC="' . $this->getRepLogo() . '" WIDTH=' . $ancho . ' HEIGHT=' . $alto . '>' . '<br>';
        }
        //See alternatives on global
        if ($this->header === true) {
		  $contents .= $this->header->sourceHtml();
        }
        //Per each Page
        foreach ($this->pages as $page_no => $page) {
		  //Adding Headers
		  if ($page->getHeaders()) {
			 foreach ($page->getHeaders() as $head) {
			     $contents .= $head->sourceHtml();
			 }
		  }
		  //Adding Content
		  foreach ($page->getPanels() as $panel) {
			 if ($panel->getTitle()) {
			     $contents .= $panel->getTitle()->sourceHtml();
			     $x = CLX_INIT_ZERO;
			     if ($panel->getLnPost() > CLX_NUM_ZERO) {
			         $x = round($panel->getLnPost() / CLX_NUM_TWO);
			         for ($i = CLX_INIT_ZERO; $i < $x; $i++) {
					   $contents .= '<div style="font-size:0.1em">&nbsp;</div><br />';
			         }
			     }
			 }
			 $contents .= $panel->getObject()->sourceHtml();
			 $x = CLX_INIT_ZERO;
			 if ($panel->getObject()->getLnPost() > CLX_NUM_ZERO) {
			     $x = round($panel->getObject()->getLnPost() / CLX_NUM_TWO);
			     for ($i = CLX_INIT_ZERO; $i < $x; $i++) {
			         $contents .= '<div style="font-size:0.1em">&nbsp;</div><br />';
			     }
			 }
		  }//end contents
		  //Adding Footer
		  foreach ($page->getFooters() as $foot) {
			 $contents .= $foot->sourceHtml() . '<br />';
		  }
        }//end for pages
        if ($this->footer === true) {
		  $contents .= $this->footer->sourceHtml();
        }

        return $contents;
    }

    /**
     * Parses report into html
     */
    function sourceXml() {
        $contents = '<?xml version="1.0"?>' . $this->nl;
        $xml = new xmlObject('xml', true);
        //Adding styles
        if ($this->header === true) {
		  $temp_xml = new xmlObject('header', true);
		  $temp_xml->addContents($this->header->sourceXml());
		  $xml->addObject($temp_xml);
        }
        $pgs = new xmlObject('pages', true);
        //Per each Page
        foreach ($this->pages as $page_no => $page) {
		  $pg = new xmlObject('page', true);
		  //Adding Headers
		  foreach ($page->getHeaders() as $head) {
			 $xml_temp = new xmlObject('page_header', true);
			 $xml_temp->addContents($head->sourceXml());
			 $pg->addObject($xml_temp);
		  }
		  //Adding Content
		  $panels = new xmlObject('panels', true);
		  foreach ($page->getPanels() as $val) {
			 $panel = new xmlObject('panel', true);
			 if ($val->getTitle()) {
			     $xml_temp = new xmlObject('panel_title', true);
			     $xml_temp->addContents($val->getTitle()->sourceXml());
			     $panel->addObject($xml_temp);
			 }
			 $xml_temp = new xmlObject('panel_content', true);
			 $xml_temp->addObject($val->getObject()->sourceXml());
			 $panel->addObject($xml_temp);
			 $panels->addObject($panel);
		  }//end contents
		  $pg->addObject($panels);
		  //Adding Footer
		  if ($page->getFooter()) {
			 $xml_temp = new xmlObject('page_footer', true);
			 $xml_temp->addContents($page->getFooter()->sourceXml());
			 $pg->addObject($xml_temp);
		  }
		  $pgs->addObject($pg);
        }//end for pages
        $xml->addObject($pgs);
        if ($this->footer === true) {
		  $temp_xml = new xmlObject('footer', true);
		  $temp_xml->addContents($this->footer->sourceXml());
		  $xml->addObject($temp_xml);
        }
        $contents .= $xml->source_(array('level' => CLX_NUM_ZERO));
        unset($xml, $pg, $xml_temp, $panel);
        return $contents;
    }

    /**
     * Export PDF Format
     * @return string
     */
    function sourcePdf($download = true, $batch = false,$filanameNew='') { // IVAN: COLOCADO PARAMETRO FILENAME PROBLEMA 500 RESOURCE
		if (is_array($this->format)) {
		  $items_count = count($this->format);
		  if($items_count===CLX_NUM_TWO && is_numeric($this->format[CLX_NUM_ZERO]) && is_numeric($this->format[CLX_NUM_ONE])){
			 $pdf = new pdf_table($this->orientation, $this->unit, $this->format);
			 $pdf->setMargin($this->format);
		  }elseif ($items_count===CLX_NUM_THREE) {
			 $pdf = new pdf_table($this->orientation, $this->unit, $this->format[CLX_NUM_ZERO], $this->format[CLX_NUM_ONE], $this->format[2]);
		  } else {
			 $pdf = new pdf_table($this->orientation, $this->unit, $this->format[CLX_NUM_ZERO], $this->format[CLX_NUM_ONE]);
		  }
        } else {
		  $pdf = new pdf_table($this->orientation, $this->unit, $this->format);
        }

        $pdf->SetFont('Times');
        $pdf->SetAuthor(kDefaultCompany);
        $arr = explode('.', $this->getRepLogo());
        if ($this->getRepLogo() && in_array(end($arr), array('jpg', 'jpeg', 'gif', 'png'))) {
		  $pdf->setReplogo($this->getRepLogo(), $this->logo_x, $this->logo_y, $this->logo_width, $this->logo_height);
        }
        //Per each Page
        if ($this->header) {
		  foreach ($this->header as $param => $this->head) {
			 $pdf->SetFont($this->head->getStyle()->getFontFace(), $this->head->getStyle()->getFont('pdf'), $this->head->getStyle()->getFontSize('pdf'));
			 $Style = $this->head->getStyle();
			 $pdf->setHeaderParameter($param, $this->head->sourceText(), $Style->getFontFace(), $Style->getFont('pdf'), $Style->getFontSize('pdf'), $Style->getBoxWidth('pdf'), $Style->getBoxHeight('pdf'), $Style->getBoxBorder('pdf'), $Style->getFontAligment('pdf'), $Style->getFontColor('pdf'), $Style->getBoxBorderColor('pdf'), $Style->getBoxBgColor('pdf'), $Style->getLn('pdf'), $Style->getPositionX('pdf'), $Style->getPositionY('pdf'), $Style->getResetY('pdf'));
		  }
        }

        if ($this->footer) {
		  foreach ($this->footer as $parameter => $this->foot) {
			 $pdf->SetFont($this->foot->getStyle()->getFontFace(), $this->foot->getStyle()->getFont('pdf'), $this->foot->getStyle()->getFontSize('pdf'));
			 $pdf->setFooter($parameter, $this->foot->sourceText(), $this->foot->getStyle());
		  }
        }
        $filename = 'clientix';
        foreach ($this->pages as $page_no => $page) {
		  $page_num = $page->getNumPage();
		  if ($page_num['view'] === true) {
			 $pdf->set_num_page($page_num);
		  }
		  if ($batch and $filanameNew!='') { //IVAN: PROBLEMA 500 RESOURCEBATCH
			 $filename =$filanameNew;
		  }
		  else if ($filename === 'clientix') {
			 $filename = $page->name;
		  }
		  $ln_header = $page->getLnHeader();
		  $pdf->ln_header = $ln_header['ln_header'];
		  $pdf->ln_header_post = $ln_header['ln_header_post'];
		  //Margen de Página
		  $pdf->setMargin(array($page->getMargin()));
		  $pdf->SetTopMargin($page->getMarginTop());

		  //Adding Headers
		  $i = CLX_INIT_ZERO;
		  if ($page->getHeaders()) {
			 foreach ($page->getHeaders() as $head) {
			     $pdf->setHeaderParameter($i++, $head->sourceText(), $head->getStyle()->getFontFace(), $head->getStyle()->getFont('pdf'), $head->getStyle()->getFontSize(), $head->getStyle()->getBoxWidth('pdf'), $head->getStyle()->getBoxHeight('pdf'), $head->getStyle()->getBoxBorderForm('pdf'), $head->getStyle()->getFontAligment('pdf'), $head->getStyle()->getFontColor('pdf'), $head->getStyle()->getBoxBorderColor('pdf'), $head->getStyle()->getBoxBgColor('pdf'), $head->getStyle()->getLn());
			 }
		  }

		  $pageBreak = $page->getAutoPage();
		  $pdf->SetAutoPageBreak($pageBreak['boolean'], $pageBreak['margin']);
		  $pdf->pageBreak = $pageBreak['margin'] * -CLX_NUM_ONE;
		  //Adding page

		  if(isset($page->orientation)){
			$orientation=$page->orientation;
		  }else{
			$orientation=$this->orientation;
		  }
		  $pdf->AddPage($orientation);
		  //Adding Content
		  foreach ($page->getPanels() as $val) {
			 //Margen de Panel
			$pdf->SetLeftMargin($val->getMarginL());

			 if ($val->getTitle()) {
			     //Set Font
			     $pdf->SetFont($val->getTitle()->getStyle()->getFontFace(), $val->getTitle()->getStyle()->getFont('pdf'), $val->getTitle()->getStyle()->getFontSize('pdf'));
			     $colorFont = $val->getTitle()->getStyle()->getFontColor('pdf');
			     $pdf->SetTextColor($colorFont['r'], $colorFont['g'], $colorFont['b']);
			     $bgcolor = $val->getTitle()->getStyle()->getBoxBgColor('pdf');

			     $pdf->SetFillColor($bgcolor['r'], $bgcolor['g'], $bgcolor['b']);
			     if (!$val->getObject()->disable_borders && $val->getTitle()->getStyle()->getBoxBorder('pdf')) {
			         $color = $val->getTitle()->getStyle()->getBoxBorderColor('pdf');
			         $pdf->SetDrawColor($color['r'], $color['g'], $color['b']);
			     } else {
			         $pdf->SetDrawColor($bgcolor['r'], $bgcolor['g'], $bgcolor['b']);
			     }
			     $pdf->MultiCell($val->getTitle()->getStyle()->getBoxWidth('pdf'), $val->getTitle()->getStyle()->getBoxHeight('pdf'), $val->getTitle()->sourceText(), $val->getTitle()->getStyle()->getBoxBorderForm('pdf'), $val->getTitle()->getStyle()->getFontAligment('pdf'), true);
			 }
			 $val->getObject()->disable_borders = $this->disable_borders;
			 $val->getObject()->sourcePdf($pdf);
			 $pdf->resetMargin();
			 if ($val->getLn()) {
			     $pdf->Ln($val->getObject()->getLnPost());
			     $pdf->Ln($val->getObject()->getLnPost());
			 }
		  }//end contents
		  //Adding Footer
		  $i = CLX_INIT_ZERO;
		  if ($page->getFooters() === true) {
			 foreach ($page->getFooters() as $foot) {
			     $pdf->setFooter("cust" . $i, $foot->sourceText(), array('fontface' => $foot->getStyle()->getFontFace(), 'fontcolor' => $foot->getStyle()->getFontColor('pdf'), 'font' => $foot->getStyle()->getFont('pdf'), 'fontsize' => $foot->getStyle()->getFontSize(), CLX_PROP_WIDTH => $foot->getStyle()->getBoxWidth('pdf'), 'height' => $foot->getStyle()->getBoxHeight('pdf'), 'border' => $foot->getStyle()->getBoxBorderForm('pdf'), 'bordercolor' => $foot->getStyle()->getBoxBorderColor('pdf'), 'ln' => $foot->getStyle()->getLn(), 'fill' => true, 'aligment' => $foot->getStyle()->getFontAligment('pdf'), 'bgcolor' => $foot->getStyle()->getBoxBgColor('pdf')));
			     $i++;
			 }
		  }
        }//end for pages
        if ($batch and $filanameNew !=''){
		  if(!empty(UserRequest::$request['GLREP_CONS_DIR']))
			 $flname = UserRequest::$request['GLREP_CONS_DIR'] . $filename . '.pdf';
		  else
			 $flname = kPathToTemp . $filename . '.pdf';
        }
        else    
        $flname = kPathToTemp . $filename . str_replace('.', '', microtime(true)) . '.pdf';
        if (empty($download)&& $batch) {
		  $pdf->Output($flname, 'F');   //IVAN: LA SIGUIENTE LINEA NO FUNCIONA
        }elseif(empty($download)) {
		  $pdf->Output($flname, 'F');
		  return note('<page t=clx_report></page><a href=' . singleQuote($flname) . ' target=' . singleQuote(' _blank') . '>' . dict('GEN_LBL_DOWNLOAD_THE_FILE') . '</a><br>');
        }else{
		  $pdf->Output($flname, 'D');
		  return CLX_CHAR_EMPTY_STRING;
        }
    }

    /**
     * Sets worksheet paper size
     * @param type $ws
     */
    private function paperSizeMsExcell(&$ws) {

        $format = (is_array($this->format))?'default':lowercase($this->format);
        $no = 1;
        switch ($format) {
		  case 'letter':
			 $no = 1;
			 break;
		  case 'legal':
			 $no = 5;
			 break;
		  case 'a4':
			 $no = 9;
			 break;
		  default:
			 $no = 1;
			 break;
        }
        $ws->setPaper($no);
    }

    /**
     * Sets paper orientation
     * @param type $ws
     */
    private function orientationMsExcell(&$ws) {
        if ($this->orientation == 'L') {
		  $ws->setLandscape();
        } else {
		  $ws->setPortrait();
        }
    }

    /**
     * sourceMsExcel
     * @return boolean
     */
    function sourceMsExcel_($download = true,$batch,$filename) { //IVAN:COLOCADO BATCH Y FILENAME PARA ARCHIVOS BATCH PROBELMA 500 RESOURCE
        ob_flush();
        // Creating a workbook
        require_once kPathToWebSite . kPathToPear . 'Spreadsheet/Excel/Writer.php';
        if ($batch) { //IVAN: COLOCADO PARA TOMAR ID DEL ARCHIVO BATCH
		  if ($filename=='') {//IVAN:PARA ARCHIVOS ARCHIVOS VIEJOS BATCH QUE NO TENGAN EL FILENAME EN EL SOURCE2, (DARA ARROR EN EL LINK PERO SE GENERA EL ARCHIVO
			 $filename='Excel.xls';
		  }
		  if(!empty(UserRequest::$request['GLREP_CONS_DIR']))
			 $workbook = new Spreadsheet_Excel_Writer(UserRequest::$request['GLREP_CONS_DIR'].$filename);
		  else
		  	$workbook = new Spreadsheet_Excel_Writer('temp/'.$filename);
        }    
        else {
        $workbook = new Spreadsheet_Excel_Writer();
        }    
        $workbook->setVersion(8); //set excell 2k+
        //Per each Page
        foreach ($this->pages as $page_no => $page) {
		  if (empty($page->name))
			 $page->name = 'Sheet';
		  $ws = $workbook->addWorksheet(decode_string($page->name) . (($page->name === 'Sheet') ? ' (' . ($page_no + CLX_NUM_ONE) . ')' : ''));

		  $this->orientationMsExcell($ws);
		  $this->paperSizeMsExcell($ws);


		  $lg = explode('.', $this->getRepLogo());
		  if ($this->getRepLogo() && end($lg) === 'bmp')
			 $ws->insertBitmap($this->logo_x, $this->logo_y, $this->getRepLogo(), CLX_NUM_ONE, CLX_NUM_ONE, $this->logo_width, $this->logo_height);
		  //Adding Headers
		  if ($page->getHeaders()) {
			 foreach ($page->getHeaders() as $head) {
			     $head->sourceMsExcel($ws, $workbook);
			 }
		  }
		  //$ws->Ln();
		  //Adding Content
		  foreach ($page->getPanels() as $val) {
			 if ($val->getTitle()) {
			     $val->getTitle()->sourceMsExcel($ws, $workbook);
			     if ($val->getLn())
			         $ws->Ln();
			 }
			 $val->getObject()->sourceMsExcel($ws, $workbook);
		  }//end contents
		  //Adding Footer
		  if ($page->getFooters()) {
			 foreach ($page->getFooters() as $foot) {
			     $foot->sourceMsExcel($ws, $workbook);
			     $ws->Ln();
			 }
		  }
		  $freeze = $page->getFreeze();

		  if (!empty($freeze)) {
			 $ws->freezePanes(array($freeze['freeze_num'], $freeze['freeze']));
		  }
		  $ws->hideGridLines();
        }//end for pages
        if($download){
		  $contents = '';
		  $workbook->send($this->filename . '.xls');
		  $workbook->close();
        }else{
		  ob_start();
		  $workbook->close();
		  $contents = ob_get_contents();
		  ob_end_clean();
        }

        return $contents;

    }

    function sourceMsWord() {
        return 'Type msword not defined';
    }

}

if (!class_exists('FPDF') && file_exists(kPathToWebSite . kPathToFpdf . 'fpdf.php')) {
    include_once kPathToWebSite . kPathToFpdf . 'fpdf.php';
}

/**
 * PDF Class for Clientix
 * @author Francisco Santana
 * @copyright Alenet, Inc.
 */
class clx_pdf extends FPDF {

    private $header_text;
    private $footer_text;
    var $img_url;
    var $logo_x;
    var $logo_y;
    var $logo_width;
    var $logo_height;
    var $def_font = 'Arial';
    var $ln_header = CLX_INIT_ZERO;
    var $ln_header_post = CLX_NUM_8;
    var $pageBreak = CLX_INIT_ZERO;
    var $position_x = CLX_INIT_ZERO;
    var $position_y = CLX_INIT_ZERO;
    var $reset_y = CLX_INIT_ZERO;
    private $marginLeft = CLX_INIT_ZERO;
    private $marginRight = CLX_INIT_ZERO;
    private $marginLeftHeader = CLX_INIT_ZERO;
    private $marginRightHeader = CLX_INIT_ZERO;
    private $marginLeftFooter = CLX_INIT_ZERO;
    private $marginRightFooter = CLX_INIT_ZERO;
    public $num_page;

    function __construct($orientation = 'P', $unit = 'mm', $format = 'A4', $width = 0, $height = 0) {
        if ($format == 'FREE')
		  parent::__construct($orientation, $unit, array($width, $height));
        else
		  parent::__construct($orientation, $unit, $format);
        $this->num_page = array(
		  'view' => false,
		  'word_before' => '',
		  'word_after' => '',
		  'nb' => false,
		  CLX_PROP_ALIGN => 'C',
		  'font' => 'Arial',
		  'style' => '',
		  'size' => '11',
		  'position' => -CLX_NUM_TEN);
    }

    function Header() {

        if ($this->marginLeftHeader != CLX_NUM_ZERO){
		  $this->SetLeftMargin($this->marginLeftHeader);
        }
        if ($this->marginRightHeader != CLX_NUM_ZERO){
		  $this->SetRightMargin($this->marginRightHeader);
        }
        if ($this->ln_header === true){
		  $this->Ln($this->ln_header);
        }
        if (is_array($this->header_text) || is_object($this->header_text)) {
		  foreach ($this->header_text as $param => $this->header_t) {
			 if (isset($this->h_sty[$param]['fontColor'])) {
			     $colorFont = $this->h_sty[$param]['fontColor'];
			     $this->SetTextColor($colorFont['r'], $colorFont['g'], $colorFont['b']);
			 }

			 if (!empty($this->h_sty[$param]['bgcolor'])) {
			     $bgcolor = $this->h_sty[$param]['bgcolor'];
			     $this->SetFillColor($bgcolor['r'], $bgcolor['g'], $bgcolor['b']);
			 } else
			     $this->SetFillColor(CLX_NUM_255, CLX_NUM_255, CLX_NUM_255);

			 if ($this->h_sty[$param]['BoxBorder']) {
			     $color = $this->h_sty[$param]['bordercolor'];
			     $this->SetDrawColor($color['r'], $color['g'], $color['b']);
			 }
			 $this->SetFont($this->h_sty[$param]['fontface'], $this->h_sty[$param]['font'], $this->h_sty[$param]['fontsize']);
			 if ($this->h_sty[$param]['Reset_Y'] == CLX_NUM_ONE) {
			     $this->setXY($this->h_sty[$param]['Position_X'], $this->h_sty[$param]['Position_Y']);
			 } else if ($this->h_sty[$param]['Position_X'] <> null) {
			     $this->setX($this->h_sty[$param]['Position_X']);
			 }
			 if ($this->h_sty[$param]['BoxWidth'] < 0) {
			     $this->h_sty[$param]['BoxWidth'] = $this->GetStringWidth($this->header_t) + 1;
			 }
			 if ($this->h_sty[$param]['BoxHeight'] < 0) {
			     $this->h_sty[$param]['BoxHeight'] = (int) ($this->h_sty[$param]['fontsize'] * 1.5);
			 }
			 $this->Cell($this->h_sty[$param]['BoxWidth'], $this->h_sty[$param]['BoxHeight'], $this->header_t, $this->h_sty[$param]['BoxBorder'], $this->h_sty[$param]['ln'], $this->h_sty[$param]['Aligment'], true);

			 if ($this->h_sty[$param]['ln'] <> 0) {
			     $this->Ln();
			 }
		  }
        }
        $this->Ln($this->ln_header_post);
        $this->resetMargin();
        if ($this->getReplogo()) {
		  $this->Image(realpath($this->img_url), $this->logo_x, $this->logo_y, $this->logo_width, $this->logo_height);
        }
    }

    function Footer() {
        //Go to 25 cm from bottom
        if ($this->pageBreak == CLX_NUM_ZERO)
		  $this->SetY(CLX_NUM_MINUS_25);
        else
		  $this->SetY($this->pageBreak);
        if ($this->marginLeftFooter != CLX_NUM_ZERO)
		  $this->SetLeftMargin($this->marginLeftFooter);
        if ($this->marginRightFooter != CLX_NUM_ZERO)
		  $this->SetRightMargin($this->marginRightFooter);
        if (is_array($this->footer_text) || is_object($this->footer_text)) {
		  foreach ($this->footer_text as $f => $this->foot) {
			 $Style = $this->f_sty[$f];
			 $colorFont = $Style->getfontcolor('pdf');
			 $this->SetTextColor($colorFont['r'], $colorFont['g'], $colorFont['b']);
			 //if (!empty($this->f_sty[$i]['bgcolor'])) {
			 //    $bgcolor = $this->f_sty[$i]['bgcolor'];
			 if ($Style->getBoxBgColor('pdf')) {
			     $bgcolor = $Style->getBoxBgColor('pdf');
			     $this->SetFillColor($bgcolor['r'], $bgcolor['g'], $bgcolor['b']);
			 } else {
			     $this->SetFillColor(CLX_NUM_255, CLX_NUM_255, CLX_NUM_255);
			 }
			 //  if ($this->f_sty[$i]['border'] === true) {
			 //      $color = $this->f_sty[$i]['bordercolor'];
			 if ($Style->getBoxBorder('pdf')) {
			     $color = $Style->getBoxBorderColor('pdf');
			     $this->SetDrawColor($color['r'], $color['g'], $color['b']);
			 }
			 $this->SetFont($Style->getFontFace('pdf'), $Style->getFont('pdf'), $Style->getFontSize('pdf'));
//			 $this->SetFont($this->f_sty[$i]['fontface'], $this->f_sty[$i]['font'], $this->f_sty[$i]['fontsize']);
//			 $this->Cell($this->f_sty[$i][CLX_PROP_WIDTH], $this->f_sty[$i]['height'], $this->footer_text[$f], $this->f_sty[$i]['border'], $this->f_sty[$i]['ln'], $this->f_sty[$i]['aligment'], true);
//			 $i++;
			 if ($Style->getPositionY('pdf') == null) {
			     $y = $this->GetY() + CLX_NUM_3_PT_5;
			     $this->SetY($y);
			 } else if ($Style->getResetY('pdf') == CLX_NUM_ONE) {

			     $this->setXY($Style->getPositionX('pdf'), $Style->getPositionY('pdf'));
			 } else if ($Style->getPositionX('pdf') > 0) {
			     $this->setX($Style->getPositionX('pdf'));
			 }
			 $Width = $Style->getBoxWidth('pdf');
			 $Height = $Style->getBoxHeight('pdf');

			 if ($Width < CLX_NUM_ZERO) {
			     $Width = $this->GetStringWidth($this->header_t) + 1;
			 }
			 if ($Height < CLX_NUM_ZERO) {
			     $Height = (int) ($this->h_sty[$param]['fontsize'] * 6);
			 }
			 $this->Cell($Width, $Height, $this->foot, $Style->getBoxBorder('pdf'), ($Style->getOrden() == "(-)") ? 0 : $Style->getLn('pdf'), $Style->getFontAligment('pdf'), true);

			 if ($Style->getOrden() == "(-)")
			     $this->reverseLn();
		  }
        }
        $this->resetMargin();
        $this->SetY(-CLX_NUM_TEN);

        if ($this->num_page['view'] === true) {
		  $this->SetY($this->num_page['position']);
		  $nb = '';
		  if ($this->num_page['nb'] === true) {
			 $this->AliasNbPages();
			 $nb = '/{nb}';
		  }
		  $this->SetFont($this->num_page['font'], $this->num_page['style'], $this->num_page['size']);
		  $this->Cell(CLX_NUM_ZERO, CLX_NUM_TEN, $this->num_page['word_before'] . $this->PageNo() . $nb . $this->num_page['word_after'], CLX_NUM_ZERO, CLX_NUM_ZERO, $this->num_page[CLX_PROP_ALIGN]);
        }
    }

    function setHeaderParameter($parameter, $header, $fontface, $font, $fontsize, $w, $h, $border, $align, $fontColor = array("r" => 0, "g" => 0, "b" => 0), $bordercolor = array(), $bgcolor = array(), $ln = 1, $position_x = 0, $position_y = 0, $reset_y = -1) {
        $this->header_text[$parameter] = $header;
        $this->h_sty[$parameter]['fontface'] = $fontface;
        $this->h_sty[$parameter]['font'] = $font;
        $this->h_sty[$parameter]['fontsize'] = $fontsize;
        $this->h_sty[$parameter]['BoxWidth'] = $w;
        $this->h_sty[$parameter]['BoxHeight'] = $h;
        $this->h_sty[$parameter]['BoxBorder'] = $border;
        $this->h_sty[$parameter]['Aligment'] = $align;
        $this->h_sty[$parameter]['fontColor'] = $fontColor;
        $this->h_sty[$parameter]['bordercolor'] = $bordercolor;
        $this->h_sty[$parameter]['bgcolor'] = $bgcolor;
        $this->h_sty[$parameter]['ln'] = $ln;
        $this->h_sty[$parameter]['Position_X'] = $position_x;
        $this->h_sty[$parameter]['Position_Y'] = $position_y;
        $this->h_sty[$parameter]['Reset_Y'] = $reset_y;
    }

    function setFooter($parameter, $footer, $style) {
        if (get_class($style) == 'clx_report_data') {
		  $style = $style->getStyle();
        }
        $this->footer_text[$parameter] = $footer;
        $this->f_sty[$parameter] = $style;
    }

    function setReplogo($url, $x, $y, $w, $h) {
        $this->img_url = $url;
        $this->logo_x = $x;
        $this->logo_y = $y;
        $this->logo_width = $w;
        $this->logo_height = $h;
    }

    function getReplogo() {
        return $this->img_url;
    }

    function setMargin($data) {
        list($margen) = $data;
        $this->marginLeft = $margen['margin_left'];
        $this->marginRight = $margen['margin_right'];
        $this->marginLeftHeader = $margen['margin_left_header'];
        $this->marginRightHeader = $margen['margin_right_header'];
        $this->marginLeftFooter = $margen['margin_left_footer'];
        $this->marginRightFooter = $margen['margin_right_footer'];
    }

    function getMargin() {
        return array('margin_left' => $this->marginLeft, 'margin_right' => $this->marginRight, 'margin_left_header' => $this->marginLeftHeader, 'margin_right_header' => $this->marginRightHeader, 'margin_left_footer' => $this->marginLeftFooter, 'margin_right_footer' => $this->marginRightFooter);
    }

    function resetMargin() {

        $this->SetX($this->resetMargin);
        $this->SetLeftMargin($this->marginLeft);
        $this->SetRightMargin($this->marginRight);
    }

    function set_num_page($num_page) {
        $this->num_page = $num_page;
    }

}

/**
 * Build a pdf table
 * @author Francisco Santana
 *
 */
class pdf_table extends clx_pdf {

    var $widths;
    var $aligns;
    var $fillR;
    var $altura;
    var $resetMargin = 0;

    /**
     * Returns Widhts
     */
    function GetWidths() {
        return $this->widths;
    }

    function GetTotalWidth() {
        $ancho = CLX_INIT_ZERO;
        $total = count($this->widths);
        for ($i = CLX_INIT_ZERO; $i < $total; $i++) {
		  $ancho = $ancho + $this->widths[$i];
        }
        return $ancho;
    }

    function SetWidths($w) {
        // Set the array of column widths
        $this->widths = $w;
    }

    function SetAligns($a) {
        // Set the array of column alignments
        $this->aligns = $a;
    }

    function SetFillR($f) {
        // Set the array of column alignments
        $this->fillR = $f;
    }

    function SetAltura($a) {
        // Set the array of column alignments
        $this->altura = $a;
    }

    function resetAltura() {
        // Set the array of column alignments
        $this->altura = array();
    }

    function RowHasPageBreak(&$data){
         // Calculate the height of the row
        $nb = CLX_INIT_ZERO;
        $total = count($data);
        for ($i = CLX_INIT_ZERO; $i < $total; $i++) {
		  $nb = max($nb, $this->NbLines($this->widths [$i], $data [$i]));
        }
        $h = CLX_NUM_FIVE * $nb;
        // Issue a page break first if needed
        return $this->HasCheckPageBreak($h);
    }

    /**
     *
     * @param unknown_type $data
     */
    function Row($data, $fillcolor = false) {
        // Calculate the height of the row
        $nb = CLX_INIT_ZERO;
        $total = count($data);
        for ($i = CLX_INIT_ZERO; $i < $total; $i++) {
		  $nb = max($nb, $this->NbLines($this->widths [$i], $data [$i]));
        }
        $h = CLX_NUM_FIVE * $nb;
        // Issue a page break first if needed
		$x = $this->GetX();
        $this->CheckPageBreak($h);
		$this->SetX($x);

        // Draw the cells of the row
        $wx = $this->widths;
        if($total<count($this->widths)){
		  $ttl = 0;
		  foreach($this->widths as $k=>$v){
			 $ttl = $ttl + $v;
		  }
		  $ttl = ($total==0)?$ttl:($ttl / $total);
		  $wx = array();
		  for($x = 0; $x<$total; $x++){
			 $wx[$x] = $ttl;
		  }
		  $h  = 5;
        }
        for ($i = CLX_INIT_ZERO; $i < $total; $i++) {
		  $w = $wx [$i];
		  $a = isset($this->aligns [$i]) ? $this->aligns [$i] : 'L';
		  // Save the current position
		  $x = $this->GetX();
		  $y = $this->GetY();
		  // Draw the border and fill if fill is on
		  $fill = ($fillcolor) ? 'DF' : 'D';
		  $this->Rect($x, $y, $w, $h, $fill);
		  // Print the text
		  $b = 'LRT'; //Multicel coloca una celda dentro del rectangulo
		  //$this->SetLineWidth(0.7);
		  $this->MultiCell($w, CLX_NUM_FIVE, $data [$i], $b, $a, $fillcolor);
		  //Add borders since multicell makes weird lines
		  $this->Rect($x, $y, $w, $h, 'D');
		  // Put the position to the right of the cell
		  $this->SetXY($x + $w, $y);
        }
        // Go to the next line
        $this->Ln($h);
    }

    /**
     *
     * @param unknown_type $data
     */
    function Row1($data, $fillcolor = false, $fillInter = false, $ln = CLX_NUM_ZERO, $estilo = false, $fuente = array(), $borde = '', &$arrayRowSpan = array()) {

        // Calculate the height of the row
        $nb = CLX_INIT_ZERO;
        /*$total = count($data);
        for ($i = CLX_INIT_ZERO; $i < $total; $i++) {
		  $nb = max($nb, $this->NbLines($this->widths [$i], $data [$i]));
        }*/
        $margen = $this->getMargin();
        if (!empty($margen)) {
		  $this->SetLeftMargin($margen['margin_left']);
		  $this->SetRightMargin($margen['margin_right']);
        }
        // Calculate the height of the row
        $nb = CLX_INIT_ZERO;
        $TextColor_ = $this->TextColor;
        $total = count($data);
        for ($i = CLX_INIT_ZERO; $i < $total; $i++) {
		  if (!empty($estilo)) {
			 $this->SetFont($fuente[$i]['letra'], $fuente[$i]['estilo'], $fuente[$i]['tamano']);
		  }
		  if (isset($fuente[$i]['cR'])) {
			 $this->SetTextColor($fuente[$i]['cR'], $fuente[$i]['cG'], $fuente[$i]['cB']);
		  } else {
			 $this->TextColor = $TextColor_;
		  }
		  $nb = max($nb, $this->NbLines($this->widths [$i], $data [$i]));
        }

        $h = CLX_NUM_FIVE * $nb;
        // Issue a page break first if needed
        $x = $this->GetX();
        $this->CheckPageBreak($h);
		$this->SetX($x);

        $colspan = CLX_INIT_ZERO;
        $line_witdh = 0.20;
		$this->SetDrawColor(0, 0, 0);
        $this->SetLineWidth($line_witdh);
        //Draw the cells of the row
        //$total = count($data);
        for ($i = CLX_INIT_ZERO; $i < $total; $i++) {
		  $w = $this->widths [$i];
		  $a = isset($this->aligns [$i]) ? $this->aligns [$i] : 'L';
		  // Save the current position
		  $x = $this->GetX();
		  $y = $this->GetY();
		  // Draw the border and fill if fill is on

		  if ($fillInter === true && $this->fillR[$i] === CLX_NUM_ONE) {
			 $fill_ = $this->fillR[$i];
			 $this->SetFillColor($fuente[$i]['fR'], $fuente[$i]['fG'], $fuente[$i]['fB']);
			 $this->SetDrawColor($fuente[$i]['fR'], $fuente[$i]['fG'], $fuente[$i]['fB']);
		  } elseif ($fillcolor === true) {
			 $fill_ = true;
			 $this->SetFillColor($fuente[$i]['fR'], $fuente[$i]['fG'], $fuente[$i]['fB']);
			 //$this->SetDrawColor($fuente[$i]['fR'], $fuente[$i]['fG'], $fuente[$i]['fB']);
		  } else {
			 $fill_ = false;
			 $this->SetDrawColor(CLX_NUM_255, CLX_NUM_255, CLX_NUM_255);
		  }

		  if (!empty($estilo)) {
			 $this->SetFont($fuente[$i]['letra'], $fuente[$i]['estilo'], $fuente[$i]['tamano']);
		  }
		  if (isset($fuente[$i]['cR'])) {
			 $this->SetTextColor($fuente[$i]['cR'], $fuente[$i]['cG'], $fuente[$i]['cB']);
		  } else {
			 $this->TextColor = $TextColor_;
		  }

		  $alt = isset($this->altura[$i]) ? $this->altura[$i] : CLX_NUM_FIVE;
		  // Print the text
		  if (isset($fuente[$i]['borde']) && $borde === '') {
			 $b = $fuente[$i]['borde'];
		  } else {
			 $b = $borde; //Multicel coloca una celda dentro del rectangulo
		  }

		  if ($b === 'LR') {
			 $this->SetDrawColor($fuente[$i]['bR'], $fuente[$i]['bG'], $fuente[$i]['bB']);
			 if ((empty($arrayRowSpan)) || (!empty($arrayRowSpan) && $arrayRowSpan[$colspan]['marca'] == CLX_NUM_ZERO)) {
			     $this->Rect($x, $y, $w, $h, 'DF');
			 }
		  } elseif ($b === 'T' || $b === 'B') {
			 $this->SetDrawColor($fuente[$i]['bR'], $fuente[$i]['bG'], $fuente[$i]['bB']);
		  } elseif (intval($b) !== CLX_NUM_ZERO && $b !== '') {
			 $this->SetDrawColor($fuente[$i]['bR'], $fuente[$i]['bG'], $fuente[$i]['bB']);
			 $b = 'LTR';
			 if ((empty($arrayRowSpan)) || (!empty($arrayRowSpan) && $arrayRowSpan[$colspan]['marca'] == CLX_NUM_ZERO)) {
			     $this->Rect($x, $y, $w, $h, 'DF');
			 }
		  }/* else {

		    $this->SetDrawColor(0, 0, 0);
		    $this->Rect($x, $y, $w, $h, 'D');
		    } */

		  if ((empty($arrayRowSpan)) || (!empty($arrayRowSpan) && $arrayRowSpan[$colspan]['marca'] == CLX_NUM_ZERO)) {
			 $xtmp = $this->GetX();
			 $ytmp = $this->GetY();
			 if ($b == 'T') {
			     
			     $this->SetDrawColor($fuente[$i]['bR'], $fuente[$i]['bG'], $fuente[$i]['bB']);
			     //$this->SetDrawColor(0, 0, 0);
			    
			 }
			 //$this->SetLineWidth(0.4);
			 $this->MultiCell($w, $alt, $data[$i], $b, $a, $fill_);
			 //$this->MultiCell($w, $alt, $data[$i], $b, $a, false);

			 if (!empty($fuente[$i]['xls_border_type']) && $fuente[$i]['xls_border_type'] == 6) {
			     //$this->SetLineWidth($line_witdh);
			     //Linea doble
			     $this->SetDrawColor((int) $fuente[$i]['bR'], (int) $fuente[$i]['bG'], (int) $fuente[$i]['bB']);
			     if ($fuente[$i]['estilo'] == 'T') {
			         $this->Line($xtmp, $ytmp - 1.00, $xtmp + ($w - 0.0), $ytmp - 1.00);
			        // $this->Line($xtmp, $ytmp - 0.2, $xtmp + ($w - 0.0), $ytmp - 0.2);
			         //$this->SetY($this->GetY() + 0.1);
			         $this->SetY($this->GetY() + $line_witdh);

			     } elseif ($fuente[$i]['estilo'] == 'B') {
			         //       $xtmp = $this->GetX();
			         //     $ytmp = $this->GetY();
			         //$ytmp = $ytmp + $alt + 0.1;
			         $ytmp = $ytmp + $alt +$line_witdh;
			         $this->Line($xtmp, $ytmp - 1.00, $xtmp + ($w - 0.0), $ytmp - 1.00);
			       //  $this->Line($xtmp, $ytmp - 0.2, $xtmp + ($w - 0.0), $ytmp - 0.2);
					$this->SetY($this->GetY() + $line_witdh);

			     }

			 }

			 if (!empty($arrayRowSpan)) {
			     $arrayRowSpan[$colspan]['marca'] = CLX_NUM_ONE;
			     $arrayRowSpan[$colspan]['cantidad'] --;
			     for ($xy = CLX_INIT_ZERO; $xy < $arrayRowSpan[$colspan][CLX_PROP_COLSPAN] - CLX_NUM_ONE; $xy++) {
			         $colspan++;
			     }
			 }
		  }
		  // Put the position to the right of the cell
		  $this->SetXY($x + $w, $y);
		  $colspan++;
        }
        $this->TextColor = $TextColor_;

        // Go to the next line
         if ($ln == CLX_NUM_ZERO) {
		  $this->Ln();
        } elseif ($ln == CLX_NUM_ONE) {
		  $this->Ln($h);
        } elseif ($ln > CLX_NUM_ONE) {
		  $this->Ln($ln);
        } else {
		  $this->Ln($h);
        }
    }

    private function HasCheckPageBreak($h) {
        // If the height h would cause an overflow, add a new page immediately
        return ($this->GetY() + $h > ($this->PageBreakTrigger))?true:false;
    }

    /**
     *
     * @param unknown_type $h
     */
    function CheckPageBreakAddPage() {
        $this->AddPage($this->CurOrientation);
    }

    /**
     *
     * @param unknown_type $h
     */
    function CheckPageBreak($h) {
        // If the height h would cause an overflow, add a new page immediately
        if ($this->GetY() + $h > ($this->PageBreakTrigger)){
		  $this->CheckPageBreakAddPage();
        }
    }

    /**
     * Computes the number of lines a MultiCell of width w will take
     * @param unknown_type $w
     * @param unknown_type $txt
     * @return number
     */
    function NbLines($w, $txt) {
        // Computes the number of lines a MultiCell of width w will take
        $cw = &$this->CurrentFont ['cw'];
        if ($w == CLX_NUM_ZERO)
		  $w = $this->w - $this->rMargin - $this->x;
        $wmax = ($w - CLX_NUM_TWO * $this->cMargin) * CLX_NUM_THOUSAND / $this->FontSize;
        $s = str_replace(CLX_CHAR_CR, '', $txt);
        $nb = strlen($s);
        if ($nb > CLX_NUM_ZERO and $s [$nb - CLX_NUM_ONE] == CLX_CHAR_NL)
		  $nb--;
        $sep = - CLX_NUM_ONE;
        $i = CLX_INIT_ZERO;
        $j = CLX_INIT_ZERO;
        $l = CLX_INIT_ZERO;
        $nl = CLX_NUM_ONE;
        while ($i < $nb) {
		  $c = $s [$i];
		  if ($c == CLX_CHAR_NL) {
			 $i++;
			 $sep = - CLX_NUM_ONE;
			 $j = $i;
			 $l = CLX_INIT_ZERO;
			 $nl++;
			 continue;
		  }
		  if ($c === ' ')
			 $sep = $i;
		  $l += $cw [$c];
		  if ($l > $wmax) {
			 if ($sep == - CLX_NUM_ONE) {
			     if ($i == $j)
			         $i++;
			 } else
			     $i = $sep + CLX_NUM_ONE;
			 $sep = - CLX_NUM_ONE;
			 $j = $i;
			 $l = CLX_INIT_ZERO;
			 $nl++;
		  } else
			 $i++;
        }
        return $nl;
    }

}
