<?php

/**
 * e-IBS Check Interface
 * General Code File
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.  * @since        $Date: 2016-01-27 09:39:41 -0500 (Wed, 27 Jan 2016) $
 * @version        $Rev: 7751 $
 *
 * CHANGE LOG
 *  Included svn prop
 */
/*
  Record of changes
  Desarrollo inicial
 */
if (!class_exists('ibs'))
    includeCodeLib('EIBS');
/*
  General Parameters:
  Libreroas de Codigo:
  Motor de Proceso:
  Cronograma:
  Depende de:
  tools/eb_sockets.php
  data/code/EIBS_utilities.php
  data/code/PROC_class_aid.php
  data/code/EIBS_util_ibiq.php
 */
/*
 * ******************************************************************************************************************
 * e-IBS Check Interface common properties and methods
 * *****************************************************************************************************************
 */

class ibs_check_interface_commons extends ibs {

    public $ibs_check_record_id_field_size = CLX_NUM_15;

    /*
     * Constructor
     * @param text, array
     * @return void
     */

    function __construct($s_comp_code = null, $parameters = array()) {
        #parent::ibs($s_comp_code, $parameters);
        parent::__construct($s_comp_code, $parameters);
        
        //   - Caso 1002467 - BC-GAP-Interfaz con IBS
        // Se habilita el modo debug por parametros del sistema
        $EBSK = getTool('EBSK');
        if (defined('EBSK_IBS_DEBUG') && EBSK_IBS_DEBUG === '1') {
            $this->debug = true;
        }
        
        multiTableName($this, array('FILO', 'FICO'));
        if (!defined('EIBS_CHKPRT_TRSFR_ACC_ENC')) {
            // Banck account code max lenght (default value)
            define('FIDU_BANKACT_CODE_MAXLEN', CLX_NUM_13);
            define('FIDU_GLAC_CODE_MAXLEN', CLX_NUM_17);
            // Check Print & Transfers account (bank acc. code)  ending number of characters
            $eibs_chkprt_trsfr_acc_enc = trim(companyParameter('EIBS_CHKPRT_TRSFR_ACC_ENC', $this->s_comp_code));
            if ($eibs_chkprt_trsfr_acc_enc === '')
                $eibs_chkprt_trsfr_acc_enc = FIDU_BANKACT_CODE_MAXLEN;
            define('EIBS_CHKPRT_TRSFR_ACC_ENC', $eibs_chkprt_trsfr_acc_enc);
            // DEFAULT USER ID FOR CHECK PRINT
            define('EIBS_CHKPRT_SPECIAL_USER', trim(companyParameter('EIBS_CHKPRT_SPECIAL_USER', $this->s_comp_code)));
            define('EIBS_ACH_SPECIAL_USER', trim(companyParameter('EIBS_ACH_SPECIAL_USER', $this->s_comp_code)));
            // DEFAULT COMPANY LOCATION FOR CHECKS PRINTING
            define('EIBS_CHKPRT_DEF_COLO', trim(companyParameter('EIBS_CHKPRT_DEF_COLO', $this->s_comp_code)));
            // Banck ID constant for journal entry export
            define('EIBS_JOURNAL_ENTRY_EXPORT_BANK', trim(companyParameter('EIBS_JEEX_EXPORT_BANK', $this->s_comp_code)));
            // Branch ID constant for journal entry export
            // 903
            define('EIBS_JOURNAL_ENTRY_EXPORT_BRANCH', trim(companyParameter('EIBS_JEEX_EXPORT_BRANCH', $this->s_comp_code)));
            // Banck ID constant for internal transfer export

            $s_eibs_itrsf_export_bank = trim(companyParameter('EIBS_ITRSF_EXPORT_BANK', $this->s_comp_code));
            if (is_null($s_eibs_itrsf_export_bank)) {
                $s_eibs_itrsf_export_bank = '';
            }
            define('EIBS_ITRSF_EXPORT_BANK', $s_eibs_itrsf_export_bank);
            // Cost Center constant for G/L and transfers export
            $s_eibs_jeex_export_costcenter = trim(companyParameter('EIBS_JEEX_EXPORT_COSTCENTER', $this->s_comp_code));
            if (is_null($s_eibs_jeex_export_costcenter)) {
                $s_eibs_jeex_export_costcenter = '';
            }
            define('EIBS_JEEX_EXPORT_COSTCENTER', $s_eibs_jeex_export_costcenter);
            // Branch ID constant for journal entry export
            // 903
            $s_eibs_itrsf_export_branch = trim(companyParameter('EIBS_ITRSF_EXPORT_BRANCH', $this->s_comp_code));
            if (is_null($s_eibs_itrsf_export_branch)) {
                $s_eibs_itrsf_export_branch = '';
            }
            define('EIBS_ITRSF_EXPORT_BRANCH', $s_eibs_itrsf_export_branch);
            
            $s_eibs_chkprt_name_wordwrap = trim(companyParameter('EIBS_CHKPRT_NAME_WORDWRAP', $this->s_comp_code));
            if(is_null($s_eibs_chkprt_name_wordwrap)) {
                $s_eibs_chkprt_name_wordwrap = '0';
            }
            define('EIBS_CHKPRT_NAME_WORDWRAP', $s_eibs_chkprt_name_wordwrap);
        }
        if (!empty($this->debug)) {
            echoLog('EIBS_CHKPRT_TRSFR_ACC_ENC=' . EIBS_CHKPRT_TRSFR_ACC_ENC);
            echoLog('FIDU_BANKACT_CODE_MAXLEN=' . FIDU_BANKACT_CODE_MAXLEN);
            echoLog('EIBS_JOURNAL_ENTRY_EXPORT_BANK=' . EIBS_JOURNAL_ENTRY_EXPORT_BANK);
            echoLog('EIBS_JOURNAL_ENTRY_EXPORT_BRANCH=' . EIBS_JOURNAL_ENTRY_EXPORT_BRANCH);
            echoLog('EIBS_CHKPRT_SPECIAL_USER=' . EIBS_CHKPRT_SPECIAL_USER);
            echoLog('EIBS_CHKPRT_DEF_COLO=' . EIBS_CHKPRT_DEF_COLO);
        }
    }

    /*
     * Returns a list of FIAT's IDCODEs ready to include it as 'WHERE IDCODE IN (...)' sql clause
     * The transactions to be included must has approved (AP) status
     * @param text
     * @return text
     */
    function fiat_idcode_list($s_trans_idcode = null) {
        $s_idcode_list = '';
        if ($s_trans_idcode != null) {
            // If the transaction code is specified... 	    
            $s_idcode_list .= singleQuote($s_trans_idcode);
        } else {
            // If the transaction code isn') . 't specified, assumes that it comes from a Listing of the UI
            $idlist = getIdlist();
            $sql = 'SELECT FIAT.IDCODE	FROM  ' . $this->fiatname . ' FIAT WHERE ID IN (' . singleQuote(implode("','", $idlist)) . ') AND FIAT.TRANS_STATUS IN (' . singleQuote('AP') . ', ' . singleQuote('CLOS') . ')';
            if (!empty($this->debug))
                echoLog(debug_dump('fiat_idcode_list - <br>' . CLX_CHAR_NL . $sql . '<br>' . CLX_CHAR_NL . 'idlist', $idlist));
            $rs = ClxCommon::$db->executeQuery($sql);
            $s_comma = '';
            while (($fiatrec = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE))) {
                $s_idcode_list .= $s_comma . singleQuote($fiatrec['IDCODE']);
                $s_comma = ',';
            }
			ClxCommon::$db->freeResult($rs);
        }
        if (!empty($this->debug)) {
            echoLog('s_idcode_list=' . $s_idcode_list);
        }
        return $s_idcode_list;
    }

    /*
     * Arranges the bank account code according the specs of IBS
     * @param text, boolean
     * @return text
     */
    function bankact_code_fix($s_bankact_code) {
        $n_maxlen = (EIBS_CHKPRT_TRSFR_ACC_ENC === '') ? CLX_NUM_ZERO : (integer) EIBS_CHKPRT_TRSFR_ACC_ENC;
        if ($n_maxlen != CLX_NUM_ZERO && strlen($s_bankact_code) > $n_maxlen) {
            $s_bankact_code = substr($s_bankact_code, -$n_maxlen);
        }
        return $s_bankact_code;
    }

    /*
     * SQL Select extra column inclusion according to some clientos FIAT structure
     * Returns list of extra columns separated by comma
     * @param void
     * @return text
     */
    function printChecks_sql_stat_ver_flds($s_disbursement_type) {
        $fiatname = TableName('FIAT');
        //  BHN
        $s_sql_extra_pars_select_local = '';
        // Budget fields (BDA / BNP)
        if (ClxCommon::$db->fieldExists($fiatname, 'FISCAL_YEAR')) {
            $s_sql_extra_pars_select_local .= '
					FIAT.FISCAL_YEAR,
					FIAT.EXT_BUDGET_ITEM,
					FIAT.EXT_ITEM_NO,
					FIAT.EXT_DOCUMENT_TYPE,
					FIAT.RESERVE_YEAR,
					FIAT.BUDGET_ITEM_TYPE_01,
					FIAT.BUDGET_ITEM_AMT_01,
					FIAT.BUDGET_ITEM_TYPE_02,
					FIAT.BUDGET_ITEM_AMT_02,
			';
        }
        
        // Fund Source (BNP / BHN)
        if (ClxCommon::$db->fieldExists($fiatname, 'FUND_SOURCE')) {
            $s_sql_extra_pars_select_local .= '
					FIAT.FUND_SOURCE,
					' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' FUND_SOURCE_DESC,
			';
        }

        // Allow Check Print (even if client is in black list)
        if (ClxCommon::$db->fieldExists($fiatname, 'CHECK_ALLOW_PR_BL')) {
            $s_sql_extra_pars_select_local .= '
					FIAT.CHECK_ALLOW_PR_BL,
			';
        }

        // Allow destination GL account (for internal transfers)
        if (ClxCommon::$db->fieldExists($fiatname, 'GLACCOUNTCODE2')) {
            $s_sql_extra_pars_select_local .= '
					FIAT.GLACCOUNTCODE2 FIAT_GLACCOUNTCODE2,
			';
        } else {
            $s_sql_extra_pars_select_local .= '	' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' FIAT_GLACCOUNTCODE2, ';
        }
        // @TODO
        if ($s_disbursement_type == EBSK_IBS_DEF_ET_TOKEN) {
            // Specific columns for Electronic Transfers (ACH)
            $s_sql_extra_pars_select_local .= '
				FIAT.BANKAC_CODE FIAT_BANKAC_CODE,
				FIAT.BANK_ACCOUNT_NO BANK_ACCOUNT_NO_ET_DEST,
				FIAT.BANK_NAME BANK_NAME_ET_DEST,
				' . singleQuote('C') . ' BANK_PROD_TYPE_ET_DEST,
				FIAT.BANK_ROUTING_NO BANK_ROUTING_NO_ET_DEST,
				' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' BANK_INTERMEDIARY_BANK_ET_DEST,
				' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' BANK_FUTHER_CREDIT_ET_DEST,
				FIAT.BANK_SWIFT BANK_SWIFT_ET_DEST,
				FIAT.BANK_ADDRESS BANK_ADDRESS_ET_DEST,
				FIAT.BANK_COUNTRY BANK_COUNTRY_ET_DEST,
				FIAT.BANK_STATE BANK_STATE_ET_DEST,
				FIAT.BANK_CITY BANK_CITY_ET_DEST,
			';
            $s_sql_extra_pars_select_local .=
                    singleQuote(' ') . ' ET_OPERATION,
				' . singleQuote('T') . ' ET_TYPE,
				' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' ET_TRANSACTION,
				FIAT.EFFTDATE VALDTE_YY,
				FIAT.EFFTDATE VALDTE_MM,
				FIAT.EFFTDATE VALDTE_DD,
			';
        }
        return $s_sql_extra_pars_select_local;
    }

    /*
     * Returns SQL SELECT statement to extract data from the FIBT, FIAT, ACCO, FIAS and FIBK
     * suitable for print checks and fund transfers
     * @param text, text, text, text
     * @return text
     */
    function printChecks_sql_statement($s_idcode_list, $s_disbursement_type, $s_sql_extra_pars_select = '', $s_sql_extra_pars_where = '') {
        if ($s_sql_extra_pars_select !== '')
            $s_sql_extra_pars_select .= ',';
        if ($s_sql_extra_pars_where !== '')
            $s_sql_extra_pars_where = ' AND ' . $s_sql_extra_pars_where;
        //  BHN
        $s_sql_extra_pars_select_local = $this->printChecks_sql_stat_ver_flds($s_disbursement_type);
        if ($this->sw_old_acco_struct === true) {
            // Old version
            $sql = '
			SELECT	' . $s_sql_extra_pars_select . ' COLO.PRINT_USER,
				FIBT.ID FIBT_ID,
				FIBT.RECORD_BLOCKING,
				FIBT.TRANS_CODE,
				FIBT.TRANS_AMT,
				FIBT.BENEFICIARY_NAME,
				FIBT.IDENTIFICATION FIBT_IDENTIFICATION,
				FIBT.LEGAL_TYPE FIBT_LEGAL_TYPE,
				FIBT.CHECK_NO CHECK_NUMBER,
				ACCO.LEGAL_TYPE,
				ACCO.ISINDIVIDUAL,
				ACCO.IDENTIFICATION ACCO_IDENTIFICATION,
				ACCO.IDNAME IDNAME_ACCO,
				ACCO.IDCODE IDCODE_ACCO,
				ACCO.LEGAL_NAME LEGAL_NAME_ACCO,
				FIAT.CONTRACT_CODE,
				FIAT.IDCODE IDCODE_FIAT,
				FIAT.IDNAME IDNAME_FIAT,
				FIAT.DESCRIPTION,
				FIAT.CURRENCY_CODE,
				FIAT.CONTRACT_CODE,
				FIAT.TRANSTBL_CODE,
				FIAT.AFFECT_CASH,
				FIAT.REVTRANS_CODE,
				FIBT.DISBURSEMENT_TYPE,
				FIAS.FIDU_BANKACT_CODE FIDU_BANKACT_CODE_FIAS,
				FIBK_FIAS.BANK_ROUTING_NO BANK_ROUTING_NO_FIBK_FIAS,
				FIBK_FIAS.BANK_SWIFT BANK_SWIFT_FIBK_FIAS,
				FIBK_FIAS.CITY CITY_FIBK_FIAS,
				FIBK_FIAS.GLACCOUNTCODE GLACCOUNTCODE_FIBK_FIAS,
				FIBK_FIAS.CURRENCY_CODE CURRENCY_CODE_FIBK_FIAS,
				FIAT.FIDU_BANKACT_CODE FIDU_BANKACT_CODE_FIAT,
				FIBK_FIAT.BANK_ROUTING_NO BANK_ROUTING_NO_FIBK_FIAT,
				FIBK_FIAT.BANK_SWIFT BANK_SWIFT_FIBK_FIAT,
				FIBK_FIAT.CITY CITY_FIBK_FIAT,
				FIBK_FIAT.GLACCOUNTCODE GLACCOUNTCODE_FIBK_FIAT,
				FIBK_FIAT.CURRENCY_CODE CURRENCY_CODE_FIBK_FIAT
			FROM
				' . $this->fibtname . ' FIBT
				LEFT JOIN ' . $this->fiatname . ' FIAT ON FIBT.TRANS_CODE = FIAT.IDCODE
				LEFT JOIN ' . $this->acconame . '  ACCO ON FIBT.ACCOUNTCODE = ACCO.IDCODE
				LEFT JOIN ' . $this->fiasname . '  FIAS ON FIAS.IDCODE = FIAT.CASH_ASSET_CODE
				LEFT JOIN ' . $this->fibkname . '  FIBK_FIAS ON FIBK_FIAS.BANK_ACCOUNT_NO = FIAS.FIDU_BANKACT_CODE
				LEFT JOIN ' . $this->fibkname . '  FIBK_FIAT ON FIBK_FIAT.BANK_ACCOUNT_NO = FIAT.FIDU_BANKACT_CODE
				LEFT JOIN ' . $this->coloname . '  COLO ON COLO.IDCODE = FIBT.COMPLOC_CODE
			WHERE
				FIBT.DISBURSEMENT_TYPE = ' . singleQuote($s_disbursement_type) . '
				AND FIBT.TRANS_CODE IN (' . $s_idcode_list . ')
				AND (ISnull(FIBT.RECORD_BLOCKING) OR FIBT.RECORD_BLOCKING = ' . singleQuote('0') . ') ' . $s_sql_extra_pars_where . '
			ORDER BY
				FIBT.TRANS_CODE
			';
        } else {
            // New version
            $this->fibtname = $this->fiatname;
            $sql = ' SELECT ' . $s_sql_extra_pars_select . ' COLO.PRINT_USER, FIAT.ID FIBT_ID, FIAT.IDCODE TRANS_CODE, FIAT.*, FIAT.BENEFICIARY_CODE FIBT_BENEFICIARY_CODE, FIAT.IDENTIFICATION FIBT_IDENTIFICATION, FIAT.LEGAL_TYPE FIBT_LEGAL_TYPE, FIAT.ADDRESS BENEFICIARY_ADDRESS, FIAT.COUNTRY BENEFICIARY_COUNTRY, FIAT.STATE BENEFICIARY_STATE, ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' BENEFICIARY_CITY, FIAT.ZIPCODE BENEFICIARY_ZIPCODE, ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' BENEFICIARY_EMAIL, FIAT.CHECK_NO CHECK_NUMBER, FIAT.GLACCOUNTCODE FIAT_GLACCOUNTCODE, FIAT.BRANCH BRANCH_CODE,'
                    . $s_sql_extra_pars_select_local . ' ACCO.LEGAL_TYPE, ACCO.ISINDIVIDUAL, ACCO.IDENTIFICATION ACCO_IDENTIFICATION, ACCO.IDNAME IDNAME_ACCO, ACCO.IDCODE IDCODE_ACCO, ACCO.LEGAL_NAME LEGAL_NAME_ACCO, FIAT.CHART_FIELD01 CONTRACT_CODE, FIAT.IDCODE IDCODE_FIAT, FIAT.IDNAME IDNAME_FIAT, FITB.DESCRIPTION, ' . singleQuote('CR') . ' AFFECT_CASH, FIAT.REV_TRANS_CODE REVTRANS_CODE, FIAS.FIDU_BANKACT_CODE FIDU_BANKACT_CODE_FIAS, FIBK_FIAS.BANK_ROUTING_NO BANK_ROUTING_NO_FIBK_FIAS, FIBK_FIAS.BANK_SWIFT BANK_SWIFT_FIBK_FIAS, FIBK_FIAS.CITY CITY_FIBK_FIAS, ( SELECT PRAT.PRO_GL_CODE GLACCOUNTCODE FROM FIDU_BANK_ACCOUNTS FIBK, GL_PROD_PRMTR_TBL PRAT WHERE PRAT.PRODUCT_CODE = FIBK.BANK_PROD_CODE AND PRAT.PROD_STATUS =' . singleQuote('VIG') . ' AND FIBK.BANK_ACCOUNT_NO = FIBK_FIAS.BANK_ACCOUNT_NO ) GLACCOUNTCODE_FIBK_FIAS, FIAT.CURRENCY_CODE CURRENCY_CODE_FIBK_FIAS, FIAT.FIDU_BANKACT_CODE FIDU_BANKACT_CODE_FIAT, FIBK_FIAT.BANK_ROUTING_NO BANK_ROUTING_NO_FIBK_FIAT, FIBK_FIAT.BANK_SWIFT BANK_SWIFT_FIBK_FIAT, FIBK_FIAT.CITY CITY_FIBK_FIAT, ( SELECT PRAT.PRO_GL_CODE GLACCOUNTCODE FROM FIDU_BANK_ACCOUNTS FIBK, GL_PROD_PRMTR_TBL PRAT WHERE PRAT.PRODUCT_CODE = FIBK.BANK_PROD_CODE AND PRAT.PROD_STATUS =' . singleQuote('VIG') . ' AND FIBK.BANK_ACCOUNT_NO = FIBK_FIAT.BANK_ACCOUNT_NO ) GLACCOUNTCODE_FIBK_FIAT, FIAT.CURRENCY_CODE CURRENCY_CODE_FIBK_FIAT FROM ' . $this->fiatname . ' FIAT LEFT JOIN ' . $this->fitbname . ' FITB ON FIAT.TRANSTBL_CODE = FITB.IDCODE LEFT JOIN ' . $this->acconame . ' ACCO ON FIAT.CHART_FIELD02 = ACCO.IDCODE LEFT JOIN ' . $this->fiasname . ' FIAS ON FIAS.IDCODE = FIAT.CHART_FIELD03 AND FIAS.GL_TYPE = ' . "'AS'" . ' AND FIAS.TRUST_ITEM_TYPE = ' . singleQuote('CA') . ' LEFT JOIN ' . $this->fibkname . ' FIBK_FIAS ON FIBK_FIAS.BANK_ACCOUNT_NO = FIAS.FIDU_BANKACT_CODE LEFT JOIN ' . $this->fibkname . ' FIBK_FIAT ON FIBK_FIAT.BANK_ACCOUNT_NO = FIAT.FIDU_BANKACT_CODE LEFT JOIN ' . $this->coloname . ' COLO ON COLO.IDCODE = FIAT.BRANCH_PRN WHERE FIAT.DISBURSEMENT_TYPE = ' . singleQuote($s_disbursement_type) . ' AND FIAT.IDCODE IN (' . $s_idcode_list . ') /*FJSH 2014-02-07*/ AND FIAT.RECORD_BLOCKING = ' . singleQuote('3') . ' AND ( ( (ISnull(FIAT.VOID) OR FIAT.VOID = 0) AND (REV_TRANS_CODE = ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' OR ISnull(REV_TRANS_CODE)) ) OR ( FIAT.VOID = 1 AND REV_TRANS_CODE <> ' . singleQuote(CLX_CHAR_EMPTY_STRING) . 'AND NOT ISnull(REV_TRANS_CODE) ) ) ' . $s_sql_extra_pars_where . '  ORDER BY FIAT.IDCODE ';
        }
        if (!empty($this->debug))
            echoLog('<br>' . CLX_CHAR_NL . $sql . '<br>' . CLX_CHAR_NL);
        return $sql;
    }

    /*
     * Get debit (DR) or credit (CR) depending on where the disbursement amount is in, to validate if check's journal entry  is well formed. For checks must be always CR
     * @param text, text
     * @return text
     */
    function printChecks_crdb_get($s_trans_code, $s_glaccountcode_fibk_fias) {
        $sql = 'SELECT
					 IF(SUM(JELN.DEBIT_AMT-JELN.CREDIT_AMT)>0, ' . singleQuote('DR') . ',' . singleQuote('CR') . ') AFFECT_CASH
				 FROM
					 GL_JOURNAL_LINES JELN,
					 GL_JOURNAL_HDR JEHD
				 WHERE
					 JEHD.IDCODE = JELN.JEHDR_CODE
					 AND JEHD.TRANS_CODE = ' . singleQuote($s_trans_code) . '
					 AND JELN.GLACCOUNTCODE = ' . singleQuote($s_glaccountcode_fibk_fias);
        if (!empty($this->debug))
            echoLog('<br>' . CLX_CHAR_NL . $sql . '<br>' . CLX_CHAR_NL);
        $ar_crdb = ClxCommon::$db->getOneRecord($sql);
        return $ar_crdb['AFFECT_CASH'];
    }

    /*
     * Returns FIBT's payee info, particullary the payee's name with the
     * logic in the beneficiary_name_build() method
     * @param text
     * @return array
     */
    function get_special_payee_record_data($s_id) {
        if ($this->sw_old_acco_struct === true) {
            // Old version
            $sql = '
				SELECT
					FIBT.TRANS_CODE,
					FIBT.TRANS_AMT,
					FIBT.BENEFICIARY_NAME,
					ACCO.LEGAL_TYPE,
					ACCO.ISINDIVIDUAL,
					ACCO.IDNAME IDNAME_ACCO,
					ACCO.IDCODE IDCODE_ACCO,
					ACCO.LEGAL_NAME LEGAL_NAME_ACCO
				FROM ' . $this->fibtname . ' FIBT
						LEFT JOIN ' . $this->acconame . ' ACCO ON FIBT.ACCOUNTCODE = ACCO.IDCODE
				WHERE
					FIBT.ID = ' . singleQuote($s_id) . '
				';
        } else {
            // New version
            $sql = '
				SELECT
					FIAT.IDCODE TRANS_CODE,
					FIAT.TRANS_AMT,
					FIAT.BENEFICIARY_NAME,
					ACCO.LEGAL_TYPE,
					ACCO.ISINDIVIDUAL,
					ACCO.IDNAME IDNAME_ACCO,
					ACCO.IDCODE IDCODE_ACCO,
					ACCO.LEGAL_NAME LEGAL_NAME_ACCO
				FROM ' . $this->fiatname . ' FIAT
						LEFT JOIN ' . $this->acconame . ' ACCO ON FIAT.BENEFICIARY_CODE = ACCO.IDCODE
				WHERE
					FIAT.ID = ' . singleQuote($s_id) . '
				';
        }
        $rs = ClxCommon::$db->executeQuery($sql);
        $fibtrec = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE);
		ClxCommon::$db->freeResult($rs);
        $fibtrec['BENEFICIARY_NAME'] = $this->beneficiary_name_build($fibtrec);
        return $fibtrec;
    }

    /*
     * Prepares the record based on data extracted from the FIBT, FIAT, ACCO, FIAS and FIBK
     * suitable for print checks and fund transfers
     * @param array, array, boolean
     * @return array
     */
    function printChecks_prepares_record($fibtrec, $mappings, $b_get_applicant_data = true) {
        $DBLT = getTool('DBLT');
        $EBSK = getTool('EBSK');
        // Prepares error fields... 	
        $fibtrec['error'] = false;
        $fibtrec['error_status_adic'] = '';
        $fibtrec['errormsg'] = array();
        // Adds credit disbursement type transaction specific data
        $fibtrec = $this->credit_trans_add_info($fibtrec);
        if ($fibtrec['error'] === true) {
            return $fibtrec;
        }
        // Prepares the data to be transmitted
        $record = $fibtrec;
        // Payee name construction
        $record['BENEFICIARY_NAME'] = $this->beneficiary_name_build($record);
        //   - Caso 1002467 - BC-GAP-Interfaz con IBS
        // Extracts trust contract info
        $fico_record = ClxCommon::$db->getRecord($this->ficoname, 'IDCODE = ' . singleQuote($fibtrec['CONTRACT_CODE']));
        /*
         * CODE_GEN_FIDUCNTRCTNUM
         * Trust # ^1
         * Fideicomiso # ^1
         */
        //   - Caso 1002467 - BC-GAP-Interfaz con IBS

        $s_desc_fidu_contract = dict('CODE_GEN_FIDUCNTRCTNUM', $fico_record['CONTRACT_NUMBER']);
        /*
         * CODE_GEN_TRANSACNUM
         * Transaction # ^1
         * Transaccion # ^1
         */
        $s_desc_transac = dict('CODE_GEN_TRANSACNUM', $fibtrec['IDCODE_FIAT']);
        $record['IDNAME_FIAT'] = $fibtrec['IDNAME_FIAT'];
        $record['CONTRACT_CODE'] = $fibtrec['CONTRACT_CODE'];
        $record['FIDU_CONTRACT'] = $s_desc_fidu_contract;

        $record['TRANS_AMT'] = rounder($fibtrec['TRANS_AMT'], CLX_NUM_TWO);
        $record['CURRENCY_CODE'] = $fibtrec['CURRENCY_CODE'];
        if ($fibtrec['DESCRIPTION'] !== '') {
            $record['DESCRIPTION'] = $fibtrec['DESCRIPTION'];
        } else {
            $record['DESCRIPTION'] = $s_desc_transac;
        }
        $i_concept_size = $this->get_sktdef_size('E01OFMCO1', $mappings);
        $record['FIDU_TRANSACTION'] = fija($fibtrec['DESCRIPTION'], $this->get_sktdef_size('E01OFMDSC', $mappings));
        $record['TRANSTBL_DESC'] = fija(ClxCommon::$db->getField($this->fitbname, 'IDNAME', 'IDCODE = ' . singleQuote($fibtrec['TRANSTBL_CODE'])), $i_concept_size);
        $record['CREDIT_DESCRIPTION'] = fija($record['CREDIT_DESCRIPTION'], $i_concept_size);
        // Get applicant data
        if (!empty($b_get_applicant_data)) {
            $record = $this->applicant_data_get($record, $fibtrec['CONTRACT_CODE']);
            $ar_txt = $this->wordwrap_name($record['APPLICANT_NAME'], CLX_NUM_30, CLX_NUM_THREE);
            $record['APPLICANT_NAME_1'] = $ar_txt[CLX_NUM_ZERO];
            $record['APPLICANT_NAME_2'] = $ar_txt[CLX_NUM_ONE];
            $record['APPLICANT_NAME_3'] = $ar_txt[CLX_NUM_TWO];
        }
        $ar_txt = $this->wordwrap_name($record['BENEFICIARY_NAME'], CLX_NUM_30, CLX_NUM_THREE);
        $record['BENEFICIARY_NAME_1'] = $ar_txt[CLX_NUM_ZERO];
        $record['BENEFICIARY_NAME_2'] = $ar_txt[CLX_NUM_ONE];
        $record['BENEFICIARY_NAME_3'] = $ar_txt[CLX_NUM_TWO];
        $record['BANK_CODE'] = EIBS_JOURNAL_ENTRY_EXPORT_BANK;

        // FIAT specific branch
        if (is_null($record['BRANCH_CODE']) || trim($record['BRANCH_CODE']) === '') {
            $record['BRANCH_CODE'] = EIBS_JOURNAL_ENTRY_EXPORT_BRANCH;
        }
        //SPECIAL_USERNAME= VA A ESTAR ATADO A LA AGENCIA DE IMPRESION (para el caso de cheques de genrecia)
        $record['SPECIAL_USERNAME'] = $fibtrec['PRINT_USER'];
        if (is_null($record['SPECIAL_USERNAME']) || trim($record['SPECIAL_USERNAME']) === '') {
            $record['SPECIAL_USERNAME'] = ClxCommon::$db->getField($this->coloname, 'PRINT_USER', 'IDCODE = ' . singleQuote(EIBS_CHKPRT_DEF_COLO));
        }
        if (is_null($record['SPECIAL_USERNAME']) || trim($record['SPECIAL_USERNAME']) === '') {
            // Takes the value from the special check print user from the general parameters
            if ($record['DISBURSEMENT_TYPE'] == EBSK_IBS_DEF_ET_TOKEN) {
                $record['SPECIAL_USERNAME'] = EIBS_ACH_SPECIAL_USER;
            } else {
                $record['SPECIAL_USERNAME'] = EIBS_CHKPRT_SPECIAL_USER;
            }
        }
        if (is_null($record['SPECIAL_USERNAME']) || trim($record['SPECIAL_USERNAME']) === '') {
            // Takes the value from the current user
            $record['SPECIAL_USERNAME'] = $_SESSION['user_record']['username'];
        }

        // Source bank account...
        if ($fibtrec['FIDU_BANKACT_CODE_FIAT'] !== '') {
            // Source bank account from transaction FIAT
            $record['FIDU_BANKACT_CODE'] = $fibtrec['FIDU_BANKACT_CODE_FIAT'];
            $record['BANK_ROUTING_NO'] = $fibtrec['BANK_ROUTING_NO_FIBK_FIAT'];
            $record['BANK_SWIFT'] = $fibtrec['BANK_SWIFT_FIBK_FIAT'];
            $s_city_code = $fibtrec['CITY_FIBK_FIAT'];
            $record['GLACCOUNTCODE'] = $fibtrec['GLACCOUNTCODE_FIBK_FIAT'];
        } elseif ($fibtrec['FIAT_GLACCOUNTCODE'] !== '') {
            $record['FIDU_BANKACT_CODE'] = '';
            $record['BANK_ROUTING_NO'] = '';
            $record['BANK_SWIFT'] = '';
            $s_city_code = '';
            $record['GLACCOUNTCODE'] = $fibtrec['FIAT_GLACCOUNTCODE'];
        } elseif ($fibtrec['FIDU_BANKACT_CODE_FIAS'] !== '') {
            // Source bank account from the asset FIAS
            $record['FIDU_BANKACT_CODE'] = $fibtrec['FIDU_BANKACT_CODE_FIAS'];
            $record['BANK_ROUTING_NO'] = $fibtrec['BANK_ROUTING_NO_FIBK_FIAS'];
            $record['BANK_SWIFT'] = $fibtrec['BANK_SWIFT_FIBK_FIAS'];
            $s_city_code = $fibtrec['CITY_FIBK_FIAS'];
            $record['GLACCOUNTCODE'] = $fibtrec['GLACCOUNTCODE_FIBK_FIAS'];
        } else {
            $record['error'] = true;
            $record['error_status_adic'] = '(X)';
            /*
             * CODE_ERR_IBS_CHKPRT_NOSRCDATA
             * Data incosistency: No source Bank or G/L Account information
             * Inconsistencia de datos: no hay informacion de Cuenta de Banco o Cuenta Contable de donde extraer los fondos
             */
            $record['errormsg'][] = dict('CODE_ERR_IBS_CHKPRT_NOSRCDATA');
        }
        // Data fixes
        if ($s_city_code !== '')
            $record['BANK_CITY'] = $DBLT->getListItem('LOCI', $s_city_code, CLX_NUM_TWO);
        // Determine fund source (BANK or LEDGER ACCOUNT)
        if (is_null($record['FIAT_GLACCOUNTCODE']) || trim($record['FIAT_GLACCOUNTCODE']) === '' || trim($record['FIAT_GLACCOUNTCODE']) === 'null') {
            // From a bank account
            $record['FIDU_GLACC_CODE'] = '';
            $record['FIDU_BANKACT_CODE'] = $this->bankact_code_fix($record['FIDU_BANKACT_CODE']);
            $record['E01DEBOPC'] = '01';
        } else {
            // From a GL account
            $record['FIDU_GLACC_CODE'] = $record['FIAT_GLACCOUNTCODE'];
            $record['FIDU_BANKACT_CODE'] = '';
            $record['E01DEBOPC'] = '02';
        }
        // It must send '0' and IBS will return the check number in this field
        $record['CHECK_NUMBER'] = $fibtrec['CHECK_NUMBER'];
        if ($fibtrec['CHECK_NUMBER'] === '') {
            $record['CHECK_NUMBER'] = '0';
        }
        $record['AFFECT_CASH'] = $this->printChecks_crdb_get($record['TRANS_CODE'], $record['GLACCOUNTCODE']);
        if ($record['DISBURSEMENT_TYPE'] === 'CH' && $record['AFFECT_CASH'] !== 'CR') {
            $record['error'] = true;
            $record['error_status_adic'] = '(X)';
            /*
             * CODE_ERR_IBS_CHKPRT_NOTCR
             * The ledger account of bank account must be present in the journal entry and the amount should be a credit.
             * La cuenta contable de la cuenta de banco debe estar presente en el asiento contable y el monto debe ir por el haber.
             */
            $record['errormsg'][] = dict('CODE_ERR_IBS_CHKPRT_NOTCR');
        }
        //  BNP
        // Fix check's funds source description
        $i_concept_size = $this->get_sktdef_size('E01SPIDS1', $mappings);
        if (!is_null($i_concept_size)) {
            // Field E01SPIDS1 exists...
            if (!is_null($record['FUND_SOURCE']) && $record['FUND_SOURCE'] !== '') {
                if ($record['FUND_SOURCE_DESC'] === '') {
                    $record['FUND_SOURCE_DESC'] = $DBLT->getListItem('FUSR', $record['FUND_SOURCE'], CLX_NUM_TWO);
                }
            }
            $record['FUND_SOURCE_DESC'] = fija($record['FUND_SOURCE_DESC'], $i_concept_size);
            $record['FUND_SOURCE'] = fija($record['FUND_SOURCE'], $this->get_sktdef_size('E01SPIDS5', $mappings));
        }
        //   BHN / BDA
        // Date digit by Digit
        $i_concept_size = $this->get_sktdef_size('E01OFME11', $mappings);
        if (!is_null($i_concept_size)) {
            // Field E01OFME11 exists... 	    
            $fiat_date = $record['EFFTDATE']; // YYYY-MM-DD
            $record['E01OFME11'] = substr($fiat_date, CLX_NUM_FIVE, CLX_NUM_ONE); // M1
            $record['E01OFME12'] = substr($fiat_date, CLX_NUM_SIX, CLX_NUM_ONE); // M2
            $record['E01OFME21'] = substr($fiat_date, CLX_NUM_8, CLX_NUM_ONE); // D1
            $record['E01OFME22'] = substr($fiat_date, CLX_NUM_NINE, CLX_NUM_ONE); // D2
            $record['E01OFME31'] = substr($fiat_date, CLX_NUM_ZERO, CLX_NUM_ONE); // Y1
            $record['E01OFME32'] = substr($fiat_date, CLX_NUM_ONE, CLX_NUM_ONE); // Y1
            $record['E01OFME33'] = substr($fiat_date, CLX_NUM_TWO, CLX_NUM_ONE); // Y1
            $record['E01OFME34'] = substr($fiat_date, CLX_NUM_THREE, CLX_NUM_ONE); // Y1
        }

        // External Transfers (ACH): Check destination account source (FIAT or FIBK)
        if ($record['DISBURSEMENT_TYPE'] == EBSK_IBS_DEF_ET_TOKEN) {
            if (!is_null($record['FIAT_BANKAC_CODE']) && $record['FIAT_BANKAC_CODE'] !== '') {  // @TODO
                // Get bank account info from table FIBK
                $a_fibk_dest = ClxCommon::$db->getRecord($this->fibkname, 'BANK_ACCOUNT_NO = ' . singleQuote($record['FIAT_BANKAC_CODE']));
                $record['BANK_ACCOUNT_NO_ET_DEST'] = $a_fibk_dest['BANK_ACCOUNT_NO'];
                $record['BANK_NAME_ET_DEST'] = $a_fibk_dest['BANK_NAME'];
                $record['BANK_PROD_TYPE_ET_DEST'] = $a_fibk_dest['BANK_PROD_TYPE'];
                $record['BANK_ROUTING_NO_ET_DEST'] = $a_fibk_dest['BANK_ROUTING_NO'];
                $record['BANK_INTERMEDIARY_BANK_ET_DEST'] = $a_fibk_dest['INTERMEDIARY_BANK'];
                $record['BANK_FUTHER_CREDIT_ET_DEST'] = $a_fibk_dest['FUTHER_CREDIT'];
                $record['BANK_SWIFT_ET_DEST'] = $a_fibk_dest['BANK_SWIFT'];
                $record['BANK_ADDRESS_ET_DEST'] = $a_fibk_dest['ADDRESS'];
                $record['BANK_COUNTRY_ET_DEST'] = $a_fibk_dest['COUNTRY'];
                $record['BANK_STATE_ET_DEST'] = $a_fibk_dest['STATE'];
                $record['BANK_CITY_ET_DEST'] = $a_fibk_dest['CITY'];
            }
            //  -  - Reported by Ruben Gonzalez at BNP... 	    // Fix specific ACH fields
            $i_concept_size = $this->get_sktdef_size('ACHOUTNME', $mappings);
            $record['BENEFICIARY_NAME'] = fija($record['BENEFICIARY_NAME'], $i_concept_size);
            $i_concept_size = $this->get_sktdef_size('ACHOUTADR', $mappings);
            $record['BENEFICIARY_ADDRESS'] = fija($record['BENEFICIARY_ADDRESS'], $i_concept_size);
            $i_concept_size = $this->get_sktdef_size('ACHOUTCSZ', $mappings);
            $record['BENEFICIARY_CSZ'] = fija($record['BENEFICIARY_CSZ'], $i_concept_size);
            $i_concept_size = $this->get_sktdef_size('ACHOUTEMA', $mappings);
            $record['BENEFICIARY_EMAIL'] = fija($record['BENEFICIARY_EMAIL'], $i_concept_size);
        }
        //   - Caso 1002467 - BC-GAP-Interfaz con IBS
        // Gets the check full description dictionary
        $chk_full_desc = dict('CODE_GEN_CHK_FULL_DESC');
        if (substr($chk_full_desc, CLX_NUM_ZERO, CLX_NUM_8) === '<a href=' || $chk_full_desc === '' || is_null($chk_full_desc)) {
            // If not registered, use default dictionary entry (as old check interface versions did)
            $chk_full_desc = $s_desc_fidu_contract . ', ' . $s_desc_transac . ', ' . $record['IDNAME_FIAT'];
        }
        // Replaces all contracts and transaction fields in the check full description
        $chk_full_desc = $this->replace_all_fields_in_string($chk_full_desc, $record);
        $chk_full_desc = $this->replace_all_fields_in_string($chk_full_desc, $fico_record);
        $i_concept_size = $this->get_sktdef_size('E01OFMCO1', $mappings);
        $record['TRANS_FULL_DESC'] = fija($chk_full_desc, $i_concept_size);
        // Caso 1002467 - BC-GAP-Interfaz con IBS
        // Adjust check print format to socket msg field size. 	
        // That's because in IBS the check format could have repeating format codes
        // for different currencies. So IBS must difference it in combination with
        // currency specified in E01OFMCCY (FIAT.CURRENCY_CODE). In CLX, List 'FCHK' related
        // to field FIAT.CHECK_PRINT_FMT will have the first 2 digit of IBS's check format
        // plus the currency code in order to register the repeating codes. 	
        $i_concept_size = $this->get_sktdef_size('E01OFMFTY', $mappings);
        if ($i_concept_size > CLX_NUM_ZERO) {
            $record['CHECK_PRINT_FMT'] = fija($record['CHECK_PRINT_FMT'], $i_concept_size);
        }
        return $record;
    }
        
    /*
     * Return an array of $lines elements with the string $atext separated with word-wrap-arround style at $chars columns
     * @param text, integer, variant
     * @return array
     */
    function wordwrap_name($atext, $chars, $lines = null) {
        if(EIBS_CHKPRT_NAME_WORDWRAP === '1') {
            // Divide el string haciendo Word Wrap Arround tomando en cuenta los espacios en blanco
            $txt = wordwrap($atext, $chars, '<oo>', true);
            $ar = explode('<oo>', $txt);
            // Complete array to be $lines entries long
            if (!is_null($lines) && count($ar) < $lines) {
                $from = count($ar) + CLX_NUM_ONE;
                for ($i = $from; $i <= $lines; $i++) {
                    $ar[$i - CLX_NUM_ONE] = '';
                }
            }
        } else {
            // Divide el string en frangmentos de $chars caracteres
            $ar = str_split($atext, $chars);
        }
        return $ar;
    }

    function credit_trans_add_info($fibtrec) {
        $DBLT = getTool('DBLT');
        // Payee identification construction
        $fibtrec['BENEFICIARY_IDENTIFICATION'] = $this->beneficiary_id_build($fibtrec);
        // Check description default value
        $fibtrec['CREDIT_DESCRIPTION'] = (isset($fibtrec['CREDIT_DESCRIPTION']) ? $fibtrec['CREDIT_DESCRIPTION'] : '');
        //   - BDA: if it's a credit disbursement transaction, get beneficiary name from the credit record
        $fibtrec['IS_CREDIT_TX'] = ($fibtrec['TRANSTBL_CODE'] === 'SYS_CRED-PYMT' || $fibtrec['TRANSTBL_CODE'] === 'SYS_CRED-DISB' || $fibtrec['TRANSTBL_CODE'] === 'SYS_CRED-ADJ') ? true : false;
        if ($fibtrec['IS_CREDIT_TX'] === true) {
            // Its a credit transaction.... 	    
            // get payee information from credit record
            $sql = '
				SELECT
					/*
					FILO.DEBTOR_CODE FILO_DEBTOR_CODE,
					FIAS.CONTRACT_CODE FIAS_CONTRACT_CODE,
					*/
					FILO.IDCODE FILO_IDCODE,
					FICO.IDNAME FICO_IDNAME,
					ACCO.IDCODE IDCODE_ACCO,
					ACCO.IDENTIFICATION,
					FILO.PROP_ITEM_TYPE,
					FILO.DECREE
				FROM
					' . $this->filoname . ' FILO,
					' . $this->fiasname . ' FIAS,
					' . $this->acconame . ' ACCO,
					' . $this->ficoname . ' FICO
				WHERE
					FIAS.IDCODE = ' . singleQuote($fibtrec['CHART_FIELD03']) . '
					AND FILO.IDCODE = FIAS.LOAN_CODE
					AND FILO.DEBTOR_CODE = ACCO.IDCODE
					AND FICO.IDCODE = FIAS.CONTRACT_CODE
				';
            $ar_credit_beneficiary = ClxCommon::$db->getOneRecord($sql);
            if (empty($ar_credit_beneficiary)) {
                $fibtrec['error'] = true;
                $a_fias = ClxCommon::$db->getRecord($this->fiasname, 'IDCODE = ' . singleQuote($fibtrec['CHART_FIELD03']));
                $s_loan_code = $a_fias['LOAN_CODE'];
                $s_contract_code = $a_fias['CONTRACT_CODE'];
                if (empty($a_fias)) {
                    /*
                     * CODE_ERR_IBS_CHKPRT_FIAS_01
                     * Contract Item code not found: ^1
                     * Codigo de subcuenta no encontrada: ^1
                     */
                    $fibtrec['errormsg'][] = dict('CODE_ERR_IBS_CHKPRT_FIAS_01', $fibtrec['CHART_FIELD03']);
                } else {
                    $s_debtor_code = ClxCommon::$db->getField($this->filoname, 'DEBTOR_CODE', 'IDCODE = ' . singleQuote($s_loan_code));
                    if ($s_debtor_code === '') {
                        /*
                         * CODE_ERR_IBS_CHKPRT_FILO_01
                         * Credit code not found: ^1
                         * Codigo de crodito no encontrado: ^1
                         */
                        $fibtrec['errormsg'][] = dict('CODE_ERR_IBS_CHKPRT_FILO_01', $s_loan_code);
                    } else {
                        $s_acco_idcode = ClxCommon::$db->getField($this->acconame, 'IDCODE', 'IDCODE = ' . singleQuote($s_debtor_code));
                        if ($s_acco_idcode === '') {
                            /*
                             * CODE_ERR_IBS_CHKPRT_ACCO_01
                             * CRM Account code not found: ^1
                             * Codigo de cuenta CRM no encontrado: ^1
                             */
                            $fibtrec['errormsg'][] = dict('CODE_ERR_IBS_CHKPRT_ACCO_01', $s_debtor_code);
                        } else {
                            //   - Caso 1002467 - BC-GAP-Interfaz con IBS
                            // Estaba apuntando a la tabla ACCO en vez de la tabla FICO...
                            $s_fico_idcode = ClxCommon::$db->getField($this->ficoname, 'IDCODE', 'IDCODE = ' . singleQuote($s_contract_code));
                            if ($s_fico_idcode === '') {
                                /*
                                 * CODE_ERR_IBS_CHKPRT_FICO_01
                                 * Contract code not found: ^1
                                 * Codigo de contrato no encontrado: ^1
                                 */
                                $fibtrec['errormsg'][] = dict('CODE_ERR_IBS_CHKPRT_FICO_01', $s_contract_code);
                            }
                        }
                    }
                }
            } else {
                $fibtrec['IDCODE_ACCO'] = $ar_credit_beneficiary['IDCODE_ACCO'];
                $fibtrec['FIBT_IDENTIFICATION'] = $ar_credit_beneficiary['IDENTIFICATION'];
                $fibtrec['BENEFICIARY_IDENTIFICATION'] = $ar_credit_beneficiary['IDENTIFICATION'];
                // build check description
                //Example: 'LEY 24 OP. 42-157-1351 RUBRO GUANDU RES. 008-2012', where:
                // LEY 24 = Informacion Adicional-Contrato o Codigo de Producto/Servicio >> FIAS.CONTRACT_CODE / FILO.PRODUCT_CODE
                // OP. 42-157-1351 (numero del prestamo) = FILO.IDCODE
                // RUBRO GUANDU (rubro en el credito) = Informacion Estadostica-Rubro (en este caso seria Arroz Secano [101]) >> FILO.PROP_ITEM_TYPE, Lista CPIT
                // RES. 008-2012 = Informacion Adicional-Decreto / Resolucion >> FILO.DECREE
                /*
                 * CODE_LBL_IBS_CHECK_DESC
                 * ^1 OP. ^2 ITEM ^3 RES. ^4
                 * ^1 OP. ^2 RUBRO ^3 RES. ^4
                 */
                $fibtrec['CREDIT_DESCRIPTION'] = dict('CODE_LBL_IBS_CHECK_DESC', $ar_credit_beneficiary['FICO_IDNAME'], $ar_credit_beneficiary['FILO_IDCODE'], $DBLT->getListItem('CPIT', $ar_credit_beneficiary['PROP_ITEM_TYPE'], CLX_NUM_TWO), $ar_credit_beneficiary['DECREE']);
            }
        }
        // fix payee identification shrinked to check record id field on IBS
        $fibtrec['BENEFICIARY_IDENTIFICATION'] = $this->beneficiary_id_fix_4_ibs($fibtrec['BENEFICIARY_IDENTIFICATION']);
        

        return $fibtrec;
    }
    
    /*
     * Replaces in string all entries names in the array with {EntryName} token
     * @param text, array
     * @return text
     */
    function replace_all_fields_in_string($s_str, $record) {
        //   - Caso 1002467 - BC-GAP-Interfaz con IBS
        foreach ($record as $key => $val) {
            $s_str = str_replace('{' . $key . '}', $val, $s_str);
        }
        return $s_str;
    }

    /*
     * Returns a payee identification shrinked to check record id field on IBS.      * If $s_identification is greater than check record id field, some character deletion will be performed
     * @param text
     * @return text
     */
    function beneficiary_id_fix_4_ibs($s_identification) {
        //   BDA : THIS CASE WAS MADE ACCORDING TO PANAMA ID STANDARD FORMAT IN IBS RECORDS...
        if (strlen($s_identification) > $this->ibs_check_record_id_field_size) {
            $s_identification = str_replace('-0', '-', $s_identification);
        }
        if (strlen($s_identification) > $this->ibs_check_record_id_field_size) {
            $s_identification = str_replace('--', '-', $s_identification);
        }
        if (strlen($s_identification) > $this->ibs_check_record_id_field_size) {
            $s_identification = str_replace('00', '0', $s_identification);
        }
        return $s_identification;
    }

    /*
     * Returns a payee name depending of the fields:
     * FIBT.BENEFICIARY_NAME, ACCO.IDNAME and ACCO.LEGAL_NAME
     * @param array
     * @return text
     */
    function beneficiary_name_build(&$fibtrec) {
        $DBLT = getTool('DBLT');
        $s_response = 'o?';
        if ($fibtrec['BENEFICIARY_NAME'] !== '') {
            $s_response = $fibtrec['BENEFICIARY_NAME'];
        } else {
            $acco_record = ClxCommon::$db->getRecord($this->acconame, 'IDCODE = ' . singleQuote($fibtrec['FIBT_BENEFICIARY_CODE']));
            if (!function_exists('accoIdname'))
                includeCodeLib('ACCO-REC');
            $s_response = acco_full_name($acco_record);
            $fibtrec['FIBT_LEGAL_TYPE'] = $acco_record['LEGAL_TYPE'];
            $fibtrec['FIBT_IDENTIFICATION'] = $acco_record['IDENTIFICATION'];
            $fibtrec['BENEFICIARY_IDENTIFICATION'] = $acco_record['IDENTIFICATION'];
            $fibtrec['BENEFICIARY_EMAIL'] = $acco_record['EMAIL'];
            $fibtrec['BENEFICIARY_ADDRESS'] = trim($acco_record['ADDRESS1'] . ' ' . $acco_record['ADDRESS2']);
            $fibtrec['BENEFICIARY_COUNTRY'] = $acco_record['COUNTRY'];
            $fibtrec['BENEFICIARY_CITY'] = $acco_record['CITY'];
            $fibtrec['BENEFICIARY_STATE'] = $acco_record['STATE'];
            $fibtrec['BENEFICIARY_ZIPCODE'] = $acco_record['ZIPCODE'];
        }
        // CITY,STATE,ZIP
        $fibtrec['BENEFICIARY_CSZ'] = ($fibtrec['BENEFICIARY_CITY'] !== '' ? ( $DBLT->getListItem('CITI', $fibtrec['BENEFICIARY_CITY'], CLX_NUM_TWO) . ', ') : '') . ($fibtrec['BENEFICIARY_STATE'] !== '' ? ( $DBLT->getListItem('STAT', $fibtrec['BENEFICIARY_STATE'], CLX_NUM_TWO) . ', ') : '') . $fibtrec['BENEFICIARY_ZIPCODE'];
        return $s_response;
    }

    /*
     * Returns a payee ID depending of the fields:
     * FIBT.BENEFICIARY_NAME, FIBT.IDENTIFICATION and ACCO.IDENTIFICATION
     * @param array
     * @return text
     */
    function beneficiary_id_build($fibtrec) {
        if ($fibtrec['BENEFICIARY_NAME'] !== '') {
            $s_response = $fibtrec['FIBT_IDENTIFICATION'];
        } else {
            $s_response = $fibtrec['ACCO_IDENTIFICATION'];
        }
        return $s_response;
    }

    /*
     * Applicant data retrieval
     * @param array, text
     * @return array
     */
    function applicant_data_get($record, $s_contract_code) {
        $DBLT = getTool('DBLT');
        if (!is_array($record))
            $record = array();
        $record['CONTRACT_CODE'] = $s_contract_code;
	$sql = 'SELECT
                ACCO.IDCODE ACCO_IDCODE,
                ACCO.IDNAME APPLICANT_NAME,
                ACCO.IDENTIFICATION APPLICANT_IDENTIFICATION,
                ACCO.IBS_CUSCUN APPLICANT_IBS_CUSCUN,
                ACLO.ADDRESS1 APPLICANT_ADDRESS1,
                ACLO.ADDRESS2 APPLICANT_ADDRESS2,
                ACLO.CITY APPLICANT_CITY,
                ACLO.STATE APPLICANT_STATE,
                ACLO.ZIPCODE APPLICANT_ZIPCODE,
                ACLO.COUNTRY APPLICANT_COUNTRY 
            FROM
                ' . $this->fipaname . ' FIPA
                LEFT JOIN ' . $this->acconame . ' ACCO ON ACCO.IDCODE = FIPA.ACCOUNTCODE ';
        if ($this->sw_old_acco_struct === true) {
            // Old version
	    $sql.= 'LEFT JOIN ' . $this->acloname . ' ACLO ON ACLO.ACCOUNTCODE = FIPA.ACCOUNTCODE AND ACLO.ISPRIMARY = 1 WHERE FIPA.CONTRACT_CODE = ' . singleQuote($s_contract_code) . ' AND FIPA.PARTICIPANT_TYPE IN (' . singleQuoteArray(array('F', 'B', 'G', 'M', 'MB')) . ')';
        } else {
            // New acco version
            $sql.= 'WHERE FIPA.CONTRACT_CODE = ' . singleQuote($s_contract_code) . ' AND FIPA.PARTICIPANT_TYPE IN (' . singleQuoteArray(array('F', 'B', 'G', 'M', 'MB')) . ')';
            $sql = str_replace('ACLO.', 'ACCO.', $sql);
        }
	$sql.= 'ORDER BY FIPA.ISPRIMARY DESC';

        if (!empty($this->debug)) {
            echoLog('<br>' . CLX_CHAR_NL . 'applicant_data_get - SQL:<br>' . CLX_CHAR_NL . pre($sql) . '<br>' . CLX_CHAR_NL);
        }
        $ar_applicant = ClxCommon::$db->getOneRecord($sql);
        if (!empty($this->debug))
            echoLog(debug_dump('ar_applicant', $ar_applicant));
        if ($ar_applicant['APPLICANT_IBS_CUSCUN'] !== '') {
            $record['APPLICANT_IBS_CUSCUN'] = $ar_applicant['APPLICANT_IBS_CUSCUN'];
            $record['APPLICANT_NAME'] = '';
            $record['APPLICANT_IDENTIFICATION'] = '';
            $record['APPLICANT_ADDRESS1'] = '';
            $record['APPLICANT_ADDRESS2'] = '';
            $record['APPLICANT_ZIPCODE'] = '';
            $record['APPLICANT_CITY'] = '';
            $record['APPLICANT_STATE'] = '';
            $record['APPLICANT_COUNTRY'] = '';
            $record['APPLICANT_ADDRESS3'] = '';
        } else {
            $record['APPLICANT_NAME'] = $ar_applicant['APPLICANT_NAME'];
            $record['APPLICANT_IDENTIFICATION'] = $ar_applicant['APPLICANT_IDENTIFICATION'];
            $record['APPLICANT_IBS_CUSCUN'] = $ar_applicant['APPLICANT_IBS_CUSCUN'];
            $record['APPLICANT_ADDRESS1'] = $ar_applicant['APPLICANT_ADDRESS1'];
            $record['APPLICANT_ADDRESS2'] = $ar_applicant['APPLICANT_ADDRESS2'];
            $record['APPLICANT_ZIPCODE'] = $ar_applicant['APPLICANT_ZIPCODE'];
            $record['APPLICANT_CITY'] = ($ar_applicant['APPLICANT_CITY'] === '') ? '' : $DBLT->getListItem('LOCI', $ar_applicant['APPLICANT_CITY'], CLX_NUM_TWO);
            $record['APPLICANT_STATE'] = ($ar_applicant['APPLICANT_STATE'] === '') ? '' : $DBLT->getListItem('VZST', $ar_applicant['APPLICANT_STATE'], CLX_NUM_TWO);
            $record['APPLICANT_COUNTRY'] = ($ar_applicant['APPLICANT_COUNTRY'] === '') ? '' : $DBLT->getListItem('COUN', $ar_applicant['APPLICANT_COUNTRY'], CLX_NUM_TWO);
            $record['APPLICANT_ADDRESS3'] = $record['APPLICANT_CITY'] . ' ' . trim($record['APPLICANT_STATE'] . ' ' . $record['APPLICANT_ZIPCODE']);
        }
        return $record;
    }

    /**
     * Sets status of transfers.      * @param string $trans_code
     * @param array $record
     */
    protected function setprintChecksResult($trans_code, $record) {
        if (empty($record))
            return;
        ClxCommon::$db->updateRecord($this->fiatname, $record, 'IDCODE = ' . singleQuote($trans_code) . ' AND RECORD_BLOCKING <> ' . "'1'" . '');
    }

}

// Class End
/*
 * ******************************************************************************************************************
 * e-IBS Check Interface
 * *****************************************************************************************************************
 */
class ibs_check_socket_interface extends ibs_check_interface_commons {
    /*
     * Constructor
     * @param text, array
     * @return void
     */

    function __construct($s_comp_code = null, $parameters = array()) {
        #parent::ibs_check_interface_commons($s_comp_code, $parameters);
        parent::__construct($s_comp_code, $parameters);

        $EBSK = getTool('EBSK');
        if (!defined('EBSK_IBS_DEF_CHKPRT_MSG')) {
            // Check emission socket id
            define('EBSK_IBS_DEF_CHKPRT_MSG', 'EOF011501');
            // Parameters for printed check reversal
            define('EBSK_IBS_DEF_CHKPRT_REV_MSG', 'EOF012001');
        }
        if (!empty($this->debug)) {
            echoLog('EBSK_IBS_DEF_CHKPRT_MSG=' . EBSK_IBS_DEF_CHKPRT_MSG);
            echoLog('EBSK_IBS_DEF_CHKPRT_REV_MSG=' . EBSK_IBS_DEF_CHKPRT_REV_MSG);
        }
    }

    /**
     *
     *
     * @param type $contents
     * @return boolean
     */
    private function setRecordBlocking(&$contents, $idlist) {
        $flag_return = false;
        /*
         * Logica:
         * 1. si no hay seleccion returnar error,
         * 2. si hay pendiente o error entonces colocar procesando para permitir continuar para procesar enviando mensaje
         * 3. si esta en procesando alguno entonces mensionar que esta en proceso y retornar
         * 4. si esta completado presentar valor de resultado
         * 5. si no hay nada mas returnar.
         */
        //Mark status to process
        if (empty($idlist))
            $idlist = CLX_INIT_ZERO;
        $print_date = xFormat::getNow('Y-m-d H:i:s');
        //Set entries to error that has been running for more than 2 hours on status 15 hours
        $sql = 'UPDATE ' . $this->fiatname . '  SET RECORD_BLOCKING = ' . singleQuote('4') . ', CHECK_PRINT_DATI = null, INTERFACE_MSG = ' . singleQuote('Timeout') . ' WHERE RECORD_BLOCKING  = ' . singleQuote('3') . ' AND ADDDATE(CHECK_PRINT_DATI, INTERVAL 1.5 HOUR) <= NOW()';
        ClxCommon::$db->executeQuery($sql);
        ClxCommon::$db->commit();
        $record_blocking = ClxCommon::$db->getOne('SELECT MAX(RECORD_BLOCKING) FROM ' . $this->fiatname . ' WHERE ID IN (' . $idlist . ')');
        $record_blocking = intval($record_blocking);
        switch ($record_blocking) {
            case CLX_NUM_ZERO:
                //Set elements from idlist
                $sql = 'UPDATE ' . $this->fiatname . ' SET RECORD_BLOCKING = ' . singleQuote('3') . ', CHECK_PRINT_DATI=' . singleQuote($print_date) . ', INTERFACE_MSG = null  WHERE RECORD_BLOCKING  = ' . singleQuote('0') . ' AND ID IN (' . $idlist . ')';
                ClxCommon::$db->executeQuery($sql);
                ClxCommon::$db->commit();
                //Set long process note
                $msg = dict('GEN_LBL_REFRESH_WAIT');
                $msg .= '<br><br><a href="javascript:location.reload();")>' . dict('GEN_LBL_CLICKTOREFRESHSTATUS') . '</a>';
                $msg = note($msg) . '<br>';
                //   - Caso 1002467 - BC-GAP-Interfaz con IBS
                // Cambio en BNP/COR para poder depurar bien la interfaz en caso que sea necesario vert la trama como antes del cambio del batch chk/trsf interface
                if ($this->debug != true) {
                    sendLongProcessWait($msg, (CLX_NUM_60 * CLX_NUM_60 * CLX_NUM_60), 'pop.html', false);
                } else {
                    echo $msg;
                }
                $flag_return = true;
                break;
            case CLX_NUM_ONE:
                //Set results from selection
                $contents = $this->printChecksResult();
                break;
            case CLX_NUM_TWO:
                //waiting for ibs asynch response
                $contents = dict('GEN_LBL_WAITING_FOR_CORE') . '<br>';
                $contents .= $this->printChecksResult();
                $contents = note($contents) . '<br>';
                break;
            case CLX_NUM_THREE:
                //in progress
                $contents = dict('CODE_LBL_CHECKSEMISSION_TITLE') . '<br>' . dict('GEN_LBL_REFRESH_WAIT');
                $contents .= '<br><br><a href="javascript:location.reload();")>' . dict('GEN_LBL_CLICKTOREFRESHSTATUS') . '</a>';
                $contents = note($contents) . '<br>';
                break;
            case CLX_NUM_FOUR:
                $contents = dict('GEN_LBL_STATUS') . ': ' . dict('GEN_LBL_FAIL') . '<br>' . CLX_CHAR_NL;
                $contents .= dict('GEN_LBL_INTERFACE_ERR_01') . '<br><br>';
                $contents .= '<br><a href="javascript:location.reload();")>' . dict('GEN_LBL_PRESSTORESEND') . '</a><br>';
                //Set results from selection
                $contents .= $this->printChecksResult();
                //Setting elements in error to pending
                ClxCommon::$db->executeQuery('UPDATE FIDU_TRANSACTIONS SET RECORD_BLOCKING = ' . singleQuote('0') . ', INTERFACE_MSG = null WHERE ID IN (' . $idlist . ') AND RECORD_BLOCKING =' . singleQuote('4') . '');
                ClxCommon::$db->commit(); //commit for other processes to see change
                $contents = note($contents);
                break;
            default:
                $contents = note(dict('TOOL_UILG_ERR_NOMATCH'));
                break;
        }
        return $flag_return;
    }

    function printChecksResult() {
        $idlist = toString(getIdlist());
        $contents = $this->printChecksResultTemplate();
        $checks = '';
        $other_ti = '';
        $other = '';
        $sql = 'SELECT INTERFACE_MSG, DISBURSEMENT_TYPE, IDCODE, TRANS_STATUS FROM ' . $this->fiatname . '  WHERE ID IN (' . $idlist . ') ORDER BY DISBURSEMENT_TYPE, CHECK_PRINT_DATI';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            //   - Caso 1002541 - BNP-Incid 189965 Rev Ope Fso
            if ($r['INTERFACE_MSG'] === '' || is_null($r['INTERFACE_MSG'])) {
                // Transaccion en lista de registros marcados pero no tiene mensaje de
                // respuesta de procesamiento.
                if ($r['TRANS_STATUS'] !== 'AP' && $r['TRANS_STATUS'] !== 'CLOS') {
                    // No tiene status apropiado. Esto significa que es rechazada por no estar aprobada... 		    
                    $r['INTERFACE_MSG'] = dict('CODE_GEN_TRANSACTION') . ' # ' . $r['IDCODE'] . ': ' . dict('GEN_LBL_INACTIVE') . ' [' . $r['TRANS_STATUS'] . ']';
                }
            }
            if ($r['DISBURSEMENT_TYPE'] === 'CH') {
                $checks .='<li>' . dict('CODE_LBL_CHECKSEMISSION_TITLE') . '<pre>' . wordwrap($r['INTERFACE_MSG']) . '</pre></li>';
            } elseif ($r['DISBURSEMENT_TYPE'] === 'TI') {
                //   pone el titulo especifico de transferencias internas
                $other_ti .='<li>' . dict('CODE_LBL_ITRANSFEREMISSION_TITLE') . '<pre>' . wordwrap($r['INTERFACE_MSG']) . '</pre></li><br>';
            } else {
                //   debe poner el titulo de las transferencias para reportar los datos de la misma
                $other .='<li>' . dict('CODE_LBL_ETREQ_TITLE') . '<pre>' . wordwrap($r['INTERFACE_MSG']) . '</pre></li><br>';
            }
        }
		ClxCommon::$db->freeResult($rs);
        //tomado de metodo de cheque
        if ($checks !== '') {
            $checks = '<b><h2><center>' . dict('CODE_LBL_CHECKSEMISSION_TITLE') . '</center></h2></b><ul>' . $checks . '</ul>';
        } else {
            //   no estaba cerrando correctamente el </h2>
            $checks = '<b><h2><center>' . dict('CODE_LBL_CHECKSEMISSION_TITLE') . '</center></h2></b><ul><li>' . dict('CODE_ERR_IBS_CHKPRT_NOTHINGTODO') . '</li></ul>';
        }
        //   - Caso 1002541 - BNP-Incid 189965 Rev Ope Fso
        // Separa el reporte de Transferencias Internas del que corresponde a las transferencias en general
        if (($other . $other_ti) !== '') {
            $other = '<b><h2><center>' . dict('CODE_LBL_ETREQ_TITLE') . '</center></h2></b><ul>' . $other . ($other_ti <> '' && $other <> '' ? '<br>' : '') . $other_ti . '</ul>';
        } else {
            $other = '<b><h2><center>' . dict('CODE_LBL_ETREQ_TITLE') . '</center></h2></b><ul><li>' . dict('CODE_ERR_IBS_ETREQ_NOTHINGTODO') . '<br>' . dict('CODE_ERR_IBS_ITRANS_NOTHINGTODO') . '</li></ul>';
        }
        $contents = str_replace('<MAIN_CONTENT_CHECKS>', $checks, $contents);
        $contents = str_replace('<MAIN_CONTENT_TRANSFER>', $other, $contents);
        return $contents;
    }

    /**
     * Template to print result
     * @return string
     */
    function printChecksResultTemplate() {
        /*
         * CODE_LBL_IBS_CHKPRT_TRUST_DEPT_TITLE
         * TRUST DEPARTMENT
         * DEPARTAMENTO DE FIDEICOMISOS
         */
        //Header
        $contents = '<center><table border="0" width=CLX_PROP_100PCT><tr><td width="30%">' . $this->comp_logo($this->s_comp_code) . '</td><td width="10%">&nbsp;</td><td width="60%" align="left" valign="middle"><table border="1" width="60%" cellspacing="1" cellpadding="5"><tr><td align="center">' . dict('CODE_LBL_IBS_CHKPRT_TRUST_DEPT_TITLE') . '</td></tr></table></td></tr></table>' . '</center><br>' . CLX_CHAR_NL;
        /*
         * CODE_LBL_CHECKS_GEN_TITLE
         * Checks/Transfers Request Report
         * Reporte de Solicitud de Emision de Cheques/Transferencias
         */
        //Main checks
        $contents .= '<MAIN_CONTENT_CHECKS><br>';
        //Main transfers
        $contents .= '<MAIN_CONTENT_TRANSFER><br>';
        //Footer
        /*
         * CODE_LBL_IBS_CHKPRT_TRUST_DEPT_SIGN_1
         * CREATED BY:
         * CREADOR POR:
         *
         * CODE_LBL_IBS_CHKPRT_TRUST_DEPT_SIGN_2
         * AUTHORIZED BY:
         * AUTORIZADO POR:
         */
        $contents .= '<center><table border="0" width=CLX_PROP_100PCT><tr><td width="20%">&nbsp;</td><td  width="15%"><table border="1" width=CLX_PROP_100PCT cellspacing="1" cellpadding="5"><tr><td align="center">' . dict('CODE_LBL_IBS_CHKPRT_TRUST_DEPT_SIGN_1') . '<BR><BR><BR></td></tr></table></td><td width="10%">&nbsp;</td><td  width="15%"><table border="1" width=CLX_PROP_100PCT cellspacing="1" cellpadding="5"><tr><td align="center">' . dict('CODE_LBL_IBS_CHKPRT_TRUST_DEPT_SIGN_2') . '<BR><BR><BR></td></tr></table></td><td width="20%">&nbsp;</td></tr></table>' . '</center><br>' . CLX_CHAR_NL;
        $contents = '<table border="1" width="98%" cellspacing=1 cellpadding=5><tr><td>' . $contents . '</td><tr></table>';
        return $contents;
    }

    /**
     * Sends the Checks / Wire Transfers (ACH) / Internal Transfers requests commands to IBS for a list of transactions
     * @param text
     * @return text
     */
    function printChecks($s_trans_idcode = null) {
        if (!empty($this->debug))
            echoLog('printChecks');
        //Mark status to process
        $idlist = toString(getIdlist());
        $contents = '';
        //setRecord block and check status if returns false message set to content
        if (!$this->setRecordBlocking($contents, $idlist)) {
            return $contents;
        }
        // Process Check print request  <MAIN_CONTENT_CHECKS>
        ClxFreeReturn($this->printChecks_run_ch($s_trans_idcode));
        // Process Wire Transfers (ACH) request <MAIN_CONTENT_CHECKS>
        ClxFreeReturn($this->printChecks_run_et($s_trans_idcode));
        // Process Internal Transfers request <MAIN_CONTENT_TRANSF>
        if (!class_exists('ibs_transfers_batch_interface'))
            includeCodeLib('EIBS_TRSF');
        $o_ibs_transfers = new ibs_transfers_batch_interface($this->s_comp_code, $this->proc_eng_params->process_parameters);
        $o_ibs_transfers->ibs_transfer_export($s_trans_idcode);
        //Mark error on cases that may be pending
        $sql = 'UPDATE ' . $this->fiatname . '  SET RECORD_BLOCKING = ' . singleQuote('4') . ', CHECK_PRINT_DATI = null WHERE RECORD_BLOCKING = ' . singleQuote('3') . ' AND ID IN (' . $idlist . ')';
        ClxCommon::$db->executeQuery($sql);
        ClxCommon::$db->commit();
        return CLX_CHAR_EMPTY_STRING;
    }

    /**
     * Sends the checks print command to IBS for a list of transactions
     * @param text
     * @return text
     */
    function printChecks_run_ch($s_trans_idcode) {
        $DBLT = &getTool('DBLT');
        $EBSK = getTool('EBSK');
        $EBSK->debug = $this->debug;
        $EBSK->tfields_only = true;
        $EBSK->remove_decimal_point = false;
        $msg_code = EBSK_IBS_DEF_CHKPRT_MSG;
        $mappings = $EBSK->getMappings($msg_code);
        // Checks if there's something to process
        $s_idcode_list = $this->fiat_idcode_list($s_trans_idcode);
        if ($s_idcode_list === '') {
            /*
             * CODE_ERR_IBS_CHKPRT_NOTRANS
             * There's not selected/approved transaction for checks emission
             * No hay transacciones seleccionadas/aprobadas para la emisión de cheques
             */
            return CLX_CHAR_EMPTY_STRING;
        }
        // Builds SQL to get transactions involved. FIAT.DISBURSEMENT_TYPE = CH (Checks), ET or ACH = (Electronic Transfer)
        $sql = $this->printChecks_sql_statement($s_idcode_list, 'CH');
        $rs = ClxCommon::$db->executeQuery($sql);
        while (($fibtrec = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE))) {
            $contents = ''; //mensaje a guardar
            $fibtrec['DESCRIPTION'] = str_replace(CLX_CHAR_NL, ' ', $fibtrec['DESCRIPTION']);
            $fibtrec['DESCRIPTION'] = str_replace(CLX_CHAR_NL, ' ', $fibtrec['DESCRIPTION']);
            // Prepares the data to be transmitted
            $record = $this->printChecks_prepares_record($fibtrec, $mappings);
            if (!empty($this->debug)) {
                $contents .= debug_dump('record', $record);
            }
            // CODE_GEN_TRANSACTION, Transaction, Transaccion
            $contents .= dict('CODE_GEN_TRANSACTION') . ' # ' . $record['TRANS_CODE'] . ' ' . $record['error_status_adic'] . CLX_CHAR_NL;
            $contents .= '' . /* Transaction Amount */
                    dict('GEN_LBL_AMOUNT') . ': ' . xFormat::number($record['TRANS_AMT'], CLX_NUM_TWO) . CLX_CHAR_NL . /* Currency */
                    dict('GEN_LBL_CURRENCY') . ': ' . $DBLT->getListItem('CURR', $record['CURRENCY_CODE']) . CLX_CHAR_NL . /* Source */
                    '[' . dict('GEN_LBL_SOURCE') . ']' . CLX_CHAR_NL . /* Customer */
                    dict('GEN_LBL_CLIENT') . ': ' . $record['LEGAL_NAME_ACCO'] . ' [' . $record['APPLICANT_IBS_CUSCUN'] . ']' . CLX_CHAR_NL . /* Source Account */
                    dict('GEN_LBL_AC_NUMBER') . ': ' . $record['FIDU_BANKACT_CODE'] . CLX_CHAR_NL . /* Target */
                    '[' . dict('GEN_LBL_TARGET') . ']' . CLX_CHAR_NL . /* Beneficiary */
                    dict('GEN_LBL_BENEFICIARY') . ': ' . $record['BENEFICIARY_NAME'] . CLX_CHAR_NL;
            if ($record['error'] === true) {
                $contents .= dict('GEN_LBL_ERROR') . ': ' . implode(CLX_CHAR_NL, $record['errormsg']);
                if (!empty($this->debug))
                    pre('**1** ' . CLX_CHAR_NL . $contents);
                //save error message and change status
                $contents = strip_tags(str_replace('<br>', CLX_CHAR_NL, $contents));
                $this->setprintChecksResult($record['TRANS_CODE'], array('RECORD_BLOCKING' => CLX_NUM_FOUR, 'INTERFACE_MSG' => $contents));
                continue;
            }
            // Send the check information and receives the ibs response
            $ar_response = $this->printChecks_send($record, $msg_code);
            $s_check_number = NULL;
            if(isset($ar_response['ar_last_sent_skt_response'][CLX_NUM_ZERO]['CHECK_NUMBER'])) {
                $s_check_number = $ar_response['ar_last_sent_skt_response'][CLX_NUM_ZERO]['CHECK_NUMBER'];
            }
            if (!is_null($s_check_number) && $s_check_number != CLX_NUM_ZERO) {
                //CHECK_NUMBER Check # Cheque #
                $contents .= dict('CHECK_NUMBER') . ': ' . $s_check_number . CLX_CHAR_NL;
            }
            if (!empty($ar_response['contents'])) {
                $contents .= dict('GEN_LBL_STATUS') . ': ' . $ar_response['contents'] . CLX_CHAR_NL;
            }
            $contents .= CLX_CHAR_NL;
            $contents = strip_tags(str_replace('<br>', CLX_CHAR_NL, $contents));
            if ($ar_response['error'] === true) {
                // Check with errors... Store check number if itos diffenrent than 0 
                if ($record['CHECK_NUMBER'] == CLX_NUM_ZERO && !empty($s_check_number)) {
                    $this->setprintChecksResult($record['TRANS_CODE'], array('CHECK_NO' => $s_check_number, 'RECORD_BLOCKING' => '4', 'INTERFACE_MSG' => $contents));
                } else {
                    $this->setprintChecksResult($record['TRANS_CODE'], array('RECORD_BLOCKING' => '4', 'INTERFACE_MSG' => $contents));
                }
            } else {
                // Check was printed successfully... 		
                $this->setprintChecksResult($record['TRANS_CODE'], array('RECORD_BLOCKING' => '1', 'CHECK_NO' => $s_check_number, 'CHECK_PRINT_DATI' => xFormat::getNow('Y-m-d H:i:s'), 'INTERFACE_MSG' => $contents));
            }
        }//end while
		ClxCommon::$db->freeResult($rs);
        return CLX_CHAR_EMPTY_STRING;
    }

    /*
     * Sends the checks print command to IBS for a list of transactions
     * @param text
     * @return text
     */
    function printChecks_run_et($s_trans_idcode) {
        $DBLT = &getTool('DBLT');
        $EBSK = getTool('EBSK');
        $EBSK->debug = $this->debug;
        $EBSK->tfields_only = true;
        $EBSK->remove_decimal_point = false;
        $msg_code = EBSK_IBS_DEF_ET_MSG;
        $mappings = $EBSK->getMappings($msg_code);
        if (!defined('EBSK_IBS_DEF_ET_TOKEN')) {
            // Token that defines Local Electronic Transfers type in FIAT.DISBURSMENT_TYPE column. Example: ET or ACH
            // Depends on clients particular installation. Default value: ET
            define('EBSK_IBS_DEF_ET_TOKEN', 'ET');
        }
        // Checks if there's something to process
        $s_idcode_list = $this->fiat_idcode_list($s_trans_idcode);
        if ($s_idcode_list === '') {
            /*
             * CODE_ERR_IBS_CHKPRT_NOTRANS
             * There's not selected/approved transaction for checks emission
             * No hay transacciones seleccionadas/aprobadas para la emisión de cheques
             */
            return CLX_CHAR_EMPTY_STRING;
        }
        // Builds SQL to get transactions involved. FIAT.DISBURSEMENT_TYPE = CH (Checks), ET or ACH = (Electronic Transfer)
        $sql = $this->printChecks_sql_statement($s_idcode_list, EBSK_IBS_DEF_ET_TOKEN);
        $rs = ClxCommon::$db->executeQuery($sql);
        while (($fibtrec = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE))) {
            $contents = '';
            $fibtrec['DESCRIPTION'] = str_replace(CLX_CHAR_NL, ' ', $fibtrec['DESCRIPTION']);
            $fibtrec['DESCRIPTION'] = str_replace(CLX_CHAR_NL, ' ', $fibtrec['DESCRIPTION']);
            // Prepares the data to be transmitted
            $record = $this->printChecks_prepares_record($fibtrec, $mappings);
            if (!empty($this->debug)) {
                $contents .= debug_dump('record', $record);
            }
            $contents .= dict('CODE_GEN_TRANSACTION') . ' # ' . $record['TRANS_CODE'] . ' ' . $record['error_status_adic'] . CLX_CHAR_NL;
            $contents .=
                    /* Transaction Amount */
                    dict('GEN_LBL_AMOUNT') . ': ' . xFormat::number($record['TRANS_AMT'], CLX_NUM_TWO) . /* Currency */
                    CLX_CHAR_NL . dict('GEN_LBL_CURRENCY') . ': ' . $DBLT->getListItem('CURR', $record['CURRENCY_CODE']) . /* Source */
                    CLX_CHAR_NL . '[' . dict('GEN_LBL_SOURCE') . '] ' . /* Customer */
                    CLX_CHAR_NL . dict('GEN_LBL_CLIENT') . ': ' . $record['LEGAL_NAME_ACCO'] . ' [' . $record['APPLICANT_IBS_CUSCUN'] . ']' . /* Source Account */
                    CLX_CHAR_NL . dict('GEN_LBL_AC_NUMBER') . ': ' . $record['FIDU_BANKACT_CODE'] . /* Target */
                    CLX_CHAR_NL . '[' . dict('GEN_LBL_TARGET') . ']' . /* Beneficiary */
                    CLX_CHAR_NL . dict('GEN_LBL_BENEFICIARY') . ': ' . $record['BENEFICIARY_NAME'] . /* Target Bank (Rounting No) */
                    CLX_CHAR_NL . dict('GEN_LBL_BANK_ACCOUNT') . ': ' . $record['BANK_ROUTING_NO_ET_DEST'] . /* Target Account */
                    CLX_CHAR_NL . dict('GEN_LBL_AC_NUMBER') . ': ' . $record['BANK_ACCOUNT_NO_ET_DEST'];
            if ($record['error'] === true) {
                $contents .= CLX_CHAR_NL . dict('GEN_LBL_ERROR') . ': ' . implode(CLX_CHAR_NL, $record['errormsg']);
                if (!empty($this->debug))
                    pre('**2** ' . CLX_CHAR_NL . $contents);
                //save error message and change status
                $this->setprintChecksResult($record['TRANS_CODE'], array('RECORD_BLOCKING' => CLX_NUM_FOUR, 'INTERFACE_MSG' => $contents));
                continue;
            }
            // Send the E.T. information and receives the ibs response
            $ar_response = $this->printChecks_send($record, $msg_code);
            if (!empty($ar_response['contents'])) {
                $contents .= CLX_CHAR_NL . dict('GEN_LBL_STATUS') . ': ' . $ar_response['contents'] . CLX_CHAR_NL;
            }
            if ($ar_response['error'] == false) {
                $s_et_error_number = $ar_response['ar_last_sent_skt_response'][CLX_NUM_ZERO]['ET_ERROR_NUMBER'];
                $s_et_msg_text = $ar_response['ar_last_sent_skt_response'][CLX_NUM_ZERO]['ET_MSG_TEXT'];
                $s_et_variable_name = $ar_response['ar_last_sent_skt_response'][CLX_NUM_ZERO]['ET_VARIABLE_NAME'];
                if ($s_et_error_number !== '') {
                    $contents .= CLX_CHAR_NL . '-' . implode(CLX_CHAR_NL . '-', $s_et_error_number . ' [' . $s_et_variable_name . '] ' . $s_et_msg_text);
                    $contents = strip_tags(str_replace('<br>', CLX_CHAR_NL, $contents));
                    $this->setprintChecksResult($record['TRANS_CODE'], array('RECORD_BLOCKING' => '4', 'INTERFACE_MSG' => $contents));
                } else {
                    // E-Transfer successfully sent... Get response data
                    $s_check_number = $ar_response['ar_last_sent_skt_response'][CLX_NUM_ZERO]['CHECK_NUMBER'];
                    $s_et_id_number = $ar_response['ar_last_sent_skt_response'][CLX_NUM_ZERO]['ET_ID_NUMBER'];
                    $s_et_batch = $ar_response['ar_last_sent_skt_response'][CLX_NUM_ZERO]['ET_BATCH'];
                    $contents .= CLX_CHAR_NL . dict('GEN_LBL_REFERENCE_NO') . ': ' . $s_check_number;
                    if ($s_et_id_number !== '')
                        $contents .= ' [' . $s_et_id_number . ']';
                    if ($s_et_batch !== '')
                        $contents .= CLX_CHAR_NL . dict('GEN_LBL_LOT') . ': ' . $s_et_batch;
                    // Check was printed successfully... 		    $contents = strip_tags(str_replace('<br>', CLX_CHAR_NL, $contents));
                    $this->setprintChecksResult($record['TRANS_CODE'], array('RECORD_BLOCKING' => '1', 'CHECK_NO' => $s_check_number, 'CHECK_PRINT_DATI' => xFormat::getNow('Y-m-d H:i:s'), 'CHECK_PRINT_BY' => $_SESSION['user_record']['username'], 'INTERFACE_MSG' => $contents));
                }
            }else {
                $contents = strip_tags(str_replace('<br>', CLX_CHAR_NL, $contents));
                $this->setprintChecksResult($record['TRANS_CODE'], array('RECORD_BLOCKING' => '4', 'INTERFACE_MSG' => $contents));
            }
        }
		ClxCommon::$db->freeResult($rs);
        return CLX_CHAR_EMPTY_STRING;
    }

    /**
     * Description: returns HTML <IMG...> with company logo
     *
     * @return string
     */
    function comp_logo($comp_code) {
        $fieldinstance = getToolObject('DBFT', 'FILE');
        $fieldrec = ClxCommon::$db->getRecord('CLX_DB_FIELDS', 'FIELDNAME=' . singleQuote('COMP_LOGO'));
        $comprec = ClxCommon::$db->getRecord($this->compname, 'IDCODE = ' . singleQuote($comp_code));
        $return = $fieldinstance->formatField('COMP_LOGO', $fieldrec, $comprec);
        return $return;
    }

    /*
     * Sends the printed checks reversal command to IBS for a list of transactions
     * @param text
     * @return text
     */
    function reverseChecks($s_trans_idcode = null, $s_disbursement_type = 'CH') {
        if (!empty($this->debug)) {
            echoLog('reverseChecks');
        }
        $DBLT = getTool('DBLT');
        $msg_code = EBSK_IBS_DEF_CHKPRT_REV_MSG;
        $b_rev_chk_skt_inactive = ($msg_code === '0' || $msg_code === 'null' || $msg_code === ''); // verifies if check reverse  interface is inactive
        if (empty($b_rev_chk_skt_inactive)) {
            $EBSK = getTool('EBSK');
            $EBSK->debug = $this->debug;
            $EBSK->tfields_only = true;
            $EBSK->remove_decimal_point = false;
            $mappings = $EBSK->getMappings($msg_code);
        }
        //  BHN
        $s_sql_extra_pars_select_local = $this->printChecks_sql_stat_ver_flds($s_disbursement_type);
        /*
         * CODE_LBL_CHECKSREV_TITLE
         * Checks Reversal Request
         * Solicitud de Reverso de Cheques
         */
        $contents = dict('CODE_LBL_CHECKSREV_TITLE') . '<br>' . CLX_CHAR_NL . '<br>' . CLX_CHAR_NL;
        $s_idcode_list = $this->fiat_idcode_list($s_trans_idcode);
        // Nothing to transmit
        if ($s_idcode_list === '') {
            /*
             * CODE_ERR_IBS_CHKREV_NOTRANS
             * There's not selected/approved transaction for checks reversal
             * No hay transacciones seleccionadas/aprobadas para el reverso de cheques
             */
            $contents .= '- ' . dict('CODE_ERR_IBS_CHKREV_NOTRANS') . '<br>' . CLX_CHAR_NL;
            //echoLog($contents);
        } else {
            // Extracts the beneficiaries from the transactions involved (ONLY CHECKS)
            $sql = '		SELECT
						FIBT.RECORD_BLOCKING,
						FIBT.TRANS_CODE,
						FIBT.TRANS_AMT,
						FIBT.BENEFICIARY_NAME,
						FIBT.IDENTIFICATION FIBT_IDENTIFICATION,
						FIBT.LEGAL_TYPE FIBT_LEGAL_TYPE, ';

	    if ($this->sw_old_acco_struct === true) {
                // old estructure
                $sql .= '			FIBT.CHECK_NUMBER,
						ACCO.LEGAL_TYPE,
						ACCO.ISINDIVIDUAL,
						ACCO.IDENTIFICATION ACCO_IDENTIFICATION,
						ACCO.IDNAME IDNAME_ACCO,
						ACCO.IDCODE IDCODE_ACCO,
						ACCO.LEGAL_NAME LEGAL_NAME_ACCO,
						FIAT.CURRENCY_CODE,
						FIBT.DISBURSEMENT_TYPE,
						FIBK_FIAS.CURRENCY_CODE CURRENCY_CODE_FIBK_FIAS,
						FIBK_FIAT.CURRENCY_CODE CURRENCY_CODE_FIBK_FIAT
					FROM ' . $this->fibtname . ' FIBT
							LEFT JOIN ' . $this->fiatname . ' FIAT ON FIBT.TRANS_CODE = FIAT.IDCODE
							LEFT JOIN ' . $this->acconame . ' ACCO ON FIBT.ACCOUNTCODE = ACCO.IDCODE
							LEFT JOIN ' . $this->fiasname . ' FIAS ON FIAS.IDCODE = FIAT.CASH_ASSET_CODE
							LEFT JOIN ' . $this->fibkname . ' FIBK_FIAS ON FIBK_FIAS.BANK_ACCOUNT_NO = FIAS.FIDU_BANKACT_CODE
							LEFT JOIN ' . $this->fibkname . ' FIBK_FIAT ON FIBK_FIAT.BANK_ACCOUNT_NO = FIAT.FIDU_BANKACT_CODE
					WHERE
						FIBT.DISBURSEMENT_TYPE = ' . singleQuote($s_disbursement_type) . '
						AND FIBT.TRANS_CODE IN (' . $s_idcode_list . ')
						AND NOT (ISnull(FIBT.RECORD_BLOCKING) OR FIBT.RECORD_BLOCKING = ' . singleQuote('0') . ')
					ORDER BY
						FIBT.TRANS_CODE
				';
            } else {
                // new estructure
                $sql .= '
						ACCO.IDNAME IDNAME_ACCO,
						ACCO.IDCODE IDCODE_ACCO,
						ACCO.LEGAL_NAME LEGAL_NAME_ACCO,
						FIAT.CURRENCY_CODE,
						FIBK_FIAS.CURRENCY_CODE CURRENCY_CODE_FIBK_FIAS,
						FIBK_FIAT.CURRENCY_CODE CURRENCY_CODE_FIBK_FIAT,
						FIAT.CHECK_NUMBER,
						FIAT.CHART_FIELD03,
						FIAT.DISBURSEMENT_TYPE,
					' . $s_sql_extra_pars_select_local . '
						FIAT.CHECK_PRINT_FMT,
					FROM
						' . $this->fiatname . ' FIAT
							LEFT JOIN ' . $this->acconame . ' ACCO ON FIAT.CHART_FIELD02 = ACCO.IDCODE
							LEFT JOIN ' . $this->fiasname . ' FIAS ON FIAS.IDCODE = FIAT.CASH_ASSET_CODE
							LEFT JOIN ' . $this->fibkname . ' FIBK_FIAS ON FIBK_FIAS.BANK_ACCOUNT_NO = FIAS.FIDU_BANKACT_CODE
							LEFT JOIN ' . $this->fibkname . ' FIBK_FIAT ON FIBK_FIAT.BANK_ACCOUNT_NO = FIAT.FIDU_BANKACT_CODE
					WHERE
						FIAT.DISBURSEMENT_TYPE = ' . singleQuote($s_disbursement_type) . '
						AND FIAT.IDCODE IN (' . $s_idcode_list . ')
						AND NOT (ISnull(FIAT.RECORD_BLOCKING) OR FIAT.RECORD_BLOCKING = ' . singleQuote('0') . ')
					ORDER BY
						FIAT.IDCODE
				';
            }
            $results = ClxCommon::$db->executeQuery($sql);
            $i_num_rows = ClxCommon::$db->numRows($results);
            // no checks to issue
            if ($i_num_rows == CLX_NUM_ZERO) {
                //No checks to be reversed
                $contents .= '- ' . dict('CODE_ERR_IBS_CHKREV_NOTHINGTODO') . '<br>' . CLX_CHAR_NL;
            } else {
                // Print each check... 		
                $i_cont_reg = CLX_INIT_ZERO;
                $s_trans_code = '';
                while (($fibtrec = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE))) {
                    // verifies if check reverse  interface is inactive
                    if (!empty($b_rev_chk_skt_inactive)) {
                        /*
                         * CODE_ERR_IBS_CHKPRT_REV_INAC
                         * Reverse check interfase is inactive, please reverse check # ^1 manually in EIBS
                         * La interfaz de cheques reversados no esto activa. Por favor haga el reverso del cheque # ^1 manualmente en EIBS
                         */
                        $contents .= '** ' . dict('CODE_GEN_TRANSACTION') . ' # ' . $s_trans_code . ': ' . dict('CODE_ERR_IBS_CHKPRT_REV_INAC', $fibtrec['CURRENCY_CODE']) . '<br>' . CLX_CHAR_NL;
                        continue;
                    }
                    // Prepares the data to be transmitted
                    $record = $this->printChecks_prepares_record($fibtrec, $mappings);
                    if (!empty($this->debug))
                        echoLog(debug_dump('record', $record));
                    $i_cont_reg++;
                    /*
                     * CODE_GEN_TRANSACTION
                     * Transaction
                     * Transaccion
                     */
                    if ($s_trans_code != $fibtrec['TRANS_CODE']) {
                        $s_trans_code = $fibtrec['TRANS_CODE'];
                        $contents .= '** ' . dict('CODE_GEN_TRANSACTION') . ' # ' . $s_trans_code . ' ' . $record['error_status_adic'] . '<br>' . CLX_CHAR_NL . '<br>' . CLX_CHAR_NL;
                    }
                    $contents .= $i_cont_reg . ' ) ' . $record['BENEFICIARY_NAME'] . ' (' . $DBLT->getListItem('CURR', $record['CURRENCY_CODE']) . ' ' . xFormat::number($record['TRANS_AMT'], CLX_NUM_TWO) . ')';
                    if ($record['error'] === true) {
                        $contents .= '<ul><li>' . implode('</li><li>', $record['errormsg']) . '</li></ul>';
                    }
					else {
						if (!empty($this->debug))
							echoLog(debug_dump('fibtrec', $fibtrec));
						// Send the check information and receives the ibs response
						$ar_response = $this->printChecks_send($record, $msg_code);
						$contents .= $i_cont_reg . ' ) ' . $record['BENEFICIARY_NAME'] . ' (' . $DBLT->getListItem('CURR', $record['CURRENCY_CODE']) . ' ' . xFormat::number($record['TRANS_AMT'], CLX_NUM_TWO) . ') ' . dict('GEN_LBL_IBSXFERSTATUS') . ': ' . $ar_response['contents'] . '<br>' . CLX_CHAR_NL;
						if ($ar_response['error'] == false) {
			    // Check was printed successfully... 			
							if (!empty($this->debug))
									ClxCommon::$db->debug = true;
							ClxCommon::$db->updateRecord($this->fibtname, array('RECORD_BLOCKING' => '1', 'CHECK_NO' => $ar_response['ar_last_sent_skt_response'][CLX_NUM_ZERO]['CHECK_NUMBER'], 'CHECK_PRINT_DATI' => xFormat::getNow('Y-m-d H:i:s'), 'CHECK_PRINT_BY' => $_SESSION['user_record']['username']), 'ID = ' . singleQuote($fibtrec['FIBT_ID']));
							if (!empty($this->debug))
								ClxCommon::$db->debug = false;
						}
					}	
                }
				ClxCommon::$db->freeResult($results);
            }
        }
        return $this->display($contents);
    }

    /*
     * Transmits check info to ibs
     * @param array
     * @return array
     */
    function printChecks_send($record, $s_skt_msg_code) {
        reset($record);
        foreach ($record as $key => $val) {
            if (is_string($record[$key])) {
                $record[$key] = $this->replace_accents($record[$key]);
            }
        }
        $record['USERNAME'] = $this->username;
        $record['TIMSYS'] = adodb_date('ymdhms'); // System Time
        if (!empty($this->debug)) {
            echoLog(debug_dump('printChecks_send - record', $record));
        }
        $result = $this->send($s_skt_msg_code, $record);
        $s_print_error_out = $this->print_error_out($result, $this->ar_last_sent_skt);
        $b_error = ($s_print_error_out != EIBS_LBL_OK);
        $ar_response = array(
            'error' => $b_error,
            'ar_result' => $result,
            'ar_last_sent_skt_response' => $this->eibs_utilities_last_send_response,
            'ar_last_sent_skt' => $this->ar_last_sent_skt,
            'contents' => $s_print_error_out
        );
        return $ar_response;
    }

    function replace_accents($str) {
        $str = htmlentities($str, ENT_COMPAT, 'utf-8');
        $str = preg_replace('/&([a-zA-Z])(uml|acute|grave|circ);/', '$1', $str);
        $str = html_entity_decode($str);
        return $str;
    }

}
