<?php

if (!class_exists('fidu_utilities')) {

    /**
     * General Code File
     * For more information contact  support@alenet.com
     *
     * @author       $Author: fsantana $
     * @copyright    Alenet, Inc.
     * @since        $Date: 2016-06-29 13:15:35 -0400 (Wed, 29 Jun 2016) $
     * @version        $Rev: 8481 $
     *
     * CHANGE LOG
     *  Included svn prop
     */
    class fidu_utilities {

        var $modulecode;  // Module Code
        var $compname;   // Contracts
        var $ficoname;   // Contracts
        var $fiasname;   // Assets
        var $inprname;   // Investment Products
        var $contents;  // Html Source
        var $keepLog = true;
        var $debug = false;
        var $datestamp = '';

        function __construct() {
            if (!empty($this->debug))
                echoLog('Utilities Started');
            $this->datestamp = date('YmdHis');
            $this->modulecode = 'FIUT'; // Module
            $this->rounding = CLX_NUM_FIVE;
            $this->display_rounding = CLX_NUM_TWO;
            multiTableName($this, array('COMP', 'FICO', 'FIPA', 'FIAS', 'INPR', 'JEHD', 'GLAC', 'JELN', 'FIAT', 'FITL', 'FIGN', 'INFU', 'INAT', 'FIDA', 'FIGA'));
            $this->contents = '';
        }

        /**
         * Contract Validations
         */
        function validateFicoForm(&$msg, $record) {
            $sql = 'SELECT COUNT(ID) FROM ' . tableName('JEHD') . ' WHERE CONTRACT_CODE = ' . singleQuote($record['FICO_IDCODE']);
            if (UserRequest::$request['ac'] === 'DELETE' && ($record['FICO_BALANCE_AMT'] <> CLX_NUM_ZERO || ClxCommon::$db->getOne($sql) > CLX_NUM_ZERO)) {
                $msg['FICO_IDCODE'] = dict('GEN_ERR_CANNOTDELETE');
            }
            //Validate if end day < accounting date +1
            $sql = 'SELECT COMP.ACCOUNTING_DATE FROM ' . tableName('COMP') . ' COMP ' . ' WHERE COMP.IDCODE = ' . singleQuote($record['FICO_COMP_CODE']) . ' ' . 'AND COMP.ACCOUNTING_DATE >= ' . singleQuote($record['FICO_CANCEL_DATE']) . ' ' .
                    'AND ' . singleQuote($record['FICO_CANCEL_DATE']) . '<>' . "''";
            $return = ClxCommon::$db->getOne($sql);
            if (!empty($return)) {
                /**
                 * GEN_ERR_CLOSMUSTBEMORETHANACCDT
                 * Settlement date must be greater than accounting date plus one day
                 * Fecha de finiquito invalida, debe ser mayor a la fecha de operacion (Fecha contable + 1 dia)
                 */
                $msg['FICO_CANCEL_DATE'] = dict('GEN_ERR_CLOSMUSTBEMORETHANACCDT') . ' = ' . adodb_next_day($return);
            }
        }

        /**
         * Contract Validations
         */
        function validateDelete($record) {
            $status = true;
            if (ClxCommon::$db->oneExistsRecords(tableName('JEHD'), 'CONTRACT_CODE = ' . singleQuote($record['IDCODE'])) > CLX_NUM_ZERO) {
                /**
                  GEN_ERR_CANNOTDELETE Contract can not be deleted. Contrato con actividades no puede ser borrado. */
                addMessage(dict('GEN_ERR_CANNOTDELETE'));
                $status = false;
            }
            return $status;
        }

        function validateFipa(&$record, &$msg) {
            if (UserRequest::$request['ac'] <> 'DELETE')
                $fm = 'FIPA_';
            if (UserRequest::$request['ac'] <> 'DELETE') {
                if (rounder(bcadd($record[$fm . 'CAPITALIZATION_PCT'], $record[$fm . 'CAPPAYMENT_PCT']), CLX_NUM_TWO) > CLX_NUM_HUNDRED) {
                    /**
                     * GEN_ERR_CAPPAYMUSTBEHUNDRED
                     * Monto de capitalizacion mas pago no puede superar 100%
                     * Capitalization amount + payment amount could not be greater than 100%
                     */
                    $msg[$fm . 'CAPPAYMENT_PCT'] = dict('GEN_ERR_CAPPAYMUSTBEHUNDRED');
                    $msg[$fm . 'CAPITALIZATION_PCT'] = dict('GEN_ERR_CAPPAYMUSTBEHUNDRED');
                }
                //Validate if end day < accounting date +1
                $sql = 'SELECT COMP.ACCOUNTING_DATE FROM ' . $this->compname . ' COMP, ' . $this->ficoname . ' FICO ' . ' WHERE FICO.COMP_CODE = COMP.IDCODE ' . 'AND COMP.ACCOUNTING_DATE >= ' . singleQuote($record[$fm . 'END_DATE']) . ' AND ' . singleQuote($record[$fm . 'END_DATE']) . '<>' . "''";
                $return = ClxCommon::$db->getOne($sql);
                if (!empty($return)) {
                    /**
                     * GEN_ERR_CLOSMUSTBEMORETHANACCDT
                     * Settlement date must be greater than accounting date plus one day
                     * Fecha de finiquito invalida, debe ser mayor a la fecha de operacion (Fecha contable + 1 dia)
                     */
                    $msg[$fm . 'END_DATE'] = dict('GEN_ERR_CLOSMUSTBEMORETHANACCDT') . ' = ' . adodb_next_day($return);
                }
            }
        }

        function validateAcco(&$record, &$msg) {
            $fm = '';
            $status = true;
            if (UserRequest::$request['ac'] <> 'DELETE')
                $fm = 'ACCO_';
            if (UserRequest::$request['ac'] === 'DELETE') {
                if (ClxCommon::$db->oneExistsRecords($this->jelnname, 'PARTICIPANT_ID = ' . singleQuote($record[$fm . 'IDCODE'])) > CLX_NUM_ZERO) {
                    $msg[$fm . 'IDCODE'] = dict('GEN_ERR_CANNOTDELETE');
                    if (UserRequest::$request['ac'] === 'DELETE')
                        addMessage(dict('GEN_ERR_CANNOTDELETE'));
                    $status = false;
                }
                if ($status === true && ClxCommon::$db->oneExistsRecords($this->fipaname, 'ACCOUNTCODE = ' . singleQuote($record[$fm . 'IDCODE'])) > CLX_NUM_ZERO) {
                    $msg[$fm . 'IDCODE'] = dict('GEN_ERR_CANNOTDELETE');
                    if (UserRequest::$request['ac'] === 'DELETE')
                        addMessage(dict('GEN_ERR_CANNOTDELETE'));
                    $status = false;
                }
            } else {

                if ($record[$fm . 'LEGAL_TYPE'] === 'G' && $record[$fm . 'INCORPORATION_COUNTRY'] === 'VE' && $record[$fm . 'VE_CODIGO_ONT'] === '') {
                    /**
                      GEN_ERR_ONTCODEMISSING Please specify ONT Code.
                      Favor Especificar el Código ONT.
                     */
                    $msg[$fm . 'VE_CODIGO_ONT'] = dict('GEN_ERR_ONTCODEMISSING');
                    addMessage(dict('GEN_ERR_ONTCODEMISSING'));
                    $status = false;
                }
            }
        }

        function validateFitb(&$record, &$msg) {
            $status = true;
            $fm = '';
            if (UserRequest::$request['ac'] <> 'CREATE' && UserRequest::$request['ac'] <> 'MODIFY' && UserRequest::$request['ac'] <> 'DUPLICATE') {
                if (UserRequest::$request['ac'] <> 'DELETE')
                    $fm = 'FITB_';
                if (ClxCommon::$db->oneExistsRecords($this->fiatname, 'TRANSTBL_CODE = ' . singleQuote($record[$fm . 'IDCODE'])) <> CLX_NUM_ZERO) {
                    $msg[$fm . 'IDCODE'] = dict('GEN_ERR_CANNOTDELETEREC');
                    $status = false;
                }
            }
            return $status;
        }

        function validateTrdo(&$record, &$msg) {
            $status = true;
            $fm = '';
            if (UserRequest::$request['ac'] <> 'DELETE')
                $fm = 'TRDO_';
            //Parametro permite apagar opcion de validacion de documentos de transaccion
            $validate = companyParameter('ALLOW_TRDO_UPDATE');
            if (!empty($validate) && ClxCommon::$db->oneExistsRecords($this->fiatname, 'TRANS_STATUS <> ' . singleQuote('PEND') . ' AND IDCODE = ' . singleQuote($record[$fm . 'TRANS_CODE'])) <> CLX_NUM_ZERO) {
                $msg[$fm . 'TRANS_CODE'] = dict('GEN_ERR_CANNOTDELETEREC');
                $status = false;
            }
            return $status;
        }

        function validateFipd(&$record, &$msg) {
            $status = true;
            $fm = '';
            if (UserRequest::$request['ac'] <> 'DELETE')
                $fm = 'FIPD_';
            if (ClxCommon::$db->oneExistsRecords($this->fiatname, 'TRANS_STATUS <> ' . singleQuote('PEND') . ' AND IDCODE = ' . singleQuote($record[$fm . 'TRANS_CODE'])) <> CLX_NUM_ZERO) {
                $msg[$fm . 'TRANS_CODE'] = dict('GEN_ERR_CANNOTDELETEREC');
                $status = false;
            }
            return $status;
        }

        function validateFibt(&$record, &$msg) {
            $fm = '';
            $status = true;
            if (UserRequest::$request['ac'] <> 'DELETE')
                $fm = 'FIBT_';
            if (UserRequest::$request['ac'] === 'DELETE') {

                if (ClxCommon::$db->oneExistsRecords($this->fiatname, 'TRANS_STATUS <> ' . singleQuote('PEND') . ' AND IDCODE = ' . singleQuote($record[$fm . 'TRANS_CODE'])) <> CLX_NUM_ZERO) {
                    $msg[$fm . 'TRANS_CODE'] = dict('GEN_ERR_CANNOTDELETEREC');
                    $status = false;
                }
            } else {

                // Validates that CRM acc. and free beneficiary name cannot be both filled
                if ($record[$fm . 'BENEFICIARY_NAME'] !== '' && ($record[$fm . 'ACCOUNTCODE_prim'] . $record[$fm . 'ACCOUNTCODE_secd']) !== '') {
                    $msg[$fm . 'ACCOUNTCODE'] = dict('CODE_ERR_FIBT_CRMYNOMALAV');
                    $status = false;
                } elseif (($record[$fm . 'BENEFICIARY_NAME'] . $record[$fm . 'ACCOUNTCODE_prim'] . $record[$fm . 'ACCOUNTCODE_secd']) === '') {
                    $msg[$fm . 'ACCOUNTCODE'] = dict('CODE_ERR_FIBT_CRMYNOMBLK');
                    $status = false;
                }
            }
            return $status;
        }

        function validateIetb(&$record, &$msg) {
            $status = true;
            $fm = '';
            if (UserRequest::$request['ac'] === 'DELETE') {
                if (UserRequest::$request['ac'] <> 'DELETE')
                    $fm = 'IETB_';
                if (ClxCommon::$db->oneExistsRecords($this->fitlname, 'INCEXP_CODE = ' . singleQuote($record[$fm . 'IDCODE'])) <> CLX_NUM_ZERO) {
                    $msg[$fm . 'IDCODE'] = dict('GEN_ERR_CANNOTDELETEREC');
                    $status = false;
                }
            }
            return $status;
        }

        function validateFign(&$record, &$msg) {
            $status = true;
            $fm = '';
            if (UserRequest::$request['ac'] <> 'DELETE')
                $fm = 'FIGN_';
            if (empty($record[$fm . 'CONTRACT_CODE']) and ! empty($record[$fm . 'CONTRACT_CODE_prim']))
                $record[$fm . 'CONTRACT_CODE'] = $record[$fm . 'CONTRACT_CODE_prim'];
            if (($record[$fm . 'GARNISH_AMT'] <> CLX_NUM_ZERO && $record[$fm . 'GARNISH_PCT'] <> CLX_NUM_ZERO) || ($record[$fm . 'GARNISH_AMT'] == CLX_NUM_ZERO && $record[$fm . 'GARNISH_PCT'] == CLX_NUM_ZERO)) {
                /**
                 * CODE_ERR_MUSTBEAMOUNTORPCT
                 * Please choose one amount or porcentage.
                 * Emgargo solo puede ser por monto o porcentage, no ambas.
                 */
                if (empty($fm))
                    addMessage(dict('CODE_ERR_MUSTBEAMOUNTORPCT'));
                $msg[$fm . 'GARNISH_AMT'] = dict('CODE_ERR_MUSTBEAMOUNTORPCT');
                $status = false;
            }
            //Test if garnishment already exists for other contract
            if (ClxCommon::$db->oneExistsRecords($this->fignname, 'IDCODE = ' . singleQuote($record[$fm . 'IDCODE']) . ' AND CONTRACT_CODE <> ' . singleQuote($record[$fm . 'CONTRACT_CODE'])) <> CLX_NUM_ZERO) {
                if (empty($fm))
                    addMessage(dict('CODE_ERR_INVALIDCONTRACT'));
                $msg[$fm . 'CONTRACT_CODE'] = dict('CODE_ERR_INVALIDCONTRACT');
                $status = false;
            }
            $fignold = ClxCommon::$db->getRecord($this->fignname, 'CONTRACT_CODE = ' . singleQuote($record[$fm . 'CONTRACT_CODE']) . ' AND IDCODE = ' . singleQuote($record[$fm . 'IDCODE']));
            if (!empty($fignold)) {
                if ($record[$fm . 'GARNISH_AMT'] <> CLX_NUM_ZERO && $fignold['GARNISH_PCT'] <> CLX_NUM_ZERO) {
                    /**
                     * CODE_ERR_MUSTBEPCT
                     * Percentage allowed only.
                     * Emgargo solo puede ser por porcentage.
                     */
                    if (empty($fm))
                        addMessage(dict('CODE_ERR_MUSTBEPCT'));
                    $msg[$fm . 'GARNISH_AMT'] = dict('CODE_ERR_MUSTBEPCT');
                    $status = false;
                }
                if ($fignold['GARNISH_AMT'] <> CLX_NUM_ZERO && $record[$fm . 'GARNISH_PCT'] <> CLX_NUM_ZERO) {
                    //must be amount
                    /**
                     * CODE_ERR_MUSTBEAMT
                     * Amount allowed only.
                     * Emgargo solo puede ser por monto.
                     */
                    if (empty($fm))
                        addMessage(dict('CODE_ERR_MUSTBEAMT'));
                    $msg[$fm . 'GARNISH_PCT'] = dict('CODE_ERR_MUSTBEAMT');
                    $status = false;
                }
            }
            if ($record[$fm . 'GARNISH_AMT'] <> CLX_NUM_ZERO && $record[$fm . 'MAXBLOCKED_AMT'] <> CLX_NUM_ZERO) {
                /**
                 * CODE_ERR_MAXNOTALLOWED
                 * Max Amount allowed only on percentages.
                 * Monto maximo solo permitido para porcentajes
                 */
                if (empty($fm))
                    addMessage(dict('CODE_ERR_MAXNOTALLOWED'));
                $msg[$fm . 'MAXBLOCKED_AMT'] = dict('CODE_ERR_MAXNOTALLOWED');
                $status = false;
            }
            if (UserRequest::$request['ac'] === 'CREATE' || UserRequest::$request['ac'] === 'DUPLICATE') {
                //Validate that modifications must be under efftdate
                if (ClxCommon::$db->oneExistsRecords($this->fignname, 'CONTRACT_CODE = ' . singleQuote($record[$fm . 'CONTRACT_CODE']) . ' AND IDCODE = ' . singleQuote($record[$fm . 'IDCODE'])) <> CLX_NUM_ZERO) {
                    /**
                     * CODE_ERR_FIGNEXISTSMUSTEFFTDATE
                     * Already created please used modify effective dated instead.
                     * Emgargo ya se encuentra registrado, favor modifique con fecha efectiva.
                     */
                    if (empty($fm))
                        addMessage(dict('CODE_ERR_FIGNEXISTSMUSTEFFTDATE'));
                    $msg[$fm . 'IDCODE'] = dict('CODE_ERR_FIGNEXISTSMUSTEFFTDATE');
                    $status = false;
                }
            }
            //Validate contract status
            if (empty($record[$fm . 'CONTRACT_CODE']) || ClxCommon::$db->oneExistsRecords($this->ficoname, 'CONTRACT_STATUS = ' . singleQuote('ACTI') . ' AND IDCODE = ' . singleQuote($record[$fm . 'CONTRACT_CODE'])) == CLX_NUM_ZERO) {
                if (empty($fm))
                    addMessage(dict('CODE_ERR_FICOSTATUSINVALID'));
                $msg[$fm . 'CONTRACT_CODE'] = dict('CODE_ERR_FICOSTATUSINVALID');
                $status = false;
            }
            //Validate participant status
            if (empty($record[$fm . 'PARTICIPANT_ID']) || ClxCommon::$db->oneExistsRecords($this->fipaname, 'STATUS = ' . singleQuote('A') . ' AND CONTRACT_CODE = ' . singleQuote($record[$fm . 'CONTRACT_CODE']) . ' AND ACCOUNTCODE = ' . singleQuote($record[$fm . 'PARTICIPANT_ID'])) == CLX_NUM_ZERO) {
                if (empty($fm))
                    addMessage(dict('CODE_ERR_INVALIDPARTICIPANT', $record[$fm . 'PARTICIPANT_ID'], $record[$fm . 'CONTRACT_CODE']));
                $msg[$fm . 'PARTICIPANT_ID'] = dict('CODE_ERR_INVALIDPARTICIPANT');
                $status = false;
            }
            //Validando que fecha de creacion de embargo sea mayor a la fecha de la contabilidad
            //Accounting Date Validation
            $sql = 'SELECT COUNT(COMP.ACCOUNTING_DATE) COUNTER FROM ' . $this->compname . ' COMP, ' . $this->ficoname . ' FICO WHERE COMP.IDCODE = FICO.COMP_CODE AND FICO.IDCODE = ' . singleQuote($record[$fm . 'CONTRACT_CODE']) . ' AND COMP.ACCOUNTING_DATE >= ' . singleQuote($record[$fm . 'CLX_EFFTDATE']);
            if (ClxCommon::$db->getOne($sql) <> CLX_NUM_ZERO) {
                /**
                 * CODE_ERR_EFFTDATENOTACCDATE
                 * Effective date must be greater than accounting date.
                 * Fecha efectiva debe ser mayor a la fecha de la contabilidad.
                 */
                if (empty($fm))
                    addMessage(dict('CODE_ERR_EFFTDATENOTACCDATE'));
                $msg[$fm . 'CLX_EFFTDATE'] = dict('CODE_ERR_EFFTDATENOTACCDATE');
                $status = false;
            }
            if (UserRequest::$request['ac'] === 'MODIFY' || UserRequest::$request['ac'] === 'DELETE') {
                //Accounting Date Validation
                $sql = 'SELECT COMP.ACCOUNTING_DATE COUNTER FROM ' . $this->compname . ' COMP, ' . $this->ficoname . ' FICO WHERE COMP.IDCODE = FICO.COMP_CODE AND FICO.IDCODE = ' . singleQuote($record[$fm . 'CONTRACT_CODE']);
                $accounting_date = ClxCommon::$db->getOne($sql);
                if (adodb_next_day($accounting_date) <> $record[$fm . 'CLX_EFFTDATE']) {
                    if (empty($fm))
                        addMessage(dict('GEN_ERR_CANNOTDELETEREC'));
                    $msg[$fm . 'IDCODE'] = dict('GEN_ERR_CANNOTDELETEREC');
                    $status = false;
                }
            }
            return $status;
        }

        function validateFiga(&$record, &$msg) {
            $fm = '';
            $status = true;
            if (UserRequest::$request['ac'] <> 'DELETE')
                $fm = 'FIGA_';
            if ($record[$fm . 'ASSET_GUARANTEE_PCT'] <> CLX_NUM_ZERO && $record[$fm . 'ASSET_GUARANTEE_AMT'] <> CLX_NUM_ZERO) {
                /*
                 * CODE_ERR_PCTORAMTREQUIRED
                 * Percentage or Amount required, please verify
                 * Porcentaje o monto requerido favor verificar
                 */
                $msg[$fm . 'ASSET_GUARANTEE_PCT'] = dict('CODE_ERR_PCTORAMTREQUIRED');
                $status = false;
            }
            if (UserRequest::$request['ac'] <> 'DELETE') {
                if (!array_key_exists($fm . 'GUARANTING_ASSET', $record))
                    $record[$fm . 'GUARANTING_ASSET'] = '';
                if ($record[$fm . 'ASSET_GUARANTEE_AMT'] <> CLX_NUM_ZERO && $record[$fm . 'ASSET_GUARANTEE_AMT'] > rounder(ClxCommon::$db->getOne('SELECT SUM(BALANCE_AMT) FROM ' . $this->fidaname . ' WHERE ASSET_CODE = ' . singleQuote($record[$fm . 'GUARANTING_ASSET'])), $this->display_rounding)) {
                    /*
                     * CODE_ERR_PCTORAMTREQUIRED
                     * Percentage or Amount required, please verify
                     * Porcentaje o monto requerido favor verificar
                     */
                    $msg[$fm . 'ASSET_GUARANTEE_AMT'] = dict('CODE_ERR_NOTSUFFICIENTFUNDS', $record[$fm . 'GUARANTING_ASSET']);
                    $status = false;
                }
            }
            if ($record[$fm . 'CONTRACT_CODE'] === '' && $record[$fm . 'CONTRACT_CODE_prim'] <> '') {
                $record[$fm . 'CONTRACT_CODE'] = $record[$fm . 'CONTRACT_CODE_prim'];
            }
            $accounting_date = ClxCommon::$db->getOne('SELECT COMP.ACCOUNTING_DATE FROM ' . $this->compname . ' COMP, ' . $this->ficoname . ' FICO WHERE COMP.IDCODE = FICO.COMP_CODE AND FICO.IDCODE = ' . singleQuote($record[$fm . 'CONTRACT_CODE']));
            if (UserRequest::$request['ac'] === 'MODIFY' && $record[$fm . 'CLX_EFFTDATE'] <> adodb_next_day($accounting_date)) {
                /*
                 * CODE_ERR_USEEFFETIVEDATE
                 * Record cannot be modified, please use modify effective date.
                 * Registro no puede ser modificado favor utilizar fecha efectiva.
                 */
                $msg[$fm . 'CLX_EFFTDATE'] = dict('CODE_ERR_USEEFFETIVEDATE');
                $status = false;
            }
            //Test if garranty already exists for other contract
            if (ClxCommon::$db->oneExistsRecords($this->figaname, 'IDCODE = ' . singleQuote($record[$fm . 'IDCODE']) . ' AND CONTRACT_CODE <> ' . singleQuote($record[$fm . 'CONTRACT_CODE'])) <> CLX_NUM_ZERO) {
                if (empty($fm))
                    addMessage(dict('CODE_ERR_INVALIDCONTRACT'));
                $msg[$fm . 'CONTRACT_CODE'] = dict('CODE_ERR_INVALIDCONTRACT');
                $status = false;
            }
            if (UserRequest::$request['ac'] <> 'MODIFY' && $record[$fm . 'CLX_EFFTDATE'] < adodb_next_day($accounting_date)) {
                /*
                 * CODE_ERR_EFFTDATEGREATEROREQUALTO
                 * Effective date must be greater than ^1.
                 * Fecha efectiva debe ser mayor o igual ^1.
                 */
                $msg[$fm . 'CLX_EFFTDATE'] = dict('CODE_ERR_EFFTDATEGREATEROREQUALTO', adodb_next_day($accounting_date));
                $status = false;
            }
            if (UserRequest::$request['ac'] <> 'MODIFY' && $record[$fm . 'CLX_EFFTDATE'] < adodb_next_day($accounting_date)) {
                /*
                 * CODE_ERR_EFFTDATEGREATEROREQUALTO
                 * Effective date must be greater than ^1.
                 * Fecha efectiva debe ser mayor o igual ^1.
                 */
                $msg[$fm . 'CLX_EFFTDATE'] = dict('CODE_ERR_EFFTDATEGREATEROREQUALTO', adodb_next_day($accounting_date));
                $status = false;
            }
            if (UserRequest::$request['ac'] <> 'MODIFY' && UserRequest::$request['ac'] <> 'DUPLICATE_EFFTDATE' && ClxCommon::$db->oneExistsRecords($this->figaname, 'GUARANTING_ASSET = ' . singleQuote($record[$fm . 'GUARANTING_ASSET'])) > CLX_NUM_ZERO) {
                /*
                 * CODE_ERR_ONEGARPERASSET
                 * Only one guaranty per subaccount allowed.
                 * Solo una garantia permitida por subcuenta.
                 */
                $msg[$fm . 'GUARANTING_ASSET'] = dict('CODE_ERR_ONEGARPERASSET');
                $status = false;
            }
            return $status;
        }

        function initFignForm(&$record) {
            if ((UserRequest::$request['ac'] === 'DUPLICATE_EFFTDATE' || UserRequest::$request['ac'] === 'CREATE') && empty(UserRequest::$request['confirm'])) {
                $record['FIGN_CLX_EFFTDATE'] = adodb_next_day(ClxCommon::$db->getOne('SELECT COMP.ACCOUNTING_DATE FROM ' . $this->compname . ' COMP, ' . $this->ficoname . ' FICO WHERE FICO.COMP_CODE = COMP.IDCODE AND FICO.IDCODE = ' . singleQuote($record['FIGN_CONTRACT_CODE'])));
            }
        }

        function initFigaForm(&$record) {
            if ((UserRequest::$request['ac'] === 'DUPLICATE_EFFTDATE' || UserRequest::$request['ac'] === 'CREATE') && empty(UserRequest::$request['confirm'])) {
                $record['FIGA_CLX_EFFTDATE'] = adodb_next_day(ClxCommon::$db->getOne('SELECT COMP.ACCOUNTING_DATE FROM ' . $this->compname . ' COMP, ' . $this->ficoname . ' FICO WHERE FICO.COMP_CODE = COMP.IDCODE AND FICO.IDCODE = ' . singleQuote($record['FIGA_CONTRACT_CODE'])));
            }
        }

        function initFiatForm(&$record) {
            if (UserRequest::$request['ac'] === 'CREATE' && empty(UserRequest::$request['confirm'])) {
                if (!empty($record['FIAT_CONTRACT_CODE']))
                    $comprec = ClxCommon::$db->getOneRecord('SELECT COMP.ACCOUNTING_DATE, COMP.DEF_BANKACC_CODE FROM ' . $this->compname . ' COMP, ' . $this->ficoname . ' FICO WHERE FICO.COMP_CODE = COMP.IDCODE AND FICO.IDCODE = ' . singleQuote($record['FIAT_CONTRACT_CODE']));
                else
                    $comprec = ClxCommon::$db->getOneRecord('SELECT COMP.ACCOUNTING_DATE, COMP.DEF_BANKACC_CODE FROM ' . $this->compname . ' COMP');
                $record['FIAT_EFFTDATE'] = adodb_next_day($comprec['ACCOUNTING_DATE']);
                $record['FIAT_FIDU_BANKACT_CODE'] = $comprec['DEF_BANKACC_CODE'];
                $record['FIAT_FIDU_BANKACT_CODE_prim'] = $comprec['DEF_BANKACC_CODE'];
            }
        }

        /**
         * Initial parameters pre database commit
         */
        function createFico(&$record) {
            if (UserRequest::$request['ac'] === 'CREATE' && !empty($record['PARENTCONTRACT_CODE'])) {
                $parentfico = ClxCommon::$db->getRecord($this->ficoname, 'IDCODE = ' . singleQuote($record['PARENTCONTRACT_CODE']));
                foreach ($parentfico as $key => $val) {
                    if ($key === 'MODIFIEDBY' ||
                            $key === 'MDATE' || $key === 'CREATEDBY' || $key === 'CDATE' || $key === 'TOTALBALANCE_AMT' || $key === 'SUBBALANCE_AMT' || $key === 'BASE_BAL_AMT' || $key === 'BALANCE_AMT' || $key === 'CANCEL_DATE' || $key === 'START_DATE' || $key === 'EXPIRATION_DATE' || $key === 'PROJECT_CODE' || $key === 'EFFTDATE' || $key === 'DESCRIPTION' || $key === 'CONTRACT_STATUS' || $key === 'PARENTCONTRACT_CODE' || $key === 'START_AMT')
                        continue;
                    else
                        $record[$key] = $val;
                }
                unset($record['ID']);
                $subcode = (ClxCommon::$db->countRecords($this->ficoname, 'PARENTCONTRACT_CODE = ' . singleQuote($record['PARENTCONTRACT_CODE'])) + CLX_NUM_ONE);
                if (strlen($subcode) < CLX_NUM_THREE)
                    $subcode = str_repeat('0', CLX_NUM_THREE - strlen($subcode)) . $subcode;
                $record['IDCODE'] = $record['PARENTCONTRACT_CODE'] . '-' . $subcode;
            }
        }

        function initFicoForm(&$record) {
            if (UserRequest::$request['ac'] === 'CREATE' && empty(UserRequest::$request['confirm'])) {
                $comprec = ClxCommon::$db->getFields($this->compname, 'ACCOUNTING_DATE,IDCODE');
                $record['FICO_START_DATE'] = adodb_next_day($comprec['ACCOUNTING_DATE']);
                $record['FICO_COMP_CODE'] = $comprec['IDCODE'];
            }
        }

        function initIetbForm(&$record) {
            if ((UserRequest::$request['ac'] === 'CREATE' || UserRequest::$request['ac'] === 'DUPLICATE') && empty(UserRequest::$request['confirm'])) {
                $record['IETB_EFFTDATE'] = adodb_next_day(ClxCommon::$db->getOne('SELECT COMP.ACCOUNTING_DATE FROM ' . $this->compname . ' COMP'));
                $record['IETB_FORMULA_CODE'] = 'DEF';
                $record['IETB_TAX_WITHOLDING_PCT'] = CLX_NUM_HUNDRED;
            }
        }

        function initInatForm(&$record) {
            if ((UserRequest::$request['ac'] === 'CREATE' || UserRequest::$request['ac'] === 'DUPLICATE') && empty(UserRequest::$request['confirm'])) {
                if (!empty($record['INAT_PORTFOLIO_CODE']))
                    $record['INAT_EFFTDATE'] = adodb_next_day(ClxCommon::$db->getOne('SELECT COMP.ACCOUNTING_DATE FROM ' . $this->compname . ' COMP, ' . $this->infuname . ' INFU WHERE INFU.COMP_CODE = COMP.IDCODE AND INFU.IDCODE = ' . singleQuote($record['INAT_PORTFOLIO_CODE'])));
                else
                    $record['INAT_EFFTDATE'] = adodb_next_day(ClxCommon::$db->getOne('SELECT ACCOUNTING_DATE FROM ' . $this->compname));
            }
            if (UserRequest::$request['ac'] === 'DUPLICATE' && empty(UserRequest::$request['confirm'])) {
                $record['INAT_IDCODE'] = 'AUTO';
                $record['INAT_TRANS_STATUS'] = 'PEND';
                $record['INAT_REVACTI_CODE'] = '';
                $record['INAT_REVACTI_CODE_prim'] = '';
                $record['INAT_REVACTI_CODE_secd'] = '';
                $record['INAT_INVESTMENT_CODE_prim'] = $record['INAT_INVESTMENT_CODE'];
            }
        }

        function initGlacForm(&$record) {
            if (UserRequest::$request['ac'] === 'CREATE' && empty(UserRequest::$request['confirm'])) {
                $record['GLAC_IDCODE'] = '';
            }
        }

        function validateTxtb(&$record, &$msg) {
            $fm = '';
            if (UserRequest::$request['ac'] === 'DELETE') {
                if (UserRequest::$request['ac'] <> 'DELETE')
                    $fm = 'TXTB_';
                if (ClxCommon::$db->oneExistsRecords($this->fitlname, 'TAX_CODE = ' . singleQuote($record[$fm . 'IDCODE'])) <> CLX_NUM_ZERO) {
                    $msg[$fm . 'IDCODE'] = dict('GEN_ERR_CANNOTDELETEREC');
                    $status = false;
                }
            }
            return $status;
        }

        /**
         * Asset Validations
         */
        function validateFipaDelete($record, $tblcode = 'FIPA') {
            $status = true;
            $fm = '';
            if (UserRequest::$request['ac'] === 'CREATE' || UserRequest::$request['ac'] === 'MODIFY')
                return $status;
            $sql = 'SELECT count(JEHD.ID) FROM ' . $this->jehdname . ' JEHD, ' . $this->jelnname . ' JELN WHERE JEHD.IDCODE = JELN.JEHDR_CODE AND JEHD.CONTRACT_CODE = ' . singleQuote($record[$fm . 'CONTRACT_CODE']) . ' AND JELN.PARTICIPANT_ID = ' . singleQuote($record[$fm . 'PARTICIPANT_ID']);
            if (ClxCommon::$db->getOne($sql) > CLX_NUM_ZERO) {
                /**
                  GEN_ERR_CANNOTDELETEREC Record cannot be deleted or modified at this moment.
                  Registro no puede ser borrado ser borrado o modificado.
                 */
                addMessage(dict('GEN_ERR_CANNOTDELETEREC'));
                $status = false;
            }
            return $status;
        }

        /**
         * Comp Validations
         */
        function validateCompDelete($record, $tblcode = 'COMP') {
            $status = true;
            $fm = '';
            if (UserRequest::$request['ac'] === 'CREATE' || UserRequest::$request['ac'] === 'MODIFY')
                return $status;
            if (UserRequest::$request['ac'] === 'MODIFY')
                $fm = $tblcode . '_';
            $sql = 'SELECT count(JEHD.ID) FROM ' . $this->compname . ' COMP, ' . $this->jehdname . ' JEHD WHERE COMP.IDCODE = JEHD.COMP_CODE AND COMP.IDCODE = ' . singleQuote($record[$fm . 'IDCODE']);
            if (ClxCommon::$db->getOne($sql) > CLX_NUM_ZERO) {
                /**
                  GEN_ERR_CANNOTDELETEREC Record cannot be deleted or modified at this moment.
                  Registro no puede ser borrado ser borrado o modificado.
                 */
                addMessage(dict('GEN_ERR_CANNOTDELETEREC'));
                $status = false;
            }
            return $status;
        }

        /**
         * Asset Validations
         */
        function validateFiasDelete($record, $tblcode = 'FIAS') {
            $status = true;
            $fm = '';
            if (UserRequest::$request['ac'] === 'CREATE')
                return $status;
            if (UserRequest::$request['ac'] === 'MODIFY')
                $fm = $tblcode . '_';
            $sql = 'SELECT COUNT(FIAT.ID) FROM ' . $this->fitlname . ' FITL, ' . $this->fiatname . ' FIAT, ' . $this->jehdname . ' JEHD WHERE JEHD.TRANS_CODE = FIAT.IDCODE AND FIAT.IDCODE = FITL.TRANS_CODE AND (FIAT.ASSET_CODE = ' . singleQuote($record[$fm . 'IDCODE']) . ' OR FIAT.CASH_ASSET_CODE = ' . singleQuote($record[$fm . 'IDCODE']) . ' OR FIAT.ASSET_CODE = ' . singleQuote($record[$fm . 'IDCODE']) . ') LIMIT 1';
            if (ClxCommon::$db->getOne($sql) > CLX_NUM_ZERO) {
                /**
                  GEN_ERR_CANNOTDELETEREC Record cannot be deleted or modified at this moment.
                  Registro no puede ser borrado ser borrado o modificado.
                 */
                addMessage(dict('GEN_ERR_CANNOTDELETEREC'));
                $status = false;
            }
            $record[$fm . 'PORTFOLIO_CODE'] = ClxCommon::$db->getField($this->fiasname, 'PORTFOLIO_CODE', 'IDCODE = ' . singleQuote($record[$fm . 'IDCODE']));
            if (!empty($record[$fm . 'PORTFOLIO_CODE'])) {
                $sql = 'SELECT SUM(JELN.DEBIT_AMT-JELN.CREDIT_AMT) BALANCE_AMT FROM ' . $this->ficoname . ' FICO,  ' . $this->fiasname . ' FIAS,  ' . $this->infuname . ' INFU,  ' . $this->jehdname . ' JEHD,  ' . $this->jelnname . ' JELN,  ' . $this->glacname . ' GLAC WHERE JEHD.IDCODE = JELN.JEHDR_CODE ' . 'AND FIAS.CONTRACT_CODE = FICO.IDCODE ' . 'AND FIAS.IDCODE = ' . singleQuote($record[$fm . 'IDCODE']) . ' ' . 'AND FIAS.PORTFOLIO_CODE = JEHD.PORTFOLIO_CODE ' . 'AND INFU.IDCODE = JEHD.PORTFOLIO_CODE ' . 'AND FICO.IDCODE = JEHD.CONTRACT_CODE ' . 'AND JELN.GLACCOUNTCODE = GLAC.IDCODE ' . 'AND JEHD.PORTFOLIO_CODE = ' . singleQuote($record[$fm . 'PORTFOLIO_CODE']) . ' ' . 'AND JEHD.JE_STATUS IN (' . singleQuoteArray(array('VER', 'CLOS')) . ') ' . 'AND GLAC.TRUST_ITEM_TYPE IN (' . singleQuoteArray(array('IN', 'AR')) . ') ' . 'GROUP BY JEHD.CONTRACT_CODE ';
                if (ClxCommon::$db->getOne($sql) <> CLX_NUM_ZERO) {
                    addMessage(dict('GEN_ERR_CANNOTDELETEREC'));
                    $status = false;
                }
            }
            return $status;
        }

        /**
         * Portafolio Validations
         */
        function validateInfuDelete($record, $tblcode = 'INFU') {
            $status = true;
            $fm = '';
            if (UserRequest::$request['ac'] === 'CREATE' || UserRequest::$request['ac'] === 'MODIFY')
                return $status;
            if (UserRequest::$request['ac'] === 'MODIFY')
                $fm = $tblcode . '_';
            if (!empty($record[$fm . 'IDCODE']) && ClxCommon::$db->oneExistsRecords($this->jehdname, 'PORTFOLIO_CODE = ' . singleQuote($record[$fm . 'IDCODE'])) > CLX_NUM_ZERO) {
                /**
                  GEN_ERR_CANNOTDELETEREC Record cannot be deleted or modified at this moment.
                  Registro no puede ser borrado ser borrado o modificado.
                 */
                addMessage(dict('GEN_ERR_CANNOTDELETEREC'));
                $status = false;
            }
            return $status;
        }

        /**
         * Create contract cash on contract create event
         * @param: array Contract Record
         */
        function createCash($record) {
            $DBLT = &getTool('DBLT');
            $comprec = ClxCommon::$db->getRecord($this->compname, 'IDCODE = ' . singleQuote($record['COMP_CODE']));
            $rec = array();
            $rec['CONTRACT_CODE'] = $record['IDCODE'];
            $rec['IDCODE'] = 'AUTO';
            $rec['IDNAME'] = $record['IDNAME'] . ', ' . $DBLT->getListItem('GLIT', 'CA');
            $rec['DESCRIPTION'] = $rec['IDNAME'];
            $rec['TRUST_ITEM_TYPE'] = 'CA';
            $rec['GL_TYPE'] = 'AS';
            $rec['EFFTDATE'] = date('Y-m-d H:i:s');
            $rec['CURRENCY_CODE'] = $record['CURRENCY_CODE'];
            $rec['ASSET_VALUE'] = CLX_NUM_0_PT_00;
            $rec['FIDU_BANKACT_CODE'] = $comprec['DEF_BANKACC_CODE'];
            $rec['GLACCOUNTCODE'] = $comprec['DEFCASHCODE'];
            $rec['CREATEDBY'] = '';
            $rec['MODIFIEDBY'] = '';
            $rec['CDATE'] = '';
            $rec['MDATE'] = '';
            if (!function_exists('setCreationData'))
                includeCodeLib('GEN_REC_UTIL');
            $rec = setCreationData($rec);
            $rec = setCreationIdCode($rec, 'FIAS');
            ClxCommon::$db->createRecord($this->fiasname, $rec, false);
        }

// End method
        /**
         * Create contract earned interest asset used required for portfolio investments.
         * @param: array Contract Record
         */
        function createEarnedInt($record) {
            if (((!empty($record['PORTFOLIO_CODE']) && $record['PORTFOLIO_CODE'] !== 'null') || $record['TRUST_ITEM_TYPE'] === 'IN' || $record['TRUST_ITEM_TYPE'] === 'CR') && (empty($record['AR_ASSET_CODE']) || $record['AR_ASSET_CODE'] === 'null')) {
                $DBLT = &getTool('DBLT');
                $rec = array();
                $rec['CONTRACT_CODE'] = $record['CONTRACT_CODE'];
                $rec['IDCODE'] = 'AUTO';
                $rec['IDNAME'] = $record['IDNAME'] . ', ' . $DBLT->getListItem('GLIT', 'AR');
                $rec['DESCRIPTION'] = $rec['IDNAME'];
                $rec['TRUST_ITEM_TYPE'] = 'AR';
                $rec['GL_TYPE'] = 'AS';
                $rec['EFFTDATE'] = date('Y-m-d H:i:s');
                $rec['CURRENCY_CODE'] = $record['CURRENCY_CODE'];
                $rec['ASSET_VALUE'] = CLX_NUM_0_PT_00;
                $rec['GLACCOUNTCODE'] = ClxCommon::$db->getOne('SELECT COMP.EARNEDINT_CODE FROM ' . $this->compname . ' COMP, ' . $this->ficoname . ' FICO WHERE COMP.IDCODE = FICO.COMP_CODE AND FICO.IDCODE = ' . singleQuote($record['CONTRACT_CODE']));
                $rec['CREATEDBY'] = '';
                $rec['MODIFIEDBY'] = '';
                $rec['CDATE'] = '';
                $rec['MDATE'] = '';
                if (!function_exists('setCreationData'))
                    includeCodeLib('GEN_REC_UTIL');
                $rec = setCreationData($rec);
                $rec = setCreationIdCode($rec, 'FIAS');
                ClxCommon::$db->createRecord($this->fiasname, $rec, false);
                //Updating Portfolio Asset
                $this->associateArToAsset($record['IDCODE'], $rec['IDCODE']);
            }
        }

// End method
        function associateCashToAsset($asset_code, $cash_asset_code) {
            ClxCommon::$db->updateRecord($this->fiasname, array('CASH_ASSET_CODE' => $cash_asset_code), 'IDCODE = ' . singleQuote($asset_code));
        }

        function associateArToAsset($asset_code, $ar_asset_code) {
            ClxCommon::$db->updateRecord($this->fiasname, array('AR_ASSET_CODE' => $ar_asset_code), 'IDCODE = ' . singleQuote($asset_code));
        }

        function getCashAsset($contract_code, $currency_code) {
            return ClxCommon::$db->getField($this->fiasname, 'IDCODE', 'CONTRACT_CODE=' . singleQuote($contract_code), 'CURRENCY_CODE = ' . singleQuote($currency_code), 'GL_TYPE = ' . singleQuote('AS') . '', 'TRUST_ITEM_TYPE = ' . singleQuote('CA') . '');
        }

// End function
        /**
         * Create sub account for prestaciones
         */
        function createSubAccountPrestaciones($trust_item_type, $contract_code, $currency_code, $cash_asset_code) {
            if ($trust_item_type === 'AD01') {
                ClxFreeReturn( $this->createSubAccount($contract_code, 'AS', 'PR', $currency_code)); //PRESTAMOS ACCOUNT
                ClxFreeReturn($this->createSubAccount($contract_code, 'AS', 'AD', $currency_code)); //ANTICIPOS
            }
        }

        /**
         * Create contract FIAS or Sub Account
         * @param: string Contract Code
         * @param: string GL Type
         */
        function createSubAccount($contract_code, $gl_type, $trust_item_type, $currency_code, $extras = array()) {
            $DBLT = &getTool('DBLT');
            $rec = array();
            $rec['CONTRACT_CODE'] = $contract_code;
            $rec['IDCODE'] = 'AUTO';
            $rec['IDNAME'] = $DBLT->getListItem('GLIT', $trust_item_type);
            $rec['TRUST_ITEM_TYPE'] = $trust_item_type;
            $rec['GL_TYPE'] = $gl_type;
            $rec['CURRENCY_CODE'] = $currency_code;
            $rec['CREATEDBY'] = '';
            $rec['MODIFIEDBY'] = '';
            $rec['CDATE'] = '';
            $rec['MDATE'] = '';
            foreach ($extras as $key => $val) {
                $rec[$key] = $val;
            }
            if (!empty($extras['EXT_PRODUCT_CODE']))
                $rec['IDNAME'] .= ' ' . $extras['EXT_PRODUCT_CODE'];
            if (!function_exists('setCreationData'))
                includeCodeLib('GEN_REC_UTIL');
            $rec = setCreationData($rec);
            $rec = setCreationIdCode($rec, 'FIAS');
            ClxCommon::$db->createRecord($this->fiasname, $rec, false);
            return $rec['IDCODE'];
        }

// End method
        /**
         * Validate FIAS
         */
        function validateFiasForm($record, &$msg) {
            getLookupValue('FIAS_PORTFOLIO_CODE', 'PORTFOLIO_CODE', $record);
            if (UserRequest::$request['ac'] === 'MODIFY') {
                $sql = 'SELECT count(JEHD.IDCODE) FROM ' . $this->fiasname . ' FIAS, ' . $this->jehdname . ' JEHD WHERE FIAS.PORTFOLIO_CODE = JEHD.PORTFOLIO_CODE AND FIAS.PORTFOLIO_CODE IS NOT null AND FIAS.PORTFOLIO_CODE <> ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' AND FIAS.CONTRACT_CODE = JEHD.CONTRACT_CODE AND FIAS.IDCODE = ' . singleQuote($record['FIAS_IDCODE']);
                if (ClxCommon::$db->getOne($sql) > CLX_NUM_ZERO) {
                    /*
                     * CODE_ERR_CANNOTCHGPORTFOLIO
                     * You cannot change the portfolio code when the balance amount is greater than zero
                     * No se puede cambiar el código de inversión si el saldo es mayor que cero
                     */
                    $msg['FIAS_PORTFOLIO_CODE'] = dict('CODE_ERR_CANNOTCHGPORTFOLIO');
                }
            }
            getLookupValue('FIAS_PV_ASSET_CODE', 'PV_ASSET_CODE', $record);
            if ($record['FIAS_TRUST_ITEM_TYPE'] <> 'PV' && !empty($record['FIAS_PV_ASSET_CODE'])) {//not a provision
                /*
                  CODE_ERR_ONLYPROVISIONS Only provisions requires to fill this field. Campo solo requerido para subcuenta de provisiones */
                $msg['FIAS_PV_ASSET_CODE'] = dict('CODE_ERR_ONLYPROVISIONS');
            }
            if ($record['FIAS_TRUST_ITEM_TYPE'] === 'PV' && empty($record['FIAS_PV_ASSET_CODE'])) {
                $msg['FIAS_PV_ASSET_CODE'] = dict('GEN_ERR_REQUIRED');
            }
            if (!empty($record['FIAS_PV_ASSET_CODE'])) {
                /*
                  CODE_ERR_PVNOTANASSET Only assets provisions are allowed. Please select another sub account. Solo provisiones para activos son permitidas. Favor seleccionar una subcuenta de activos */
                if (ClxCommon::$db->oneExistsRecords($this->fiasname, 'IDCODE = ' . singleQuote($record['FIAS_PV_ASSET_CODE']) . ' AND (TRUST_ITEM_TYPE = ' . singleQuote('PV') . ' OR GL_TYPE <> ' . singleQuote('AS') . ')') <> CLX_NUM_ZERO) {
                    $msg['FIAS_PV_ASSET_CODE'] = dict('CODE_ERR_PVNOTANASSET');
                }
            }
            if ($record['FIAS_TRUST_ITEM_TYPE'] === 'IN') {
                if (empty($record['FIAS_PORTFOLIO_CODE'])) {
                    /*
                     * CODE_ERR_INVALIDPORTFOLIO
                     * Invalid portfolio code, please verify
                     * Código de inversión invalido, favor verificar
                     */
                    $msg['FIAS_PORTFOLIO_CODE'] = dict('CODE_ERR_INVALIDPORTFOLIO');
                }
                $sql = 'SELECT CURRENCY_CODE FROM ' . $this->infuname . ' WHERE IDCODE = ' . singleQuote($record['FIAS_PORTFOLIO_CODE']);
                if (ClxCommon::$db->getOne($sql) <> $record['FIAS_CURRENCY_CODE']) {
                    /*
                     * CODE_ERR_PORTINVACURR
                     * Portfolio currency donot match asset currency please verify.
                     * Moneda de portafolio no coincide con activo de inversion.
                     */
                    $msg['FIAS_CURRENCY_CODE'] = dict('CODE_ERR_CASHINVACURR');
                }
            }//End if investment Asset
            if (UserRequest::$request['ac'] === 'CREATE' && !empty($record['FIAS_PORTFOLIO_CODE']) && ClxCommon::$db->oneExistsRecords($this->fiasname, 'PORTFOLIO_CODE = ' . singleQuote($record['FIAS_PORTFOLIO_CODE']) . ' AND CONTRACT_CODE = ' . singleQuote($record['FIAS_CONTRACT_CODE'])) <> CLX_NUM_ZERO) {
                /*
                 * CODE_ERR_ARASSETREQ
                 * Account Recievables asset required
                 * Activo de cuentas por cobrar requerido.
                 */
                $msg['FIAS_PORTFOLIO_CODE'] = dict('INAT_IMPORT_DUPLICATE_IDCODE', $record['FIAS_PORTFOLIO_CODE']);
            }
            if (!empty($record['FIAS_PORTFOLIO_CODE']) && $record['FIAS_TRUST_ITEM_TYPE'] <> 'IN') {
                /*
                 * CODE_ERR_EMPTYPORTREQ
                 * Portfolio code most be empty on any asset different than investment.
                 * Código de portafolio debe estar en blanco para cualquier otro activo diferente al de inversiones.
                 */
                $msg['FIAS_PORTFOLIO_CODE'] = dict('CODE_ERR_EMPTYPORTREQ');
            }
            return $msg;
        }

        /**
         * Contract Closing Process.
         *
         */
        function closeContract() {

        }

    }

}//not class exists
