<?php

/**
 * LA Sistemas Migration Module
 * For more information contact  support@alenet.com
 *
 * @author       $Author: fsantana $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2015-09-03 12:24:19 -0400 (Thu, 03 Sep 2015) $
 * @version    	$Rev: 6814 $
 */
// LIBRARY CODE: LAIN_MIGRA
class lain_migration {

    var $ar_parameters;
    var $s_import_def_code = 'LAIN_MIG';
    var $fromdate = '';
    var $todate = '';
    var $rif;
    var $cietclass;
    var $lain_reports_class;
    var $a_flatfile_headers;
    var $s_title;
    var $s_logfile_filename_init_string = 'lain_import';
    var $s_logfile_extension = '.htm';
    var $s_logfile_folder = 'export/';
    var $s_logfile_folder_prefix = '';
    var $i_reindex_rec_limit = CLX_NUM_THOUSAND;

    /*
     * Constructor
     * @param array
     * @return void
     */

    function __construct($parameters = array()) {
        $this->ar_parameters = $parameters;
        // Instance extended import methods & loads required classes
        if (!class_exists('clx_import_extended_tool'))
		  includeCodeLib('CIET');
        $this->cietclass = new clx_import_extended_tool($this->ar_parameters, $this->s_import_def_code);
        if (!class_exists('lain_reports'))
		  includeCodeLib('LAIN_REP');
        $this->lain_reports_class = new lain_reports();
        multiTableName($this, array('ACCO'));
    }

// Method end
    /*
     * Migration entry point
     * @param void
     * @return boolean
     */

    function run($filename = '') {
        // Reads the import definition header
        //$this->cietclass->read_import_header($this->s_import_def_code);
        // Verifies process engine parameters
        if (!$this->cietclass->o_proc_eng_params->verify_parameters()) {
		  echoLog('<h1>' . $this->cietclass->o_proc_eng_params->errormsg . '</h1>');
		  return false;
        }
        $this->cietclass->o_proc_eng_params->set_tablename($this->cietclass->a_import_header['TABLECODE']);
        // Reads the file name (if not specified as method's parameter, look for it at process parameter)
        if (trim($filename) === '') {
		  $filename = $this->cietclass->o_proc_eng_params->process_parameters['ANYFILE'];
        }
        $filename = trim($filename);
        //Setting dates
        $this->default_date();
        $this->s_title = '<br>' .
			 $this->cietclass->s_title . '<br>' .
			 dict('GEN_LBL_START_DATE') . ': ' . $this->fromdate .
			 '<br>' .
			 dict('GEN_LBL_END_DATE') . ': ' . $this->todate .
			 '<br>';
        echoLog($this->s_title);
        // Verifies the input file existance
        $a_report_file_existance_and_openit_response = $this->cietclass->report_file_existance_and_openit($filename);
        if (!empty($a_report_file_existance_and_openit_response['error'])) {
		  echoLog($a_report_file_existance_and_openit_response['errormsg']);
		  return false;
        }
        // Opens log file
        if (!$this->log_file_open($filename)) {
		  return false;
        }
        // Report table
        $this->log_file_write_wcr($this->s_title);
        $this->log_file_write_wcr($a_report_file_existance_and_openit_response['response']);
        $this->log_file_write($this->cietclass->report_init());
        // Report headers
        $this->log_file_write(CLX_CHAR_NL . '<tr>' .
			 $this->cietclass->s_record_data_to_show . '<td>IDNAME</td>' .
			 '<td>IDCODE</td>' .
			 '<td>CDATE</td>' .
			 '<td>IMPORT<BR>STATUS</td>' .
			 '</tr>' . CLX_CHAR_NL);
        // Starts reading flat file
        $i_imported_rec_counter = CLX_INIT_ZERO;
        $i_reindex_rec_counter = CLX_INIT_ZERO;
        while ($line = $this->cietclass->read_next_line()) {
		  // store current line
		  $record = $this->cietclass->read_line_into_record_array($line);
		  // Prepares reporting detail line
		  $i_imported_rec_counter++;
		  $this->cietclass->s_record_data_to_show = str_replace('%i_imported_rec_counter%', $i_imported_rec_counter, $this->cietclass->s_record_data_to_show);
		  // Record creation
		  $this->rif = $record['IDENTIFICATION'];
		  $this->lain_reports_class->connectToLa($this->rif, $this->fromdate, $this->todate);
		  //sleep($this->n_sleep_timeout);
		  ClxCommon::$db->commit();
		  // Verifies creation status....
		  $a_op_estatus = $this->record_creation_verification();
		  $this->cietclass->s_record_data_to_show = str_replace('%extra_info%', '<td>' . $a_op_estatus['IDNAME'] . '</td>' .
			     '<td>' . $a_op_estatus['IDCODE'] . '</td>' .
			     '<td>' . $a_op_estatus['CDATE'] . '</td>' .
			     '<td>' . $a_op_estatus['ERROR'] . '</td>', $this->cietclass->s_record_data_to_show);
		  // Reporting...
		  $this->log_file_write('<TR>' . $this->cietclass->s_record_data_to_show . '</TR>' . CLX_CHAR_NL);
		  // Reindex each 1000 record processed
		  if ($i_reindex_rec_counter++ > $this->i_reindex_rec_limit) {
			 $i_reindex_rec_counter = CLX_INIT_ZERO;
			 ClxCommon::$db->buldStats();
		  }
        }
        // Final optimization
        ClxCommon::$db->buldStats();
        // Finish and clean up
        $this->terminates_process();
        return true;
    }

// Method end
    /*
     * Setting dates
     * @param void
     * @return void
     */

    function default_date() {

        if (is_null($this->todate) || empty($this->todate)) {
		  $this->fromdate = trim($this->ar_parameters['SDATE']);
        }
        if (is_null($this->fromdate) || empty($this->fromdate)) {
		  $this->todate = trim($this->ar_parameters['EDATE']);
        }
        if (is_null($this->todate) || empty($this->todate)) {
		  $this->todate = date('Y-m-d');
        }
        $this->todate = date('Y-m-d', strtotime($this->todate));
        if (is_null($this->fromdate) || empty($this->fromdate)) {
		  $this->fromdate = substr($this->todate, CLX_NUM_ZERO, CLX_NUM_FOUR) . '-01-01';
        }
        $this->fromdate = date('Y-m-d', strtotime($this->fromdate));
    }

// Method end
    /*
     * Finish and clean up
     * @param void
     * @return string
     */

    function record_creation_verification() {
        $a_response = array('ERROR' => dict('GEN_LBL_ERROR'),
		  'IDCODE' => CLX_CHAR_HTML_NBSP,
		  'IDNAME' => CLX_CHAR_HTML_NBSP,
		  'CDATE' => CLX_CHAR_HTML_NBSP
        );
        /*
		Servicio: LA_CLIENTE
		Mensaje de consulta 	LA_CLIENTE_REQ Mensaje de Respuesta 	LA_CLIENTE_RES */
        $sql = 'SELECT  IDCODE,  IDNAME, CDATE FROM  ' . $this->acconame . '  WHERE  IDENTIFICATION = ' . singleQuote($this->rif) . '  AND LEGAL_TYPE = ' . singleQuote(substr($this->rif, CLX_NUM_ZERO, CLX_NUM_ONE));
        $o_records = ClxCommon::$db->executeQuery($sql);
        if ($a_record = ClxCommon::$db->getNextRecord($o_records)) {
		  $a_response['ERROR'] = dict('GEN_LBL_OK');
		  $a_response['IDCODE'] = $a_record['IDCODE'];
		  $a_response['IDNAME'] = $a_record['IDNAME'];
		  $a_response['CDATE'] = $a_record['CDATE'];
        }
		ClxCommon::$db->freeResult($o_records);
        unset($o_records);
        return $a_response;
    }

// Method end
    /*
     * Finish and clean up
     * @param void
     * @return void
     */

    function terminates_process() {
        $this->log_file_write($this->cietclass->report_ending());
        $this->cietclass->terminates_process();
        $this->log_file_close();
        $this->log_file_put_link();
        echoLog('');
    }

// Method end
    /*
     * Opens the error log flat text file
     * @param void
     * @return boolean
     */

    function log_file_open() {
        $this->s_log_filename = $this->log_file_filename_constructor();
        $this->o_logfile_mngr = new clx_logfile_mngr($this->s_logfile_folder_prefix . $this->s_logfile_folder . $this->s_log_filename);
        if ($this->o_logfile_mngr->error === true) {
		  $this->cietclass->proc_log_write($this->o_logfile_mngr->errorMsg);
		  return false;
        }
        return true;
    }

// Method End
    /**
     * Returns the log flat text file name
     * @param void
     * @return string
     */
    function log_file_filename_constructor() {
        $s_response = $this->s_logfile_filename_init_string . '_log_' . adodb_date('Ymd') . '_' . adodb_date('hms') . $this->s_logfile_extension;
        return $s_response;
    }

// Method End
    /**
     *  Write a line  with a CR+LF on log file
     * @param void
     * @return string
     */
    function log_file_write($msg) {
        $this->o_logfile_mngr->write($msg);
    }

// Method End

    function log_file_write_wcr($msg) {
        $this->log_file_write($msg . '<br>');
    }

// Method End
    /**
     *  Close log file
     * @param void
     * @return string
     */
    function log_file_close() {
        $this->o_logfile_mngr->close();
    }

// Method End
    /**
     * Put a link to the file
     * @param void
     * @return string
     */
    function log_file_put_link() {
        $filespec = $this->s_logfile_folder . $this->s_log_filename;
        echo $this->cietclass->o_proc_eng_params->get_file_link(array(array('FILENAME' => $filespec,
		  )
			 )
        );
    }

}

// End class