<?php

/**
 * Description of ceoc
 *
 * @author JuanCarlos
 */
class Ceoc {

    /**
     * Contructor
     *
     * @param array $procParameters
     * @return void
     */
    function __construct($procParameters = array()) {
        multiTableName($this, array('CEOC', 'CEEM', 'ACCO', 'AAGR', 'CEPR', 'FIAT', 'FIBK', 'FIAS', 'FICO', 'AAGI', 'AMPR'));
        $this->batch_id = (!empty($procParameters['BATCH_ID'])) ? $procParameters['BATCH_ID'] : batch_id();
        $this->now = xFormat::getNow('Y-m-d H:i:s');
    }

    function validate_form(&$record, &$msg) {
        if (UserRequest::$request['ac'] === 'CREATE' || UserRequest::$request['ac'] === 'MODIFY' || UserRequest::$request['ac'] === 'DUPLICATE') {
            if ($record['CEOC_CALC_AMT_01'] == CLX_NUM_ZERO && $record['CEOC_CALC_AMT_02'] == CLX_NUM_ZERO &&
                    $record['CEOC_CALC_AMT_03'] == CLX_NUM_ZERO && $record['CEOC_CALC_AMT_04'] == CLX_NUM_ZERO && $record['CEOC_CALC_AMT_05'] == CLX_NUM_ZERO && $record['CEOC_CALC_AMT_06'] == CLX_NUM_ZERO && $record['CEOC_CALC_AMT_07'] == CLX_NUM_ZERO && $record['CEOC_CALC_AMT_08'] == CLX_NUM_ZERO && $record['CEOC_CALC_AMT_09'] == CLX_NUM_ZERO) {
                $msg['CEOC_CALC_AMT_01'] = smsg('MSG-417');
            }
            $sql = 'SELECT AAGR.IDCODE FROM ' . $this->aagrname . ' AAGR WHERE AAGR.IDCODE = ' . singleQuote($record['CEOC_EXT_REF_NO']);
            $verifica = ClxCommon::$db->getOneRecord($sql);
            if (empty($verifica)) {
                $msg['CEOC_EXT_REF_NO'] = smsg('MSG-418', $record['CEOC_EXT_REF_NO']);
            }
            if ($record['CEOC_EFFTDATE'] > $record['CEOC_VALID_UNTIL_DATE']) {
                $msg['CEOC_EFFTDATE'] = smsg('MSG-419');
				}
        }
        if (UserRequest::$request['ac'] === 'DUPLICATE') {
            $record['CEOC_IDCODE'] = 'AUTO';
            $record['CEOC_STATUS_CEOC'] = 'PEND';
            $record['CEOC_CREATEDBY'] = '';
            $record['CEOC_CDATE'] = '';
            $record['CEOC_MODIFIEDBY'] = '';
            $record['CEOC_MDATE'] = '';
        }
    }

    function save_form_post(&$record) {
        $sql = 'SELECT FICO.TAXID, AAGR.CONTRACTEE_CODE, FICO.IDNAME FROM ' . $this->aagrname . ' AAGR, ' . $this->ficoname . ' FICO, ' . $this->ceocname . ' CEOC WHERE AAGR.IDCODE = ' . singleQuote($record['EXT_REF_NO']) . ' AND FICO.IDCODE = AAGR.CHART_FIELD01';
        $resultado = ClxCommon::$db->getOneRecord($sql);
        $sql = 'UPDATE ' . $this->ceocname . ' CEOC SET CEOC.IDENTIFICATION = ' . singleQuote($resultado['TAXID']) . ', CEOC.CONTRACTEE_CODE = ' . singleQuote($resultado['CONTRACTEE_CODE']) . ', CEOC.IDNAME = ' . singleQuote($resultado['IDNAME']) . ' WHERE CEOC.IDCODE = ' . singleQuote($record['IDCODE']);
        ClxCommon::$db->executeQuery($sql);
    }

    /**
     * Build transaction
     * @param: string comp_code
     * @param: string [efftdate]
     * @param: string [contract_code]
     * @param: string [participant_id]
     */
    function build_transactions(&$record, $operation_date = '', $ceoc_code = '', $currency_code = '', $exchange_rt = CLX_NUM_ZERO, $montoRecibido = CLX_NUM_ZERO, $display = true) {
        //Determine batch id
        $batch_id = batch_id();
        //If efftdate is not provided then search operation date day as operation date
        if (!class_exists('Comp'))
            includeCodeLib('COMP');
        $comp = new Comp();
        if ($operation_date === '')
            $operation_date = $comp->comp_operation_date($record['COMP_CODE']);
        $fiscal_period = $comp->comp_fiscal_period($record['COMP_CODE'], $operation_date);
        if (!empty($display)) {
            echoLog(' PROCESSING CONTRACT INTRUCTIONS CEOC. BATCH ID: ' . $batch_id);
            echoLog('Efective charge date : ' . $operation_date);
            echoLog('Company: ' . (($record['COMP_CODE'] === '') ? 'ALL' : $record['COMP_CODE']));
            echoLog('Contract: ' . (($record['CHART_FIELD01'] === '') ? 'ALL' : $record['CHART_FIELD01']));
            echoLog('Participant: ' . (($record['CHART_FIELD02'] === '') ? 'ALL' : $record['CHART_FIELD02']));
        }
        $sql = 'SELECT COUNT(CEOC.ID) COUNTER FROM ' . $this->ceocname . ' CEOC  WHERE CEOC.STATUS_CEOC = ' . singleQuote('PAG') . '';
        if ($ceoc_code <> '')
            $sql .= ' AND CEOC.IDCODE = ' . singleQuote($ceoc_code);
        if (!empty($display))
            echoLog('Total de transacciones a procesar: ' . ClxCommon::$db->getOne($sql));
        if (!class_exists('Gloq'))
            includeCodeLib('GLOQ');
        $gloq = new Gloq();
        if (!function_exists('fiat_build'))
            includeCodeLib('FIAT');
        $fiat = new Fiat();
        $sql = 'SELECT ACCO.IDNAME, ACCO.IDENTIFICATION FROM ' . $this->acconame . ' ACCO WHERE ACCO.IDCODE = ' . singleQuote($record['CHART_FIELD02']);
        $beneficiario = ClxCommon::$db->getOneRecord($sql);
        $sql = 'SELECT FIAS.CONTRACT_CODE, FIBK.BANK_ACCOUNT_NO, FIBK.CURRENCY_CODE FROM ' . $this->fibkname . ' FIBK, ' . $this->fiasname . ' FIAS WHERE FIAS.FIDU_BANKACT_CODE = FIBK.BANK_ACCOUNT_NO AND FIAS.CONTRACT_CODE = ' . singleQuote($record['CHART_FIELD01']) . ' AND FIBK.CURRENCY_CODE = ' . singleQuote($currency_code);
        $infoExtra = ClxCommon::$db->getOneRecord($sql);
        $sql = 'SELECT CEOC.*, CEPR.IDNAME AS CEPR_IDNAME, CEPR.CEOC_SERVICE_NO, CEPR.TRANSTBL_CODE, CEPR.APPROVE_PAYMENT FROM ' . $this->ceocname . ' CEOC, ' . $this->ceprname . ' CEPR WHERE CEOC.SERVICIO_CEOC = CEPR.IDCODE AND CEOC.STATUS_CEOC = ' . singleQuote('PAG') . '';
        if ($ceoc_code <> '')
            $sql .= ' AND CEOC.IDCODE = ' . singleQuote($ceoc_code);
        $results = ClxCommon::$db->executeQuery($sql);
        while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
            $trans_rec = array('COMP_CODE' => $record['COMP_CODE'],
                'IDNAME' => $result['CEPR_IDNAME'],
                'TRANSTBL_CODE' => $result['TRANSTBL_CODE'],
                'EFFTDATE' => $operation_date,
                'REAL_DATE' => $operation_date,
                'AVAILABILITY_DATE' => $operation_date,
                'TRANS_AMT' => $montoRecibido,
                'CURRENCY_CODE' => $currency_code,
                'FIDU_BANKACT_CODE' => '',
                'CHART_FIELD01' => $record['CHART_FIELD01'],
                'CHART_FIELD02' => $record['CHART_FIELD02'],
                'CHART_FIELD03' => $record['CHART_FIELD03'],
                'CHART_FIELD04' => $record['CHART_FIELD04'],
                'GUARANTEE_CODE' => $record['GUARANTOR_01'],
                'DISBURSEMENT_TYPE' => 'CA',
                'BENEFICIARY_CODE' => $record['CHART_FIELD02'],
                'BENEFICIARY_NAME' => $beneficiario['IDNAME'],
                'IDENTIFICATION' => $beneficiario['IDENTIFICATION'],
                'FIDU_BANKACT_CODE' => $infoExtra['BANK_ACCOUNT_NO'],
                'BANKAC_CODE' => $infoExtra['BANK_ACCOUNT_NO'],
                'FISCAL_PERIOD' => $fiscal_period,
                'COMMISSION_CODE' => 'null',
                'ITEM_REF_NO' => $result['EXT_REF_NO'],
                'BATCH_ID' => $batch_id,
                'EXCHANGE_RT' => $exchange_rt,
                'REFERENCE_NO' => $ceoc_code);
            $xrate = $comp->comp_current_exchange($trans_rec ['COMP_CODE'], $trans_rec ['CURRENCY_CODE'], $trans_rec ['EFFTDATE']);
            $trans_rec ['CUR_OPERATOR'] = $xrate ['CUR_OPERATOR'];
            $trans_code = $fiat->fiat_build($record['COMP_CODE'], $result['TRANSTBL_CODE'], $record['CHART_FIELD01'], $record['CHART_FIELD02'], $montoRecibido, $trans_rec);
            unset($trans_rec);
            if ($result['APPROVE_PAYMENT'] == CLX_NUM_ONE) {
                $gloq->pushFiatListing($trans_code, true);
                $sql = 'UPDATE ' . $this->ceocname . ' CEOC SET CEOC.STATUS_CEOC = IF(CEOC.STATUS_CEOC = ' . singleQuote('PAG') . ', ' . singleQuote('PAGP') . ', CEOC.STATUS_CEOC) WHERE CEOC.IDCODE = ' . singleQuote($ceoc_code);
                ClxCommon::$db->executeQuery($sql);
            }
        }
        $infoRetorna = array('TRANS_CODE' => $trans_code, 'BATCH_ID' => $batch_id);
        $sql = 'SELECT COUNT(ID) COUNTER FROM ' . $this->fiatname . ' WHERE BATCH_ID = ' . singleQuote($batch_id);
        if (!empty($display))
            echoLog('Total instruction completed: ' . ClxCommon::$db->getOne($sql));
        return $infoRetorna;
    }

    function getServicio($empresa, $servicio) {
        return ClxCommon::$db->getField($this->ceprname, 'IDCODE', 'CEOC_SERVICE_NO = ' . singleQuote($servicio) . ' AND EMP_CODE = ' . singleQuote($empresa));
    }

    function getServicioId($servicio) {
        return ClxCommon::$db->getField($this->ceprname, 'IDCODE', 'CEOC_SERVICE_NO = ' . singleQuote($servicio) . ' AND EMP_CODE = ' . singleQuote($empresa));
    }

    function checkExpiration() {
        $sql = 'SELECT AMPR.COMP_CODE, AMPR.RENT_DUE_PDAYS FROM ' . $this->amprname . ' AMPR';
        $comp_results = ClxCommon::$db->executeQuery($sql);
        while (($reg_comp = ClxCommon::$db->getNextRecord($comp_results))) {
            $sql = ' SELECT AAGR.IDCODE, AAGR.CLX_EFFTDATE EFFTDATE, AAGR.NEXT_RENT_DATE, PRAT.RENT2_GL_CODE GL01_CODE, PRAT.RENT_GL_CODE GL02_CODE, AAGR.AGREEMENT_AMT BALANCE_AMT, GLCP.COMP_CODE COMP_CODE, AAGR.CHART_FIELD01, AAGR.CHART_FIELD02, FIAE.CHART_FIELD03, null CHART_FIELD04, AAGR.CURRENCY_CODE, 1 CUR_OPERATOR, 1 EXCHANGE_RT, AAGR.AGREEMENT_AMT DEBIT_TRANS_AMT, 0 CREDIT_TRANS_AMT, AAGR.AGREEMENT_AMT DEBIT_AMT, 0 CREDIT_AMT, AAGR.IDCODE LOAN_CODE, ' . singleQuote('RENT') . ' ITEM_CODE, ' . singleQuote('RENT') . ' ITEM02_CODE FROM GL_PROD_PRMTR_TBL PRAT, AMGT_ASSETS FIAE, GL_PARAMETERS_TBL GLCP, AMGT_AGREEMENTS AAGR, FIDU_ITEMS FIAS, FIDU_PARTICIPANTS FIPA WHERE GLCP.COMP_CODE = AAGR.COMP_CODE AND FIPA.CONTRACT_CODE = AAGR.CHART_FIELD01 AND FIPA.ACCOUNTCODE = AAGR.CHART_FIELD02 AND FIPA.STATUS = ' . singleQuote('A') . ' AND PRAT.PRODUCT_CODE = FIAE.PRODUCT_CODE AND PRAT.PROD_STATUS = ' . singleQuote('VIG') . ' AND FIAE.IDCODE = AAGR.ASSETCODE AND FIAS.CONTRACT_CODE = AAGR.CHART_FIELD01 AND FIAS.ASSET_CODE = AAGR.ASSETCODE   AND AAGR.START_DATE < NOW() AND (AAGR.END_DATE >= NOW() OR AAGR.END_DATE IS NULL) AND GLCP.COMP_CODE = ' . singleQuote($reg_comp['COMP_CODE']) . ' AND AAGR.AGREEMNT_CLASS = ' . singleQuote('300') . ' AND AAGR.AGREEMENT_AMT <> 0 AND AAGR.NEXT_RENT_DATE = CURDATE() + ' . $reg_comp['RENT_DUE_PDAYS'] . ' AND FIAE.CLX_EFFTDATE = (SELECT MAX(FIAEX.CLX_EFFTDATE) FROM AMGT_ASSETS FIAEX WHERE FIAEX.IDCODE = FIAE.IDCODE AND FIAEX.CLX_EFFTDATE <= NOW()) AND AAGR.CLX_EFFTDATE = (SELECT MAX(AAGRX.CLX_EFFTDATE) FROM AMGT_AGREEMENTS AAGRX WHERE AAGRX.IDCODE = AAGR.IDCODE AND AAGRX.CLX_EFFTDATE<=NOW())';
            $results = ClxCommon::$db->executeQuery($sql);
            while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
                $sql = 'SELECT AAGI.SEQNUM, AAGI.AGREEMENT_AMT, AAGI.STATUS FROM ' . $this->aaginame . ' AAGI WHERE AAGI.AGREEMENT_CODE = ' . singleQuote($result['IDCODE']) . ' AND AAGI.EFFTDATE = ' . singleQuote($result['EFFTDATE']);
                $resultados = ClxCommon::$db->executeQuery($sql);
                $balance_con2 = CLX_INIT_ZERO;
                while (($resultado = ClxCommon::$db->getNextRecord($resultados))) {
                    if ($resultado['STATUS'] === 'A')
                        $balance_con2 = $balance_con2 + $resultado['AGREEMENT_AMT'];
                }
				ClxCommon::$db->freeResult($resultados);
                $balance_total = $result['BALANCE_AMT'] + $balance_con2;
                $record = array('IDCODE' => 'AUTO',
                    'IDNAME' => '',
                    'EXT_REF_NO' => $result['IDCODE'],
                    'SERVICIO_CEOC' => CLX_NUM_ONE,
                    'EFFTDATE' => $result['EFFTDATE'],
                    'VALID_UNTIL_DATE' => $result['NEXT_RENT_DATE'],
                    'CALC_AMT_01' => $result['BALANCE_AMT'],
                    'CALC_AMT_02' => $balance_con2,
                    'BALANCE_AMT' => $balance_total,
                    'CURRENCY_CODE' => $result['CURRENCY_CODE'],
                    'CURRENCY_CODE_CEOC' => $result['CURRENCY_CODE'],
                    'STATUS_CEOC' => 'PEND',
                    'CREATEDBY' => '',
                    'CDATE' => '',
                    'MODIFIEDBY' => '',
                    'MDATE' => '');
                if (!function_exists('setCreationData'))
                    includeCodeLib('GEN_REC_UTIL');
                setCreationIdCode($record, 'CEOC');
                setCreationData($record);
                $sql = 'INSERT INTO ' . $this->ceocname . '(IDCODE, IDNAME, EXT_REF_NO, SERVICIO_CEOC, EFFTDATE, VALID_UNTIL_DATE, CALC_AMT_01, CALC_AMT_02, BALANCE_AMT, CURRENCY_CODE, CURRENCY_CODE_CEOC, STATUS_CEOC, CREATEDBY, CDATE, MODIFIEDBY, MDATE) VALUES(' . singleQuote($record['IDCODE']) . ', ' . singleQuote($record['IDNAME']) . ', ' . singleQuote($record['EXT_REF_NO']) . ', ' . singleQuote($record['SERVICIO_CEOC']) . ', ' . singleQuote($record['EFFTDATE']) . ', ' . singleQuote($record['VALID_UNTIL_DATE']) . ', ' . singleQuote($record['CALC_AMT_01']) . ', ' . singleQuote($record['CALC_AMT_02']) . ', ' . singleQuote($record['BALANCE_AMT']) . ', ' . singleQuote($record['CURRENCY_CODE_CEOC']) . ', ' . singleQuote($record['CURRENCY_CODE']) . ', ' . singleQuote($record['STATUS_CEOC']) . ', ' . singleQuote($record['CREATEDBY']) . ', ' . singleQuote($record['CDATE']) . ', ' . singleQuote($record['MODIFIEDBY']) . ', ' . singleQuote($record['MDATE']) . ')';
                ClxCommon::$db->executeQuery($sql);
                $sql = 'SELECT FICO.TAXID, AAGR.CONTRACTEE_CODE, FICO.IDNAME FROM ' . $this->aagrname . ' AAGR, ' . $this->ficoname . ' FICO, ' . $this->ceocname . ' CEOC WHERE AAGR.IDCODE = ' . singleQuote($record['EXT_REF_NO']) . ' AND FICO.IDCODE = AAGR.CHART_FIELD01';
                $resultado = ClxCommon::$db->getOneRecord($sql);
                $sql = 'UPDATE ' . $this->ceocname . ' CEOC SET CEOC.IDENTIFICATION = ' . singleQuote($resultado['TAXID']) . ', CEOC.CONTRACTEE_CODE = ' . singleQuote($resultado['CONTRACTEE_CODE']) . ', CEOC.IDNAME = ' . singleQuote($resultado['IDNAME']) . ' WHERE CEOC.IDCODE = ' . singleQuote($record['IDCODE']);
                ClxCommon::$db->executeQuery($sql);
                unset($record);
            }
			ClxCommon::$db->freeResult($results);
        }
		ClxCommon::$db->freeResult($comp_results);
        return;
    }

    function validate_form_param(&$record, &$msg = '') {
        if (UserRequest::$request['ac'] === 'CREATE' || UserRequest::$request['ac'] === 'DUPLICATE') {
            if (is_numeric($record['CEPR_CEOC_SERVICE_NO'])) {
                $verifica = ClxCommon::$db->oneExistsRecords($this->ceprname, 'CEOC_SERVICE_NO = ' . $record['CEPR_CEOC_SERVICE_NO'] . ' AND EMP_CODE = ' . singleQuote($record['CEPR_EMP_CODE']));
                if ($verifica > CLX_NUM_ZERO) {
                    $msg['CEPR_CEOC_SERVICE_NO'] = smsg('MSG-445', $record['CEPR_EMP_CODE']);
                }
            }
        } elseif (UserRequest::$request['ac'] === 'MODIFY') {
            if (is_numeric($record['CEPR_CEOC_SERVICE_NO'])) {
                $verifica = ClxCommon::$db->oneExistsRecords($this->ceprname, 'CEOC_SERVICE_NO = ' . $record['CEPR_CEOC_SERVICE_NO'] . ' AND EMP_CODE = ' . singleQuote($record['CEPR_EMP_CODE']) . ' AND IDCODE <> ' . singleQuote($record['CEPR_IDCODE']));
                if ($verifica > CLX_NUM_ZERO) {
                    $msg['CEPR_CEOC_SERVICE_NO'] = smsg('MSG-445', $record['CEPR_EMP_CODE']);
                }
            }
        }
        if (UserRequest::$request['ac'] === 'DUPLICATE') {
            $record['CEPR_IDCODE'] = 'AUTO';
        }
    }

    function anular() {
        $idlist = toString(getIdlist());
        if (!function_exists('setModificationData'))
            includeCodeLib('GEN_REC_UTIL');
        $sql = 'SELECT CEOC.ID, CEOC.MODIFIEDBY, CEOC.MDATE FROM ' . $this->ceocname . ' CEOC WHERE CEOC.ID IN (' . $idlist . ')';
        $lista = ClxCommon::$db->executeQuery($sql);
        while (($record = ClxCommon::$db->getNextRecord($lista))) {
            setModificationData($record);
            $sql = 'UPDATE ' . $this->ceocname . ' CEOC SET CEOC.STATUS_CEOC = ' . singleQuote('ANU') . ', CEOC.MODIFIEDBY = ' . singleQuote($record['MODIFIEDBY']) . ', CEOC.MDATE = ' . singleQuote($record['MDATE']) . ' WHERE CEOC.ID = ' . $record['ID'];
            ClxCommon::$db->executeQuery($sql);
        }
    }

    function init_list_info(&$record) {
        if (UserRequest::$request['ac'] === 'DISPLAY') {
            $record_pag = unserialize($record['CEIN_INF_CEOC']);
            $aux = '';
            foreach ($record_pag as $key => $valor) {
                $aux .= dict('GEN_LBL_CEOC_' . $key) . ': ' . $valor . '<Br />';
            }
            $record['CEIN_INF_CEOC'] = $aux;
        }
    }

}
