<?php

if (!class_exists('Fils')) {

    /**
     * General Code File
     * For more information contact  support@alenet.com
     *
     * @author       $Author: fsantana $
     * @copyright    Alenet, Inc.
     * @since        $Date: 2016-03-09 10:19:31 -0500 (Wed, 09 Mar 2016) $
     * @version        $Rev: 7938 $
     *
     * CHANGE LOG
     *  Included svn prop
     */
    class Fils {

        /**
         * Contructor
         *
         * @param array $procParameters
         * @return void
         */
        function __construct($procParameters = array()) {
            multiTableName($this, array('FILO', 'INFA', 'FILS', 'COMP', 'GLCP', 'INDX', 'PRLT', 'PRAT', 'PROD', 'CRCL', 'CREB', 'PRLM'));
            $this->batch_id = (!empty($procParameters['BATCH_ID'])) ? $procParameters['BATCH_ID'] : batch_id();
            $this->username = $_SESSION['user_record']['username'];
            $this->now = xFormat::getNow('Y-m-d H:i:s');
            $this->je_source = '';
        }

        /**
         * Initialice fiat form - Executed on init event
         *
         * @param array $record
         */
        function init_form(&$record) {
            if (!class_exists('Comp')){
                includeCodeLib('COMP');
            }
            $filorec = ClxCommon::$db->getRecord($this->filoname, 'IDCODE = ' . singleQuote($record['FILS_LOAN_CODE']));
            $record['FILS_COMP_CODE'] = $filorec['COMP_CODE'];
            $record['FILS_DEBTOR_CODE'] = $filorec['DEBTOR_CODE'];
            $record['FILS_INTEREST_CALC'] = $filorec['INTEREST_CALC'];
        }

        /**
         * Validate record information
         * Executed on table validate
         *
         * @param array $record
         * @param array $msg
         * @return boolean
         */
        function validate_form(&$record, &$msg) {
            
            $idlists = (empty($record['idlist']))?array($record['FILS_ID']):explode(",", $record['idlist']);
            foreach($idlists as $idlist){
                 $sql = 'SELECT SEQNUM, LOAN_CODE, EFFTDATE, PERIOD_START_DATE, PERIOD_END_DATE, DUE_PAY_DATE FROM ' . $this->filsname . ' WHERE ID=' . singleQuote($idlist);
                $reg = ClxCommon::$db->getOneRecord($sql);
                $record['FILS_LOAN_CODE']=$reg['LOAN_CODE'];
                $record['FILS_EFFTDATE']=$reg['EFFTDATE'];
                $record['FILS_SEQNUM']=$reg['SEQNUM'];
                $record['FILS_PERIOD_START_DATE']=$reg['PERIOD_START_DATE'];
                $record['FILS_PERIOD_END_DATE']=$reg['PERIOD_END_DATE'];
                $record['FILS_DUE_PAY_DATE']=$reg['DUE_PAY_DATE'];

                $sql = 'SELECT FILO.CRED_PAY_METHOD, FILO.START_DATE FROM CRED_CREDITS FILO WHERE FILO.CLX_EFFTDATE = ' . singleQuote($record['FILS_EFFTDATE']) . ' AND FILO.IDCODE = ' . singleQuote($record['FILS_LOAN_CODE']);
                $prmts = ClxCommon::$db->getOneRecord($sql);
                //Determine date of previous installment
                $sql = 'SELECT IFNULL(MAX(PERIOD_END_DATE), ' . singleQuote($prmts['START_DATE']) . ') START_DATE FROM ' . $this->filsname . ' WHERE LOAN_CODE = ' . singleQuote($record['FILS_LOAN_CODE']) . ' AND SEQNUM < ' . $record['FILS_SEQNUM'];
                $dt = ClxCommon::$db->getOne($sql);
                //check start date
                if ($dt > $record['FILS_PERIOD_START_DATE']) {
                    $msg['FILS_PERIOD_START_DATE'] = smsg('MSG-255');
                }
                //check period end date
                if ($record['FILS_PERIOD_START_DATE'] >= $record['FILS_PERIOD_END_DATE']) {
                    $msg['FILS_PERIOD_END_DATE'] = smsg('MSG-256');
                }
                //checking payment date
                if ($record['FILS_PERIOD_END_DATE'] > $record['FILS_DUE_PAY_DATE']) {
                    $msg['FILS_DUE_PAY_DATE'] = smsg('MSG-257');
                }
            }
            return (count($msg) > CLX_NUM_ZERO) ? false : true;
        }

        /**
         * Update Parent) Information one record is saved
         */
        function save_form(&$record) {

        }

        /**
         * Update (FILO) after saving the record
         */
        function save_form_post(&$record) {
            $filorec = ClxCommon::$db->getRecord($this->filoname, 'IDCODE = ' . singleQuote($record['LOAN_CODE']) . ' AND CLX_EFFTDATE = ' . singleQuote($record['EFFTDATE']));
            //En caso de ser importacion entonces salir ya que viene de un excel
            if (UserRequest::$request['ac'] === 'IMPORT' || kDefaultIsCron == CLX_NUM_ONE) {
                return;
            }
            //Al modificar el cronograma
            //recalcular la tabla de amortizacion
            if ($filorec['CRED_PAY_METHOD'] <> CLX_NUM_7) {
                ClxCommon::$db->executeQuery('CALL CRED_SET_FILS(' . singleQuote($record['LOAN_CODE']) . ', ' . singleQuote($record['EFFTDATE']) . ',' . singleQuote($this->username) . ', 0, 0)');
                return;
            }

            $sql = 'SELECT MAX(INSTALLMENT_NO) QTY FROM ' . $this->filsname . '  WHERE LOAN_CODE = ' . singleQuote($record['LOAN_CODE']) . ' AND EFFTDATE = ' . singleQuote($record['EFFTDATE']);
            $nopay = ClxCommon::$db->getOne($sql);
            $rs = ClxCommon::$db->getRecords($this->filsname, 'LOAN_CODE = ' . singleQuote($record['LOAN_CODE']) . ' AND EFFTDATE = ' . singleQuote($record['EFFTDATE']) . ' ORDER BY SEQNUM ASC');
            $i = CLX_NUM_ONE;
            $p = $filorec['LOAN_AMT'];
            $intaccamt = CLX_INIT_ZERO;
            $rseintamt = CLX_INIT_ZERO;
            while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
                $r['NO_PAYMENTS'] = $nopay;
                $r['LOAN_AMT'] = no_format($p, CLX_NUM_TWO);
                $eq_pay = $r['PRINCIPAL_PAY'] + $r['BALLOON_PAY_AMT'] + $r['RESERVE_PAY'];

                if($r['LOAN_INTEREST']>0){
                    $binterest_rate = $r['LOAN_INTEREST'] * ($filorec['INTEREST_CALC'] / $r['INTEREST_CALC']);
                }else{
                    $binterest_rate = 0;
                }
                $r['INTEREST_PAY'] = round($r['LOAN_AMT']*(($binterest_rate/100)/360)*$r['NO_DAYS'], CLX_NUM_TWO);
                $r['INTEREST_PERIOD'] = $r['INTEREST_PAY'];
                if ($r['NO_DAYS'] == CLX_NUM_ZERO) {
                    $r['DAILY_INTEREST'] = CLX_INIT_ZERO;
                } else {
                    $r['DAILY_INTEREST'] = round($r['INTEREST_PAY'] / $r['NO_DAYS'], CLX_NUM_TWO);
                }
                $r['INTEREST_AMT'] = $intaccamt + $r['INTEREST_PAY'];
                $intaccamt = $r['INTEREST_AMT'];
                if ($r['RESERVE_INDEX'] <> 'N/A' && $r['RESERVE_PCT'] <> CLX_NUM_ZERO) {
                    $binterest_rate = $r['RESERVE_PCT'] * (CLX_360_DAYS / $r['INTEREST_CALC']);
                    $r['RESERVE_PAY'] = ROUND($r['LOAN_AMT'] * (($binterest_rate / CLX_NUM_HUNDRED) / $r['INTEREST_CALC']) * $r['NO_DAYS'], CLX_NUM_TWO);
                    $r['DAILY_RESERVE'] = ROUND($r['LOAN_AMT'] * (($binterest_rate / CLX_NUM_HUNDRED) / $r['INTEREST_CALC']), CLX_NUM_TWO);
                    $r['RESERVE_AMT'] = $rseintamt + $r['RESERVE_PAY'];
                    $rseintamt = $r['RESERVE_AMT'];
                } else {
                    $r['RESERVE_PAY'] = CLX_INIT_ZERO;
                    $r['DAILY_RESERVE'] = CLX_INIT_ZERO;
                    $r['RESERVE_AMT'] = CLX_INIT_ZERO;
                }
                $p -= $eq_pay;
                if ($nopay == $r['INSTALLMENT_NO'] && $p <> CLX_NUM_ZERO) {
                    $r['PRINCIPAL_PAY'] += $p;
                    $eq_pay += $p;
                    $p -= $p;
                }

                $r['PRINCIPAL_BAL'] = no_format($p);
                $r['PAYMENT_AMT'] = no_format($eq_pay + $r['INTEREST_PAY'] + $r['FEES_AMT']);
                ClxCommon::$db->updateRecord($this->filsname, $r, 'ID = ' . $r['ID']);
                $i++;
            }
			ClxCommon::$db->freeResult($rs);
            //update no of payments if manual case
            if (!empty($nopay) && $filorec['CRED_PAY_METHOD'] == CLX_NUM_7) {
                $sql = 'SELECT MAX(PERIOD_END_DATE) FROM ' . $this->filsname . '  WHERE LOAN_CODE = ' . singleQuote($record['LOAN_CODE']) . ' AND EFFTDATE = ' . singleQuote($record['EFFTDATE']);
                $end_date = ClxCommon::$db->getOne($sql);

                //Updating filo information
                $sql = 'UPDATE ' . $this->filoname . '  SET NO_PAYMENTS = ' . $nopay . ', END_DATE = ' . singleQuote($end_date) . ' WHERE IDCODE = ' . singleQuote($record['LOAN_CODE']) . ' AND CLX_EFFTDATE = ' . singleQuote($record['EFFTDATE']);
                ClxCommon::$db->executeQuery($sql);
            }
        }

    }

}