<?php

/**
 * General Code File
 * For more information contact  support@alenet.com
 *
 * @author       $Author: fsantana $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2015-10-12 07:55:36 -0400 (Mon, 12 Oct 2015) $
 * @version        $Rev: 7215 $
 *
 * CHANGE LOG
 *  Included svn prop
 */
class Fimt {

    /**
     * Contructor
     *
     * @param array $procParameters
     * @return void
     */
    function __construct($procParameters = array()) {
        multiTableName($this, array('FITB', 'FITS', 'FIBK', 'COMP', 'FICO', 'FIPD', 'FIPA', 'FTMP', 'TTPA', 'JEIT', 'JEHD', 'JELN', 'FIRT', 'FIAC', 'FIMT', 'FETR', 'FIAT', 'FIRS', 'ACCO', 'GLCP', 'JACP', 'GLAC', 'JCBV'));
        $this->batch_id = (!empty($procParameters['BATCH_ID'])) ? $procParameters['BATCH_ID'] : batch_id();
        $this->username = $_SESSION['user_record']['username'];
        $this->now = xFormat::getNow('Y-m-d H:i:s');
    }

    /**
     * Initialice fimt form
     * Executed on init event
     */
    function fimt_init_form(&$record) {
        if (!class_exists('Comp'))
            includeCodeLib('COMP');
        $comp = new Comp();
        $comprec = $comp->comp_operation_parameters($record['FIMT_COMP_CODE']);
        if (UserRequest::$request['ac'] === 'CREATE' || UserRequest::$request['ac'] === 'DUPLICATE') {

            $record['FIMT_CLX_AUTH_STATUS'] = 'PEND';
            $record['FIMT_TRANS_STATUS'] = 'PEND';
            $record['FIMT_VOID'] = '0';
            if (UserRequest::$request['ac'] === 'DUPLICATE')
                $record['FIMT_IDCODE'] = 'AUTO';
            $record['FIMT_EFFTDATE'] = $comprec['OPERATION_DATE'];
            $record['FIMT_AVAILABILITY_DATE'] = $record['FIMT_EFFTDATE'];
            $record['FIMT_FISCAL_PERIOD'] = $comprec['FISCAL_PERIOD'];
            $record['FIMT_LGGROUP_CODE'] = $comprec['LGGROUP_CODE'];
        }
        if (UserRequest::$request['ac'] === 'CREATE') {
            $record['FIMT_REQ_CHANNEL'] = 'OFC';
            $record['FIMT_CURRENCY_CODE'] = $comprec['CURRENCY_CODE'];
            $record['FIMT_COMP_CODE'] = $comprec['IDCODE'];
            $record['FIMT_BRANCH_PRN_prim'] = companyParameter('BRANCH_PRN', $comprec['IDCODE']);
            $record['FIMT_FIDU_BANKACT_CODE_prim'] = companyParameter('FIDU_BANK_ACCOUNT', $comprec['IDCODE']);
        }
        unset($comp);
    }

    /**
     * Validate record information
     * Executed on table validate
     */
    function fimt_validate_form(&$record, &$msg) {
        //validar situacion de transaccion pendiente o no
        if ($record['FIMT_CLX_AUTH_STATUS'] <> 'PEND') {
            $msg['FIMT_TRANS_STATUS'] = smsg('MSG-24', $record['FIMT_IDCODE']);
            if (UserRequest::$request['ac'] === 'DELETE')
                addMessage($msg['FIMT_TRANS_STATUS']);
            return false;
        }
        if (UserRequest::$request['ac'] === 'DELETE')
            return true;
        if (!class_exists('Jehd'))
            includeCodeLib('JEHD');
        if (!$this->fimt_is_editable($record['FIMT_IDCODE'])) {
            $msg['FIMT_IDCODE'] = smsg('MSG-24', $record['FIMT_IDCODE']);
        }
        //Validar que la fecha de la operacion sea menor o igual a la contabilidad +1 dia
        if (!class_exists('Comp'))
            includeCodeLib('COMP');
        $comp = new Comp();
        $comprec = $comp->comp_operation_parameters($record['FIMT_COMP_CODE']);
        unset($comp);
        //Validar que la fecha de la operacion sea menor o igual a la contabilidad +1 dia
        $operation_date = $comprec['OPERATION_DATE'];
        $bckdtdop = $comprec['BCKDATE_ALLWCE'];
        $record['FIMT_EFFTDATE'] = xFormat::anytimedate($record['FIMT_EFFTDATE'], 'Y-m-d');
        if (empty($bckdtdop))
		$bckdtdop = CLX_INIT_ZERO;
        if (day_diff($record['FIMT_EFFTDATE'], $operation_date) < $bckdtdop) {
            $msg['FIMT_EFFTDATE'] = smsg('MSG-25', $bckdtdop, $record['FIMT_IDCODE']);
        }
        //validar situacion de transaccion pendiente o no
        if ($record['FIMT_IDCODE'] <> 'AUTO' && !$this->fimt_is_editable($record['FIMT_IDCODE'])) {

            $msg['FIMT_TRANS_STATUS'] = smsg('MSG-25', $bckdtdop, $record['FIMT_IDCODE']);
        }
        //Si existe retiro de efectivo verificar que exista la cuenta de banco del fideicomiso
        if (ClxCommon::$db->oneExistsRecords($this->fitsname, 'TRANSTBL_CODE = ' . singleQuote($record['FIMT_TRANSTBL_CODE']) . ' AND AFFECT_TYPE = ' . singleQuote('CA') . '') > CLX_NUM_ZERO && empty($record['FIMT_FIDU_BANKACT_CODE'])) {
            $msg['FIMT_FIDU_BANKACT_CODE'] = smsg('MSG-26', $record['FIMT_IDCODE']);
        }
        //Test fi bank account has gl
        if (!empty($record['FIMT_FIDU_BANKACT_CODE'])) {
            if (ClxCommon::$db->getField($this->fibkname, 'GLACCOUNTCODE', 'BANK_ACCOUNT_NO =' . singleQuote($record['FIMT_FIDU_BANKACT_CODE'])) === '') {
                $msg['FIMT_FIDU_BANKACT_CODE'] = smsg('MSG-27', $record['FIMT_IDCODE']);
            }
        }
        //validando fechas del contrato
        $ficorec = ClxCommon::$db->getFields($this->ficoname, 'COMP_CODE, START_DATE, EXPIRATION_DATE, CANCEL_DATE', 'IDCODE = ' . singleQuote($record['FIMT_CHART_FIELD01']));
        //Contract Start Date
        if (strtotime($ficorec['START_DATE']) > strtotime($record['FIMT_EFFTDATE'])) {
            $msg['FIMT_EFFTDATE'] = smsg('MSG-28', $record['FIMT_IDCODE']);
        }
        //Contract Expiration date
        if (!empty($ficorec['EXPIRATION_DATE']) && strtotime($ficorec['EXPIRATION_DATE']) < strtotime($record['FIMT_EFFTDATE'])) {
            $msg['FIMT_EFFTDATE'] = smsg('MSG-29', $record['FIMT_IDCODE']);
        }
        //Contract Cancel date
        if (!empty($ficorec['CANCEL_DATE']) && strtotime(adodb_next_day($ficorec['CANCEL_DATE'], 'Y-m-d')) < strtotime($record['FIMT_EFFTDATE'])) {
            $msg['FIMT_EFFTDATE'] = smsg('MSG-30', $record['FIMT_IDCODE']);
        }
        //Validar que contrato vs compania
        if ($ficorec['COMP_CODE'] <> $record['FIMT_COMP_CODE']) {
            $msg['FIMT_EFFTDATE'] = smsg('MSG-31', $record['FIMT_IDCODE']);
        }
        //Validar participante este en ese contrato
        $sql = 'SELECT COUNT(FIPA.ID) ' . 'FROM ' . $this->fipaname . ' FIPA, ' . $this->ficoname . ' FICO ' . ' WHERE (FICO.IDCODE = FIPA.CONTRACT_CODE OR FICO.PARENTCONTRACT_CODE = FIPA.CONTRACT_CODE) ' . 'AND FIPA.CONTRACT_CODE = ' . singleQuote($record['FIMT_CHART_FIELD01']) . ' AND STATUS=' . singleQuote('A') . '';
        if (ClxCommon::$db->getOne($sql) == CLX_NUM_ZERO) {
            $msg['FIMT_CHART_FIELD01'] = smsg('MSG-32', '', $record['FIMT_CHART_FIELD01']);
        }
        //Validar que la fecha de disponible sea mayor a la fecha efectiva FIMT_AVAILABILITY_DATE
        if (adodb_strtotime($record['FIMT_AVAILABILITY_DATE']) < adodb_strtotime($record['FIMT_EFFTDATE'])) {
            $msg['FIMT_AVAILABILITY_DATE'] = smsg('MSG-33', $record['FIMT_IDCODE']);
        }
        //not cero
        if ($record['FIMT_TRANS_AMT'] == CLX_NUM_ZERO) {
            $msg['FIMT_TRANS_AMT'] = smsg('MSG-34', $record['FIMT_IDCODE']);
        }
        //Si tipo de desembolso validar que exista beneficiario
        //FIMT_BENEFICIARY_CODE FIMT_BENEFICIARY_NAME
        getLookupValue('FIMT_BENEFICIARY_CODE', 'BENEFICIARY_CODE', $record); //beneficiario
        $is_disbursement = (ClxCommon::$db->oneExistsRecords($this->fitsname, 'TRANSTBL_CODE = ' . singleQuote($record['FIMT_TRANSTBL_CODE']) . ' AND AFFECT_TYPE = ' . singleQuote('CA') . ' AND DEBIT_CREDIT=' . "'CR'" . '') > CLX_NUM_ZERO);
        if ($is_disbursement === true && (empty($record['FIMT_BENEFICIARY_CODE']) && empty($record['FIMT_BENEFICIARY_NAME']))) {
            $msg['FIMT_BENEFICIARY_CODE'] = smsg('MSG-35', $record['FIMT_IDCODE']);
        }
        //Si hay beneficiario que exista la cuenta de banco
        if ($is_disbursement === true && empty($record['FIMT_DISBURSEMENT_TYPE'])) {
            $msg['FIMT_DISBURSEMENT_TYPE'] = smsg('MSG-36', $record['FIMT_IDCODE']);
        }
        //Si el desembolso es de tipo cheque solicitar BRANCH_PRN
        getLookupValue('FIMT_BRANCH_PRN', 'BRANCH_PRN', $record); //beneficiario
        if ($record['FIMT_DISBURSEMENT_TYPE'] === 'CH' && empty($record['FIMT_BRANCH_PRN'])) {
            $msg['FIMT_BRANCH_PRN'] = smsg('MSG-37', $record['FIMT_IDCODE']);
        }
        //Si es cheque solicitar el numero de cheque
        if ($record['FIMT_DISBURSEMENT_TYPE'] === 'CH' && empty($record['FIMT_CHECK_NO'])) {
            $msg['FIMT_CHECK_NO'] = smsg('MSG-38', $record['FIMT_IDCODE']);
        }
        //Validar que el tipo de transaccion
        $sql = 'SELECT COUNT(FITB.ID) FROM ' . $this->fitbname . ' FITB, ' . $this->ficoname . ' FICO WHERE (FITB.FI_TYPE = FICO.FI_TYPE OR FITB.FI_TYPE IS NULL) AND FITB.IDCODE = ' . singleQuote($record['FIMT_TRANSTBL_CODE']) . ' AND FICO.IDCODE = ' . singleQuote($record['FIMT_CHART_FIELD01']);
        if (ClxCommon::$db->getOne($sql) == CLX_NUM_ZERO) {
            $msg['FIMT_TRANSTBL_CODE'] = smsg('MSG-39', $record['FIMT_IDCODE']);
        }
        //Validar el estatus del asiento que no este aprobado
        if (ClxCommon::$db->oneExistsRecords($this->jehdname, 'TRANS_CODE = ' . singleQuote($record['FIMT_IDCODE']) . ' AND JE_STATUS = ' . singleQuote('AP') . '') > CLX_NUM_ZERO) {
            $msg['FIMT_TRANS_STATUS'] = smsg('MSG-24', $record['FIMT_IDCODE']);
        }
        //Validar que el periodo contable coincida con fecha de transaccion
        $sql = 'SELECT COUNT(JACP.ID) QTY FROM ' . $this->ficoname . ' FICO, ' . $this->jacpname.' JACP WHERE JACP.COMP_CODE = FICO.COMP_CODE AND FICO.IDCODE = ' . singleQuote($record['FIMT_CHART_FIELD01']) . ' AND JACP.FROM_DATE <= ' . singleQuote($record['FIMT_EFFTDATE']) . ' AND JACP.TO_DATE >= ' . singleQuote($record['FIMT_EFFTDATE']) . ' AND JACP.IDCODE = ' . singleQuote($record['FIMT_FISCAL_PERIOD']);
        if (ClxCommon::$db->getOne($sql) == CLX_NUM_ZERO) {
            $msg['FIMT_FISCAL_PERIOD'] = smsg('MSG-68', $record['FIMT_IDCODE']);
        }
        //Authorization methods perform all journal validations
        if (UserRequest::$request['ac'] === 'REQAUTHORIZATION') {
            if ($record['FIMT_EFFTDATE'] > $operation_date) {
                $msg['FIMT_EFFTDATE'] = smsg('MSG-40', $record['FIMT_IDCODE']);
            }
            $sql = 'SELECT COUNT(FIPA.ID) FROM ' . $this->fipaname . ' FIPA, ' . $this->fimtname.' FIMT, ' . $this->fipdname.' FIPD WHERE FIPD.CHART_FIELD02 = FIPA.ACCOUNTCODE AND FIPA.CONTRACT_CODE = FIMT.CHART_FIELD01 AND FIMT.IDCODE = FIPD.MASS_TRANS_CODE AND FIMT.MASS_TRANS_PART = ' . singleQuote('0') . ' AND	FIMT.IDCODE = ' . singleQuote($record['FIMT_IDCODE']) . ' AND (FIPA.STATUS = ' . singleQuote('I') . '  OR FIPA.CLX_AUTH_STATUS = ' . singleQuote('PEND') . ')';
            if (ClxCommon::$db->getOne($sql) > CLX_NUM_ZERO) {
                $msg['FIMT_CHART_FIELD02'] = smsg('MSG-32', '', $record['FIMT_CHART_FIELD01']);
            }
            $sql = 'SELECT COUNT(FIPA.ID) FROM ' . $this->fipaname . ' FIPA, ' . $this->fimtname.' FIMT, ' . $this->fipdname.' FIPD WHERE FIPD.CHART_FIELD02 = FIPA.ACCOUNTCODE AND FIPA.CONTRACT_CODE = FIMT.CHART_FIELD01 AND FIMT.IDCODE = FIPD.MASS_TRANS_CODE AND FIMT.MASS_TRANS_PART = ' . singleQuote('0') . ' AND	FIMT.IDCODE = ' . singleQuote($record['FIMT_IDCODE']) - ' ';
            if ($record['FIMT_MASS_TRANS_PART'] === '0' && ClxCommon::$db->getOne($sql) == CLX_NUM_ZERO) {
                $msg['FIMT_CHART_FIELD02'] = smsg('MSG-32', '', $record['FIMT_CHART_FIELD01']);
            }
            return (count($msg) > CLX_NUM_ZERO) ? false : true;
        }
    }

    /**
     * Update Parent (FIMT) Information one record is saved
     */
    function fimt_save_form(&$record) {

    }

    // Reversal functions
    /**
     * Execute bottom demand from listing jehd
     */
    function fimt_reverse_build() {
        $idlist = toString(getIdlist());
        $results = ClxCommon::$db->executeQuery('SELECT IDCODE FROM ' . $this->fimtname . ' FIMT WHERE ID IN (' . $idlist . ')');
        while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
            ClxFreeReturn($this->fimt_reverse($result['IDCODE']));
        }
		ClxCommon::$db->freeResult($results);
    }

    /**
     * Reverses transaction
     */
    function fimt_reverse($trans_code) {
        $new_trans_code = '';
        //Validating if not reverse of reverse
        if (!$this->fimt_reverse_validate($trans_code))
            return false;
        //Creating reverse operation.
        if (!function_exists('setCreationData'))
            includeCodeLib('GEN_REC_UTIL');
        $revtrans = ClxCommon::$db->getRecord($this->fimtname, 'IDCODE = ' . singleQuote($trans_code));
        if (empty($revtrans))
            return false;
        if (!class_exists('Fico'))
            includeCodeLib('FICO');
        //Validar que la fecha de la operacion
        //sea menor o igual a la contabilidad +1 dia
        unset($revtrans['ID'], $revtrans['EFFTDATE']);
        $fico = new Fico();
        $comprec = $fico->fico_comp_operation_parameters($revtrans['CHART_FIELD01']);
        unset($fico);
        $revtrans['EFFTDATE'] = $comprec['OPERATION_DATE'];
        $revtrans['AVAILABILITY_DATE'] = $comprec['OPERATION_DATE'];
        $revtrans['TRANS_STATUS'] = 'PEND';
        $revtrans['CLX_AUTH_STATUS'] = 'PEND';
        $revtrans['REV_TRANS_CODE'] = $trans_code;
        $revtrans['IDCODE'] = 'AUTO';
        //$revtrans['TRANS_AMT'] = rounder(bcmul('-1',$revtrans['TRANS_AMT']), 2);
        setCreationData($revtrans);
        setCreationIdCode($revtrans, 'FIMT');
        ClxCommon::$db->createRecord($this->fimtname, $revtrans, false);
        $new_trans_code = $revtrans['IDCODE'];
        unset($revtrans);
        return $new_trans_code;
    }

    function fimt_reverse_by_je_source($comp_code, $je_source, $operation_date) {
        //Reject any posible todays entry from this process thas has run today
        $sql = 'UPDATE ' . $this->fimtname . ' SET TRANS_STATUS = ' . singleQuote('REJ') . ', CLX_AUTH_STATUS = ' . singleQuote('REJ') . ' WHERE JE_SOURCE = ' . singleQuote($je_source) . ' AND COMP_CODE = ' . singleQuote($comp_code) . ' AND EFFTDATE >= ' . singleQuote($operation_date) . ' AND TRANS_STATUS =' . singleQuote('PEND') . '';
        ClxCommon::$db->executeQuery($sql);
        //reverse any left
        $sql = 'SELECT DISTINCT FIMT.IDCODE FROM ' . $this->fimtname . ' FIMT, ' . $this->jehdname . ' JEHD WHERE FIMT.IDCODE = JEHD.TRANS_CODE AND FIMT.JE_SOURCE = ' . singleQuote($je_source) . ' AND FIMT.COMP_CODE = ' . singleQuote($comp_code) . ' AND FIMT.EFFTDATE = ' . singleQuote($operation_date) . ' AND FIMT.TRANS_STATUS = ' . singleQuote('AP') . ' AND FIMT.REVERSED = 0';
        $results = ClxCommon::$db->executeQuery($sql);
        while (($result = ClxCommon::$db->getNextRecord($results, NUMERIC))) {
            $rev_trans_code = $this->fimt_reverse($result[CLX_NUM_ZERO]);
            echoLog(dict('GEN_LBL_REVERSING') . ' FIMT: ' . $result[CLX_NUM_ZERO] . '->' . $rev_trans_code);
            ClxFreeReturn($this->fimt_approve_by_transcode($rev_trans_code));
        }
		ClxCommon::$db->freeResult($results);
        unset($rev_jehdr_code);
    }

    /**
     * Reversal validations
     */
    function fimt_reverse_validate($trans_code) {
        $record = ClxCommon::$db->getRecord($this->fimtname, 'IDCODE = ' . singleQuote($trans_code) . ' AND TRANS_STATUS = ' . singleQuote('AP') . '');
        $status = true;
        if (empty($record)) {
            addMessage(smsg('MSG-42', $trans_code));
            return false;
        }
        if (!empty($record['IDCODE']) && ClxCommon::$db->oneExistsRecords($this->fimtname, 'IDCODE = ' . singleQuote($trans_code) . ' AND (VOID=1 OR RECORD_BLOCKING<>0)') > CLX_NUM_ZERO) {
            addMessage(smsg('MSG-43', $trans_code));
            return false;
        }
        $sql = 'SELECT COUNT(ID) FROM ' . $this->jehdname . '  WHERE TRANS_CODE = ' . singleQuote($trans_code) . ' AND REV_JEHDR_CODE IS NOT null AND JE_STATUS IN(' . singleQuote('AP') . ',' . singleQuote('PEND') . ') ';
        if (ClxCommon::$db->getOne($sql)) {
            addMessage(smsg('MSG-44', $trans_code));
            return false;
        }
        return $status;
    }

    /**
     * Test if transaction can be modified
     */
    function fimt_is_editable($idcode) {
        return true;
    }

    /**
     * Builds fimt Record
     * @return: String idcode
     * @param: String comp_code
     * @param: String transtbl_code
     * @param: String contract_code
     * @param: String participant_id
     * @param: String trans_amt
     * @param: Array record
     */
    function fimt_build($comp_code, $transtbl_code, $participation_type, $contract_code, $trans_amt, &$record) {
        if (empty($record['IDCODE'])) {
            $record['IDCODE'] = 'AUTO';
        }
        if (empty($record['CLX_AUTH_STATUS'])) {
            $record['CLX_AUTH_STATUS'] = 'PEND';
            $record['TRANS_STATUS'] = 'PEND';
        }
        if (!class_exists('Comp'))
            includeCodeLib('COMP');
        $comp = new Comp();
        $comprec = $comp->comp_operation_parameters($comp_code);
        //Validar que la fecha de la operacion sea menor o igual a la contabilidad +1 dia
        $record['COMP_CODE'] = $comp_code;
        if (!array_key_exists('EFFTDATE', $record))
            $record['EFFTDATE'] = $comprec['OPERATION_DATE'];
        if (!array_key_exists('CURRENCY_CODE', $record))
            $record['CURRENCY_CODE'] = $comprec['CURRENCY_CODE'];
        if (empty($record['REQ_CHANNEL']))
            $record['REQ_CHANNEL'] = 'SYS';
        if (empty($record['FISCAL_PERIOD'])) {
            $record['FISCAL_PERIOD'] = $comp->comp_fiscal_period($comp_code, $record['EFFTDATE']);
        }
        $record['CHART_FIELD01'] = $contract_code;
        $record['TRANSTBL_CODE'] = $transtbl_code;
        $record['TRANS_AMT'] = $trans_amt;
        $record['MASS_TRANS_PART'] = $participation_type;
        $record['CREATEDBY'] = '';
        $record['MODIFIEDBY'] = '';
        $record['CDATE'] = '';
        $record['MDATE'] = '';
        if (!function_exists('setCreationData'))
            includeCodeLib('GEN_REC_UTIL');
        setCreationData($record);
        setCreationIdCode($record, 'FIMT');
        $record['TRANS_STATUS'] = 'PEND';
        $record['CLX_AUTH_STATUS'] = 'PEND';
        if (empty($record['CHART_FIELD01']))
            $record['CHART_FIELD01'] = 'null';
        if (empty($record['CHART_FIELD03']))
            $record['CHART_FIELD03'] = 'null';
        if (empty($record['CHART_FIELD04']))
            $record['CHART_FIELD04'] = 'null';
        ClxCommon::$db->createRecord($this->fimtname, $record, false);
        return $record['IDCODE'];
    }

    /**
     * Test is transaction is restricted
     *
     * @param String $trans_code
     */
    function fimt_is_restricted($trans_code) {
        $sql = 'SELECT COUNT(FITB.ID) FROM ' . $this->fimtname . ' FIMT, ' . $this->fitbname . ' FITB WHERE FIMT.TRANSTBL_CODE = FITB.IDCODE AND FITB.RESTRICTED = 1 AND FIMT.IDCODE = ' . singleQuote($trans_code);
        return (ClxCommon::$db->getOne($sql) > CLX_NUM_ZERO) ? true : false;
    }

    /**
     * Update transactions elements that could be updated
     *
     * @param Array $record
     */
    function fimt_update_restricted(&$record) {
        $trans_code = $record['IDCODE'];
        $jehdr_code = ClxCommon::$db->getField($this->jehdname, 'IDCODE', 'TRANS_CODE = ' . singleQuote($trans_code) . ' AND JE_STATUS = ' . singleQuote('PEND') . '');
        if (empty($jehdr_code))
            return false;
        $updateRec = array();
        $sql = 'SELECT DISTINCT A.FIELDNAME FROM CLX_DB_TABLEFIELDS A, CLX_DB_TABLEFIELDS B WHERE A.FIELDNAME = B.FIELDNAME AND A.TABLECODE = ' . singleQuote('FIMT') . ' AND B.TABLECODE = ' . singleQuote('JEHD') . ' AND A.FIELDNAME NOT IN (' . singleQuoteArray(array('BATCH_ID', 'CDATE', 'CLX_AUTH_STATUS', 'CREATEDBY', 'ID', 'IDCODE', 'REVERSED', 'TRANSTBL_CODE')) . ')';
        $flds = ClxCommon::$db->executeQuery($sql);
        while (($fld = ClxCommon::$db->getNextRecord($flds, NUMERIC)) === true) {
            if (array_key_exists($fld[CLX_NUM_ZERO], $record))
                $updateRec[$fld[CLX_NUM_ZERO]] = $record[$fld[CLX_NUM_ZERO]];
        }
		ClxCommon::$db->freeResult($flds);
        if (!empty($updateRec)) {
            ClxCommon::$db->updateRecord($this->jehdname, $updateRec, 'IDCODE = ' . singleQuote($jehdr_code));
        }
        //Updating lines
        $updateRec = array();
        $sql = 'SELECT DISTINCT A.FIELDNAME FROM CLX_DB_TABLEFIELDS A, CLX_DB_TABLEFIELDS B WHERE A.FIELDNAME = B.FIELDNAME AND A.TABLECODE = ' . singleQuote('FIMT') . ' AND B.TABLECODE = ' . singleQuote('JELN') . ' AND A.FIELDNAME NOT IN (' . singleQuoteArray(array('ID', 'ITEM_REF_NO', 'JEHDR_CODE', 'RECONCILIED')) . ')';
        $flds = ClxCommon::$db->executeQuery($sql);
        while (($fld = ClxCommon::$db->getNextRecord($flds, NUMERIC)) === true) {
            if (array_key_exists($fld[CLX_NUM_ZERO], $record))
                $updateRec[$fld[CLX_NUM_ZERO]] = $record[$fld[CLX_NUM_ZERO]];
        }
		ClxCommon::$db->freeResult($flds);
        if (!empty($updateRec)) {
            ClxCommon::$db->updateRecord($this->jelnname, $updateRec, 'JEHDR_CODE = ' . singleQuote($jehdr_code));
        }
        return true;
    }

    /**
     * Approve transaction by transaction code
     * @param: string transaction code
     * @return: approval status
     */
    function fimt_approve_by_transcode($trans_code) {
        //Search mass transaction details
        $transrec = ClxCommon::$db->getRecord($this->fimtname, 'IDCODE = ' . singleQuote($trans_code));
        //Load temporary table - Req on mysql
        $DBTB = getTool('DBTB');
        $DBTB->buildTemporaryTable('FTMP');
        //Clear temporary table
        ClxCommon::$db->executeQuery('DELETE FROM ' . $this->ftmpname . '');
        //Trasaction class
        if (!class_exists('Fiat'))
            includeCodeLib('FIAT');
        $fiat = new Fiat();
        //Calculate participations according formulas
        if ($transrec['MASS_TRANS_PART'] === '0') {
            //do based on distribution of fipd
            $sql = 'INSERT INTO ' . $this->ftmpname . ' (COMP_CODE, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, CALC_AMT_01, BATCH_ID) SELECT FIMT.COMP_CODE, FIMT.CHART_FIELD01, FIPD.CHART_FIELD02, FIMT.CHART_FIELD03, FIMT.CHART_FIELD04, FIMT.CURRENCY_CODE, FIMT.EXCHANGE_RT, FIMT.CUR_OPERATOR, FIPD.PARTICIPATION_AMT CALC_AMT_01, FIMT.IDCODE BATCH_ID FROM ' . $this->fimtname . ' FIMT, ' . $this->fipdname . ' FIPD WHERE FIMT.IDCODE = FIPD.MASS_TRANS_CODE AND FIMT.IDCODE = ' . singleQuote($transrec['IDCODE']);
            ClxCommon::$db->executeQuery($sql);
        } else {
            //do based on formula EQ -capital AR-Earnings
            $sql = 'INSERT INTO ' . $this->ftmpname . ' (COMP_CODE, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, CALC_AMT_01, CALC_AMT_02, CALC_AMT_03, AVAILABLE_AMT, BATCH_ID) SELECT JCBV.COMP_CODE, FIMT.CHART_FIELD01, JCBV.CHART_FIELD02, FIMT.CHART_FIELD03, FIMT.CHART_FIELD04, FIMT.CURRENCY_CODE, FIMT.EXCHANGE_RT, FIMT.CUR_OPERATOR, 0 CALC_AMT_01, FIMT.TRANS_AMT CALC_AMT_02, SUM(JCBV.BASE_BAL_AMT) CALC_AMT_03, 0 AVAILABLE_AMT, FIMT.IDCODE BATCH_ID FROM ' . $this->fimtname . ' FIMT, ' . $this->glcpname . ' GLCP, ' . $this->glacname . ' GLAC, ' . $this->jcbvname . ' JCBV WHERE GLCP.COMP_CODE = FIMT.COMP_CODE AND GLCP.GLGROUP_CODE = GLAC.GLGROUP_CODE AND GLAC.IDCODE = JCBV.GLACCOUNTCODE AND JCBV.CHART_FIELD01 = FIMT.CHART_FIELD01 #AND (JCBV.CHART_FIELD03 = FIMT.CHART_FIELD03 OR FIMT.CHART_FIELD03 IS NULL) #AND (JCBV.CHART_FIELD04 = FIMT.CHART_FIELD04 OR FIMT.CHART_FIELD04 IS NULL) AND GLAC.TRUST_ITEM_TYPE = FIMT.MASS_TRANS_PART AND FIMT.IDCODE = ' . singleQuote($transrec['IDCODE']) . ' GROUP BY JCBV.COMP_CODE, FIMT.CHART_FIELD01, JCBV.CHART_FIELD02, FIMT.CHART_FIELD03, FIMT.CHART_FIELD04, FIMT.CURRENCY_CODE, FIMT.EXCHANGE_RT, FIMT.CUR_OPERATOR, FIMT.TRANS_AMT, FIMT.IDCODE';
            ClxCommon::$db->executeQuery($sql);
            //total amount
            $sql = 'UPDATE ' . $this->ftmpname . ' SET AVAILABLE_AMT = IFNULL((SELECT SUM(JCBV.BASE_BAL_AMT) FROM ' . $this->fimtname . ' FIMT, ' . $this->glcpname . ' GLCP, ' . $this->glacname . ' GLAC, ' . $this->jcbvname . ' JCBV WHERE GLCP.COMP_CODE = FIMT.COMP_CODE AND GLCP.GLGROUP_CODE = GLAC.GLGROUP_CODE AND GLAC.IDCODE = JCBV.GLACCOUNTCODE AND JCBV.CHART_FIELD01 = FIMT.CHART_FIELD01 AND GLAC.TRUST_ITEM_TYPE = FIMT.MASS_TRANS_PART AND FIMT.IDCODE = ' . singleQuote($transrec['IDCODE']) . '),0)';
            ClxCommon::$db->executeQuery($sql);
            //final calculations
            $sql = 'UPDATE ' . $this->ftmpname . ' SET CALC_AMT_01 = ROUND((CALC_AMT_02*(CALC_AMT_03/AVAILABLE_AMT)), 2)';
            ClxCommon::$db->executeQuery($sql);
            //test if operation is complete or need to add difference o dist
            $sql = 'SELECT (MAX(CALC_AMT_02) - SUM(CALC_AMT_01)) CALC_AMT_01 FROM ' . $this->ftmpname;
            $diff = ClxCommon::$db->getOne($sql);
            if ($diff <> CLX_NUM_ZERO) {
                //Adjust any distribution difference
                $sql = 'UPDATE ' . $this->ftmpname . ' SET CALC_AMT_01 = CALC_AMT_01+' . $diff . ' LIMIT 1';
                ClxCommon::$db->executeQuery($sql);
            }
        }
        //Build transaccions
        $sql = ' SELECT ' . singleQuote('AUTO') . ' IDCODE, FIMT.IDNAME, FIMT.EFFTDATE, FIMT.TRANSTBL_CODE, ' . singleQuote('PEND') . ' CLX_AUTH_STATUS, FIMT.CURRENCY_CODE, FIMT.CURRENCY_CODE, FIMT.EXCHANGE_RT, FIMT.CUR_OPERATOR, FIMT.LGGROUP_CODE, FIMT.ITEM_REF_NO, FIMT.COMP_CODE, IFNULL(FTMP.CHART_FIELD01,' . singleQuote('null') . ') CHART_FIELD01, IFNULL(FTMP.CHART_FIELD02,' . singleQuote('null') . ') CHART_FIELD02, IFNULL(FTMP.CHART_FIELD03,' . singleQuote('null') . ') CHART_FIELD03, IFNULL(FTMP.CHART_FIELD04,' . singleQuote('null') . ') CHART_FIELD04, FIMT.FIDU_BANKACT_CODE, FTMP.CALC_AMT_01 TRANS_AMT, 0 RECONCILIED, FIMT.DISBURSEMENT_TYPE, FIMT.AVAILABILITY_DATE, FIMT.BENEFICIARY_NAME, FIMT.BRANCH_PRN, FIMT.RECORD_BLOCKING, FIMT.REFERENCE_NO, FIMT.IDCODE BATCH_ID, FIMT.IDCODE MASS_TRANS_CODE, FIMT.JE_SOURCE, FIMT.CREATEDBY, FIMT.CDATE, FIMT.MODIFIEDBY, FIMT.MDATE FROM ' . $this->ftmpname . ' FTMP, $this->fimtname FIMT WHERE FTMP.BATCH_ID = FIMT.IDCODE ORDER BY FTMP.CHART_FIELD01, FTMP.CHART_FIELD02';
        $results = ClxCommon::$db->executeQuery($sql);
        while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
            $tc = $fiat->fiat_build($result['COMP_CODE'], $result['TRANSTBL_CODE'], $result['CHART_FIELD01'], $result['CHART_FIELD02'], $result['TRANS_AMT'], $result);
            //Approving all transactions
            $fiat->fiat_approve_by_transcode($tc);
        }
		ClxCommon::$db->freeResult($results);
        //Set status
        ClxCommon::$db->executeQuery('UPDATE ' . $this->fimtname . ' SET CLX_AUTH_STATUS = ' . singleQuote('AP') . ', TRANS_STATUS = ' . singleQuote('AP') . ' WHERE TRANS_STATUS IN (' . singleQuoteArray(array('PEND', 'OPQU')) . ') AND IDCODE = ' . singleQuote($trans_code));
        $status = (ClxCommon::$db->oneExistsRecords($this->fimtname, 'TRANS_STATUS = ' . singleQuote('AP') . ' AND IDCODE = ' . singleQuote($trans_code)) > CLX_NUM_ZERO);
        return $status;
    }

    /**
     * Events running post autorizations
     */
    function fimt_post_autorize(&$record) {
        if (UserRequest::$request['ac'] === 'DELETE')
            return true;
        if (!class_exists('Gloq'))
            includeCodeLib('GLOQ');
        $gloq = new Gloq();
        $gloq->pushFimtListing($record['IDCODE']);
        return true;
    }

}

class Fipd {

    /**
     * Contructor
     *
     * @param array $procParameters
     * @return void
     */
    function __construct($procParameters = array()) {
        multiTableName($this, array('FITB', 'FITS', 'FIBK', 'COMP', 'FICO', 'FIPD', 'FIPA', 'FTMP', 'TTPA', 'JEIT', 'JEHD', 'JELN', 'FIRT', 'FIAC', 'FIMT', 'FETR', 'FIAT', 'FIRS', 'ACCO', 'GLCP', 'JACP', 'GLAC', 'JCBV'));
        $this->batch_id = (!empty($procParameters['BATCH_ID'])) ? $procParameters['BATCH_ID'] : batch_id();
        $this->username = $_SESSION['user_record']['username'];
        $this->now = xFormat::getNow('Y-m-d H:i:s');
    }

    /**
     * Enter description here...
     *
     * @param unknown_type $record
     */
    function fipd_init_form(&$record) {
        if (UserRequest::$request['ac'] === 'CREATE' || UserRequest::$request['ac'] === 'DUPLICATE' || UserRequest::$request['ac'] === 'MODIFY') {
            $record['FIPD_CHART_FIELD01'] = ClxCommon::$db->getField($this->fimtname, 'CHART_FIELD01', 'IDCODE = ' . singleQuote($record['FIPD_MASS_TRANS_CODE']));
        }
    }

//emd method
    /**
     * Validate record information
     * Executed on table validate
     */
    function fipd_validate_form(&$record, &$msg) {
        $fimtrec = ClxCommon::$db->getRecord($this->fimtname, 'IDCODE = ' . singleQuote($record['FIPD_MASS_TRANS_CODE']));
        //validar situacion de transaccion pendiente o no
        if ($fimtrec['CLX_AUTH_STATUS'] <> 'PEND') {
            $msg['FIPD_PARTICIPANT_ID'] = smsg('MSG-24', $fimtrec['IDCODE']);
            if (UserRequest::$request['ac'] === 'DELETE')
                addMessage($msg['FIPD_PARTICIPANT_ID']);
            return false;
        }
        //Validar participante este en ese contrato
        $sql = 'SELECT COUNT(FIPA.ID) FROM ' . $this->fipaname . ' FIPA, ' . $this->ficoname . ' FICO WHERE FICO.IDCODE = FIPA.CONTRACT_CODE AND FIPA.CONTRACT_CODE = ' . singleQuote($fimtrec['CHART_FIELD01']) . ' AND FIPA.ACCOUNTCODE = ' . singleQuote($record['FIPD_CHART_FIELD02']) . ' AND STATUS=' . singleQuote('A') . '';
        if (ClxCommon::$db->getOne($sql) == CLX_NUM_ZERO) {
            $msg['FIPD_CHART_FIELD02'] = smsg('MSG-32', '', $fimtrec['CHART_FIELD01']);
        }
        //not cero
        if ($record['FIPD_PARTICIPATION_AMT'] == CLX_NUM_ZERO) {
            $msg['FIPD_PARTICIPATION_AMT'] = smsg('MSG-34', $fimtrec['IDCODE']);
        }
        //Authorization methods perform all journal validations
        if (UserRequest::$request['ac'] === 'REQAUTHORIZATION') {
            return (count($msg) > CLX_NUM_ZERO) ? false : true;
        }
    }

    /**
     * Update Parent (FIMT) Information one record is saved
     */
    function fipd_save_form(&$record) {
        $sql = 'UPDATE ' . $this->fimtname . ' FIMT SET TRANS_AMT = (SELECT SUM(PARTICIPATION_AMT) FROM ' . $this->fipdname . ' FIPD WHERE FIPD.MASS_TRANS_CODE = FIMT.IDCODE) WHERE FIMT.IDCODE = ' . singleQuote($record['MASS_TRANS_CODE']);
        ClxCommon::$db->executeQuery($sql);
    }

}
