<?php

/**
 * General Code File
 * For more information contact  support@alenet.com
 *
 * @author       $Author: imrsnik $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2015-09-30 10:14:32 -0400 (Wed, 30 Sep 2015) $
 * @version        $Rev: 7065 $
 *
 * CHANGE LOG
 *  Included svn prop
 */
class Ibca {

    /**
     * Constructor
     * @param array [$procParameters]
     */
    function __construct($procParameters = array()) {
        multiTableName($this, array('IBCA', 'ACCO', 'IBCP', 'FICO', 'FIPA', 'JEHD', 'JELN', 'FIAT'));
        $this->batch_id = (!empty($procParameters['BATCH_ID'])) ? $procParameters['BATCH_ID'] : xFormat::getNow('YmdHis');
        $this->username = $_SESSION['user_record']['username'];
        $this->now = xFormat::getNow('Y-m-d H:i:s');
        $this->today = xFormat::getNow('Y-m-d');
        $this->je_source = '';
    }

    /**
     * Initialice fiat form - Executed on init event
     *
     * @param array $record
     */
    function init_form(&$record) {

    }

    /**
     * Validate record information
     * Executed on table validate
     *
     * @param array $record
     * @param array $msg
     * @return boolean
     */
    function validate_form(&$record, &$msg) {
        //Validar si registro fue completado que no se modifique el registro
        if ($record['IBCA_REC_STATUS'] === 'END' || $record['IBCA_REC_STATUS'] === 'CANC') {
            $msg['IBCA_REC_STATUS'] = smsg('MSG-247');
        }
    }

    function save_form(&$record) {

    }

    function save_form_post(&$record) {

    }

    //Webservices Methods
    /**
     * Prepares response for balance request
     * @param type $record
     * @return string
     */
    function ibs_caj_01_res($record) {
        //Respuesta
        $response = array('ERR_MSG_CODE' => '0',
            'ERROR_MSGS' => '',
            'CAJ_TRANS_CODE' => $record['CAJ_TRANS_CODE'],
            'CONTRACT_NUMBER' => '',
            'CLIENT_INFO' => '',
            'BALANCE_AMT' => '0.00',
            'CURRENCY_CODE' => '',
            'DESCRIPTION' => '');
        if (adodb_strtotime($record['OPERATION_DATE']) == -CLX_NUM_ONE) {
            $response['ERR_MSG_CODE'] = '1';
            $response['ERROR_MSGS'] = dict('GEN_ERR_INVALIDDATE');
            return $response;
        }
        //Revisar operacion
        switch ($record['CAJ_TRANS_CODE']) {
            case 'CRPG':
                includeCodeLib('FIAT');
                $fiat = new Fiat();
                $fiatrec = array();
                $loan_code = $record['ITEM_REF_NO'];
                $transtbl_code = 'SYS_CRED-PYMT';
                $operation_date = $record['OPERATION_DATE'];
                $fiat->loadCreditOperationRecord($loan_code, $transtbl_code, $operation_date, $fiatrec);
                if (empty($fiatrec)) {
                    //invalid credit
                    $response['ERR_MSG_CODE'] = '422';
                    $response['ERROR_MSGS'] = smsg_no_description('MSG-422') . ' (MSG-422)';
                } else {
                    //Validando identificacion.
                    if ($fiatrec['FIAT_CHART_FIELD02_terc'] <> $record['IDENTIFICATION']) {
                        $response['ERR_MSG_CODE'] = '423';
                        $response['ERROR_MSGS'] = smsg_no_description('MSG-423') . ' (MSG-423) ';
                    } else {
                        //Lugar de origen del registro
                        //Validar forma aca
                        $response['CONTRACT_NUMBER'] = $fiatrec['FIAT_CHART_FIELD01_prim'];
                        $response['CLIENT_INFO'] = $fiatrec['FIAT_CHART_FIELD02_secd'];
                        $response['BALANCE_AMT'] = rounder($fiatrec['FIAT_TRANS_AMT'], CLX_NUM_TWO);
                        $response['BALANCE_AMT'] = rounder($fiatrec['FIAT_INSTALLMENT_PRE'], CLX_NUM_TWO);
                        $response['CURRENCY_CODE'] = $fiatrec['FIAT_CURRENCY_CODE'];
                        $response['DESCRIPTION'] = $fiatrec['FIAT_IDNAME'];
                    }
                }
                break;
            default:
                //invalid error
                $response['ERR_MSG_CODE'] = '421';
                $response['ERROR_MSGS'] = smsg_no_description('MSG-421') . ' (MSG-421)';
                break;
        }
        return $response;
    }

    /**
     * Prepares response for balance request
     * @param type $record
     * @return string
     */
    function ibs_caj_02_res($record) {
        //Respuesta
        $response = array('ERR_MSG_CODE' => '0', 'ERROR_MSGS' => '');
        if ($this->ibs_caj_02_res_validate($record, $response)) { //de validar correcto
            //Crear registro de interfaz, transaccion post procesada luego
            //Check if key exists /TELLER_ID+SEQNUM+OPERATION_DATE
            if (ClxCommon::$db->oneExistsRecords($this->ibcaname, 'TELLER_ID = ' . singleQuote($record['TELLER_ID']) . ' AND CAJ_TRANS_CODE = ' . singleQuote($record['CAJ_TRANS_CODE']) . ' AND SEQNUM = ' . singleQuote($record['SEQNUM']) . ' AND OPERATION_DATE = ' . singleQuote($record['OPERATION_DATE'])) > CLX_NUM_ZERO) {
                $response['ERROR_MSGS'] = dict('GEN_LBL_ALREADYEXISTS');
                return $response;
            }
            $DBTB = &getTool('DBTB');
            $tablerec = $DBTB->getTable('IBCA');
            $ibca_rec = $DBTB->tableDefaultRecord('IBCA');
            $ibca_rec['REC_STATUS'] = 'IDLE';
            unset($ibca_rec['ERROR_MSGS'], $ibca_rec['ID'], $ibca_rec['JEHDR_CODE'], $ibca_rec['BATCH_ID'], $ibca_rec['ERR_MSG_CODE']);
            foreach ($record as $k => $v) {
                if (array_key_exists($k, $ibca_rec)) {
                    $ibca_rec[$k] = $v;
                }
            }
            if ($ibca_rec['CLIENT_INFO'] === '')
                $ibca_rec['CLIENT_INFO'] = 'null';
            if ($ibca_rec['DESCRIPTION'] === '')
                $ibca_rec['DESCRIPTION'] = 'null';
            $ibca_rec['CREATEDBY'] = '';
            $ibca_rec['MODIFIEDBY'] = '';
            $ibca_rec['BALANCE_AMT'] = $ibca_rec['TRANS_AMT'];
            $ibca_rec['EXT_CLIENT_NO'] = ClxCommon::$db->getField($this->acconame, 'IDCODE', 'IDENTIFICATION = ' . singleQuote($ibca_rec['IDENTIFICATION']));
            $EVNT = &getTool('EVNT');
            $EVNT->execute5('DBTB', $tablerec['ID'], 'CREP', $ibca_rec, $response); // Create
            $id = ClxCommon::$db->createRecord($this->ibcaname, $ibca_rec, true);
            if (empty($id)) {
                $response = array('ERR_MSG_CODE' => '1', 'ERROR_MSGS' => dict('GEN_ERR_CHECKTRAP'));
            }
        }
        return $response;
    }

    /**
     * Validate Method
     * @param type $record
     * @param type $response
     * @return type
     */
    private function ibs_caj_02_res_validate(&$record, &$response) {
        //Validar operacion
        if (empty($record['TELLER_ID'])) {
            //Invalid teller id
            $response = array('ERR_MSG_CODE' => '1', 'ERROR_MSGS' => 'TELLER_ID ' . dict('GEN_LBL_FIELDREQUIRED'));
        }
        //SEQNUM
        if (empty($record['SEQNUM'])) {
            //Invalid sequence number
            $response = array('ERR_MSG_CODE' => '1', 'ERROR_MSGS' => 'SEQNUM ' . dict('GEN_LBL_FIELDREQUIRED'));
        }
        //OPERATION_DATE
        if (adodb_strtotime($record['OPERATION_DATE']) == -CLX_NUM_ONE) {
            $response = array('ERR_MSG_CODE' => '1', 'ERROR_MSGS' => dict('GEN_ERR_INVALIDDATE'));
        }
        //Validar contrato de existir valor
        if (!empty($record['CONTRACT_NUMBER'])) {
            if (ClxCommon::$db->oneExistsRecords('FIDU_CONTRACTS', 'CONTRACT_NUMBER = ' . singleQuote($record['CONTRACT_NUMBER'])) == CLX_NUM_ZERO) {
                $response = array('ERR_MSG_CODE' => '424', 'ERROR_MSGS' => smsg_no_description('MSG-424') . ' (MSG-421)');
            }
        }
        //Validar credito
        switch ($record['CAJ_TRANS_CODE']) {
            case 'REV':
                //Validando que exista operacion de reverso
                $sql = 'SELECT COUNT(IBCA.ID) QTY FROM ' . $this->ibcaname . ' IBCA WHERE IBCA.CAJ_TRANS_CODE<>' . singleQuote('REV') . ' AND IBCA.TELLER_ID = ' . singleQuote($record['TELLER_ID']) . ' AND IBCA.SEQNUM = ' . singleQuote($record['SEQNUM']) . ' AND IBCA.OPERATION_DATE = ' . singleQuote($record['OPERATION_DATE']);
                if (ClxCommon::$db->getOne($sql) == CLX_NUM_ZERO) {
                    $response = array('ERR_MSG_CODE' => '426', 'ERROR_MSGS' => smsg_no_description('MSG-426') . ' (MSG-426)');
                }
                break;
            case 'CRPG':
                if (empty($record['CONTRACT_NUMBER'])) {
                    $response = array('ERR_MSG_CODE' => '424', 'ERROR_MSGS' => smsg_no_description('MSG-424') . ' (MSG-421)');
                }
                if (empty($record['ITEM_REF_NO'])) {
                    //invalid credit
                    $response['ERR_MSG_CODE'] = '422';
                    $response['ERROR_MSGS'] = smsg_no_description('MSG-422') . ' (MSG-422)';
                }
                break;
            default:
                //Validando operacion que exista
                if (ClxCommon::$db->oneExistsRecords($this->ibcpname, 'IDCODE = ' . singleQuote($record['CAJ_TRANS_CODE'])) == CLX_NUM_ZERO) {
                    $response = array('ERR_MSG_CODE' => '421', 'ERROR_MSGS' => smsg_no_description('MSG-421') . ' (MSG-421)');
                }
                break;
        }
        return ($response['ERR_MSG_CODE'] == CLX_NUM_ZERO) ? true : false;
    }

    //Post Ws IBCA Record Process Methods
    /**
     * Sets the reprocess button.
     *
     */
    function ibca_caj_reprocess_btn() {
        ClxCommon::$db->updateRecord($this->ibcaname, array('REC_STATUS' => 'IDLE', 'ERR_MSG_CODE' => null), 'ID IN (' . toString(getIdlist()) . ') AND REC_STATUS NOT IN (' . singleQuoteArryay(array('END', 'CANC')) . ')');
    }

    function ibca_caj_process_btn() {
        $sql = 'SELECT ID FROM ' . $this->ibcaname . ' WHERE ID IN (' . toString(getIdlist()) . ') AND REC_STATUS = ' . singleQuote('IDLE');
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            $this->ibca_build_transaction($r['ID']);
        }
		ClxCommon::$db->freeResult($rs);
    }

    /**
     * Executes operations from pending queue
     */
    function ibca_caj_process_cron() {
        $sql = 'UPDATE ' . $this->ibcaname . ' SET BATCH_ID = ' . singleQuote($this->batch_id) . ',REC_STATUS = ' . singleQuote('RUN') . ' WHERE REC_STATUS = ' . singleQuote('IDLE');
        ClxCommon::$db->executeQuery($sql);
        ClxCommon::$db->commit();
        $sql = 'SELECT ID FROM ' . $this->ibcaname . ' WHERE REC_STATUS = ' . singleQuote('RUN') . ' AND BATCH_ID = ' . singleQuote($this->batch_id);
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            $this->ibca_build_transaction($r['ID']);
        }
		ClxCommon::$db->freeResult($rs);
    }

    /**
     * Sets error message
     * @param type $id
     * @param type $msg_code
     */
    private function ibca_caj_set_msg($id, $msg_code) {
        ClxCommon::$db->updateRecord($this->ibcaname, array('ERR_MSG_CODE' => $msg_code), 'ID = ' . $id);
        //Cambiar error solo si no esta completado.
        ClxCommon::$db->updateRecord($this->ibcaname, array('REC_STATUS' => 'ERRO'), 'REC_STATUS <> ' . singleQuote('END') . ' AND ID = ' . $id);
    }

    /**
     * Validate operation creation
     * @param array $record
     * @return boolean
     */
    private function ibca_build_transaction_validate(&$record) {
        if (empty($record)) {
            return false;
        }
        //Validar que exista operacion
        if (ClxCommon::$db->oneExistsRecords($this->ibcpname, 'IDCODE = ' . singleQuote($record['CAJ_TRANS_CODE'])) == CLX_NUM_ZERO) {
            $this->ibca_caj_set_msg($record['ID'], 'MSG-421');
            return false;
        }
        //Verificar que asiento este aprobado.
        if (!empty($record['JEHDR_CODE'])) {
            if (ClxCommon::$db->oneExistsRecords($this->jehdname, 'IDCODE = ' . singleQuote($record['JEHDR_CODE']) . ' AND JE_STATUS = ' . singleQuote('PEND') . '') > CLX_NUM_ZERO) {
                $this->ibca_caj_set_msg($record['ID'], 'MSG-425');
                return false;
            }
        }
        //Validando que exista operacion de reverso
        if ($record['CAJ_TRANS_CODE'] === 'REV') {
            $sql = 'SELECT COUNT(IBCA.ID) QTY FROM ' . $this->ibcaname . ' IBCA WHERE IBCA.CAJ_TRANS_CODE<>' . singleQuote('REV') . ' AND IBCA.TELLER_ID = ' . singleQuote($record['TELLER_ID']) . ' AND IBCA.SEQNUM = ' . singleQuote($record['SEQNUM']) . ' AND IBCA.OPERATION_DATE = ' . singleQuote($record['OPERATION_DATE']);
            if (ClxCommon::$db->getOne($sql) == CLX_NUM_ZERO) {
                $this->ibca_caj_set_msg($record['ID'], 'MSG-426');
                return false;
            }
        }
        return true;
    }

    /**
     * Process cash entries records
     * @param type $id
     */
    private function ibca_build_transaction($id) {
        $record = ClxCommon::$db->getRecord($this->ibcaname, 'ID = ' . $id);
        ClxCommon::$db->updateRecord($this->ibcaname, array('REC_STATUS' => 'RUN', 'ERR_MSG_CODE' => null), 'ID = '.$id.' AND REC_STATUS NOT IN (' . singleQuoteArray(array('END', 'CANC')) . ')');
        //Crear transaccion con monto de transaccion
        $sql = 'SELECT FICO.COMP_CODE, FICO.IDCODE CHART_FIELD01, FIPA.ACCOUNTCODE CHART_FIELD02 FROM ' . $this->ficoname . ' FICO LEFT JOIN ' . $this->fipaname . ' FIPA ON FIPA.CONTRACT_CODE = FICO.IDCODE WHERE FICO.CONTRACT_NUMBER = ' . singleQuote($record['CONTRACT_NUMBER']) . 'ORDER BY FIPA.ISPRIMARY DESC LIMIT 1';
        $fico_rec = ClxCommon::$db->getOneRecord($sql);
        if (!empty($fico_rec)) {
            $record = array_merge($record, $fico_rec);
        } else {
            $record['COMP_CODE'] = '';
            $record['CHART_FIELD01'] = '';
            $record['CHART_FIELD02'] = '';
        }
        //Validar or retornar en error
        if (!$this->ibca_build_transaction_validate($record)) {
            return;
        }
        //Construir transaccion
        $ibcprec = ClxCommon::$db->getRecord($this->ibcpname, 'IDCODE = ' . singleQuote($record['CAJ_TRANS_CODE']));
        $savepost = UserRequest::$post;
        $trans_rec = array();
        $trans_rec['CURRENCY_CODE'] = $record['CURRENCY_CODE'];
        $trans_rec['COMP_CODE'] = $record['COMP_CODE'];
        $trans_rec['CHART_FIELD01'] = $record['CHART_FIELD01'];
        $trans_rec['CHART_FIELD02'] = $record['CHART_FIELD02'];
        $trans_rec['TRANS_AMT'] = $record['TRANS_AMT'];
        $trans_rec['EFFTDATE'] = $record['OPERATION_DATE'];
        $trans_rec['AVAILABILITY_DATE'] = $record['OPERATION_DATE'];
        $trans_rec['TRANSTBL_CODE'] = $ibcprec['TRANSTBL_CODE'];
        $trans_rec ['BRANCH'] = $record['EXT_BRANCH_CODE'];
        $trans_rec['IDNAME'] = $ibcprec['IDNAME'];
        $trans_rec['DISBURSEMENT_TYPE'] = 'OT';
        $trans_rec['GLACCOUNTCODE'] = '';
        $trans_rec['ITEM_REF_NO'] = $record['ITEM_REF_NO'];
        if (!class_exists('Comp'))
            includeCodeLib('COMP');
        if (!function_exists('fiat_build'))
            includeCodeLib('FIAT');
        $comp = new Comp();
        $fiat = new Fiat();
        $comp_op = $comp->comp_operation_parameters($trans_rec ['COMP_CODE']);
        $trans_rec['FISCAL_PERIOD'] = '';
        $trans_rec['LGGROUP_CODE'] = $comp_op['LGGROUP_CODE'];
        $xrate = $comp->comp_current_exchange($trans_rec ['COMP_CODE'], $trans_rec ['CURRENCY_CODE'], $trans_rec ['EFFTDATE']);
        $trans_rec ['EXCHANGE_RT'] = $xrate ['EXCHANGE_RT'];
        $trans_rec ['CUR_OPERATOR'] = $xrate ['CUR_OPERATOR'];
        includeCodeLib('FIAT');
        $fiat = new Fiat();
        switch ($record['CAJ_TRANS_CODE']) {
            case 'REV':
                $sql = 'SELECT JEHD.TRANS_CODE FROM ' . $this->ibcaname . ' IBCA, ' . $this->jehdname . ' JEHD WHERE IBCA.JEHDR_CODE = JEHD.IDCODE AND TELLER_ID = ' . singleQuote($record['TELLER_ID']) . ' AND SEQNUM = ' . singleQuote($record['SEQNUM']) . ' AND OPERATION_DATE = ' . singleQuote($record['OPERATION_DATE']) . ' AND IBCA.ID <> ' . singleQuote($id);
                $trans_code = ClxCommon::$db->getOne($sql);
                if (!empty($trans_code)) {
                    $trans_code = $fiat->fiat_reverse($trans_code);
                } else {
                    $trans_code = 'null';
                }
                break;
            case 'CRPG':
                //Pagos de creditos campos adicionales
                $trans_rec['CHART_FIELD03'] = ClxCommon::$db->getField('FIDU_ITEMS', 'IDCODE', 'CONTRACT_CODE = ' . singleQuote($trans_rec['CHART_FIELD01']) . ' AND LOAN_CODE = ' . singleQuote($record['ITEM_REF_NO']));
                $trans_rec['ADJUSTMENT'] = CLX_INIT_ZERO;
                $trans_rec['FIDU_BANKACT_CODE'] = ClxCommon::$db->getField('CRED_CREDITS', 'PAY_BANKACT_NO', 'IDCODE = ' . singleQuote($record['ITEM_REF_NO']));
                UserRequest::$post['FIAT_TRANSTBL_CODE'] = $trans_rec['TRANSTBL_CODE'];
                UserRequest::$post['FIAT_PRINCIPAL_PAY'] = CLX_INIT_ZERO;
                UserRequest::$post['FIAT_INTEREST_PAY'] = CLX_INIT_ZERO;
                UserRequest::$post['FIAT_PAY_APPLIES_TO'] = '2';
                UserRequest::$post['FIAT_PRINCIPAL_EFFECT'] = 'D';
                UserRequest::$post['FIAT_INTEREST_EFFECT'] = 'D';
                $loan_code = $record['ITEM_REF_NO'];
                $transtbl_code = 'SYS_CRED-PYMT';
                $operation_date = $record['OPERATION_DATE'];
                $fiat->loadCreditOperationRecord($loan_code, $transtbl_code, $operation_date, UserRequest::$post);
                $seq = explode(',', UserRequest::$post['FIAT_PAY_SEQUENCE']);
                $trans_amt = $record['TRANS_AMT'];
                //Reduciendo monto de acuerdo a secuencia de pago
                foreach ($seq as $k) {
                    switch ($k) {
                        case 'PAY_SEQNO_FEE_01':
                            if (UserRequest::$post['FIAT_FEE_01_PAY'] == CLX_NUM_ZERO) {
                                continue;
                            } elseif (UserRequest::$post['FIAT_FEE_01_PAY'] >= $trans_amt) {
                                UserRequest::$post['FIAT_FEE_01_PAY'] = $trans_amt;
                                $trans_amt = CLX_INIT_ZERO;
                            } else {
                                $trans_amt -= UserRequest::$post['FIAT_FEE_01_PAY'];
                            }
                            break;
                        case 'PAY_SEQNO_FEE_02':
                            if (UserRequest::$post['FIAT_FEE_02_PAY'] == CLX_NUM_ZERO) {
                                continue;
                            } elseif (UserRequest::$post['FIAT_FEE_02_PAY'] >= $trans_amt) {
                                UserRequest::$post['FIAT_FEE_02_PAY'] = $trans_amt;
                                $trans_amt = CLX_INIT_ZERO;
                            } else {
                                $trans_amt -= UserRequest::$post['FIAT_FEE_02_PAY'];
                            }
                            break;
                        case 'PAY_SEQNO_FEE_03':
                            if (UserRequest::$post['FIAT_FEE_03_PAY'] == CLX_NUM_ZERO) {
                                continue;
                            } elseif (UserRequest::$post['FIAT_FEE_03_PAY'] >= $trans_amt) {
                                UserRequest::$post['FIAT_FEE_03_PAY'] = $trans_amt;
                                $trans_amt = CLX_INIT_ZERO;
                            } else {
                                $trans_amt -= UserRequest::$post['FIAT_FEE_03_PAY'];
                            }
                            break;
                        case 'PAY_SEQNO_FEE_04':
                            if (UserRequest::$post['FIAT_FEE_04_PAY'] == CLX_NUM_ZERO) {
                                continue;
                            } elseif (UserRequest::$post['FIAT_FEE_04_PAY'] >= $trans_amt) {
                                UserRequest::$post['FIAT_FEE_04_PAY'] = $trans_amt;
                                $trans_amt = CLX_INIT_ZERO;
                            } else {
                                $trans_amt -= UserRequest::$post['FIAT_FEE_04_PAY'];
                            }
                            break;
                        case 'PAY_SEQNO_FEE_05':
                            if (UserRequest::$post['FIAT_FEE_05_PAY'] == CLX_NUM_ZERO) {
                                continue;
                            } elseif (UserRequest::$post['FIAT_FEE_05_PAY'] >= $trans_amt) {
                                UserRequest::$post['FIAT_FEE_05_PAY'] = $trans_amt;
                                $trans_amt = CLX_INIT_ZERO;
                            } else {
                                $trans_amt -= UserRequest::$post['FIAT_FEE_05_PAY'];
                            }
                            break;
                        case 'PAY_SEQNO_AR':
                            if (UserRequest::$post['FIAT_INTEREST_PRE'] == CLX_NUM_ZERO) {
                                continue;
                            } elseif (UserRequest::$post['FIAT_INTEREST_PRE'] >= $trans_amt) {
                                UserRequest::$post['FIAT_INTEREST_PRE'] = $trans_amt;
                                $trans_amt = CLX_INIT_ZERO;
                            } else {
                                $trans_amt -= UserRequest::$post['FIAT_INTEREST_PRE'];
                            }
                            break;
                        case 'PAY_SEQNO_PRO':
                            if (UserRequest::$post['FIAT_PRINCIPAL_PRE'] == CLX_NUM_ZERO) {
                                continue;
                            } elseif (UserRequest::$post['FIAT_PRINCIPAL_PRE'] >= $trans_amt) {
                                UserRequest::$post['FIAT_PRINCIPAL_PRE'] = $trans_amt;
                                $trans_amt = CLX_INIT_ZERO;
                            } else {
                                $trans_amt -= UserRequest::$post['FIAT_PRINCIPAL_PRE'];
                            }
                            break;
                        default:
                            break;
                    }
                }
                if ($trans_amt > CLX_NUM_ZERO) {
                    UserRequest::$post['FIAT_PRINCIPAL_PAY']+=$trans_amt;
                    $trans_amt = CLX_INIT_ZERO;
                }
                break;
            default://pendiente que esto siempre ocurre menos en reverso ya que no hay break
                //Cualquier otra
                if (!empty($record['JEHDR_CODE'])) {
                    $trans_rec['IDCODE'] = ClxCommon::$db->getField($this->jehdname, 'TRANS_CODE', 'IDCODE = ' . singleQuote($record['JEHDR_CODE']));
                }
                //build transaction
                $trans_code = $fiat->fiat_build($trans_rec['COMP_CODE'], $trans_rec['TRANSTBL_CODE'], $trans_rec['CHART_FIELD01'], $trans_rec['CHART_FIELD02'], $trans_rec['TRANS_AMT'], $trans_rec);
                break;
        }
        //Updating Journal
        $sql = 'UPDATE ' . $this->ibcaname . ' IBCA, ' . $this->jehdname . ' JEHD SET IBCA.JEHDR_CODE = JEHD.IDCODE WHERE JEHD.TRANS_CODE = ' . singleQuote($trans_code) . ' AND IBCA.ID = ' . $id;
        ClxCommon::$db->executeQuery($sql);
        if ($ibcprec['APPROVE_PAYMENT'] == CLX_NUM_ONE) {
            //sending to queue
            if (!class_exists('Gloq'))
                includeCodeLib('GLOQ');
            $gloq = new Gloq();
            $gloq->pushFiatListing($trans_code);
        }
        //Mark Record as complete
        ClxCommon::$db->updateRecord($this->ibcaname, array('REC_STATUS' => 'END', 'ERR_MSG_CODE' => null), 'ID = ' . $id . ' AND REC_STATUS NOT IN (' . singleQuoteArray(array('END', 'CANC')) . ')');
        UserRequest::$post = $savepost;
    }

}
