<?php

/**
 * General Code File
 * For more information contact  support@alenet.com
 *
 * @author       $Author: fsantana $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2015-10-12 14:54:28 -0400 (Mon, 12 Oct 2015) $
 * @version        $Rev: 7234 $
 *
 * CHANGE LOG
 *  Included svn prop
 */
class Oppo {

    /**
     * Contructor
     *
     * @param array $procParameters
     * @return void
     */
    function __construct($procParameters = array()) {
        multiTableName($this, array('SLAP', 'SLAA', 'ACTI', 'PROD', 'OPCL', 'ACCO', 'CMBP', 'OPPO', 'OPPH', 'OPST'));
        $this->batch_id = (!empty($procParameters ['BATCH_ID'])) ? $procParameters ['BATCH_ID'] : batch_id();
        $this->username = $_SESSION ['user_record'] ['username'];
        $this->now = xFormat::getNow('Y-m-d H:i:s');
        $this->today = xFormat::getNow('Y-m-d');
    }

    /**
     * Initialice fiai form
     * Executed on init event
     */
    function init_form(&$record) {
        if (UserRequest::$request['ac'] === 'DUPLICATE') {
            $record ['OPPO_IDCODE'] = 'AUTO';
        }
        if (UserRequest::$request['ac'] === 'CREATE' && empty($record['OPPO_ASSIGNEDTO_USER'])) {
            $record['OPPO_ASSIGNEDTO_USER'] = $this->username;
            $record['OPPO_AGENT_ACCODE'] = '';
            $record['OPPO_AGENT_ACCODE_prim'] = '';
            $record['OPPO_AGENT_ACCODE_secd'] = '';   
        }
    }

    /**
     * Validate record information
     * Executed on table validate
     */
    function validate_form(&$record, &$msg) {
        $status = true;
        // If The Opportunity is being Closed, user must provide Close Date, and
        // Actual Revenue;
        if ($record ['OPPO_OPPSTAGE'] === 'W' && $record ['OPPO_OPPSTAGE'] === '09') {
            if (empty($record ['OPPO_ACTUAL_CLOSE_DT']) || $record ['OPPO_ACTUAL_REVENUE'] == CLX_NUM_ZERO) {
                $msg ['OPPO_ACTUAL_CLOSE_DT'] = dict('GEN_ERR_OPPO_CLOSEDT_REVENUE');
                $status = false;
            }
        }
        if ($record ['OPPO_OPPSTAGE'] !== 'W' && $record ['OPPO_OPPSTAGE'] !== '09') {
            if (!empty($record ['OPPO_ACTUAL_CLOSE_DT']) || $record ['OPPO_ACTUAL_REVENUE'] != CLX_NUM_ZERO) {
                $msg ['OPPO_ACTUAL_CLOSE_DT'] = dict('GEN_ERR_OPPO_NOTCLOSED');
                $status = false;
            }
        }
        // Validate if Assigned User in opportunity belongs to Assigned Team in Team participants;
        $teamparticipant = null;
        if (!empty($record ['OPPO_TEAMCODE']) && !empty($record ['OPPO_ASSIGNEDTO_USER'])) {
            // ClxCommon::$db->debug=true;
            $teamparticipant = ClxCommon::$db->getField(tablename('SLTP'), 'TEAMMEMBER', 'TEAMCODE = ' . singleQuote($record ['OPPO_TEAMCODE']), 'TEAMMEMBER = ' . singleQuote($record ['OPPO_ASSIGNEDTO_USER']));
            // ClxCommon::$db->debug=false;
            if (empty($teamparticipant)) {
                $msg ['OPPO_TEAMCODE'] = dict('GEN_ERR_OPPO_NOTEAMMEMBER');
                $status = false;
            }
        }
        //Check List validations...
        $this->validate_checklist_form($record, $msg);
        return $status;
    }

    /**
     * Save form post database comit
     *
     * @param unknown_type $record
     */
    function save_form_post(&$record) {
        if (UserRequest::$request['ac'] === 'CREATE' || UserRequest::$request['ac'] === 'DUPLICATE') {
            //called also in convert to account
            includeCodeLib('SLAP');
            if (class_exists('SLAP')) {
                $a = new Slap ();
                $a->process_opportunity($record);
            }
        }
        //Process checklist
        $this->save_form_post_checklist($record);
    }

    /**
     * Activity save form
     *
     * @param array $record
     */
    public function save_form(&$record) {
        $this->add_history($record ['IDCODE']);
        $this->add_stage($record ['IDCODE'], $record['OPPSTAGE']);
    }

    /**
     * Set new stage history
     * @param string $oppo_code
     */
    private function add_stage($oppo_code, $stage) {
        $opstrec = array('OPPORTUNITYCODE' => $oppo_code, 'EFFTDATE' => $this->today, 'OPPSTAGE' => $stage, 'END_DATE' => 'null', 'STAGE_DURATION' => CLX_NUM_ZERO);
        $sql = 'SELECT OPST.* FROM ' . $this->opstname . ' OPST WHERE OPST.OPPORTUNITYCODE = ' . singleQuote($oppo_code) . ' AND OPST.EFFTDATE = (SELECT MAX(OPST1.EFFTDATE) FROM ' . $this->opstname . ' OPST1 WHERE OPST1.OPPORTUNITYCODE = ' . singleQuote($oppo_code) . ') ORDER BY ID DESC';
        $r = ClxCommon::$db->getOneRecord($sql);
        if (empty($r)) {
            ClxCommon::$db->createRecord($this->opstname, $opstrec, false);
        } else {
            if ($r['OPPSTAGE'] <> $stage) {
                $sql = 'UPDATE ' . $this->opstname . '  SET END_DATE = now(), STAGE_DURATION = DATEDIFF(now(), EFFTDATE)+1 WHERE OPPORTUNITYCODE = ' . singleQuote($r['OPPORTUNITYCODE']) . ' AND ID = ' . $r['ID'];
                ClxCommon::$db->executeQuery($sql);
                ClxCommon::$db->createRecord($this->opstname, $opstrec, false);
            }
        }
    }

    /**
     * Copies activity record to history
     *
     * @param integer $activity_id
     */
    private function add_history($oppo_code) {
        $rec = ClxCommon::$db->getRecord($this->opponame, 'IDCODE = ' . singleQuote($oppo_code));
        if (!empty($rec)) {
            unset($rec ['ID']);
            //Remove fields that not exists in history to prevent errors
            $sql = 'SELECT DBTF.FIELDNAME FROM CLX_DB_TABLEFIELDS DBTF LEFT JOIN CLX_DB_TABLEFIELDS DBTF2 ON DBTF.FIELDNAME = DBTF2.FIELDNAME AND DBTF2.TABLECODE = ' . singleQuote('OPPH') . ' WHERE DBTF.TABLECODE = ' . singleQuote('OPPO') . ' AND DBTF2.FIELDNAME IS NULL';
            $rs = ClxCommon::$db->executeQuery($sql);
            while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
                unset($rec[$r['FIELDNAME']]);
            }
			ClxCommon::$db->freeResult($rs);
            //Adding history
            ClxCommon::$db->createRecord($this->opphname, $rec, false);
        }
    }

    //Checklist methods
    /**
     * Performs check list validations for business processes
     * @param type $record
     * @param type $msg
     */
    private function validate_checklist_form(&$record, &$msg) {
        if (empty($record['OPPO_CMBP_CODE']))
            return;
			//check if checklist is complete already
        if (ClxCommon::$db->oneExistsRecords($this->opponame, 'IDCODE =' . singleQuote($record['OPPO_IDCODE']) . ' AND CHKLST_COMPLETE = 1') > CLX_NUM_ZERO) {
            return;
        }
        //Validar que el proceso de negocio funcione para el caso adevuado
        //Business process table CMBP
        $DBQY = &getTool('DBQY');
        /*
         * 1. Detectar si query usa tabla acco o tabla oppo.
         * 2. si existe tabla acco incorporar oppo.accountcode
         * 3. si existe tabla oppo incluir oppo.idcode
         * 3. De no existir ni oppo ni acco entonces dejar como se encuentra
         * 4. Incorporar sql en seccion where como as exists
         */
        //Crear tabla temporal con registro de oppo para mantener relacion en criterios.
        $sql = 'SELECT DBTF.FIELDNAME, DBFL.FIELDTYPE, DBFL.FIELD_EXTRA1 FROM CLX_DB_TABLEFIELDS DBTF, CLX_DB_FIELDS DBFL WHERE DBTF.TABLECODE = ' . singleQuote('OPPO') . ' AND DBFL.FIELDNAME = DBTF.FIELDNAME AND DBFL.FIELDTYPE NOT IN (' . singleQuote('TEX') . ',' . singleQuote('REC') . ',' . singleQuote('FIL') . ', ' . singleQuote('PRI') . ') ORDER BY DBTF.SEQNUM ASC';
        $oppo_table = '(SELECT ';
        $comma = '';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            $key = 'OPPO_' . $r['FIELDNAME'];
            if (array_key_exists($key, $record)) {
                $oppo_table .= $comma . ' ' . singleQuote(clxCommon::$db->addslashes($record[$key])) . ' ' . $r['FIELDNAME'];
                $comma = ',';
            }
        }
		ClxCommon::$db->freeResult($rs);
        $oppo_table .= ' FROM DUAL)';
        $sql_sub = '';
        //Read process criterias to test if valid process.
        $sql = 'SELECT CMCT.IDCODE CRITERIA_CODE, CMCT.IDNAME CRITERIA_NAME, CMBC.ANDOR_OPER, CMBC.OPEN_BRACKETS, CMCT.QUERYCODE, CMBC.CLOSE_BRACKETS FROM CMP_BUS_CRITERIA CMBC, CMP_BUS_CRITERIA_TBL CMCT WHERE CMBC.PROCRULE_CODE = ' . singleQuote($record['OPPO_CMBP_CODE']) . ' AND CMBC.CMCT_CODE = CMCT.IDCODE AND CMCT.STATUS = ' . singleQuote('A') . ' AND CMBC.STATUS = ' . singleQuote('A') . ' ORDER BY CMBC.PROCRULE_CODE ASC, CMBC.SEQNUM ASC';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            $queryinfo = $DBQY->getquery('DBQY', $r['QUERYCODE']);
            $and_oper = '';
            switch ($r['ANDOR_OPER']) {
                case 'NOT':
                    $and_oper = ' AND NOT ';
                    break;
                case 'OR':
                    $and_oper = ' OR ';
                    break;
                default:
                    $and_oper = ' AND ';
                    break;
            }
            if (strpos($queryinfo['FROMCLAUSE'], ' OPPO') !== false) {
                //Remplazando tablas de oppo por tabla temporal con registro de oppo
                $queryinfo['FROMCLAUSE'] = str_replace('SFA_OPPORTUNITIES', $oppo_table, $queryinfo['FROMCLAUSE']);
            }
            $sql_sub .= CLX_CHAR_CR . $and_oper . $r['OPEN_BRACKETS'] . ' EXISTS(SELECT 1 VAL FROM ' . $queryinfo['FROMCLAUSE'] . ' WHERE 1=1 ';
            if (strpos($queryinfo['FROMCLAUSE'], 'CRM_ACCOUNTS ACCO') !== false) {
                $sql_sub .= ' AND ACCO.IDCODE = ACCO1.IDCODE '; /* acco1 en tabla tempora luego */
            }
            // SFA_OPPORTUNITIES OPPO
            if (strlen($queryinfo['WHERECLAUSE']) > CLX_NUM_ZERO) {
                $sql_sub .= ' AND ' . $queryinfo['WHERECLAUSE'];
            }
            $sql_sub .= ')' . $r['CLOSE_BRACKETS'];
        }
		ClxCommon::$db->freeResult($rs);
        if ($sql_sub <> '') {
            $sql_crit = 'SELECT COUNT(ACCO1.ID) QTY FROM CRM_ACCOUNTS ACCO1 WHERE ACCO1.IDCODE = ' . singleQuote($record['OPPO_ACCOUNTCODE']) . ' ' . $sql_sub;
            if (ClxCommon::$db->getOne($sql_crit) == CLX_NUM_ZERO) {
                $msg['OPPO_CMBP_CODE'] = smsg('MSG-484');
            }
        }
    }

    private function checklist_doc_crit_valid($cmbd_code, $oppo_code, $acco_code) {
        $boolean = true;
        if (ClxCommon::$db->oneExistsRecords('CMP_DOC_CRITERIA', 'CMBD_CODE = ' . singleQuote($cmbd_code)) == CLX_NUM_ZERO) {
            return $boolean; //sin criterios retorna verdadero
        }
        $DBQY = &getTool('DBQY');
        $sql_sub = '';
        //Read process criterias to test if valid process.
        $sql = 'SELECT CMCT.IDCODE CRITERIA_CODE, CMCT.IDNAME CRITERIA_NAME, CMDC.ANDOR_OPER, CMDC.OPEN_BRACKETS, CMCT.QUERYCODE, CMDC.CLOSE_BRACKETS FROM CMP_DOC_CRITERIA CMDC, CMP_BUS_CRITERIA_TBL CMCT WHERE CMDC.CMBD_CODE = ' . singleQuote($cmbd_code) . ' AND CMDC.CMCT_CODE = CMCT.IDCODE AND CMCT.STATUS = ' . singleQuote('A') . ' AND CMDC.STATUS = ' . singleQuote('A') . ' ORDER BY CMDC.CMBD_CODE ASC, CMDC.SEQNUM ASC';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            $queryinfo = $DBQY->getquery('DBQY', $r['QUERYCODE']);
            $and_oper = '';
            switch ($r['ANDOR_OPER']) {
                case 'NOT':
                    $and_oper = ' AND NOT ';
                    break;
                case 'OR':
                    $and_oper = ' OR ';
                    break;
                default:
                    $and_oper = ' AND ';
                    break;
            }
            $sql_sub .= CLX_CHAR_CR . $and_oper . $r['OPEN_BRACKETS'] . ' EXISTS(SELECT 1 VAL FROM ' . $queryinfo['FROMCLAUSE'] . ' WHERE 1=1 ';
            if (strpos($queryinfo['FROMCLAUSE'], 'CRM_ACCOUNTS ACCO') !== false) {
                $sql_sub .= ' AND ACCO.IDCODE = ' . singleQuote($acco_code) . ' ';
            }
            if (strpos($queryinfo['FROMCLAUSE'], 'SFA_OPPORTUNITIES OPPO') !== false) {
                $sql_sub .= ' AND OPPO.IDCODE = ' . singleQuote($oppo_code) . ' ';
            }
            // SFA_OPPORTUNITIES OPPO
            if (strlen($queryinfo['WHERECLAUSE']) > CLX_NUM_ZERO) {
                $sql_sub .= ' AND ' . $queryinfo['WHERECLAUSE'];
            }
            $sql_sub .= ')' . $r['CLOSE_BRACKETS'];
        }
		ClxCommon::$db->freeResult($rs);
        if ($sql_sub <> '') {
            $sql_crit = 'SELECT COUNT(ACCO1.ID) QTY FROM CRM_ACCOUNTS ACCO1, SFA_OPPORTUNITIES OPPO1 WHERE OPPO1.IDCODE = ' . singleQuote($oppo_code) . ' AND ACCO1.IDCODE = ' . singleQuote($acco_code) . ' ' . $sql_sub;
            if (ClxCommon::$db->getOne($sql_crit) == CLX_NUM_ZERO) {
                $boolean = false; //No se encontro criterio retornando false para prevenir ejecucion
            }
        }
        return $boolean;
    }

    /**
     * Saves informaion
     *
     * @param type $record
     * @return type
     */
    private function save_form_post_checklist(&$record) {

        if (empty($record['CMBP_CODE']))
            return;
        $curlan = substr(getCurrentLanguage(), CLX_NUM_ZERO, CLX_NUM_TWO);
        $DBTB = &getTool('DBTB');
        $opclrec_def = $DBTB->tableDefaultRecord('OPCL');
        //Hacer documentos para los principales
        $sql = 'SELECT CMBD.IDCODE CMBD_CODE, CMDT.DOC_TYPE, CMBD.IDNAME, CMBD.PROCRULE_CODE, CMBD.DOC_TYPE_CODE, CMBD.IS_REQUIRED, CMBD.DOCRULE_TYPE, ACCO.IDCODE ACCOUNTCODE FROM CMP_BUS_PROCS CMBP, CMP_RULE_DOCS CMBD, CMP_DOCUMENTS_TBL CMDT, CRM_ACCOUNTS ACCO WHERE CMBP.IDCODE = ' . singleQuote($record['CMBP_CODE']) . ' AND ACCO.IDCODE = ' . singleQuote($record['ACCOUNTCODE']) . ' AND CMBP.STATUS = ' . singleQuote('A') . ' AND CMBD.STATUS = ' . singleQuote('A') . ' AND CMBD.DOCRULE_TYPE IN (' . singleQuote('01') . ',' . singleQuote('03') . ') AND CMDT.IDCODE = CMBD.DOC_TYPE_CODE AND CMBD.PROCRULE_CODE = CMBP.IDCODE ORDER BY CMBD.SEQNUM ASC';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            if (!$this->checklist_doc_crit_valid($r['CMBD_CODE'], $record['IDCODE'], $r['ACCOUNTCODE']))
                continue;
            $opclrec = $opclrec_def;
            $opclrec['CMBD_CODE'] = $r['CMBD_CODE'];
            $opclrec['OPPO_CODE'] = $record['IDCODE'];
            $opclrec['ACCOUNTCODE'] = $r['ACCOUNTCODE'];
            $opclrec['CMBD_CODE'] = $r['CMBD_CODE'];
            $opclrec['ISREQUIRED'] = $r['IS_REQUIRED'];
            $opclrec['NOTES'] = $r['IDNAME'];
            $this->save_form_post_checklis_opcl($opclrec);
        }
		ClxCommon::$db->freeResult($rs);
        //Cargando documentos para dependientes
        $sql = 'SELECT CMDT.DOC_TYPE, CMBD.IDCODE CMBD_CODE, CMBD.IDNAME, CMBD.PROCRULE_CODE, CMBD.DOC_TYPE_CODE, CMBD.IS_REQUIRED, CMBD.DOCRULE_TYPE, ACCO.IDCODE ACCOUNTCODE, DYAC.RELATIONSHIP, CLX_DBLI_ITEM_LABEL(' . singleQuote('STRE') . ', DYAC.RELATIONSHIP, ' . singleQuote($curlan) . ') RELATIONSHIP_NAME FROM CMP_BUS_PROCS CMBP, CMP_RULE_DOCS CMBD, CMP_DOCUMENTS_TBL CMDT, CRM_ACCOUNTS ACCO, CRM_REL_ACCOUNTS DYAC WHERE CMBP.IDCODE = ' . singleQuote($record['CMBP_CODE']) . ' AND DYAC.PARENT_IDCODE = ' . singleQuote($record['ACCOUNTCODE']) . ' AND DYAC.PARENT_TABLECODE = ' . singleQuote('ACCO') . ' AND DYAC.CHILD_IDCODE = ACCO.IDCODE AND DYAC.CHILD_TABLECODE = ' . singleQuote('ACCO') . ' AND CMBP.STATUS = ' . singleQuote('A') . ' AND CMBD.STATUS = ' . singleQuote('A') . ' AND CMBD.DOCRULE_TYPE IN (' . singleQuote('02') . ',' . singleQuote('03') . ') AND CMDT.IDCODE = CMBD.DOC_TYPE_CODE AND CMBD.PROCRULE_CODE = CMBP.IDCODE ORDER BY CMBD.SEQNUM ASC';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            if (!$this->checklist_doc_crit_valid($r['CMBD_CODE'], $record['IDCODE'], $r['ACCOUNTCODE']))
                continue;
            $opclrec = $opclrec_def;
            $opclrec['CMBD_CODE'] = $r['CMBD_CODE'];
            $opclrec['ACCOUNTCODE'] = $r['ACCOUNTCODE']; //check if dependant
            $opclrec['OPPO_CODE'] = $record['IDCODE'];
            $opclrec['ISREQUIRED'] = $r['IS_REQUIRED'];
            $opclrec['NOTES'] = $r['RELATIONSHIP_NAME'] . ': ' . $r['IDNAME'];
            $this->save_form_post_checklis_opcl($opclrec);
        }
		ClxCommon::$db->freeResult($rs);
    }

    /**
     * Saves informaion
     *
     * @param type $record
     * @return type
     */
    private function save_form_post_checklis_opcl(&$opclrec) {
        //check if not exists already
        $sql = 'SELECT COUNT(*) FROM ' . $this->opclname . '  WHERE OPPO_CODE = ' . singleQuote($opclrec['OPPO_CODE']) . ' AND ACCOUNTCODE = ' . singleQuote($opclrec['ACCOUNTCODE']) . ' AND CMBD_CODE = ' . singleQuote($opclrec['CMBD_CODE']);
        if (ClxCommon::$db->getOne($sql) > CLX_NUM_ZERO)
            return;
        if (!function_exists('setCreationData')) {
            includeCodeLib('GEN_REC_UTIL');
        }
        //Default items
        $opclrec['IDCODE'] = 'AUTO';
        $opclrec['LISTITEM_STAT'] = 'P';
        setCreationIdCode($opclrec, 'OPCL');
        setCreationData($opclrec);
        ClxCommon::$db->createRecord($this->opclname, $opclrec, false);
    }

}
