<?php

/**
 * Object Class
 * For more information contact  support@alenet.com
 * @package		Clientix.Tool.Event
 * @abstract		System Events Manager - System Alert Message.
 * @author       $Author: fsantana $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2015-09-03 12:24:19 -0400 (Thu, 03 Sep 2015) $
 * @version        $Rev: 6814 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
if (!class_exists('actn') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'actn.php';

class actn_rec_data extends actn {

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'RARD';
        $this->name = 'Set Record Creation / Modification Information';
        $this->type = 'ACTN';
        $this->version = '1.1.0.0';
        $this->description = 'Set Record Creation / Modification Information.';
    }

    /**
     * Build code object panel on edit event.
     * @param array $msg
     * @param array $actionrecord
     * @return string
     */
    function panel($msg, $actionrecord) {
        if ($actionrecord['EVNT_action'] != $this->code) {
		  $this->defaultPanel($actionrecord);
        }

        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_WIDTH => '90%', CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell(dict('GEN_LBL_PROPERTIES') . ':', array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addYesNo(dict('GEN_LBL_USER'), $this->code . 'updateuser', $actionrecord['RARD_properties'][0], 0, false);
        $table->addYesNo(dict('GEN_LBL_DATE'), $this->code . 'updatedate', $actionrecord['RARD_properties'][1], 0, false);
        $table->addYesNo(dict('GEN_LBL_CODE'), $this->code . 'updatecode', $actionrecord['RARD_properties'][2], 0, false);

        $div = new htmlDiv($this->code, 'visibility:hidden; display:none', $table->source());
        return CLX_CHAR_NL . CLX_CHAR_NL . $div->source() . CLX_CHAR_NL . CLX_CHAR_NL;
    }

    /**
     * Build object record from user post
     * @param array $actionrecord
     */
    function getPanel(&$actionrecord) {

        $actionrecord['extra3'] = '';
        if (!empty(UserRequest::$post[$this->code . 'updatecode']) && UserRequest::$post['EVNT_ownercode'] === 'DBTB' && !empty(UserRequest::$post['EVNT_ownerid'])) {
		  $actionrecord['extra3'] = ClxCommon::$db->getField('CLX_DB_TABLES', 'IDCODE', 'ID=' . UserRequest::$post['EVNT_ownerid']);
        }else{
		  $actionrecord['extra3'] = 'null';
        }
        $actionrecord['extra1'] = CLX_INIT_ZERO;
        $actionrecord['extra2'] = ((!empty(UserRequest::$post[$this->code . 'updateuser'])) ? 1 : 0) . '|' . ((!empty(UserRequest::$post[$this->code . 'updatedate'])) ? 1 : 0) . '|' . ((!empty(UserRequest::$post[$this->code . 'updatecode'])) ? 1 : 0);
    }

    /**
     * Sets edit record required on forms of user interface.
     * @param array $actionrecord
     * @param array $values
     */
    function setPanel(&$actionrecord, $values) {
        if ($values['ACTION'] == $this->code) {
		  if (empty($values['EXTRA2'])) {
			 $this->defaultPanel($actionrecord);
		  } else {
			 $actionrecord[$this->code . '_properties'] = $this->recDecodeOptions($values['EXTRA2']);
		  }
        } else {
		  $this->defaultPanel($actionrecord);
        }
    }

    /**
     * Sets the default values for form on create event.
     * @param array $actionrecord
     */
    function defaultPanel(&$actionrecord) {
        $actionrecord[$this->code . '_properties'] = array(0, 0, 0);
    }

    /**
     * Builds description about the object on event panel listing.
     * @param array $record
     * @return string
     */
    function describe($record) {

        $options = $this->recDecodeOptions($record['EXTRA2']);
        $yesno = array(dict('GEN_LBL_NO'),dict('GEN_LBL_YES'));
        $lbl = $this->name.' [ '.dict('GEN_LBL_USER') . ': ' . $yesno[$options[0]].', '.dict('GEN_LBL_DATE') . ': ' . $yesno[$options[1]].', '.dict('GEN_LBL_CODE') . ': ' . $yesno[$options[2]].' ]';
        return $lbl;
    }

    /**
     * Execute code according object type.
     * @param string $event
     * @param array $record
     * @param array $msg
     */
    function execute3($event, &$record, &$msg) {IF (ISSET($GLOBALS['ADDRDEBUGEVENTS'])) { echo __METHOD__.' '. __FILE__ .':'.__LINE__; pre($event);pre($record); }

        $event['EXTRA2'] = $this->recDecodeOptions($event['EXTRA2']);

        //$event['EXTRA2']//[0]=user, [1]=date,[2]=idcode
        $action = $this->recEventEffect($event['EVENT']);

        if ($event['EXTRA2'][0] === CLX_NUM_ONE) {IF (ISSET($GLOBALS['ADDRDEBUGEVENTS'])) { echo __METHOD__.' '. __FILE__ .':'.__LINE__;  pre('---->userdata'); }
		  $this->setUserData($action, $record);
        }
        if ($event['EXTRA2'][1] === CLX_NUM_ONE) {IF (ISSET($GLOBALS['ADDRDEBUGEVENTS'])) { echo __METHOD__.' '. __FILE__ .':'.__LINE__;  pre('---->date inf'); }
		  $this->setDateData($action, $record);
        }
		IF (ISSET($GLOBALS['ADDRDEBUGEVENTS'])) {pre($record);}
        if ($event['EXTRA2'][2] === CLX_NUM_ONE && !empty($event['EXTRA3'])) {
		  $this->setCreationIdCode($record, $event['EXTRA3']);
        }
        return true;
    }

    /**
     * Returns exploded items
     * @param type $options
     * @return type
     */
    private function recDecodeOptions($options) {
        $val = explode('|', $options);
        return array(intval($val[0]),intval($val[1]),intval($val[2]));
    }

    /**
     * Return event type
     * @param type $event
     * @return type
     */
    private function recEventEffect($event) {
        switch ($event) {
		  case 'IREC': case 'CREC': case 'CREP':
			 $return = 'CREATE';
			 break;
		  case 'LREC': case 'UREC': case 'UREP': case 'DREP': case 'DREC': case 'AREP': case 'AREC': case 'ARRP':
			 $return = 'MODIFY';
			 break;
		  default:
			 $return = CLX_CHAR_EMPTY_STRING;
			 break;
        }
        return $return;
    }

    /**
     * Sets record data for user information
     * @param type $action
     * @param type $record
     */
    private function setUserData($action, &$record) {

        if ($action === 'CREATE' && array_key_exists('CREATEDBY', $record)) {
		  $record['CREATEDBY'] = $_SESSION['user_record']['username'];
        }

        if ($action === 'CREATE' && array_key_exists('OWNERCODE', $record)) {
		  $record['OWNERCODE'] = $_SESSION['user_record']['username'];
        }
        if ($action === 'CREATE' && array_key_exists('USERNAME', $record)) {
		  $record['USERNAME'] = $_SESSION['user_record']['username'];
        }

        if (($action === 'CREATE' || $action ==='MODIFY') && array_key_exists('MODIFIEDBY', $record)) {
		  $record['MODIFIEDBY'] = $_SESSION['user_record']['username'];
        }
    }

    /**
     * Sets date data for user information
     * @param type $action
     * @param type $record
     */
    private function setDateData($action, &$record) {
        if ($action === 'CREATE' && array_key_exists('CDATE', $record)) {
		  $record['CDATE'] = xFormat::getNow('Y-m-d H:i:s');
        }

        if (($action === 'CREATE' || $action ==='MODIFY') && array_key_exists('MDATE', $record)) {
		  $record['MDATE'] = xFormat::getNow('Y-m-d H:i:s');
        }
    }

    /**
     * Creates idcodes
     * @param type $record
     * @param type $tablecode
     * @return string
     */
    function setCreationIdCode(&$record, $tablecode) {
        if(!array_key_exists('IDCODE', $record)) return;
        if (trim(uppercase($record['IDCODE'])) === 'AUTO'){
		  $record['IDCODE'] = 'NEW';//change in case there is no codes
        }
        if ($record['IDCODE'] === 'NEW' && ClxCommon::$db->oneExistsRecords('BASE_IDCODES', 'TABLECODE=' . singleQuote($tablecode)) > CLX_NUM_ZERO) {
		  //Commit set to the end of execution
		  $sql = 'SELECT * FROM BASE_IDCODES WHERE TABLECODE = ' . singleQuote($tablecode) . ' FOR UPDATE';
		  $crec = ClxCommon::$db->getOneRecord($sql);
		  $record['IDCODE'] = getIdcode($crec['IDCODE_FORMAT'], $crec['LAST_IDCODE'], $crec['INCREMENTS']);
		  //Setting New ID Codes Entry
		  ClxCommon::$db->updateRecord('BASE_IDCODES', array('LAST_IDCODE' => $record['IDCODE'], 'CDATE' => $crec['CDATE'], 'MDATE' => xFormat::getNow()), 'ID=' . $crec['ID']);
		  if (kDefaultIsCron == CLX_NUM_ZERO) {
			 ClxCommon::$db->commit();
		  }
        }
    }



    /**
     * Public method to prevent duplicates
     * @param type $action
     * @param type $setuser
     * @param type $setdate
     * @param type $setidcode
     * @param type $record
     */
    public function setRecordInformation($action, $setuser, $setdate, $setidcode, $tablecode, &$record) {
        if(!empty($setuser)){
		  $this->setUserData($action, $record);
        }
        if(!empty($setdate)){
		  $this->setDateData($action, $record);
        }

        if(!empty($setidcode)){
		  $this->setCreationIdCode($record, $tablecode);
        }

    }

}
