<?php

/**
 * Object Class
 * For more information contact  support@alenet.com
 * @package		Clientix.Tool.Event
 * @abstract		System Events Manager - SQL Event.
 * @author       $Author: fsantana $
 * @copyright    Alenet, Inc.
 * @since        $Date:  10:51:16 -0430 (Lun, 13 Sep 2010) $
 * @version        $Rev: 2016 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
if (!class_exists('actn') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'actn.php';

class actn_sql extends actn {

    var $do_section_type = array('NOT' => 'GEN_LBL_NONE', 'DS' => 'GEN_LBL_DO_SELECT', 'DW' => 'GEN_LBL_DO_WHILE', 'DWH' => 'GEN_LBL_DO_WHEN', 'DU' => 'GEN_LBL_DO_UNTIL');

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'SQL';
        $this->name = 'Sql Script';
        $this->type = 'ACTN';
        $this->version = '1.1.1.0';
        $this->description = 'Executes code.';
        $this->debug = false;
        $this->loadedlibraries = array();
    }

    /**
     * Build code object panel on edit event.
     * @param array $msg
     * @param array $actionrecord
     * @return string
     */
    function panel($msg, $actionrecord) {
        if ($actionrecord['EVNT_action'] != $this->code) {
		  $this->defaultPanel($actionrecord);
        }
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_WIDTH => '90%', CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell('Code Properties', array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addText($msg, dict('GEN_LBL_CALL') . ':', 'SQL_call', $actionrecord['SQL_call'], CLX_NUM_30, CLX_NUM_120);
        $SQL = &getTool('CODE');
        $scripts = $SQL->placeChooseCodeLibrary($msg, 'EVNT_language', dict('GEN_LBL_LIBRARY') . ':', 'SQL_library', $actionrecord['SQL_library'], $table);
        $sel = new htmlSelect('SQL_database');
        $sel->addItem(dict('GEN_LBL_ALL'), 'all', ($actionrecord['SQL_database'] === 'all'));
        foreach (ClxCommon::$db->dbTypes as $key => $val) {
		  $sel->addItem(dict($val), $key, ($actionrecord['SQL_database'] == $key));
        }
        $table->addCell(dict('GEN_LBL_DATABASE'), array(CLX_PROP_CLASS => CLX_CLASS_PROMPT));
        $table->addCell($sel->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        $table->addSeparator(CLX_CLASS_DATA);
        $table->addCell(dict('GEN_LBL_DO_SECTION_TYPE'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $do_section = new htmlSelect('SQL_do_section_type');
        foreach ($this->do_section_type as $key => $val) {
		  $do_section->addItem(dict($val), $key, ($actionrecord['SQL_do_section_type'] == $key));
        }
        $table->addData(dict('GEN_LBL_DO_SECTION_TYPE') . ':', $do_section->source());
        if (!empty($msg['SQL_do_section'])) {
		  $table->addCell($msg['SQL_do_section'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $table->width));
        }
        $PRSE = &getTool('PRSE');
        $table->addData(dict('GEN_LBL_DO_SECTION') . ':', $PRSE->chooseRecord('SQL_do_section', $actionrecord['SQL_do_section']));
        $div = new htmlDiv($this->code, 'visibility:hidden; display:none', $table->source());
        return $scripts . CLX_CHAR_NL . CLX_CHAR_NL . $div->source() . CLX_CHAR_NL . CLX_CHAR_NL;
    }

    /**
     * Validate Panel User Input before saving record.
     * @param array $msg
     */
    function checkPanel(&$msg) {
        if (empty(UserRequest::$post['SQL_call'])) {
		  $msg['SQL_call'] = dict('EVNT_LBL_TYPE_CALL');
        }
        if (!empty(UserRequest::$post['SQL_library']) && !$this->libExists(UserRequest::$post['SQL_library'])) {
		  $msg['SQL_library'] = dict('EVNT_LBL_LIB_EXISTS');
        }
        if (!empty(UserRequest::$post['SQL_call']) && !empty(UserRequest::$post['SQL_library'])) {
		  $msg['SQL_call'] = dict('EVNT_ONLY_LIB_OR_CALL');
		  $msg['SQL_library'] = $msg['SQL_call'];
        }
        //checking section
        if (UserRequest::$post['SQL_do_section_type'] !== 'NOT') {
		  if (empty(UserRequest::$post['SQL_do_section'])) {
			 $msg['SQL_do_section'] = dict('GEN_LBL_DO_SECTION_null');
		  } else {
			 $callingpage = peekHistory();
			 $sql = 'SELECT COUNT(ID) FROM CLX_PR_SECTIONS WHERE IDCODE = ' . singleQuote(UserRequest::$post['SQL_do_section']) . ' AND PROCESSCODE = ' . singleQuote($callingpage['processcode']);
			 if (ClxCommon::$db->getOne($sql) == CLX_NUM_ZERO) {
			     $msg['SQL_do_section'] = dict('GEN_LBL_DO_SECTION_WRONG');
			 }
		  }
        }
    }

    /**
     * Build object record from user post
     * @param array $actionrecord
     */
    function getPanel(&$actionrecord) {
        $actionrecord['EXTRA1'] = CLX_INIT_ZERO;
        $actionrecord['EXTRA2'] = clx_html_spchrs_decode(UserRequest::$post['SQL_call']);
        $actionrecord['EXTRA3'] = UserRequest::$post['SQL_library'];
        $actionrecord['EXTRA4'] = UserRequest::$post['SQL_database'];
        $actionrecord['EXTRA5'] = '';
        $actionrecord['EXTRA6'] = '';
        $actionrecord['DO_SECTION'] = UserRequest::$post['SQL_do_section'];
        $actionrecord['DO_SECTION_TYPE'] = UserRequest::$post['SQL_do_section_type'];
    }

    /**
     * Sets edit record required on forms of user interface.
     * @param array $actionrecord
     * @param array $values
     */
    function setPanel(&$actionrecord, $values) {
        if ($values['ACTION'] == $this->code) {
		  $actionrecord['SQL_call'] = $values['EXTRA2'];
		  $actionrecord['SQL_library'] = $values['EXTRA3'];
		  $actionrecord['SQL_database'] = $values['EXTRA4'];
		  $actionrecord['SQL_do_section'] = $values['DO_SECTION'];
		  $actionrecord['SQL_do_section_type'] = $values['DO_SECTION_TYPE'];
        } else {
		  $this->defaultPanel($actionrecord);
        }
    }

    /**
     * Sets the default values for form on create event.
     * @param array $actionrecord
     */
    function defaultPanel(&$actionrecord) {
        $actionrecord['SQL_call'] = '';
        $actionrecord['SQL_library'] = '';
        $actionrecord['SQL_database'] = 'all';
        $actionrecord['SQL_do_section_type'] = 'NOT';
        $actionrecord['SQL_do_section'] = '';
        $actionrecord['SQL_seqnum'] = '0';
    }

    /**
     * Build panel for display form
     * @param obj $table
     * @param array $record
     */
    function viewForm(&$table, $record) {
        $table->addData(dict('GEN_LBL_ACTION') . ':', $this->name);
        $table->addData(dict('GEN_LBL_CALL') . ':', '<pre>' . $record['SQL_call'] . '</pre>');
        $lbl = ($record['SQL_database'] === 'all') ? 'GEN_LBL_ALL' : ClxCommon::$db->database_type[$record['SQL_database']];
        $table->addData(dict('GEN_LBL_database') . ':', dict($lbl));
        if (!empty($record['SQL_library'])) {
		  $table->addData(dict('GEN_LBL_CODE_LIBRARY') . ':', '<pre>' . $record['SQL_library'] . '<pre>');
        }
    }

    /**
     * Builds description about the object on event panel listing.
     * @param array $record
     * @return string
     */
    function describe($record) {
        $return = dict('GEN_LBL_STATEMENT') . ': <pre>' . $record['EXTRA2'] . '</pre>';
        if ($record['EXTRA3'] !== '') {
		  $return .= '<br>' . dict('GEN_LBL_CODELIB') . ': ' . $record['EXTRA3'];
        }
        if (!empty($record['EXTRA4']) && $record['EXTRA4'] !== '') {
		  $return .= '<br>' . dict('GEN_LBL_DATABASE') . ': ' . (($record['EXTRA4'] === 'all') ? dict('GEN_LBL_ALL') : dict(ClxCommon::$db->dbTypes[$record['EXTRA4']]));
        }
        if ($record['DO_SECTION_TYPE'] !== 'NOT') {
		  $return .= '<br>' . dict('GEN_LBL_DO_SECTION_TYPE') . ': ' . dict($this->do_section_type[$record['DO_SECTION_TYPE']]);
		  $return .= '<br>' . dict('GEN_LBL_DO_SECTION') . ': ' . $record['DO_SECTION'];
        }
        return $return;
    }

    /**
     * Execute code according object type.
     * @param string $event
     * @param array $record
     * @param array $msg
     */
    function execute3($event, &$record, &$msg) {IF (ISSET($GLOBALS['ADDRDEBUGEVENTS'])) { echo __METHOD__.' '. __FILE__ .':'.__LINE__; pre($event);pre($record); }
        $code = '';
        $libcode = $event['EXTRA3'];
        if ($libcode !== '') {
		  $librec = ClxCommon::$db->getRecord('CLX_CODELIB', 'IDCODE=' . singleQuote($libcode));
		  if (empty($librec)) {
			 $BUGS = getTool('BUGS');
			 $BUGS->report(E_USER_ERROR, dict('TOOL_CODE_ERR_NO_RECORD') . ': ' . dict('GEN_LBL_CODELIB') . ' ' . $event['EXTRA3'], 'objects/actn_event.php', 'unknown line', $record, CLX_NUM_ONE, 'Lib code: ' . $libcode);
		  } elseif ($librec['TYPE'] === 'ENBD') {
			 $code = stripslashes($librec['CONTENTS']);
		  } elseif ($librec['TYPE'] === 'LINK') {
			 include_once kPathToWebSite . $librec['FILENAME'];
			 $code = file_get_contents(kPathToWebSite . $librec['FILENAME']);
		  }
        } else {
		  $code = $event['EXTRA2'];
        }
        if (!empty($this->debug)) {
		  echo '<!-- ', $code, ' ', $librec['FILENAME'], ' ', $event['EXTRA2'], ' -->';
        }
        $rs = false;
        if (strlen($code) > CLX_NUM_ZERO) {
		  $rs = ClxCommon::$db->executeQuery($code);
			apc_clear_from_sql($code);
        }
        return $rs;
    }

    // PROCESS ENGINE RELATED METHODS
    /**
     * Execute code according object type.
     * @param string $event
     * @param array $record
     * @param array $parameters
     * @param array $msg
     * @param bool $debug
     * @param bool $debug
     */
    function execute_process_event($event, &$record, &$parameters, &$msg, $debug = false) {
		$inittime=microtime(true);
        //process object pass by reference(from getTool) from execution
        $code = $event['EXTRA2'];
        $libcode = $event['EXTRA3'];


        if ($libcode <> '') {
		  $librec = ClxCommon::$db->getRecord('CLX_CODELIB', 'IDCODE=' . singleQuote($libcode));
		  if (empty($librec)) {
			 $BUGS = getTool('BUGS');
			 $BUGS->report(E_USER_ERROR, dict('TOOL_CODE_ERR_NO_RECORD') . ': ' . dict('GEN_LBL_CODELIB') . ' ' . $event['EXTRA3'], 'objects/actn_event.php', 'unknown line', $record, CLX_NUM_ONE, 'Lib code: ' . $libcode);
		  } elseif ($librec['TYPE'] === 'ENBD') {
			 $code = stripslashes($librec['CONTENTS']);
		  } elseif ($librec['TYPE'] === 'LINK') {
			 include_oncekPathToWebSite . $librec['FILENAME'];
			 $code = file_get_contents(kPathToWebSite . $librec['FILENAME']);
		  }
        }


        $rs = false;
        $code = $this->prepare_process_code($code, $record, $parameters);
        if (strlen($code) > CLX_NUM_ZERO) {
		  if (!empty($debug)) {
			 pre($code);
		  }
		  if ( isset($_SESSION["_dEBUGX"]) and $_SESSION["_dEBUGX"]==99) {
			$tta=fopen(kPathToWebSite."/".kPathToTemp."/process_executions_events_sql.txt","a+");
			fwrite($tta,chr(13).chr(10).'Init sql , record: '.print_r($record,true).chr(13).chr(10));
			fwrite($tta,chr(13).chr(10).'Event section:'. print_r($event,true).chr(13).chr(10));
			fwrite($tta,chr(13).chr(10).'Tiempo section:'. (microtime(true)-$inittime).chr(13).chr(10));
			fclose($tta);
		  }	
		  $rs = ClxCommon::$db->executeQuerySilent($code);
		  if ( isset($_SESSION["_dEBUGX"]) and $_SESSION["_dEBUGX"]==99) {
			$tta=fopen(kPathToWebSite."/".kPathToTemp."/process_executions_events_sql.txt","a+");
			fwrite($tta,chr(13).chr(10).'Tiempo section:'. (microtime(true)-$inittime).chr(13).chr(10));
			fclose($tta);
		  } 	
		  $errorNum = ClxCommon::$db->errorNum();
		  if ($errorNum != CLX_NUM_ZERO) {
		$inner = alert('Error' . $errorNum.' '.ClxCommon::$db->errorMsg());
			 echo $inner;
		  }
        }

        return $rs;
    }

    /**
     * Traslate step vars into actual values prior execution.
     * Only values found will ve translated other will remain.
     * @param string $code
     * @param array $record
     * @param array $parameters
     */
    function prepare_process_code($code, &$record, &$parameters) {

        if (is_array($record) && strpos(lowercase($code), '$record') !== false) {
		  foreach ($record as $key => $val) {
			 $code = str_replace(array('$record[' . singleQuote($key) . ']', '$record[' . quote($key) . ']', '$RECORD[' . singleQuote($key) . ']', '$RECORD[' . quote($key) . ']'), $val, $code);
		  }
        }

        if (is_array($parameters) && strpos(lowercase($code), '$parameters') !== false) {
		  foreach ($parameters as $key => $val) {
			 $code = str_replace(array('$parameters[' . singleQuote($key) . ']', '$parameters[' . quote($key) . ']', '$PARAMETERS[' . singleQuote($key) . ']', '$PARAMETERS[' . quote($key) . ']'), $val, $code);
		  }
        }

        //Search if system message may be passed
        $matches = array();
        preg_match_all('(smsg\((.*?)\))', $code, $matches);
        if (!empty($matches[CLX_NUM_ZERO])) {
		  foreach ($matches[CLX_NUM_ZERO] as $key => $val) {

			 $val = $matches[CLX_NUM_ZERO][$key];
			 //$msg_code = str_replace(array(CLX_CHAR_SINGLE_QUOTE, CLX_CHAR_DOUBLE_QUOTE, CLX_CHAR_SINGLE_QUOTE, CLX_CHAR_DOUBLE_QUOTE,CLX_CHAR_EMPTY_STRING, ' ', ','), CLX_CHAR_EMPTY_STRING,$matches[CLX_NUM_ONE][$key]);
			 $fields = explode(',',str_replace(array(CLX_CHAR_SINGLE_QUOTE, CLX_CHAR_DOUBLE_QUOTE, CLX_CHAR_SINGLE_QUOTE, CLX_CHAR_DOUBLE_QUOTE,CLX_CHAR_EMPTY_STRING, ' '), CLX_CHAR_EMPTY_STRING,$matches[CLX_NUM_ONE][$key]));
			 $msg_code = $fields[0];
			 $newval = smsg($msg_code, $fields);
			 ClxCommon::$db->addslashes($newval);
			 $code = str_replace($val, $newval, $code);
		  }
        }

        return $code;
    }

}
