<?php

/**
 * Object Class
 * For more information contact  support@alenet.com
 * @package		Clientix.Tool.Event
 * @abstract		System Events Manager - Task Event.
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
if (!class_exists('actn') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'actn.php';

class actn_task extends actn {

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'TASK';
        $this->name = 'Task Scheduler';
        $this->type = 'ACTN';
        $this->version = '1.1.0.0';
        $this->description = 'Creates a task.';
    }

    /**
     * Build code object panel on edit event.
     * @param array $msg
     * @param array $actionrecord
     * @return string
     */
    function panel($msg, $actionrecord) {
        if ($actionrecord['EVNT_action'] != $this->code) {
		  $this->defaultPanel($actionrecord);
        }
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_WIDTH => '90%', CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell('Task Properties', array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $ACUS = &getTool('ACUS');
        $ACUS->placeChooseUser($msg, dict('GEN_LBL_ASSIGNED_TO') . ':', 'TASK_assignedto', $actionrecord['TASK_assignedto'], $table);
        $table->addField($msg, dict('GEN_LBL_SUBJECT') . ':', 'TASK_subject', $actionrecord['TASK_subject'], CLX_NUM_64, CLX_NUM_128);
        $table->addField($msg, dict('GEN_LBL_START_DATE') . ':', 'TASK_start_date', $actionrecord['TASK_start_date'], CLX_NUM_32, CLX_NUM_32);
        $table->addField($msg, dict('GEN_LBL_DUE_DATE') . ':', 'TASK_due_date', $actionrecord['TASK_due_date'], CLX_NUM_32, CLX_NUM_32);
        $USTK = &getTool('USTK');
        $pop = new HTMLSelect('TASK_priority');
        foreach ($USTK->priorities as $key => $value) {
		  $pop->addItem(dict($value), $key, (!empty($actionrecord['TASK_priority']) && $actionrecord['TASK_priority'] == $key));
        }
        $table->addData(dict('GEN_LBL_PRIORITY') . ':', $pop->source());
        $DBLT = &getTool('DBLT');
        $types = $DBLT->getList('TKLT', 'TASK_type', (!empty($actionrecord['TASK_type'])) ? $actionrecord['TASK_type'] : '');
        $table->addData(dict('GEN_LBL_TYPE') . ':', $types->source());
        $table->addCell(dict('GEN_LBL_DESCRIPTION'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if (isset($msg['TASK_description'])) {
		  $table->addCell($msg['TASK_description'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        }
        $c = new htmlObject('textarea', htmlspecialchars($actionrecord['TASK_description']), array('name' => 'TASK_description', 'rows' => CLX_NUM_15, 'cols' => CLX_NUM_80));
        $c->forceclose = true;
        $table->addCell($c->source(), array(CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $div = new htmlDiv($this->code, 'visibility:hidden; display:none', $table->source());
        return CLX_CHAR_NL . CLX_CHAR_NL . $div->source() . CLX_CHAR_NL . CLX_CHAR_NL;
    }

    /**
     * Validate Panel User Input before saving record.
     * @param array $msg
     */
    function checkPanel(&$msg) {
        if (UserRequest::$post['EVNT_language'] <> 'PHP' && UserRequest::$post['EVNT_language'] <> 'COMP') {
		  $msg['EVNT_action'] = dict('EVNT_LBL_PHP_ONLY');
        } else {
		  $ACUS = &getTool('ACUS');
		  if (!$ACUS->userExists(UserRequest::$post['TASK_assignedto'])) {
			 $msg['TASK_assignedto'] = dict('GEN_LBL_USER_DONT_EXIST');
		  }
		  if (empty(UserRequest::$post['TASK_subject'])) {
			 $msg['TASK_subject'] = dict('GEN_LBL_SUBJECT_MISS');
		  }
		  xValidate::date($msg, dict('GEN_LBL_START_DATE'), UserRequest::$post['TASK_start_date'], 'TASK_start_date');
		  xValidate::date($msg, dict('GEN_LBL_DUE_DATE'), UserRequest::$post['TASK_due_date'], 'TASK_due_date');
        }
    }

    /**
     * Build object record from user post
     * @param array $actionrecord
     */
    function getPanel(&$actionrecord) {
        $actionrecord['EXTRA1'] = CLX_INIT_ZERO;
        $actionrecord['EXTRA2'] = UserRequest::$post['TASK_assignedto'];
        $actionrecord['EXTRA3'] = UserRequest::$post['TASK_subject'];
        $actionrecord['EXTRA4'] = UserRequest::$post['TASK_start_date'];
        $actionrecord['EXTRA5'] = UserRequest::$post['TASK_due_date'];
        $actionrecord['EXTRA6'] = UserRequest::$post['TASK_priority'] . '|' . UserRequest::$post['TASK_type'] . '|' . UserRequest::$post['TASK_description'];
    }

    /**
     * Sets edit record required on forms of user interface.
     * @param array $actionrecord
     * @param array $values
     */
    function setPanel(&$actionrecord, $values) {
        if ($values['ACTION'] == $this->code) {
		  $actionrecord['TASK_assignedto'] = $values['EXTRA2'];
		  $actionrecord['TASK_subject'] = $values['EXTRA3'];
		  $actionrecord['TASK_start_date'] = $values['EXTRA4'];
		  $actionrecord['TASK_due_date'] = $values['EXTRA5'];
		  $info = explode('|', $values['EXTRA6']);
		  $actionrecord['TASK_priority'] = $info[CLX_NUM_ZERO];
		  $actionrecord['TASK_type'] = $info[CLX_NUM_ONE];
		  $actionrecord['TASK_description'] = $info[CLX_NUM_TWO];
        } else {
		  $this->defaultPanel($actionrecord);
        }
    }

    /**
     * Sets the default values for form on create event.
     * @param array $actionrecord
     */
    function defaultPanel(&$actionrecord) {
        $actionrecord['TASK_assignedto'] = $_SESSION['user_record']['username'];
        $actionrecord['TASK_subject'] = '';
        $actionrecord['TASK_start_date'] = 'today';
        $actionrecord['TASK_due_date'] = 'next week';
        $actionrecord['TASK_priority'] = 'MED';
        $actionrecord['TASK_type'] = 'WORK';
        $actionrecord['TASK_description'] = '';
    }

    /**
     * Build panel for display form
     * @param obj $table
     * @param array $record
     */
    function viewForm(&$table, $record) {
        $table->addData(dict('GEN_LBL_ACTION') . ':', $this->name);
        $table->addData(dict('GEN_LBL_ASSIGNED_TO') . ':', $record['TASK_assignedto']);
        $table->addData(dict('GEN_LBL_SUBJECT') . ':', $record['TASK_subject']);
        $table->addData(dict('GEN_LBL_START_DATE') . ':', $record['TASK_start_date']);
        $table->addData(dict('GEN_LBL_DUE_DATE') . ':', $record['TASK_due_date']);
        $table->addData(dict('GEN_LBL_DESCRIPTION') . ':', $record['TASK_description']);
    }

    /**
     * Builds description about the object on event panel listing.
     * @param array $record
     * @return string
     */
    function describe($record) {
        return dict('GEN_LBL_CREATE_TASK_TO') . ': ' . $record['EXTRA2'];
    }

    /**
     * Execute code according object type.
     * @param string $event
     * @param array $record
     * @param array $msg
     */
    function execute3($event, &$record, &$msg) {IF (ISSET($GLOBALS['ADDRDEBUGEVENTS'])) { echo __METHOD__.' '. __FILE__ .':'.__LINE__; pre($event);pre($record); }
        $info = explode('|', $event['EXTRA6']);
        $record = array();
        $record['STATUS'] = 'IDLE';
        $record['TYPE'] = $info[CLX_NUM_ONE];
        $record['SUBJECT'] = $event['EXTRA3'];
        $record['DESCRIPTION'] = $info[CLX_NUM_TWO];
        $record['PERCENT_COMPLETE'] = '0';
        $record['COMPLETE_DATE'] = 'null';
        $record['PRIORITY'] = $info[CLX_NUM_ZERO];
        $record['OWNERCODE'] = $_SESSION['user_record']['username'];
        $record['ASSIGNEDTO'] = $event['EXTRA2'];
        $record['REMINDER_DATETIME'] = 'null';
        $record['RECURRING'] = 'NONE';
        $record['DEPENDS'] = '0';
        $record['START_DATE'] = xFormat::date($event['EXTRA4'], 'Y-m-d');
        $record['DUE_DATE'] = xFormat::date($event['EXTRA5'], 'Y-m-d');
        ClxCommon::$db->createRecordVoid('CLX_US_TASKS', $record, false);
    }

}
