<?php

/**
 * General field object class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version    	$Rev: 9615 $
 *
 * Change Log
 *  Added queryFieldColumn Method
 *  Added Oracle on field properties
 *  Oracle uppercase record changed
 * 		métodos: queryFieldGroups() y queryFieldSorts(): Se toma en cuenta la nomenclatura utilizada para los campos derivados (work fields), la cual lleva un underscore en vez de un punto como separador entre el alias de la tabla y el nombre del campo
 */
class dbft extends clx_object {

    var $operators;
    var $debug = false;
    var $obj_cache = false;

    /**
     * CONSTRUCTOR
     */
    function __construct() {
        $this->code = 'DBFT';
        $this->name = 'Generic Clientix Field';
        $this->type = '----';
        $this->version = '0.7.11.3';
        $this->description = 'This is a Generic Clientix Field';
        $this->multifield = false; // Describes it field is a multi field see properties;
        $this->has_lookup = true; // Decribes if field can be set as copy field on lookup
        $this->has_question = true; // Describes if field can be used on queries criteria
        $this->operators = array('==' => 'GEN_LBL_EQUALTO', '!= ' => 'GEN_LBL_NOTEQUALTO');
    	IF (ISSET($GLOBALS['ADDRDEBUGEDIT']) && $GLOBALS['ADDRDEBUGEDIT']==$_SERVER['REMOTE_ADDR']) {
			$this->debug=true;
		}

    }

    /**
     * The following procedures are intented to be used by the Database
     * Administrator to interface with the user when creating new fields in the database.
     */
    function panel($msg, $fieldrecord, &$form) {
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell(dict('GEN_LBL_FIELDPANEL') . ': ' . $this->code, array(CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $div = new htmlDiv($this->code, 'visibility:hidden; display:none', $table->source());
        return CLX_CHAR_NL . CLX_CHAR_NL . $div->source() . CLX_CHAR_NL . CLX_CHAR_NL;
    }

    function checkPanel(&$msg) {
        if ($this->debug) {
		  print_r($msg);
        }
    }

    function savePanel(&$fieldrecord) {
        if ($this->debug) {
		  print_r($fieldrecord);
        }
    }

    function loadPanel(&$fieldrecord, $values) {
        if ($this->debug) {
		  print_r($fieldrecord);
		  print_r($values);
        }
    }

    function deletePanel($fieldrecord) {
        if ($this->debug) {
		  print_r($fieldrecord);
        }
    }

    function defaultPanel(&$fieldrecord) {
        if ($this->debug) {
		  print_r($fieldrecord);
        }
    }

    function viewForm(&$table, $record) {
        if ($this->debug) {
		  print_r($table);
		  print_r($record);
        }
    }

    function fieldProperties($fieldrecord) {
        $props = array();
        switch (kDatabaseType) {
		  case 'oci':
			 $props['type'] = 'number(1)';
			 break;
		  default:
			 $props['type'] = 'tinyint(1) unsigned';
			 break;
        }
        if ($fieldrecord['FIELD_DEFAULT'] === 'null' || strlen($fieldrecord['FIELD_DEFAULT']) == CLX_NUM_ZERO) {
		  $props['default'] = 'null';
        } else {
		  $props['default'] = $fieldrecord['FIELD_DEFAULT'];
        }
        $props['properties'] = '';
        $props['null'] = ($fieldrecord['FIELD_REQUIRED']) ? 'NO' : 'YES';
        return $props;
    }

    /**
     * Inserts field according to type
     * @param type $tablename
     * @param type $fieldrecord
     */
    function createPhysicalFieldByType($tablename, $fieldrecord) {
        if (kDatabaseType == 'oci') {
	        $fieldinfo = $this->fieldProperties($fieldrecord);
			$extra='';
			if ($fieldinfo['default']=='null' && $fieldinfo['null']=='NO' &&  ClxCommon::$db->countRecords($tablename, '') !== CLX_NUM_ZERO) {
				$fieldinfo['null']='YES';
				$extra=' '.dict('GEN_LBL_CHANGE_NOT_NULL');
			}
     		ClxCommon::$db->createField($tablename, $fieldrecord['FIELDNAME'], $fieldinfo);

    	}
    	else {
        	$props = $this->fieldProperties($fieldrecord);
        	ClxCommon::$db->createField($tablename, $fieldrecord['FIELDNAME'], $props);
		}
    }

    function modifyField($oldtablename, $oldfieldrecord, $newtablename, $newfieldrecord) {
        if ($oldtablename != $newtablename) { // move this field to other table!
		  $this->deleteField($oldtablename, $oldfieldrecord);
		  $this->createPhysicalFieldByType($newtablename, $newfieldrecord);
        } else {
		  $props = $this->fieldProperties($newfieldrecord);
		  if ($this->multifield === true) {
			 reset($props);
			 foreach ($props as $key => $prop) {
			     ClxCommon::$db->modifyField($newtablename, $oldfieldrecord['FIELDNAME'] . '_' . $key, $newfieldrecord['FIELDNAME'] . '_' . $key, $prop);
			 }
		  } else {
			 ClxCommon::$db->modifyField($newtablename, $oldfieldrecord['FIELDNAME'], $newfieldrecord['FIELDNAME'], $props);
		  }
        }
    }

    function deleteField($tablename, $fieldrecord) {
        ClxCommon::$db->dropField($tablename, $fieldrecord['FIELDNAME']);
    }

    function defaultField($form_fieldname, $fieldrecord, &$record) {
        $record[$form_fieldname] = trim($fieldrecord['FIELD_DEFAULT']);
    }

    function placeEditField($msg, $form_fieldname, $fieldrecord, $record, &$form, $jsevents) {
		IF (ISSET($GLOBALS['ADDRDEBUGDICT']) && $GLOBALS['ADDRDEBUGDICT']==$_SERVER['REMOTE_ADDR'])  { pre( '<font color="green">DBFT : '.__LINE__ ." ".$form_fieldname.'</font>') ;}
                   
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_WIDTH => CLX_PROP_100PCT));
        if ($fieldrecord['FIELD_FORM_REQUIRED'] != CLX_NUM_TWO) {
		  $fieldrecord['FIELD_REQUIRED'] = $fieldrecord['FIELD_FORM_REQUIRED'];
        }
        if (!empty($fieldrecord['FIELD_HIDDEN'])) {
		  $form->addArgument($form_fieldname, (array_key_exists($form_fieldname, $record)) ? $record[$form_fieldname] : '');
		  if (empty($fieldrecord['FIELD_HIDDEN'])) {
			 $table = $this->placeViewField($form_fieldname, $fieldrecord, $record, $msg);
		  }
			IF (ISSET($GLOBALS['ADDRDEBUGDICT']) && $GLOBALS['ADDRDEBUGDICT']==$_SERVER['REMOTE_ADDR'])  { pre( '<font color="green">DBFT : '.__LINE__ ." ".$form_fieldname.'</font>') ;}
        
        } else {
			IF (ISSET($GLOBALS['ADDRDEBUGDICT']) && $GLOBALS['ADDRDEBUGDICT']==$_SERVER['REMOTE_ADDR'])  { pre( '<font color="green">DBFT : '.__LINE__ ." ".$form_fieldname.'</font>') ;}
        
		  $cpt = dict($fieldrecord['CAPTION']);
		  if (strlen($cpt) > CLX_NUM_ZERO) {
			 $table->addCell($cpt, array(CLX_PROP_CLASS => $fieldrecord['CAPTION_CLASS'], CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  }
		  if (isset($msg[$form_fieldname])) {
			 $table->addCell($msg[$form_fieldname], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  }
		  $fieldobj = $this->fieldObject($form_fieldname, $fieldrecord, $record);
		  foreach ($jsevents as $key => $value) {
			 $fieldobj->setProp($key, $value);
		  }
		  $pfx = dict($fieldrecord['FIELDPREFIX']);
		  $fieldsource = (strlen($pfx) > CLX_NUM_ZERO) ? $pfx : '';
			IF (ISSET($GLOBALS['ADDRDEBUGDICT']) && $GLOBALS['ADDRDEBUGDICT']==$_SERVER['REMOTE_ADDR'])  { 
				pre( '<font color="green">DBFT : '.__LINE__ ." ".$form_fieldname.'</font>') ;
          	 //  pre($fieldobj->source());
			}
			
			$fieldsource .= $fieldobj->source();
					  $sfx = dict($fieldrecord['FIELDSUFIX']);
		
		$fieldsource .= (strlen($sfx) > CLX_NUM_ZERO) ? $sfx : '';
		  $class = ($fieldrecord['FIELD_REQUIRED']) ? CLX_CLASS_PROMPTREQUIRED : CLX_CLASS_PROMPT;
		  //Added to Display Date Format
		  $fieldlabel = dict($fieldrecord['FIELDFORMLABEL']);
		  $table->addCell($fieldlabel, array(CLX_PROP_CLASS => $class, CLX_PROP_ID => 'LBL_' . $form_fieldname));
           
		  if (array_key_exists('SRCH_LISTING', $fieldrecord) && $fieldrecord['SRCH_LISTING'] <> '') {
                $fieldsource .= getSkinIcon('view', 'javascript:formToCookie(' . singleQuote('theForm') . ');openWindow(' . singleQuote(CLX_CHAR_EMPTY_STRING) . ', 	' . singleQuote('pop.php?to=EDIT&nofr=1&no_menu=1&no_alerts=1&pfid=' . $fieldrecord['PFID'] . '&ac=POPLIST&li=' . $fieldrecord['SRCH_LISTING']) . ',800, 400,null,'.singlequote($form_fieldname).','.singlequote( __FILE__ .'_'. __LINE__ ) .');', dict('GEN_LBL_SEARCH'));
		  }
			
		  $table->addCell($fieldsource, array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_ID => 'FLD_' . $form_fieldname));
			//IF (ISSET($GLOBALS['ADDRDEBUGDICT']) && $GLOBALS['ADDRDEBUGDICT']==$_SERVER['REMOTE_ADDR'])  {
			//echo $table->source();

		//	}
			if ( $this->debug==true && isset($GLOBALS['placelistcount']) && $GLOBALS['placelistcount']<150  ) {
				$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo4.txt","a+");$posx=$pos=0;
				$te=debug_backtrace();
				fwrite($tta,chr(13).chr(10).$pos."_".$te[$posx]["function"]."dbft-hora:".date("h:i:s d/m/Y").chr(13).chr(10));
				for ($t=0;$t<12;$t++) {
					if (isset($te[$t]["file"]))
						fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));
					
				}
				if (!isset($GLOBALS['placelistcount']))
					$GLOBALS['placelistcount']=1;
				else
					$GLOBALS['placelistcount']++;
				fwrite($tta,chr(13).chr(10)."placeListFieldText: ".print_r($fieldsource,true));
				fclose($tta);
			}
		
        }
        return $table;
    }

    function fieldObject(&$form_fieldname, &$fieldrecord, $record) {
        if (!array_key_exists($form_fieldname, $record) || strlen($record[$form_fieldname]) == CLX_NUM_ZERO) {
		  $record[$form_fieldname] = $fieldrecord['FIELD_DEFAULT'];
        }

        if ($fieldrecord['FIELDTYPE'] === 'SOBJ') {
		  $field_displaysize = '';
		  $field_size = $field_displaysize;
        } else {

        }
		IF (ISSET($GLOBALS['ADDRDEBUGDICT']) && $GLOBALS['ADDRDEBUGDICT']==$_SERVER['REMOTE_ADDR'])  {
				pre( "=". __LINE__ ."====>dbft ". $form_fieldname);
		}
        //Toma valor de la forma. si el valor de la forma es 0 toma el del campo y si el campo es 0 entonces lo deja en blanco
        $field_size = $fieldrecord['FIELD_SIZE'];
        if (empty($field_size) && array_key_exists('FIELD_DB_SIZE', $fieldrecord))
		  $field_size = $fieldrecord['FIELD_DB_SIZE'];
        $field_displaysize = $fieldrecord['FIELD_DISPLAYSIZE'];
        if (empty($field_displaysize) && array_key_exists('FIELD_DB_SIZE', $fieldrecord))
		  $field_displaysize = $fieldrecord['FIELD_DB_DISPLAYSIZE'];
        //Verificando campos que tengan display size = 0
        if ($field_size == CLX_NUM_ZERO)
		  $field_size = '';
        if ($field_displaysize == CLX_NUM_ZERO)
		  $field_displaysize = '';
        if (!empty($fieldrecord['FIELD_PASSWORD']) && $fieldrecord['FIELD_PASSWORD']) {
		  $fieldobj = new htmlPassword($form_fieldname, $record[$form_fieldname], $field_displaysize, $field_size);
        } else {
			IF (ISSET($GLOBALS['ADDRDEBUGDICT']) && $GLOBALS['ADDRDEBUGDICT']==$_SERVER['REMOTE_ADDR'])  {
				pre( "=". __LINE__ ."====>dbft ". $form_fieldname);
			}
		  $fieldobj = new htmlField($form_fieldname, $record[$form_fieldname], $field_displaysize, $field_size);
		  //Added instead of displaying read only files
		  if (!empty($fieldrecord['FIELD_READONLY'])and ( !$fieldrecord["FIELD_REQUIRED"] or ( $fieldrecord["FIELD_REQUIRED"] and $record[$form_fieldname] <> '' ))) {
			 $fieldobj->setProp('readonly', 'true');
			 $fieldobj->setProp(CLX_PROP_CLASS, 'readonly');
		  }
		  return $fieldobj;
        }
        return $fieldobj;
    }

    function placeViewField($form_fieldname, $fieldrecord, $record, $msg = array()) {
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_WIDTH => CLX_PROP_100PCT));
        if (empty($fieldrecord['FIELD_HIDDEN'])) {
		  $cpt = dict($fieldrecord['CAPTION']);
		  if (strlen($cpt) > CLX_NUM_ZERO) {
			 $table->addCell($cpt, array(CLX_PROP_CLASS => $fieldrecord['CAPTION_CLASS'], CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  }
		  if (!empty($msg[$form_fieldname])) {
			 $table->addCell($msg[$form_fieldname], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  }
		  //$pfx = dict($fieldrecord['FIELDPREFIX']);
		  //$sfx = dict($fieldrecord['FIELDSUFIX']);
		  $pfx = (array_key_exists('FIELDPREFIX_VAL', $fieldrecord)) ? $fieldrecord['FIELDPREFIX_VAL'] : dict($fieldrecord['FIELDPREFIX']);
		  $sfx = (array_key_exists('FIELDSUFIX_VAL', $fieldrecord)) ? $fieldrecord['FIELDSUFIX_VAL'] : dict($fieldrecord['FIELDSUFIX']);
		  $cell = $this->formatField($form_fieldname, $fieldrecord, $record);
		  // Field Anchor
		  $cell = $this->placeFieldAnchor($form_fieldname, $fieldrecord, $record, $cell);
		  $table->addCell(dict($fieldrecord['FIELDFORMLABEL']), array(CLX_PROP_CLASS => CLX_CLASS_PROMPT));
		  $table->addCell($pfx . ((strlen($cell) == CLX_NUM_ZERO) ? CLX_CHAR_HTML_NBSP : $cell) . $sfx, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        }
        return $table;
    }

    function placeSearchField($form_fieldname, $fieldrecord, $record, &$form) {
		if ( $this->debug==true ) {$posx=0;$pos=1611;$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");fwrite($tta,print_r($record,true));$te=debug_backtrace(); for ($t=0;$t<12;$t++) {if (isset($te[$t]["file"])){if ($t==$posx)fwrite($tta,chr(13).chr(10).$pos."_".$te[$posx]["function"]."_dbft -hora:".date("h:i:s d/m/Y").chr(13).chr(10));fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));}}fclose($tta);}
		$table = new htmlTable(CLX_WIDTH_FOUR);
        $table->setProps(array(CLX_PROP_WIDTH => CLX_PROP_100PCT));
        // Prompt
        $table->addCell(dict($fieldrecord['FIELDFORMLABEL']), array(CLX_PROP_CLASS => CLX_CLASS_PROMPT, CLX_PROP_WIDTH => CLX_NUM_HUNDRED));
        // Operator
        $operators = new htmlSelect($form_fieldname . '_operator');
        $operators->addItem('', '', !isset($record[$form_fieldname . '_operator']));
        foreach ($this->operators as $key => $value) {
		  $operators->addItem(dict($value), $key, (!empty($record[$form_fieldname . '_operator']) && $record[$form_fieldname . '_operator'] == $key));
        }
        $table->addCell($operators->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_WIDTH => '150'));
        // Field
        $pfx = dict($fieldrecord['FIELDPREFIX']);
        $sfx = dict($fieldrecord['FIELDSUFIX']);
        $fieldobj = $this->searchFieldObject($form_fieldname, $fieldrecord, $record);
        $table->addCell($pfx . $fieldobj->source() . $sfx, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        // Booleans
        if (empty($record[$form_fieldname . '_boolean']))
		  $record[$form_fieldname . '_boolean'] = '';
        $booleans = new htmlSelect($form_fieldname . '_boolean');
        $booleans->addItem('', '', ($record[$form_fieldname . '_boolean'] === ''));
        $booleans->addItem('And', 'AND', ($record[$form_fieldname . '_boolean'] === 'AND'));
        $booleans->addItem('Or', 'OR', ($record[$form_fieldname . '_boolean'] === 'OR'));
        $table->addCell($booleans->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_ALIGN => CLX_PROP_RIGHT, CLX_PROP_WIDTH => '20'));
        return $table;
    }

    function searchFieldObject($form_fieldname, $fieldrecord, $record) {
        return $this->fieldObject($form_fieldname, $fieldrecord, $record);
    }

    /**
     * Load session filters
     * @param type $tablecode
     * @param type $fieldname
     * @param type $fieldrecord
     * @param type $record
     * @return string
     */
    function setQuickFilters($tablecode, $fieldname, $fieldrecord, $record) {
        $criteria = '';
        if (array_key_exists($tablecode . '_' . $fieldname, $record) && strlen($record[$tablecode . '_' . $fieldname]) > CLX_NUM_ZERO) {
		  $criteria = $tablecode . '.' . $fieldname .' = ' .singleQuote(str_replace(CLX_CHAR_SINGLE_QUOTE, '', $record[$tablecode . '_' . $fieldname]));
        }
        return $criteria;
    }

    function placeListField($form_fieldname, $fieldrecord, $record, $class, &$table, $csv=1) {if ( isset($_SESSION["_dEBUGX"]) and $_SESSION["_dEBUGX"]==99) {$rra=fopen(kPathToWebSite."/".kPathToTemp."archivox4x.txt","a+");	fwrite($rra,"p9:".memory_get_usage().chr(13).chr(10));	fclose($rra);}
		if ( !empty($_SESSION["_dEBUG"]) && $_SESSION["_dEBUG"]==99 ) {$posx=0;$pos=1611;$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");fwrite($tta,print_r($record,true));$te=debug_backtrace(); for ($t=0;$t<12;$t++) {if (isset($te[$t]["file"])){if ($t==$posx)fwrite($tta,chr(13).chr(10).$pos."_".$te[$posx]["function"]."_dbft hora:".date("h:i:s d/m/Y").chr(13).chr(10));fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));}}fclose($tta);}    $curprefix = '';


		$curprefix = '';
        if (array_key_exists('DBQF_CURRENCY_FIELD_TBL', $fieldrecord)) {
			$curfieldname = $fieldrecord['DBQF_CURRENCY_FIELD_TBL'] . '_' . $fieldrecord['DBQF_CURRENCY_FIELD'];
			if (array_key_exists($curfieldname, $record)) {
				$fieldrecord['DBQF_CURRENCY_FIELD_INFO']['DBQF_FIELD_FORMAT'] = $fieldrecord['DBQF_FIELD_FORMAT'];
				$fieldinstance = getToolObject('DBFT', $fieldrecord['DBQF_CURRENCY_FIELD_INFO']['FIELDTYPE']);
				$curprefix = $fieldinstance->formatField($curfieldname, $fieldrecord['DBQF_CURRENCY_FIELD_INFO'], $record);
		  }
        }   if ( isset($_SESSION["_dEBUGX"]) and $_SESSION["_dEBUGX"]==99) {$rra=fopen(kPathToWebSite."/".kPathToTemp."archivox4x.txt","a+");	fwrite($rra,"p29:".memory_get_usage().chr(13).chr(10));	fclose($rra);}
		IF ($csv==1 && isset($_GET['ac']) && ($_GET['ac'] =='BUILDCSV' OR $_GET['ac'] =='BUILDCSV_BATCH')) {
			$cell = $form_fieldname;
			$table->addCell($cell, "class", $class);
        }
		else {
	        //Alignment
	        switch ($fieldrecord['FIELD_ALIGN']) {
			  case CLX_NUM_ONE:
				 $align = 'text-align: center; white-space: no-wrap;';
				 break;
			  case CLX_NUM_TWO:
				 $align = 'text-align: right; white-space: no-wrap;';
				 break;
			  default:
				 $align = 'text-align: left; white-space: no-wrap;';
				 break;
	        }
			$pfx = (array_key_exists('FIELDPREFIX_VAL', $fieldrecord)) ? $fieldrecord['FIELDPREFIX_VAL'] : dict($fieldrecord['FIELDPREFIX']);
			$sfx = (array_key_exists('FIELDSUFIX_VAL', $fieldrecord)) ? $fieldrecord['FIELDSUFIX_VAL'] : dict($fieldrecord['FIELDSUFIX']);
			$form_fieldname = uppercase($form_fieldname);
			$cell = $this->formatField($form_fieldname, $fieldrecord, $record);
				// Field Anchor
			$cell = $this->placeFieldAnchor($form_fieldname, $fieldrecord, $record, $cell);
			if ($csv==1) {if ( isset($_SESSION["_dEBUGX"]) and $_SESSION["_dEBUGX"]==99) {$rra=fopen(kPathToWebSite."/".kPathToTemp."archivox4x.txt","a+");	fwrite($rra,"p98:".memory_get_usage().chr(13).chr(10));	fclose($rra);}
				$table->addCell($pfx . ' ' . $curprefix . ' ' . $cell . ' ' . $sfx, array(CLX_PROP_CLASS => $class));
				if ( isset($_SESSION["_dEBUGX"]) and $_SESSION["_dEBUGX"]==99) {$rra=fopen(kPathToWebSite."/".kPathToTemp."archivox4x.txt","a+");	fwrite($rra,"p99:".memory_get_usage().chr(13).chr(10));	fclose($rra);}
			}else {
				if ( isset($_SESSION["_dEBUGX"]) and $_SESSION["_dEBUGX"]==99) {$rra=fopen(kPathToWebSite."/".kPathToTemp."archivox4x.txt","a+");	fwrite($rra,"p97:".memory_get_usage().chr(13).chr(10));	fclose($rra);}
				$div = new htmlObject('div');
				$div->setProp('style', $align);
				$div->setProp(CLX_PROP_CLASS, $class);
				
		        $div->setContents($pfx . ' ' . $curprefix . ' ' . $cell . ' ' . $sfx);
		        $table->addCell($div->source(), array(CLX_PROP_CLASS => $class));
				$div=null;
				unset($pfx);
				unset($sfx);
				unset($cell);
				unset($form_fieldname);
				unset($fieldrecord);
				unset($div);
				if ( isset($_SESSION["_dEBUGX"]) and $_SESSION["_dEBUGX"]==99) {$rra=fopen(kPathToWebSite."/".kPathToTemp."archivox4x.txt","a+");	fwrite($rra,"p96:".memory_get_usage().chr(13).chr(10));	fclose($rra);}
			}
		}
		if ( $this->debug==true && !isset($GLOBALS['placelistcount']) && $GLOBALS['placelistcount']<200) {
			$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo4.txt","a+");
			fwrite($tta,chr(13).chr(10).$pos."_".$te[$posx]["function"]."dbft_-hora:".date("h:i:s d/m/Y").chr(13).chr(10));
			$te=debug_backtrace();
			for ($t=0;$t<12;$t++) {
				if (isset($te[$t]["file"]))
					fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));
				
			}
			if (!isset($GLOBALS['placelistcount']))
				$GLOBALS['placelistcount']=1;
			else
				$GLOBALS['placelistcount']++;
			fwrite($tta,chr(13).chr(10)."placeListFieldText: ".print_r($cell,true));
			fclose($tta);
		}
    }

    /**
     * Sets cache if required
     * @param type $fieldrecord
     */
    function set_obj_cache(&$fieldrecord) {

    }

    /**
     * Formats field in text only
     * @param string $form_fieldname
     * @param array $fieldrecord
     * @param array $record
     * @param bool $append_quote when true value will attach ' to prevent number interpretation
     */
    function placeListFieldText($form_fieldname, $fieldrecord, $record, $append_quote = true, $csv=1) {
		$pfx = (array_key_exists('FIELDPREFIX_VAL', $fieldrecord)) ? $fieldrecord['FIELDPREFIX_VAL'] : dict($fieldrecord['FIELDPREFIX']);
        $sfx = (array_key_exists('FIELDSUFIX_VAL', $fieldrecord)) ? $fieldrecord['FIELDSUFIX_VAL'] : dict($fieldrecord['FIELDSUFIX']);
       // $form_fieldname = uppercase($form_fieldname);
        if (!empty($fieldrecord['DBQF_FIELD_FORMAT']) && $fieldrecord['DBQF_FIELD_FORMAT'] == CLX_NUM_ONE) {
            if ( isset($_SESSION["_dEBUGX"]) and $_SESSION["_dEBUGX"]==99) {$rra=fopen(kPathToWebSite."/".kPathToTemp."archivox4x.txt","a+");	fwrite($rra,"p85:".memory_get_usage().chr(13).chr(10));	fclose($rra);}
		  	$cell = $record[$form_fieldname];
        } else {
		  $cell = $this->formatField($form_fieldname, $fieldrecord, $record);
        }
		$cell = html_entity_decode (strip_tags(str_replace(array(CLX_CHAR_NL, CLX_CHAR_CR, CLX_CHAR_TAB), '', $pfx . $cell . $sfx)));
		if ( isset($_SESSION["_dEBUGX"]) and $_SESSION["_dEBUGX"]==99) {$rra=fopen(kPathToWebSite."/".kPathToTemp."archivox4x.txt","a+");	fwrite($rra,"p88:".memory_get_usage().chr(13).chr(10));	fclose($rra);	}
        $cell=trim($cell);
		if ($append_quote === true && substr($cell,0,1)==='0' ) {
            $cell = CLX_CHAR_SINGLE_QUOTE.trim($cell);
		}
		unset ($pfx); unset($sfx);	
		unset($form_fieldname);
		unset($fieldrecord);
		unset($record);
        return $cell;
    }

    /**
     * Formats field in text only
     * @param string $form_fieldname
     * @param array $fieldrecord
     * @param array $record
     * @param bool $append_quote when true value will attach ' to prevent number interpretation
     */
    function placeListFieldNoCodes($form_fieldname, $fieldrecord, $record, $append_quote = true) {
        $fieldrecord['NOCODES'] = true;
        return $this->placeListFieldText($form_fieldname, $fieldrecord, $record, $append_quote,0);
    }

    function validateField(&$msg, $form_fieldname, $fieldrecord, &$record, &$table, &$form) {
        if ($this->debug) {
		  print_r($msg);
		  print_r($form_fieldname);
		  print_r($fieldrecord);
		  print_r($record);
		  print_r($table);
		  print_r($form);
        }
    }

    function validateImportField(&$msg, $fieldrecord, $fieldvalue) {
        if ($fieldrecord['FIELD_REQUIRED'] == CLX_NUM_ZERO && $fieldvalue === 'null') {
		  return;
        } else {
		  if ((strlen($fieldvalue) == CLX_NUM_ZERO) && $fieldrecord['FIELD_REQUIRED']) {
			 $msg[$fieldrecord['FIELDNAME']] = dict('GEN_LBL_FIELDREQUIRED');
		  }
        }
    }

    function saveField($form_fieldname, $fieldrecord, &$record, $recordid) {
        if (array_key_exists($form_fieldname, UserRequest::$post)) {
		  if (is_array(UserRequest::$post[$form_fieldname])) {
			 UserRequest::$post[$form_fieldname] = toString(UserRequest::$post[$form_fieldname], CLX_CHAR_SINGLE_QUOTE);
		  }
		  $record[$fieldrecord['FIELDNAME']] = UserRequest::$post[$form_fieldname];
        } else {
		  $record[$fieldrecord['FIELDNAME']] = false;
        }
        if ($fieldrecord['FIELD_REQUIRED'] == CLX_NUM_ZERO && $record[$fieldrecord['FIELDNAME']] === '') {
		  $record[$fieldrecord['FIELDNAME']] = 'null';
        }
    }

    function saveFieldImport($form_fieldname, $fieldrecord, &$record) {
        if ($record[$form_fieldname] === '' && $fieldrecord['FIELD_REQUIRED'] == CLX_NUM_ZERO) {
		  $record[$form_fieldname] = 'null';
        }
    }

    function loadField($form_fieldname, &$fieldrecord, &$record, $values) {
        if (!isset($values[$fieldrecord['FIELDNAME']])){
		   $record[$form_fieldname]='';	
		}else{
			$record[$form_fieldname] = $values[$fieldrecord['FIELDNAME']];
		}	
    }

    function removeField($fieldrecord, $record) {

    }

    function formatField($form_fieldname, $fieldrecord, $record) {
        $form_fieldname = uppercase($form_fieldname);
        //$record = array_change_key_case($record, CASE_UPPER);
        if (!empty($fieldrecord['FIELD_PASSWORD']) && $fieldrecord['FIELD_PASSWORD']) {
		  $la = strlen($record[$form_fieldname]);
		  $input = '';
			if ( isset($_SESSION["_dEBUGX"]) and $_SESSION["_dEBUGX"]==99) {$rra=fopen(kPathToWebSite."/".kPathToTemp."archivox4x.txt","a+");	fwrite($rra,"p16:".memory_get_usage().chr(13).chr(10));	fclose($rra);	}
		  return str_pad($input, $la, '*');
        } else {
		  if (!array_key_exists($form_fieldname, $record)) {
			 $record[$form_fieldname] = '';
		  }
			if ( isset($_SESSION["_dEBUGX"]) and $_SESSION["_dEBUGX"]==99) {$rra=fopen(kPathToWebSite."/".kPathToTemp."archivox4x.txt","a+");	fwrite($rra,"p15:".memory_get_usage().chr(13).chr(10));	fclose($rra);	}
		  return $record[$form_fieldname];
        }
    }

    function queryFieldColumn($fieldname, $tablecode = '', $expression = '') {
        if (strlen($tablecode) > CLX_NUM_ZERO) {
		  $fieldname = $tablecode . '.' . $fieldname . ' ' . $tablecode . '_' . $fieldname;
        }
        if (strlen($expression) > CLX_NUM_ZERO) {
		  $fieldname = $expression;
        }
        return $fieldname;
    }

// End method
    function queryFieldSorts($fieldname, $tablecode = '', $sort = '', $work_field = false) {
        if (strlen($tablecode) > CLX_NUM_ZERO) {
		  if ($work_field === true) {
			 $fieldname = $tablecode . '_' . $fieldname;
		  } else {
			 $fieldname = $tablecode . '.' . $fieldname;
		  }
        }
        return $fieldname . ' ' . $sort;
    }

// End method
    function queryFieldGroups($fieldname, $tablecode = '', $work_field = false) {
        if (strlen($tablecode) > CLX_NUM_ZERO) {
		  if ($work_field === true) {
			 switch (kDatabaseType) {
			     case 'oci':
			         $fieldname = $tablecode . '.' . $fieldname;
			         break;
			     default:
			         $fieldname = $tablecode . '_' . $fieldname;
			         break;
			 }
		  } else {
			 $fieldname = $tablecode . '.' . $fieldname;
		  }
        }
        return $fieldname;
    }

// End method
    /**
     * Similar to placeEdit but with the diference that this is used as html field object for tools
     * the point is to reuse the field funtionalities like calendar an others.
     * @param: array $msg, message array
     * @param: string $val, field value.
     * @param: string $type, type of field DATE, BOOL, ETC
     * @param: string $fname, form fieldname
     * @param: string $name, field tablename
     * @param: string $label, form label
     * @param: string $size
     */
    function placeEditFieldObj($formid, $msg, $val, $type, $fname, $name, $label, $size, $dplsize, $pfx, $sfx, $formula, $required, $hidden, $readonly, $jsevents = array(), $caption = '', $caption_class = '', $form = false) {
        $form_fieldname = $fname;
        $record = array($fname => $val);
        $fieldrecord = array();
        $fieldrecord['FIELDNAME'] = $name;
        $fieldrecord['FIELDFORMLABEL'] = $label;
        $fieldrecord['FIELDLISTLABEL'] = $label;
        $fieldrecord['FIELDPREFIX'] = $pfx;
        $fieldrecord['FIELDSUFIX'] = $sfx;
        $fieldrecord['FIELDTYPE'] = $type;
        $fieldrecord['FIELDFORMULA'] = $formula;
        $fieldrecord['FIELD_SIZE'] = $size;
        $fieldrecord['FIELD_DISPLAYSIZE'] = $dplsize;
        $fieldrecord['FIELD_DB_DISPLAYSIZE'] = $dplsize;
        $fieldrecord['FIELD_DB_SIZE'] = $dplsize;
        $fieldrecord['FIELD_REQUIRED'] = $required;
        $fieldrecord['FIELD_READONLY'] = $readonly;
        $fieldrecord['FIELD_HIDDEN'] = $hidden;
        $fieldrecord['CAPTION'] = $caption;
        $fieldrecord['CAPTION_CLASS'] = $caption_class;
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_WIDTH => CLX_PROP_100PCT));
        if ($fieldrecord['FIELD_HIDDEN'] === true) {
		  $form->addArgument($form_fieldname, $record[$form_fieldname]);
		  if ($fieldrecord['FIELD_READONLY'] === true && !$fieldrecord['FIELD_HIDDEN'] === true) {
			 $table = $this->placeViewField($form_fieldname, $fieldrecord, $record, $msg);
		  }
        } else {
		  $cpt = dict($fieldrecord['CAPTION']);
		  if (strlen($cpt) > CLX_NUM_ZERO) {
			 $table->addCell($cpt, array(CLX_PROP_CLASS => $fieldrecord['CAPTION_CLASS'], CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  }
		  if (isset($msg[$form_fieldname])) {
			 $table->addCell($msg[$form_fieldname], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  }
		  $fieldobj = $this->fieldObject($form_fieldname, $fieldrecord, $record);
		  foreach ($jsevents as $key => $value) {
			 $fieldobj->setProp($key, $value);
		  }
		  $pfx = dict($fieldrecord['FIELDPREFIX']);
		  $fieldsource = (strlen($pfx) > CLX_NUM_ZERO) ? $pfx : '';
		  $fieldsource .= $fieldobj->source();
		  $sfx = dict($fieldrecord['FIELDSUFIX']);
		  $fieldsource .= (strlen($sfx) > CLX_NUM_ZERO) ? $sfx : '';
		  $class = ($fieldrecord['FIELD_REQUIRED']) ? CLX_CLASS_PROMPTREQUIRED : CLX_CLASS_PROMPT;
		  //Added to Display Date Format
		  if ($fieldrecord['FIELDTYPE'] === 'DATE') {
			 $EDIT = &getTool('EDIT');
			 $format_label_dict = $EDIT->date_in_formats[$_SESSION['prefs']['DATE_FORMAT']];
			 $fieldlabel = dict($fieldrecord['FIELDFORMLABEL']) . '<br><span class=' . singleQuote('promptdateforma') . '>' . dict($format_label_dict) . '</span>';
		  } elseif ($fieldrecord['FIELDTYPE'] === 'DATI') {
			 $EDIT = &getTool('EDIT');
			 $format_label_dict = $EDIT->date_in_formats[$_SESSION['prefs']['DATE_FORMAT']] . "_TIME";
			 $fieldlabel = dict($fieldrecord['FIELDFORMLABEL']) . '<br><span class=' . singleQuote('promptdateforma') . '>' . dict($format_label_dict) . '</span>';
		  } else {
			 $fieldlabel = dict($fieldrecord['FIELDFORMLABEL']);
		  }
		  $table->addCell($fieldlabel, array(CLX_PROP_CLASS => $class));
		  //Added to display calendar link
		  if (!$fieldrecord['FIELD_READONLY'] === true && ($fieldrecord['FIELDTYPE'] === 'DATE' || $fieldrecord['FIELDTYPE'] === 'DATI')) {
			 $format_javascript = $_SESSION['prefs']['DATE_FORMAT'];
			 $format_javascript = str_replace('d', 'dd', $format_javascript);
			 $format_javascript = str_replace('m', 'mm', $format_javascript);
			 $format_javascript = str_replace('Y', 'yy', $format_javascript);

			 $picker = '<script type="text/javascript">
		  			    $(function() {
		  			    	var pickOpts = {
		  			    	 dateFormat: "' . $format_javascript . '",
		  			    	 showTime: true,
  							     stepMinutes: 1,
         						 stepHours: 1,
         						 altTimeField: "",
		  			    	 duration: "",
						   showTime: ' . (($fieldrecord['FIELDTYPE'] === 'DATI') ? 'true' : 'false') . ',' .
			         'constrainInput: false,
						   time24h: true,
						   changeMonth: true,
								 changeYear: true,
						   showOn: "button",
						   buttonImage: "./' . kPathToSkins . $_SESSION['skin'] . '/images/calendar/cal.gif",
						   buttonImageOnly: true
		  			    	};
			         		$("#' . $form_fieldname . '").datepicker(pickOpts);
			         	});
			         	</script>';
			 $fieldsource .=CLX_CHAR_NL . $picker;
		  }
		  //document.forms['tstest'].elements['input1']
		  $table->addCell($fieldsource, array(CLX_PROP_CLASS => CLX_CLASS_DATA, 'nowrap' => 'true'));
        }
        return $table;
    }

    /**
     * Set anchor for field
     */
    function placeFieldAnchor($form_fieldname, $fieldrecord, $record, $label = false) {
        if (empty($label)) {
		  $label = (array_key_exists($form_fieldname, $record)) ? $record[$form_fieldname] : '';
        }
        $label = trim(str_replace(CLX_CHAR_HTML_NBSP, '', $label));
        $anchor = $fieldrecord['FIELD_ANCHOR'];
        if (strlen($anchor) > CLX_NUM_ZERO && strlen($label) > CLX_NUM_ZERO) {
		  $formcode = substr($form_fieldname, CLX_NUM_ZERO, CLX_NUM_FOUR);
		  if (strstr($anchor, '!') !== false) {
			 reset($record);
			 foreach ($record as $k => $v) {
			     $anchor = str_replace('!' . str_replace($formcode . '_', '', $k), $record[$k], $anchor);
			 }
		  }
		  $a = new htmlAnchor($label, $anchor);
		  $label = $a->source();
		  unset($a, $anchor);
        }
        return $label;
    }

// End method
    function validateFieldQuestionary(&$msg, $form_fieldname, $fieldrecord, $tablecode, $record, $recordid) {

    }

    function parseField($form_fieldname, $fieldrec, &$record) {

    }

}

// End class