<?php

/**
 * Bar Chart field object class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version    	$Rev: 9615 $
 *
 * Change log
 *  Added Oracle on field properties
 *
 */
if (!class_exists('dbft') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'dbft.php';

class dbft_barcode extends dbft {

    var $codetype = array();

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'BARS';
        $this->name = 'Barcode';
        $this->type = 'DBFT';
        $this->version = '1.0.2.3';
        $this->description = 'Field that saves any text up to 255 characters long. and return barcode';
        $this->multifield = false; // Describes it field is a multi field see properties;
        $this->has_lookup = true; // Decribes if field can be set as copy field on lookup
        $this->has_question = true; // Describes if field can be used on queries criteria
        $this->yes_no = array('GEN_LBL_NO', 'GEN_LBL_YES');
        $this->formats = array('GEN_LBL_ANY', 'GEN_LBL_LOWERCASE', 'GEN_LBL_UPPERCASE', 'GEN_LBL_CAPITALIZE');
        $this->text_align = array('GEN_LBL_LEFT', 'GEN_LBL_CENTER', 'GEN_LBL_RIGHT');
        $this->operators = array('===' => 'GEN_LBL_EQUALTO', '!== ' => 'GEN_LBL_NOTEQUALTO',
		  '%=' => 'GEN_LBL_STARTSWITH',
		  '=%' => 'GEN_LBL_ENDWITH',
		  '%%' => 'GEN_LBL_CONTEINS');
        $this->codetype = array('CODE39' => 'CODE39', 'CODE39EXT' => 'CODE39EXT', 'CODE11' => 'CODE11', 'CODABAR' => 'CODABAR', 'CODE93' => 'CODE93', 'CODE93EXT' => 'CODE93EXT', 'CODE128' => 'CODE128', 'EAN13' => 'EAN13', 'EAN8' => 'EAN8', 'INTERLEAVED25' => 'INTERLEAVED25', 'IND25' => 'IND25', 'MAT25' => 'MAT25', 'MSI' => 'MSI', 'POSTNET' => 'POSTNET', 'UPCA' => 'UPCA', 'UPCE' => 'UPCE', 'PLANET' => 'PLANET', 'UCC128' => 'UCC128');
    	IF (ISSET($GLOBALS['ADDRDEBUGEDIT']) && $GLOBALS['ADDRDEBUGEDIT']==$_SERVER['REMOTE_ADDR']) {
			$this->debug=true;
		}

    }

    function panel($msg, $fieldrecord, &$form) {
        $c = $this->code;
        if ($fieldrecord['DBFL_fieldtype'] != $c) {
		  $this->defaultPanel($fieldrecord);
        }
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell(dict('GEN_LBL_BARCODEFIELDPROP'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addField($msg, dict('GEN_LBL_SIZE') . ':', $this->code . '_size', $fieldrecord[$c . '_size'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $sel = new htmlSelect($c . '_required');
        foreach ($this->yes_no as $key => $val) {
		  $sel->addItem(dict($val), $key, ($fieldrecord[$c . '_required'] == $key));
        }
        $table->addData(dict('GEN_LBL_REQUIRED') . ':', $sel->source());
        $sel = new htmlSelect($c . '_format');
        foreach ($this->formats as $key => $val) {
		  $sel->addItem(dict($val), $key, ($fieldrecord[$c . '_format'] == $key));
        }
        $table->addData(dict('GEN_LBL_CAPS') . ':', $sel->source());
        $sel = new htmlSelect($c . '_align');
        foreach ($this->text_align as $key => $val) {
		  $sel->addItem(dict($val), $key, ($fieldrecord[$c . '_align'] == $key));
        }
        $table->addData(dict('GEN_LBL_ALIGN') . ':', $sel->source());
        $yesno = new htmlSelect($c . '_extra1');
        reset($this->codetype);
        foreach ($this->codetype as $key => $val) {
		  $yesno->addItem($val, $key, ($fieldrecord[$c . '_extra1'] == $key));
        }
        $table->addData(dict('GEN_LBL_CODE_TYPE') . ':', $yesno->source());
        $table->addField($msg, dict('GEN_LBL_DEFAULT') . ':', $c . '_default', $fieldrecord[$c . '_default'], CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_DISPLAY_SIZE') . ':', $c . '_displaysize', $fieldrecord[$c . '_displaysize'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $div = new htmlDiv($this->code, 'visibility:hidden; display:none', $table->source());
        return CLX_CHAR_NL . CLX_CHAR_NL . $div->source() . CLX_CHAR_NL . CLX_CHAR_NL;
    }

    function checkPanel(&$msg) {
        $c = $this->code;
        xValidate::integer($msg, $c . '_size', UserRequest::$post[$c . '_size'], CLX_NUM_ONE, CLX_NUM_255, 'size');
        xValidate::integer($msg, $c . '_displaysize', UserRequest::$post[$c . '_displaysize'], CLX_NUM_ONE, CLX_NUM_255, 'display size');
        if (UserRequest::$post[$c . '_displaysize'] > UserRequest::$post[$c . '_size']) {
		  $msg[$c . '_displaysize'] = dict('GEN_LBL_MAX_DISPLAYSIZE', UserRequest::$post[$c . '_size']);
        }
    }

    function savePanel(&$fieldrecord) {
        $c = $this->code;
        $fieldrecord['FIELD_SIZE'] = UserRequest::$post[$c . '_size'];
        $fieldrecord['FIELD_REQUIRED'] = UserRequest::$post[$c . '_required'];
        $fieldrecord['FIELD_FORMAT'] = '0';
        $fieldrecord['FIELD_ALIGN'] = UserRequest::$post[$c . '_align'];
        $fieldrecord['FIELD_DEFAULT'] = UserRequest::$post[$c . '_default'];
        $fieldrecord['FIELD_MINVALUE'] = '';
        $fieldrecord['FIELD_MAXVALUE'] = '';
        $fieldrecord['FIELD_DISPLAYSIZE'] = UserRequest::$post[$c . '_displaysize'];
        $fieldrecord['FIELD_EXTRA1'] = UserRequest::$post[$c . '_extra1'];
        $fieldrecord['FIELD_EXTRA2'] = '';
        $fieldrecord['FIELD_EXTRA3'] = '';
        $fieldrecord['FIELD_EXTRA4'] = '';
        $fieldrecord['FIELD_EXTRA5'] = '';
    }

    function loadPanel(&$fieldrecord, $values) {
        $c = $this->code;
        if ($values['FIELDTYPE'] == $c) {
		  $fieldrecord[$c . '_size'] = $values['FIELD_SIZE'];
		  $fieldrecord[$c . '_required'] = $values['FIELD_REQUIRED'];
		  $fieldrecord[$c . '_format'] = $values['FIELD_FORMAT'];
		  $fieldrecord[$c . '_align'] = $values['FIELD_ALIGN'];
		  $fieldrecord[$c . '_default'] = $values['FIELD_DEFAULT'];
		  $fieldrecord[$c . '_displaysize'] = $values['FIELD_DISPLAYSIZE'];
		  $fieldrecord[$c . '_extra1'] = $values['FIELD_EXTRA1'];
        } else {
		  $this->defaultPanel($fieldrecord);
        }
    }

    function defaultPanel(&$fieldrecord) {
        $c = $this->code;
        $fieldrecord[$c . '_size'] = CLX_NUM_32;
        $fieldrecord[$c . '_required'] = CLX_INIT_ZERO;
        $fieldrecord[$c . '_format'] = '';
        $fieldrecord[$c . '_align'] = CLX_INIT_ZERO;
        $fieldrecord[$c . '_default'] = 'null';
        $fieldrecord[$c . '_minvalue'] = '';
        $fieldrecord[$c . '_maxvalue'] = '';
        $fieldrecord[$c . '_displaysize'] = '32';
        $fieldrecord[$c . '_extra1'] = 'UCC128';
        $fieldrecord[$c . '_extra2'] = '';
        $fieldrecord[$c . '_extra3'] = '';
        $fieldrecord[$c . '_extra4'] = '';
        $fieldrecord[$c . '_extra5'] = '';
    }

    function viewForm(&$table, $record) {
        $c = $this->code;
        $table->addData(dict('GEN_LBL_TYPE') . ':', $this->name);
        $table->addData(dict('GEN_LBL_SIZE') . ':', $record[$c . '_size']);
        $table->addData(dict('GEN_LBL_REQUIRED') . ':', dict($this->yes_no[$record[$c . '_required']]));
        $table->addData(dict('GEN_LBL_ALIGN') . ':', dict($this->text_align[$record[$c . '_align']]));
        $table->addData(dict('GEN_LBL_FORMAT') . ':', dict($this->formats[$record[$c . '_format']]));
        $table->addData(dict('GEN_LBL_DEFAULT') . ':', $record[$c . '_default']);
        $table->addData(dict('GEN_LBL_DISPLAY_SIZE') . ':', $record[$c . '_displaysize']);
        $table->addData(dict('GEN_LBL_CODE_TYPE') . ':', $this->codetype[$record[$c . '_extra1']]);
    }

    function fieldProperties($fieldrecord) {
        $props = array();
        // DB implementation
        switch (kDatabaseType) {
		  case 'oci':
			 $props['type'] = ($fieldrecord['FIELD_SIZE'] == CLX_NUM_ONE) ? 'CHAR(1)' : 'VARCHAR2(' . $fieldrecord['FIELD_SIZE'] . ')';
			 break;
		  default:
			 $props['type'] = ($fieldrecord['FIELD_SIZE'] == CLX_NUM_ONE) ? 'CHAR(1)' : 'VARCHAR(' . $fieldrecord['FIELD_SIZE'] . ')';
			 break;
        }
        if ($fieldrecord['FIELD_DEFAULT'] === 'null' || strlen($fieldrecord['FIELD_DEFAULT']) == CLX_NUM_ZERO) {
		  $props['default'] = 'null';
        } else {
		  $props['default'] = singleQuote($fieldrecord['FIELD_DEFAULT']);
		  $props['default_raw'] = $fieldrecord['FIELD_DEFAULT'];
        }
        $props['properties'] = '';
        $props['null'] = ($fieldrecord['FIELD_REQUIRED']) ? 'NO' : 'YES';
        return $props;
    }

    function validateField(&$msg, $form_fieldname, $fieldrecord, &$record, &$table, &$form) {
        //Setting form require pref if any
        if (array_key_exists('FIELD_FORM_REQUIRED', $fieldrecord) && $fieldrecord['FIELD_FORM_REQUIRED'] != CLX_NUM_TWO) {
		  $fieldrecord['FIELD_REQUIRED'] = $fieldrecord['FIELD_FORM_REQUIRED'];
        }
        // Note: !array_key_exists was added to enable lookups to validate
        if (!array_key_exists('FIELD_HIDDEN', $fieldrecord))
		  $fieldrecord['FIELD_HIDDEN'] = false;
        if (empty($fieldrecord['FIELD_HIDDEN'])) {
		  if (UserRequest::$request['ac']!='DELETE' && $fieldrecord['FIELD_REQUIRED'] === '1') {
			 xValidate::required($msg, $form_fieldname, $record[$form_fieldname]);
		  }
        }
    }

    function placeViewField($form_fieldname, $fieldrecord, $record, $msg = array()) {
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_WIDTH => CLX_PROP_100PCT));
        if (empty($fieldrecord['FIELD_HIDDEN'])) {
		  $cpt = dict($fieldrecord['CAPTION']);
		  if (strlen($cpt) > CLX_NUM_ZERO) {
			 $table->addCell($cpt, array(CLX_PROP_CLASS => $fieldrecord['CAPTION_CLASS'], CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  }
		  if (isset($msg[$form_fieldname])) {
			 $table->addCell($msg[$form_fieldname], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  }
		  $pfx = dict($fieldrecord['FIELDPREFIX']);
		  $sfx = dict($fieldrecord['FIELDSUFIX']);
		  $cell = $this->formatField($form_fieldname, $fieldrecord, $record);
		  $cell = trim($this->formatField($form_fieldname, $fieldrecord, $record));
		  if (strlen($cell) > CLX_NUM_ZERO) {
			 $cell = $this->barAppletCall($cell, '200', CLX_NUM_HUNDRED, 'CODE128');
		  }
		  // Field Anchor
		  $cell = $this->placeFieldAnchor($form_fieldname, $fieldrecord, $record, $cell);
		  $table->addCell(dict($fieldrecord['FIELDFORMLABEL']), array(CLX_PROP_CLASS => CLX_CLASS_PROMPT));
		  $table->addCell($pfx . ((strlen($cell) == CLX_NUM_ZERO) ? CLX_CHAR_HTML_NBSP : $cell) . $sfx, CLX_PROP_CLASS, CLX_CLASS_DATA);
        }
        return $table;
    }

    function placeListField($form_fieldname, $fieldrecord, $record, $class, &$table,$csv=1) {
        $pfx = dict($fieldrecord['FIELDPREFIX']);
        $sfx = dict($fieldrecord['FIELDSUFIX']);
        $cell = trim($this->formatField($form_fieldname, $fieldrecord, $record));
        if (strlen($cell) > CLX_NUM_ZERO) {
		  $cell = $this->barAppletCall($cell, '200', CLX_NUM_HUNDRED, $fieldrecord['FIELD_EXTRA1']);
        }
        // Field Anchor
        $cell = $this->placeFieldAnchor($form_fieldname, $fieldrecord, $record, $cell);
        $table->addCell($pfx . $cell . $sfx, array(CLX_PROP_CLASS => $class));
		if ( $this->debug==true&&!isset($GLOBALS['placelistcount']) && $GLOBALS['placelistcount']<200 ) {
			$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo4.txt","a+");
			fwrite($tta,chr(13).chr(10).$pos."_".$te[$posx]["function"]."dbft_barcode-hora:".date("h:i:s d/m/Y").chr(13).chr(10));
			$te=debug_backtrace();
			for ($t=0;$t<12;$t++) {
				if (isset($te[$t]["file"]))
					fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));
				
			}

			fwrite($tta,chr(13).chr(10)."placeListFieldText: ".print_r($cell,true));
			fclose($tta);
		}
    }

    function barAppletCall($value, $width = '701', $height = '150', $type = 'CODE128') {
        $source = '<APPLET CODEBASE = "./data/code/"	ARCHIVE  = "LinearBarCode.jar"' . CLX_CHAR_NL . '
					CODE = "com.idautomation.linear.BCApplet.class" NAME = "barcode"' . CLX_CHAR_NL . ' WIDTH = ' . $width . ' HEIGHT = ' . $height . CLX_CHAR_NL . ' HSPACE = 0 VSPACE = 0' . CLX_CHAR_NL . ' ALIGN = top ALT = "Java 1.1 enabled browser to view barcode">' . CLX_CHAR_NL . '
					<!-- idautomation.com -->' . CLX_CHAR_NL . '
					<PARAM NAME = "BARCODE" VALUE = "' . $value . '">' . CLX_CHAR_NL . '
					<PARAM NAME = "CODE_TYPE" VALUE = "' . $type . '">' . CLX_CHAR_NL . '
					<PARAM NAME = "BAR_HEIGHT" VALUE = "2">' . CLX_CHAR_NL . '
					<PARAM NAME = "TEXT_FONT" VALUE = "ARIAL|BOLD|11">' . CLX_CHAR_NL . '
					</APPLET>' . CLX_CHAR_NL;
        return $source;
    }

}
