<?php

/**
 * Binary large field object class (BLOB)
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version    	$Rev: 9615 $
 *
 * Change log
 *  Added Oracle on field properties
 *
 */
if (!class_exists('dbft') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'dbft.php';

class dbft_blob extends dbft {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'BLOB';
        $this->name = 'Blob';
        $this->type = 'DBFT';
        $this->version = '0.2.3.1';
        $this->description = 'Can somebody place a nice description here?';
        $this->multifield = false; // Describes it field is a multi field see properties;
        $this->has_lookup = true; // Decribes if field can be set as copy field on lookup
        $this->has_question = true; // Describes if field can be used on queries criteria
        $this->operators = array('===' => 'GEN_LBL_EQUALTO', '!== ' => 'GEN_LBL_NOTEQUALTO');
    }

    function panel($msg, $fieldrecord, &$form) {
        $c = $this->code;
        if ($fieldrecord['DBFL_fieldtype'] != $c) {
		  $this->defaultPanel($fieldrecord);
        }
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell(dict('GEN_LBL_BLOBFIELDPROP'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $size = new htmlSelect($c . '_size');
        $size->addItem(dict('GEN_LBL_SMALL'), '0', ($fieldrecord[$c . '_size'] == CLX_NUM_ZERO));
        $size->addItem(dict('GEN_LBL_MEDIUM'), '1', ($fieldrecord[$c . '_size'] == CLX_NUM_ONE));
        $size->addItem(dict('GEN_LBL_LARGE'), '2', ($fieldrecord[$c . '_size'] == CLX_NUM_TWO));
        $table->addData(dict('GEN_LBL_SIZE'), $size->source());
        $div = new htmlDiv($this->code, 'visibility:hidden; display:none', $table->source());
        return CLX_CHAR_NL . CLX_CHAR_NL . $div->source() . CLX_CHAR_NL . CLX_CHAR_NL;
    }

    function savePanel(&$fieldrecord) {
        $c = $this->code;
        $fieldrecord['FIELD_SIZE'] = UserRequest::$post[$c . '_size'];
        $fieldrecord['FIELD_REQUIRED'] = CLX_INIT_ZERO;
        $fieldrecord['FIELD_FORMAT'] = '';
        $fieldrecord['FIELD_ALIGN'] = '0';
        $fieldrecord['FIELD_DEFAULT'] = '';
        $fieldrecord['FIELD_MINVALUE'] = '';
        $fieldrecord['FIELD_MAXVALUE'] = '';
        $fieldrecord['FIELD_EXTRA1'] = '';
        $fieldrecord['FIELD_EXTRA2'] = '';
        $fieldrecord['FIELD_EXTRA3'] = '';
        $fieldrecord['FIELD_EXTRA4'] = '';
        $fieldrecord['FIELD_EXTRA5'] = '';
    }

    function loadPanel(&$fieldrecord, $values) {
        $c = $this->code;
        if ($values['FIELDTYPE'] == $c) {
		  $fieldrecord[$c . '_size'] = $values['FIELD_SIZE'];
		  $fieldrecord[$c . '_required'] = CLX_INIT_ZERO;
        } else {
		  $this->defaultPanel($fieldrecord);
        }
    }

    function defaultPanel(&$fieldrecord) {
        $c = $this->code;
        $fieldrecord[$c . '_size'] = CLX_NUM_ONE;
        $fieldrecord[$c . '_required'] = CLX_INIT_ZERO;
        $fieldrecord[$c . '_format'] = '';
        $fieldrecord[$c . '_align'] = CLX_INIT_ZERO;
        $fieldrecord[$c . '_default'] = '';
        $fieldrecord[$c . '_minvalue'] = '';
        $fieldrecord[$c . '_maxvalue'] = '';
        $fieldrecord[$c . '_extra1'] = '';
        $fieldrecord[$c . '_extra2'] = '';
        $fieldrecord[$c . '_extra3'] = '';
        $fieldrecord[$c . '_extra4'] = '';
        $fieldrecord[$c . '_extra5'] = '';
    }

    function viewForm(&$table, $record) {
        $size = array(dict('GEN_LBL_SMALL'), dict('GEN_LBL_MEDIUM'), dict('GEN_LBL_LARGE'));
        $table->addData(dict('GEN_LBL_SIZE') . ':', $size[$record['BLOB_size']]);
    }

    function fieldProperties($fieldrecord) {
        // DB implementation
        switch (kDatabaseType) {
		  case 'oci':
			 $size = array(CLX_NUM_ZERO => 'BLOB', CLX_NUM_ONE => 'BLOB', CLX_NUM_TWO => 'BLOB');
			 break;
		  default:
			 $size = array(CLX_NUM_ZERO => 'BLOB', CLX_NUM_ONE => 'MEDIUMBLOB', CLX_NUM_TWO => 'LONGBLOB');
			 break;
        }
        $props = array();
        $props['default'] = 'null'; //No defaults are allowed on blobs or text (see mysql blob)
        $props['properties'] = '';
        $props['type'] = $size[$fieldrecord['FIELD_SIZE']];
        $props['null'] = 'YES'; //will be allways nullable and handled on form
        return $props;
    }

    function validateField(&$msg, $form_fieldname, $fieldrecord, &$record, &$table, &$form) {
        //Setting form require pref if any
        if (array_key_exists('FIELD_FORM_REQUIRED', $fieldrecord) && $fieldrecord['FIELD_FORM_REQUIRED'] != CLX_NUM_TWO) {
		  $fieldrecord['FIELD_REQUIRED'] = $fieldrecord['FIELD_FORM_REQUIRED'];
        }
        // Note: !array_key_exists was added to enable lookups to validate
        if (!array_key_exists('FIELD_HIDDEN', $fieldrecord))
		  $fieldrecord['FIELD_HIDDEN'] = false;
        if (empty($fieldrecord['FIELD_HIDDEN']) === true) {
		  if (UserRequest::$request['ac']!='DELETE' && $fieldrecord['FIELD_REQUIRED'] === '1') {
			 xValidate::required($msg, $form_fieldname, $record[$form_fieldname]);
		  }
        }//end if
    }

    function fieldObject(&$form_fieldname, &$fieldrecord, $record) {
        $fieldobj = new htmlTextArea($form_fieldname, $record[$form_fieldname], CLX_NUM_FOUR, CLX_NUM_48);
        //Added instead of displaying read only files
        if (!empty($fieldrecord['FIELD_READONLY'])) {
		  $fieldobj->setProp('readonly', 'true');
		  $fieldobj->setProp(CLX_PROP_CLASS, 'readonly');
        }
        return $fieldobj;
    }

    function formatField($form_fieldname, $fieldrecord, $record) {
            //  comentada consume memoria pero uppercase no trabaja acentos los lookups demasiado en el export  $form_fieldname = uppercase($form_fieldname);
		$form_fieldname = uppercase($form_fieldname);

        $record = array_change_key_case($record, CASE_UPPER);
            //  comentada consume memoria pero uppercase no trabaja acentos los lookups demasiado en el export  $form_fieldname = uppercase($form_fieldname);
		$form_fieldname = uppercase($form_fieldname);

        return nl2br($record[$form_fieldname]);
    }

}
