<?php

/**
 * System  field object class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-10-09 15:15:00 -0400 (Tue, 09 Oct 2018) $
 * @version    	$Rev: 10228 $
 */
if (!class_exists('dbft') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'dbft.php';

class dbft_sysobject extends dbft {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'SOBJ';
        $this->name = 'System Object';
        $this->type = 'DBFT';
        $this->version = '0.2.6.2';
        $this->description = 'Selects a system object';
        $this->multifield = false; // Describes it field is a multi field see properties;
        $this->has_lookup = true; // Decribes if field can be set as copy field on lookup
        $this->has_question = true; // Describes if field can be used on queries criteria
        $this->operators = array('===' => 'GEN_LBL_EQUALTO',
		  '!== ' => 'GEN_LBL_NOTEQUALTO'
        );
    	IF (ISSET($GLOBALS['ADDRDEBUGEDIT']) && $GLOBALS['ADDRDEBUGEDIT']==$_SERVER['REMOTE_ADDR']) {
			$this->debug=true;
		}

    }

    function panel($msg, $fieldrecord, &$form) {
        $c = $this->code;
        if ($fieldrecord['DBFL_fieldtype'] != $c) {
		  $this->defaultPanel($fieldrecord);
        }
        $contents = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell(dict('GEN_LBL_SYSOBJFIELDPROP'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $yesno = new htmlSelect($c . '_required');
        $yesno->addItem(dict('GEN_LBL_NO'), '0', ($fieldrecord[$c . '_required'] == CLX_NUM_ZERO));
        $yesno->addItem(dict('GEN_LBL_YES'), '1', ($fieldrecord[$c . '_required'] == CLX_NUM_ONE));
        $table->addData(dict('GEN_LBL_REQUIRED') . ':', $yesno->source());
        $records = ClxCommon::$db->getRecords('CLX_TOOLS', 'HAS_PACKAGE=1  OR IDCODE IN (' . singleQuote('SMS') . ',' . singleQuote('MAG') . ') ORDER BY IDNAME');
        $pop = new htmlSelect($c . '_extra1');
        while (($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE))) {
		  $pop->addItem($record['IDNAME'], $record['IDCODE'], ($fieldrecord[$c . '_extra1'] == $record['IDCODE']));
        }
		ClxCommon::$db->freeResult( $records );
        $table->addData(dict('GEN_LBL_TOOL'), $pop->source());
        $div = new htmlDiv($this->code, 'visibility:hidden; display:none', $table->source());
        $contents .= CLX_CHAR_NL . CLX_CHAR_NL . $div->source();
        return $contents . CLX_CHAR_NL . CLX_CHAR_NL;
    }

    function checkPanel(&$msg) {
        $c = $this->code;
        if (!empty(UserRequest::$post[$c . '_required'])) {

        }
    }

    function savePanel(&$fieldrecord) {
        $c = $this->code;
        $fieldrecord['FIELD_SIZE'] = '';
        $fieldrecord['FIELD_REQUIRED'] = UserRequest::$post[$c . '_required'];
        $fieldrecord['FIELD_FORMAT'] = '';
        $fieldrecord['FIELD_ALIGN'] = '0';
        $fieldrecord['FIELD_DEFAULT'] = '';
        $fieldrecord['FIELD_MINVALUE'] = '';
        $fieldrecord['FIELD_MAXVALUE'] = '';
        $fieldrecord['FIELD_EXTRA1'] = UserRequest::$post[$c . '_extra1'];
        $fieldrecord['FIELD_EXTRA2'] = '';
        $fieldrecord['FIELD_EXTRA3'] = '';
        $fieldrecord['FIELD_EXTRA4'] = '';
        $fieldrecord['FIELD_EXTRA5'] = '';
    }

    function loadPanel(&$fieldrecord, $values) {
        $c = $this->code;
        if ($values['FIELDTYPE'] == $c) {
		  $fieldrecord[$c . '_required'] = $values['FIELD_REQUIRED'];
		  $fieldrecord[$c . '_extra1'] = $values['FIELD_EXTRA1'];
        } else {
		  $this->defaultPanel($fieldrecord);
        }
    }

    function defaultPanel(&$fieldrecord) {
        $c = $this->code;
        $fieldrecord[$c . '_size'] = CLX_NUM_30;
        $fieldrecord[$c . '_required'] = CLX_INIT_ZERO;
        $fieldrecord[$c . '_format'] = '';
        $fieldrecord[$c . '_align'] = CLX_INIT_ZERO;
        $fieldrecord[$c . '_default'] = '0';
        $fieldrecord[$c . '_minvalue'] = '';
        $fieldrecord[$c . '_maxvalue'] = '';
        $fieldrecord[$c . '_extra1'] = '';
        $fieldrecord[$c . '_extra2'] = '';
        $fieldrecord[$c . '_extra3'] = '';
        $fieldrecord[$c . '_extra4'] = '';
        $fieldrecord[$c . '_extra5'] = '';
    }

    function viewForm(&$table, $record) {
        $yesno = array(CLX_NUM_ZERO => dict('GEN_LBL_NO'), CLX_NUM_ONE => dict('GEN_LBL_YES'));
        $table->addData(dict('GEN_LBL_TYPE') . ':', $this->name);
        $table->addData(dict('GEN_LBL_REQUIRED') . ':', $yesno[$record[$this->code . '_required']]);
        $table->addData(dict('GEN_LBL_TOOL') . ':', $record[$this->code . '_extra1']);
    }

    function fieldType($fieldrecord) {
        // DB implementation
        switch (kDatabaseType) {
		  case 'oci':
			 $return = 'VARCHAR2(30)';
			 break;
		  default:
			 $return = 'VARCHAR(30)';
			 break;
        }
        return $return;
    }

    function fieldProperties($fieldrecord) {
        $props = array();
        // DB implementation
        switch (kDatabaseType) {
		  case 'oci':
			 $props['type'] = 'VARCHAR2(30)';
			 break;
		  default:
			 $props['type'] = 'VARCHAR(30)';
			 break;
        }
        if ($fieldrecord['FIELD_DEFAULT'] === 'null' || strlen($fieldrecord['FIELD_DEFAULT']) == CLX_NUM_ZERO) {
		  $props['default'] = 'null';
        } else {
		  $props['default'] = singleQuote($fieldrecord['FIELD_DEFAULT']);
		  $props['default_raw'] = $fieldrecord['FIELD_DEFAULT'];
        }
        $props['properties'] = '';
        $props['null'] = ($fieldrecord['FIELD_REQUIRED']) ? 'NO' : 'YES';
        return $props;
    }

    function placeEditField($msg, $form_fieldname, $fieldrecord, $record, &$form, $jsevents) {
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_WIDTH => CLX_PROP_100PCT));
        // Setting form require pref if any
        if ($fieldrecord['FIELD_FORM_REQUIRED'] != CLX_NUM_TWO) {
		  $fieldrecord['FIELD_REQUIRED'] = $fieldrecord['FIELD_FORM_REQUIRED'];
        }
        if (($fieldrecord['FIELD_REQUIRED'] != CLX_NUM_ONE && $fieldrecord['FIELD_READONLY']) || $fieldrecord['FIELD_HIDDEN']) {
		  $form->addArgument($form_fieldname, (array_key_exists($form_fieldname, $record)) ? $record[$form_fieldname] : '');
		  if (!empty($fieldrecord['FIELD_READONLY']) && empty($fieldrecord['FIELD_HIDDEN'])) {
			 $table = $this->placeViewField($form_fieldname, $fieldrecord, $record);
		  }
        } else {
		  $cpt = dict($fieldrecord['CAPTION']);
		  if (strlen($cpt) > CLX_NUM_ZERO) {
			 $table->addCell($cpt, array(CLX_PROP_CLASS => $fieldrecord['CAPTION_CLASS'], CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  }
		  if (isset($msg[$form_fieldname])) {
			 $table->addCell($msg[$form_fieldname], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  }
		  $toolcode = $fieldrecord['FIELD_EXTRA1'];
		  $tool = &getTool($toolcode);
		  $fieldsource = $tool->chooseRecord($form_fieldname, (array_key_exists($form_fieldname, $record)) ? $record[$form_fieldname] : '', array());
		  $pfx = dict($fieldrecord['FIELDPREFIX']);
		  $sfx = dict($fieldrecord['FIELDSUFIX']);
		  $class = ($fieldrecord['FIELD_REQUIRED']) ? CLX_CLASS_PROMPTREQUIRED : CLX_CLASS_PROMPT;
		  $table->addCell(dict($fieldrecord['FIELDFORMLABEL']), array(CLX_PROP_CLASS => $class));
		  $table->addCell($pfx . $fieldsource . $sfx, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        }//end if hidden
		if ( $this->debug==true &&!isset($GLOBALS['placelistcount']) && $GLOBALS['placelistcount']<200) {
			$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo4.txt","a+");
			fwrite($tta,chr(13).chr(10).$pos."_".$te[$posx]["function"]."dbft_primary-hora:".date("h:i:s d/m/Y").chr(13).chr(10));
			$te=debug_backtrace();
			for ($t=0;$t<12;$t++) {
				if (isset($te[$t]["file"]))
					fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));

			}

			fwrite($tta,chr(13).chr(10)."placeListFieldText: ".print_r($fieldsource,true));
			fclose($tta);
		}
        return $table;
    }

    function validateField(&$msg, $form_fieldname, $fieldrecord, &$record, &$table, &$form) {
        //Setting form require pref if any
        if (array_key_exists('FIELD_FORM_REQUIRED', $fieldrecord) && $fieldrecord['FIELD_FORM_REQUIRED'] != CLX_NUM_TWO) {
		  $fieldrecord['FIELD_REQUIRED'] = $fieldrecord['FIELD_FORM_REQUIRED'];
        }
        // Note: !array_key_exists was added to enable lookups to validate
        if (!array_key_exists('FIELD_HIDDEN', $fieldrecord))
		  $fieldrecord['FIELD_HIDDEN'] = false;
        //Setting hidden to override required
        if ($fieldrecord['FIELD_HIDDEN'] === true) {
		  $fieldrecord['FIELD_REQUIRED'] = CLX_INIT_ZERO;
        }
        $something = (strlen($record[$form_fieldname]) > CLX_NUM_ZERO);
        //$required = ($fieldrecord['FIELD_REQUIRED']);
        $required = (!empty($fieldrecord['FIELD_REQUIRED']));
        if ($required === true and ! $something === true) {
			IF (UserRequest::$request['ac']!='DELETE' )
		  $msg[$form_fieldname] = dict('GEN_ERR_REQUIRED_FIELD');
        } elseif ($required === true or $something === true) {
		  $toolcode = $fieldrecord['FIELD_EXTRA1'];
		  $tool = &getTool($toolcode);
		  $tool->recordExists($form_fieldname, $record[$form_fieldname], $msg);
        } else {
		  $record[$form_fieldname] = '';
        }
    }

    function validateImportField(&$msg, $fieldrecord, $fieldvalue) {
        if ($fieldrecord['FIELD_REQUIRED'] == CLX_NUM_ZERO && $fieldvalue === 'null') {
		  return;
        } else {
		  $something = (strlen($fieldvalue) > CLX_NUM_ZERO);
		  $required = ($fieldrecord['FIELD_REQUIRED']);
		  if (empty($something) and ! empty($required)) {
			 $msg[$fieldrecord['FIELDNAME']] = dict('GEN_ERR_REQUIRED_FIELD');
		  } elseif (empty($something)) {
			 $toolcode = $fieldrecord['FIELD_EXTRA1'];
			 $tool = &getTool($toolcode);
			 $tool->recordExists($fieldrecord['FIELDNAME'], $fieldvalue, $msg);
		  }
        }
    }

    function formatField($form_fieldname, $fieldrecord, $record) {

        $form_fieldname = uppercase($form_fieldname);
        if(!array_key_exists($form_fieldname, $record)) {
		  $record[$form_fieldname] = '';
        }
        $record = array_change_key_case($record, CASE_UPPER);
        $toolcode = $fieldrecord['FIELD_EXTRA1'];
        $tool = &getTool($toolcode);
        $contents = $tool->describe($record[$form_fieldname]);
        if ($contents === ',  ()') {
		  if (strlen($record[$form_fieldname]) > CLX_NUM_ZERO) {
			 $contents = '(' . $record[$form_fieldname] . ')';
		  } else {
			 $contents = CLX_CHAR_HTML_NBSP;
		  }
        }
        return $contents;
    }

}
