<?php

/**
 * Object Class
 * For more information contact  support@alenet.com
 * Report Class to generate reports as pdf.
 * @package		Clientix.Tool.Analytics
 * @abstract		Report Type Object
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-12-11 05:22:50 -0400 (ma., 11 dic. 2018) $
 * @version        $Rev: 10413 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
if (!class_exists('rpft') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'rpft.php';
require_once kPathToWebSite . kPathToPear.'Spreadsheet/Excel/Writer.php';
require_once kPathToWebSite . 'classes/clx_report.php';
include_once kPathToWebSite . 'classes/clx_report.php';
include_once kPathToWebSite . 'classes/clx_report_page.php';
include_once kPathToWebSite . 'classes/clx_report_panel.php';
include_once kPathToWebSite . 'classes/clx_report_object.php';
include_once kPathToWebSite . 'classes/clx_report_object_table.php';
include_once kPathToWebSite . 'classes/clx_report_data.php';
include_once kPathToWebSite . 'classes/clx_style.php';

class rpft_pdfr extends rpft {

    var $pdf; //or report
    var $page; //The page
    var $table; //A table
    var $styles = array();

    // CONSTRUCTOR //
    function __construct() {
	$this->code = 'RPPP';
	$this->name = 'PDF Report';
	$this->type = 'RPFT';
	$this->version = '5.5.0.0';
	$this->description = 'PDF Report Format';
	$this->windowurl = 'window.php';
	$this->report = null;
	$this->show_prompt_format = true; //untill output is develop
    }

// End method
    /**
     * (non-PHPdoc)
     * @see rpft::build()
     */
    function build($reportid) {
	OB_START();
	//Loads report information
	$this->getReportRecord($reportid);
	//check access
	$DBQY = &getTool('DBQY');
	$ANRP = &getTool('ANRP');
	if (empty($DBQY->getAccess()->getCanRead())) {
	    return warning(dict('GEN_ERR_NO_ACCESS') . ' [RPFT-CAN_READ]' . ' ' . $ANRP->getCode(false) . ' Query:' . $this->reportrec['QUERYCODE']);
	}

	//Init Styles
	$this->init_styles();

	//Init pdf properties
	$orientation = ($this->reportrec['PDFPRINTORIENTATION'] ==0) ? 'P' : 'L';
	$unit = 'mm';
	$formato = array('0' => 'A4', '1' => 'A5', '2' => 'Letter', '3' => 'LEGAL', '4' => 'OFFICE', '5' => 'FREE');
	$format = array($formato[$this->reportrec['PDFPRINTFORMAT']],$this->reportrec['PDFPRINTWIDTH'],$this->reportrec['PDFPRINTHEIGHT']);
        //ANRP_pdf_print_format
	$this->report = new clx_report($orientation, $unit, $format);
	$this->page = new clx_report_page ();
        if(empty($this->reportrec['HEADERCODE'])){
		  $this->page->setMarginTop(CLX_NUM_TEN);
		  $this->page->setMarginLeft(CLX_NUM_TEN);
        }
	//Report columns and groups
	$this->getReportColumns();
	$this->getReportGroups();
	$this->getReportFunctions();



	//Start the report
	$this->start();
	//Init Report Table
	$this->table = new clx_report_object_table ();
        $this->table->setPdfAnrpMode(true);
	$this->headers($orientation, $format);

	$records = $this->getRecords();

	$lastrecord = null;
	while (($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE))) {
	    //run formulas
	    $this->runRecordEvents($reportid, $record);
	    if ($lastrecord != null) {
		$this->insertFunctions($record, $lastrecord);
	    }
	    $forcebreak = false;
	    foreach ($this->groups as $groupid => $grouprec) {
			$value = $record[uppercase($grouprec['TABLECODE'] . '_' . $grouprec['FIELDNAME'])];
			if (($this->groups_values[$groupid] != $value) or $forcebreak === true) {
				//Addint Totals
				$this->insertBreak($groupid, $record);
				$this->groups_values[$groupid] = $value;
				$forcebreak = true;
			}
	    }
	    //Sumarize for totals
	    $this->calculateFunctions($record);
	    //Addint Details
	    $this->class = CLX_CLASS_DATA;
	    $this->detail($record);
	    $lastrecord = $record;
	}

	// Last Group Total
	$this->insertFunctions($record, $lastrecord);
	// General Total
	if ($this->reportrec['HIDE_GRAN_TOTAL'] == CLX_NUM_ZERO) {
	    $this->insertGroupFunction('OVERALLTOTAL');
	}
	//Adding table to panel and panel to page
        $panel = new clx_report_panel($this->table);
        $panel->setMarginL(CLX_NUM_TEN);
	$this->page->addPanel($panel);
	return $this->finish();
    }

    /**
     * (non-PHPdoc)
     * @see rpft::start()
     */
    function start() {
		multiTableName($this, array('COMP'));

			//Change this for later to add logo to report
		$comp_code = '';
		foreach (UserRequest::$request as $key => $val) {
			if (strpos($key, '_COMP_CODE', CLX_NUM_ZERO) > CLX_NUM_ZERO) {
			$comp_code = $val;
			break;
			}
		}
			if ($comp_code <> '') {
			$this->report->setReplogo($this->comp_logo($comp_code), CLX_NUM_TEN, CLX_NUM_FOUR, CLX_NUM_36_PT_5, CLX_NUM_24_PT_6);
				//$this->report->setReplogo($this->comp_logo($comp_code), 0, 0, 0, 0);
		}

		if (!array_key_exists('TITLE', $this->reportrec))
			$this->reportrec['TITLE'] = '';
		if (!array_key_exists('SUBTITLE', $this->reportrec))
			$this->reportrec['SUBTITLE'] = '';
		$title = dict($this->reportrec['TITLE']);
		$subtitle = dict($this->reportrec['SUBTITLE']);
		$this->width = count($this->columns);
		$this->styles['head']->setBoxColSpan($this->width);
		$this->styles['head2']->setBoxColSpan($this->width);
		$this->styles['head3']->setBoxColSpan($this->width);
		$this->Report_headers();


		if (strlen($title) > CLX_NUM_ZERO) {
			$this->report->setHeader(new clx_report_data($title, $this->styles ['head'])); //0BEDC3EA55A71148DE716239FF8A7AE0=Fideicomisos de Garantias Autos
		}
		if (strlen($subtitle) > CLX_NUM_ZERO) {
			$this->report->setHeader(new clx_report_data($subtitle, $this->styles ['head2']));
		}
		//Additional Title 3
		$this->report->setHeader(new clx_report_data(dict('GEN_LBL_REPORT_DATE', xFormat::getNow($this->format=$format = $_SESSION['prefs']['DATE_FORMAT'].' H:i:s')), $this->styles ['head3']));


		$this->needheaders = true; //Not quite sure why this. OHHH! this was set to tell when to build the header for a table or notor no
		$this->linecount = CLX_INIT_ZERO; //Well is not really line cero on pdf
		$comprec = ClxCommon::$db->getRecord($this->compname, 'IDCODE = ' . singleQuote($comp_code));
		$this->report->footer_start($comp_code);
		$this->report->footer_put_pdf_lines($comprec ['VE_TXTINFO'], $this->styles ['footer']);
		$this->Report_Footers();


    }

    /**
     * (non-PHPdoc)
     * @see rpft::finish()
     */
    function finish(){
		$this->report->addPage($this->page);
		$batchoption = (isset(UserRequest::$request['ANRP_rbat']) ? UserRequest::$request['ANRP_rbat'] : '');
		if($batchoption === '1'){
			return $this->report->source2('pdf', false, '', true );
		} elseif($this->is_output_to_file()) {
			$this->report->source2('pdf', false, basename($this->output_to_file['file_name'], '.pdf'), true );
			return CLX_CHAR_EMPTY_STRING;
		}else{
			//Dump report
			$this->report->source2('pdf', true);
		}
    }

    /**
     * (non-PHPdoc)
     * @see rpft::detail()
     */
    function detail($record, $total_line = false) {
	if ($this->reportrec['SHOW_DETAIL'] || $total_line) {
	    foreach ($this->columns as $fieldindex => $fieldrec) { // add formating here!!!!
		$fn = $fieldrec['TABLECODE'] . '_' . $fieldrec['FIELDNAME'];
		$this->class = ($this->linecount % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT;
		//Some format for the numbers
		if ($fieldrec['FIELDTYPE'] === 'DECN') {
		    $this->class = ($this->class == CLX_CLASS_DATA) ? 'datanum' : 'dataaltnum';
		} else if ($fieldrec['FIELDTYPE'] === 'INTN') {
		    $this->class = ($this->class == CLX_CLASS_DATA) ? 'datanum' : 'dataaltnum';
		}
		if ($total_line && $record[$fn] === '') {
		    $val = ' ';
		} else {
		    if ($record[$fn] === '')
			$record[$fn] = ' ';
		    $fieldinstance = getToolObject('DBFT', $fieldrec['FIELDTYPE']);
		    $val = $fieldinstance->placeListFieldText($fn, $fieldrec, $record, false,0);
		}
		//Adds a cell to the report dont get confused with row!
		$this->table->addRow(new clx_report_data($val, $this->styles [$this->class]));
	    }
	    $this->linecount++;
	}
    }

    /**
     * (non-PHPdoc)
     * @see rpft::headers()
     */
    function headers() {
	foreach ($this->columns as $fieldrec) {
	    $clabel = $fieldrec['FIELDLISTLABEL'];
	    if (isset($fieldrec['ANRC_FIELDLISTLABEL']) && !is_null($fieldrec['ANRC_FIELDLISTLABEL']) && $fieldrec['ANRC_FIELDLISTLABEL'] !== '') {
		$clabel = $fieldrec['ANRC_FIELDLISTLABEL'];
	    }
	    $this->table->addColumn(new clx_report_data(dict($clabel), $this->styles ['tbhead']));
	}
	$this->needheaders = false;
    }

    /**
     * (non-PHPdoc)
     * @see rpft::insertbreak()
     */
    function insertbreak($groupid, $record) {
	if ($this->needheaders) {
	    $this->headers();
	}
	$grouprec = $this->groups[$groupid];
	$level = $grouprec['SEQNUM'];
	$fn = uppercase($grouprec['TABLECODE'] . '_' . $grouprec['FIELDNAME']);
	$val = $this->fieldFormat($fn, $record, $grouprec);
	$linelabel = $grouprec['FIELDLISTLABEL'];
	if (isset($grouprec['RG_FIELDLISTLABEL']) && !is_null($grouprec['RG_FIELDLISTLABEL']) && $grouprec['RG_FIELDLISTLABEL'] !== '') {
	    $linelabel = $grouprec['RG_FIELDLISTLABEL'];
	}
	$val = $this->get_indent_space($level, ' ') . dict($linelabel) . ': ' . $val;
	$this->styles ['group']->setBoxColSpan($this->total_columns);
	$this->table->addRow(new clx_report_data($val, $this->styles ['group'])); //Group title
    }

    //See why this
    function insertFunction($groupid, $value, $functionid) {
	//Review
	if (!array_key_exists($groupid, $this->groups)) {
	    $grouprec = array();
	    $level = CLX_INIT_ZERO;
	} else {
	    $grouprec = $this->groups[$groupid];
	    $level = $grouprec['SEQNUM'];
	}
	$value = $this->get_indent_space($level, CLX_CHAR_HTML_NBSP) . $value;
	if ($this->output_buffer) {
	    $this->styles ['group']->setBoxColSpan($this->total_columns);
	    $this->table->addRow(new clx_report_data($value, $this->styles ['group'])); //Group title
	}
    }

    /**
     * (non-PHPdoc)
     * @see rpft::insertBlankLine()
     */
    function insertBlankLine() {

    }

    /**
     * Styles For report
     */
    private function init_styles() {
	$this->styles = array();
	$this->styles ['all'] = new clx_style('all');
	$this->styles ['all']->setFontSize(CLX_NUM_8);
	$this->styles ['all']->setBoxHeight(CLX_NUM_40);
	$this->styles ['all']->setBoxPadding(CLX_NUM_TWO);
	$this->styles ['all']->setBoxWidth(CLX_NUM_ZERO);
	$this->styles ['all']->setFontColor('#000000');
	$this->styles ['group'] = clone $this->styles ['all'];
	$this->styles ['group']->setBoxBgColor('#eeeeee');
	$this->styles [CLX_CLASS_DATA] = clone $this->styles ['all'];
	$this->styles [CLX_CLASS_DATA]->setBoxBorder(CLX_NUM_ONE, 'all');
	$this->styles [CLX_CLASS_DATA]->setBoxBorderColor('#ffffff');
	$this->styles [CLX_CLASS_DATAALT] = clone $this->styles ['all'];
	$this->styles [CLX_CLASS_DATAALT]->setBoxBgColor('#cccccc');
	$this->styles [CLX_CLASS_DATAALT]->setBoxBorder(CLX_NUM_ONE, 'all');
	$this->styles [CLX_CLASS_DATAALT]->setBoxBorderColor('#ffffff');
	$this->styles ['datanum'] = clone $this->styles [CLX_CLASS_DATA];
	$this->styles ['datanum']->setFontAligment(CLX_PROP_RIGHT);
	$this->styles ['dataaltnum'] = clone $this->styles [CLX_CLASS_DATAALT];
	$this->styles ['dataaltnum']->setFontAligment(CLX_PROP_RIGHT);
	$this->styles ['total'] = clone $this->styles ['all'];
	$this->styles ['total']->setBoxBgColor('#eeeeee');
	$this->styles ['total']->setFontBold();
	$this->styles ['total']->setFontSize(CLX_NUM_TEN);
	$this->styles ['total']->setFontAligment(CLX_PROP_RIGHT);
	$this->styles ['total']->setBoxBorder(CLX_NUM_ONE, 'all');
	$this->styles ['total']->setBoxBorderColor('#eeeeeef');
	$this->styles ['subtotal'] = clone $this->styles ['total'];
	$this->styles ['tbhead'] = clone $this->styles [CLX_CLASS_DATA];
	$this->styles ['tbhead']->setFontBold();
	$this->styles ['tbhead']->setBoxBorder(CLX_NUM_ONE, 'all');
	$this->styles ['tbhead']->setBoxBorderColor('#ffffff');
	$this->styles ['tbhead']->setFontColor('#ffffff');
	$this->styles ['tbhead']->setBoxBgColor('#aaaaaa');
	$this->styles ['tbhead']->setBoxHeight(CLX_NUM_60);
	$this->styles ['tbhead']->setFontSize(CLX_NUM_TEN);
	$this->styles ['head'] = clone $this->styles ['all'];
	$this->styles ['head']->setFontAligment('center');
	$this->styles ['head']->setFontBold();
	$this->styles ['head']->setFontSize(CLX_NUM_14);
	$this->styles ['head']->setBoxHeight(CLX_NUM_20);
	$this->styles ['head2'] = clone $this->styles ['head'];
	$this->styles ['head2']->setFontSize(CLX_NUM_TEN);
	$this->styles ['head3'] = clone $this->styles ['head'];
	$this->styles ['head3']->setFontSize(CLX_NUM_8);
	$this->styles ['head3']->setFontNormal();
	$this->styles ['footer'] = clone $this->styles ['all'];
	$this->styles ['head']->setFontBoldUnderline();
        foreach($this->styles as $key=>$val){
		  $this->styles [$key]->setName($key);
        }
    }

    function Report_Headers() {
	$ANHF = &getTool('ANHF');
	$parametros = array('type_report' => 'PDF', 'search' => $this->reportrec['HEADERCODE'], 'position' => 'Header');


	$value = $ANHF->search_Data($parametros);
        if(empty($value)){
		  return;
        }
	$position = 'H';
	$codehtmlpdf = new html_fpdf_normalize;
	$data_pdf1 = $codehtmlpdf->fraction_html(clx_change_especial_char_html($ANHF->Merge(clx_keywords_simple($value[$position . 'RIGHT']),UserRequest::$request)), $this->report->get_orientation(),$this->report->get_unit(), $this->report->get_format(), 'RIGHT', 'HEADER');
	$data_pdf2 = $codehtmlpdf->fraction_html(clx_change_especial_char_html($ANHF->Merge(clx_keywords_simple($value[$position . 'CENTER']),UserRequest::$request)), $this->report->get_orientation(),$this->report->get_unit(), $this->report->get_format(), 'CENTER', 'HEADER');
	$data_pdf3 = $codehtmlpdf->fraction_html(clx_change_especial_char_html($ANHF->Merge(clx_keywords_simple($value[$position . 'LEFT']),UserRequest::$request)), $this->report->get_orientation(),$this->report->get_unit(), $this->report->get_format(), 'LEFT', 'HEADER');
	
	
//	$xtable = new clx_report_object_table ();

	$max=count($data_pdf1);
	
	
	switch($this->reportrec['PDFPRINTORIENTATION']) {
		case '0':
			$position_center=81;
			$position_right=163;
			break;
		case '1':
			switch($this->reportrec['PDFPRINTFORMAT']) {
				case '0':
					$position_center=110;
					$position_right=220;
					break;
				case '3':
					$position_center=150;
					$position_right=300;
					break;
			}
			break;
	}
	

	for ($t = CLX_NUM_ZERO; $t < $max; $t++) {
		$data_pdf1[$t]['style']["position_x"]=$position_right;
	    $this->report->setHeader(new clx_report_data($data_pdf1[$t]['data'], clx_style::__set_state($data_pdf1[$t]['style'])));
	}
	$max=count($data_pdf2);

	for ($t = CLX_NUM_ZERO; $t < $max; $t++) {
		$data_pdf2[$t]['style']["position_x"]=$position_center;
	    $this->report->setHeader(new clx_report_data($data_pdf2[$t]['data'], clx_style::__set_state($data_pdf2[$t]['style'])));
	}
	$max=count($data_pdf3);
	for ($t = CLX_NUM_ZERO; $t < $max; $t++) {
		$this->report->setHeader(new clx_report_data($data_pdf3[$t]['data'], clx_style::__set_state($data_pdf3[$t]['style'])));
	}

    }

    function Report_Footers() {

		$ANHF = &getTool('ANHF');
		$parametros = array('type_report' => 'PDF', 'search' => $this->reportrec['HEADERCODE'], 'position' => 'FOOTER');


		$value = $ANHF->search_Data($parametros);
        if(empty($value)){
		  return;
        }
		$position = 'F';

		$codehtmlpdf = new html_fpdf_normalize;
		$data_pdf1 = $codehtmlpdf->fraction_html(clx_change_especial_char_html($ANHF->Merge(clx_keywords_simple($value[$position . 'RIGHT']),UserRequest::$request)), $this->report->get_orientation(),$this->report->get_unit(), $this->report->get_format(), 'RIGHT', 'FOOTER');
		$data_pdf2 = $codehtmlpdf->fraction_html(clx_change_especial_char_html($ANHF->Merge(clx_keywords_simple($value[$position . 'CENTER']),UserRequest::$request)), $this->report->get_orientation(), $this->report->get_unit(),$this->report->get_format(), 'CENTER', 'FOOTER');
		$data_pdf3 = $codehtmlpdf->fraction_html(clx_change_especial_char_html($ANHF->Merge(clx_keywords_simple($value[$position . 'LEFT']),UserRequest::$request)), $this->report->get_orientation(), $this->report->get_unit(),$this->report->get_format(), 'LEFT', 'FOOTER');

		for ($t = count($data_pdf1) - CLX_NUM_ONE; $t >= CLX_NUM_ZERO; $t--) {
			$data_pdf1[$t]['style']["position_x"]=163;
		    $this->report->setFooter('left' . $t, new clx_report_data($data_pdf1[$t]['data'], clx_style::__set_state($data_pdf1[$t]['style'])));
		}
		for ($t = count($data_pdf2) - CLX_NUM_ONE; $t >= CLX_NUM_ZERO; $t--) {
			$data_pdf2[$t]['style']["position_x"]=82;
		    $this->report->setFooter('center' . $t, new clx_report_data($data_pdf2[$t]['data'], clx_style::__set_state($data_pdf2[$t]['style'])));
		}
		for ($t = count($data_pdf3) - CLX_NUM_ONE; $t >= CLX_NUM_ZERO; $t--) {
		    $this->report->setFooter('right' . $t, new clx_report_data($data_pdf3[$t]['data'], clx_style::__set_state($data_pdf3[$t]['style'])));
		}
    }

    /**
     * Esto permite colocar la compania eliminar luego
     *
     * @return string
     */
    private function comp_logo($comp_code) {
	$tablename = tableName('COMP');
	if ($tablename === '')
	    return CLX_CHAR_EMPTY_STRING;
	$comprec = ClxCommon::$db->getField($tablename, 'COMP_LOGO', 'IDCODE = ' . singleQuote($comp_code));
	if (empty($comprec))
	    return CLX_CHAR_EMPTY_STRING;
	$OBJS = &getTool('OBJS');
	$fieldinstance = $OBJS->getObject_('DBFT', 'FILE');
	return $fieldinstance->fileName($comprec);
    }

}

class html_fpdf_normalize {

    function fraction_html($code, $orientation, $unit,$pagetype, $position, $position_area) {
        $datos = array();
        if(empty($code)){
		  return $datos;
        }

	$trans = array('<html>' => '', '</html>' => '',  '&nbsp;' => ' ', '<' => '<initag:', '>' => ' fintag:>', "'" => '|',  '<br>' => 'initag:br', '<br />' => 'initag:br', '<br>' => 'initag:br' /* ,'<br />'=>chr(13) */);
	$code=preg_replace("%^[a-zA-Z0-9]+(\.){1}(php|jpg|gif|png|jpeg){1}$%","imagen",$code);
	$code=preg_replace('/<img (.+?)>/', '', $code); 
	$code = preg_replace('/<script[^>]*>.*<\/script>/i', '', $code);
	$code = preg_replace('/<style[^>]*>.*<\/style>/i', '', $code);
	$code = str_replace(chr(CLX_NUM_NINE) . '>', '>', $code);
	$code = str_replace(chr(CLX_NUM_TEN) . '>', '>', $code);
	$code = str_replace(chr(CLX_NUM_13) . '>', '>', $code);
	$code = strtr($code, $trans);
	
	$token = preg_split('/[\<\>\=\s]+/', $code);
	$fin = count($token);
	$type = array();
	$span_style = array('');
	$data = '';
	$salto = -CLX_NUM_ONE;
	$max=count($token);
	for ($t = CLX_NUM_ZERO; $t < $max; $t++) {
	    switch ($token[$t]) {
		case 'initag:p':
		case 'initag:P':
		    if ($salto == CLX_NUM_ZERO)
			$salto = CLX_NUM_ONE;
		    else
			$salto = CLX_NUM_ZERO;
		    $style = '';
		    $data = '';
		     $type[] = 'p';
		    $command = 'basura';
		    $data = '';
		    $max2=count($token);
		    for ($r = $t + CLX_NUM_ONE; $r < $max2; $r++) {
			switch ($token[$r]) {
			    case'initag:br':
				$span_style[count($span_style)] = $span_style[count($span_style) - CLX_NUM_ONE];
				$datos[] = array('style' => $span_style[count($span_style) - CLX_NUM_ONE], 'type' => $type[count($type) - 1], 'data' => $data, 'Cont_br', 'salto' => $salto);
				$salto = CLX_NUM_ZERO;
				$data = '';
				break;
			    case 'initag:/p' :
				$t = $r;
				if (count($datos) > CLX_NUM_ZERO)
				    $datos[count($datos) - CLX_NUM_ONE]['salto'] = CLX_NUM_ONE;
				$r = $fin;
				$span_style = array('');
				break;
			    case 'initag:span':
				$span_style[count($span_style)] = $span_style[count($span_style) - CLX_NUM_ONE];
				if (strlen($data) > CLX_NUM_ONE) {
				    $datos[] = array('style' => $span_style[count($span_style) - CLX_NUM_ONE], 'type' => $type[count($type) - 1], 'data' => $data, 'Cont_span', 'salto' => $salto);
				    $salto = CLX_NUM_ZERO;
				    $data = '';
				}
				break;
			    case 'initag:/span':
				if (strlen($data) > CLX_NUM_ONE) {
				    $datos[] = array('style' => $span_style[count($span_style) - 1], 'type' => $type[count($type) - 1], 'data' => $data, 'Cont_c_span', 'salto' => $salto);
				    $salto = CLX_NUM_ZERO;
				    $data = '';
				}
				array_splice($span_style, -CLX_NUM_ONE);
				break;
			    case 'initag:img':

				$datos[] = array('style' => $span_style[count($span_style) - 1], 'type' => $type[count($type) - 1], 'data' => $data, 'img', 'salto' => $salto);
				break;
			    case 'initag:/em':
			    case 'initag:/strong':
				break;
			    case 'initag:/ol' :
			    case 'initag:/ul' :
				if (strlen($data) > CLX_NUM_ONE) {
				    $datos[] = array('style' => $span_style[count($span_style) - 1], 'type' => $type[count($type) - 1], 'data' => $data, 'Cont_c_ol_ul', 'salto' => $salto);
				    $salto = CLX_NUM_ZERO;
				    $data = '';
				}
				break;
			    case 'initag:em': $style.='|font-italic|';
				break;
			    case 'initag:strong': $style.='|font-weight|';
				break;
			    case 'initag:ol' :
				if (strlen($data) > CLX_NUM_ONE) {
				    $datos[] = array('style' => $span_style[count($span_style) - 1], 'type' => $type[count($type) - 1], 'data' => $data, 'Cont_ol', 'salto' => $salto);
				    $salto = CLX_NUM_ZERO;
				    $data = '';
				}
				$type[] = 'ol';
				break;

			    case 'initag:ul' :
				if (strlen($data) > CLX_NUM_ONE) {
				    $datos[] = array('style' => $span_style[count($span_style) - 1], 'type' => $type[count($type) - 1], 'data' => $data, 'Cont_ul', 'salto' => $salto);
				    $salto = CLX_NUM_ZERO;
				    $data = '';
				}
				$type[] = 'ul';
				break;
			    case 'initag:/li' :
				break;
			    case 'initag:li' : $data.='(li)';
				break;
			    case 'style' :
				$command = 'style';
				break;
			    case 'fintag:' :
				
				$command = 'data';
				break;
			    default:
				if ($command == 'style') {
				    $span_style[count($span_style) - CLX_NUM_ONE].=$token[$r] . ' ';
				} else
				    $$command.=$token[$r] . ' ';
				break;
			}
		    }
		    //-----------------
		    if (strlen($data) > CLX_NUM_ZERO) {
			$datos[] = array('style' => $span_style[count($span_style) - CLX_NUM_ONE], 'type' => $type[count($type) - CLX_NUM_ONE], 'data' => $data, 'Cont_otro', 'salto' => $salto);
			$salto = CLX_NUM_ZERO;
			$data = '';
		    }

		    break;

		case 'initag:ol' :
		case 'initag:ul' :
		    $xtipo = substr($token[$r], -CLX_NUM_TWO);
		    if (strlen($data) > CLX_NUM_ZERO) {
			$datos[] = array('style' => $style, 'type' => $type[count($type) - 1], 'data' => $data, 'cont_' . $xtipo, 'salto' => $salto);
			$salto = CLX_NUM_ZERO;
			$data = '';
		    }
		    $type[] = $xtipo;
		    $max3=count($token);
		    for ($r = $t + CLX_NUM_ONE; $r < $max3; $r++) {
			switch ($token[$r]) {
			    case 'initag:span':
			    case 'initag:/span':
			    case 'initag:/em':
			    case 'initag:/strong':
				break;
			    case 'initag:/' . $xtipo :
				if (strlen($data) > CLX_NUM_ZERO) {
				    $datos[] = array('style' => $span_style[count($span_style) - CLX_NUM_ONE], 'type' => $type[count($type) - 1], 'data' => $data, 'Cont_c_' . $xtipo, 'salto' => $salto);
				    $data = '';
				    array_splice($type, -CLX_NUM_ONE);
				}
				$t = $r;
				$r = $fin;
				break;
			    case 'initag:strong': $style.='|font-weight|';
				break;
			    case 'initag:em': $style.='|font-italic|';
				break;
			    case 'initag:' . $xtipo :
				if (strlen($data) > CLX_NUM_ZERO) {
				    $datos[] = array('style' => $span_style[count($span_style) - CLX_NUM_ONE], 'type' => $type[count($type) - 1], 'data' => $data, 'Cont_c_' . $xtipo, 'salto' => $salto);
				    $salto = CLX_NUM_ZERO;
				    $data = '';
				}
				$type[] = $xtipo;
				break;
			    case 'initag:/li' :
				break;
			    case 'initag:li' : $data.='(li)';
				break;
			    case 'style' :
				$command = 'style';
				break;
			    case 'fintag:' :
				$command = 'data';
				break;
			    default:
				$$command.=$token[$r] . ' ';
				break;
			}
		    }
		    if (strlen($data) > CLX_NUM_ZERO) {
			$datos[] = array('style' => $span_style[count($span_style) - CLX_NUM_ONE], 'type' => $type[count($type) - 1], 'data' => $data, 'Cont_' . $xtipo_otro);
			$salto = CLX_NUM_ZERO;
			$data = '';
		    }

		    break;
		default:
		    break;

	    }
	}


	$datos = $this->normalize_data_style($datos, $orientation, $position, $unit, $position_area,$pagetype);

	return $datos;
    }

    function normalize_data_style($datos, $orientation,  $position,$unit, $position_area, $pagetype = 'A4') {
	$font_height = CLX_NUM_ZERO;
	$datos_new = array();
	if ($position_area == 'HEADER') {
	    $reset_y = CLX_NUM_ONE;
	    $sihubosalto = CLX_NUM_ONE;
	} else {
	    $reset_y = CLX_NUM_ZERO;
	    $sihubosalto = CLX_NUM_ZERO;
	}
	if (is_array($pagetype)) { //FREE SIZE PAPER
	    $size_x=$pagetype[CLX_NUM_ONE];
	    $size_y=$pagetype[CLX_NUM_TWO];
	}
	else {
	    switch (lowercase($pagetype)) { //PAPER
		    case 'a4' :
			$size_x = CLX_NUM_210;
			$size_y = CLX_NUM_297;
			break;
		    case 'a5' : $size_x = CLX_NUM_148;
			$size_y = CLX_NUM_210;
			break;
		    case 'letter' :
		    case 'carta' : $size_x = CLX_NUM_216;
			$size_y = CLX_NUM_279;
			break;
		    case 'legal' : $size_x = CLX_NUM_216;
			$size_y = CLX_NUM_355;
			break;
		    case'oficio' : $size_x = CLX_NUM_216;
			$size_y = CLX_NUM_330;
			break;
		    default:
			break;
	    }
	}
	if ($orientation == 'L') {
	    $temp = $size_x;
	    $size_x = $size_y;
	    $size_y = $temp;
	}
	if ($unit == 'pt')
	    $k = 1;
	elseif ($unit == 'mm')
	    $k = 72 / 25.4;
	elseif ($unit == 'cm')
	    $k = 72 / 2.54;
	elseif ($unit == 'in')
	    $k = 72;
	$size_x/=$k;
	$size_y/=$k;

	$max=count($datos);
	for ($t = CLX_NUM_ZERO; $t < $max; $t++) {
	    if ($position_area == 'FOOTER' and $t + CLX_NUM_ONE == count($datos))
		$reset_y = CLX_NUM_ONE;
	    $datos[$t]['style']=str_replace('"','',$datos[$t]['style']);
	    $token = preg_split("/[\;\|]+/", $datos[$t]['style']);
	    // DEFAULT OPTIONS
	    $font_family = 'Times';
	    $font_size = CLX_NUM_8;
	    $font_aligment = 'left';
	    $font_bold = false;
	    $font_underline = false;
	    $font_italic = null;
	    $padding_left = '';
	    $box_bg_color = null;
	    $font_color = '#000000';
	    if ($position_area == 'HEADER') {
		$position_y = CLX_NUM_25;
		$position_x = CLX_NUM_25;
		$ln = CLX_NUM_ONE;
		$orden = '(-)';
		$margen = CLX_NUM_30;
	    } else {
		$position_y = -CLX_NUM_15;
		$position_x = CLX_NUM_TEN;
		$ln = -CLX_NUM_ONE;
		$orden = '(+)';
		$margen = CLX_NUM_30;
	    }
	    $max2=count($token);

	    for ($r = CLX_NUM_ZERO; $r < $max2; $r++) {
		$cuttoken = explode(':', $token[$r]);
		$margenizq = (int) ($margen / CLX_NUM_TWO);
		$tercio = (int) (($size_x - $margenizq) / CLX_NUM_THREE);


		switch (uppercase($position)) {
		    case 'LEFT' : $position_x = $margenizq;
			break;
		    case 'CENTER' : $position_x = $margenizq + $tercio;
			break;
		    case 'RIGHT' : $position_x = $margenizq + $tercio * CLX_NUM_TWO;
			break;
		    default:
			break;
		}
		if (strlen(trim($token[$r]))>CLX_NUM_ZERO) {

		    switch (trim($cuttoken[CLX_NUM_ZERO])) {
			case 'text-align': $font_aligment = trim($cuttoken[CLX_NUM_ONE]);
			    break;
			case 'font-size' : $font_size = trim($cuttoken[CLX_NUM_ONE]);
			    break;
			case 'font-weight': $font_bold = true;
			    break;
			case 'text-decoration': $font_underline = true;
			    break;
			case 'font-italic:' : $font_italic = true;
			    break;
			case 'font-family' :
			    $font_family = trim($cuttoken[CLX_NUM_ONE]);
			    $font_family = str_ireplace(',chicago', '', $font_family);
			    $font_family = str_ireplace(',avant garde', '', $font_family);
			    $font_family = str_ireplace('comic sans ms,sans-serif', 'comic sans ms', $font_family);
			    $font_family = str_ireplace('trebuchet ms,', '', $font_family);
			    break;
			case 'padding-left' :
			    $padding_left = str_replace('px', '', trim($cuttoken[CLX_NUM_ONE]));
			    $padding_left = str_repeat(chr(CLX_NUM_NINE), (int) ($padding_left / CLX_NUM_30));
			    break;
			case 'color':
			    $font_color = uppercase(trim($cuttoken[CLX_NUM_ONE]));
			    break;
			case 'background-color':
			    $box_bg_color = uppercase(trim($cuttoken[CLX_NUM_ONE]));
			    break;
			default:
			    break;
		    }
		}
	    }

	    if ($reset_y == CLX_NUM_ONE) {
		if ($position_area == 'HEADER')
		    $position_y = CLX_NUM_TEN;
		else
		    $position_y = -CLX_NUM_15;
	    }
	    if ($t > CLX_NUM_ZERO and empty($sihubosalto)) {
		$position_x = null;
	    }
	    if (!$datos[$t]['salto']) {
		if (empty($reset_y))
		    $position_y = CLX_NUM_ZERO;
		$ln = CLX_NUM_ZERO;
		$sihubosalto = CLX_NUM_ZERO;
	    } else
		$sihubosalto = CLX_NUM_ONE;
	    if (((int) ($font_size * CLX_NUM_1_PT_5)) > $font_height)
		$font_height = ((int) ($font_size * CLX_NUM_1_PT_5));

	    $datos[$t]['style'] = array('name' => 'customize', 'font_face' => $font_family, 'font_color' => $font_color,
		'font_size' => $font_size, 'font_aligment' => $font_aligment, 'font_bold' => $font_bold,
		'font_underline' => $font_underline, 'font_italic' => $font_italic,
		'box_width' => -CLX_NUM_ONE, 'box_height' => -CLX_NUM_ONE, 'box_border' => CLX_NUM_ZERO,
		'box_border_size' => null, 'box_border_style' => null, 'box_border_color' => null,
		'box_bg_color' => $box_bg_color, 'box_col_span' => CLX_NUM_ONE, 'box_row_span' => CLX_NUM_ONE, 'box_padding' => CLX_NUM_ZERO,
		'data_type' => 'string', 'data_format' => null, 'position_x' => $position_x,
		'position_y' => $position_y, 'reset_y' => $reset_y, 'orden' => $orden,
		'number1' => null, 'number2' => null, 'number3' => null, 'ln' => $ln, 'wrap_text' => null
	    );

	    $datos[$t]['data'] = $padding_left . $datos[$t]['data'];

	    if ($datos[$t]['type'] <> 'p') {
		if ($datos[$t]['type'] == 'ul')
		    $pos_lista = '\u2022 '; //&#8226; bullet viñeta
		else
		    $pos_lista = $r.'. ';;
		$valores = explode('(li)', $datos[$t]['data']);
		$max=count($valores);
		for ($r = CLX_NUM_ONE; $r < $max; $r++) {
		    $datos[$t]['data'] = chr(CLX_NUM_NINE) . $pos_lista . $valores[$r];
		    $datos_new[] = $datos[$t];
		    $datos[$t]['style']['reset_y'] = -CLX_NUM_ONE;
		}
	    } else {
		$datos_new[] = $datos[$t];
	    }
	    $reset_y = -CLX_NUM_ONE;
	}

	if (count($datos_new) > CLX_NUM_ZERO) {
	    $datos_new[count($datos_new) - CLX_NUM_ONE]['style']['ln'] = CLX_NUM_ONE;
	}
	if ($font_height > CLX_NUM_ZERO) {
	    $max3=count($datos_new);
	    for ($t = CLX_NUM_ZERO; $t < $max3; $t++) {
		$datos_new[$t]['style']['box_height'] = $font_height;
	    }
	}
//	print_r($datos_new);


	return $datos_new;
    }

}
