<?php

/**
 * Object Class
 * For more information contact  support@alenet.com
 *
 * @package		Clientix.Tool.Interface
 * @abstract		System Events Manager - System Alert Message.
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
if (!class_exists('uimi') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'uimi.php';

class uimi_execute extends uimi {

    /**
     * Contructor
     */
    function __construct() {
        $this->code = 'EXEC';
        $this->name = 'Execute external application';
        $this->type = 'UIMI';
        $this->version = '0.1.0.0';
        $this->description = 'This is a Clientix Menu Item that executes an external application';
    }

    /**
     * Build Meny item type on edit menu.
     * @param array $msg
     * @param array $record
     */
    function panel($msg, $record) {
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => '90%'));
        $table->addCell(dict('OBJ_MITM_EXEC_TITLE'), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addField($msg, dict('GEN_LBL_PATHNAME'), 'EXEC_extra1', (array_key_exists('EXEC_extra1', $record)) ? $record['EXEC_extra1'] : '', CLX_NUM_48, CLX_NUM_255);
        $div = new htmlDiv($this->code, 'visibility:hidden; display:none', $table->source());
        return CLX_CHAR_NL . CLX_CHAR_NL . $div->source() . CLX_CHAR_NL . CLX_CHAR_NL;
    }

    /**
     * Verifies Menu Item Panel.
     * @param array $msg
     */
    function checkPanel(&$msg) {
        if (UserRequest::$post['EXEC_extra1'] === '') {
		  $msg['EXEC_extra1'] = 'You have to write a pathname to a executable here.';
        }
    }

    /**
     * Loads form values in record for future save.
     * @param array $record
     */
    function getPanel(&$record) {
        $record['EXTRA1'] = UserRequest::$post['EXEC_extra1'];
    }

    /**
     * Sets form values from menu record
     * @param array $record
     * @param array $values
     */
    function setPanel(&$record, $values) {
        $record['EXEC_extra1'] = $values['EXTRA1'];
    }

    /**
     * Loads Default Panel Values
     * @param array $record
     */
    function defaultPanel(&$record) {
        $record['EXEC_extra1'] = '';
    }

    /**
     * Displays Form Object for menu items tool
     * @param obj $table
     * @param array $record
     */
    function viewForm(&$table, $record) {
        $table->addData(dict('GEN_LBL_PATHNAME') . ':', $record['EXEC_extra1']);
    }

    /**
     * Return string with description of menu item.
     * @param array $record
     * @return string
     */
    function describe($record) {
        return 'Execute application: ' . $record['EXTRA1'];
    }

    /**
     * Returns URL Form Menu Item
     * @param array $record
     * @return string
     */
    function makeURL($record) {
        return sprintf('tool.php?to=EDIT&ac=OS_EXECUTE&idlist=%s&menuid=%s', $record['ID'], $record['IDCODE']);
    }

}
