<?php

/**
 * Sql Statement Utilities
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-10-15 10:19:43 -0400 (Mon, 15 Oct 2018) $
 * @version    	$Rev: 10245 $
 *
 * 7/5/2005 Oracle uppercase record changed
 */
if (!class_exists('util') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'util.php';

class util_query extends util {

	private $stm_cnt;
	private $info;
	private $exec_together = false;
	private $comment_block = false;
	private $current_statement = '';
	private $supress_comment_processing = false;

	const STATEMENT_LABEL_PREFIX = 'Statement #';
	const TOKEN_END_STATEMENT = ';';
	const TOKEN_COMMENT_BLOCK_BEGIN = '/*';
	const TOKEN_COMMENT_BLOCK_END = '*/';
	const TOKEN_COMMENT_ONE_LINE_01 = '--';
	const TOKEN_COMMENT_ONE_LINE_02 = '#';
	
    // CONSTRUCTOR //
    function __construct() {
		$this->code = 'SQLS';
        $this->name = 'SQL Statement Tool';
        $this->type = 'UTIL';
        $this->version = '0.1.0.0';
        $this->description = 'Executes a SQL statement.';
        $this->haspanel = true;
        $this->debug = false;
		if (!defined('TOKEN_EOL')) {
			//define('TOKEN_EOL', CLX_CHAR_CR.CLX_CHAR_NL);
			define('TOKEN_EOL', CLX_CHAR_NL);
		}
//$this->debug = true;
    }

    function panel($msg) {
        $contents = '';
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, 'UTIL');
        $form->addArgument('ut', $this->code);
        $form->addArgument('ac', 'PANEL');
        $form->addArgument('confirm', '1');
        $form->addArgument('sure', (!empty(UserRequest::$post['sure'])) ? UserRequest::$post['sure'] : '');
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell(dict('GEN_LBL_SQLEXE'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addCell(dict('GEN_LBL_SQLWARN'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $t = new htmlObject('textarea', stripslashes((!empty(UserRequest::$post['utl_sql'])) ? UserRequest::$post['utl_sql'] : ''), array('name' => 'utl_sql', 'rows' => CLX_NUM_15, 'cols' => CLX_NUM_90));
        $t->forceclose = true;
        if (isset($msg['utl_sql'])) {
		  $table->addCell($msg['utl_sql'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        }
        $table->addCell($t->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $buttons = '';
        $c = new htmlSubmit(dict('GEN_LBL_EXECUTE'));
        $buttons .= $c->source();
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents = $form->source();
        return $contents;
    }

    function checkPanel(&$msg) {
        $msg = array();
        if (empty(UserRequest::$post['utl_sql'])) {
		  $msg['utl_sql'] = dict('GEN_LBL_MUSTTYPESQL');
        } elseif (UserRequest::$post['sure'] != CLX_NUM_ONE) {
		  $msg['utl_sql'] = dict('GEN_LBL_SURESQL');
		  UserRequest::$post['sure'] = CLX_NUM_ONE;
        }
        return (count($msg) == CLX_NUM_ZERO) ? true : false;
    }

    function execute_() {
        $this->info = '';
        pushHistory($this->code, 'IGNORE_THIS_ENTRY', CLX_NUM_ZERO);
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, 'UTIL');
        $form->addArgument('ac', 'UTL_LIST');
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell(dict('GEN_LBL_SQLRESULT'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $sql = stripslashes(clx_html_spchrs_decode(UserRequest::$post['utl_sql']));
        //$sql = clx_html_spchrs_decode(UserRequest::$post['utl_sql']);
		switch (kDatabaseType) {
			case 'oci':
				$this->exec_together = $this->analyze_sql_oci($sql);
				break;
			default:
				$this->exec_together = $this->analyze_sql_mysql($sql);
		}
		if($this->exec_together === true) {
			$sql = array($sql);
			//$table->addData(dict('GEN_LBL_STATEMENT'), '<b><pre>' . UserRequest::$post['utl_sql'] . '</pre></b>');
		} else {
			if (strpos($sql, self::TOKEN_END_STATEMENT) !== false) {
				//$sql = explode(self::TOKEN_END_STATEMENT, $sql);
				// Split lines (no statements) in order to detect multiline comment /* ... */
				$sql = explode(TOKEN_EOL, $sql);
			} else {
				$sql = array($sql);
			}
			//$table->addData(dict('GEN_LBL_STATEMENT'), '<b>' . str_replace(self::TOKEN_END_STATEMENT, ';<br>', UserRequest::$post['utl_sql']) . '</b>');
		}
		$table->addData(dict('GEN_LBL_STATEMENT'), '<b><pre>' . UserRequest::$post['utl_sql'] . '</pre></b>');
		ClxCommon::$db->backtrace_on_error = false;
		if($this->debug === true) {
			pre('** Start ** SQL execution');
		}
		$this->stm_cnt = 0;
		foreach ($sql as $key => $val) {
		  //$this->executeSql($val, $info);
		  $this->executeSql($val);
        }
		if($this->debug === true) {
			pre('** End ** SQL execution');
		}
        //$table->addData('Result:', $info);
        $table->addData('Result: ', $this->info);
        $this->info = null;
        $c = new htmlButtonReturn();
        $buttons = $c->source();
		unset($c);
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        unset($buttons);
        $form->setContents($table->source());
		unset($table);
        $contents = $form->source();
		unset($form);
		if($this->debug === true) {
			pre('End ['.$this->type.'-'.$this->code.'] TOOL execution');
		}
		ClxCommon::$db->backtrace_on_error = true;
        return $contents;
    }

    /**
     * Executes sql
     *
     * @param string $sql
     * @param strin &$info
     * @return void
     */
    //protected function executeSql($sql, &$info) {
    protected function executeSql($sql) {

		if($this->exec_together === true) {
			$sql = trim($sql);
		} else {
//			$sql = trim(str_replace(self::TOKEN_END_STATEMENT, '', $sql));
			$sql = trim($sql);
			// Skips comments
			if($this->supress_comment_processing === false) {
				//if($this->comment_block === false) {
					//if (strpos($sql, self::TOKEN_COMMENT_ONE_LINE_01) !== false) {
					if (substr($sql, CLX_INIT_ZERO, strlen(self::TOKEN_COMMENT_ONE_LINE_01)) === self::TOKEN_COMMENT_ONE_LINE_01) {
						// Inline comment --
						//$this->current_statement .= substr($sql, CLX_INIT_ZERO, strpos($sql, self::TOKEN_COMMENT_ONE_LINE_01)+strlen(self::TOKEN_COMMENT_ONE_LINE_01));
						return;
					}
					//if (strpos($sql, self::TOKEN_COMMENT_ONE_LINE_02) !== false) {
					if (substr($sql, CLX_INIT_ZERO, strlen(self::TOKEN_COMMENT_ONE_LINE_02)) === self::TOKEN_COMMENT_ONE_LINE_02) {
						// Inline comment #
						//$this->current_statement .= substr($sql, CLX_INIT_ZERO, strpos($sql, self::TOKEN_COMMENT_ONE_LINE_02)+strlen(self::TOKEN_COMMENT_ONE_LINE_02));
						return;
					}
					//if (strpos($sql, self::TOKEN_COMMENT_BLOCK_BEGIN) !== false) {
					if (substr($sql, CLX_INIT_ZERO, strlen(self::TOKEN_COMMENT_BLOCK_BEGIN)) === self::TOKEN_COMMENT_BLOCK_BEGIN) {
						if($this->debug === true) {
							pre('entro 1: ' . $sql);
						}
						// Begin comment block with /*
						$sql = ltrim($sql, self::TOKEN_COMMENT_BLOCK_BEGIN);
						$this->comment_block = true;
						if (strpos($sql, self::TOKEN_COMMENT_BLOCK_END) !== false) {
							// End comment block with */
							$sql = substr($sql, strpos($sql, self::TOKEN_COMMENT_BLOCK_END)+strlen(self::TOKEN_COMMENT_BLOCK_END));
							$this->comment_block = false;
						}
					}
				//}
				//if (strpos($sql, self::TOKEN_COMMENT_BLOCK_END) !== false) {
				if (substr($sql, CLX_INIT_ZERO, strlen(self::TOKEN_COMMENT_BLOCK_END)) === self::TOKEN_COMMENT_BLOCK_END) {
					// End comment block with */
					if($this->debug === true) {
						pre('salio 1: ' . $sql);
					}
					$sql = substr($sql, strpos($sql, self::TOKEN_COMMENT_BLOCK_END)+strlen(self::TOKEN_COMMENT_BLOCK_END));
					$this->comment_block = false;
				}
				if (substr($sql, -strlen(self::TOKEN_COMMENT_BLOCK_END)) === self::TOKEN_COMMENT_BLOCK_END) {
					// End comment block with */
					if($this->debug === true) {
						pre('salio 2: ' . $sql);
					}
					$sql = substr($sql, strpos($sql, self::TOKEN_COMMENT_BLOCK_END)+strlen(self::TOKEN_COMMENT_BLOCK_END));
					$this->comment_block = false;
					return;
				}
				if($this->comment_block === true) {
					return;
				}
			}
			$this->current_statement .= $sql . TOKEN_EOL;
			if (strpos($sql, self::TOKEN_END_STATEMENT) === false) {
				// If no end of statement token ";", add line to statement
			  return;
			}
			$sql = trim(str_replace(self::TOKEN_END_STATEMENT, '', $this->current_statement));
			$this->current_statement = '';
		}

		if (strlen($sql) == CLX_NUM_ZERO) {
		  return;
        }

		if($this->exec_together === true) {
			//$info .= '<b><font size=1>'.(++$this->stm_cnt).') <PRE>' . $sql . ';</PRE><b><br>' . CLX_CHAR_NL;
			$this->info .= '<b><font size=1>'.self::STATEMENT_LABEL_PREFIX.' '.(++$this->stm_cnt).': <PRE>' . $sql . '</PRE><b><br>' . CLX_CHAR_NL;
		} else {
			//$info .= '<b><font size=1><li>'.(++$this->stm_cnt).') ' . $sql . ';</li><b><br>' . CLX_CHAR_NL;
			$this->info .= '<b><font size=1><li>'.self::STATEMENT_LABEL_PREFIX.' '.(++$this->stm_cnt).': <PRE>' . $sql . self::TOKEN_END_STATEMENT . '</PRE></li><b><br>' . CLX_CHAR_NL;
		}
		if($this->debug === true) {
			pre($this->stm_cnt.') '.$sql);
		}
        //$result = ClxCommon::$db->executeQuerySilent($sql);
        ClxCommon::$db->temporary_supress_squote_change = true;
        $result = ClxCommon::$db->executeQuery($sql);
        $err = ClxCommon::$db->errorMsg();
        if (!empty($err)) {
		  //$info .= dict('GEN_ERR_SQLERR') . '<br>';
		  //$info .= $err . ' (Err. No. ' . ClxCommon::$db->errorNum() . ')';
		  $this->info .= dict('GEN_ERR_SQLERR') . '<br>';
		  $this->info .= $err . ' (Err. No. ' . ClxCommon::$db->errorNum() . ')';
        } else {
		  if (is_resource($result) || !empty($result->num_rows)) {
				if($this->debug === true) {
					pre('** Start ** Result DUMP');
				}
			 while ($row = ClxCommon::$db->getNextRecord($result)) {
			     //$info .= implode(',', $row) . '<br>' . CLX_CHAR_NL;
			     $this->info .= implode(' | ', $row) . '<br>' . CLX_CHAR_NL;
			 }
				if($this->debug === true) {
					pre('** End ** Result DUMP');
				}
		  } else {
			 //$info .=dict('GEN_LBL_COMPLETED') . '<br>' . CLX_CHAR_NL;
			 $this->info .= dict('GEN_LBL_COMPLETED') . '<br>' . CLX_CHAR_NL;
		  }
        }
		unset($result);
        //$info .='</font><hr>';
        $this->info .='</font><hr>';
    }
	
	function analyze_sql_mysql($sql) {
		$clues = array();
		$clues[] = 'DELIMITER $$';
		$clues[] = 'CREATE PROCEDURE';
		$clues[] = 'CREATE FUNCTION';
		$clues[] = 'DECLARE';
		$clues[] = 'BEGIN';
		$clues[] = 'END $$';
		$clues[] = 'DELIMITER ;';
		return $this->verify_statement_existance($sql, $clues);
	}
	
	function analyze_sql_oci($sql) {
		$clues = array();
		$clues[] = 'DECLARE';
		$clues[] = 'BEGIN';
		$clues[] = 'END;';
		return $this->verify_statement_existance($sql, $clues);
	}
	
	function verify_statement_existance($sql, $clues) {
		foreach($clues as $val) {
			if(strpos($sql, $val) !== false) {
				return true;
			}
		}
		return false;
	}
}
