<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-22 15:25:30 -0400 (ma., 22 ene. 2019) $
 * @version        $Rev: 10493 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class ac_menuitems extends clx_tool {

    /**
     * Array of dictionary entries required for wizard
     * Loads when wizard is called.
     * @var array
     */
    var $dict = array();

    /**
     * Line break for access rights export to excel
     * @var string
     */
    var $lbreak = CLX_CHAR_NL; //Line Break
    /**
     * Field break for access rights export to excel
     * @var string
     */
    var $fbreak = CLX_CHAR_TAB; //Field Selarator

    /**
     * Constructor
     */

    function __construct() {
        $this->code = 'ACMI';
        $this->name = 'Menu Items Access Manager';
        $this->system = 'Security';
        $this->version = '0.2.0.1';

        $this->has_access = true;


        $this->tablename = 'CLX_AC_MENUITEMS';
        $this->record_name_singular = 'menu access';
        $this->record_name_plural = 'menu accesses';
        $this->primary_field = 'OWNERCODE';
        $this->secondary_field = 'OWNERREF';
		IF (ISSET($GLOBALS['ADDRDEBUGACMENUITEMS']) && $GLOBALS['ADDRDEBUGACMENUITEMS']==$_SERVER['REMOTE_ADDR']) {
			$this->debug=true;
		}	
    }

    function install() {
        $UIMI = &getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_SECURITY:MENU_ACCESS', 'TOOL', $this->code, 'ac=LIST');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['OWNERCODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['OWNERREF'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $fields['MENUID'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['TOOLCODE'] = array('type' => 'varchar(10)', 'null' => 'NO', 'properties' => '', 'default' => '' . singleQuote('NOTOO') . '', 'default_raw' => 'NOTOOL');
        //$fields['TABLECODE'] = array('type' => 'varchar(10)', 'null' => 'NO', 'properties' => '', 'default' => '' . singleQuote('NOTABLE') . '', 'default_raw' => 'NOTABLE');
        $fields['TABLECODE'] = array('type' => 'varchar(10)', 'null' => 'YES', 'properties' => '', 'default' => '' . singleQuote('NOTABLE') . '', 'default_raw' => 'NOTABLE');
        $fields['CANREAD'] = array('type' => 'tinyint(1)', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CANMODIFY'] = array('type' => 'tinyint(1)', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CANDELETE'] = array('type' => 'tinyint(1)', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CANAPPEND'] = array('type' => 'tinyint(1)', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['ISADMIN'] = array('type' => 'tinyint(1)', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'OWNERCODE,OWNERREF,MENUID,TOOLCODE,TABLECODE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB', 'fields' => $fields, 'indexes' => $indexes);
    }

    function action($ac, $idlist) {
        $contents = '';
        $msg = array();
        if ($this->getAccess()->canRead()) {
		  switch ($ac) {
			 case 'WIZARD':
			 case 'REPORT':
			 case 'EXPORT':
			     if (($this->getAccess()->canDisplay()) || $this->getAccess()->canModify()) {
			         $contents = $this->controlWizard();
			     } else {
			         $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [AC_MENUITEMS-CAN_APPEND & CAN_MODIFY][' . $ac . '] ' . $this->getCode(false));
			     }
			     break;
			 case 'MODIFY':
			     if ($this->getAccess()->canModify()) {
			         $id = array_pop($idlist);
			         $record = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
			         UserRequest::$post['ACMI_ownercode'] = $record['OWNERCODE'];
			         UserRequest::$post['ACMI_ownerref'] = $record['OWNERREF'];
			         UserRequest::$post['ACMI_menuid'] = $record['MENUID'];
			         UserRequest::$post['step'] = CLX_NUM_THREE;
			         $contents = $this->editForm3(UserRequest::$post, $msg);
			     } else {
			         $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [AC_MENUITEMS-CAN_MODIFY][' . $ac . '] ' . $this->getCode(false));
			     }
			     break;
			 default:
			     $contents = parent::action($ac, $idlist);
			     break;
		  }
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [AC_MENUITEMS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makelist($listingcode = null) {
        //$t = time('U'); genera un warning
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'ACMIList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = 'a.*, m.NAME MENUNAME';
        $fromclause = 'CLX_AC_MENUITEMS a, CLX_UI_MENUITEMS m';
        $whereclause = 'a.MENUID = m.IDCODE';
        $sortclause = 'OWNERREF';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $table = new htmlTable(CLX_WIDTH_FOUR);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ACMI_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_NAME'), dict('GEN_LBL_MENU'), dict('GEN_LBL_ACCESS')));
        $i = CLX_INIT_ZERO;

        while ($record = $page->nextRecord()) {
		  $recordid = $record['ID'];
		  $select = new htmlCheckbox('selector_' . $recordid, false);
		  $select->setProp('onclick', 'selectButtons("' . $form->getProp(CLX_PROP_ID) . '");');
		  $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
		  if (empty($record['canread']))
			 $record['canread'] = CLX_INIT_ZERO;
		  $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['OWNERCODE'] . ':' . $record['OWNERREF'], dict($record['MENUNAME']), dict($this->yesno[$record['canread']])));
		  $i++;
        }
        $buttons = '';
        if ($this->getAccess()->canDelete()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canModify()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDisplay()) {
		  $b = new htmlFormSubmit(dict('TOOL_ACMI_BUTTON_CREATEWIZ'), 'WIZARD', $form->getProp(CLX_PROP_ID));
		  $b->setProp(CLX_PROP_ID, 'ON');
		  $buttons .= $b->source();
		  $b = new htmlFormSubmit(dict('GEN_LBL_REPORT'), 'REPORT', $form->getProp(CLX_PROP_ID));
		  $b->setProp(CLX_PROP_ID, 'ON');
		  $buttons .= $b->source();
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_EXPORT'), array('action' => 'EXPORT', 'form' => $form->getProp(CLX_PROP_ID), $b->setProp(CLX_PROP_ID, 'ON')))->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    function controlWizard() {
        $contents = '';
        $step = (isset(UserRequest::$post['step'])) ? UserRequest::$post['step'] : '0';
        $msg = array();
        switch ($step) {
		  case '0':
			 $contents = $this->editForm1(UserRequest::$post, $msg);
			 break;
		  case '1':
			 if ($this->checkForm1($msg)) {
			     $contents = $this->editForm2(UserRequest::$post, $msg);
			 } else {
			     $contents = $this->editForm1(UserRequest::$post, $msg);
			 }
			 break;
		  case '2':
			 if ($this->checkForm2($msg)) {
			     $contents = $this->editForm3(UserRequest::$post, $msg);
			 } else {
			     $contents = $this->editForm2(UserRequest::$post, $msg);
			 }
			 break;
		  case '3':
			 if ($this->checkForm3($msg)) {
			     $this->saveWizard();
			     $info = popHistory();
			     $contents = executeHistory($info);
			 } else {
			     $contents = $this->editForm3(UserRequest::$post, $msg);
			 }
			 break;
		  default:
			 break;
        }
        return $contents;
    }

    function editForm1($record, $msg) {
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', UserRequest::$post['ac']);
        $form->addArgument('step', '1');
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ACMI_TITLE_CREATEWIZ'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addCell(dict('TOOL_ACMI_TITLE_CREATEWIZ1'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $codes = new HTMLSelect('ACMI_ownercode');
        $codes->addItem(dict('GEN_LBL_USER'), 'USER', (!empty($record['ACMI_ownercode']) && $record['ACMI_ownercode'] === 'USER'));
        $codes->addItem(dict('GEN_LBL_USERGROUP'), 'GRUP', (!empty($record['ACMI_ownercode']) && $record['ACMI_ownercode'] === 'GRUP'));
        $table->addData(dict('GEN_LBL_GIVE_ACCESS_TO') . ':', $codes->source());
        $form->addContents($table->source());
        $buttons = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CONTINUE'), array('action' => UserRequest::$post['ac'], 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    function checkForm1(&$msg) {
        $msg = array();
        return count($msg) == CLX_NUM_ZERO;
    }

    function editForm2($record, $msg) {
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', UserRequest::$post['ac']);
        $form->addArgument('step', '2');
        $form->addArgument('ACMI_ownercode', $record['ACMI_ownercode']);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ACMI_TITLE_CREATEWIZ'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if ($record['ACMI_ownercode'] === 'USER') {
		  $table->addCell(dict('TOOL_ACMI_TITLE_CREATEWIZ2U'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  $ACUS = &getTool('ACUS');
		  $ACUS->placeChooseUser($msg, dict('GEN_LBL_USERNAME') . ':', 'ACMI_ownerref', (!empty($record['ACMI_ownerref'])) ? $record['ACMI_ownerref'] : '', $table);
        } elseif ($record['ACMI_ownercode'] === 'GRUP') {
		  $table->addCell(dict('TOOL_ACMI_TITLE_CREATEWIZ2G'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  $ACGR = &getTool('ACGR');
		  $ACGR->placeChooseGroup($msg, dict('GEN_LBL_USERGROUP') . ':', 'ACMI_ownerref', (!empty($record['ACMI_ownerref'])) ? $record['ACMI_ownerref'] : '', $table);
        }
        $form->addContents($table->source());
        $buttons = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CONTINUE'), array('action' => 'WIZARD', 'form' => UserRequest::$post['ac']))->source();
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    function checkForm2(&$msg) {
        $msg = array();
        if (UserRequest::$post['ACMI_ownercode'] === 'USER') {
		  $ACUS = &getTool('ACUS');
		  if (!$ACUS->userExists(UserRequest::$post['ACMI_ownerref'])) {
			 $msg['ACMI_ownerref'] = 'There is no user with this code.';
		  }
        } elseif (UserRequest::$post['ACMI_ownercode'] === 'GRUP') {
		  $ACGR = &getTool('ACGR');
		  if (!$ACGR->groupExists(UserRequest::$post['ACMI_ownerref'])) {
			 $msg['ACMI_ownerref'] = 'There is no group with this code.';
		  }
        }
        return count($msg) == CLX_NUM_ZERO;
    }

    function editForm3($record, $msg) {
        $flg_editor = (UserRequest::$post['ac'] === 'WIZARD' ? true : false); // true = access rights editor, false = report mode
        $flg_export = (UserRequest::$post['ac'] === 'EXPORT' ? true : false); // true = EXPORT TO EXCEL, false = other mode
        $variables = array();
        $script = '';
        if (!empty($flg_editor)) {
		  $script = setJavaScriptTag(getJavaScript('ACMI_editform3.js', $variables));
        }
        $this->dict['GEN_LBL_TABLE'] = dict('GEN_LBL_TABLE');
        // the Form --------------------------------------
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', UserRequest::$post['ac']);
        $form->addArgument('step', '3');
        $form->addArgument('ACMI_ownercode', $record['ACMI_ownercode']);
        $form->addArgument('ACMI_ownerref', $record['ACMI_ownerref']);
        // header info --------------------------------------
        if (!empty($flg_export)) {
		  $table = array();
		  $table[] = dict('MENU_ACCESS');
		  $table[] = '';
		  if ($record['ACMI_ownercode'] === 'USER') {
			 $table[] = dict('GEN_LBL_USERNAME') . ': ' . $record['ACMI_ownerref'];
		  } elseif ($record['ACMI_ownercode'] === 'GRUP') {
			 $table[] = dict('GEN_LBL_USERGROUP') . ': ' . $record['ACMI_ownerref'];
		  }
		  $table[] = '';
        } else {
		  $table = new htmlTable(CLX_WIDTH_TWO);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $table->addCell(dict('TOOL_ACMI_TITLE_CREATEWIZ'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  $table->addCell(dict('TOOL_ACMI_TITLE_CREATEWIZ3'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  if ($record['ACMI_ownercode'] === 'USER') {
			 $table->addData(dict('GEN_LBL_USERNAME') . ':', $record['ACMI_ownerref']);
		  } elseif ($record['ACMI_ownercode'] === 'GRUP') {
			 $table->addData(dict('GEN_LBL_USERGROUP') . ':', $record['ACMI_ownerref']);
		  }
		  $form->addContents($table->source());
        }
        // the records --------------------------------------
        if (!empty($flg_export)) {
		  $table[] = dict('GEN_LBL_IDCODE') . CLX_CHAR_TAB . dict('MENU_MENU_ITEMS') . $this->fbreak . dict('GEN_LBL_CANREAD') . $this->fbreak . dict('GEN_LBL_CANMODIFY') . $this->fbreak . dict('GEN_LBL_CANDELETE') . $this->fbreak . dict('GEN_LBL_CANAPPEND') . $this->fbreak . dict('GEN_LBL_ISADMIN');
        } else {
		  $savekey = CLX_INIT_ZERO;
		  $table = new htmlTable(CLX_WIDTH_SIX);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  //$table->addCells(CLX_CLASS_HEADERS,array( dict('GEN_LBL_MENU')));
		  $canread = CLX_INIT_ZERO; //total selector empty
		  $check1 = new htmlCheckBox('canread_ALL_MENUS', $canread, CLX_NUM_ONE);
		  $check1->setProp(CLX_PROP_CLASS, 'canread ALL_MENUS');
		  $check2 = new htmlCheckBox('canmodify_ALL_MENUS', $canread, CLX_NUM_ONE);
		  $check2->setProp(CLX_PROP_CLASS, 'canmodify ALL_MENUS');
		  $check3 = new htmlCheckBox('candelete_ALL_MENUS', $canread, CLX_NUM_ONE);
		  $check3->setProp(CLX_PROP_CLASS, 'candelete ALL_MENUS');
		  $check4 = new htmlCheckBox('canappend_ALL_MENUS', $canread, CLX_NUM_ONE);
		  $check4->setProp(CLX_PROP_CLASS, 'canappend ALL_MENUS');
		  $check5 = new htmlCheckBox('isadmin_ALL_MENUS', $canread, CLX_NUM_ONE);
		  $check5->setProp(CLX_PROP_CLASS, 'isadmin ALL_MENUS');
		  $tabs = '';
		  $name = '';
		  $table->addCell($tabs . $name, array(CLX_PROP_WIDTH => '34%', CLX_PROP_CLASS => CLX_CLASS_HEADERS));
		  //$table->addCell(dict('GEN_LBL_DISPLAYMENU') . '<br>' . $check0->source(),array(CLX_PROP_WIDTH=>'11%',CLX_PROP_CLASS=>CLX_CLASS_HEADERS));
		  if (!empty($flg_editor)) {
			 $table->addCell(dict('GEN_LBL_CANREAD') . '<br>' . $check1->source(), array(CLX_PROP_WIDTH => '11%', CLX_PROP_CLASS => CLX_CLASS_HEADERS));
			 $table->addCell(dict('GEN_LBL_CANMODIFY') . '<br>' . $check2->source(), array(CLX_PROP_WIDTH => '11%', CLX_PROP_CLASS => CLX_CLASS_HEADERS));
			 $table->addCell(dict('GEN_LBL_CANDELETE') . '<br>' . $check3->source(), array(CLX_PROP_WIDTH => '11%', CLX_PROP_CLASS => CLX_CLASS_HEADERS));
			 $table->addCell(dict('GEN_LBL_CANAPPEND') . '<br>' . $check4->source(), array(CLX_PROP_WIDTH => '11%', CLX_PROP_CLASS => CLX_CLASS_HEADERS));
			 $table->addCell(dict('GEN_LBL_ISADMIN') . '<br>' . $check5->source(), array(CLX_PROP_WIDTH => '11%', CLX_PROP_CLASS => CLX_CLASS_HEADERS));
		  } else {
			 $table->addCell('<center>' . dict('GEN_LBL_CANREAD') . '</center>', array(CLX_PROP_WIDTH => '11%', CLX_PROP_CLASS => CLX_CLASS_HEADERS));
			 $table->addCell('<center>' . dict('GEN_LBL_CANMODIFY') . '</center>', array(CLX_PROP_WIDTH => '11%', CLX_PROP_CLASS => CLX_CLASS_HEADERS));
			 $table->addCell('<center>' . dict('GEN_LBL_CANDELETE') . '</center>', array(CLX_PROP_WIDTH => '11%', CLX_PROP_CLASS => CLX_CLASS_HEADERS));
			 $table->addCell('<center>' . dict('GEN_LBL_CANAPPEND') . '</center>', array(CLX_PROP_WIDTH => '11%', CLX_PROP_CLASS => CLX_CLASS_HEADERS));
			 $table->addCell('<center>' . dict('GEN_LBL_ISADMIN') . '</center>', array(CLX_PROP_WIDTH => '11%', CLX_PROP_CLASS => CLX_CLASS_HEADERS));
		  }
        }
        $level = CLX_INIT_ZERO;
        $menurecs = ClxCommon::$db->getRecords('CLX_UI_MENUITEMS', 'PARENT=' . singleQuote('0') . ' AND ACTIVE=1 ORDER BY SEQNUM');
        while ($menurec = ClxCommon::$db->getNextRecord($menurecs)) {
		  $this->listMenuItems($table, $record['ACMI_ownercode'], $record['ACMI_ownerref'], $menurec, $level, CLX_NUM_ZERO, 'ALL_MENUS', $savekey);
        }
		ClxCommon::$db->freeResult( $menurecs );
        if (empty($flg_export)) {
		  $form->addContents($table->source());
        }
        // buttons  --------------------------------------
        $buttons = '';
        if (!empty($flg_editor)) {
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => UserRequest::$request['ac'], 'form' => $form->getProp(CLX_PROP_ID)))->source();
		  $c = new htmlButtonCancel();
		  $buttons .= $c->source();
        }
        // Footers
        if ($buttons !== '') {
		  $table = new htmlTable(CLX_WIDTH_ONE);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $table->addCells(CLX_CLASS_FOOTERS, array($buttons));
		  $form->addContents($table->source());
        }
        if (!empty($flg_export)) {
		  // Send excel file to response and salir
		  return $this->output_to_excel($this->export_line_to_excel($table), 'ac_menuitems.xls');
        }
        return $form->source() . $script;
    }

    function listMenuItems(&$table, $ownercode, $ownerref, $menurec, $level, $parent, $class, &$savekey) {
        $flg_editor = (UserRequest::$post['ac'] === 'WIZARD' ? true : false); // true = access rights editor, false = report mode
        $flg_export = (UserRequest::$post['ac'] === 'EXPORT' ? true : false); // true = EXPORT TO EXCEL, false = other mode
        $savekey++;
        $menuid = $menurec['IDCODE'];
        $tabs = '';
		IF ($this->debug) {
			pre($menurec);	
			
		}	
		$span='';
		if ($this->debug===true) {
			$span=' <span>'.$menurec['NAME'].'</span>';
		}
        if ($flg_editor && $menurec['TYPE'] === 'MENU') {
            $name = '<b><a href="javascript:donothing();" onclick="showpanel(' . singleQuote('MENU' . $menuid) . ');">' . dict($menurec['NAME'], false) . '</a>'.$span.'</b>';
        } elseif ($flg_editor && $menurec['TYPE'] === 'LSTR') {
            $name = '<b><a href="javascript:donothing();" onclick="showpanel(' . singleQuote('MENU' . $menuid . '_TBLS') . ');">' . dict($menurec['NAME'], false) . '</a>'.$span. '</b>';
        } else {
            $name = dict($menurec['NAME'], false).$span;
        }
        for ($j = CLX_NUM_ZERO; $j < $level; $j++) {
		  $tabs .= str_repeat(CLX_CHAR_HTML_NBSP, 6);
        }
        if (!empty($class)) {
		  $class = $class . ' ' . $menuid;
        } else {
		  $class = $menuid;
        }
        //Set key for security
        switch ($menurec['TYPE']) {
		  case 'TOOL':
			 $toolcode = $menurec['EXTRA1'];
			 $tablecode = 'NOTABLE';
			 break;
		  case 'LSTR':
			 $toolcode = 'EDIT';
			 $tablecode = 'NOTABLE';
			 break;
		  case 'ETRE':
			 $toolcode = 'ACTE';
			 $tablecode = 'NOTABLE';
			 break;
		  case 'CODM':
			 $toolcode = 'CODE';
			 $tablecode = 'NOTABLE';
			 break;
		  case 'PAGE':
			 $toolcode = 'UIPG';
			 $tablecode = 'NOTABLE';
			 break;
		  default:
			 $toolcode = 'NOTOOL';
			 $tablecode = 'NOTABLE';
			 break;
        }
        //adding separator || to split on save
        $menukey = $menuid . '||' . $toolcode . '||' . $tablecode;
        $hidden1 = new htmlFieldHidden('menukey[' . $savekey . ']' . $menuid, $menukey);
        $sec = $this->getWizardCurrentAccess($ownercode, $ownerref, $menuid, $toolcode, $tablecode);
        $a_check = array();

        if (!empty($flg_editor)) {
		  $check1 = new htmlCheckBox('canread_' . $savekey, $sec['CANREAD'], CLX_NUM_ONE);
		  $check1->setProp(CLX_PROP_CLASS, 'canread ' . $class);
		  $check2 = new htmlCheckBox('canmodify_' . $savekey, $sec['CANMODIFY'], CLX_NUM_ONE);
		  $check2->setProp(CLX_PROP_CLASS, 'canmodify ' . $class);
		  $check3 = new htmlCheckBox('candelete_' . $savekey, $sec['CANDELETE'], CLX_NUM_ONE);
		  $check3->setProp(CLX_PROP_CLASS, 'candelete ' . $class);
		  $check4 = new htmlCheckBox('canappend_' . $savekey, $sec['CANAPPEND'], CLX_NUM_ONE);
		  $check4->setProp(CLX_PROP_CLASS, 'canappend ' . $class);
		  ;
		  $check5 = new htmlCheckBox('isadmin_' . $savekey, $sec['ISADMIN'], CLX_NUM_ONE);
		  $check5->setProp(CLX_PROP_CLASS, 'isadmin ' . $class);
		  $a_check[CLX_NUM_ONE] = $check1->source();
		  $a_check[CLX_NUM_TWO] = $check2->source();
		  $a_check[CLX_NUM_THREE] = $check3->source();
		  $a_check[CLX_NUM_FOUR] = $check4->source();
		  $a_check[CLX_NUM_FIVE] = $check5->source();
        } else {
		  // Report
		  $a_check[CLX_NUM_ONE] = $sec['CANREAD'];
		  $a_check[CLX_NUM_TWO] = $sec['CANMODIFY'];
		  $a_check[CLX_NUM_THREE] = $sec['CANDELETE'];
		  $a_check[CLX_NUM_FOUR] = $sec['CANAPPEND'];
		  $a_check[CLX_NUM_FIVE] = $sec['ISADMIN'];
		  $a_check = $this->process_a_check($a_check);
        }
        if (!empty($flg_export)) {
		  $table[] = $menurec['IDCODE'] . CLX_CHAR_TAB . $tabs . $name . $this->fbreak . $a_check[CLX_NUM_ONE] . $this->fbreak . $a_check[CLX_NUM_TWO] . $this->fbreak . $a_check[CLX_NUM_THREE] . $this->fbreak . $a_check[CLX_NUM_FOUR] . $this->fbreak . $a_check[CLX_NUM_FIVE];
        } else {
		  $table->addCell($tabs . $name . $hidden1->source(), array(CLX_PROP_WIDTH => '34%', CLX_PROP_CLASS => CLX_CLASS_DATA));
		  $table->addCell($a_check[CLX_NUM_ONE], array(CLX_PROP_WIDTH => '11%', CLX_PROP_CLASS => CLX_CLASS_DATA));
		  $table->addCell($a_check[CLX_NUM_TWO], array(CLX_PROP_WIDTH => '11%', CLX_PROP_CLASS => CLX_CLASS_DATA));
		  $table->addCell($a_check[CLX_NUM_THREE], array(CLX_PROP_WIDTH => '11%', CLX_PROP_CLASS => CLX_CLASS_DATA));
		  $table->addCell($a_check[CLX_NUM_FOUR], array(CLX_PROP_WIDTH => '11%', CLX_PROP_CLASS => CLX_CLASS_DATA));
		  $table->addCell($a_check[CLX_NUM_FIVE], array(CLX_PROP_WIDTH => '11%', CLX_PROP_CLASS => CLX_CLASS_DATA));
        }
        if ($menurec['TYPE'] === 'MENU') {
		  if (!empty($flg_export)) {
			 $sonstable = array();
		  } else {
			 $sonstable = new htmlTable($table->width);
			 $sonstable->setProp(CLX_PROP_WIDTH, CLX_PROP_100PCT);
		  }
		  $sons = ClxCommon::$db->getRecords('CLX_UI_MENUITEMS', 'PARENT="' . $menuid . '" AND ACTIVE=1 ORDER BY SEQNUM');
		  while ($son = ClxCommon::$db->getNextRecord($sons)) {
			 $this->listMenuItems($sonstable, $ownercode, $ownerref, $son, $level + CLX_NUM_ONE, $menuid, $class, $savekey);
		  }
		  ClxCommon::$db->freeResult( $sons );
		  if (!empty($flg_export)) {
			 $table[] = $this->export_line_to_excel($sonstable);
		  } else {
			 $div = new htmlDiv('MENU' . $menuid, ($flg_editor ? 'display:none;' : ''), $sonstable->source());
			 $table->addCell($div->source(), array(CLX_PROP_COLSPAN => $table->width));
		  }
        }
        //adding tables
        if ($menurec['TYPE'] === 'LSTR') {
		  if (!empty($flg_export)) {
			 $sonstable = array();
		  } else {
			 $sonstable = new htmlTable($table->width);
			 $sonstable->setProp(CLX_PROP_WIDTH, CLX_PROP_100PCT);
		  }
		  $donetbl = array();
		  $this->listMenuItemsTables($sonstable, $ownercode, $ownerref, $menuid, $menurec['EXTRA1'], $level + CLX_NUM_ONE, $menuid, $class, CLX_NUM_ZERO, $savekey, $donetbl);
		  unset($donetbl);
		  if (!empty($flg_export)) {
			 $table[] = $this->export_line_to_excel($sonstable);
		  } else {
			 $div = new htmlDiv('MENU' . $menuid . '_TBLS', ($flg_editor ? 'display:none;' : ''), $sonstable->source());
			 $table->addCell($div->source(), array(CLX_PROP_COLSPAN => $table->width));
		  }
        }
    }

    function listMenuItemsTables(&$table, $ownercode, $ownerref, $menuid, $listingcode, $level, $parent, $class, $loopnum, &$savekey, &$donetbl) {
        $flg_editor = (UserRequest::$post['ac'] === 'WIZARD' ? true : false); // true = access rights editor, false = report mode
        $flg_export = (UserRequest::$post['ac'] === 'EXPORT' ? true : false); // true = EXPORT TO EXCEL, false = other mode
        if ($loopnum > CLX_NUM_THREE) {
		  return;
        }

        $tabs = '';
        //Getting Table Names, Codes and Dictionaries
        $sql = 'SELECT DBTB.IDCODE, DBTB.IDNAME, DBTB.MODULECODE, IFNULL(DICT.VALUE, IFNULL((SELECT MAX(D.VALUE) FROM CLX_DICTIONARY D WHERE D.ENTRY = DBTB.RECORDNAME_PLURAL AND UPPER(D.LANGUAGE) =' . singleQuote(uppercase(kSystemLanguage)) . '), DBTB.RECORDNAME_PLURAL)) RECORDNAME_PLURAL FROM CLX_UI_LISTINGS UILG, CLX_DB_TABLES DBTB LEFT JOIN CLX_DICTIONARY DICT ON DICT.ENTRY = DBTB.RECORDNAME_PLURAL AND UPPER(DICT.LANGUAGE) = ' . singleQuote(UPPERCASE(getCurrentLanguage())) . '   WHERE DBTB.IDCODE = UILG.TABLECODE AND UILG.IDCODE = ' . singleQuote($listingcode);
        $tbrec = ClxCommon::$db->getOneRecord($sql);
		 
        if (empty($tbrec['IDCODE']) && !empty($donetbl[$menuid . '||' . $tbrec['IDCODE']]))
		  return;
        $donetbl[$menuid . '||' . $tbrec['IDCODE']] = 1;

        $savekey++;
        $name = $this->dict['GEN_LBL_TABLE'] . ': ' . $tbrec['RECORDNAME_PLURAL'] . ' (' . $tbrec['IDCODE'] . ')';
        for ($j = CLX_NUM_ZERO; $j < $level; $j++) {
		  $tabs .= '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
        }
        $class = ($class) ? $class . ' ' . $menuid : $menuid;
        //Set key
        $menukey = $menuid . '||EDIT||' . $tbrec['IDCODE'];
        $hidden1 = new htmlFieldHidden('menukey[' . $savekey . ']' . $menuid, $menukey);
        //Contatenando codigo de menu y tabla para selector.
        $sec = $this->getWizardCurrentAccess($ownercode, $ownerref, $menuid, 'EDIT', $tbrec['IDCODE']);
        $a_check = array();
        if (!empty($flg_editor)) {
		  $check1 = new htmlCheckBox('canread_' . $savekey, $sec['CANREAD'], CLX_NUM_ONE);
		  $check1->setProp(CLX_PROP_CLASS, 'canread ' . $class . ' tbl_' . $savekey);
		  $check2 = new htmlCheckBox('canmodify_' . $savekey, $sec['CANMODIFY'], CLX_NUM_ONE);
		  $check2->setProp(CLX_PROP_CLASS, 'canmodify ' . $class . ' tbl_' . $savekey);
		  $check3 = new htmlCheckBox('candelete_' . $savekey, $sec['CANDELETE'], CLX_NUM_ONE);
		  $check3->setProp(CLX_PROP_CLASS, 'candelete ' . $class . ' tbl_' . $savekey);
		  $check4 = new htmlCheckBox('canappend_' . $savekey, $sec['CANAPPEND'], CLX_NUM_ONE);
		  $check4->setProp(CLX_PROP_CLASS, 'canappend ' . $class . ' tbl_' . $savekey);
		  $check5 = new htmlCheckBox('isadmin_' . $savekey, $sec['ISADMIN'], CLX_NUM_ONE);
		  $check5->setProp(CLX_PROP_CLASS, 'isadmin ' . $class . ' tbl_' . $savekey);
		  $a_check[CLX_NUM_ONE] = $check1->source();
		  $a_check[CLX_NUM_TWO] = $check2->source();
		  $a_check[CLX_NUM_THREE] = $check3->source();
		  $a_check[CLX_NUM_FOUR] = $check4->source();
		  $a_check[CLX_NUM_FIVE] = $check5->source();
        } else {
		  // Report
		  $a_check[CLX_NUM_ONE] = $sec['CANREAD'];
		  $a_check[CLX_NUM_TWO] = $sec['CANMODIFY'];
		  $a_check[CLX_NUM_THREE] = $sec['CANDELETE'];
		  $a_check[CLX_NUM_FOUR] = $sec['CANAPPEND'];
		  $a_check[CLX_NUM_FIVE] = $sec['ISADMIN'];
		  $a_check = $this->process_a_check($a_check);
        }
        if (!empty($flg_export)) {
		  $table[] = CLX_CHAR_TAB . $tabs . $name . $this->fbreak . $a_check[CLX_NUM_ONE] . $this->fbreak . $a_check[CLX_NUM_TWO] . $this->fbreak . $a_check[CLX_NUM_THREE] . $this->fbreak . $a_check[CLX_NUM_FOUR] . $this->fbreak . $a_check[CLX_NUM_FIVE];
        } else {
		  $table->addCell($tabs . $name . $hidden1->source(), array(CLX_PROP_WIDTH => '34%', CLX_PROP_CLASS => CLX_CLASS_DATA));
		  $table->addCell($a_check[CLX_NUM_ONE], array(CLX_PROP_WIDTH => '11%', CLX_PROP_CLASS => CLX_CLASS_DATA));
		  $table->addCell($a_check[CLX_NUM_TWO], array(CLX_PROP_WIDTH => '11%', CLX_PROP_CLASS => CLX_CLASS_DATA));
		  $table->addCell($a_check[CLX_NUM_THREE], array(CLX_PROP_WIDTH => '11%', CLX_PROP_CLASS => CLX_CLASS_DATA));
		  $table->addCell($a_check[CLX_NUM_FOUR], array(CLX_PROP_WIDTH => '11%', CLX_PROP_CLASS => CLX_CLASS_DATA));
		  $table->addCell($a_check[CLX_NUM_FIVE], array(CLX_PROP_WIDTH => '11%', CLX_PROP_CLASS => CLX_CLASS_DATA));
        }
        //adding sublistings
        $sql = 'SELECT DISTINCT FL.LISTINGCODE FROM CLX_UI_LISTINGS LI, CLX_UI_LISTINGS LI2, CLX_UI_FORM_LISTINGS FL WHERE FL.FORMCODE IN (LI.DISPLAY_FORM, LI.MODIFY_FORM, LI.DUPLICATE_FORM, LI.EFFTDATE_FORM, LI.DELETE_FORM, LI.SEARCH_FORM) AND LI2.IDCODE = FL.LISTINGCODE AND FL.LISTINGCODE <> LI.IDCODE AND LI.IDCODE = ' . singleQuote($listingcode);
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $this->listMenuItemsTables($table, $ownercode, $ownerref, $menuid, $r['LISTINGCODE'], $level, $menuid, $class, $loopnum + CLX_NUM_ONE, $savekey, $donetbl);
        }
		ClxCommon::$db->freeResult( $rs );
    }

    function checkForm3(&$msg) {
        $msg = array();
        return count($msg) == CLX_NUM_ZERO;
    }

    function process_a_check($a_check) {
        $flg_export = (UserRequest::$post['ac'] === 'EXPORT') ? true : false; // true = EXPORT TO EXCEL, false = other mode
        reset($a_check);
        foreach ($a_check as $key => $val) {
		  $val = ($val === CLX_NUM_ZERO) ? '' : 'X';
		  if($flg_export === false){
			 $a_check[$key] = '<center>' . $val . '</center>';
		  }else{
			 $a_check[$key] = $val;
		  }
        }
        return $a_check;
    }

    function export_line_to_excel($table) {
        return str_replace(CLX_CHAR_HTML_NBSP, ' ', implode($this->lbreak, $table)) . $this->lbreak;
    }

    function output_to_excel($content, $filename) {
        // Directly sends the excel file to the output stream (fails on large datasets)
        ob_clean();
		sendDownloadFile($filename, 'xls', 'application/vnd.ms-excel; encoding');
        return decode_string($content);
    }

    /**
     * Loads menu menu access
     * @param string $ownercode
     * @param string $ownerref
     * @param string $menuid
     * @param string $toolcode
     * @param string $tablecode
     */
    function getWizardCurrentAccess($ownercode, $ownerref, $menuid, $toolcode, $tablecode) {
        $rec = ClxCommon::$db->getFields($this->tablename, 'CANREAD,CANMODIFY,CANDELETE,CANAPPEND,ISADMIN', 'OWNERCODE=' . singleQuote($ownercode), 'OWNERREF=' . singleQuote($ownerref), 'MENUID=' . singleQuote($menuid), 'TOOLCODE=' . singleQuote($toolcode), 'TABLECODE=' . singleQuote($tablecode));
        if (empty($rec)) {
		  return array('CANREAD' => CLX_NUM_ZERO,
			 'CANMODIFY' => CLX_NUM_ZERO,
			 'CANDELETE' => CLX_NUM_ZERO,
			 'CANAPPEND' => CLX_NUM_ZERO,
			 'ISADMIN' => CLX_NUM_ZERO);
        } else {
		  return array('CANREAD' => intval($rec['CANREAD']),
			 'CANMODIFY' => intval($rec['CANMODIFY']),
			 'CANDELETE' => intval($rec['CANDELETE']),
			 'CANAPPEND' => intval($rec['CANAPPEND']),
			 'ISADMIN' => intval($rec['ISADMIN']));
        }
    }

    function saveWizard() {
        $OBJS = getTool('OBJS');
        $ownercode = UserRequest::$post['ACMI_ownercode'];
        $ownerref = UserRequest::$post['ACMI_ownerref'];
		if ($this->debug===true) {
			pre('UserRequest__post');
			pre(UserRequest::$post);
			pre('_POST');
			pre($_POST);
		}
        foreach (UserRequest::$post['menukey'] as $key => $val) {
		  $menurec = explode('||', $val);

		  // Save the menu access.
		  $record = array('OWNERCODE' => $ownercode, 'OWNERREF' => $ownerref, 'MENUID' => $menurec[CLX_NUM_ZERO], 'TOOLCODE' => $menurec[CLX_NUM_ONE], 'TABLECODE' => $menurec[CLX_NUM_TWO],
			 'CANREAD' => (empty(UserRequest::$post['canread_' . $key]) ? CLX_NUM_ZERO : CLX_NUM_ONE),
			 'CANMODIFY' => (empty(UserRequest::$post['canmodify_' . $key]) ? CLX_NUM_ZERO : CLX_NUM_ONE),
			 'CANDELETE' => (empty(UserRequest::$post['candelete_' . $key]) ? CLX_NUM_ZERO : CLX_NUM_ONE),
			 'CANAPPEND' => (empty(UserRequest::$post['canappend_' . $key]) ? CLX_NUM_ZERO : CLX_NUM_ONE),
			 'ISADMIN' => (empty(UserRequest::$post['isadmin_' . $key]) ? CLX_NUM_ZERO : CLX_NUM_ONE));
		  //$crit = 'OWNERCODE=' . singleQuote($ownercode) . ' AND OWNERREF=' . singleQuote($ownerref) . ' AND MENUID=' . singleQuote($menurec[CLX_NUM_ZERO]) . ' AND TOOLCODE=' . singleQuote($menurec[CLX_NUM_ONE]) . ' AND TABLECODE=' . singleQuote($menurec[CLX_NUM_TWO]);
		  $crit = 'OWNERCODE=' . singleQuote($ownercode) . ' AND OWNERREF=' . singleQuote($ownerref) . ' AND MENUID=' . singleQuote($menurec[CLX_NUM_ZERO]) . ' AND TOOLCODE=' . singleQuote($menurec[CLX_NUM_ONE]) . ' AND ('.($menurec[CLX_NUM_TWO] === CLX_CHAR_EMPTY_STRING || is_null($menurec[CLX_NUM_TWO]) ? 'TABLECODE IS NULL OR ' : '').'TABLECODE=' . singleQuote($menurec[CLX_NUM_TWO]).')';
            if ($this->debug===true) {
				pre( 'record');
				pre($record);
				pre('ownercode');
				pre($ownercode);
				pre('crit');
				pre($crit);
			}
		  $id = ClxCommon::$db->loadDbRecord($this->tablename, $record, false, $crit);
		  unset($id);
        }//end for
        //
        //Setting Tools and table Security required for selected menus
        //this section deals with masive updates
        foreach ($OBJS->objectList('UIMI') as $val) {
		  $menu = getToolObject('UIMI', $val);
		  $menu->saveAccessWizard($ownercode, $ownerref);
        }
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['ACMI_ownercode'] . ':' . $record['ACMI_ownerref']);
        $table->addData(dict('GEN_LBL_MENU') . ':', $record['ACMI_menuid']);
    }

    function filterFormFill($msg, &$table) {
        $codes = new HTMLSelect('ACMI_ownercode');
        $codes->addItem('', '', true);
        $codes->addItem(dict('GEN_LBL_USER'), 'USER', false);
        $codes->addItem(dict('GEN_LBL_USERGROUP'), 'GRUP', false);
        if (!array_key_exists('ACMI_ownerref', UserRequest::$post)) {
		  UserRequest::$post['ACMI_ownerref'] = '';
        }
        $field = new HTMLField('ACMI_ownerref', UserRequest::$post['ACMI_ownerref'], CLX_NUM_30, CLX_NUM_30);
        $table->addData(dict('GEN_LBL_GIVE_ACCESS_TO') . ':', $codes->source() . $field->source());
    }

    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['ACMI_ownercode'])) {
		  $filter .= $boolean . ' OWNERCODE = ' . singleQuote(UserRequest::$post['ACMI_ownercode']);
		  $boolean = 'AND';
		  $filter .= $boolean . ' OWNERREF LIKE ' . singleQuote(UserRequest::$post['ACMI_ownerref'] . '%');
		  $boolean = 'AND';
        }
        $_SESSION['paging']['ACMIList']['sortby'] = '';
        $_SESSION['paging']['ACMIList']['criteria'] = $filter;
        $_SESSION['paging']['ACMIList']['startat'] = '0';
    }
	///////////////////////////////////////////////////
	///comentado 22/01/2019 para utilizar la clx_tool
	//////////////////////////////////////////////////
    /*function clearfilter() {
        $_SESSION['paging']['ACMIList']['criteria'] = '';
    }*/

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $recordid);
        $record = array();
        $record['ACMI_id'] = $recordid;
        $record['ACMI_ownercode'] = $values['OWNERCODE'];
        $record['ACMI_ownerref'] = $values['OWNERREF'];
        $record['ACMI_menuid'] = $values['MENUID'];
        $record['ACMI_canread'] = intval($values['CANREAD']);
        $record['ACMI_canmodify'] = intval($values['CANMODIFY']);
        $record['ACMI_candelete'] = intval($values['CANDELETE']);
        $record['ACMI_canappend'] = intval($values['CANAPPEND']);
        $record['ACMI_isadmin'] = intval($values['ISADMIN']);
        return $record;
    }

    function deleteRecordSk($id) {
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
    }

    function defaultRecord() {
        $record = array();
        $record['ACMI_ownercode'] = 'USER';
        $record['ACMI_ownerref'] = $_SESSION['user_record']['username'];
        $record['ACMI_menuid'] = CLX_INIT_ZERO;
        $record['ACMI_canread'] = CLX_INIT_ZERO;
        $record['ACMI_canread'] = CLX_INIT_ZERO;
        $record['ACMI_canmodify'] = CLX_INIT_ZERO;
        $record['ACMI_candelete'] = CLX_INIT_ZERO;
        $record['ACMI_canappend'] = CLX_INIT_ZERO;
        $record['ACMI_isadmin'] = CLX_INIT_ZERO;
        return $record;
    }

    function userGroups($username) {
        $list = array();
        $records = ClxCommon::$db->getRecords('CLX_AC_GROUP_MEMBERS', 'UPPER(USERNAME)=' . singleQuote(uppercase($username)));
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
		  $list[] = singlequote($record['GROUPCODE']);
        }
		ClxCommon::$db->freeResult( $records );
        return $list;
    }

    function securityQuery() {
        $username = $_SESSION['user_record']['username'];
        $usersql = '(OWNERCODE=' . singleQuote('USER') . ' AND UPPER(OWNERREF)=' . singleQuote(uppercase($username)) . ')';
        $usergroups = $this->userGroups($username);
        if (count($usergroups) > CLX_NUM_ZERO) {
		  $usergroups = toString($usergroups);
		  $sql = '(' . $usersql . ' or (OWNERCODE=' . singleQuote('GRUP') . ' AND OWNERREF IN (' . $usergroups . ')))';
        } else {
		  $sql = $usersql;
        }
        return $sql;
    }

}
