<?php

/**
 * Process Seccion Manager
 *
 * @package		Clientix.Tool.Business Process
 * @abstract		Business Process Manager
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version    	$Rev: 9615 $
 *
 */
class ac_user_fsecure extends clx_tool {

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'ACFS';
        $this->name = 'Field Access Manager';
        $this->system = 'Security';
        $this->version = '5.5.0.0';




        $this->tablename = 'CLX_AC_USER_FSECURE';
        $this->record_name_singular = 'GEN_LBL_AC_USER_FSECURE';
        $this->record_name_plural = 'GEN_LBL_AC_USER_FSECURES';
        $this->primary_field = 'username';
        $this->secondary_field = 'FIELDNAME';
        $this->terciary_field = 'FVALUE';

        $this->editform = array(CLX_CLASS_TITLE => 'TOOL_ACFS_TITLE_CREATE', // ETIQUETA
		  'panels' => array(
			 array('type' => 'normal',
			     'entries' => array(
			         array('label' => 'GEN_LBL_USERNAME',
					   'fields' => array(
						  array('name' => 'username', 'type' => 'lookup', 'default' => '', 'tool' => 'ACUS', 'readonly' => CLX_NUM_ONE,
						      'validations' => array(
								array('type' => 'string', 'required' => '1'), array('type' => 'lookup', 'fieldname' => 'username', 'tablename' => 'CLX_AC_USERS'))
						  )
					   )
			         ), array('label' => 'GEN_LBL_FIELDNAME',
					   'fields' => array(
						  array('name' => 'fieldname', 'type' => 'lookup', 'default' => '', 'tool' => 'DBFL', 'validations' => array(
								array('type' => 'string', 'required' => '1'), array('type' => 'lookup', 'fieldname' => 'fieldname', 'tablename' => 'CLX_DB_FIELDS'))
						  )
					   )
			         ), array('label' => 'GEN_LBL_FIELDVALUE',
					   'caption' => $this->code . '_FIELDVALUE_CAPTION', 'fields' => array(
						  array('name' => 'fieldvalue',
						      'type' => 'string',
						      'default' => '',
						      'size' => '255',
						      'displaysize' => '30',
						      'validations' => array(
								array('type' => 'string', 'required' => CLX_NUM_ONE))
						  )
					   )
			         )
			     )
			 )
		  )
        );
        $this->viewform = $this->editform;
        $this->viewform [CLX_CLASS_TITLE] = 'TOOL_ACFS_TITLE_DISPLAY';
        $this->toolpanelkeys = array(array('name' => 'USERNAME', 'type' => 'string'));
        $this->toolpanel = array('name' => $this->tablename, CLX_CLASS_TITLE => $this->record_name_plural, 'has_reorder' => false,
		  'has_export' => false,
		  'has_filter' => false,
		  'has_duplicate' => false,
		  'fields' => array(
			 array('name' => 'ID', 'label' => 'GEN_LBL_ID', 'type' => 'number'),
			 array('name' => 'USERNAME', 'label' => 'GEN_LBL_USERNAME', 'type' => 'string'),
			 array('name' => 'FIELDNAME', 'label' => 'GEN_LBL_FIELDNAME', 'type' => 'string'),
			 array('name' => 'FIELDVALUE', 'label' => 'GEN_LBL_FIELDVALUE', 'type' => 'string')
		  )
        );
    }

    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::toolTableStructure()
     */
    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['USERNAME'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $fields['FIELDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['FIELDVALUE'] = array('type' => 'varchar(255)', 'null' => 'NO', 'properties' => '');
        $fields['CDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['MDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['MODIFIEDBY'] = array('type' => 'varchar(30)', 'null' => 'YES', 'properties' => '');
        $fields['CREATEDBY'] = array('type' => 'varchar(30)', 'null' => 'YES', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes [$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes [$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'USERNAME,FIELDNAME');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB', 'fields' => $fields, 'indexes' => $indexes);
    }

// end method
    /**
     * Form Checker
     *
     * @param void
     * @return void
     */
    function checkForm(&$msg) {
        parent::checkForm($msg);
        if (UserRequest::$request['ac'] <> 'MODIFY') {
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, 'UPPER(USERNAME) = ' . singleQuote(uppercase(UserRequest::$post[$this->code . '_username'])) . ' AND FIELDNAME = ' . singleQuote(UserRequest::$post[$this->code . '_fieldname']));
		  if ($qty > CLX_NUM_ZERO) {
			 $msg[$this->code . '_fieldname'] = dict('GEN_ERR_UNIQUE');
		  }
        }
        return count($msg) == CLX_NUM_ZERO;
    }

    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::defaultRecord()
     */
    function defaultRecord() {
        $callingpage = peekHistory();
        $record = parent::defaultRecord();
        $record [$this->code . '_username'] = $callingpage ['username'];
        if (ACFS_DEF_FIELDNAME <> '0') {
		  $record [$this->code . '_fieldname'] = ACFS_DEF_FIELDNAME;
        }
        if (ACFS_DEF_FIELDVALUE <> '0') {
		  $record [$this->code . '_fieldvalue'] = ACFS_DEF_FIELDVALUE;
        }
        return $record;
    }

// end method
    /**
     * (non-PHPdoc)
     * @see clx_tool::saveRecord()
     */
    function createRecord_() {
        $record = array();
        $record['USERNAME'] = UserRequest::$post[$this->code . '_username'];
        $record['FIELDNAME'] = UserRequest::$post[$this->code . '_fieldname'];
        $record['FIELDVALUE'] = UserRequest::$post[$this->code . '_fieldvalue'];
        $record['CDATE'] = xFormat::date('NOW');
        $record['MDATE'] = $record['CDATE'];
        $record['CREATEDBY'] = $_SESSION['user_record']['username'];
        $record['MODIFIEDBY'] = $_SESSION['user_record']['username'];
        ClxCommon::$db->createRecord($this->tablename, $record, false);
    }

    /**
     * (non-PHPdoc)
     * @see clx_tool::saveRecord()
     */
    function saveRecord() {
        $record = array();
        $record['USERNAME'] = UserRequest::$post[$this->code . '_username'];
        $record['FIELDNAME'] = UserRequest::$post[$this->code . '_fieldname'];
        $record['FIELDVALUE'] = UserRequest::$post[$this->code . '_fieldvalue'];
        $record['MDATE'] = xFormat::date('NOW');
        $record['MODIFIEDBY'] = $_SESSION['user_record']['username'];
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post[$this->code . '_id']);
    }

    /**
     * Registers field security to session
     */
    function set_session_field_security() {
        $crits = array();
        $rs = ClxCommon::$db->getRecords($this->tablename, 'UPPER(USERNAME) = ' . singleQuote(uppercase($_SESSION['user_record']['username'])));
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $crits[$r['FIELDNAME']] = toArray($r['FIELDVALUE']);
        }
		ClxCommon::$db->freeResult($rs);
        $_SESSION['user_record'][$this->code . '_sec'] = $crits;
    }

    /**
     * Builds critaria for queries
     * @param string $querycode
     * @return string
     */
    function get_query_criteria($querycode) {
        $criteria = '';
        $comma = '';

        if (isset($_SESSION['user_record']['isadmin']) && $_SESSION['user_record']['isadmin'] == CLX_NUM_ONE) {
		  return $criteria;
        }

        if (!empty($_SESSION['user_record']['ACFS_sec'])) {
		  //test direct criteria
		  $sql = 'SELECT DISTINCT DBTF.TABLECODE, DBTF.FIELDNAME FROM CLX_AC_USER_FSECURE ACFS, CLX_DB_QUERY_TABLES DBQT, CLX_DB_TABLEFIELDS DBTF WHERE DBTF.TABLECODE = DBQT.TABLECODE AND DBTF.FIELDNAME = ACFS.FIELDNAME AND UPPER(DBQT.QUERYCODE) = ' . singleQuote(uppercase($querycode)) . ' AND UPPER(ACFS.USERNAME) = ' . singleQuote(uppercase($_SESSION['user_record']['username']));
		  $rs = ClxCommon::$db->executeQuery($sql);
		  while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
			 if (count($_SESSION['user_record']['ACFS_sec'][$r['FIELDNAME']]) > CLX_NUM_ONE) {
			     $criteria .= $comma . '(UPPER(' . $r['TABLECODE'] . '.' . $r['FIELDNAME'] . ') IN (' . singleQuote(uppercase(implode(singleQuote(','), $_SESSION['user_record']['ACFS_sec'][$r['FIELDNAME']]))) . ')' . ' OR ' . $r['TABLECODE'] . '.' . $r['FIELDNAME'] . ' IS NULL)';
			     $comma = ' AND ';
			 } elseif (count($_SESSION['user_record']['ACFS_sec'][$r['FIELDNAME']]) == CLX_NUM_ONE) {
			     $criteria .= $comma . '(UPPER(' . $r['TABLECODE'] . '.' . $r['FIELDNAME'] . ')=' . singleQuote(uppercase($_SESSION['user_record']['ACFS_sec'][$r['FIELDNAME']][CLX_NUM_ZERO])) . ' OR ' . $r['TABLECODE'] . '.' . $r['FIELDNAME'] . ' IS NULL)';
			     $comma = ' AND ';
			 }
		  }
		  ClxCommon::$db->freeResult($rs);
		  if ($criteria <> '') {
			 return $criteria;
		  }
		  //Adding lookup condition in case field is lookup an table is the lookup table
		  //for example if the field is lookup comp_code should also filter comp.idcode
		  //or the copy field.
		  $sql = 'SELECT DISTINCT ACFS.FIELDNAME PRMT_FIELDNAME, DBFL.FIELD_EXTRA1 TABLECODE, DBFL.FIELD_EXTRA2 FIELDNAME FROM CLX_AC_USER_FSECURE ACFS, CLX_DB_QUERY_TABLES DBQT, CLX_DB_TABLEFIELDS DBTF, CLX_DB_FIELDS DBFL WHERE DBTF.TABLECODE = DBQT.TABLECODE AND DBFL.FIELDNAME = ACFS.FIELDNAME AND DBTF.FIELDNAME = DBFL.FIELD_EXTRA2 AND DBTF.TABLECODE = DBFL.FIELD_EXTRA1 AND DBFL.FIELDTYPE = ' . singleQuote('LOOK') . ' AND DBQT.QUERYCODE = ' . singleQuote($querycode) . ' AND UPPER(ACFS.USERNAME) = ' . singleQuote($_SESSION['user_record']['username']);
		  $rs = ClxCommon::$db->executeQuery($sql);
		  while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
			 if (count($_SESSION['user_record']['ACFS_sec'][$r['PRMT_FIELDNAME']]) > CLX_NUM_ONE) {
			     $criteria .= $comma . $r['TABLECODE'] . '.' . $r['FIELDNAME'] . ' IN (' . singleQuote(implode(singleQuote(','), $_SESSION['user_record']['ACFS_sec'][$r['PRMT_FIELDNAME']])) . ')';
			     $comma = ' AND ';
			 } elseif (count($_SESSION['user_record']['ACFS_sec'][$r['PRMT_FIELDNAME']]) == CLX_NUM_ONE) {
			     $criteria .= $comma . $r['TABLECODE'] . '.' . $r['FIELDNAME'] . '=' . singleQuote($_SESSION['user_record']['ACFS_sec'][$r['PRMT_FIELDNAME']][CLX_NUM_ZERO]);
			     $comma = ' AND ';
			 }
		  }
		  ClxCommon::$db->freeResult($rs);
		  if ($criteria <> '') {
			 return $criteria;
		  }
		  //Adding lookup condition in case field is lookup in a parent table
		  //and is not a join table of the query. Order by critical to get first lookup from table.
		  //evaluar de limitar a requerido. FIELD_REQUIRED
		  $sql = 'SELECT DBTB.MODULECODE, DBTB.IDNAME, DBTB.IDCODE ALIAS, DBTF.TABLECODE, DBTF.FIELDNAME, DBTF.FIELD_REQUIRED, DBFL.FIELD_EXTRA2 PARENT_FIELD, ACFS.FIELDNAME FILTER_FIELD FROM CLX_DB_TABLES DBTB, CLX_DB_QUERY_TABLES DBQT, CLX_DB_TABLEFIELDS DBTF, CLX_DB_FIELDS DBFL, CLX_AC_USER_FSECURE ACFS WHERE DBTF.TABLECODE = DBQT.TABLECODE AND DBTF.FIELDNAME = DBFL.FIELDNAME AND DBTB.IDCODE = DBFL.FIELD_EXTRA1 AND DBFL.FIELDTYPE = ' . singleQuote('LOOK') . ' AND DBQT.QUERYCODE = ' . singleQuote($querycode) . ' AND UPPER(ACFS.USERNAME) = ' . singleQuote(uppercase($_SESSION['user_record']['username'])) . ' AND EXISTS(SELECT DBTF2.ID FROM CLX_DB_TABLEFIELDS DBTF2 WHERE DBTF2.FIELDNAME = ACFS.FIELDNAME AND DBTF2.TABLECODE = DBFL.FIELD_EXTRA1) ORDER BY DBTF.FIELD_REQUIRED DESC, DBTF.SEQNUM ASC LIMIT 1';
		  $rs = ClxCommon::$db->executeQuery($sql);
		  while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {

			 $criteria .= $comma . ' (EXISTS(SELECT ' . singleQuote('FIELDSEC') . ' ';
			 $criteria .= 'FROM ' . $r['MODULECODE'] . '_' . $r['IDNAME'] . ' ' . $r['ALIAS'] . ' ';
			 $criteria .= 'WHERE ' . $r['ALIAS'] . '.' . $r['PARENT_FIELD'] . ' = ' . $r['TABLECODE'] . '.' . $r['FIELDNAME'];
			 $criteria .= ' AND ' . $r['ALIAS'] . '.' . $r['FILTER_FIELD'];
			 if (count($_SESSION['user_record']['ACFS_sec'][$r['FILTER_FIELD']]) > CLX_NUM_ONE) {
			     $criteria .= ' IN (' . singleQuote(implode(singleQuote(','), $_SESSION['user_record']['ACFS_sec'][$r['FILTER_FIELD']])) . ')';
			 } elseif (count($_SESSION['user_record']['ACFS_sec'][$r['FILTER_FIELD']]) == CLX_NUM_ONE) {
			     $criteria .= ' = ' . singleQuote($_SESSION['user_record']['ACFS_sec'][$r['FILTER_FIELD']][CLX_NUM_ZERO]);
			 }
			 $criteria .= ') OR '. $r['TABLECODE'] . '.' . $r['FIELDNAME'].' IS NULL OR '. $r['TABLECODE'] . '.' . $r['FIELDNAME'].' = '.singleQuote(CLX_CHAR_EMPTY_STRING).')';
			 if ($r['FIELD_REQUIRED'] == CLX_NUM_ZERO) {
			     $criteria = '(' . $criteria . ' OR ' . $r['TABLECODE'] . '.' . $r['FIELDNAME'] . ' = ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' OR ' . $r['TABLECODE'] . '.' . $r['FIELDNAME'] . ' IS NULL) ';
			 }
			 $comma = ' AND ';
		  }
		  ClxCommon::$db->freeResult($rs);

        }
        return $criteria;
    }

    /**
     * Builds critaria for queries
     * @param string $querycode
     * @return string
     */
    function get_lookup_criteria($tablecode, $table_alias) {
        $criteria = '';
        $comma = '';

        if ($_SESSION['user_record']['isadmin'] == CLX_NUM_ONE) {
		  return $criteria;
        }

        if (!empty($_SESSION['user_record']['ACFS_sec'])) {
		  $sql = 'SELECT DISTINCT DBTF.TABLECODE, DBTF.FIELDNAME FROM CLX_AC_USER_FSECURE ACFS, CLX_DB_TABLEFIELDS DBTF WHERE DBTF.TABLECODE = ' . singleQuote($tablecode) . ' AND DBTF.FIELDNAME = ACFS.FIELDNAME AND UPPER(ACFS.USERNAME) = ' . singleQuote(uppercase($_SESSION['user_record']['username']));
		  $rs = ClxCommon::$db->executeQuery($sql);
		  while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
			 if (count($_SESSION['user_record']['ACFS_sec'][$r['FIELDNAME']]) > CLX_NUM_ONE) {
			     $criteria .= $comma . $table_alias . '.' . $r['FIELDNAME'] . ' IN (' . singleQuote(implode('' . singleQuote(',') . '', $_SESSION['user_record']['ACFS_sec'][$r['FIELDNAME']])) . ')';
			     $comma = ' AND ';
			 } elseif (count($_SESSION['user_record']['ACFS_sec'][$r['FIELDNAME']]) == CLX_NUM_ONE) {
			     $criteria .= $comma . $table_alias . '.' . $r['FIELDNAME'] . '=' . singleQuote($_SESSION['user_record']['ACFS_sec'][$r['FIELDNAME']][CLX_NUM_ZERO]);
			     $comma = ' AND ';
			 }
		  }
		  ClxCommon::$db->freeResult($rs);
		  //si ya se encontro solo retorna.
		  if ($criteria <> '') {
			 return $criteria;
		  }
		  //Adding lookup condition in case field is lookup an table is the lookup table
		  //for example if the field is lookup comp_code should also filter comp.idcode
		  //or the copy field
		  $sql = 'SELECT DISTINCT ACFS.FIELDNAME PRMT_FIELDNAME, DBFL.FIELD_EXTRA1 TABLECODE, DBFL.FIELD_EXTRA2 FIELDNAME FROM CLX_AC_USER_FSECURE ACFS, CLX_DB_TABLEFIELDS DBTF, CLX_DB_FIELDS DBFL WHERE DBTF.TABLECODE = ' . singleQuote($tablecode) . ' AND DBFL.FIELDNAME = ACFS.FIELDNAME AND DBTF.FIELDNAME = DBFL.FIELD_EXTRA2 AND DBTF.TABLECODE = DBFL.FIELD_EXTRA1 AND DBFL.FIELDTYPE = ' . singleQuote('LOOK') . ' AND UPPER(ACFS.USERNAME) = ' . singleQuote(uppercase($_SESSION['user_record']['username']));
		  $rs = ClxCommon::$db->executeQuery($sql);
		  while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
			 if (count($_SESSION['user_record']['ACFS_sec'][$r['PRMT_FIELDNAME']]) > CLX_NUM_ONE) {
			     $criteria .= $comma . $table_alias . '.' . $r['FIELDNAME'] . ' IN (' . singleQuote(implode('' . singleQuote(',') . '', $_SESSION['user_record']['ACFS_sec'][$r['PRMT_FIELDNAME']])) . ')';
			     $comma = ' AND ';
			 } elseif (count($_SESSION['user_record']['ACFS_sec'][$r['PRMT_FIELDNAME']]) == CLX_NUM_ONE) {
			     $criteria .= $comma . $table_alias . '.' . $r['FIELDNAME'] . '=' . singleQuote($_SESSION['user_record']['ACFS_sec'][$r['PRMT_FIELDNAME']][CLX_NUM_ZERO]);
			     $comma = ' AND ';
			 }
		  }
		  ClxCommon::$db->freeResult($rs);
        }
        return $criteria;
    }

}

// End class
