<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Included label
 *  Oracle uppercase record changed
 */
class an_Graphics_ranges extends clx_tool {

    var $serie_types;

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'ANGR';
        $this->name = 'Graphics Ranges';
        $this->system = 'Analytics';
        $this->version = '4.0.0.0';


        $this->accesskey = 'ANGM';

        $this->tablename = 'CLX_AN_GRAPHICS_RANGES';
        $this->record_name_singular = 'Graphic ranges';
        $this->record_name_plural = 'Graphics ranges';
        $this->primary_field = 'CHARTCODE';
        $this->secondary_field = 'SEQNUM';
        $this->seqnum_field = 'SEQNUM';
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['CHARTCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['SEQNUM'] = array('type' => 'tinyint(4) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['FROM_VALUE'] = array('type' => 'varchar(40)', 'null' => 'NO', 'properties' => '');
        $fields['TO_VALUE'] = array('type' => 'varchar(40)', 'null' => 'NO', 'properties' => '');
        $fields['DISPLAY_LABEL'] = array('type' => 'varchar(20)', 'null' => 'NO', 'properties' => '');
        $fields['COLOR'] = array('type' => 'varchar(20)', 'null' => 'NO', 'properties' => '');
        $fields['TRANSPARENCY'] = array('type' => 'tinyint(1)', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'CHARTCODE,SEQNUM');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB', 'fields' => $fields, 'indexes' => $indexes);
    }

    function makepanel($groupcode = null) {
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'ANGRList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $form->addArgument('chartcode', $groupcode);
        $selectclause = '*';
        $fromclause = $this->tablename;
        $whereclause = 'CHARTCODE=' . singleQuote($groupcode);
        $sortclause = 'SEQNUM';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $table = new htmlTable(CLX_WIDTH_SIX);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ANGR_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_SIX));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_DISPLAY_LABEL'), dict('GEN_LBL_FROM'), dict('GEN_LBL_TO'), dict('GEN_LBL_COLOR'), dict('GEN_LBL_TRANSPARENCY')));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
		  $recordid = $record['ID'];
		  $select = new htmlCheckbox('selector_' . $recordid, false);
		  $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
		  $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
		  $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['DISPLAY_LABEL'], $record['FROM_VALUE'], $record['TO_VALUE'], $record['COLOR'], $record['TRANSPARENCY']));
		  $i++;
        }
        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        if ($this->getAccess()->canModify()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDelete()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDisplay()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_RANGE')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        }
        if ($page->getProp('qty') > CLX_NUM_ONE) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_REORDER'), array('action' => 'REORDER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        }
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    function editForm($ac, $idlist, $record, $msg) {
        $modify = (UserRequest::$request['ac'] === 'MODIFY');
        $title = ($modify) ? dict('TOOL_ANGR_TITLE_MODIFY') : dict('TOOL_ANGR_TITLE_CREATE');
        //Build the form
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('ANGR_id', $record['ANGR_id']);
        $form->addArgument('idlist', toString($idlist));
        $form->addArgument('ANGR_chartcode', $record['ANGR_chartcode']);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addField($msg, dict('GEN_LBL_DISPLAY_LABEL') . ':', 'ANGR_display_label', $record['ANGR_display_label'], CLX_NUM_40, CLX_NUM_40);
        $table->addField($msg, dict('GEN_LBL_FROM') . ':', 'ANGR_from_value', $record['ANGR_from_value'], CLX_NUM_40, CLX_NUM_40);
        $table->addField($msg, dict('GEN_LBL_TO') . ':', 'ANGR_to_value', $record['ANGR_to_value'], CLX_NUM_40, CLX_NUM_40);
        $table->addField($msg, dict('GEN_LBL_COLOR') . ':', 'ANGR_color', $record['ANGR_color'], CLX_NUM_20, CLX_NUM_20);
        $table->addField($msg, dict('GEN_LBL_TRANSPARENCY') . '(100 => Min, 0 => Max)' . ':', 'ANGR_transparency', $record['ANGR_transparency'], CLX_NUM_20, CLX_NUM_20);
        //$table->addField($msg, dict('GEN_LBL_SEQNUM') . ':', 'ANGR_seqnum', $record['ANGR_seqnum'], 10, 10);
        $buttons = '';
        if (!empty($modify)) {
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => $ac, 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        }
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    function checkForm(&$msg) {
        $msg = array();
        xValidate::required($msg, 'ANGR_chartcode', UserRequest::$post['ANGR_chartcode']);
        xValidate::number($msg, 'ANGR_from_value', UserRequest::$post['ANGR_from_value'], '', '', 'ANGR_from_value');
        xValidate::number($msg, 'ANGR_to_value', UserRequest::$post['ANGR_to_value'], '', '', 'ANGR_to_value');
        xValidate::integer($msg, 'ANGR_transparency', UserRequest::$post['ANGR_transparency'], CLX_NUM_ZERO, CLX_NUM_HUNDRED, 'ANGR_transparency');
        return count($msg) == CLX_NUM_ZERO;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_FROMVALUE') . ':', $record['ANGR_from_value']);
    }

    function reorderRecords() {
        $records = ClxCommon::$db->getRecords($this->tablename, 'CHARTCODE=' . singleQuote(UserRequest::$post['chartcode']) . ' ORDER BY SEQNUM');
        $list = array();
        while (($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE))) {
		  $list[$record['ID']] = $record['FROM_VALUE'] . '-' . $record['TO_VALUE'];
        }
        return $list;
    }

    function reorderAction() {
        parent::reorderAction();
        $callingpage = peekHistory();
        $chartcode = $callingpage['chartcode'];
        $ANGR = &getTool('ANGR');
        $ANGR->versionIncrease('IDCODE', $chartcode);
    }

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $recordid);
        $record = array();
        $record['ANGR_id'] = $recordid;
        $record['ANGR_chartcode'] = $values['CHARTCODE'];
        $record['ANGR_display_label'] = $values['DISPLAY_LABEL'];
        $record['ANGR_from_value'] = $values['FROM_VALUE'];
        $record['ANGR_to_value'] = $values['TO_VALUE'];
        $record['ANGR_color'] = $values['COLOR'];
        $record['ANGR_transparency'] = $values['TRANSPARENCY'];
        $record['ANGR_seqnum'] = $values['SEQNUM'];
        return $record;
    }

    function saveRecord() {
        $record = array();
        $record['CHARTCODE'] = UserRequest::$post['ANGR_chartcode'];
        $record['DISPLAY_LABEL'] = UserRequest::$post['ANGR_display_label'];
        $record['FROM_VALUE'] = UserRequest::$post['ANGR_from_value'];
        $record['TO_VALUE'] = UserRequest::$post['ANGR_to_value'];
        $record['COLOR'] = UserRequest::$post['ANGR_color'];
        $record['TRANSPARENCY'] = UserRequest::$post['ANGR_transparency'];
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['ANGR_id']);
        $ANGR = &getTool('ANGR');
        $ANGR->versionIncrease('IDCODE', UserRequest::$post['ANGS_chartcode']);
    }

    function createRecordVoid_() {
        ClxFreeReturn($this->createRecord_());
    }

    function createRecord_() {
        $record = array();
        $record['CHARTCODE'] = UserRequest::$post['ANGR_chartcode'];
        $record['DISPLAY_LABEL'] = UserRequest::$post['ANGR_display_label'];
        $record['FROM_VALUE'] = UserRequest::$post['ANGR_from_value'];
        $record['TO_VALUE'] = UserRequest::$post['ANGR_to_value'];
        $record['COLOR'] = UserRequest::$post['ANGR_color'];
        $record['TRANSPARENCY'] = UserRequest::$post['ANGR_transparency'];
        $record['SEQNUM'] = ClxCommon::$db->getField($this->tablename, 'SEQNUM', 'CHARTCODE=' . singleQuote($record['CHARTCODE']) . ' ORDER BY SEQNUM DESC') + CLX_NUM_ONE;
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $ANGR = &getTool('ANGR');
        $ANGR->versionIncrease('IDCODE', UserRequest::$post['ANGS_chartcode']);
        return $recordid;
    }

    function deleteRecordSk($id) {
        $graphicrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        $ANGR = &getTool('ANGR');
        $ANGR->versionIncrease('IDCODE', $graphicrec['CHARTCODE']);
    }

    function defaultRecord() {
        $record = array();
        $record['ANGR_id'] = CLX_INIT_ZERO;
        $record['ANGR_chartcode'] = UserRequest::$request['chartcode'];
        $record['ANGR_display_label'] = '';
        $record['ANGR_from_value'] = '0';
        $record['ANGR_to_value'] = CLX_NUM_HUNDRED;
        $record['ANGR_color'] = '#00FF00';
        $record['ANGR_transparency'] = CLX_NUM_HUNDRED;
        $record['ANGR_seqnum'] = '1';
        return $record;
    }

}
