<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class db_export_fields extends clx_tool {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'DBEF';
        $this->name = 'Export Fields';
        $this->system = 'Database';
        $this->version = '0.1.2.0';



        $this->accesskey = 'DBEX';
        $this->tablename = 'CLX_DB_EXPORT_FIELDS';
        $this->record_name_singular = 'export field';
        $this->record_name_plural = 'export fields';
        $this->primary_field = 'EXPORTCODE';
        $this->secondary_field = 'FIELDNAME';
        $this->seqnum_field = 'SEQNUM';
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['EXPORTCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['TABLECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['FIELDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['FIELDALIAS'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $fields['SEQNUM'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'EXPORTCODE,SEQNUM');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
		  'fields' => $fields,
		  'indexes' => $indexes);
    }

    function pack($record) {
        $o = $this->packRecord($record);
        $id = $record['ID'];
        $objs = new XMLObject($this->code . '_OBJECTS');
        $this->packsub($objs, 'DBET', 'SELECT * FROM CLX_DB_EXPORT_TRANSLATIONS WHERE EXPFIELDID=' . $id);
        $o->addObject($objs);
        return $o;
    }

    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null, 'id' => null), $parameters));
        list($package, $formid) = $parameters;
        $record = $this->unpackRecord($package);
        $expfieldid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        if ($package->hasSubObject($this->code . '_OBJECTS')) {
		  $objs = $package->getObjectXml($this->code . '_OBJECTS');
		  foreach ($objs->objects as $o) {
			 $toolcode = $o->tag;
			 $tool = getTool($toolcode);
			 $tool->unpack(array('package' => $o, 'id' => $expfieldid));
		  }
        }
    }

    function makepanel($groupcode = null) {
        if ($this->getAccess()->canRead()) {
		  $t = time('U');
		  $to = $this->code;
		  $here = 'tool.php?to=' . $to . '&t=' . $t;
		  $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
		  $form->setProp(CLX_PROP_ID, 'DBEFPanel');
		  $form->addArgument(CLX_PROP_TO, $this->code);
		  $form->addArgument('exportcode', $groupcode);
		  $form->addArgument('ac', 'NOT_DEFINED');
		  $selectclause = '*';
		  $fromclause = $this->tablename;
		  $whereclause = 'EXPORTCODE=' . singleQuote($groupcode);
		  $sortclause = 'SEQNUM';
		  $page = new xPaging();
		  $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
		  $page->setProp('printheaders', false);
		  $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
		  $table = new htmlTable(CLX_WIDTH_FIVE);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $page->header($table);
		  $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_FIELDNAME'), dict('GEN_LBL_SEQNUM'), dict('GEN_LBL_FIELDALIAS'), dict('GEN_LBL_TRANSLATION')));
		  $i = CLX_INIT_ZERO;
		  while ($record = $page->nextRecord()) {
			 $recordid = $record['ID'];
			 $select = new htmlCheckbox('selector_' . $recordid, false);
			 $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
			 $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
                $qty = (ClxCommon::$db->oneExistsRecords('CLX_DB_EXPORT_TRANSLATIONS', 'EXPFIELDID = ' . $record['ID']) > CLX_NUM_ZERO) ? CLX_NUM_ONE : CLX_NUM_ZERO;
			 $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['FIELDNAME'], $record['SEQNUM'], $record['FIELDALIAS'], dict($this->yesno[$qty])));
			 $i++;
		  }
		  $buttons = '';
		  /* no meaning for display */
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  if ($this->getAccess()->canDisplay() || $this->getAccess()->canModify()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE') . ' / ' . dict('GEN_LBL_MODIFY'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => ''))->source();
		  }
		  $form->addContents($page->footers($table, $buttons));
		  return buildPanelInterface(dict('TOOL_DBEF_TITLE_LIST'), 'EXPORT_FIELDS_' . $groupcode, $form->source(), CLX_NUM_MINUS_1);
        } else {
		  return CLX_CHAR_EMPTY_STRING;
        }
    }

    function editForm($ac, $idlist, $record, $msg) {
        $contents = '';
        $select_size = CLX_NUM_12;
        $from_sel = $this->code . '_query_fields';
        $to_sel = $this->code . '_sel_fields';
        $scripts = 'var selitem =' . singleQuote(CLX_CHAR_EMPTY_STRING) . ';' . CLX_CHAR_NL . ' var itemslabels = new Object(); ' . CLX_CHAR_NL;

        $scripts1 = setJavaScriptTag(getJavaScriptFilename('DBEF_editform.js'), true);
        $options = '';
        //$callingpage = peekHistory();
        $exportcode = $record['DBEF_exportcode'];
        // The Form ------------------------------------------------------------
        $title = dict('TOOL_DBIF_TITLE');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('idlist', toString($idlist));
        $form->addArgument('DBEF_exportcode', $exportcode);
        $form->addArgument('DBEF_items', '');
        $form->addArgument('DBEF_items_labels', '');
        $form->setProp('onsubmit', 'return processForm()');
        // Table Title  --------------------------------------------------------
        $table = new htmlTable(CLX_WIDTH_ONE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_TITLE));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_ONE));
        $form->addContents($table->source());
        //Query Fields --------------------------------------------------------------
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell(dict('TOOL_DBQF_FIELDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_FIELDS'), CLX_CHAR_HTML_NBSP));
        $pop = new HTMLSelect($from_sel);
        $pop->setProp('multiple', true);
        $pop->setProp('size', $select_size);
        $selected = true;
        $sql = 'SELECT DISTINCT DBQF.TABLECODE, DBQF.FIELDNAME, DICT.VALUE FROM CLX_DB_EXPORTS DBEX, CLX_DB_QUERY_FIELDS DBQF, CLX_DB_FIELDS DBFL, CLX_DICTIONARY DICT WHERE DBEX.QUERYCODE = DBQF.QUERYCODE AND (DBQF.FIELDNAME = DBFL.FIELDNAME OR DBQF.FIELDNAME = CONCAT(DBFL.FIELDNAME,' . singleQuote('_WRK') . ')) AND UPPER(DICT.ENTRY) = UPPER(DBFL.FIELDFORMLABEL) AND UPPER(DICT.LANGUAGE) = ' . singleQuote(uppercase(kSystemLanguage)) . ' AND DBEX.IDCODE = ' . singleQuote($exportcode) . '  AND NOT EXISTS( SELECT DBEF.ID FROM ' . $this->tablename . ' DBEF WHERE DBEF.FIELDNAME = DBQF.FIELDNAME AND DBEF.TABLECODE = DBQF.TABLECODE AND DBEF.EXPORTCODE = DBEX.IDCODE)';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $code = $r['TABLECODE'] . '.' . $r['FIELDNAME'];
		  $pop->addItem($r['VALUE'] . ' (' . $code . ')', $code, $selected);
		  $selected = false;
        }
		ClxCommon::$db->freeResult($rs);
        $table->addCell($pop->source(), array(CLX_PROP_ALIGN => 'center'));
        //Selectors
        //Select Buttons
        $c = new htmlButton(dict('GEN_LBL_SELECT_ALL'));
        $c->setProp('onclick', 'moveSelectItemsAll(' . singleQuote($from_sel) . ', ' . singleQuote($to_sel) . ', false)');
        $buttons = $c->source() . '<br>';
        $c = new htmlButton(dict('GEN_LBL_SELECT'));
        $c->setProp('onclick', 'moveSelectItems(' . singleQuote($from_sel) . ', ' . singleQuote($to_sel) . ', false)');
        $buttons .= $c->source() . '<br>';
        $table->addCell($buttons, array());
        //Selected Fields --------------------------------------------------------------
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL_FIELDS'), CLX_CHAR_HTML_NBSP));
        $pop = new HTMLSelect($to_sel);
        $pop->setProp('onChange', 'updateFields()');
        $pop->setProp('size', $select_size);
        $pop->setProp('multiple', true);
        //add selected items here
        $sql = 'SELECT DBEF.TABLECODE, DBEF.FIELDNAME, DBEF.FIELDALIAS, DICT.VALUE FROM ' . $this->tablename . ' DBEF, CLX_DB_FIELDS DBFL, CLX_DICTIONARY DICT WHERE (DBEF.FIELDNAME = DBFL.FIELDNAME OR DBEF.FIELDNAME = CONCAT(DBFL.FIELDNAME,' . singleQuote('_WRK') . ')) AND UPPER(DICT.ENTRY) = UPPER(DBFL.FIELDFORMLABEL) AND UPPER(DICT.LANGUAGE) = ' . singleQuote(uppercase(kSystemLanguage)) . ' AND DBEF.EXPORTCODE = ' . singleQuote($exportcode) . ' ORDER BY DBEF.SEQNUM ASC';
        $rs = ClxCommon::$db->executeQuery($sql);
        $selected = true;
        $init_label = '';
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $code = $r['TABLECODE'] . '.' . $r['FIELDNAME'];
		  $pop->addItem($r['VALUE'] . ' (' . $code . ')', $code, $selected);
		  if ($selected === true) {
			 $init_label = $r['FIELDALIAS'];
			 $scripts .='selitem = ' . singleQuote($code) . ';' . CLX_CHAR_NL;
		  }
		  $scripts .= 'itemslabels[' . singleQuote($code) . '] = ' . singleQuote($r['FIELDALIAS']) . ';' . CLX_CHAR_NL;
		  $selected = false;
        }
		ClxCommon::$db->freeResult($rs);
        $table->addCell($pop->source(), array(CLX_PROP_ALIGN => 'center'));
        //Label Area
        $field = new htmlField($this->code . '_label', $init_label, CLX_NUM_60, CLX_NUM_255);
        $options .= '<br>' . dict('GEN_LBL_LABEL') . ':<br>' . $field->source() . '<br>';
        $c = new htmlButton(dict('GEN_LBL_REMOVE'));
        $c->setProp('onclick', 'moveSelectItems(' . singleQuote($to_sel) . ', ' . singleQuote($from_sel) . ', false)');
        $buttons = $c->source() . '<br>';
        $c = new htmlButton(dict('GEN_LBL_REMOVE_ALL'));
        $c->setProp('onclick', 'moveSelectItemsAll(' . singleQuote($to_sel) . ', ' . singleQuote($from_sel) . ', false)');
        $buttons .= $c->source() . '<br><br>';
        //moveSelectedItem
        $c = new htmlButton(dict('GEN_LBL_TOP'));
        $c->setProp('onclick', 'moveSelectedItem(' . singleQuote($to_sel) . ',' . singleQuote('first') . ')');
        $buttons .= $c->source() . '<br>';
        $c = new htmlButton(dict('GEN_LBL_UP'));
        $c->setProp('onclick', 'moveSelectedItem(' . singleQuote($to_sel) . ',' . singleQuote('up') . ')');
        $buttons .= $c->source() . '<br>';
        $c = new htmlButton(dict('GEN_LBL_DOWN'));
        $c->setProp('onclick', 'moveSelectedItem(' . singleQuote($to_sel) . ',' . singleQuote('down') . ')');
        $buttons .= $c->source() . '<br>';
        $c = new htmlButton(dict('GEN_LBL_BOTTOM'));
        $c->setProp('onclick', 'moveSelectedItem(' . singleQuote($to_sel) . ',' . singleQuote('last') . ')');
        $buttons .= $c->source() . '<br>';
        $table->addCell($buttons . $options, array());
        $form->addContents($table->source());
        // Controls ans Such ---------------------------------------------------
        $table = new htmlTable(CLX_WIDTH_ONE, CLX_PROP_CLASS, CLX_CLASS_FORM, CLX_PROP_WIDTH, '90%');
        $buttons = '';
        $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_ONE));
        $form->addContents($table->source());
        $contents .= $form->source();
        return $scripts1 . setJavaScriptTag($scripts, false) . $contents;
    }

    function viewFormFill($record) {
        addHistory('expfieldid', $record['DBEF_id']);
        $contents = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_DBEF_TITLE_VIEW'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_EXPORTCODE') . ':', $record['DBEF_exportcode']);
        $table->addData(dict('GEN_LBL_FIELDNAME') . ':', $record['DBEF_tablecode'] . '.' . $record['DBEF_fieldname']);
        $table->addData(dict('GEN_LBL_FIELDALIAS') . ':', $record['DBEF_fieldalias']);
        $table->addData(dict('GEN_LBL_SEQNUM') . ':', $record['DBEF_seqnum']);
        $contents .= $table->source();
        $DBET = &getTool('DBET');
        $contents .= $DBET->makepanel($record['DBEF_id']);
        return $contents;
    }

    /**
     * Creates record
     *
     * @return integer
     */
    function createRecordVoid_() {
        ClxFreeReturn($this->createRecord_());
    }

    function createRecord_() {
        return $this->saveRecord();
    }

    function saveRecord() {

        $sel_fields = explode('||', UserRequest::$post['DBEF_items']);
        $sel_lbl = explode('||', UserRequest::$post['DBEF_items_labels']);
        $sql = 'SELECT ID, TABLECODE, FIELDNAME FROM ' . $this->tablename . '  WHERE EXPORTCODE = ' . singleQuote(UserRequest::$post['DBEF_exportcode']);
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  if (!in_array($r['TABLECODE'] . '.' . $r['FIELDNAME'], $sel_fields)) {
			 ClxCommon::$db->deleteRecords('CLX_DB_EXPORT_TRANSLATIONS', 'EXPFIELDID = ' . $r['ID']);
			 ClxCommon::$db->deleteRecords($this->tablename, 'ID = ' . $r['ID']);
		  }
        }
		ClxCommon::$db->freeResult($rs);
        //Move all seqnum to prevent duplicates
        $sql = 'UPDATE ' . $this->tablename . ' SET SEQNUM = SEQNUM+100 WHERE EXPORTCODE = ' . singleQuote(UserRequest::$post['DBEF_exportcode']);
        ClxCommon::$db->executeQuery($sql);
        foreach ($sel_fields as $key => $val) {
		  $seq = $key + CLX_NUM_ONE;
		  $field = explode('.', $val);

		  $record = array('EXPORTCODE' => UserRequest::$post['DBEF_exportcode'],
			 'SEQNUM' => $seq,
			 'FIELDNAME' => $field[CLX_NUM_ONE],
			 'FIELDALIAS' => (($sel_lbl[$key] === 'undefined') ? '' : $sel_lbl[$key]),
			 'TABLECODE' => $field[CLX_NUM_ZERO]);
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, 'EXPORTCODE=' . singleQuote($record['EXPORTCODE']) . ' AND FIELDNAME=' . singleQuote($record['FIELDNAME']) . ' AND TABLECODE=' . singleQuote($record['TABLECODE']));
		  if ($qty > CLX_NUM_ZERO) {
			 $recordid = ClxCommon::$db->updateRecord($this->tablename, $record, 'EXPORTCODE=' . singleQuote($record['EXPORTCODE']) . ' AND FIELDNAME=' . singleQuote($record['FIELDNAME']) . ' AND TABLECODE=' . singleQuote($record['TABLECODE']));
		  } else {
			 $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
		  }
        }//end for
        $this->versionIncrease('ID', $recordid);
        return $recordid;
    }

    function defaultRecord() {
        $callingpage = peekHistory();
        $record = array('DBEF_exportcode' => $callingpage['exportcode']);
        return $record;
    }

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID= ' . $recordid);
        $record = array();
        $record[$this->code . '_id'] = $values['ID'];
        $record[$this->code . '_exportcode'] = $values['EXPORTCODE'];
        $record[$this->code . '_fieldname'] = $values['FIELDNAME'];
        $record[$this->code . '_tablecode'] = $values['TABLECODE'];
        $record[$this->code . '_fieldalias'] = $values['FIELDALIAS'];
        $record[$this->code . '_seqnum'] = $values['SEQNUM'];
        return $record;
    }

    function versionIncrease($fieldname, $fieldvalue) {
        $exp = ClxCommon::$db->getField($this->tablename, 'EXPORTCODE', $fieldname . '=' . singleQuote($fieldvalue));
        $DBEX = &getTool('DBEX');
        $DBEX->versionIncrease('IDCODE', $exp);
    }

}
