<?php

/**
 * Service Messages Manager
 * Webservices Service Messages Manager
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-22 15:45:22 -0400 (ma., 22 ene. 2019) $
 * @version    	$Rev: 10496 $
 *
 * CHANGE LOG
 *  Included label
 */
class sk_messages extends clx_tool {

    var $import_types;
    var $importForm;
    var $msgcode;  //Message Code
    var $errors;  //Field Message Errors array
    var $msgrecord;  //Holds message record array
    var $priormsgcode; //Prior Message Code
    var $priormsgrecord; //Prior Message Code
    var $skco_code;  //Socket Connector code
    var $tfields_only = false; //Set to true to return only Tablafields values on record
    var $remove_decimal_point = false;
    var $debug = false;

    function __construct() {
        $this->code = 'SKMS';
        $this->name = 'Service Messages Manager';
        $this->system = 'Sockets';
        $this->version = '5.5.0.0';

        $this->has_access = true;
        $this->has_package = true;

        $this->tablename = 'CLX_SK_MESSAGES';
        $this->record_name_singular = 'GEN_LBL_MESSAGE';
        $this->record_name_plural = 'GEN_LBL_MESSAGES';
        $this->primary_key = array('IDCODE');
        $this->primary_field = 'IDCODE';
        $this->secondary_field = 'IDNAME';

        //see also eb_sockets
        $this->action_types = array('NONE' => 'GEN_LBL_NOACTION',
            'SELECT' => 'GEN_LBL_SELECT',
            'CREATE' => 'GEN_LBL_CREATE',
            'REPLACE' => 'GEN_SKMS_REPLACE',
            'MODIFY' => 'GEN_LBL_MODIFY',
            'LOAD' => 'GEN_SKMS_LOAD',
            'DELETE' => 'GEN_LBL_DELETE',
            'CODE' => 'GEN_LBL_EXECUTE_CODE'
        );
        $this->message_types = array('CSV' => 'GEN_LBL_DELIMITED_FIELDS',
            'FRL' => 'GEN_LBL_FRL',
            'XML' => 'GEN_LBL_XML',
            'FPR' => 'GEN_LBL_FIELD_PAIRS'
        );
        //Ini message Record
        $this->msgrecord = array();
        $this->msgcode = '';
        $this->priormsgrecord = array();
        $this->priormsgcode = '';
        $this->skco_code = '';
        $this->errors = array();
        $this->editform = array(CLX_CLASS_TITLE => 'TOOL_EBSK_TITLE_CREATE',
            'panels' => array(
                array('type' => 'normal',
                    'entries' => array(
                        array('label' => 'GEN_LBL_CODE',
                            'fields' => array(
                                array('name' => 'idcode', 'type' => 'string', 'default' => '', 'size' => '16', 'displaysize' => '16', 'dontmodify' => '1',
                                    'validations' => array(
                                        array('type' => 'sqlobject', 'required' => '1'), array('type' => 'unique', 'fieldname' => 'idcode', 'tablename' => $this->tablename))
                                )
                            )
                        ), array('label' => 'GEN_LBL_NAME',
                            'fields' => array(
                                array('name' => 'idname', 'type' => 'string', 'default' => '', 'size' => '64', 'displaysize' => '48', 'validations' => array(
                                        array('type' => 'string', 'required' => '1'))
                                )
                            )
                        ), array('label' => 'GEN_LBL_DESCRIPTION',
                            'fields' => array(
                                array('name' => 'description', 'type' => 'text', 'default' => '', 'cols' => '48', 'rows' => '4'))
                        ), array('label' => 'GEN_LBL_VERSION',
                            'fields' => array(
                                array('name' => 'version', 'type' => 'string', 'default' => '1.0.0', 'size' => '32', 'displaysize' => '16', 'validations' => array(
                                        array('type' => 'string', 'required' => '1'))
                                )
                            )
                        )
                    )
                ), array('type' => 'normal',
                    'entries' => array(
                        array('label' => 'GEN_LBL_ACTION_TYPE',
                            'fields' => array(
                                array('name' => 'action_type', 'type' => CLX_CLASS_LIST, 'default' => 'NONE', 'options' => $this->action_types, 'validations' => array(
                                        array('type' => 'string', 'required' => '1'))
                                )
                            )
                        ),
                        array('label' => 'GEN_LBL_LIBRARY_CODE_AND_CALL',
                            'fields' => array(
                                array('name' => 'library_code', 'type' => 'lookup', 'default' => '', 'tool' => 'CODE',
                                    'validations' => array(
                                        array('type' => 'lookup', 'fieldname' => 'IDCODE', 'tablename' => 'CLX_CODELIB'))
                                ), array('name' => 'code_call', 'type' => 'text', 'default' => '', 'cols' => '48', 'rows' => '4', 'label' => 'GEN_LBL_LN_AND_CALL')
                            )
                        ), array('label' => 'GEN_LBL_HEADERMSG_CODE',
                            'fields' => array(
                                array('name' => 'headermsg_code', 'type' => 'lookup', 'default' => '', 'tool' => $this->code, 'validations' => array(
                                        array('type' => 'lookup', 'fieldname' => 'IDCODE', 'tablename' => $this->tablename))
                                )
                            )
                        ), array('label' => 'GEN_LBL_MSG_TYPE',
                            'fields' => array(
                                array('name' => 'msg_type', 'default' => 'XML', 'type' => 'dynamic', 'panels' => $this->message_types)
                            )
                        )
                    )
                ), array('type' => 'dynamic',
                    'code' => 'XML'
                ), array(
                    'type' => 'dynamic',
                    'code' => 'CSV',
                    'entries' => array(
                        array('label' => 'GEN_LBL_FIELD_DELIM',
                            'fields' => array(
                                array('name' => 'field_delim', 'type' => 'radiobutton', 'value' => '[9]', CLX_CLASS_PROMPT => 'GEN_LBL_TAB'), array('name' => 'field_delim', 'type' => 'radiobutton', 'value' => ',', CLX_CLASS_PROMPT => 'GEN_LBL_COMMA'), array('name' => 'field_delim', 'type' => 'radiobutton', 'value' => ';', CLX_CLASS_PROMPT => 'GEN_LBL_SEMICOLON'),
                                array('name' => 'field_delim', 'type' => 'radiobutton', 'value' => '[32]', CLX_CLASS_PROMPT => 'GEN_LBL_SPACE'), array('name' => 'field_delim', 'type' => 'radiobutton', 'value' => 'other', CLX_CLASS_PROMPT => 'GEN_LBL_OTHER'), array('name' => 'field_delim_other', 'type' => 'string', 'default' => '', 'size' => '24', 'displaysize' => '24'))
                        ), array('label' => 'GEN_LBL_RECORD_DELIM',
                            'fields' => array(
                                array('name' => 'record_delim', 'type' => 'radiobutton', 'value' => '[13]', CLX_CLASS_PROMPT => 'GEN_LBL_CARRIAGE_RETURN'), array('name' => 'record_delim', 'type' => 'radiobutton', 'value' => '[10]', CLX_CLASS_PROMPT => 'GEN_LBL_LINE_FEED'), array('name' => 'record_delim', 'type' => 'radiobutton', 'value' => '[13][10]', CLX_CLASS_PROMPT => 'GEN_LBL_CRLF'), array('name' => 'record_delim', 'type' => 'radiobutton', 'value' => 'other', CLX_CLASS_PROMPT => 'GEN_LBL_OTHER'), array('name' => 'record_delim_other', 'type' => 'string', 'default' => '', 'size' => '24', 'displaysize' => '24'))
                        )
                    )
                ), array(
                    'type' => 'dynamic',
                    'code' => 'FRL',
                    'entries' => array(
                        array('label' => 'GEN_LBL_HEADER_SIZE',
                            'fields' => array(
                                array('name' => 'header_size', 'type' => 'string', 'default' => '', 'size' => '64', 'readonly' => true, 'displaysize' => '48')
                            )
                        ), array('label' => 'GEN_LBL_RECORD_LENGTH',
                            'fields' => array(
                                array('name' => 'record_length', 'type' => 'string', 'default' => '', 'readonly' => true, 'size' => '32', 'displaysize' => '16',
                                )
                            )
                        )
                    )
                )
            )
        );
        $this->viewform = $this->editform;
		//$GLOBALS['ADDRDEBUGSK']=0;
		if (isset($GLOBALS['ADDRDEBUGSK'])){
			$this->debug=true;
		}
    }

    function install() {
        $UIMI = getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_SERVCONNECTOR:MENU_SKMESSAGES', 'TOOL', $this->code, 'ac=LIST');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['IDCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['VERSION'] = array('type' => 'varchar(32)', 'null' => 'NO', 'properties' => '');
        $fields['VERSION_COUNTER'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['MDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['DESCRIPTION'] = array('type' => 'mediumtext', 'properties' => '');
        $fields['MSG_TYPE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['ACTION_TYPE'] = array('type' => 'varchar(10)', 'null' => 'YES', 'properties' => '');
        $fields['FIELD_DELIM'] = array('type' => 'varchar(16)', 'null' => 'YES', 'properties' => '');
        $fields['RECORD_DELIM'] = array('type' => 'varchar(16)', 'null' => 'YES', 'properties' => '');
        $fields['HEADER_SIZE'] = array('type' => 'varchar(16)', 'null' => 'YES', 'properties' => '');
        $fields['RECORD_LENGTH'] = array('type' => 'varchar(16)', 'null' => 'YES', 'properties' => '');
        $fields['HEADERMSG_CODE'] = array('type' => 'varchar(16)', 'null' => 'YES', 'properties' => '');
        $fields['PROCESS_CODE'] = array('type' => 'varchar(16)', 'properties' => '');
        $fields['LIBRARY_CODE'] = array('type' => 'varchar(16)', 'properties' => '');
        $fields['CODE_CALL'] = array('type' => 'mediumtext', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'IDCODE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'IDNAME');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'MSG_TYPE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
            'fields' => $fields,
            'indexes' => $indexes);
    }

//-- M E S A G E S
    function pack($record) {
        $o = $this->packRecord($record);
        $objs = new XMLObject($this->code . '_OBJECTS');
        $this->packsub($objs, 'SKME', 'SELECT * FROM CLX_SK_ELEMENTS WHERE MSG_CODE =' . singleQuote($record['IDCODE']));
        $this->packsub($objs, 'SKMP', 'SELECT * FROM CLX_SK_MAPPINGS WHERE MSG_CODE =' . singleQuote($record['IDCODE']));
        $o->addObject($objs);
        return $o;
    }

    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        $exists = (ClxCommon::$db->oneExistsRecords($this->tablename, 'UPPER(IDCODE)=' . singleQuote(uppercase($record['IDCODE']))) > CLX_NUM_ZERO);
        if ($exists === true) {
            ClxCommon::$db->deleteRecords($this->tablename, 'UPPER(IDCODE)=' . singleQuote(uppercase($record['IDCODE'])));
        }
        $id = ClxCommon::$db->createRecord($this->tablename, $record, false);
        if ($package->hasSubObject($this->code . '_OBJECTS')) {
            $objs = $package->getObjectXml($this->code . '_OBJECTS');
            foreach ($objs->objects as $o) {
                $toolcode = $o->tag;
                $tool = getTool($toolcode);
                $tool->unpack(array('package' => $o, 'id' => $id));
            }
        }
    }

    function action($ac, $idlist) {//pre($ac);
		IF (ISSET($GLOBALS['ADDRDEBUGSK']) && $GLOBALS['ADDRDEBUGSK']==$_SERVER['REMOTE_ADDR']) {$pos=0;$te=debug_backtrace();$posx=0;	for ($t=0;$t<12;$t++) {	if (isset($te[$t]["file"])){if ($t==$posx)echo $pos."_".$te[$posx]["function"]."_util(pre)-hora:".date("h:i:s d/m/Y")."<br>";  echo substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"]."<br>";
		}}}

        if ($this->getAccess()->canRead()) {
            $contents = ($ac == 'CHECK') ? $this->check() : parent::action($ac, $idlist);
        }
		else if ($ac == 'SCHECK') {
			
		} else {
            $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [SK_MESSAGES-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makelist($listingcode = null) {
         //$t = time('U'); genera un warning
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'SKMSList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = $this->tablename;
        $whereclause = '';
        $sortclause = '';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $table = new htmlTable(CLX_WIDTH_7);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_EBSK_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_7));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_CODE'), dict('GEN_LBL_NAME'), dict('GEN_LBL_MSG_TYPE'), dict('GEN_LBL_VERSION'), dict('GEN_LBL_DESCRIPTION'), dict('GEN_LBL_ACTION_TYPE')));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
            $recordid = $record['ID'];
            $select = new htmlCheckbox('selector_' . $recordid, false);
            $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
            $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
            $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array(
                $record['IDCODE'], $record['IDNAME'], dict($this->message_types[$record['MSG_TYPE']]), $record['VERSION'] . '-' . $record['VERSION_COUNTER'], xFormat::text($record['DESCRIPTION']),
                dict($this->action_types[$record['ACTION_TYPE']])));
            $i++;
        }
        $buttons = '';
        $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        if ($this->getAccess()->canModify()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
		if ($this->getAccess()->canDisplay()) {
			//$form_id=toString(getIdlist());
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DUPLICATE'), array('action' => 'DUPLICATE', 'form' =>  $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ONE'))->source();
        }
        if ($this->getAccess()->canDelete()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDisplay()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_SKMS_CREATEMSG')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    function searchForm() {
        $contents = '';
        $msg = array();
        // Search Form
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'SEARCH');
        $form->addArgument('confirm', '1');
        $form->addArgument(CLX_PROP_TARGET, UserRequest::$request[CLX_PROP_TARGET]);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_SKMS_TITLE_SEARCH'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'SKMS_idcode', (array_key_exists('SKMS_idcode', UserRequest::$post)) ? UserRequest::$post['SKMS_idcode'] : '', CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'SKMS_idname', (array_key_exists('SKMS_idname', UserRequest::$post)) ? UserRequest::$post['SKMS_idname'] : '', CLX_NUM_32, CLX_NUM_64);
        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents .= $form->source();
        // Search Procedure
        if (isset(UserRequest::$post['confirm'])) {
            $table = new htmlTable(CLX_WIDTH_TWO);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $criteria = '';
            if (UserRequest::$post['SKMS_idcode'] <> '') {
                $criteria .= 'UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['SKMS_idcode']) . '%');
            }
            if (UserRequest::$post['SKMS_idname'] <> '') {
                $criteria .= 'UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['SKMS_idname'])) . '%');
            }
            $qty = ClxCommon::$db->oneExistsRecordsS($this->tablename, $criteria);
            if ($qty == CLX_NUM_ZERO) {
                $table->addCell(dict('TOOL_GEN_ERR_NOMATCH'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
            } else {
                $table->addCell(dict('TOOL_GEN_MSG_FOUNDRECORDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
                $records = ClxCommon::$db->getRecords($this->tablename, $criteria);
                $i = CLX_INIT_ZERO;
                while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
                    $js = 'javascript:setLookupValue(' . quote(UserRequest::$request[CLX_PROP_TARGET]) . ', ' . quote($record['IDCODE']) . ');';
                    $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<paget=sk_messages>map_fields</page><a href=' . singleQuote($js) . '>' . $record['IDCODE'] . '</a>', $record['IDNAME']));
                    $i++;
                }
				ClxCommon::$db->freeResult( $records);
            }
            $contents .= $table->source();
        }
        return $contents;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['SKMS_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['SKMS_idname']);
    }

    function viewFormFill($record) {
        addHistory('msg_code', $record['SKMS_idcode']);
        $contents = '';
        // Elements
        $SKME = &getTool('SKME');
        $contents .= $SKME->makepanel($record['SKMS_idcode']);
        $contents .= '<br>';
        $SKMP = &getTool('SKMP');
        $contents .= $SKMP->makepanel($record['SKMS_idcode']);
        return $contents;
    }

    function checkForm(&$msg) {
        $msg = array();
        parent::checkForm($msg);
        if (UserRequest::$post['SKMS_action_type'] === 'PROC' && ClxCommon::$db->oneExistsRecords('PROC_PROCESSES', 'IDCODE = ' . singleQuote(UserRequest::$post['SKMS_process_code'])) == CLX_NUM_ZERO) {
            $msg['SKMS_process_code'] = dict('GEN_LBL_FIELDREQUIRED');
        }
        if (UserRequest::$post['SKMS_action_type'] === 'CODE' && (ClxCommon::$db->oneExistsRecords('CLX_CODELIB', 'IDCODE = ' . singleQuote(UserRequest::$post['SKMS_library_code'])) == CLX_NUM_ZERO || empty(UserRequest::$post['SKMS_code_call']))) {
            $msg['SKMS_library_code'] = dict('GEN_LBL_FIELDREQUIRED');
            $msg['SKMS_code_call'] = dict('GEN_LBL_FIELDREQUIRED');
        }
        return count($msg) == CLX_NUM_ZERO;
    }

    function filterFormFill($msg, &$table) {
        if (!array_key_exists('SKMS_idcode', UserRequest::$post))
            UserRequest::$post['SKMS_idcode'] = '';
        if (!array_key_exists('SKMS_idname', UserRequest::$post))
            UserRequest::$post['SKMS_idname'] = '';
        $table->addField($msg, dict('GEN_LBL_CODE'), 'SKMS_idcode', UserRequest::$post['SKMS_idcode'], CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_NAME'), 'SKMS_idname', UserRequest::$post['SKMS_idname'], CLX_NUM_32, CLX_NUM_64);
        $sort = new HtmlSelect('SKMS_sort');
        $sort->addItem('', '', true);
        $sort->addItem(dict('GEN_LBL_CODE'), 'idcode', false);
        $sort->addItem(dict('GEN_LBL_NAME'), 'idname', false);
        $table->addData(dict('GEN_LBL_ORDERBY') . ':', $sort->source());
    }
	/**
     * Build tool display form
     * @param $idlist
     * @return string
     */
    function viewForm_($idlist) {
        $contents = '';
        $buttons = '';
        $id = $idlist[count($idlist) - CLX_NUM_ONE];
        $record = $this->loadToolRecord($id);
        pushHistory($this->code, 'DISPLAY', $idlist);
        // Buttons Form
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, CLX_CLASS_FOOTERS);
        $form->addArgument('ac', 'NOT_DEFINED');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('idlist', toString($idlist));
        $form->addArgument('next', CLX_NUM_ONE);
        $form->addArgument('relfm', UserRequest::$request['relfm']);

        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));


        $c = new htmlButtonReturn();
        $buttons .= $c->source();
        if (count($idlist) > CLX_NUM_ONE) {
            $c = new htmlFormSubmit(dict('GEN_LBL_VIEWNEXT_SOMETHING', dict('GEN_LBL_RECORD')), 'DISPLAY', $form->getProp(CLX_PROP_ID));
            $buttons .= $c->source();
        }
        $buttons .= $this->viewFormButtons($form);
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $buttons_form = $form->source();
        // View Form -----------------------------------------------------------
        if (!clx_is_empty($this->viewform)) {
            $form = new clx_form($this->code, $this->viewform);
            $record = $this->loadToolRecord($id);
            $contents .= $form->viewForm_($record);
            $contents .= $buttons_form;
            $contents .= $this->viewFormFill($record);
        } else {
            $contents .= $this->viewFormFill($record);
            $contents .= $buttons_form;
        }
        return  $contents;
    }


    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['SKMS_idcode'])) {
            $filter .= $boolean . ' UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['SKMS_idcode']) . '%');
            $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['SKMS_idname'])) {
            $filter .= $boolean . ' UPPER(IDNAME) LIKE ' . singleQuote(clx_change_especial_char_html (uppercase(UserRequest::$post['SKMS_idname'])) . '%');
            $boolean = 'AND';
        }
        $_SESSION['paging']['SKMSList']['sortby'] = UserRequest::$post['SKMS_sort'];
        $_SESSION['paging']['SKMSList']['criteria'] = $filter;
        $_SESSION['paging']['SKMSList']['startat'] = '0';
    }
	///////////////////////////////////////////////////
	///comentado 22/01/2019 para utilizar la clx_tool
	//////////////////////////////////////////////////
    /*function clearfilter() {
        $_SESSION['paging']['SKMSList']['criteria'] = '';
    }*/

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID= ' . $recordid);
        $record = array();
        $record['SKMS_id'] = $recordid;
        $record['SKMS_idcode'] = $values['IDCODE'];
        $record['SKMS_idname'] = $values['IDNAME'];
        $record['SKMS_version'] = $values['VERSION'];
        $record['SKMS_version_counter'] = $values['VERSION_COUNTER'];
        $record['SKMS_cdate'] = xFormat::date($values['CDATE']);
        $record['SKMS_mdate'] = xFormat::date($values['MDATE']);
        $record['SKMS_description'] = $values['DESCRIPTION'];
        $record['SKMS_header_size'] = $values['HEADER_SIZE'];
        $record['SKMS_headermsg_code'] = $values['HEADERMSG_CODE'];
        $record['SKMS_msg_type'] = $values['MSG_TYPE'];
        $record['SKMS_field_delim'] = $values['FIELD_DELIM'];
        $record['SKMS_record_delim'] = $values['RECORD_DELIM'];
        $record['SKMS_field_delim_other'] = $values['FIELD_DELIM'];
        $record['SKMS_record_delim_other'] = $values['RECORD_DELIM'];
        $record['SKMS_record_length'] = $values['RECORD_LENGTH'];
        $record['SKMS_action_type'] = $values['ACTION_TYPE'];
        $record['SKMS_process_code'] = $values['PROCESS_CODE'];
        $record['SKMS_library_code'] = $values['LIBRARY_CODE'];
        $record['SKMS_code_call'] = $values['CODE_CALL'];
        return $record;
    }

    function saveRecord() {
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['IDNAME'] = UserRequest::$post['SKMS_idname'];
        $record['VERSION'] = UserRequest::$post['SKMS_version'];
        $record['MDATE'] = $d;
        $record['DESCRIPTION'] = UserRequest::$post['SKMS_description'];
        $record['HEADERMSG_CODE'] = UserRequest::$post['SKMS_headermsg_code'];
        $record['ACTION_TYPE'] = UserRequest::$post['SKMS_action_type'];
        $record['LIBRARY_CODE'] = UserRequest::$post['SKMS_library_code'];
        $record['CODE_CALL'] = UserRequest::$post['SKMS_code_call'];
        $record['MSG_TYPE'] = UserRequest::$post['SKMS_msg_type'];
        if ($record['MSG_TYPE'] === 'CSV') {
            if (UserRequest::$post['SKMS_field_delim'] === 'other') {
                $record['FIELD_DELIM'] = UserRequest::$post['SKMS_field_delim_other'];
            } else {
                $record['FIELD_DELIM'] = UserRequest::$post['SKMS_field_delim'];
            }
            if (UserRequest::$post['SKMS_record_delim'] === 'other') {
                $record['RECORD_DELIM'] = UserRequest::$post['SKMS_record_delim_other'];
            } else {
                $record['RECORD_DELIM'] = UserRequest::$post['SKMS_record_delim'];
            }
        } elseif ($record['MSG_TYPE'] === 'FRL') {
            $record['RECORD_LENGTH'] = UserRequest::$post['SKMS_record_length'];
        } elseif ($record['MSG_TYPE'] === 'XML') {
            $record['HEADER_SIZE'] = null;
            $record['RECORD_LENGTH'] = null;
        }
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['SKMS_id']);
        $this->versionIncrease('ID', UserRequest::$post['SKMS_id']);
    }

    function createRecord_() {
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['IDCODE'] = UserRequest::$post['SKMS_idcode'];
        $record['IDNAME'] = UserRequest::$post['SKMS_idname'];
        $record['VERSION'] = UserRequest::$post['SKMS_version'];
        $record['VERSION_COUNTER'] = '0';
        $record['CDATE'] = $d;
        $record['MDATE'] = $d;
        $record['DESCRIPTION'] = UserRequest::$post['SKMS_description'];
        $record['HEADERMSG_CODE'] = UserRequest::$post['SKMS_headermsg_code'];
        $record['ACTION_TYPE'] = UserRequest::$post['SKMS_action_type'];
        $record['LIBRARY_CODE'] = UserRequest::$post['SKMS_library_code'];
        $record['CODE_CALL'] = UserRequest::$post['SKMS_code_call'];
        $record['MSG_TYPE'] = UserRequest::$post['SKMS_msg_type'];
        if ($record['MSG_TYPE'] === 'CSV') {
            if (UserRequest::$post['SKMS_field_delim'] === 'other') {
                $record['FIELD_DELIM'] = UserRequest::$post['SKMS_field_delim_other'];
            } else {
                $record['FIELD_DELIM'] = UserRequest::$post['SKMS_field_delim'];
            }
            if (UserRequest::$post['SKMS_record_delim'] === 'other') {
                $record['RECORD_DELIM'] = UserRequest::$post['SKMS_record_delim_other'];
            } else {
                $record['RECORD_DELIM'] = UserRequest::$post['SKMS_record_delim'];
            }
        } elseif ($record['MSG_TYPE'] === 'FRL') {
            $record['HEADER_SIZE'] = UserRequest::$post['SKMS_header_size'];
            $record['RECORD_LENGTH'] = UserRequest::$post['SKMS_record_length'] + UserRequest::$post['SKMS_header_size'];
        } elseif ($record['MSG_TYPE'] === 'XML') {
            $record['HEADER_SIZE'] = null;
            $record['RECORD_LENGTH'] = null;
        } else {
            $record['HEADER_SIZE'] = null;
            $record['RECORD_LENGTH'] = null;
        }
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $this->versionIncrease('IDCODE', $record['IDCODE']);
        //Creating Auto first Map for Header
        if ($record['MSG_TYPE'] <> 'XML' && $record['HEADER_SIZE'] > CLX_NUM_ZERO) {
            $rec = array();
            $rec['FILE_FIELDNAME'] = 'HEADER';
            $rec['type'] = 'CALC';
            $rec['DATA_TYPE'] = 'CHAR';
            $rec['FORMULA'] = ClxCommon::$db->addslashes(singleQuote($record['IDCODE']));
            $rec['START_POSITION'] = CLX_INIT_ZERO;
            $rec['FIELD_LENGTH'] = $record['HEADER_SIZE'];
            $rec['EBSOCKETCODE'] = $record['IDCODE'];
            $rec['SEQNUM'] = CLX_INIT_ZERO;
            $rec['description'] = 'General Header.';
            ClxCommon::$db->createRecord('CLX_EB_SOCKET_MAPPINGS', $rec, false);
        }
        return $recordid;
    }

    function deleteRecordSk($id) {
        $record = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        $idcode = $record['IDCODE'];
        $SKME = getTool('SKME');
        $elements = ClxCommon::$db->getRecords($SKME->tablename, 'MSG_CODE=' . singleQuote($idcode));
        while (($element = ClxCommon::$db->getNextRecord($elements))) {
            $SKME->deleteRecordSk($element['ID']);
        }
		ClxCommon::$db->freeResult( $elements );
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        $SKSV = &getTool('SKSV');
        $SKSV->versionIncrease('REQUEST_CODE', $record['IDCODE']);
        $SKSV->versionIncrease('RESPONSE_CODE', $record['IDCODE']);
    }

    function defaultRecord() {
        $record = array();
        $record['SKMS_id'] = CLX_INIT_ZERO;
        $record['SKMS_idcode'] = '';
        $record['SKMS_idname'] = '';
        $record['SKMS_version'] = '1.0.0';
        $record['SKMS_cdate'] = '';
        $record['SKMS_mdate'] = '';
        $record['SKMS_description'] = '';
        $record['SKMS_header_size'] = '';
        $record['SKMS_headermsg_code'] = '';
        $record['SKMS_msg_type'] = 'XML';
        $record['SKMS_field_delim'] = '[9]';
        $record['SKMS_record_delim'] = '[13][10]';
        $record['SKMS_record_length'] = '';
        $record['SKMS_action_type'] = 'NONE';
        $record['SKMS_process_code'] = '';
        $record['SKMS_library_code'] = '';
        $record['SKMS_code_call'] = '';
        return $record;
    }

    function versionIncrease($fieldname, $fieldvalue) {
        parent::versionIncrease($fieldname, $fieldvalue);
        $message_code = ClxCommon::$db->getField($this->tablename, 'IDCODE', $fieldname . '=' . singleQuote($fieldvalue));
        $SKSV = &getTool('SKSV');
        $SKSV->versionIncrease('IDCODE', $message_code);
    }

    function saveErrors($ebsocketcode, $linenum, $msg, &$saved_errors) {
        $record = array();
        $record['EBSOCKETCODE'] = $ebsocketcode;
        $record['LINENUM'] = $linenum;
        $record['ERRORNUM'] = '0';
        foreach ($msg as $fieldname => $error) {
            $record['FIELDNAME'] = $fieldname;
            $record['ERRORMSG'] = addslashes($error);
            ClxCommon::$db->createRecord('CLX_EB_SOCKET_ERRORS', $record, false);
            $saved_errors++;
        }
    }

    function getMappings($ebsocketcode) {
        $mappings = array();
        $records = ClxCommon::$db->getRecords('CLX_EB_SOCKET_MAPPINGS', 'EBSOCKETCODE = ' . singleQuote($ebsocketcode) . ' ORDER BY SEQNUM');
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
            $mappings[$record['ID']] = $record;
        }
		ClxCommon::$db->freeResult( $records);
        return $mappings;
    }

    function getExceptions($ebsocketcode) {
        $exceptions = array();
        $records = ClxCommon::$db->getRecords('CLX_EB_SOCKET_EXCEPTIONS', 'EBSOCKETCODE = ' . singleQuote($ebsocketcode) . ' ORDER BY SEQNUM');
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
            $exceptions[$record['TABLE_FIELDNAME']] = $record;
        }
		ClxCommon::$db->freeResult( $records);
        return $exceptions;
    }

    function getTranslations($ebsocketcode, $mappings) {
        $translations = array();
        foreach ($mappings as $fieldname => $info) {
            $mapid = $info['ID'];
            $list = array();
            $records = ClxCommon::$db->getRecords('CLX_EB_SOCKET_TRANSLATIONS', 'MAPID = ' . $mapid);
            while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
                $list[$record['ORIGINAL_VALUE']] = $record;
            }
			ClxCommon::$db->freeResult( $records );
            if (!empty($list))
                $translations[$mapid] = $list;
        }
        return $translations;
    }

    function translate(&$field_value, $translations) {
        if (array_key_exists($field_value, $translations)) {
            $info = $translations[$field_value];
            if ($info['case_sensitive'] === true) {
                if (strcmp($field_value, $info['ORIGINAL_VALUE']) == CLX_NUM_ZERO) {
                    $field_value = $info['REPLACE_VALUE'];
                }
            } else {
                $field_value = $info['REPLACE_VALUE'];
            }
        }
    }

    function executeFormula($maprecord, $line, $buffer) {
        $cal = new ClxExpression();
        return $cal->parseRecordFormula($maprecord['FORMULA']);
    }

    function isException($exceptions, $buffer) {
        $is = false;
        foreach ($exceptions as $exception) {
            switch ($exception['OPERATOR']) {
                case 'EQ':
                    $is = ($is or ( substr($buffer, $exception['POSITION'], strlen($exception['value'])) == $exception['value']));
                    break;
                case 'NQ':
                    $is = ($is or ( substr($buffer, $exception['POSITION'], strlen($exception['value'])) != $exception['value']));
                    break;
                case 'LS':
                    $is = ($is or ( substr($buffer, $exception['POSITION'], strlen($exception['value'])) < $exception['value']));
                    break;
                case 'LQ':
                    $is = ($is or ( substr($buffer, $exception['POSITION'], strlen($exception['value'])) <= $exception['value']));
                    break;
                case 'GQ':
                    $is = ($is or ( substr($buffer, $exception['POSITION'], strlen($exception['value'])) >= $exception['value']));
                    break;
                case 'GT':
                    $is = ($is or ( substr($buffer, $exception['POSITION'], strlen($exception['value'])) > $exception['value']));
                    break;
                default:
                    break;
            }
        }
        return $is;
    }

    function getLength($idcode) {
        $SKME = &getTool('SKME');
        $sql = 'SELECT SUM(ELE_LENGTH) LENGTH FROM ' . $SKME->tablename . ' WHERE MSG_CODE =' . singleQuote($idcode);
        return ClxCommon::$db->getOne($sql);
    }

    function saveLength($idcode) {
        $a = array();
        $a['RECORD_LENGTH'] = $this->getLength($idcode);
        ClxCommon::$db->updateRecord($this->tablename, $a, 'IDCODE = ' . singleQuote($idcode));
    }

    /**
     * Load Message
     * Fill message record with all values from record.
     */
    function createMessage($message_code, $record) {
        $this->msgrecord = array();
        $this->msgcode = $message_code;
        //Priparing Header Message, basically message recieve prior to actual message
        $msgrec = ClxCommon::$db->getRecord($this->tablename, 'IDCODE =' . singleQuote($message_code));
        $this->skco_code = $msgrec['SKCONNECTOR_CODE']; //Set socket connector information
        //Check if exists prior message
        if ($msgrec['HEADERMSG_CODE'] <> '') {
            $a = new sk_messages();
            $a->createMessage($msgrec['HEADERMSG_CODE'], array());
            $this->priormsgcode = $a->msgcode;
            $this->priormsgrecord = $a->msgrecord;
            unset($a);
        } else {
            $this->priormsgcode = '';
            $this->priormsgrecord = array();
        }
        $mappings = $this->getMappings($message_code);
        $translations = $this->getTranslations($message_code, $mappings);
        //$fields = ClxCommon::$db->getRecords('CLX_EB_SOCKET_MAPPINGS', 'EBSOCKETCODE ='' . $message_code).' ORDER BY SEQNUM ASC');
        foreach ($mappings as $key => $field) {
            $val = '';
            if (!empty($field['TABLE_FIELDNAME']) && array_key_exists($field['TABLE_FIELDNAME'], $record)) {
                $val = $record[$field['TABLE_FIELDNAME']];
            }
            //Check is formula
            if ($field['TYPE'] === 'CALC') {
                $val = $this->executeFormula($field, CLX_NUM_ZERO, $val);
            }
            if ($field['DATA_TYPE'] === 'DECN') {
                //Validate number of decimals on string
                if (!empty($field['FIELD_DECIMALS']) > CLX_NUM_ZERO && strlen($val) > CLX_NUM_ZERO) {
                    //has decimal?
                    if (strpos($val, '.') === false) {
                        /**
                         * TOOL_SKMS_ERR_NODECIMAL
                         * Decimal value required on ^1
                         * Valor decimal requerido on ^1
                         */
                        $this->errors[$field['FILE_FIELDNAME']] = dict('TOOL_SKMS_ERR_NODECIMAL', $field['FILE_FIELDNAME']);
                        if (!empty($this->debug))
                            echoLog($this->errors[$field['FILE_FIELDNAME']]);
                    }elseif ($field['FIELD_DECIMALS'] < strlen(substr($val, strpos($val, '.') + CLX_NUM_ONE, strlen($val) - strpos($val, '.')))) {
                        /**
                         * TOOL_SKMS_ERR_DECIMALTOLONG
                         * Decimal positions grater than maximun allowed for field for message.
                         * Número de decimales es mayor que el requerido por el campo en la trama.
                         */
                        $this->errors[$field['FILE_FIELDNAME']] = dict('TOOL_SKMS_ERR_DECIMALTOLONG');
                        if (!empty($this->debug))
                            echoLog($this->errors[$field['FILE_FIELDNAME']]);
                    }
                }
                //Decimals must be an integer and should go without comma or dots or simbols
                if ($this->remove_decimal_point) {
                    $val = str_replace('.', '', $val);
                    $val = str_replace(',', '', $val);
                    $val = str_replace('-', '', $val);
                }
            }
            //Handle Transalations
            if (array_key_exists($key, $translations))
                $this->translate($val, $translations[$key]);
            if (strlen($val) > $field['FIELD_LENGTH']) {
                /**
                 * TOOL_SKMS_ERR_DATATOLONG
                 * Field ^1 smaller than data length. value: ^2, Required Size: ^3 String size: ^4';
                 * Campo ^1 mas pequeño que el tamaño del campo. Valor: ^2, Tamaño Requirido: ^3 Tamaño de Valor: ^4';
                 */
                $this->errors[$field['FILE_FIELDNAME']] = dict('TOOL_SKMS_ERR_DATATOLONG', $field['FILE_FIELDNAME'] . '->' . $field['TABLE_FIELDNAME'], $val, $field['FIELD_LENGTH'], strlen($val));
                if (!empty($this->debug))
                    eechoLog($this->errors[$field['FILE_FIELDNAME']]);
            }
            $this->msgrecord[] = array('FILEFIELD' => $field['FILE_FIELDNAME'], 'FIELD' => $field['TABLE_FIELDNAME'],
                'TYPE' => $field['DATA_TYPE'],
                'SIZE' => $field['FIELD_LENGTH'],
                'START' => $field['START_POSITION'],
                'DECIMALS' => $field['FIELD_DECIMALS'],
                'VAL' => $val);
        }
        return $this->msgrecord;
    }

    /**
     * Reads a Message
     * Fill message record with all values from flat file line.
     */
    function readMessage($message_code, $s_ffile_line) {
        $this->msgrecord = array();
        $this->msgcode = $message_code;
        //Priparing Header Message, basically message recieve prior to actual message
        $msgrec = ClxCommon::$db->getRecord($this->tablename, 'IDCODE =' . singleQuote($message_code));
        $this->skco_code = $msgrec['SKCONNECTOR_CODE']; //Set socket connector information
        //Check if exists prior message
        if ($msgrec['HEADERMSG_CODE'] <> '') {
            $a = new sk_messages();
            $a->createMessage($msgrec['HEADERMSG_CODE'], array());
            $this->priormsgcode = $a->msgcode;
            $this->priormsgrecord = $a->msgrecord;
            unset($a);
        } else {
            $this->priormsgcode = '';
            $this->priormsgrecord = array();
        }
        $mappings = $this->getMappings($message_code);
        $translations = $this->getTranslations($message_code, $mappings);
        //$fields = ClxCommon::$db->getRecords('CLX_EB_SOCKET_MAPPINGS', 'EBSOCKETCODE ='' . $message_code).' ORDER BY SEQNUM ASC');
        $i_record_rd_pointer = CLX_INIT_ZERO;
        foreach ($mappings as $key => $field) {
            // Read field from pointer up to the specified lenght
            $val = substr($s_ffile_line, $i_record_rd_pointer, $field['FIELD_LENGTH']);
            // Check if is it a formula
            if ($field['TYPE'] === 'CALC') {
                $val = $this->executeFormula($field, CLX_NUM_ZERO, $val);
            }
            if ($field['DATA_TYPE'] === 'DECN') {
                //Validate number of decimals on string
                if (!empty($field['FIELD_DECIMALS']) > CLX_NUM_ZERO) {
                    //has decimal?
                    if (strpos($val, '.') === false) {
                        // No decimal point, so add to at the specified number of decimal positions
                        $i_integert_part_len = $field['FIELD_LENGTH'] - $field['FIELD_DECIMALS'];
                        $val = substr($val, CLX_NUM_ZERO, $i_integert_part_len) . '.' . substr($val, $i_integert_part_len, $field['FIELD_DECIMALS']);
                    } elseif ($field['FIELD_DECIMALS'] < strlen(substr($val, strpos($val, '.') + CLX_NUM_ONE, strlen($val) - strpos($val, '.')))) {
                        /**
                         * TOOL_SKMS_ERR_DECIMALTOLONG
                         * Decimal positions grater than maximun allowed for field for message.
                         * Número de decimales es mayor que el requerido por el campo en la trama.
                         */
                        $this->errors[$field['FILE_FIELDNAME']] = dict('TOOL_SKMS_ERR_DECIMALTOLONG');
                        if (!empty($this->debug))
                            echoLog($this->errors[$field['FILE_FIELDNAME']]);
                    }
                }
            }
            //Handle Transalations
            if (array_key_exists($key, $translations))
                $this->translate($val, $translations[$key]);
            $this->msgrecord[] = array('FILEFIELD' => $field['FILE_FIELDNAME'],
                'TABLE' => $field['TABLEFULLNAME'],
                'FIELD' => $field['TABLE_FIELDNAME'],
                'TYPE' => $field['DATA_TYPE'],
                'SIZE' => $field['FIELD_LENGTH'],
                'DECIMALS' => $field['FIELD_DECIMALS'],
                'START' => $field['START_POSITION'],
                'POINTER' => $i_record_rd_pointer,
                'VAL' => $val
            );
            // Pointer increment
            $i_record_rd_pointer += $field['FIELD_LENGTH'];
        }
        return $this->msgrecord;
    }

    /**
     * Converts Array Message into string
     */
    function toString() {
        $return = '';
        foreach ($this->msgrecord as $key => $val) {
            $s = '';
            if ($val['VAL'] <> '') {
                $s = $val['VAL'];
                $val['SIZE'] -=strlen($val['VAL']);
            }
            if ($val['TYPE'] === 'CHAR')
                $s .= str_repeat(' ', $val['SIZE']);
            else
                $s = str_repeat('0', $val['SIZE']) . $s;
            //Adding to return
            $return .= $s;
        }
        return $return;
    }

    /**
     * bool Return true if error has being found on message
     */
    function hasErrors() {
        return empty($this->errors);
    }

    function recieveMessage($in) {
        if (!empty($this->debug))
            echoLog('Response: ->' . $in . '<- <br>');
        $return = array();
        $length = strlen($in);
        //Check if prior message if not collect them and continue Usually use to return error messages
        if ($this->priormsgcode <> '' && $length > CLX_NUM_ZERO) {
            if ($length > CLX_NUM_ZERO) {
                if (!empty($this->debug))
                    echoLog('-Process Prior: ' . $this->priormsgcode . '<br>');
                $in = substr($in, CLX_NUM_TWO, strlen($in));
                $done = CLX_INIT_ZERO;
                $record = array();
                foreach ($this->priormsgrecord as $key => $val) {
                    $done += $val['SIZE'];
                    $value = trim(substr($in, $val['START'], $val['SIZE']));
                    switch ($val['TYPE']) {
                        case 'DECN':
                            if ($val['DECIMALS'] > CLX_NUM_ZERO) {//Set decimal positions
                                $int = substr($value, CLX_NUM_ZERO, (strlen($value) - $val['DECIMALS']));
                                $decn = substr($value, -$val['DECIMALS']);
                                $value = (float) ($int . '.' . $decn);
                            }
                            break;
                        default:
                            break;
                    }//End switch
                    if (strlen($value) == CLX_NUM_ZERO)
                        continue;
                    $record[$val['FILEFIELD']] = $value;
                    if ($val['FIELD'] <> '')
                        $record[$val['FIELD']] = $value;
                }
                $return[] = $record;
                $in = substr($in, $done, $length);
                if (!empty($this->debug))
                    echoLog('Response Post Prior: ->' . $in . '<- <br><br>');
            }
        }else {
            $return[] = array(); //If no prior then empty
        }
        //Processing Records
        while (($length = strlen($in)) > CLX_NUM_ZERO) {
            $record = array();
            $in = substr($in, CLX_NUM_TWO, $length);
            $done = CLX_INIT_ZERO;
            foreach ($this->msgrecord as $key => $val) {
                $done +=$val['SIZE'];
                $value = trim(substr($in, $val['START'], $val['SIZE']));
                switch ($val['TYPE']) {
                    case 'DECN':
                        if ($val['DECIMALS'] > CLX_NUM_ZERO) {//Set decimal positions
                            $int = substr($value, CLX_NUM_ZERO, (strlen($value) - $val['DECIMALS']));
                            $decn = substr($value, -$val['DECIMALS']);
                            $value = (float) ($int . '.' . $decn);
                        }
                        break;
                    default:
                        break;
                }//End switch
                if (empty($this->tfields_only))
                    $record[$val['FILEFIELD']] = $value;
                if ($val['FIELD'] <> '')
                    $record[$val['FIELD']] = $value;
            }
            $return[] = $record;
            $in = substr($in, $done, $length);
        }
        return $return;
    }

//End methid
    /**
     * Sends Message to socket
     */
    function send() {
        if ($this->hasErrors()) {
            //Connector
            $SKCO = getTool('SKCO');
            $skcorec = ClxCommon::$db->getRecord($SKCO->tablename, 'IDCODE =' . singleQuote($this->skco_code));
            if (empty($skcorec)) {
                $result = false;
            } else {
                $sck = new clx_socket($skcorec['IP_ADDRESS'], $skcorec['PORT_NUMBER'], true, $this->debug);
                if ($sck->connect()) {
                    $in = $this->toString();
                    if (!empty($this->debug))
                        echoLog('Request Message Length: ' . strlen($in) . ' -><b><pre>' . $in . '</pre></b><-<br>');
                    $sck->write($sck->stringByteLength($in));
                    if ($sck->write($in)) {
                        $out = $sck->read();
                        $result = $this->recieveMessage($out);
                        if (!empty($this->debug))
                            echoLog('Response Length: ' . strlen($out) . ' -><b>' . $out . '</b><- <br>');
                    } else {
                        $result = false;
                    }
                    $sck->close();
                } else {
                    $result = false;
                }
            }
        } else {
            $result = $this->errors;
        }
        return $result;
    }

    function check() {
        $idlist = getIdlist();
        $record = $this->record();
        //$this->debug = true;
        $msg_record = ClxCommon::$db->getRecord($this->tablename, 'ID = ' . singleQuote($idlist[CLX_NUM_ZERO]));
        $this->skco_code = $msg_record['SKCONNECTOR_CODE'];
        //Some Message Info
        $contents = 'Message Code:' . $msg_record['IDCODE'] . '<br>';
        $contents .= 'Message Definition Length:' . $this->getLength($msg_record['IDCODE']) . '<br>';
        ClxFreeReturn($this->createMessage($msg_record['IDCODE'], $record));
        $strmsg = $this->toString();
        $contents .= 'Prepared Message Length:' . strlen($strmsg) . '<br>';
        $txt = new htmlTextArea('', 'Start->' . $strmsg . '<-End', CLX_NUM_TEN, CLX_NUM_110);
        $contents .= 'Message:<br>' . $txt->source();
        $i = CLX_INIT_ZERO;
        $result = $this->send();
        $txt = new htmlTextArea('', var_export($result, true), CLX_NUM_TEN, CLX_NUM_110);
        $contents .= '<br>Response:<br>' . $txt->source();
        $table = new htmlTable(CLX_WIDTH_ONE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell('E-IBS Exchange', array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_ONE));
        $table->addCell($contents, array(CLX_PROP_CLASS => ($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT));
        $c = new htmlButtonGoto(dict('GEN_LBL_CLOSE'), 'tool.php?to=SKMS&ac=LIST');
        $table->addCell($c->source(), array(CLX_PROP_CLASS => 'footer'));
        return $table->source();
    }

    function record() {
        $record = array();
        return $record;
    }

}

// End class
