<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @package		Clientix.Tool.Interface
 * @abstract		User Interface - Application Form Panels
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-07-24 20:14:46 -0400 (Tue, 24 Jul 2018) $
 * @version        $Rev: 9862 $
 *
 * CHANGE LOG
 *  Included label
 */
class ui_form_panels extends clx_tool {

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'UIFP';
        $this->name = 'Form Panels';
        $this->system = 'Interface';
        $this->version = '0.5.6.0';
        $this->accesskey = 'UIFM';



        $this->tablename = 'CLX_UI_FORM_PANELS';
        $this->record_name_singular = 'form panel';
        $this->record_name_plural = 'form panels';
        $this->primary_field = 'IDCODE';
        $this->secondary_field = 'IDNAME';
        $this->seqnum_field = 'SEQNUM';
        $this->primary_key = array('IDCODE');
    }

    /**
     * Describes the tool table structure.
     * @return array ('type' => 'varchar(255)', 'properties'=>'auto increment',
     * 				'index'=>'', default = '', 'dict'=>true)
     * 				 dict true or false means that field holds dictionary entry
     */
    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['IDCODE'] = array('type' => 'varchar(32)', 'null' => 'NO', 'properties' => '');
        $fields['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');

        $fields['FORMCODE'] = array('type' => 'varchar(32)', 'null' => 'NO', 'properties' => '');
        $fields['PANEL_CLASS'] = array('type' => 'varchar(255)', 'properties' => '');
        $fields['DESCRIPTION'] = array('type' => 'mediumtext', 'properties' => '');
        $fields['TITLE'] = array('type' => 'varchar(255)', 'properties' => '', 'dict' => true);
        $fields['CAPTION'] = array('type' => 'varchar(255)', 'properties' => '', 'dict' => true);
        $fields['NUMCOLS'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ONE);
        $fields['TYPE'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['SEQNUM'] = array('type' => 'tinyint(4) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['ACTIVE'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => '1', 'properties' => '');
        $fields['INITIAL_STATE'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        //$fields['FORM_OBJ_CODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['FORM_OBJ_CODE'] = array('type' => 'varchar(16)', 'null' => 'YES', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'IDCODE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'FORMCODE');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'ACTIVE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB', 'fields' => $fields, 'indexes' => $indexes);
    }

    /**
     * Pack tool record on xml Format
     * @param array record
     * @return obj
     */
    function pack($record) {
        $o = $this->packRecord($record);
        $objs = new XMLObject($this->code . '_OBJECTS');
        $this->packsub($objs, 'UIFF', 'SELECT * FROM CLX_UI_FORM_PANEL_FIELDS WHERE PANELID=' . singleQuote($record['IDCODE']));
        $this->packsub($objs, 'EVNT', 'SELECT * FROM CLX_EVENTS WHERE OWNERCODE=' . singleQuote($this->code) . ' AND OWNERID=' . singleQuote($record['IDCODE']));
        $o->addObject($objs);
        return $o;
    }

    /**
     * Loads package into system
     * @param string $package
     */
    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null, 'id' => null), $parameters));
        list($package, $formid) = $parameters;
        $record = $this->unpackRecord($package);
        ClxCommon::$db->createRecordVoid($this->tablename, $record, false);
        if ($package->hasSubObject($this->code . '_OBJECTS')) {
		  $objs = $package->getObjectXml($this->code . '_OBJECTS');
		  foreach ($objs->objects as $o) {
			 $toolcode = $o->tag;
			 $tool = &getTool($toolcode);
			 $tool->unpack(array('package' => $o, 'id' => $record['IDCODE']));
		  }
        }
    }

    /**
     * Builds Panel listing on tool
     *
     * @param string $groupcode
     * @return string
     */
    function makepanel($formcode = null) {
        if ($this->getAccess()->canRead()) {
		  $t = time();
		  $to = $this->code;
		  $here = 'tool.php?to=' . $to . '&t=' . $t;
		  $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
		  $form->setProp(CLX_PROP_ID, 'UIFPPanel');
		  $form->addArgument(CLX_PROP_TO, $this->code);
		  $form->addArgument('formcode', $formcode);
		  $form->addArgument('ac', 'NOT_DEFINED');
		  $selectclause = '*';
		  $fromclause = $this->tablename;
		  $whereclause = 'FORMCODE=' . singleQuote($formcode);
		  $sortclause = 'SEQNUM ASC';
		  $page = new xPaging();
		  $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
		  $page->setProp('printheaders', false);
		  $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
		  $table = new htmlTable(CLX_WIDTH_NINE);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $page->header($table);
		  $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_IDCODE'), dict('GEN_LBL_NAME'), dict('GEN_LBL_TYPE'), dict('GEN_LBL_ACTIVE'), dict('GEN_LBL_INITIALSTATE'), dict('GEN_LBL_DESCRIPTION'), dict('GEN_LBL_NUMCOL'), dict('GEN_LBL_FORM_OBJ_CODE')));
		  $types = array(dict('GEN_LBL_PLAIN'), dict('GEN_LBL_COLAPSABLE'));
		  $states = array(dict('GEN_LBL_COLAPSED'), dict('GEN_LBL_EXPANDED'));
		  $i = CLX_INIT_ZERO;
		  while ($record = $page->nextRecord()) {
			 $recordid = $record['ID'];
			 $select = new htmlCheckbox('selector_' . $recordid, false);
			 $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
			 $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
			 $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array(
			     $record['IDCODE'], $record['IDNAME'], $types[$record['TYPE']], dict($this->yesno[$record['ACTIVE']]), $states[$record['INITIAL_STATE']], xFormat::text($record['DESCRIPTION']),
			     $record['NUMCOLS'], $record['FORM_OBJ_CODE']));
			 $i++;
		  }
		  $buttons = '';
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  if ($this->getAccess()->canModify()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  }
		  if ($this->getAccess()->canDelete()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  }
		  if ($this->getAccess()->canDisplay()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_PANEL')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
		  }
		  if (($page->getProp('qty') > CLX_NUM_ONE) && $this->getAccess()->canModify()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_REORDER'), array('action' => 'REORDER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
		  }
		  $form->addContents($page->footers($table, $buttons));
		  $modulecode = '';
		  return buildPanelInterface(dict('TOOL_UIFP_TITLE_LIST'), $modulecode . 'TABLES', $form->source(), -1 * CLX_NUM_ONE);
        } else {
		  return CLX_CHAR_EMPTY_STRING;
        }
    }

    /**
     * Tool Edit Form
     * @param string $ac
     * @param string $idlist
     * @param array $record
     * @param array $msg
     * @return string
     */
    function editForm($ac, $idlist, $record, $msg) {IF ($GLOBALS['ADDRDEBUGEDITFORM_']==1) { ECHO '__METHOD____FILE____LINE__'. __METHOD__.' '.__FILE__.':'.__LINE__.'<br>'; }
        $script = setJavaScriptTag(getJavaScriptFilename('UIFP_editform.js'), true);
        $modify = (UserRequest::$request['ac'] === 'MODIFY');
        $title = ($modify) ? dict('TOOL_UIFP_TITLE_MODIFY') : dict('TOOL_UIFP_TITLE_CREATE');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('UIFP_id', $record['UIFP_id']);
        $form->addArgument('idlist', toString($idlist));
        $callingpage = peekHistory();
        $formcode = $callingpage['formcode'];
        $form->addArgument('UIFP_formcode', $formcode);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if (!empty($modify)) {
		  $form->addArgument('UIFP_idcode', $record['UIFP_idcode']);
		  $table->addData(dict('GEN_LBL_IDCODE') . ':', $record['UIFP_idcode']);
        } else {
		  $table->addField($msg, dict('GEN_LBL_IDCODE') . ':', 'UIFP_idcode', $record['UIFP_idcode'], CLX_NUM_20, CLX_NUM_16);
        }
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'UIFP_idname', $record['UIFP_idname'], CLX_NUM_32, CLX_NUM_64);
        $types = new htmlSelect('UIFP_type');
        $types->addItem(dict('GEN_LBL_PLAIN'), '0', ($record['UIFP_type'] === '0'));
        $types->addItem(dict('GEN_LBL_COLAPSABLE'), '1', ($record['UIFP_type'] === '1'));
        $types->setProp('onchange', 'checkForm();');
        $table->addData(dict('GEN_LBL_TYPE') . ':', $types->source());
        $state = new htmlSelect('UIFP_initial_state');
        $state->addItem(dict('GEN_LBL_COLAPSED'), '0', ($record['UIFP_initial_state'] === '0'));
        $state->addItem(dict('GEN_LBL_EXPANDED'), '1', ($record['UIFP_initial_state'] === '1'));
        $state->setProp('onchange', 'checkForm();');
        $table->addData(dict('GEN_LBL_INITIALSTATE') . ':', $state->source());
        $table->addYesNo(dict('GEN_LBL_ACTIVE'), 'UIFP_active', $record['UIFP_active'], CLX_NUM_20);
        $table->addField($msg, dict('TOOL_UIFP_PROMPT_PANELCLASS') . ':', 'UIFP_panel_class', $record['UIFP_panel_class'], CLX_NUM_48, CLX_NUM_64);
        $table->addText($msg, dict('GEN_LBL_DESCRIPTION') . ':', 'UIFP_description', $record['UIFP_description'], CLX_NUM_FOUR, CLX_NUM_48);

        $DICT = &getTool('DICT');
        $DICT->editEntry($msg, dict('GEN_LBL_TITLE') . ':', 'UIFP_title', $record, $table, $form);
        $DICT->editEntry($msg, dict('GEN_LBL_CAPTION') . ':', 'UIFP_caption', $record, $table, $form);
        $table->addField($msg, dict('GEN_LBL_NUMCOL') . ':', 'UIFP_numcols', $record['UIFP_numcols'], CLX_NUM_TWO, CLX_NUM_TWO);

        $UIFM = &getTool('UIFM');
        $UIFM->placeChooseForm($msg, dict('TOOL_UIFP_FORM_OBJ_CODE') . ':', 'UIFP_form_obj_code', $record['UIFP_form_obj_code'], $table);
        $form->addContents($table->source() . '<br>');
        if (empty($modify)) { // This is create
		  $tablecode = ClxCommon::$db->getField('CLX_UI_FORMS', 'TABLECODE', 'IDCODE=' . singleQuote($formcode));
            $panelqty = ClxCommon::$db->oneExistsRecords($this->tablename, 'FORMCODE=' . singleQuote($formcode));
		  if ($panelqty > CLX_NUM_ZERO) {
			 $records = ClxCommon::$db->getRecords($this->tablename, 'FORMCODE=' . singleQuote($formcode));
			 $panelids = array();
			 while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
			     $panelids[] = singleQuote($record['IDCODE']);
			 }
			 ClxCommon::$db->freeResult( $records );
			 $panelids = toString($panelids);
		  }
		  $records = ClxCommon::$db->getRecords('CLX_DB_TABLEFIELDS', 'TABLECODE=' . singleQuote($tablecode) . ' ORDER BY FIELDNAME');
		  $fields = array();
		  while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
			 if ($panelqty > CLX_NUM_ZERO) {
                    $qty = ClxCommon::$db->oneExistsRecords('CLX_UI_FORM_PANEL_FIELDS', 'PANELID IN (' . $panelids . ') AND FIELDNAME=' . singleQuote($record['FIELDNAME']));
			     if ($qty == CLX_NUM_ZERO) {
			         $fields[] = $record['FIELDNAME'];
			     }
			 } else {
			     $fields[] = $record['FIELDNAME'];
			 }
		  }
		  ClxCommon::$db->freeResult( $records );
		  $table = new htmlTable(CLX_WIDTH_FOUR);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $table->addCell(dict('TOOL_UIFP_TITLE_INSERTFIELDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
		  $qty = count($fields);
		  if ($qty == CLX_NUM_ZERO) {
			 $table->addCell(dict('TOOL_UIFP_ERR_NOMOREFIELDS'), array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
		  } else {
			 for ($i = CLX_NUM_ZERO; $i < $qty; $i++) {
			     $c = new htmlCheckbox('UIFP_FIELD_' . $fields[$i], false);
			     $table->addCell($c->source() . ' ' . $fields[$i], array(CLX_PROP_CLASS => CLX_CLASS_DATA));
			 }
			 $dif = CLX_WIDTH_FOUR - $i % CLX_WIDTH_FOUR;
			 if ($dif != CLX_WIDTH_FOUR)
			     for ($i = CLX_NUM_ZERO; $i < $dif; $i++) {
			         $table->addCell(CLX_CHAR_HTML_NBSP, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
			     }
		  }
		  $form->addContents($table->source() . '<br>');
        }
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons = '';
        if (!empty($modify)) {
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        }
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => 'buttons', CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $script . $form->source();
    }

    /**
     * Verify Tool Edit Forms
     * @param array $msg
     * @return bool
     */
    function checkForm(&$msg) {
        $msg = array();
        $action = UserRequest::$post['ac'];
        xValidate::required($msg, 'UIFP_idcode', UserRequest::$post['UIFP_idcode']);
        xValidate::required($msg, 'UIFP_idname', UserRequest::$post['UIFP_idname']);
        xValidate::integer($msg, 'UIFP_numcols', UserRequest::$post['UIFP_numcols'], CLX_NUM_ONE, CLX_NUM_NINE, dict('GEN_LBL_NUMCOL'));
        if ($action === 'CREATE') {
		  xValidate::unique($msg, 'UIFP_idcode', 'UIFP_idcode', $this->tablename, 'IDCODE=' . singleQuote(UserRequest::$post['UIFP_idcode']));
        }
        $DICT = &getTool('DICT');
        $DICT->validateEntry('UIFP_title', UserRequest::$post, false, $msg);
        $DICT->validateEntry('UIFP_caption', UserRequest::$post, false, $msg);
        return (count($msg) === CLX_NUM_ZERO) ? true : false;
    }

    /**
     * Builds confirm delete form for tool
     * @param obj $table
     * @param array $record
     */
    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_IDCODE') . ':', $record['UIFP_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['UIFP_idname']);
    }

    /**
     * Build display form contents
     * @param array $record
     * @return string
     */
    function viewFormFill($record) {
        addHistory('panelid', $record['UIFP_idcode']);
        addHistory('ownercode', $this->code);
        addHistory('ownerid', $record['UIFP_idcode']);
        $contents = '';
        $formrec = ClxCommon::$db->getRecord('CLX_UI_FORMS', 'IDCODE=' . singleQuote($record['UIFP_formcode']));
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_UIFP_TITLE_VIEW') . ': ' . $record['UIFP_idname'], array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_FORM') . ':', $formrec['IDNAME'] . ' (' . $formrec['IDCODE'] . ')');
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['UIFP_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['UIFP_idname']);
        $table->addData(dict('GEN_LBL_ACTIVE') . ':', dict($this->yesno[$record['UIFP_active']]));
        $table->addData(dict('GEN_LBL_DESCRIPTION') . ':', xFormat::text($record['UIFP_description']));
        $table->addData(dict('GEN_LBL_NUMCOL') . ':', $record['UIFP_numcols']);
        $DICT = &getTool('DICT');
        $DICT->viewEntry(dict('GEN_LBL_TITLE') . ':', $record['UIFP_title'], $table);
        $DICT->viewEntry(dict('GEN_LBL_CAPTION') . ':', $record['UIFP_caption'], $table);
        $contents .= $table->source();
        if (trim($record['UIFP_form_obj_code']) === '') {
		  $UIFF = &getTool('UIFF');
		  $contents .= $UIFF->makepanel($record['UIFP_idcode']);
        }
        $EVNT = &getTool('EVNT');
        $contents .= $EVNT->makepanel($this->code, $record['UIFP_idcode']);
        return $contents;
    }

    /**
     * Creates reorder list
     * @return string
     */
    function reorderRecords() {
        $records = ClxCommon::$db->getRecords($this->tablename, 'FORMCODE=' . singleQuote(UserRequest::$post['formcode']) . ' ORDER BY SEQNUM');
        $list = array();
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
		  $list[$record['ID']] = $record['IDNAME'] . '(' . $record['IDCODE'] . ')';
        }
		ClxCommon::$db->freeResult( $records);
        return $list;
    }

    /**
     * Performs reorder record action
     */
    function reorderAction() {
        parent::reorderAction();
        $callingpage = peekHistory();
        $formcode = $callingpage['formcode'];
        $UIFM = &getTool('UIFM');
        $UIFM->versionIncrease('IDCODE', $formcode);
    }

    /**
     * Loads current record on selected id
     *
     * @param integer $id
     * @return array
     */
    function loadToolRecord($recordid) {
        $DICT = &getTool('DICT');
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID= ' . $recordid);
        $record = array();
        $record['UIFP_id'] = $recordid;
        $record['UIFP_idcode'] = $values['IDCODE'];
        $record['UIFP_idname'] = $values['IDNAME'];
        $record['UIFP_formcode'] = $values['FORMCODE'];
        $record['UIFP_panel_class'] = $values['PANEL_CLASS'];
        $record['UIFP_description'] = $values['DESCRIPTION'];
        $record['UIFP_active'] = $values['ACTIVE'];
        $DICT->loadEntry('UIFP_title', $values['TITLE'], $record);
        $DICT->loadEntry('UIFP_caption', $values['CAPTION'], $record);
        $record['UIFP_numcols'] = $values['NUMCOLS'];
        $record['UIFP_type'] = $values['TYPE'];
        $record['UIFP_seqnum'] = $values['SEQNUM'];
        $record['UIFP_initial_state'] = $values['INITIAL_STATE'];
        $record['UIFP_form_obj_code'] = $values['FORM_OBJ_CODE'];
        return $record;
    }

    /**
     * Update records
     *
     * @return integer
     */
    function saveRecord() {
        $DICT = &getTool('DICT');
 //       $formrec = ClxCommon::$db->getRecord('CLX_UI_FORMS', 'IDCODE=' . singleQuote(UserRequest::$post['UIFP_formcode']));
        $record = array();
        $record['IDNAME'] = UserRequest::$post['UIFP_idname'];
        $record['DESCRIPTION'] = UserRequest::$post['UIFP_description'];
        $record['PANEL_CLASS'] = UserRequest::$post['UIFP_panel_class'];
        $record['ACTIVE'] = UserRequest::$post['UIFP_active'];
        $record['TITLE'] = $DICT->createEntry('UIFP_title', UserRequest::$post);
        $record['caption'] = $DICT->createEntry('UIFP_caption', UserRequest::$post);
        $record['NUMCOLS'] = UserRequest::$post['UIFP_numcols'];
        $record['TYPE'] = UserRequest::$post['UIFP_type'];
        $record['INITIAL_STATE'] = UserRequest::$post['UIFP_initial_state'];
        $record['FORM_OBJ_CODE'] = UserRequest::$post['UIFP_form_obj_code'];
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['UIFP_id']);
        $UIFM = &getTool('UIFM');
        $UIFM->versionIncrease('IDCODE', UserRequest::$post['UIFP_formcode']);
    }

    /**
     * Creates record
     *
     * @return integer
     */
    function createRecord_() {
        $DICT = &getTool('DICT');
        $record = array();
        $record['IDCODE'] = UserRequest::$post['UIFP_idcode'];
        $record['IDNAME'] = UserRequest::$post['UIFP_idname'];
        $record['FORMCODE'] = UserRequest::$post['UIFP_formcode'];
        $record['DESCRIPTION'] = UserRequest::$post['UIFP_description'];
        $record['PANEL_CLASS'] = UserRequest::$post['UIFP_panel_class'];
        $record['NUMCOLS'] = UserRequest::$post['UIFP_numcols'];
        $record['TYPE'] = UserRequest::$post['UIFP_type'];
        $record['ACTIVE'] = UserRequest::$post['UIFP_active'];
        $record['INITIAL_STATE'] = UserRequest::$post['UIFP_initial_state'];
        $record['FORM_OBJ_CODE'] = UserRequest::$post['UIFP_form_obj_code'];
        $record['TITLE'] = $DICT->createEntry('UIFP_title', UserRequest::$post);
        $record['caption'] = $DICT->createEntry('UIFP_caption', UserRequest::$post);
        $record['SEQNUM'] = ClxCommon::$db->getField($this->tablename, 'SEQNUM', 'FORMCODE=' . singleQuote(UserRequest::$post['UIFP_formcode']) . ' ORDER BY SEQNUM DESC') + CLX_NUM_ONE;
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $panelidcode = UserRequest::$post['UIFP_idcode'];
        $UIFM = &getTool('UIFM');
        $UIFM->versionIncrease('IDCODE', UserRequest::$post['UIFP_formcode']);
        $i = CLX_INIT_ZERO;
        reset(UserRequest::$post);
        foreach (UserRequest::$post as $key => $value) {
		  if (substr($key, CLX_NUM_ZERO, CLX_NUM_11) === 'UIFP_FIELD_') {
			 $fieldname = substr($key, CLX_NUM_11);
			 $fieldrec = ClxCommon::$db->getRecord('CLX_DB_FIELDS', 'FIELDNAME=' . singleQuote($fieldname));
			 $sql = 'SELECT TF.FIELD_DEFAULT, TF.FIELD_REQUIRED FROM CLX_DB_TABLEFIELDS TF, CLX_UI_FORMS F WHERE TF.TABLECODE = F.TABLECODE AND F.IDCODE = ' . singleQuote(UserRequest::$post['UIFP_formcode']) . ' AND TF.FIELDNAME = ' . singleQuote($fieldname);
			 $table_field = ClxCommon::$db->getOneRecord($sql);
			 if (!empty($table_field)) {
			     $fieldrec['FIELD_DEFAULT'] = $table_field['FIELD_DEFAULT'];
			     $fieldrec['FIELD_REQUIRED'] = $table_field['FIELD_REQUIRED'];
			 }
			 $record = array();
			 $record['PANELID'] = $panelidcode;
			 $record['caption'] = '';
			 $record['FIELDNAME'] = $fieldname;
			 $record['FIELDSEQNUM'] = $i;
			 $record['FIELDFORMLABEL'] = $fieldrec['FIELDFORMLABEL'];
			 $record['FIELDPREFIX'] = $fieldrec['FIELDPREFIX'];
			 $record['FIELDSUFIX'] = $fieldrec['FIELDSUFIX'];
			 $record['CAPTION_CLASS'] = CLX_CLASS_DATA;
			 $record['FIELD_DISPLAYSIZE'] = CLX_INIT_ZERO;
			 $record['FIELD_READONLY'] = CLX_INIT_ZERO;
			 $record['FIELD_PASSWORD'] = CLX_INIT_ZERO;
			 $record['FIELD_HIDDEN'] = CLX_INIT_ZERO;
			 $record['FIELD_REQUIRED'] = $fieldrec['FIELD_REQUIRED'];
			 $record['FIELD_DEFAULT'] = $fieldrec['FIELD_DEFAULT'];
			 $record['TABLECODE'] = 'null';
			 $record['MASK_CODE'] = 'null';
			 $record['SRCH_LISTING'] = 'null';
			 ClxCommon::$db->createRecord('CLX_UI_FORM_PANEL_FIELDS', $record, false);
			 $i++;
		  }
        }
        return $recordid;
    }

    /**
     * Deletes record on selected id
     *
     * @param integer $id
     */
    function deleteRecordSk($id) {
        $panelrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . singleQuote($id));
        $UIFF = &getTool('UIFF');
        $fields = ClxCommon::$db->getRecords('CLX_UI_FORM_PANEL_FIELDS', 'PANELID=' . singleQuote($panelrec['IDCODE']));
        while (($field = ClxCommon::$db->getNextRecord($fields))) {
		  $UIFF->deleteRecordSk($field['ID']);
        }
		ClxCommon::$db->freeResult( $fields);
        ClxCommon::$db->deleteRecords('CLX_EVENTS', 'OWNERCODE=' . singleQuote($this->code) . ' AND OWNERID=' . singleQuote($panelrec['IDCODE']));
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        $UIFM = &getTool('UIFM');
        $UIFM->versionIncrease('IDCODE', $panelrec['FORMCODE']);
    }

    /**
     * Defines default record on initializacion
     *
     * @return array
     */
    function defaultRecord() {
        $callingpage = peekHistory();
        $qty = ClxCommon::$db->countRecords($this->tablename, 'FORMCODE =' . singleQuote($callingpage['formcode'])) + CLX_NUM_ONE;
        $len = strlen($qty . '');
        if ($len < CLX_NUM_THREE) {
		  $idcode = $callingpage['formcode'] . '_' . str_repeat('0', CLX_NUM_THREE - $len) . $qty;
        } else {
		  $idcode = $callingpage['formcode'] . '_' . $qty;
        }
        $record = array();
        $record['UIFP_id'] = '0';
        $record['UIFP_idcode'] = $idcode;
        $record['UIFP_idname'] = '';
        $record['UIFP_formcode'] = '';
        $record['UIFP_panel_class'] = CLX_CLASS_FORM;
        $record['UIFP_description'] = '';
        $record['UIFP_title'] = '';
        $record['UIFP_title_DICT'] = '';
        $record['UIFP_caption'] = '';
        $record['UIFP_caption_DICT'] = '';
        $record['UIFP_numcols'] = '1';
        $record['UIFP_type'] = '0';
        $record['UIFP_seqnum'] = '0';
        $record['UIFP_initial_state'] = '1';
        $record['UIFP_active'] = '1';
        $record['UIFP_form_obj_code'] = '';
        return $record;
    }

    /**
     * Increase tool version
     * @param string $fieldname
     * @param string $fieldvalue
     */
    function versionIncrease($fieldname, $fieldvalue) {
        $formcode = ClxCommon::$db->getField($this->tablename, 'FORMCODE', $fieldname . '=' . singleQuote($fieldvalue));
        $UIFM = &getTool('UIFM');
        $UIFM->versionIncrease('IDCODE', $formcode);
    }

}

// End class
