<?php

/**
 * Listing Class
 * For more information contact  support@alenet.com
 *
 * @author	   $Author: alenetdev $
 * @copyright	Alenet, Inc.
 * @since		$Date: 2019-01-22 15:17:13 -0400 (ma., 22 ene. 2019) $
 * @version		$Rev: 10481 $
 *
 * CHANGE LOG
 *  Modifyed class style on caption and subtitle from data to subtitle and caption on displaylist
 *  Oracle uppercase record changed
 *  Duplicate listing enabled
 */
class ui_listing extends clx_tool {

    var $debug = false;
    var $summary_view_type; // List of summary view types
    private $debugExport = false;
	private $listingFormatLimit = 543000;

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'UILG';
        $this->name = 'Listing Manager';
        $this->system = 'Interface';
        $this->version = '5.6.1.0';

        $this->has_access = true;
        $this->has_package = true;
        $this->has_pagelet = true;
        $this->tablename = 'CLX_UI_LISTINGS';
        $this->record_name_singular = 'listing';
        $this->record_name_plural = 'listings';
        $this->primary_field = 'IDCODE';
        $this->secondary_field = 'IDNAME';

        $this->primary_key = array('IDCODE');
        $this->summary_view_type = array(dict('GEN_LBL_NO'), dict('GEN_LBL_BOTH'), dict('GEN_LBL_UP'), dict('GEN_LBL_DOWN'));
        $this->enable_rec_buttons = true;
        $this->quickfilterform = array('panels' => array(
			 array('type' => 'normal',
			     'entries' => array(
			         array('label' => 'GEN_LBL_CODE', 'fields' => array(array('name' => 'idcode', 'type' => 'string', 'default' => '', 'size' => '16', 'displaysize' => '16'))),
			         array('label' => 'GEN_LBL_NAME', 'fields' => array(array('name' => 'idname', 'type' => 'string', 'default' => '', 'size' => '64', 'displaysize' => '48'))),
			         array('label' => 'GEN_LBL_MODULECODE', 'fields' => array(array('name' => 'modulecode', 'type' => 'lookup', 'default' => '', 'readonly' => false, 'tool' => 'DBMO'))),
			         array('label' => 'GEN_LBL_TABLECODE', 'fields' => array(array('name' => 'tablecode', 'type' => 'lookup', 'default' => '', 'readonly' => false, 'tool' => 'DBTB')))
        ))));
		$this->outTypesExport = array(/*'xls' => 'GEN_LBL_XLS_2', 'xlsx' => 'GEN_LBL_XLS_1','ods' => 'GEN_LBL_ODS'*/
								'tsxls'=>'GEN_LBL_TSXLS','txt1'=>'GEN_LBL_TXT1','txt2'=>'GEN_LBL_TXT2','csv'=>'GEN_LBL_CSV','json'=>'GEN_LBL_JSON' );
		
		
		
		if (!isset($this->debug))
			$this->debug=false;
		IF (ISSET($GLOBALS['ADDRDEBUGUILISTING']) && $GLOBALS['ADDRDEBUGUILISTING']==$_SERVER['REMOTE_ADDR']) {
			$this->debug=true;
		}
    }

    function install() {
        $UIMI = &getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_INTERFACE:MENU_LISTINGS', 'TOOL', $this->code, 'ac=LIST');
    }

    function toolTableStructure() {
        $fields = array();
        $fields ['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields ['IDCODE'] = array('type' => 'varchar(32)', 'null' => 'NO', 'properties' => '');
        $fields ['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields ['VERSION'] = array('type' => 'varchar(32)', 'null' => 'NO', 'properties' => '');
        $fields ['VERSION_COUNTER'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields ['CDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields ['MDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields ['USERNAME'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $fields ['DESCRIPTION'] = array('type' => 'mediumtext', 'properties' => '');
        $fields ['TITLE'] = array('type' => 'varchar(255)', 'properties' => '', 'dict' => true);
        $fields ['CAPTION'] = array('type' => 'varchar(255)', 'properties' => '', 'dict' => true);
        $fields ['MODULECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields ['TABLECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields ['QUERYCODE'] = array('type' => 'varchar(16)', 'properties' => '');
        $fields ['CREATE_FORM'] = array('type' => 'varchar(16)', 'properties' => '');
        $fields ['MODIFY_FORM'] = array('type' => 'varchar(16)', 'properties' => '');
        $fields ['DUPLICATE_FORM'] = array('type' => 'varchar(16)', 'properties' => '');
        $fields ['EFFTDATE_FORM'] = array('type' => 'varchar(16)', 'properties' => '');
        $fields ['DISPLAY_FORM'] = array('type' => 'varchar(16)', 'properties' => '');
        $fields ['DELETE_FORM'] = array('type' => 'varchar(16)', 'properties' => '');
        $fields ['SEARCH_FORM'] = array('type' => 'varchar(16)', 'properties' => '');
        $fields ['TOOLTIP_FORM'] = array('type' => 'varchar(16)', 'properties' => '');
        $fields ['VIEW_SUMMARY'] = array('type' => 'tinyint(1) unsigned', 'null' => 'YES', 'properties' => '');
        $fields ['VIEW_SUMMARY_EXP'] = array('type' => 'varchar(255)', 'properties' => '');
        $fields ['CONTROL_EXPORT'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields ['CONTROL_VIEWS'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields ['CONTROL_FILTERS'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields ['HIDE_EFFTDATE'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields ['EXEONSEARCH'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields ['AUTH_DISABLED'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields ['AUTH_BTCH_FILD'] = array('type' => 'varchar(64)', 'null' => 'YES', 'properties' => '');
        $fields ['AUTH_BTCH_FILDSUM'] = array('type' => 'varchar(64)', 'null' => 'YES', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes [$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes [$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'IDCODE');
        $indexes [$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'IDNAME');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB', 'fields' => $fields, 'indexes' => $indexes);
    }

    // L I S T I N G S
    function pack($record) {
        $o = $this->packRecord($record);
        $code = $record ['IDCODE'];
        $objs = new XMLObject($this->code . '_OBJECTS');
        $this->packsub($objs, 'UILB', 'SELECT * FROM CLX_UI_LISTING_BUTTONS WHERE LISTINGCODE=' . singleQuote($code));
        $this->packsub($objs, 'EVNT', 'SELECT * FROM CLX_EVENTS WHERE OWNERCODE=' . singleQuote($this->code) . ' AND OWNERID=' . $record ['ID']);
        // Packing Dictionary
        $this->packsub($objs, 'DICT', 'SELECT DICT.* FROM CLX_DICTIONARY DICT, CLX_UI_LISTINGS X WHERE (DICT.ENTRY=X.TITLE OR DICT.ENTRY=X.CAPTION) AND X.IDCODE=' . singleQuote($code));
        $this->packsub($objs, 'DICT', 'SELECT DICT.* FROM CLX_DICTIONARY DICT, CLX_UI_LISTING_BUTTONS X WHERE DICT.ENTRY=X.NAME AND X.LISTINGCODE=' . singleQuote($code));
        $o->addObject($objs);
        return $o;
    }

    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        $listingcode = uppercase($record ['IDCODE']);
        $record ['MODULECODE'] = uppercase($record ['MODULECODE']);
        $record ['TABLECODE'] = uppercase($record ['TABLECODE']);
        $record ['QUERYCODE'] = uppercase($record ['QUERYCODE']);
        $record ['CREATE_FORM'] = uppercase($record ['CREATE_FORM']);
        $record ['MODIFY_FORM'] = uppercase($record ['MODIFY_FORM']);
        $record ['DISPLAY_FORM'] = uppercase($record ['DISPLAY_FORM']);
        $record ['DUPLICATE_FORM'] = uppercase($record ['DUPLICATE_FORM']);
        $record ['EFFTDATE_FORM'] = uppercase($record ['EFFTDATE_FORM']);
        $record ['DELETE_FORM'] = uppercase($record ['DELETE_FORM']);
        $record ['SEARCH_FORM'] = uppercase($record ['SEARCH_FORM']);
        $record ['TOOLTIP_FORM'] = uppercase($record ['TOOLTIP_FORM']);

		if (ClxCommon::$db->oneExistsRecords($this->tablename, 'IDCODE=' . singleQuote($listingcode)) > CLX_NUM_ZERO) {
		  ClxCommon::$db->updateRecord($this->tablename, $record, 'IDCODE=' . singleQuote($listingcode));
        } else {
		  ClxCommon::$db->createRecordVoid($this->tablename, $record, false);
        }
        $listingid = ClxCommon::$db->getField($this->tablename, 'ID', 'IDCODE=' . singleQuote($listingcode));
        ClxCommon::$db->deleteRecords('CLX_EVENTS', 'OWNERCODE=' . singleQuote($this->code) . ' AND OWNERID=' . singlequote( $listingid));
        ClxCommon::$db->deleteRecords('CLX_UI_LISTING_BUTTONS', 'LISTINGCODE=' . singleQuote($listingcode));

        if ($package->hasSubObject($this->code . '_OBJECTS')) {
		  $objs = $package->getObjectXml($this->code . '_OBJECTS');
		  foreach ($objs->objects as $o) {
			 $toolcode = $o->tag;
			 $tool = &getTool($toolcode);
			 $tool->unpack(array('package' => $o, 'id' => $listingid));
		  }
        }
    }

    function action($ac, $idlist) {
        if ($this->getAccess()->canRead()) {
		  switch ($ac) {
			 case 'BUILDCSV' :
			     $contents = $this->build_csv_start($idlist);
			     break;
			 case 'BUILDCSV_START' :
					IF (!ISSET(UserRequest::$request ['UILG_format'])) {
						$contents  = $this->FORMsendListingCsv(UserRequest::$request ['li'], 'anrj');;
					}
					ELSE	
			     $contents = $this->sendListingCsv(UserRequest::$request ['li'], 'anrj');
			     break;
			 case 'BUILDCSV_BATCH' :
			     $contents = $this->sendListingCsv_phase2(UserRequest::$request ['li']);
			     break;
			 case 'BUILDCSV_STREAMING' :
			     $contents = $this->sendListingCsv_phase2(UserRequest::$request ['li'], 'var');
			     break;
			 case 'PREVIEW_LST' :
			     $contents = $this->listingPreview();
			     break;
			 default :
			     $contents = parent::action($ac, $idlist);
			     break;
		  }
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [UI_LISTING-CAN_READ_1][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    /**
     * Debug Export Getter
     * @return type
     */
    public function getDebugExport() {
        return $this->debugExport;
    }

    function makelist($listingcode = null) {
        //$t = time('U'); genera un warning
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'UILGList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = 'CLX_UI_LISTINGS';
        $whereclause = '';
        $sortclause = '';
        $page = new xPaging ();
		$page->querycode=$querycode;
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
       $table = new htmlTable((ISSET($GLOBALS['ADDRDEBUGLISTINGTVIEW']))?CLX_WIDTH_NINE:CLX_WIDTH_7);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_UILG_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => (ISSET($GLOBALS['ADDRDEBUGLISTVIEW']))?CLX_WIDTH_NINE:CLX_WIDTH_7));
        $page->header($table);
        
		$arreglocells=array(dict('GEN_LBL_SEL'), dict('GEN_LBL_MODULECODE'), dict('GEN_LBL_TABLECODE'), dict('GEN_LBL_CODE'), dict('GEN_LBL_NAME'), dict('GEN_LBL_VERSION'), dict('GEN_LBL_DESCRIPTION'));      
		if (ISSET($GLOBALS['ADDRDEBUGLISTINGTVIEW'])){
			$arreglocells[]='buttons'; $arreglocells[]='mdate';
		}	
		$table->addCells(CLX_CLASS_HEADERS,$arreglocells );
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
		  $recordid = $record ['ID'];
		  $select = new htmlCheckbox('selector_' . $recordid, false);
		  $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
		  $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_ALIGN => CLX_PROP_CENTER));
			
			$arreglocells=array($record ['MODULECODE'], $record ['TABLECODE'], $record ['IDCODE'], $record ['IDNAME'], $record ['VERSION'] . '-' . $record ['VERSION_COUNTER'], xFormat::text($record ['DESCRIPTION']));
			if (ISSET($GLOBALS['ADDRDEBUGLISTINGTVIEW'])) {
				$arreglocells[]=  ClxCommon::$db->getOne('SELECT LISTAGG( VALUE,'.singlequote(' - ').') WITHIN GROUP (ORDER BY ROWNUM) BUT FROM CLX_UI_LISTING_BUTTONS X,CLX_DICTIONARY Y WHERE X.NAME=Y.ENTRY AND LISTINGCODE='.singlequote($record ['IDCODE']));
				$arreglocells[]='<nobr>'.SUBSTR($record['MDATE'],0,10).'</nobr>'; 
			}
		    $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT,$arreglocells );
		  $i++;
        }
        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        if ($this->getAccess()->canModify()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDelete()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canAppend()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DUPLICATE'), array('action' => 'DUPLICATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ONE'))->source();
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_LISTING')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        // CR-2018-08-15 - INICIO - Agrega boton de Vista Previa 
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_EXECUTE'), array('action' => 'PREVIEW_LST', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ONE'))->source();
        // CR-2018-08-15 - FIN - Agrega boton de Vista Previa 
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    function searchForm() {
        $msg = array();
        $contents = '';
        // Search Form
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server ['PHP_SELF']);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'SEARCH');
        $form->addArgument('confirm', '1');
        $form->addArgument(CLX_PROP_TARGET, UserRequest::$request [CLX_PROP_TARGET]);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_UILG_TITLE_SEARCH'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'UILG_idcode', (!empty(UserRequest::$post ['UILG_idcode'])) ? UserRequest::$post ['UILG_idcode'] : '', CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'UILG_idname', (!empty(UserRequest::$post ['UILG_idname'])) ? UserRequest::$post ['UILG_idname'] : '', CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_MODULECODE') . ':', 'UILG_modulecode', (!empty(UserRequest::$post ['UILG_modulecode'])) ? UserRequest::$post ['UILG_modulecode'] : '', CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_TABLECODE') . ':', 'UILG_tablecode', (!empty(UserRequest::$post ['UILG_tablecode'])) ? UserRequest::$post ['UILG_tablecode'] : '', CLX_NUM_FOUR, CLX_NUM_FOUR);
        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents .= $form->source();
        // Search Procedure
        if (!empty(UserRequest::$post ['confirm'])) {
		  $table = new htmlTable(CLX_WIDTH_TWO);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $criteria = '';
		  if (UserRequest::$post ['UILG_idcode'] !== '') {
			 $criteria .= 'UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post ['UILG_idcode']) . '%');
		  }
		  if (UserRequest::$post ['UILG_idname'] !== '') {
			 $criteria .= 'UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post ['UILG_idname'])) . '%');
		  }
		  if (UserRequest::$post ['UILG_modulecode'] !== '') {
			 $criteria .= 'UPPER(MODULECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post ['UILG_modulecode']) . '%');
		  }
		  if (UserRequest::$post ['UILG_tablecode'] !== '') {
			 $criteria .= 'UPPER(TABLECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post ['UILG_tablecode']) . '%');
		  }
			$qty = CLX_NUM_ZERO;
			$table_or_view_exists = ClxCommon::$db->tableExists('CLX_UI_LISTINGS');
			if($table_or_view_exists )  {
				$qty = ClxCommon::$db->oneExistsRecords('CLX_UI_LISTINGS', $criteria);
			}
		  if ($qty == CLX_NUM_ZERO) {
			 $table->addCell(dict('TOOL_UILG_ERR_NOMATCH'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  } else {
			 $table->addCell(dict('TOOL_UILG_MSG_FOUNDRECORDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
			 $records = ClxCommon::$db->getRecords('CLX_UI_LISTINGS', $criteria);
			 $i = CLX_INIT_ZERO;
			 while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
			     $js = 'javascript:setLookupValue(' . quote(UserRequest::$request [CLX_PROP_TARGET]) . ', ' . quote($record ['IDCODE']) . ');';
			     $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<page t=ui_listing></page><a href=' . singleQuote($js) . '>' . $record ['IDCODE'] . '</a>', $record ['IDNAME']));
			     $i++;
			 }
			 ClxCommon::$db->freeResult($records);

		  }
		  
		  $contents .= $table->source();
        }
        return $contents;
    }

    function editForm($ac, $idlist, $record, $msg) {
        $modify = (UserRequest::$request['ac'] === 'MODIFY');
        $title = ($modify) ? dict('TOOL_UILG_TITLE_MODIFY') : dict('TOOL_UILG_TITLE_CREATE');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('UILG_id', $record ['UILG_id']);
        $form->addArgument('idlist', toString($idlist));
        $form->addArgument('relfm', UserRequest::$request ['relfm']);
        // [menuid] =>
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if (!empty($modify)) {
		  $table->addData(dict('GEN_LBL_CODE') . ':', $record ['UILG_idcode']);
		  $form->addArgument('UILG_idcode', $record ['UILG_idcode']);
        } else {
		  $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'UILG_idcode', $record ['UILG_idcode'], CLX_NUM_16, CLX_NUM_16);
        }
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'UILG_idname', $record ['UILG_idname'], CLX_NUM_48, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_VERSION') . ':', 'UILG_version', $record ['UILG_version'], CLX_NUM_16, CLX_NUM_16);
        $table->addText($msg, dict('GEN_LBL_DESCRIPTION') . ':', 'UILG_description', $record ['UILG_description'], CLX_NUM_FOUR, CLX_NUM_48);
        $DBMO = &getTool('DBMO');
        $DBMO->placeChooseModule($msg, dict('TOOL_UILG_PROMPT_MODULECODE') . ':', 'UILG_modulecode', $record ['UILG_modulecode'], $table);
        $DBTB = &getTool('DBTB');
        $DBTB->placeChooseTable($msg, dict('TOOL_UILG_PROMPT_TABLECODE') . ':', 'UILG_tablecode', $record ['UILG_tablecode'], $table);
        $DBQY = &getTool('DBQY');
        $DBQY->placeChooseQuery($msg, dict('GEN_LBL_QUERY') . ':', 'UILG_querycode', $record ['UILG_querycode'], $table);
        $DICT = &getTool('DICT');
        $DICT->editEntry($msg, dict('GEN_LBL_TITLE') . ':', 'UILG_title', $record, $table, $form);
        $DICT->editEntry($msg, dict('GEN_LBL_CAPTION') . ':', 'UILG_caption', $record, $table, $form);
        $form->addContents($table->source() . '<br>');
        // Listing Forms ----------------------------
        $table = new htmlTable(CLX_WIDTH_FOUR);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_UILG_TITLE_FORMS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
        $UIFM = &getTool('UIFM');
        $UIFM->placeChooseForm($msg, dict('TOOL_UILG_PROMPT_ADDFORM') . ':', 'UILG_create_form', $record ['UILG_create_form'], $table);
        $UIFM->placeChooseForm($msg, dict('TOOL_UILG_PROMPT_MODFORM') . ':', 'UILG_modify_form', $record ['UILG_modify_form'], $table);
        $UIFM->placeChooseForm($msg, dict('TOOL_UILG_PROMPT_DPLFORM') . ':', 'UILG_display_form', $record ['UILG_display_form'], $table);
        $UIFM->placeChooseForm($msg, dict('TOOL_UILG_PROMPT_DUPFORM') . ':', 'UILG_duplicate_form', $record ['UILG_duplicate_form'], $table);
        $UIFM->placeChooseForm($msg, dict('TOOL_UILG_PROMPT_EFFTFORM') . ':', 'UILG_efftdate_form', $record ['UILG_efftdate_form'], $table);
        $UIFM->placeChooseForm($msg, dict('TOOL_UILG_PROMPT_DELFORM') . ':', 'UILG_delete_form', $record ['UILG_delete_form'], $table);
        $UIFM->placeChooseForm($msg, dict('TOOL_UILG_PROMPT_SCHFORM') . ':', 'UILG_search_form', $record ['UILG_search_form'], $table);
        $UIFM->placeChooseForm($msg, dict('GEN_LBL_TOOLTIP_FORM') . ':', 'UILG_tooltip_form', $record ['UILG_tooltip_form'], $table);
        if (empty($msg))
		  $table->addData(CLX_CHAR_HTML_NBSP, CLX_CHAR_HTML_NBSP);
        $form->addContents($table->source() . '<br>');
        // Listing Controls ----------------------------
        $table = new htmlTable(CLX_WIDTH_FOUR);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_UILG_TITLE_CONTROLS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
        $pop = new HTMLSelect('UILG_control_filters');
        $pop->addItem(dict('GEN_LBL_VISIBLE'), CLX_NUM_ONE, ($record ['UILG_control_filters'] == CLX_NUM_ONE));
        $pop->addItem(dict('GEN_LBL_INVISIBLE'), CLX_NUM_ZERO, ($record ['UILG_control_filters'] == CLX_NUM_ZERO));
        $table->addData(dict('TOOL_UILG_PROMPT_CFILTERS') . ':', $pop->source());
        $pop = new HTMLSelect('UILG_control_views');
        $pop->addItem(dict('GEN_LBL_VISIBLE'), CLX_NUM_ONE, ($record ['UILG_control_views'] == CLX_NUM_ONE));
        $pop->addItem(dict('GEN_LBL_INVISIBLE'), CLX_NUM_ZERO, ($record ['UILG_control_views'] == CLX_NUM_ZERO));
        $table->addData(dict('TOOL_UILG_PROMPT_CVIEW') . ':', $pop->source());
        $pop = new HTMLSelect('UILG_view_summary');
        foreach ($this->summary_view_type as $key => $val) {
		  $pop->addItem($val, $key, ($record ['UILG_view_summary'] == $key));
        }
        // $pop->addItem(dict('GEN_LBL_NO'), 0,
        $table->addData(dict('TOOL_UILG_PROMPT_VIEW_SUMARY') . ':', $pop->source());
        $table->addField($msg, dict('TOOL_UILG_PROMPT_VIEW_SUMARY_EXP') . ':<br><font size=' . singleQuote(CLX_CHAR_EMPTY_STRING) . '>!TBLCODE_QRYCOLFIELD1 = !TBLCODE_QRYCOLFIELD2 + !TBLCODE_QRYCOLFIELD3...; FORMULA2; ETC;</font>', 'UILG_view_summary_exp', $record ['UILG_view_summary_exp'], CLX_NUM_30, CLX_NUM_255);
        $table->addYesNo(dict('TOOL_UILG_PROMPT_EXEONSEARCH'), 'UILG_exeonsearch', $record ['UILG_exeonsearch'], CLX_NUM_ZERO, false);
        $table->addYesNo(dict('TOOL_UILG_PROMPT_CEXPORT'), 'UILG_control_export', $record ['UILG_control_export'], CLX_NUM_ZERO, false);
        $table->addYesNo(dict('GEN_LBL_HIDE_EFFTDATE'), 'UILG_hide_efftdate', $record ['UILG_hide_efftdate'], CLX_NUM_ZERO, false);
        $form->addContents($table->source());


        //Authorization options
        $tableWidth = CLX_WIDTH_TWO;
        $table = new htmlTable($tableWidth);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_UILG_TITLE_AUTHCONTROLS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => $tableWidth));
        $table->addYesNo(dict('GEN_LBL_AUTH_DISABLED'), 'UILG_auth_disabled', $record ['UILG_auth_disabled'], CLX_NUM_ZERO, false);
        $DBFL = &getTool("DBFL");
        $DBFL->placeChooseField($msg, dict('TOOL_UILG_PROMPT_AUTH_BTCH_FILD') . ':', 'UILG_auth_btch_fild', $record ['UILG_auth_btch_fild'], $table);
        $DBFL->placeChooseField($msg, dict('TOOL_UILG_PROMPT_AUTH_BTCH_FILDSUM') . ':', 'UILG_auth_btch_fildsum', $record ["UILG_auth_btch_fildsum"], $table);
        $form->addContents($table->source() . '<br>');

        $buttons = '';
        $table = new htmlTable(CLX_WIDTH_FOUR);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        if (UserRequest::$request['ac'] === 'MODIFY') {
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } elseif (UserRequest::$request['ac'] === 'DUPLICATE') {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'DUPLICATE', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        }
        $c = new htmlButtonCancel ();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
        $form->addContents($table->source());
		return $form->source();
    }

    function checkForm(&$msg) {

        $msg = array();
        $action = UserRequest::$post ['ac'];
        xValidate::required($msg, 'UILG_idcode', UserRequest::$post ['UILG_idcode']);
        if ($action !== 'MODIFY') {
		  xValidate::unique($msg, 'UILG_idcode', 'code', $this->tablename, 'IDCODE=' . singleQuote(UserRequest::$post ['UILG_idcode']));
        }
        $DBMO = &getTool('DBMO');
        if (!$DBMO->moduleExists(UserRequest::$post ['UILG_modulecode'])) {
		  $msg ['UILG_modulecode'] = dict('TOOL_UILG_ERR_NOMODULECODE');
        }
        $DBTB = &getTool('DBTB');
        if (!$DBTB->tableExists(UserRequest::$post ['UILG_tablecode'])) {
		  $msg ['UILG_tablecode'] = dict('TOOL_UILG_ERR_NOTABLECODE');
        }
        $DBQY = &getTool('DBQY');
        if (!$DBQY->queryExists(UserRequest::$post ['UILG_querycode'])) {
		  $msg ['UILG_querycode'] = dict('TOOL_UILG_ERR_NOQUERYCODE');
        }
        $UIFM = &getTool('UIFM');
        $forms = array('UILG_create_form', 'UILG_modify_form',
		  'UILG_display_form', 'UILG_delete_form', 'UILG_search_form', 'UILG_duplicate_form', 'UILG_efftdate_form', 'UILG_tooltip_form');
        reset($forms);
        foreach ($forms as $key => $val) {
		  if (!empty(UserRequest::$post [$val])) {
			 if (!$UIFM->formExists(UserRequest::$post [$val])) {
			     $msg [$val] = dict('TOOL_UILG_ERR_NOFORMCODE');
			 } elseif (!$UIFM->formTable(UserRequest::$post [$val], UserRequest::$post ['UILG_tablecode'])) {
			     // Skeeping from search form
			     if ($key != CLX_NUM_FOUR)
			         $msg ['UILG_create_form'] = dict('TOOL_UILG_ERR_NOTABLEFORMMATCH', UserRequest::$post ['UILG_tablecode']);
			 }
		  }
        }
        if (UserRequest::$post ['UILG_exeonsearch'] == CLX_NUM_ONE && UserRequest::$post ['UILG_search_form'] === '') {
		  $msg ['UILG_search_form'] = dict('TOOL_UILG_ERR_SRCHFORMREQUIRED');
        }
        $DICT = &getTool('DICT');
        $DICT->validateEntry('UILG_title', UserRequest::$post, false, $msg);
        $DICT->validateEntry('UILG_caption', UserRequest::$post, false, $msg);

        $DBTF = &getTool('DBTF');
        if (!empty(UserRequest::$post ['UILG_auth_btch_fild']) && !$DBTF->fieldExists(UserRequest::$post ['UILG_tablecode'], UserRequest::$post ['UILG_auth_btch_fild'])) {
		  $msg['UILG_auth_btch_fild'] = dict('GEN_ERR_TABLEFIELDDONTEXIST');
        }

        if (!empty(UserRequest::$post ['UILG_auth_btch_fildsum']) && !$DBTF->fieldExists(UserRequest::$post ['UILG_tablecode'], UserRequest::$post ['UILG_auth_btch_fildsum'])) {
		  $msg['UILG_auth_btch_fildsum'] = dict('GEN_ERR_TABLEFIELDDONTEXIST');
        }
		if (!empty(UserRequest::$post ['UILG_auth_btch_fildsum']) && empty($msg['UILG_auth_btch_fildsum']) && ClxCommon::$db->oneExistsRecords('CLX_DB_FIELDS', 'FIELDNAME = ' . singleQuote(UserRequest::$post ['UILG_auth_btch_fildsum']) . ' AND FIELDTYPE IN (' . singleQuote('DECN') . ',' . singleQuote('INTN') . ',' . singleQuote('BOOL') . ')') == 0) {
		  $msg['UILG_auth_btch_fildsum'] = dict('GEN_ERR_FIELDNOTNUMERIC');
        }

        return count($msg) == CLX_NUM_ZERO;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_CODE') . ':', $record ['UILG_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record ['UILG_idname']);
    }

    function viewFormFill($record) {
        addHistory('listingcode', $record ['UILG_idcode']);
        addHistory('ownercode', $this->code);
        addHistory('ownerid', $record ['UILG_id']);
        $contents = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_UILG_TITLE_VIEW') . ': ' . $record ['UILG_idname'], array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_CODE') . ':', $record ['UILG_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record ['UILG_idname']);
        $table->addData(dict('GEN_LBL_VERSION') . ':', $record ['UILG_version'] . '-' . $record ['UILG_version_counter']);
        $table->addData(dict('GEN_LBL_DESCRIPTION') . ':', xFormat::text($record ['UILG_description']));
        $table->addData(dict('TOOL_UILG_PROMPT_MODULECODE') . ':', $record ['UILG_modulecode']);
        $table->addData(dict('TOOL_UILG_PROMPT_TABLECODE') . ':', $record ['UILG_tablecode']);
        $table->addData(dict('GEN_LBL_QUERY') . ':', $record ['UILG_querycode']);
        $DICT = &getTool('DICT');
        $DICT->viewEntry(dict('GEN_LBL_TITLE') . ':', $record ['UILG_title'], $table);
        $DICT->viewEntry(dict('GEN_LBL_CAPTION') . ':', $record ['UILG_caption'], $table);
        $contents .= $table->source();
        $table = new htmlTable(CLX_WIDTH_FOUR);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addData(dict('TOOL_UILG_PROMPT_ADDFORM') . ':', $record ['UILG_create_form']);
        $table->addData(dict('TOOL_UILG_PROMPT_MODFORM') . ':', $record ['UILG_modify_form']);
        $table->addData(dict('TOOL_UILG_PROMPT_DPLFORM') . ':', $record ['UILG_display_form']);
        $table->addData(dict('TOOL_UILG_PROMPT_DUPFORM') . ':', $record ['UILG_duplicate_form']);
        $table->addData(dict('TOOL_UILG_PROMPT_EFFTFORM') . ':', $record ['UILG_efftdate_form']);
        $table->addData(dict('TOOL_UILG_PROMPT_DELFORM') . ':', $record ['UILG_delete_form']);
        $table->addData(dict('TOOL_UILG_PROMPT_SCHFORM') . ':', $record ['UILG_search_form']);
        $table->addData(dict('GEN_LBL_TOOLTIP_FORM') . ':', $record ['UILG_tooltip_form']);
        $table->addData(CLX_CHAR_HTML_NBSP, CLX_CHAR_HTML_NBSP);
        $contents .= buildPanelInterface(dict('TOOL_UILG_TITLE_FORMS'), 'FORMS', $table->source(), false);
        $vis = array(dict('GEN_LBL_INVISIBLE'), dict('GEN_LBL_VISIBLE'));
        $yesno = array(dict('GEN_LBL_NO'), dict('GEN_LBL_YES'));
        $table = new htmlTable(CLX_WIDTH_FOUR);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addData(dict('TOOL_UILG_PROMPT_CVIEW') . ':', $vis [$record ['UILG_control_views']]);
        $table->addData(dict('TOOL_UILG_PROMPT_CFILTERS') . ':', $vis [$record ['UILG_control_filters']]);
        $table->addData(dict('TOOL_UILG_PROMPT_VIEW_SUMMARY') . ':', $this->summary_view_type [$record ['UILG_view_summary']]);
        $table->addData(dict('TOOL_UILG_PROMPT_VIEW_SUMMARY_EXP') . ':', $record ['UILG_view_summary_exp']);
        $table->addData(dict('TOOL_UILG_PROMPT_EXEONSEARCH') . ':', $yesno [$record ['UILG_exeonsearch']]);
        $table->addData(dict('TOOL_UILG_PROMPT_CEXPORT') . ':', $yesno [$record ['UILG_control_export']]);
        $table->addData(dict('TOOL_UILG_HIDE_EFFTDATE') . ':', $yesno [$record ['UILG_hide_efftdate']]);
        $contents .= buildPanelInterface(dict('TOOL_UILG_TITLE_CONTROLS'), 'CONTROLS', $table->source(), false);
        //Authorization options
        $tableWidth = CLX_WIDTH_TWO;
        $table = new htmlTable($tableWidth);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addData(dict("GEN_LBL_AUTH_DISABLED") . ":", $yesno[$record ["UILG_auth_disabled"]]);
        $table->addData(dict("TOOL_UILG_PROMPT_AUTH_BTCH_FILD") . ":", $record ["UILG_auth_btch_fild"]);
        $table->addData(dict("TOOL_UILG_PROMPT_AUTH_BTCH_FILDSUM") . ":", $record ["UILG_auth_btch_fildsum"]);
        $contents .= buildPanelInterface(dict('TOOL_UILG_TITLE_AUTHCONTROLS'), 'CONTROLSAUTH', $table->source(), false);
        $UILB = &getTool('UILB');
        $contents .= $UILB->makepanel($record ['UILG_idcode']);
        $EVNT = &getTool('EVNT');
        $contents .= $EVNT->makepanel($this->code, $record ['UILG_id']);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addData(dict('GEN_LBL_CDATE') . ':', $record ['UILG_cdate']);
        $table->addData(dict('GEN_LBL_MDATE') . ':', $record ['UILG_mdate']);
        $table->addData(dict('GEN_LBL_USERNAME') . ':', $record ['UILG_username']);
        $contents .= buildPanelInterface(dict('GEN_TTL_SYSTEMINFO'), 'SYSTEMINFO', $table->source(), false);
        return $contents;
    }

    function filterFormFill($msg, &$table) {
        if (!array_key_exists('UILG_idcode', UserRequest::$post))
		  UserRequest::$post ['UILG_idcode'] = '';
        if (!array_key_exists('UILG_idname', UserRequest::$post))
		  UserRequest::$post ['UILG_idname'] = '';
        if (!array_key_exists('UILG_modulecode', UserRequest::$post))
		  UserRequest::$post ['UILG_modulecode'] = '';
        if (!array_key_exists('UILG_tablecode', UserRequest::$post))
		  UserRequest::$post ['UILG_tablecode'] = '';
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'UILG_idcode', UserRequest::$post ['UILG_idcode'], CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'UILG_idname', UserRequest::$post ['UILG_idname'], CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_MODULECODE') . ':', 'UILG_modulecode', UserRequest::$post ['UILG_modulecode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_TABLECODE') . ':', 'UILG_tablecode', UserRequest::$post ['UILG_tablecode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $sort = new HtmlSelect('UILG_sort');
        $sort->addItem('', '', true);
        $sort->addItem(dict('GEN_LBL_MODULECODE'), 'modulecode', false);
        $sort->addItem(dict('GEN_LBL_TABLECODE'), 'tablecode', false);
        $sort->addItem(dict('GEN_LBL_CODE'), 'idcode', false);
        $sort->addItem(dict('GEN_LBL_NAME'), 'idname', false);
        $table->addData(dict('GEN_LBL_ORDERBY') . ':', $sort->source());
    }

    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post ['UILG_idcode'])) {
		  $filter .= $boolean . ' UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post ['UILG_idcode']) . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post ['UILG_idname'])) {
		  $filter .= $boolean . ' UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post ['UILG_idname'])) . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post ['UILG_modulecode'])) {
		  $filter .= $boolean . ' UPPER(MODULECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post ['UILG_modulecode']) . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post ['UILG_tablecode'])) {
		  $filter .= $boolean . '  UPPER(TABLECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post ['UILG_tablecode']) . '%');
		  $boolean = 'AND';
        }
        if (array_key_exists('UILG_sort', UserRequest::$post)) {
		  $_SESSION ['paging'] ['UILGList'] ['sortby'] = UserRequest::$post ['UILG_sort'];
        }
        $_SESSION ['paging'] ['UILGList'] ['criteria'] = $filter;
        $_SESSION ['paging'] ['UILGList'] ['startat'] = '0';
        $_SESSION ['paging'] [$this->code . 'List'] ['qfilter'] = UserRequest::$post;
        unset($_SESSION ['paging'] [$this->code . 'List'] ['qfilter'] [CLX_PROP_TO], $_SESSION ['paging'] [$this->code . 'List'] ['qfilter'] ['ac'], $_SESSION ['paging'] [$this->code . 'List'] ['qfilter'] ['qy'], $_SESSION ['paging'] [$this->code . 'List'] ['qfilter'] ['fm'], $_SESSION ['paging'] [$this->code . 'List'] ['qfilter'] ['li'], $_SESSION ['paging'] [$this->code . 'List'] ['qfilter'] ['obj_uid'], $_SESSION ['paging'] [$this->code . 'List'] ['qfilter'] ['startat'], $_SESSION ['paging'] [$this->code . 'List'] ['qfilter'] ['pc'], $_SESSION ['paging'] [$this->code . 'List'] ['qfilter'] ['nofr'], $_SESSION ['paging'] [$this->code . 'List'] ['qfilter'] ['pfid'], $_SESSION ['paging'] [$this->code . 'List'] ['qfilter'] ['rf'], $_SESSION ['paging'] [$this->code . 'List'] ['qfilter'] ['rfi'], $_SESSION ['paging'] [$this->code . 'List'] ['qfilter'] ['submit1'], $_SESSION ['paging'] [$this->code . 'List'] ['qfilter'] ['confirm'], $_SESSION ['paging'] [$this->code . 'List'] ['qfilter'] ['idlist']);
    }
	///////////////////////////////////////////////////
	///comentado 22/01/2019 para utilizar la clx_tool
	//////////////////////////////////////////////////
	/* 
    function clearfilter() {
        $_SESSION ['paging'] ['UILGList'] ['criteria'] = '';
	}*/

    function loadToolRecord($recordid) {
        $DICT = &getTool('DICT');
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID= ' . $recordid);
        $record = array();
        $record ['UILG_id'] = $recordid;
        $record ['UILG_idcode'] = $values ['IDCODE'];
        $record ['UILG_idname'] = $values ['IDNAME'];
        $record ['UILG_version'] = $values ['VERSION'];
        $record ['UILG_version_counter'] = $values ['VERSION_COUNTER'];
        $record ['UILG_cdate'] = xFormat::date($values ['CDATE']);
        $record ['UILG_mdate'] = xFormat::date($values ['MDATE']);
        $record ['UILG_username'] = $values ['USERNAME'];
        $record ['UILG_description'] = $values ['DESCRIPTION'];
        $DICT->loadEntry('UILG_title', $values ['TITLE'], $record);
        $DICT->loadEntry('UILG_caption', $values ['CAPTION'], $record);
        $record ['UILG_modulecode'] = $values ['MODULECODE'];
        $record ['UILG_tablecode'] = $values ['TABLECODE'];
        $record ['UILG_querycode'] = $values ['QUERYCODE'];
        $record ['UILG_create_form'] = $values ['CREATE_FORM'];
        $record ['UILG_modify_form'] = $values ['MODIFY_FORM'];
        $record ['UILG_display_form'] = $values ['DISPLAY_FORM'];
        $record ['UILG_duplicate_form'] = $values ['DUPLICATE_FORM'];
        $record ['UILG_efftdate_form'] = $values ['EFFTDATE_FORM'];
        $record ['UILG_delete_form'] = $values ['DELETE_FORM'];
        $record ['UILG_search_form'] = $values ['SEARCH_FORM'];
        $record ['UILG_tooltip_form'] = $values ['TOOLTIP_FORM'];
        $record ['UILG_view_summary'] = $values ['VIEW_SUMMARY'];
        $record ['UILG_view_summary_exp'] = $values ['VIEW_SUMMARY_EXP'];
        $record ['UILG_control_views'] = $values ['CONTROL_VIEWS'];
        $record ['UILG_control_filters'] = $values ['CONTROL_FILTERS'];
        $record ['UILG_exeonsearch'] = $values ['EXEONSEARCH'];
        $record ['UILG_control_export'] = $values ['CONTROL_EXPORT'];
        $record ['UILG_hide_efftdate'] = $values ['HIDE_EFFTDATE'];

        $record ['UILG_auth_disabled'] = $values ['AUTH_DISABLED'];
        $record ['UILG_auth_btch_fild'] = $values ['AUTH_BTCH_FILD'];
        $record ['UILG_auth_btch_fildsum'] = $values ['AUTH_BTCH_FILDSUM'];
        return $record;
    }

    function saveRecord() {
        $DICT = &getTool('DICT');
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record ['IDNAME'] = UserRequest::$post ['UILG_idname'];
        $record ['VERSION'] = UserRequest::$post ['UILG_version'];
        $record ['MDATE'] = $d;
        $record ['USERNAME'] = $_SESSION ['user_record'] ['username'];
        $record ['DESCRIPTION'] = UserRequest::$post ['UILG_description'];
        $record ['TITLE'] = $DICT->saveEntry('UILG_title', UserRequest::$post);
        $record ['CAPTION'] = $DICT->saveEntry('UILG_caption', UserRequest::$post);
        $record ['QUERYCODE'] = UserRequest::$post ['UILG_querycode'];
        $record ['MODULECODE'] = UserRequest::$post ['UILG_modulecode'];
        $record ['TABLECODE'] = UserRequest::$post ['UILG_tablecode'];
        $record ['CREATE_FORM'] = UserRequest::$post ['UILG_create_form'];
        $record ['MODIFY_FORM'] = UserRequest::$post ['UILG_modify_form'];
        $record ['DISPLAY_FORM'] = UserRequest::$post ['UILG_display_form'];
        $record ['DUPLICATE_FORM'] = UserRequest::$post ['UILG_duplicate_form'];
        $record ['EFFTDATE_FORM'] = UserRequest::$post ['UILG_efftdate_form'];
        $record ['DELETE_FORM'] = UserRequest::$post ['UILG_delete_form'];
        $record ['SEARCH_FORM'] = UserRequest::$post ['UILG_search_form'];
        $record ['TOOLTIP_FORM'] = UserRequest::$post ['UILG_tooltip_form'];
        $record ['VIEW_SUMMARY_EXP'] = UserRequest::$post ['UILG_view_summary_exp'];
        $record ['VIEW_SUMMARY'] = UserRequest::$post ['UILG_view_summary'];
        $record ['CONTROL_VIEWS'] = UserRequest::$post ['UILG_control_views'];
        $record ['CONTROL_FILTERS'] = UserRequest::$post ['UILG_control_filters'];
        $record ['EXEONSEARCH'] = UserRequest::$post ['UILG_exeonsearch'];
        $record ['CONTROL_EXPORT'] = UserRequest::$post ['UILG_control_export'];
        $record ['HIDE_EFFTDATE'] = UserRequest::$post ['UILG_hide_efftdate'];
        $record ['AUTH_DISABLED'] = UserRequest::$post ['UILG_auth_disabled'];
        $record ['AUTH_BTCH_FILD'] = UserRequest::$post ['UILG_auth_btch_fild'];
        $record ['AUTH_BTCH_FILDSUM'] = UserRequest::$post ['UILG_auth_btch_fildsum'];

        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post ['UILG_id']);
        $this->versionIncrease('ID', UserRequest::$post ['UILG_id']);
    }

    function createRecord_() {
        $DICT = &getTool('DICT');
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record ['IDCODE'] = UserRequest::$post ['UILG_idcode'];
        $record ['IDNAME'] = UserRequest::$post ['UILG_idname'];
        $record ['VERSION'] = UserRequest::$post ['UILG_version'];
        $record ['VERSION_COUNTER'] = '0';
        $record ['CDATE'] = $d;
        $record ['MDATE'] = $d;
        $record ['USERNAME'] = $_SESSION ['user_record'] ['username'];
        $record ['DESCRIPTION'] = UserRequest::$post ['UILG_description'];
        $record ['TITLE'] = $DICT->createEntry('UILG_title', UserRequest::$post);
        $record ['CAPTION'] = $DICT->createEntry('UILG_caption', UserRequest::$post);
        $record ['MODULECODE'] = UserRequest::$post ['UILG_modulecode'];
        $record ['TABLECODE'] = UserRequest::$post ['UILG_tablecode'];
        $record ['QUERYCODE'] = UserRequest::$post ['UILG_querycode'];
        $record ['CREATE_FORM'] = UserRequest::$post ['UILG_create_form'];
        $record ['MODIFY_FORM'] = UserRequest::$post ['UILG_modify_form'];
        $record ['DISPLAY_FORM'] = UserRequest::$post ['UILG_display_form'];
        $record ['DUPLICATE_FORM'] = UserRequest::$post ['UILG_duplicate_form'];
        $record ['EFFTDATE_FORM'] = UserRequest::$post ['UILG_efftdate_form'];
        $record ['DELETE_FORM'] = UserRequest::$post ['UILG_delete_form'];
        $record ['SEARCH_FORM'] = UserRequest::$post ['UILG_search_form'];
        $record ['TOOLTIP_FORM'] = UserRequest::$post ['UILG_tooltip_form'];
        $record ['VIEW_SUMMARY_EXP'] = UserRequest::$post ['UILG_view_summary_exp'];
        $record ['VIEW_SUMMARY'] = UserRequest::$post ['UILG_view_summary'];
        $record ['CONTROL_VIEWS'] = UserRequest::$post ['UILG_control_views'];
        $record ['CONTROL_FILTERS'] = UserRequest::$post ['UILG_control_filters'];
        $record ['EXEONSEARCH'] = UserRequest::$post ['UILG_exeonsearch'];
        $record ['CONTROL_EXPORT'] = UserRequest::$post ['UILG_control_export'];
        $record ['HIDE_EFFTDATE'] = UserRequest::$post ['UILG_hide_efftdate'];
        $record ['AUTH_DISABLED'] = UserRequest::$post ['UILG_auth_disabled'];
        $record ['AUTH_BTCH_FILD'] = UserRequest::$post ['UILG_auth_btch_fild'];
        $record ['AUTH_BTCH_FILDSUM'] = UserRequest::$post ['UILG_auth_btch_fildsum'];
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $record ['MODULECODE']);
        return $recordid;
    }

    function deleteRecordSk($id) {
        $listingrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        $UILB = &getTool('UILB');
        ClxCommon::$db->deleteRecords($UILB->tablename, 'LISTINGCODE =' . singleQuote($listingrec ['IDCODE']));
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $listingrec ['MODULECODE']);
    }

    function duplicateRecord() {
        $originalid = UserRequest::$post ['UILG_id'];
        $originalrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $originalid);
        $originalcode = $originalrec ['IDCODE'];
        $id = $this->createRecord_();
        $code = UserRequest::$post ['UILG_idcode'];
        // Buttons
        $records = ClxCommon::$db->getRecords('CLX_UI_LISTING_BUTTONS', 'LISTINGCODE=' . singleQuote($originalcode));
        while ($record = ClxCommon::$db->getNextRecord($records)) {
		  unset($record ['ID']);
		  $record ['LISTINGCODE'] = $code;
		  ClxCommon::$db->createRecord('CLX_UI_LISTING_BUTTONS', $record, false);
        }
		ClxCommon::$db->freeResult($records);

        // Events
        $records = ClxCommon::$db->getRecords('CLX_EVENTS', 'OWNERCODE=' . singleQuote($this->code) . ' AND OWNERID=' . singleQuote($originalid));
        while ($record = ClxCommon::$db->getNextRecord($records)) {
		  unset($record ['ID']);
		  $record ['OWNERID'] = $id;
		  ClxCommon::$db->createRecord('CLX_EVENTS', $record, false);
        }
		ClxCommon::$db->freeResult($records);

        return $id;
    }

    function defaultRecord() {
        $record = array();
        $record ['UILG_id'] = CLX_INIT_ZERO;
        $record ['UILG_idcode'] = '';
        $record ['UILG_idname'] = '';
        $record ['UILG_version'] = '1.0.0';
        $record ['UILG_cdate'] = '';
        $record ['UILG_mdate'] = '';
        $record ['UILG_username'] = $_SESSION ['user_record'] ['username'];
        $record ['UILG_description'] = '';
        $record ['UILG_title'] = '';
        $record ['UILG_title_DICT'] = '';
        $record ['UILG_caption'] = '';
        $record ['UILG_caption_DICT'] = '';
        $record ['UILG_modulecode'] = '';
        $record ['UILG_tablecode'] = '';
        $record ['UILG_querycode'] = '';
        $record ['UILG_create_form'] = '';
        $record ['UILG_modify_form'] = '';
        $record ['UILG_display_form'] = '';
        $record ['UILG_duplicate_form'] = '';
        $record ['UILG_efftdate_form'] = '';
        $record ['UILG_delete_form'] = '';
        $record ['UILG_search_form'] = '';
        $record ['UILG_tooltip_form'] = '';
        $record ['UILG_view_summary_exp'] = '';
        $record ['UILG_view_summary'] = '0';
        $record ['UILG_control_views'] = '1';
        $record ['UILG_control_filters'] = '1';
        $record ['UILG_exeonsearch'] = '0';
        $record ['UILG_control_export'] = '0';
        $record ['UILG_hide_efftdate'] = '0';
        $record ['UILG_auth_disabled'] = '0';
        $record ['UILG_auth_btch_fild'] = '';
        $record ['UILG_auth_btch_fildsum'] = '';
        return $record;
    }

    function versionIncrease($fieldname, $fieldvalue) {
        parent::versionIncrease($fieldname, $fieldvalue);
        $modulecode = ClxCommon::$db->getField($this->tablename, 'MODULECODE', $fieldname . '=' . singleQuote($fieldvalue));
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $modulecode);
    }

    function placeChooseListing($msg, $prompt, $name, $default, &$table) {

        $searchIcon = getSkinIconPop('view', dict('GEN_LBL_SEARCH'), 'UILG', 'SEARCH', $name);
        if (!empty($msg [$name])) {
		  $table->addCell($msg [$name], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $table->width));
        }
        $table->addCell($prompt, array(CLX_PROP_CLASS => CLX_CLASS_PROMPT));
        $c = new htmlField($name, $default, CLX_NUM_16, CLX_NUM_16);
        $table->addCell($c->source() . $searchIcon, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        return CLX_CHAR_EMPTY_STRING;
    }

    function listingExists(&$idcode) {
		$fetched=false;
		if(kApcTtl_25>0) $xidcode = apc_fetchFct($findVar=kSpaceName . '_listing_exist_'.$idcode, $fetched);
		if (empty($fetched)) {
			$qty = ClxCommon::$db->oneExistsRecords($this->tablename, 'IDCODE=' . singleQuote($idcode));
        if ($qty > CLX_NUM_ZERO) {
		  $idcode = ClxCommon::$db->getField($this->tablename, 'IDCODE', 'IDCODE=' . singleQuote($idcode));
				if (kApcTtl_25>0 ) {
					apc_addFct($findVar,$idcode,kApcTtl_25);
				}
			}
		}
		else {
			$idcode=$xidcode;
			$qty=1;
        }
        return $qty > CLX_NUM_ZERO;
    }

    function pagelet($code, $ac = null, $objectid = null) {
        $source = '';
		if(ClxCommon::$db->oneExistsRecords($this->tablename, 'IDCODE = '.singleQuote($code))===CLX_NUM_ZERO){
		  $BUGS = &getTool('BUGS');
		  if (!empty($BUGS)) {
			 $BUGS->report(E_USER_WARNING, 'Missing Listing on page: '.$code, 'tools/ui_pages.php', '611', UserRequest::$request, CLX_NUM_ZERO, '');
		  }
		  $source = note(dict('GEN_LBL_INACTIVE').' Ref: UILG.'.$code);
        }else{
		  $source=$this->buildListing($code);
        }
        return $source;
    }

    /**
     * Loads Events for listing record and performs event
     *
     * @param string $listingid
     * @param array $record
     */
    function loadListingRecord(&$listingid, &$record) {
        $msg = array();
        $EVNT = &getTool('EVNT');
        $EVNT->execute5('UILG', $listingid, 'LIRL', $record, $msg); // Listing Load
        // Record
    }

    /**
     * Builds Listing
     *
     * @param string $listingcode
     * @param string $target
     * @param string $pop_list
     * @param string $efftdated
     * @param string $dynrel_search
     * @return string
     */
    function buildListing($listingcode, $target = '', $pop_list = false, $efftdated = true, $dynrel_search = false,$parentrecord=array()) {
		IF ($this->debug==true) {echo"parameter buildlisting<br>";PRE(func_get_args()); 
			PRE(' SHOW CLX_DEFAULT_SQL_COMPANY');
			$Erecord = ClxCommon::$db->getRecord('CLX_DEFAULT_SQL_COMPANY', '1=1');
			PRE($Erecord);
		
		}//pre($GLOBALS);
        $contents = '';
        $fields = '';
        $bcmath = (function_exists('bcadd')) ? true : false;
        $UIVW = &getTool('UIVW');
        $DBQY = &getTool('DBQY');
        $EDIT = &getTool('EDIT');
        $EVNT = &getTool('EVNT');
        $UIFI = &getTool('UIFI');
        $DBTB = &getTool('DBTB');
        $whereclause = '';
        $target = ($target === '') ? getCurrentScript() : $target;
		$fetched=false;
		if(kApcTtl_25>0) $listingrec = apc_fetchFct($findVar=kSpaceName . '_listing_'.$listingcode, $fetched);
				
		if (empty($fetched)) {
			$listingrec = ClxCommon::$db->getRecord('CLX_UI_LISTINGS', 'IDCODE=' . singleQuote($listingcode));
			if (kApcTtl_25>0 && !empty($listingrec)) {
				apc_addFct($findVar, $listingrec ,kApcTtl_25);
			
			}
		}
		
		if (empty($listingrec)) {
		  $listingrec = array('TABLECODE' => '', 'QUERYCODE' => '', 'TITLE' => '', 'SUBTITLE' => '', 'CAPTION' => '');
        }
        $querycode = $listingrec ['QUERYCODE'];
        $qaccess = $DBQY->getAppAccess($querycode);
        if (!$qaccess->canRead()) {
		  return warning(dict('GEN_ERR_NO_ACCESS') . ' [UI_LISTING-CAN_READ][BUILDLISTING] ' . $this->getCode(false) . ' - Query:' . $querycode);
        }
        if($listingrec ['TABLECODE'] !== CLX_CHAR_EMPTY_STRING) {
		  $tablename = tableName($listingrec ['TABLECODE']);
//		  if (!ClxCommon::$db->tableExists($tablename)) {
		  $table_or_view_exists = ClxCommon::$db->tableExists($tablename);
		  if(!$table_or_view_exists && kDatabaseType === 'oci') {
			 $table_or_view_exists = ClxCommon::$db->viewExists($tablename);
		  }
		  if (!$table_or_view_exists) {
			 return warning(dict('GEN_ERR_TABLE_DOES_NOT_EXISTS') . ': ' . $listingrec ['TABLECODE'] . ' (' . $tablename . ') [UI_LISTING][BUILDLISTING] ' . $this->getCode(false) . ' - Query:' . $querycode);
		  }
        }
        // Quick Filters
        $contents .= $this->makeQuickFilterForm($listingrec, $querycode);
		
        //Init temporary table if required
        $this->init_temp_tables($querycode);
        //Capture listing code first
        $page = new xPaging ();
        $page->setProp('li', $listingcode);
		$page->querycode=$querycode;
        // Creating form object, and init list Xpaging class
        $form = new htmlForm(CLX_PROP_ACTION, $target);
        $form->setProp(CLX_PROP_ID, $querycode . '_LIST');
        $page->initPagingHistory($form->getProp(CLX_PROP_ID));
        // Disable efftdate as required on listing
        if (!empty($listingrec ['HIDE_EFFTDATE'])) {
		  $DBQY->set_efftdate_status(false);
        } else {
		  $DBQY->set_efftdate_status(!$page->getProp('showhistory'));
        }
        // Effective Date Status enable or disable into de query tool
        if (!empty($listingrec ['TOOLTIP_FORM'])) {
		  $contents .= setJavaScriptTag(getJavaScriptFilename('UILG_dialog.js'), true);
		  $this->listing_title = str_replace('\'', '', dict($listingrec ['TITLE']));
        }
        // Listing Init Event
        $msg = array();
        $EVNT->execute5('UILG', $listingrec ['ID'], 'LINI', UserRequest::$post, $msg);
        // CLX_AUTH_STATUS ADDITION
        $clx_auth_status_select = '';
        $this->has_authorization = false;
        if ($EDIT->requireAuthorization($listingrec ['TABLECODE'])) {
		  $clx_auth_status_select = ', ' . $listingrec ['TABLECODE'] . '.CLX_AUTH_STATUS';
		  $this->has_authorization = true;
        }
        // Init filter
        // Evalutate if srch listing to set goback link
        $has_srch_listing = false;
        $src_fl_select = '';
        $src_fl_copy = '';
		IF ($this->debug==true) {echo"request<br>";		pre(UserRequest::$request);echo"post<br>";		pre(UserRequest::$post);echo"session<br>";		pre($_SESSION);}
		if (!empty(UserRequest::$request['pfid'])) {
		  $has_srch_listing = true;
		  // Turning off some features until analisis is completed on each
		  $this->enable_rec_buttons = false;
		  $listingrec ['CREATE_FORM'] = '';
		  $listingrec ['MODIFY_FORM'] = '';
		  $listingrec ['DISPLAY_FORM'] = '';
		  $listingrec ['DELETE_FORM'] = '';
		  $listingrec ['DUPLICATE_FORM'] = '';
		  $listingrec ['EFFTDATE_FORM'] = '';
		  $listingrec ['TOOLTIP_FORM'] = '';
		  $listingrec ['CONTROL_VIEWS'] = CLX_INIT_ZERO;
		  $listingrec ['CONTROL_FILTERS'] = CLX_INIT_ZERO;
		  $listingrec ['VIEW_SUMMARY'] = CLX_INIT_ZERO;
			//$listingrec ['EXEONSEARCH'] = CLX_INIT_ZERO;
		  $listingrec ['CONTROL_EXPORT'] = CLX_INIT_ZERO;
		  $sql = 'SELECT FPF.FIELD_RELATED, F.TABLECODE, FPF.TABLECODE FFIELD_TABLECODE, FPF.FIELDNAME, DBF.FIELDTYPE, DBF.FIELD_EXTRA1, DBF.FIELD_EXTRA2, DBF.FIELD_EXTRA3, DBF.FIELD_EXTRA4, DBF.FIELD_EXTRA9 FROM CLX_UI_FORMS F, CLX_UI_FORM_PANELS FP, CLX_UI_FORM_PANEL_FIELDS FPF, CLX_DB_FIELDS DBF WHERE F.IDCODE = FP.FORMCODE AND FP.IDCODE = FPF.PANELID AND DBF.FIELDNAME = FPF.FIELDNAME AND FPF.ID = ' . UserRequest::$request ['pfid'];
		  $src_fl = ClxCommon::$db->getOneRecord($sql);
			IF ($this->debug==true) {echo "src_fl_sql para lookup"; pre($sql); }
		  // Search srch_listing criterias
		  $src_fl_where = ' ';
		  $and = ' ';
		  $here = '';
			
		  $results = ClxCommon::$db->getRecords('CLX_UI_SRCH_LIST_CRITS', 'PFID = ' . UserRequest::$request ['pfid']);
		  $parentForm = parentFormDecode();
		  IF ($this->debug==true) {echo "parentForm"; pre($parentForm); }
		  while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
				// 2019-01-15 MF - Se agrego el null porque en oracle el query da el resultado null y en mysql vacio
			 //if ($result ['VALUE'] !== '') {
			 if ($result ['VALUE'] !== '' && $result ['VALUE'] !== null) {
			     $src_fl_where .= $and . $UIFI->buildOperator($result ['OPERATOR'], $result ['QRY_FIELDNAME'], $result ['VALUE']);
			 } else {
			     if (array_key_exists($result ['FRM_FIELDNAME'] . '_prim', $parentForm)) {
			         $fieldname = substr($result ['FRM_FIELDNAME'], CLX_NUM_FIVE);
			         getLookupValue($result ['FRM_FIELDNAME'], $fieldname, $parentForm);
			     }
			     $here .= '&' . $result ['FRM_FIELDNAME'] . '=' . $parentForm [$result ['FRM_FIELDNAME']];
					if (empty($parentForm [$result ['FRM_FIELDNAME']])) {
						if ($this->debug==true)  {
							echo ":r1:",$result ['FRM_FIELDNAME'];
						}
						if  (($pos=strpos($result ['FRM_FIELDNAME'],'.'))>0) {
							if ($this->debug==true)  {
								echo ':r2:', substr($result ['FRM_FIELDNAME'],$pos); 
							}
							$src_fl_where .= $and . $UIFI->buildOperator($result ['OPERATOR'], $result ['QRY_FIELDNAME'], $parentForm [substr($result ['FRM_FIELDNAME'],$pos)]);
					
						}
						if ($this->debug==true)  {
							echo "fin :r:";
						}	
					}
					else {
			     $src_fl_where .= $and . $UIFI->buildOperator($result ['OPERATOR'], $result ['QRY_FIELDNAME'], $parentForm [$result ['FRM_FIELDNAME']]);
			 }
					if ($this->debug==true)  {
						echo ":r3:";
						pre( $result ['QRY_FIELDNAME']);
						echo ':r4:';
						pre($parentForm [$result ['FRM_FIELDNAME']]);
						echo ':r5:';
					}
				}
			 $and = ' AND ';
		  }
		  ClxCommon::$db->freeResult($results);

		  unset($and);
		  $src_fl_prim = '';
		  $src_fl_secd = '';
		  $src_fl_terc = '';

		  if (!empty($src_fl) && $src_fl['FIELDTYPE'] === 'LOOK') {
				if ($this->debug==true) { echo "src_fl_select0";	pre($src_fl_select);}
			 $src_fl_select = ', ' . $src_fl ['FIELD_EXTRA1'] . '.' . $src_fl ['FIELD_EXTRA2'] . ' ' . $src_fl ['TABLECODE'] . '_' . $src_fl ['FIELDNAME'] . ' ';
			 $src_fl_select .= ', ' . $src_fl ['FIELD_EXTRA1'] . '.' . $src_fl ['FIELD_EXTRA3'] . ' ' . $src_fl ['TABLECODE'] . '_' . $src_fl ['FIELDNAME'] . '_prim ';
			 $src_fl_copy = $src_fl ['TABLECODE'] . '_' . $src_fl ['FIELDNAME'];
				if ($this->debug==true) { echo "src_fl_select1";	pre($src_fl_select);}
			 if (!empty($src_fl ['FFIELD_TABLECODE'])) {
			     $src_fm_fl_copy = $src_fl ['FFIELD_TABLECODE'] . '_' . $src_fl ['FIELDNAME'];
			 } else {
			     $src_fm_fl_copy = $src_fl ['TABLECODE'] . '_' . $src_fl ['FIELDNAME'];
			 }
				if ($this->debug==true) { echo "src_fl_select2";	pre($src_fl_select);}

			 if (kDatabaseType === 'oci') {
			     $src_fl_prim = $src_fl ['TABLECODE'] . '_' . $src_fl ['FIELDNAME'] . '_PRIM';
			 } else {
			     $src_fl_prim = $src_fl ['TABLECODE'] . '_' . $src_fl ['FIELDNAME'] . '_prim';
			 }
				if ($this->debug==true) { echo "src_fl_select3";	pre($src_fl_select);}
			 if (!empty($src_fl ['FFIELD_TABLECODE'])) {
			     $src_fm_fl_prim = $src_fl ['FFIELD_TABLECODE'] . '_' . $src_fl ['FIELDNAME'] . '_prim';
			 } else {
			     $src_fm_fl_prim = $src_fl ['TABLECODE'] . '_' . $src_fl ['FIELDNAME'] . '_prim';
			 }
				if ($this->debug==true) { echo "src_fl_select4";	pre($src_fl_select);}
			 if ($src_fl ['FIELD_EXTRA4'] !== '') {
			     $src_fl_select .= ', ' . $src_fl ['FIELD_EXTRA1'] . '.' . $src_fl ['FIELD_EXTRA4'] . ' ' . $src_fl ['TABLECODE'] . '_' . $src_fl ['FIELDNAME'] . '_secd ';
			     if (kDatabaseType === 'oci') {
			         $src_fl_secd = $src_fl ['TABLECODE'] . '_' . $src_fl ['FIELDNAME'] . '_SECD';
			     } else {
			         $src_fl_secd = $src_fl ['TABLECODE'] . '_' . $src_fl ['FIELDNAME'] . '_secd';
			     }
			     if (!empty($src_fl ['FFIELD_TABLECODE'])) {
			         $src_fm_fl_secd = $src_fl ['FFIELD_TABLECODE'] . '_' . $src_fl ['FIELDNAME'] . '_secd';
			     } else {
			         $src_fm_fl_secd = $src_fl ['TABLECODE'] . '_' . $src_fl ['FIELDNAME'] . '_secd';
			     }
			 }
				if ($this->debug==true) { echo "src_fl_select5";	pre($src_fl_select);}
			 if ($src_fl ['FIELD_EXTRA9'] !== '' and ! is_null($src_fl ['FIELD_EXTRA9']) ) {
			     $src_fl_select .= ', ' . $src_fl ['FIELD_EXTRA1'] . '.' . $src_fl ['FIELD_EXTRA9'] . ' ' . $src_fl ['TABLECODE'] . '_' . $src_fl ['FIELDNAME'] . '_terc ';
			     if (kDatabaseType === 'oci') {
			         $src_fl_terc = $src_fl ['TABLECODE'] . '_' . $src_fl ['FIELDNAME'] . '_TERC';
			     } else {
			         $src_fl_terc = $src_fl ['TABLECODE'] . '_' . $src_fl ['FIELDNAME'] . '_terc';
			     }
			     if (!empty($src_fl ['FFIELD_TABLECODE'])) {
			         $src_fm_fl_terc = $src_fl ['FFIELD_TABLECODE'] . '_' . $src_fl ['FIELDNAME'] . '_terc';
			     } else {
			         $src_fm_fl_terc = $src_fl ['TABLECODE'] . '_' . $src_fl ['FIELDNAME'] . '_terc';
			     }
			 }
				if ($this->debug==true) { echo "src_fl_select6";	pre($src_fl_select);}
		
		  } elseif (!empty($src_fl)) {

			 $src_fl_copy = '';
				if (ClxCommon::$db->oneExistsRecords('CLX_DB_TABLEFIELDS', 'TABLECODE = ' . singleQuote($listingrec ['TABLECODE']) . ' AND FIELDNAME = ' . singleQuote($src_fl ['FIELDNAME'])) > CLX_NUM_ZERO) {
			     $src_fl_select = ', ' . $listingrec ['TABLECODE'] . '.' . $src_fl ['FIELDNAME'] . ' ' . $src_fl ['TABLECODE'] . '_' . $src_fl ['FIELDNAME'];
			     $src_fl_prim = $listingrec ['TABLECODE'] . '_' . $src_fl ['FIELDNAME'];
			     if (!empty($src_fl ['FFIELD_TABLECODE'])) {
			         $src_fm_fl_prim = $src_fl ['FFIELD_TABLECODE'] . '_' . $src_fl ['FIELDNAME'];
			     } else {
			         $src_fm_fl_prim = $src_fl ['TABLECODE'] . '_' . $src_fl ['FIELDNAME'];
			     }
			 } else {

			 }
		  }
		  unset($src_fl);
		  //$contents .= sprintf('<script language = "JavaScript" src="%s" type="text/javascript"></script>', getJavaScriptFilename('util.js'));
        } else {
		  $src_fl_select = '';
		  $src_fl_copy = '';
		  UserRequest::$request ['pfid'] = CLX_INIT_ZERO;
		  UserRequest::$request ['nofr'] = CLX_INIT_ZERO;
		  $src_fl_where = '';
        }

        // Normal Listing
        $tablecode = $listingrec ['TABLECODE']; // Agregado para provar linea
        $listingtablecode = $listingrec ['TABLECODE'];
        $listingtablerec = $DBTB->getTable($listingtablecode);
        if ($this->debug==true) {		   echoLog('listing code: ' . $listingcode . '<br>' . 'query code: ' . $querycode . '<br>' . 'table code: ' . $tablecode . '<br>');}
		
	
		if (!isset($_SESSION['paging'][$querycode.'_LIST']['oldshowhistory']) || $_SESSION['paging'][$querycode.'_LIST']['showhistory'] != $_SESSION['paging'][$querycode.'_LIST']['oldshowhistory'] ) {
			apc_clear_querycode($querycode);
			$_SESSION['paging'][$querycode.'_LIST']['oldshowhistory']=$_SESSION['paging'][$querycode.'_LIST']['showhistory'];
		}
					
        $view = $UIVW->loadView($querycode);
		$page->addviewcolumn=$view ['COLUMNS'];
		if ($this->debug==true) { echo "view query";	pre($view);}
		$sql='SELECT CONCAT(CONCAT(A.TABLECODE,'.singlequote('.').'),FIELDNAME) FIELD,CONCAT(CONCAT(A.TABLECODE,'.singleQuote('_').'),FIELDNAME) ALIAS FROM CLX_DB_QUERY_TABLES A, CLX_DB_TABLEFIELDS B WHERE A.TABLECODE=B.TABLECODE AND A.QUERYCODE='.singlequote($querycode);
		$rs=ClxCommon::$db->executequery($sql);
		IF ($this->debug==true) {echo"view original " . __LINE__ . "<br>";		PRE($view); }
		
		$fieldsAllquery=array();
        while ($fieldsAllquery[] = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
			
		}
		ClxCommon::$db->freeResult($rs);

        if ((strlen(dict($listingrec ['TITLE'])) + strlen(dict($listingrec ['CAPTION'])) + strlen($view ['SUBTITLE'])) > CLX_NUM_ZERO) {
		  $table = new htmlTable(CLX_WIDTH_ONE);
		  $table->setProps(array(CLX_PROP_CLASS => 'gen_title'));
		  if (strlen(dict($listingrec ['TITLE'])) > CLX_NUM_ZERO) {
			 $table->addCell(dict($listingrec ['TITLE']), array(CLX_PROP_CLASS => CLX_CLASS_TITLE));
		  }
		  if (strlen($view ['SUBTITLE']) > CLX_NUM_ZERO) {
			 $table->addCell($view ['SUBTITLE'], array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE));
		  } elseif (strlen(dict($listingrec ['CAPTION'])) > CLX_NUM_ZERO) {
			 $table->addCell(dict($listingrec ['CAPTION']), array(CLX_PROP_CLASS => 'caption'));
		  }
		  $contents .= $table->source();
        }
        // The Listing
        $form = new htmlForm(CLX_PROP_ACTION, $target);
        $form->setProp(CLX_PROP_ID, $querycode . '_LIST');
        $form->addArgument(CLX_PROP_TO, 'EDIT');
        $form->addArgument('ac', 'NOT_DEFINED');
        $form->addArgument('li', $listingcode);
        $form->addArgument('qy', $querycode);
        addHistory('li', $listingcode);
        addHistory('qy', $querycode);
		$comma='';
		if($listingtablecode=='RCTI') {
			$selectclause ='';
			$clx_auth_status_select= str_replace(',','',$clx_auth_status_select);
		}
		ELSE {
			$comma=', ';
			$selectclause = $listingtablecode . '.ID ' . $listingtablecode . '_ID ';
		}
		$selectclause .= $clx_auth_status_select;
        IF ($this->debug==true) {echo"selectclause1<br>";		PRE($selectclause); }
        if (!empty($view ['SELECTCLAUSE'])) {
		  $ocigroup = $selectclause;
		  if (strpos($view ['SELECTCLAUSE'], trim($selectclause)) !== false) {
			 $selectclause = $view ['SELECTCLAUSE'];
		  } else {
			 $selectclause .= ', ' . $view ['SELECTCLAUSE'];
		  }
        }
        IF ($this->debug==true) {echo"selectclause2<br>";		PRE($selectclause); }
        if ($src_fl_select !== '') {
			IF ($this->debug==true) {echo"src_fl_select8<br>";		PRE($src_fl_select); }
		  if ($selectclause === '') {
			 $selectclause .= $src_fl_select;
		  } else {
			 $selectclause .= $src_fl_select;
		  }
        }
        
		IF ($this->debug==true) {echo"selectclause3<br>";		PRE($selectclause); }
        $whereclause = '';
				IF ($this->debug==true) {echo"view where " . __LINE__ . "<br>";		PRE($view ['WHERECLAUSE']); }

        if (strlen($view ['WHERECLAUSE']) > CLX_NUM_ZERO) {
		  $whereclause = $view ['WHERECLAUSE'];
        }
				IF ($this->debug==true) {echo"whereclause " . __LINE__ . "<br>";		PRE($whereclause); }
        if (trim($src_fl_where) !== '') {
			IF ($this->debug==true) {echo"src_fl_where<br>";		PRE($src_fl_where); }
			
		  if (empty($whereclause)) {
			 $whereclause = $src_fl_where;
		  } else {
			 $whereclause .= ' AND ' . $src_fl_where;
		  }
        }
        if (!empty($GLOBALS['extra_whereclause'])) {
		  if (empty($extra_whereclause)) {
			 $whereclause = $GLOBALS['extra_whereclause'];
		  } else {
			 $whereclause .= ' AND ' . $GLOBALS['extra_whereclause'];
		  }
		}
		IF ($this->debug==true) {echo"whereclause " . __LINE__ . "<br>";		PRE($whereclause); }
		
		//pre("x:".strpos($_SERVER['HTTP_REFERER'],'RCTA_PAGES_01' ));
		IF (isset($_SERVER['HTTP_REFERER']) && strpos($_SERVER['HTTP_REFERER'],'RCTA_PAGES_01' )>0 && isset($_GET['pc']) && substr($_GET['pc'],0,11)=='RCTM_MATCH_') {
			$matchx=explode('_', $_GET['pc']);
			$match=$matchx[count($matchx)-1];
			IF (!empty($whereclause))
				$whereclause.= ' AND ';
			$sql='SELECT TABLE_CODE FROM CLX_DB_TABLE_HIST WHERE TABLE_CODE_HIST='.singlequote($tablecode);			
			$tablecodeA= ClxCommon::$db->getOne($sql);
		//	pre($sql);
			$whereclause.= ' EXISTS ( SELECT 1 FROM RCTA_TRN_MATCH_RES_SUGERIDOS WHERE RCTM_CODE='.singlequote($match).' AND TABLECODE='.singlequote($tablecodeA) .' and REC_ID = '.$tablecode.'.id'.
									 ' UNION '.
									 ' SELECT 1 FROM RCTA_TRN_MATCH_RES_SUGERIDOS WHERE RCTM_CODE='.singlequote($match).' AND TABLECODE2='.singlequote($tablecodeA) .' and REC_ID2 = '.$tablecode.'.id'.
									 ' UNION '.
									 ' SELECT 1 FROM RCTA_TRN_MATCH_RES WHERE RCTM_CODE='.singlequote($match).' AND TABLECODE='.singlequote($tablecodeA) .' and REC_ID = '.$tablecode.'.id'.
									 ' UNION '.
									 ' SELECT 1 FROM RCTA_TRN_MATCH_RES WHERE RCTM_CODE='.singlequote($match).' AND TABLECODE2='.singlequote($tablecodeA) .' and REC_ID2 = '.$tablecode.'.id'.
									')';
			
			
			//PRE($whereclause);
			//DIE;
			
		}
	
		IF ($this->debug==true) {echo"whereclause " . __LINE__ . "<br>";		PRE($whereclause); }
		
        $fromclause = $view ['FROMCLAUSE'];
        $sortclause = $view ['SORTCLAUSE'];
        $groupclause = $view ['GROUPCLAUSE'];
	//	pre('select * from '.$fromclause." where ".$whereclause);
		IF ($this->debug==true) {echo"from<br>";		PRE($fromclause); }
        if ($groupclause !== '' && kDatabaseType === 'oci') {
		  $groupclause .= ',' . substr($ocigroup, CLX_NUM_ZERO, strpos($ocigroup, ' '));
		  if ($sortclause !== '') {
			 $sortclause .= ',' . substr($ocigroup, CLX_NUM_ZERO, strpos($ocigroup, ' '));
		  } else {
			 $sortclause = $groupclause;
		  }
        }
		
        // set max
        //$t = time('U'); genera un warning
        $t = time();
        $target = UserRequest::$server ['PHP_SELF'];
        $to = (empty(UserRequest::$request [CLX_PROP_TO])) ? $this->code : UserRequest::$request [CLX_PROP_TO];
        $ac = 'LIST';
        $pfid = (empty(UserRequest::$request ['pfid'])) ? CLX_NUM_ZERO : UserRequest::$request ['pfid'];
        $menuid = '';
        $here = $target . '?to=' . $to . '&ac=' . $ac . '&t=' . $t . '&mi=' . $menuid . '&pfid=' . $pfid;
        $callingpage = peekHistory();
        if (!empty($callingpage['relfm']) && !empty($callingpage['relfm'])) {
		  $here .= '&relfm=' . $callingpage['relfm'];
        }
        if (!empty($callingpage['parentli']) && !empty($callingpage['parentli'])) {
		  $here .= '&parentli=' . $callingpage['parentli'] . '&childli=' . $callingpage['parentli'];
        }

        // Listing must be executed after Search
        if ($listingrec ['EXEONSEARCH'] == CLX_NUM_ONE) {
		  $page->setProp('exeonsearch', true);
        }
        $page->setProp('singular', dict($listingtablerec ['RECORDNAME_SINGULAR']));
        $page->setProp('plural', dict($listingtablerec ['RECORDNAME_PLURAL']));
		IF ($this->debug==true) {echo"whereclause2<br>";		PRE($whereclause); }

       
		$page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, $groupclause,$listingcode,$fieldsAllquery,true);
		$tableWidth = count($view ['COLUMNS']) + CLX_NUM_ONE;
        $table = new htmlTable($tableWidth);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST));
        $page->header($table);
        $table->addCell(dict('GEN_LBL_SEL'), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS));
        $summary_fields = array();
        $i = CLX_NUM_ONE;
        //Incorporando campos de sumarizacion de listados
        $fieldinstances = array();
        reset($view ['COLUMNS']);
        foreach ($view ['COLUMNS'] as $key => $column) {
		  $fieldinstances[$column ['FIELDTYPE']] = getToolObject('DBFT', $column ['FIELDTYPE']);
		  $fieldinstances[$column ['FIELDTYPE']]->set_obj_cache($column);
		  if ($column ['FIELDTYPE'] === 'DECN' || $column ['FIELDTYPE'] === 'INTN' || $column ['FIELDTYPE'] === 'BOOL') {
			 if (!array_key_exists('WORKFIELD', $column) || empty($column ['WORKFIELD'])) {
			     $summary_fields [$i] = array('LISTFIELDNAME' => $key, 'FIELDNAME' => $column ['FIELDNAME'], 'TABLECODE' => substr($key, CLX_NUM_ZERO, strpos($key, '_')), 'WORKFIELD' => CLX_NUM_ZERO);
			 } else {

			 }
		  }
		  $table->addCell(dict($column ['FIELDLISTLABEL']), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS));
		  $i++;
        }
		// adding top summary
        if ($listingrec ['VIEW_SUMMARY'] == CLX_NUM_ONE || $listingrec ['VIEW_SUMMARY'] == CLX_NUM_TWO) {
		  // Build summary from sql
		  if (count($summary_fields) > CLX_NUM_ZERO) {
			 $op = '';
			 foreach ($summary_fields as $key => $val) {
			     $fields .= $op . ' SUM(' . $val ['TABLECODE'] . '.' . $val ['FIELDNAME'] . ') ' . $val ['LISTFIELDNAME'];
			     $op = ',';
			 }
			 $sql = 'SELECT ' . $fields . ' FROM ' . $page->specs ['fromclause'] . ' ' . $page->specs ['whereclause'] . ' ' . $page->specs ['groupclause'] . ' ' . $page->specs ['sortclause'];
			 $sum = ClxCommon::$db->getOneRecord($sql);
		  }
		  // Build SQL Total With total from sql
		  $i = CLX_NUM_ONE;
		  reset($view ['COLUMNS']);
		  $table->addCell(CLX_CHAR_HTML_NBSP, array(CLX_PROP_CLASS => 'view_summary'));
		  foreach ($view ['COLUMNS'] as $key => $column) {
			 if (array_key_exists($i, $summary_fields)) {
			     //$fieldinstance = getToolObject('DBFT', $column ['FIELDTYPE']);
			     if (empty($sum))
			         $sum = array();
			     //$fieldinstance->placeListField($key, $column, $sum, 'view_summary', $table);
			     $fieldinstances[$column ['FIELDTYPE']]->placeListField($key, $column, $sum, 'view_summary', $table);
			 } else { // see
			     $table->addCell(CLX_CHAR_HTML_NBSP, array(CLX_PROP_CLASS => 'view_summary'));
			 }
			 $i++;
		  }
		  unset($summary_fields, $sums, $sum, $op, $fields);
        }
        // View Summary Accumulator
        if (intval($listingrec ['VIEW_SUMMARY']) === CLX_NUM_ONE || intval($listingrec ['VIEW_SUMMARY']) === CLX_NUM_THREE) {
		  $view_summary_data = array();
		  reset($view ['COLUMNS']);
		  $formula = $listingrec ['VIEW_SUMMARY_EXP'];
		  foreach ($view ['COLUMNS'] as $key => $fieldrec) {
			 $formula = str_replace('!' . $key, '$view_summary_data[' . singleQuote($key) . '][' . singleQuote('valu') . ']', $formula);
			 $view_summary_data [$key] = array('fieldobject' => '', 'value' => '');
		  }
		  $listingrec ['VIEW_SUMMARY_EXP'] = $formula;
		  unset($formula);
        }
        // Continue with Accumulators
        $i = CLX_NUM_ONE;
		
        while ($record = $page->nextRecord()) {
		  // Executing Listing Event per record
		  $this->loadListingRecord($listingrec ['ID'], $record);
		  $recordid = $record [$listingtablecode . '_ID'];
		  $select = new htmlCheckbox('selector_' . $recordid, false);
		  $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
		  // Adding update button on record
		  if (empty($dynrel_search) && !empty($this->enable_rec_buttons)) {
			 $recbuttons = $this->addRecordButtons($qaccess, $listingrec, $listingtablerec, $querycode, $page, $form, false, $recordid);
		  } elseif (!empty($dynrel_search)) { // setting link information
			 // this is link button on record.
			 $recbuttons = '';
			 if ($qaccess->canAppend() && !empty($listingrec ['MODIFY_FORM'])) {
			     $url = $target . '?to=EDIT&ac=LINK_BUILD&li=' . $listingcode . '&qy=' . $querycode . '&relfm=' . UserRequest::$request ['relfm'] . '&idlist=' . $recordid . '&ctc=' . $tablecode;
			     $recbuttons .= getSkinIcon('modify', $url, 'Link');
			 }
		  } else {
			 $recbuttons = '';
		  }
		  // Authorization Icon
		  if ($this->has_authorization) {
			 if ($record ['CLX_AUTH_STATUS'] !== 'PEND') {
			     if ($record ['CLX_AUTH_STATUS'] === 'AP') {
			         $recbuttons .= getSkinIcon('stamp_green', '#', 'GEN_LBL_APPROVED');
			     } elseif ($record ['CLX_AUTH_STATUS'] === 'REJ') {
			         $recbuttons .= getSkinIcon('stamp_red', '#', 'GEN_LBL_REJECTED');
			     } else {
			         $recbuttons .= getSkinIcon('stamp_yellow', $target . '?to=EDIT&ac=LIST&li=AUTH&menuid=AUTH', 'AUTHTORIZATION_REQUIRED');
			     }
			 }
		  }        
		  // Listing search link
		  if ($has_srch_listing === true) {
			 $url = 'javascript:';

			 if (!empty($src_fl_copy))
			 //$url .= singleQuote($src_fm_fl_copy) . ', ' . singleQuote($this->js_add_scape_caracteres($record [$src_fl_copy])) . ');';
					if(!isset($record [$src_fm_fl_copy]))
					$record [$src_fm_fl_copy]='';
                if(!empty($src_fm_fl_copy))
			     $url .= 'setLookupValueNoClose(' . singleQuote($src_fm_fl_copy) . ', ' . singleQuote($this->js_add_scape_caracteres($record [$src_fm_fl_copy])) . ');';
			 if (!empty($src_fl_prim))
			     $url .= 'setLookupValueNoClose(' . singleQuote($src_fm_fl_prim) . ', ' . singleQuote($this->js_add_scape_caracteres($record [$src_fl_prim])) . ');';
			 if (!empty($src_fl_secd))
			     $url .= 'setLookupValueNoClose(' . singleQuote($src_fm_fl_secd) . ', ' . singleQuote($this->js_add_scape_caracteres($record [$src_fl_secd])) . ');';
			 if (!empty($src_fl_terc))
			     $url .= 'setLookupValueNoClose(' . singleQuote($src_fm_fl_terc) . ', ' . singleQuote($this->js_add_scape_caracteres($record [$src_fl_terc])) . ');';

			 $url .= 'closeWindowAndPref();';
			 $recbuttons .= getSkinIcon('modify', $url, dict('GEN_LBL_SELECT'));
		  }
		  $table->addCell($select->source() . $recbuttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, 'nowrap' => 'true'));
		  $class = ($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATAALT : CLX_CLASS_DATA;
		  reset($view ['COLUMNS']);
		  foreach ($view ['COLUMNS'] as $key => $fieldrec) {
			 // 	  pre($fieldrec);
			 $store = ''; // Key record key value to return it after ouput is
			 // generated
			 $key_original = $key; // Save de key
			 if (substr($key, strlen($key) - CLX_NUM_FOUR, CLX_NUM_FOUR) === '_WRK') {
					//comentado por ivan 24/2/2022 el contenido de una columna desaparecia y luego intercambiado de "var"_wrk a no "var"
					/*$fieldkey = substr($key, CLX_NUM_ZERO, - CLX_NUM_FOUR);
			     $store = (array_key_exists($fieldkey, $record)) ? $record [$fieldkey] : '';
			     $record [$fieldkey] = $record [$key];
					$key = $fieldkey;*/
			 } else {
			     if ($fieldrec ['FIELDTYPE'] === 'RECU') {
			         $store = '';
			     } else {
			         $store = $record [$key];
			     }
			 }
			// pre("--===------------");
			// pre($store);
			 //$fieldinstance = getToolObject('DBFT', $fieldrec ['FIELDTYPE']);
			 //$fieldinstance->placeListField($key, $fieldrec, $record, $class, $table);
				if ( $this->debug==true ) {
					$rra=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");
					fwrite($rra,chr(13).chr(10).chr(13).chr(10)."ui_listing");
					fwrite($rra,chr(13).chr(10).print_r($fieldrec,true));
					fwrite($rra, chr(13).chr(10)."--x1-".chr(13).chr(10).chr(13).chr(10).print_r($key,true));
					fwrite($rra, chr(13).chr(10)."--x2-".chr(13).chr(10).print_r($fieldrec['FIELDTYPE'],true));
					fclose($rra);
				}

				
				if (substr($key_original,-14)=='BENCHMARK_STAT') {	
					//pre($record [$key_original]);
					//echo $record ["SRRQ_IDCODE"].',', $store,"<br>";
					$semaforo='&nbsp;';
					if ($store=='01') {
						$semaforo='<img src="'.kPathToSkins . $_SESSION['skin'].'/icons/verde.png" width="18" height="18">';
					}
					else if ($store=='02') {
						$semaforo='<img src="'.kPathToSkins . $_SESSION['skin'].'/icons/amarillo.png" width="18" height="18">';
					}
					else if ($store=='03') {
						$semaforo='<img src="'.kPathToSkins . $_SESSION['skin'].'/icons/rojo.png" width="18" height="18">';
					}
					
					$table->addCell('<div align="center" >'.$semaforo.'</div>', array(CLX_PROP_CLASS => $class ,
							'style'=>"vertical-align:middle"
					));
				}	
				else {
					$fieldinstances[$fieldrec ['FIELDTYPE']]->placeListField($key, $fieldrec, $record, $class, $table, 0);
				}
			 // View Summary Accumulator
			 if ($listingrec ['VIEW_SUMMARY'] == CLX_NUM_ONE || $listingrec ['VIEW_SUMMARY'] == CLX_NUM_THREE) {
			     // Process View Accumulator
			     if ($fieldrec ['FIELDTYPE'] === 'DECN') {
			         $view_summary_data [$key_original] ['fieldobject'] = $fieldinstances[$fieldrec ['FIELDTYPE']];
			         if (!empty($bcmath))
					   $view_summary_data [$key_original] ['value'] = bcadd($view_summary_data [$key_original] ['value'], rounder($record [$key_original], $fieldrec ['FIELD_EXTRA2']));
			         else
					   $view_summary_data [$key_original] ['value'] = $view_summary_data [$key_original] ['value'] + rounder($record [$key_original], $fieldrec ['FIELD_EXTRA2']);
			     } elseif ($fieldrec ['FIELDTYPE'] === 'INTN' || $fieldrec ['FIELDTYPE'] === 'BOOL') {
			         $view_summary_data [$key_original] ['fieldobject'] = $fieldinstances[$fieldrec ['FIELDTYPE']];
			         if (!empty($bcmath))
					   $view_summary_data [$key_original] ['value'] = bcadd($view_summary_data [$key_original] ['value'], $record [$key_original], CLX_NUM_ZERO);
			         else
					   $view_summary_data [$key_original] ['value'] = $view_summary_data [$key_original] ['value'] + $record [$key_original];
			     }
			 }
			 // Return value.
			 $record [$key_original] = $store;
		  }
		  $i++;
        }    
        // Build View Summary Columns
        if ($listingrec ['VIEW_SUMMARY'] == CLX_NUM_ONE || $listingrec ['VIEW_SUMMARY'] == CLX_NUM_THREE) {
		  $table->addCell(CLX_CHAR_HTML_NBSP, array(CLX_PROP_CLASS => 'view_summary'));
		  // Evaluate Formula
		  $listingrec ['VIEW_SUMMARY_EXP'] = '';
		  foreach ($view_summary_data as $key => $view_summary_rec) {
			 if ($view_summary_rec ['value'] !== '') {
			     if ($view_summary_rec ['fieldobject']->code === 'BOOL') {
			         $table->addCell(xFormat::number($view_summary_rec ['value'], CLX_NUM_ZERO), array(CLX_PROP_CLASS => 'view_summary'));
			     } else {
			         $view_summary_rec ['fieldobject']->placeListField($key, $view ['COLUMNS'] [$key], array($key => $view_summary_rec ['value']
						  ), 'view_summary', $table);
			     }
			 } else {
			     $table->addCell(CLX_CHAR_HTML_NBSP, array(CLX_PROP_CLASS => 'view_summary'));
			 }
		  }
        }
        // remove buttons if search listing
        if ($pop_list === true) {
		  $buttons = '';
        } else {
		  $buttons = $this->addButtons($qaccess, $listingrec, $listingtablerec, $querycode, $page, $form, false);
        }
        if ($dynrel_search === true) {
		  $b = new htmlButtonReturn ();
		  $buttons .= $b->source();
        }
        $form->addContents($page->footers($table, $buttons));
        $contents .= $form->source();

        return $contents;
    }

    /**
     * Builds Listing for display forms
     */
    function buildPanel($formlistingrec, $formdata, $efftdated = true) {
        $contents = '';
        $bcmath = function_exists('bcadd');
        $UIVW = &getTool('UIVW');
        $DBQY = &getTool('DBQY');
        $EDIT = &getTool('EDIT');
        $EVNT = &getTool('EVNT');
        $UIFI = &getTool('UIFI');
        $DBTB = &getTool('DBTB');
        $listingcode = $formlistingrec ['LISTINGCODE'];
        $listingrec = ClxCommon::$db->getRecord('CLX_UI_LISTINGS', 'IDCODE=' . singleQuote($listingcode));
        if (!empty($listingrec ['HIDE_EFFTDATE'])) {
		  $efftdated = false;
        }
        if (!empty($listingrec ['TOOLTIP_FORM'])) {
		  $contents .= setJavaScriptTag(getJavaScriptFilename('UILG_dialog.js'), true);
		  $this->listing_title = str_replace('\'', '', dict($listingrec ['TITLE']));
        }
        $fields = '';
        $summary_fields = array();
        $boolean = '';
        $menuid = '';
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t . '&mi=' . $menuid;
        $target = 'tool.php';
        $querycode = $listingrec ['QUERYCODE'];
        //Init temporary table if required
        $this->init_temp_tables($querycode);
        // Listing Init Event
        $msg = array();
        $EVNT->execute5('UILG', $listingrec ['ID'], 'LINI', UserRequest::$post, $msg);
        // End Listing Init Event
        // CLX_AUTH_STATUS ADDITION
        $clx_auth_status_select = '';
        $this->has_authorization = false;
        if ($EDIT->requireAuthorization($listingrec ['TABLECODE'])) {
		  $clx_auth_status_select = ', ' . $listingrec ['TABLECODE'] . '.CLX_AUTH_STATUS';
		  $this->has_authorization = true;
        }
        // Init filter
        $qaccess = $DBQY->getAppAccess($querycode);
        if (!$qaccess->canRead()) {
		  return CLX_CHAR_EMPTY_STRING;
        } else {
		  $listingtablecode = $listingrec ['TABLECODE'];
		  $listingtablerec = $DBTB->getTable($listingtablecode);
		  $contents = '';
		  // The Listing
		  $action = (!empty(UserRequest::$server ['PHP_SELF'])) ? UserRequest::$server ['PHP_SELF'] : 'tool.php';
		  $form = new htmlForm(CLX_PROP_ACTION, $action);
		  $form->setProp(CLX_PROP_ID, $querycode . '_LIST');
		  $page = new xPaging ();
		  $page->initPagingHistory($form->getProp(CLX_PROP_ID));
		  $form->addArgument(CLX_PROP_TO, 'EDIT');
		  $form->addArgument('ac', 'NOT_DEFINED');
		  $form->addArgument('li', $listingcode);
		  $form->addArgument('qy', $querycode);
		  $form->addArgument('relfm', $formlistingrec ['RELFORMCODE']);
		  $form->addArgument('is_panel', CLX_NUM_ONE);
		  // Disable efftdate as required on listing
		  if (!empty($listingrec ['HIDE_EFFTDATE'])) {
			 $DBQY->set_efftdate_status(false);
		  } else {
			 $DBQY->set_efftdate_status(!$page->getProp('showhistory'));
		  }
			if ($_SESSION['paging'][$querycode.'_LIST']['showhistory'] != $_SESSION['paging'][$querycode.'_LIST']['oldshowhistory'] ) {
				apc_clear_querycode($querycode);
				$_SESSION['paging'][$querycode.'_LIST']['oldshowhistory']=$_SESSION['paging'][$querycode.'_LIST']['showhistory'];
			}
					
		
		  $view = $UIVW->loadView($querycode);
		  $isrelated = false;
		  $DBDR = &getTool('DBDR');
		  if ($DBDR->relationExists($formdata ['TABLECODE'], $listingrec ['TABLECODE'])) {
				$criteria = $DBDR->buildListingCriteria($formdata ['TABLECODE'], $listingrec ['TABLECODE'], $formdata [$formdata ['TABLECODE'] . '_ID'], $view, $isrelated);
				if (strlen($criteria) > CLX_NUM_ZERO) {
			     $boolean = ' AND ';
			 }
		  } else {
			 $criteria = '';
			 $boolean = '';
		  }
		  // Adding criterias
		  //$formlistingcriteria = ClxCommon::$db->getRecords('CLX_UI_FORM_LISTING_CRITS', 'FORMLISTINGID=' . singleQuote($formlistingrec ['IDCODE']));
			$formlistingcriteria = ClxCommon::$db->executequery('SELECT * FROM CLX_UI_FORM_LISTING_CRITS WHERE FORMLISTINGID=' . singleQuote($formlistingrec ['ID']));
			
		  while ($formlistingcriteriarec = ClxCommon::$db->getNextRecord($formlistingcriteria)) {
				
			 if (trim($formlistingcriteriarec ['VALUE']) !== '') {
			     $criteria .= $boolean . $UIFI->buildOperator($formlistingcriteriarec ['OPERATOR'], $formlistingcriteriarec ['FIELDNAME1'], $formlistingcriteriarec ['VALUE']);
			 } else {
			     $criteria .= $boolean . $UIFI->buildOperator($formlistingcriteriarec ['OPERATOR'], $formlistingcriteriarec ['FIELDNAME1'], $formdata[$formlistingcriteriarec ['FIELDNAME2']]);
			 }
			 $boolean = ' AND ';
		  }
		  ClxCommon::$db->freeResult($formlistingcriteria);

		  // Quick Filters
		  $contents .= $this->makeQuickFilterForm($listingrec, $querycode);
		  $selectclause = $listingtablecode . '.ID ' . $listingtablecode . '_ID';
		  if (!empty($view ['SELECTCLAUSE'])) {
//			 $selectclause .= ', ' . $view ['SELECTCLAUSE'];
			 if (strpos($view ['SELECTCLAUSE'], trim($selectclause)) !== false) {
			     $selectclause = $view ['SELECTCLAUSE'];
			 } else {
			     $selectclause .= ', ' . $view ['SELECTCLAUSE'];
			 }
		  }
		  $selectclause .= $clx_auth_status_select;
		  $fromclause = $view ['FROMCLAUSE'];
		  if (empty($view ['WHERECLAUSE'])) {
			 $whereclause = $criteria;
		  } else {
			 $whereclause = $criteria . $boolean . $view ['WHERECLAUSE'];
		  }
			//cambio para incluir los campos de la forma de base de datos en el where
			foreach ($formdata as $key=>$value) {
				$whereclause=str_replace('$record['.singlequote($key).']', $value,$whereclause); 
				$whereclause=str_replace('$record['.quote($key).']', $value,$whereclause); 
			}		
		  $sortclause = $view ['SORTCLAUSE'];
		  $groupclause = $view ['GROUPCLAUSE'];
           	if ($this->debug==true) {
			

			 echoLog('Here:' . $here . '<br>' .
			         'Select clause:' . $selectclause . '<br>' .
			         'From clause:' . $fromclause . '<br>' .
			         'Where:' . $whereclause . '<br>' .
			         'Sort:' . $sortclause . '<br>' .
			         'Group by:' . $groupclause . '<br>');
		  }
		  /*
		   * Alter to allow pagination $page->setProp('rpp', 10000);
		   */
		  // $page->setProp('printheaders', false);
		  $page->setProp('singular', dict($listingtablerec ['RECORDNAME_SINGULAR']));
		  $page->setProp('plural', dict($listingtablerec ['RECORDNAME_PLURAL']));
		  $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, $groupclause);
		  $buttons = $this->addButtons($qaccess, $listingrec, $listingtablerec, $querycode, $page, $form, $isrelated, true);
		  $hasbuttons = (strlen($buttons) > CLX_NUM_ZERO);
		  $tableWidth = count($view ['COLUMNS']) + $hasbuttons;
		  $table = new htmlTable($tableWidth);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $cpt = dict($listingrec ['CAPTION']);
		  if (strlen($cpt) > CLX_NUM_ZERO) {
			 $table->addCell($cpt, array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => $tableWidth));
		  }
		  $page->header($table);
		  if ($hasbuttons == true) {
			 $table->addCell(dict('GEN_LBL_SEL'), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS, CLX_PROP_WIDTH => '10'));
		  }
		  $fieldinstances = array();
		  $record = array();
		  $summary_fields = array();
		  reset($view ['COLUMNS']);
		  $i = CLX_NUM_ONE;
		  foreach ($view ['COLUMNS'] as $key => $column) {
			 $fieldinstances[$column ['FIELDTYPE']] = getToolObject('DBFT', $column ['FIELDTYPE']);
			 $fieldinstances[$column ['FIELDTYPE']]->set_obj_cache($column);
				IF (ISSET($GLOBALS['ADDRDEBUGUILISTING']) && $GLOBALS['ADDRDEBUGUILISTING']==$_SERVER['REMOTE_ADDR']) {
					PRE('---------------------LISTA CACHE ------------------');
					PRE($fieldinstances[$column ['FIELDTYPE']]);
				}	
			 $store = ''; // Key record key value to return it after ouput is
			 // generated
			 $key_original = $key; // Save de key
			 if (substr($key, strlen($key) - CLX_NUM_FOUR, CLX_NUM_FOUR) === '_WRK') {
			     $fieldkey = substr($key, CLX_NUM_ZERO, - CLX_NUM_FOUR);
			     $store = (array_key_exists($fieldkey, $record)) ? $record [$fieldkey] : CLX_CHAR_EMPTY_STRING;
			     $record [$fieldkey] = (array_key_exists($key, $record)) ? $record [$key] : '';
			     $key = $fieldkey;
			 } else {
			     $store = (isset($record) && is_array($record) && array_key_exists($key, $record)) ? $record [$key] : CLX_CHAR_EMPTY_STRING;
			 }
			 $table->addCell(dict($column ['FIELDLISTLABEL']), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS));
			 // Summary fields
			 if ($column ['FIELDTYPE'] === 'DECN' || $column ['FIELDTYPE'] === 'INTN' || $column ['FIELDTYPE'] === 'BOOL') {
			     if (!array_key_exists('WORKFIELD', $column))
			         $column['WORKFIELD'] = '';
			     if ($column['WORKFIELD'] === true) {
			         $summary_fields [$i] = array('LISTFIELDNAME' => $key, 'FIELDNAME' => $column ['FIELDNAME'], 'TABLECODE' => substr($key, CLX_NUM_ZERO, strpos($key, '_')), 'EXPRESSION' => str_replace($key . '_WRK', '', $column['EXPRESSION']), 'WORKFIELD' => CLX_NUM_ONE);
			     } else {
			         $summary_fields [$i] = array('LISTFIELDNAME' => $key, 'FIELDNAME' => $column ['FIELDNAME'], 'TABLECODE' => substr($key, CLX_NUM_ZERO, strpos($key, '_')), 'WORKFIELD' => CLX_NUM_ZERO);
			     }
			 }
			 // Return value.
			 $record [$key_original] = $store;
			 $i++;
		  }

		  // Build summary
		  if ($listingrec ['VIEW_SUMMARY'] == CLX_NUM_ONE || $listingrec ['VIEW_SUMMARY'] == CLX_NUM_TWO) {
			 if (count($summary_fields) > CLX_NUM_ZERO) {
			     $op = '';
			     foreach ($summary_fields as $key => $val) {
			         if ($val['WORKFIELD'] == CLX_NUM_ONE) {
					   $fields .= $op . ' SUM(' . $val ['EXPRESSION'] . ') ' . $val ['LISTFIELDNAME'];
			         } else {
					   $fields .= $op . ' SUM(' . $val ['TABLECODE'] . '.' . $val ['FIELDNAME'] . ') ' . $val ['LISTFIELDNAME'];
			         }
			         $op = ',';
			     }
			     $sql = 'SELECT ' . $fields . ' FROM ' . $page->specs ['fromclause'] . ' ' . $page->specs ['whereclause'] . ' ' . $page->specs ['groupclause'] . ' ' . $page->specs ['sortclause'];
			     $sums = ClxCommon::$db->executeQuery($sql);
			     $sum = ClxCommon::$db->getNextRecord($sums);
				 ClxCommon::$db->freeResult($sums);

			 }
			 // Build SQL Total With total from sql
			 $i = CLX_NUM_ONE;
			 reset($view ['COLUMNS']);

			 if (!empty($hasbuttons)) {
			     $table->addCell(CLX_CHAR_HTML_NBSP, array(CLX_PROP_CLASS => 'view_summary'));
			 }
			 foreach ($view ['COLUMNS'] as $key => $column) {
			     if (array_key_exists($i, $summary_fields)) {
			         //$fieldinstance = getToolObject('DBFT', $column ['FIELDTYPE']);
			         //$fieldinstance->placeListField($key, $column, $sum, 'view_summary', $table);
			         $fieldinstances[$column ['FIELDTYPE']]->placeListField($key, $column, $sum, 'view_summary', $table);
			     } else {
			         $table->addCell(CLX_CHAR_HTML_NBSP, array(CLX_PROP_CLASS => 'view_summary'));
			     }
			     $i++;
			 }
			 unset($summary_fields, $sums, $sum, $op, $fields);
		  } // If view Summary
		  // View Summary Accumulator down
		  if ($listingrec ['VIEW_SUMMARY'] == CLX_NUM_ONE || $listingrec ['VIEW_SUMMARY'] == CLX_NUM_THREE) {
			 $view_summary_data = array();
			 reset($view ['COLUMNS']);
			 $formula = $listingrec ['VIEW_SUMMARY_EXP'];
			 foreach ($view ['COLUMNS'] as $key => $fieldrec) {
			     $formula = str_replace('!' . $key, '$view_summary_data[' . singleQuote($key) . '][' . singleQuote('valu') . ']', $formula);
			     $view_summary_data [$key] = array('fieldobject' => '', 'value' => '');
			 }
			 $listingrec ['VIEW_SUMMARY_EXP'] = $formula;
			 unset($formula);
		  }

		  $i = CLX_INIT_ZERO;
		  while ($record = $page->nextRecord()) {
			 // Executing Listing Event per record
			 $this->loadListingRecord($listingrec ['ID'], $record);
			 if ($hasbuttons == true) {
			     $recordid = $record [$listingtablecode . '_ID'];
			     $select = new htmlCheckbox('selector_' . $recordid, false);
			     $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
			     if ($this->enable_rec_buttons === true) {
			         $recbuttons = $this->addRecordButtons($qaccess, $listingrec, $listingtablerec, $querycode, $page, $form, false, $recordid, $formlistingrec ['RELFORMCODE']);
			     } else {
			         $recbuttons = '';
			     }
			     // Authorization Icon
			     if ($this->has_authorization === true) {
			         if ($record ['CLX_AUTH_STATUS'] !== 'PEND') {
					   if ($record ['CLX_AUTH_STATUS'] === 'AP') {
						  $recbuttons .= getSkinIcon('stamp_green', '#', 'GEN_LBL_APPROVED');
					   } elseif ($record ['CLX_AUTH_STATUS'] === 'REJ') {
						  $recbuttons .= getSkinIcon('stamp_red', '#', 'GEN_LBL_REJECTED');
					   } else {
						  $recbuttons .= getSkinIcon('stamp_yellow', $target . '?to=EDIT&ac=LIST&li=AUTH&menuid=AUTH', 'AUTHTORIZATION_REQUIRED');
					   }
			         }
			     }
			     $table->addCell($select->source() . $recbuttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, 'nowrap' => 'true'));
			 }
			 $class = ($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATAALT : CLX_CLASS_DATA;
			 reset($view ['COLUMNS']);
			 foreach ($view ['COLUMNS'] as $key => $fieldrec) {
			     $store = ''; // Key record key value to return it after ouput is
			     // generated
			     $key_original = $key; // Save de key
			     if (substr($key, strlen($key) - CLX_NUM_FOUR, CLX_NUM_FOUR) === '_WRK') {
			         $fieldkey = substr($key, CLX_NUM_ZERO, - CLX_NUM_FOUR);
			         $store = (array_key_exists($fieldkey, $record)) ? $record [$fieldkey] : '';
			         $record [$fieldkey] = $record [$key];
			         $key = $fieldkey;
			     } else {
			         $store = (array_key_exists($key, $record)) ? $record [$key] : '';
			     }
			     //$fieldinstance = getToolObject('DBFT', $fieldrec ['FIELDTYPE']);
			     //$fieldinstance->placeListField($key, $fieldrec, $record, $class, $table);
			     $fieldinstances[$fieldrec ['FIELDTYPE']]->placeListField($key, $fieldrec, $record, $class, $table);
			     // View Summary Accumulator
			     if ($listingrec ['VIEW_SUMMARY'] == CLX_NUM_ONE || $listingrec ['VIEW_SUMMARY'] == CLX_NUM_THREE) {
			         // Process View Accumulator
			         if ($fieldrec ['FIELDTYPE'] === 'DECN') {
					   $view_summary_data [$key_original] ['fieldobject'] = $fieldinstances[$fieldrec ['FIELDTYPE']];
					   if (!empty($bcmath))
						  $view_summary_data [$key_original] ['value'] = bcadd($view_summary_data [$key_original] ['value'], rounder($record [$key_original], $fieldrec ['FIELD_EXTRA2']));
					   else
						  $view_summary_data [$key_original] ['value'] = $view_summary_data [$key_original] ['value'] + rounder($record [$key_original], $fieldrec ['FIELD_EXTRA2']);
			         } elseif ($fieldrec ['FIELDTYPE'] === 'INTN' || $fieldrec ['FIELDTYPE'] === 'BOOL') {
					   $view_summary_data [$key_original] ['fieldobject'] = $fieldinstances[$fieldrec ['FIELDTYPE']];
					   if (!empty($bcmath))
						  $view_summary_data [$key_original] ['value'] = bcadd($view_summary_data [$key_original] ['value'], $record [$key_original], CLX_NUM_ZERO);
					   else
						  $view_summary_data [$key_original] ['value'] = $view_summary_data [$key_original] ['value'] + $record [$key_original];
			         }
			     }
			     // Return value.
			     $record [$key_original] = $store;
			 }
			 $i++;
		  }

		  // Build View Summary Columns
		  if ($listingrec ['VIEW_SUMMARY'] == CLX_NUM_ONE || $listingrec ['VIEW_SUMMARY'] == CLX_NUM_THREE) {
			 if (!empty($hasbuttons)) {
			     $table->addCell(CLX_CHAR_HTML_NBSP, array(CLX_PROP_CLASS => 'view_summary'));
			 }
			 // Evaluate Formula
			 $listingrec ['VIEW_SUMMARY_EXP'] = '';
			 foreach ($view_summary_data as $key => $view_summary_rec) {
			     if ($view_summary_rec ['value'] !== '') {
			         if ($view_summary_rec ['fieldobject']->code === 'BOOL') {
					   $table->addCell(xFormat::number($view_summary_rec ['value'], CLX_NUM_ZERO), array(CLX_PROP_CLASS => 'view_summary'));
			         } else {
					   $view_summary_rec ['fieldobject']->placeListField($key, $view ['COLUMNS'] [$key], array($key => $view_summary_rec ['value']), 'view_summary', $table);
			         }
			     } else {
			         $table->addCell(CLX_CHAR_HTML_NBSP, array(CLX_PROP_CLASS => 'view_summary'));
			     }
			 }
		  }
			$form->addContents($page->footers($table, $buttons));
		  $contents .= $form->source();

		  return buildPanelInterface(dict($listingrec ['TITLE']), $querycode . '_LISTING', $contents, CLX_NUM_MINUS_1);
        }
    }

    function makeQuickFilterForm($listingrec, $querycode, $obj_uid = '') {
        if (!empty($listingrec ['SEARCH_FORM'])) {
		  $frame = new htmlTable(CLX_WIDTH_ONE);
		  $frame->setProps(array(CLX_PROP_WIDTH => '85%', 'border' => CLX_NUM_ZERO, 'cellspacing' => '5', CLX_PROP_ALIGN => 'center'));
		  $schformrec = ClxCommon::$db->getRecord('CLX_UI_FORMS', 'IDCODE=' . singleQuote($listingrec ['SEARCH_FORM']));
		  $msg = array();
		  if (UserRequest::$request['ac'] === 'QFILTER') {
			 $EDIT = &getTool('EDIT');
			 $EDIT->checkQuickSearchForm($msg, $schformrec);
		  }
		  $UIFM = &getTool('UIFM');
		  $action = (!empty(UserRequest::$server ['PHP_SELF'])) ? UserRequest::$server ['PHP_SELF'] : 'tool.php';
		  $form = new htmlForm(CLX_PROP_ACTION, $action);
		  $form->setProp(CLX_PROP_ID, 'theForm');
		  $form->addArgument(CLX_PROP_TO, 'EDIT');
		  $form->addArgument('ac', 'QFILTER');
		  $form->addArgument('qy', $querycode);
		  $form->addArgument('fm', $listingrec ['SEARCH_FORM']);
		  $form->addArgument('li', $listingrec ['IDCODE']);
		  $form->addArgument('obj_uid', $obj_uid);
		  $form->addArgument('startat', CLX_NUM_ZERO);
		  if (empty(UserRequest::$request ['pfid'])) {
			 UserRequest::$request ['pfid'] = CLX_INIT_ZERO;
		  }
		  if (empty(UserRequest::$request ['nofr']))
			 UserRequest::$request ['nofr'] = CLX_INIT_ZERO;
		  if (empty(UserRequest::$request ['pc'])) {
			 UserRequest::$request ['pc'] = '';
		  }
		  $form->addArgument('pc', UserRequest::$request ['pc']);
		  $form->addArgument('nofr', UserRequest::$request ['nofr']);
		  $form->addArgument('pfid', UserRequest::$request ['pfid']);
		  if (array_key_exists('rf', UserRequest::$request)) {
			 $form->addArgument('rf', UserRequest::$request ['rf']);
		  }
		  if (array_key_exists('rfi', UserRequest::$request)) {
			 $form->addArgument('rfi', UserRequest::$request ['rfi']);
		  }
		  if (empty($schformrec) && !empty($listingrec ['SEARCH_FORM'])) {
			 $BUGS = &getTool('BUGS');
			 $ermmsg = dict('TOOL_UILG_ERR_NOFORMCODE') . ':' . $listingrec ['SEARCH_FORM'];
			 $BUGS->report(E_USER_ERROR, $ermmsg, UserRequest::$server ['SCRIPT_NAME'], '1911', $ermmsg . ' to=UILG', CLX_NUM_ONE, '');
		  }
		  if (!empty($_SESSION ['paging'] [$querycode . '_LIST'] ['qfilter'])) {
			 $form->addContents($UIFM->userEditForm($schformrec, '', $_SESSION ['paging'] [$querycode . '_LIST'] ['qfilter'], $msg, $form));
		  } else {
			 $form->addContents($UIFM->userEditForm($schformrec, '', array(), $msg, $form));
		  }
		  $table = new htmlTable(CLX_WIDTH_ONE);
		  $table->setProps(array(CLX_PROP_CLASS => 'buttons'));
		  $btn = htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'QFILTER', 'form' => $form->getProp(CLX_PROP_ID)))->source();

		  $btn.= htmlButton::getButton('submit', dict('GEN_LBL_CLEAR_FILTER'), array('action' => 'CLEARQFILTER', 'form' => $form->getProp(CLX_PROP_ID)))->source();
		  $table->addCell($btn, array(CLX_PROP_CLASS => 'buttons'));
		  $form->addContents($table->source());
		  $frame->addCell($form->source(), array());
		  return $frame->source();
        } else {
		  return '';
        }
    }

    /**
     * Set Record buttons
     */
    function addRecordButtons(ClxAccess $access, $listingrec, $listingtablerec, $querycode, $page, $form, $isrelated, $recordid, $relformcode = '') {
        $listingcode = $listingrec ['IDCODE'];
        $buttons = '';
        if ($this->disableButtons())
		  return $buttons;
        $target = getCurrentScript();
        if ($access->canModify() && (!empty($listingrec ['MODIFY_FORM']))) {
		  $url = $target . '?to=EDIT&ac=MODIFY&li=' . $listingcode . '&qy=' . $querycode . '&relfm=' . $relformcode . '&idlist=' . $recordid;
		  $buttons .= getSkinIcon('modify', $url, 'Modify');
        }
        if ($access->canRead() && (!empty($listingrec ['DISPLAY_FORM']))) {
		  $url = $target . '?to=EDIT&ac=DISPLAY&li=' . $listingcode . '&qy=' . $querycode . '&relfm=' . $relformcode . '&idlist=' . $recordid;
		  $buttons .= getSkinIcon('view', $url, 'Display');
        }
        if ($access->canRead() && (!empty($listingrec ['TOOLTIP_FORM']))) {
		  $url = 'window.php?to=EDIT&ac=TOOLTIP&li=' . $listingcode . '&qy=' . $querycode . '&relfm=' . $relformcode . '&idlist=' . $recordid;
		  $url = 'javascript: createDialog(' . singleQuote($this->listing_title) . ', ' . singleQuote($url) . ',true)';
		  $buttons .= getSkinIcon('tooltip', $url, 'Summary');
        }
        return $buttons;
    }

    function addButtons(ClxAccess $access, $listingrec, $listingtablerec, $querycode, $page, $form, $isrelated, $is_panel = false) {
        $listingcode = $listingrec ['IDCODE'];
        $buttons = '';
        if ($this->disableButtons()) {
		  return $buttons;
        }
        $buttons = '';
        if ($access->canRead() && (!empty($listingrec ['DISPLAY_FORM']))) {
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  $buttons .= $this->addButtonPrint($listingrec, $form);
		  $buttons .= $this->addButtonQuestinary($listingrec, $form);
        }
        // Adding button to export as csv
        // $form->getProp(CLX_PROP_ID));
        if ($access->canRead() && intval($listingrec ['CONTROL_EXPORT']) === CLX_NUM_ONE) {
		  $url = 'tool.php?&to=UILG&ac=BUILDCSV&li=' . $listingrec ['IDCODE'] . '&showhistory=' . $page->getProp('showhistory');
		  //$url = 'pop.php?to=ANRJ&ac=SHOW_PANEL&idlist=' . $listingrec ['IDCODE'] ;
		  if (!empty($is_panel))
			 $url .= '&is_panel=1';
		  $b = new htmlButtonGoto(dict('GEN_LBL_EXPORT'), $url);
		  $buttons .= $b->source();
        }
        if ($access->canModify() && (!empty($listingrec ['MODIFY_FORM']))) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($access->canModify() && (!empty($listingrec ['DUPLICATE_FORM']))) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DUPLICATE'), array('action' => 'DUPLICATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ONE'))->source();
        }
        // Efftdate
        if ($access->canModify() && (!empty($listingrec ['EFFTDATE_FORM']))) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DUPLICATE_EFFTDATE'), array('action' => 'DUPLICATE_EFFTDATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ONE'))->source();
        }
        $parent_tbl = (!empty($callingpage ['parentrecord'] ['TABLECODE'])) ? $callingpage ['parentrecord'] ['TABLECODE'] : false;
        if (empty($parent_tbl)) {
		  $mnrel = CLX_INIT_ZERO;
        } else {
			$mnrel = ClxCommon::$db->oneExistsRecordsS('CLX_DB_DYNRELATIONS', 'PARENT_TABLECODE=' . singleQuote($parent_tbl) . ' AND CHILD_TABLECODE=' . singleQuote($listingrec ['TABLECODE']) . ' AND RELATION_TYPE = 2 AND ACTIVE = 1 ');
        }
        if ($access->canDelete() && !empty($listingrec ['DELETE_FORM'])) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  // Dynamic Relation
		  if (!empty($isrelated) || $mnrel > CLX_NUM_ZERO) {
			 $b = new htmlFormSubmit(dict('GEN_LBL_UNLINK'), 'UNLINK', $form->getProp(CLX_PROP_ID));
			 $b->setProp(CLX_PROP_ID, 'MULT');
			 $buttons .= $b->source();
		  }
        }
        if ($access->canAppend() && !empty($listingrec ['CREATE_FORM']) && empty($listingrec['AUTH_DISABLED'])) {
		  $b = new htmlFormSubmit(dict('GEN_LBL_CREATE_SOMETHING', dict($listingtablerec['RECORDNAME_SINGULAR'])), 'CREATE', $form->getProp(CLX_PROP_ID));
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', ''), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
		  // Dynamic Relation
		  if (!empty($isrelated) || $mnrel > CLX_NUM_ZERO) {
			 $b = new htmlFormSubmit(dict('GEN_LBL_LINK'), 'LINK', $form->getProp(CLX_PROP_ID));
			 $b->setProp(CLX_PROP_ID, 'ON');
			 $buttons .= $b->source();
		  }
        }
        if ($access->canModify() && !empty($listingrec ['CREATE_FORM']) && $this->has_authorization && empty($listingrec['AUTH_DISABLED'])) {
		  $b = new htmlFormSubmit(dict('GEN_LBL_REQAUTHORIZATION'), 'REQAUTHORIZATION', $form->getProp(CLX_PROP_ID));
		  $b->setProp(CLX_PROP_ID, 'MULT');
		  $buttons .= $b->source();
        }
        if ($access->canModify() && !empty($listingrec ['CREATE_FORM']) && $this->has_authorization && !empty($listingrec['AUTH_BTCH_FILD']) && empty($listingrec['AUTH_DISABLED'])) {
		  $b = new htmlFormSubmit(dict('GEN_LBL_REQAUTHORIZATIONBATCH'), 'REQAUTHORIZATIONBATCH', $form->getProp(CLX_PROP_ID));
		  $b->setProp(CLX_PROP_ID, 'ONE');
		  $buttons .= $b->source();
        }
        if (!$this->disableButtons()) {
		  $buttons .= $this->addCustomButtons($access, $listingcode, '', $form); // Any
        }
        $buttons .= $this->addCustomJumpListingButtons($access, $listingcode, '', $form); // Any
        if (!empty($listingrec ['CONTROL_VIEWS'])) {
		  $b = new htmlFormSubmit(dict('GEN_LBL_VIEWS'), 'VIEWS', $form->getProp(CLX_PROP_ID));
		  $buttons .= '<br>' . $b->source();
        }
        if (!empty($listingrec ['CONTROL_FILTERS'])) {
		  $UIFT = &getTool('UIFT');
		  $buttons .= $UIFT->pop($querycode, $form);
        }
        return $buttons;
    }

    function addCustomButtons(ClxAccess $access, $listingcode, $highlight, $form) {
        $buttons = '';
        $records = ClxCommon::$db->getRecords('CLX_UI_LISTING_BUTTONS', '(JUMPLISTCODE IS NULL OR JUMPLISTCODE=' . singleQuote(CLX_CHAR_EMPTY_STRING) . ') AND LISTINGCODE=' . singleQuote($listingcode) . ' ORDER BY SEQNUM');
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
		  $status = false;
		  switch ($record ['ACCESSES']) {
			 case 'canread';
			     $status = $access->canRead();
			     break;
			 case 'canmodify':
			     $status = $access->canModify();
			     break;
			 case 'candelete':
			     $status = $access->canDelete();
			     break;
			 case 'DISPLAY':
			     $status = $access->canDisplay();
			     break;
			 case 'isadmin':
			     $status = $access->isAdmin();
			     break;
			 default;
			     $status = true;
			     break;
		  }
		 

		  if (empty($record ['ACCESSES']) || ($record ['ACCESSES'] === 'NONE') || $status) {
			 $buttons .=htmlButton::getButton('submit', dict($record ['NAME']), array('action' => 'BUTTON_' . $record ['ID'], 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => $record ['HIGHLIGHT']))->source();
		  }
        }
		ClxCommon::$db->freeResult($records);
        return $buttons;
    }

    /**
     * Return false if butons are enabled
     * @global type $ac
     * @return type
     */
    private function disableButtons() {
        return (UserRequest::$request['ac'] === 'CODE' || UserRequest::$request['ac'] === 'MODIFY' || UserRequest::$request['ac'] === 'CREATE' || UserRequest::$request['ac'] === 'DUPLICATE' || UserRequest::$request['ac'] === 'DUPLICATE_EFFTDATE') ? true : false;
    }

    function addCustomJumpListingButtons($access, $listingcode, $highlight, $form) {
        // Some initializations
        $DBQY = &getTool('DBQY');
        $buttons = '';
        // search posible buttons
        $sql = 'SELECT UILB.ID, UILB.NAME, UILB.ACCESSES, UILG.QUERYCODE, UILB.HIGHLIGHT, UILB.LISTINGCODE, UILB.JUMPLISTCODE, UILG.TABLECODE FROM CLX_UI_LISTING_BUTTONS UILB, CLX_UI_LISTINGS UILG WHERE UILG.IDCODE = UILB.JUMPLISTCODE AND UILB.LISTINGCODE = ' . singleQuote($listingcode) . ' ORDER BY SEQNUM';
        $records = ClxCommon::$db->executeQuery($sql);
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {

			if ((($record ['ACCESSES'] === 'NONE' ) || $access->checkUserAccessByType($record['ACCESSES'])) &&  !empty($record['TABLECODE'])&& !empty($record['QUERYCODE'])) {
			 $sel = new htmlSelect('jm');
			 $sel->setProp(CLX_PROP_ID, $record ['HIGHLIGHT']);
			 $sel->setProp('onchange', 'javascript: setFormAction(' . singleQuote($form->getProp(CLX_PROP_ID)) . ', ' . singleQuote('JUMP_LIST') . '); this.form.submit();');
			 //Labels
			 $sel->addItem(dict($record ['NAME']), '', false);
			 $sel->addItem(str_repeat('-', strlen($record ['NAME'])), '', false);
			 // Search Listing Query
			 $sql = $DBQY->getSql($this->code, $record ['QUERYCODE']);
			 //Get Listing ID for jump menus
			 $sql = substr($sql, strpos($sql, 'FROM'), (strlen($sql) - strpos($sql, 'FROM')));
			 $sql = 'SELECT ' . $record['TABLECODE'] . '.ID, ' . $record['TABLECODE'] . '.IDNAME ' . $sql;
			 $items = ClxCommon::$db->executeQuery($sql);
			 while ($item = ClxCommon::$db->getNextRecord($items)) {
			     //Passing Listing btn id:Jlitm id as jm
			     $sel->addItem($item ['IDNAME'], $record ['ID'] . ':' . $item ['ID'], false);
			 }
			 ClxCommon::$db->freeResult($items);

			 $buttons .= $sel->source();
		  }
		  ClxCommon::$db->freeResult($records);

        }
        return $buttons;
    }

    function addButtonPrint($listingrec, $form) {
		$fetched=false;
		$button = '';
		
		if(kApcTtl_25>0) $qty = apc_fetchFct($findVar=kSpaceName . 'button_print_'.$listingrec ['TABLECODE'], $fetched);
				
		if (empty($fetched)) {
			$qty = ClxCommon::$db->oneExistsRecords('CLX_AN_OUTPUT_GEN', 'TABLECODE=' . singleQuote($listingrec ['TABLECODE']) . ' AND (ERROR IS NULL OR ERROR <> 1)');

			if (kApcTtl_25>0 ) {
				apc_addFct($findVar, $qty ,kApcTtl_25);
			}
		}
		if ($qty > CLX_NUM_ZERO) {
			$b = new htmlFormSubmit(dict('GEN_LBL_PRINT'), 'PRINT_DIALOG', $form->getProp(CLX_PROP_ID));
			$b->setProp(CLX_PROP_ID, 'MULT');
			$button = $b->source();
		}
        
        return $button;
    }

    /**
     * Build Questionary Button
     */
    function addButtonQuestinary($listingrec, $form) {
        $button = '';

		$qty = ClxCommon::$db->oneExistsRecords('CLX_AN_QUESTIONARIES', 'TABLECODE=' . singleQuote($listingrec ['TABLECODE']));
        if ($qty > CLX_NUM_ZERO) {
		  $b = new htmlFormSubmit(dict('GEN_LBL_QUESTIONARY'), 'QUESTIONARY_DIALOG', $form->getProp(CLX_PROP_ID));
		  $b->setProp(CLX_PROP_ID, 'ONE');
		  $button = $b->source();
        }
        return $button;
    }

    // Export Listing as CSV or Export Button
    var $s_sw_output_mode;
    var $o_outputfile_handle = null;
    var $s_output_filename;
    var $s_output_dir = 'export';
    var $s_filename = 'csv_export';
    var $s_extension = 'txt';
    var $n_csvex_timeout = CLX_NUM_7200;

    /**
     * Export button: RUN dialog in a pop window (cjrd )
     */
    function build_csv_start($idlist) {


        if ($this->getDebugExport()) {
		  return $this->sendListingCsv_phase2(UserRequest::$request ['li'], 'anrj');
        }

        if ($this->getAccess()->canRead()) {
		  $listingcode = UserRequest::$request ['li'];



		  $to = UserRequest::$request [CLX_PROP_TO];
		  $u = time();
		  $url = 'pop.php?u=' . $u . '&nomenu=1&noalert=1&to=' . $to . '&ac=BUILDCSV_START&li=' . $listingcode . '&' . toString($idlist);
		  $_SESSION ['ANRJ_history'] [$u] = peekHistory();
		  $_SESSION ['ANRJ_history'] [$u] ['showhistory'] = UserRequest::$request ['showhistory'];
		  $variables = array();
		  $variables ['JAVA_SCRIPT_3'] = '<meta http-equiv="refresh" content="2">';
			
			if (!defined('UTIL_VIEW_DIALOG') || UTIL_VIEW_DIALOG==1 ) {
				$variables ['CONTENTS'] = 'setTimeout("openWindow(' . singleQuote(dict('GEN_LBL_EXPORT')) . ', ' . singleQuote($url) . ', 500, 500,null,null,'.singlequote( __FILE__ .'_'. __LINE__ ) .')",1000);';
				$jscode = 'setTimeout("openWindow(' . singleQuote(CLX_CHAR_EMPTY_STRING) . ', ' . singleQuote('pop.php?to=' . $this->code . '&ac=REPORT_BUILD&ANRP_id=' . $reportid) . ', 670, 480,null,null,'.singlequote( __FILE__ .'_'. __LINE__ ) .');",1000)';

		  $contents = setJavaScriptTag($variables ['CONTENTS'], false);
			}
			else {
				
				$contents =windowsmodal(singleQuote(dict('GEN_LBL_EXPORT')),$url,1);
			}
			
			
		  $info = popHistory();
		  $contents .= executeHistory($info);
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [UI_LISTING-CAN_READ_1][BUILD_CSV_START] ' . $this->getCode(false));
        }
        return $contents;
    }

    /**
     * Calls the Analisys Report Job Tool to do a background batch processing
     */
    function FormSendListingCsv($listingcode, $s_sw_output_mode = 'var') {
	
		$s_title_dict_code = ClxCommon::$db->getField('CLX_UI_LISTINGS', 'TITLE', 'IDCODE=' . singleQuote($listingcode));
        $s_title = dict('GEN_LBL_EXPORT') . ' : ' . dict($s_title_dict_code);
			
		$formcode = 'theForm';
        $action = (!empty(UserRequest::$server ['PHP_SELF'])) ? UserRequest::$server ['PHP_SELF'] : 'tool.php';
        // Create the Form
        $form = new htmlForm(CLX_PROP_ACTION, $action );
        $form->addArgument(CLX_PROP_TO, UserRequest::$request['to']);
        $form->addArgument('ac', UserRequest::$request['ac']);
        $form->addArgument('fm', $formcode);
        $form->addArgument('li', UserRequest::$request['li']);
        $form->addArgument('u', UserRequest::$request['u']);
		$form->addArgument('nomenu', UserRequest::$request['nomenu']);
		$form->addArgument('noalert', UserRequest::$request['noalert']);
		$form->addArgument('mc', UserRequest::$request['mc']);
		$form->addArgument('relfm', UserRequest::$request['relfm']);
        $buttons = '';
		
		$table2 = new htmlTable(CLX_WIDTH_ONE);
		$table2->addCell(dict('OPTION_EXPORT'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE));
 		$table2->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));
		$table2->addSeparator("");
		$table2->addSeparator("");
		
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => 'buttons', CLX_PROP_WIDTH => CLX_PROP_100PCT));
         

		//$null=array();
		//$msg=array();
		//$f = new clx_field_list($form, $this->outTypesExport);
		
	    //$table->addData(dict('GEN_LBL_FORMAT_OUTPUT'), $f->editForm_($form, $null, $msg));

		$default=true;
		
		$codes = new HTMLSelect($this->code . '_format');
		$codes->setProp('size',3);
		foreach ($this->outTypesExport as $key=>$value) {
			$codes->addItem(dict($value), $key, $default);
			$default=false;
		}
		$table->addcell(dict('GEN_LBL_FORMAT_OUTPUT'),'');
		$table->addcell($codes->source(),'');
		$buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SEND'), array('action' => UserRequest::$request['ac'], 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => 'buttons'));
        $table2->addcell($table->source(),'');
        $btable = $table2->source();
		$form->addContents($btable);
        
		
        return $form->source();
    }		
		
		
    function sendListingCsv($listingcode, $s_sw_output_mode = 'var') {
        $s_title_dict_code = ClxCommon::$db->getField('CLX_UI_LISTINGS', 'TITLE', 'IDCODE=' . singleQuote($listingcode));
        $s_title = dict('GEN_LBL_EXPORT') . ' : ' . dict($s_title_dict_code);
        $_SESSION ['TOOL_PARAMETERS'] = array('u' => UserRequest::$request ['u'], 'li' => $listingcode, 'output_mode' => $s_sw_output_mode);
        $target = 'index.php';
        if (array_key_exists('PHP_SELF', UserRequest::$server)) {
		  $target = UserRequest::$server ['PHP_SELF'];
        }
        $UILG_format=UserRequest::$request ['UILG_format'];
		if ($UILG_format=='txt1') {$extension='txt';
		}else if ($UILG_format=='txt2') {$extension='txt';
		}else if ($UILG_format=='csv') {$extension='csv';
		}else if ($UILG_format=='json') {$extension='json';
		}else if ($UILG_format=='tsxls') {$extension='xls';
		}
			
		$url = $target . '?u=' . UserRequest::$request ['u'] . '&nomenu=1&noalert=1&to=ANRJ' . '&ac=SET_BATCH' . '&tto=' . UserRequest::$request [CLX_PROP_TO] . '&tac=BUILDCSV_BATCH' . '&tmt='.$extension . '&tid=' . toString(getIdList()) . '&tsd=now' . '&tname=' . urlencode($s_title).'&UILG_format='.UserRequest::$request ['UILG_format'];
        header('location: ' . $url);
        return '';
    }

    /**
     * Call Listing Preview
     */
    function listingPreview() {
        if(is_null($listingcode)) {
		  $id=getIdList();
		  $listingcode = ClxCommon::$db->getField($this->tablename, 'IDCODE', 'ID=' . singleQuote($id[0]));
        }
        $target = 'index.php';
        if (array_key_exists('PHP_SELF', UserRequest::$server)) {
		  $target = UserRequest::$server ['PHP_SELF'];
        }
        $menuid = '';
        if (!clx_is_empty(UserRequest::$request['menuid'])) {
		  $menuid = UserRequest::$request['menuid'];
        } elseif(!clx_is_empty($_SESSION['paging'][$this->code . 'List']['useMenuSecurity'])) {
		  $menuid = $_SESSION['paging'][$this->code . 'List']['useMenuSecurity'];
        }
        $url = $target . '?to=EDIT&ac=LIST&li=' . $listingcode . '&menuid=' . $menuid;
        header('location: ' . $url);
        return '';
    }

    function sendListingCsv_phase2($listingcode, $s_sw_output_mode = null) {

        if ($s_sw_output_mode == null)
		  $s_sw_output_mode = UserRequest::$request ['output_mode'];
        if ($s_sw_output_mode === 'anrj') {
		  // Process it through the Analisys Report Job Tool, sending the
		  // result to thr output streaming via PHP echo
			
			// 2019-01-15 CR para arreglar el export en los listados...
		  //return $this->buildListingCsv($listingcode, 'var');
		  return $this->buildListingCsv($listingcode, (isset($this->output_to_file['file_name']) ? 'file' : 'var'));
			
        } elseif ($s_sw_output_mode === 'file') {
		  // Process it generating a file into 'export' directory
		  return $this->buildListingCsv($listingcode, $s_sw_output_mode);
        } else {
			ob_clean();
		  sendDownloadFile('export', 'xml');
		  return $this->buildListingCsv($listingcode, $s_sw_output_mode);
        }
    }

    /**
     * Added to deliver listing to csv
     */
    function buildListingCsv($listingcode, $s_sw_output_mode = 'var') {
		$rra=fopen(kPathToWebSite."/".kPathToTemp."archivox1x.txt","a+");
		fwrite($rra,print_r(UserRequest::$request,true));
		fwrite($rra,print_r(UserRequest::$post,true));
		fwrite($rra,print_r(UserRequest::$get,true));
		
		fclose($rra);			
        
		$UILG_format=UserRequest::$request['UILG_format'];
        UserResponse::setRunTimeLimit($this->n_csvex_timeout);
        $this->s_sw_output_mode = $s_sw_output_mode;
        $here = '';
        $tableWitdh = CLX_NUM_ONE;
        $contents = '';
		$lbreak = "([|1|]}"; // Line Break
		$lbreakori = CLX_CHAR_NL;
		if ($UILG_format=='txt1') {
			$fbreakori=',';
		}	
		else if ($UILG_format=='txt2') {
			$fbreakori=CLX_CHAR_TAB;
		}	
		else if ($UILG_format=='csv') {
			$fbreakori = ';'; // Field Selarator
        }
		else if ($UILG_format=='json') {
			$Aexport=array();
			$Alexport=array();
			$fbreakori='';
		}
		else if ($UILG_format=='tsxls') {
			$fbreakori = CLX_CHAR_TAB; // Field Selarator
	    }

		$fbreak = "[|2|]}"; // Field Selarator
        $UIVW = &getTool('UIVW');
        $DBQY = &getTool('DBQY');
        $UIFI = &getTool('UIFI');
        $DBTB = &getTool('DBTB');
        $listingrec = ClxCommon::$db->getRecord('CLX_UI_LISTINGS', 'IDCODE=' . singleQuote($listingcode));
        if (empty($listingrec)) {
		  $listingrec = array('TABLECODE' => '', 'QUERYCODE' => '', 'TITLE' => '', 'SUBTITLE' => '', 'CAPTION' => '');
        }
        $querycode = $listingrec ['QUERYCODE'];
        //Init temporary table if required
        $this->init_temp_tables($querycode);
        $msg = array();
        $EVNT = &getTool('EVNT');
        // Get Report Information
        if (isset(UserRequest::$request ['u'])) {
		  $callingpage = (!empty($_SESSION ['ANRJ_history'] [UserRequest::$request ['u']])) ? $_SESSION ['ANRJ_history'] [UserRequest::$request ['u']] : array();
		  pushHistoryRecord($callingpage);
        } else {
		  $callingpage = array();
        }
        if (!isset($callingpage ['showhistory'])) {
		  $callingpage ['showhistory'] = CLX_INIT_ZERO;
        }
        $EVNT->execute5('UILG', $listingrec ['ID'], 'LINI', UserRequest::$post, $msg);
        $qaccess = $DBQY->getAppAccess($querycode);
        if ($qaccess->canRead()) {

		  $listingtablecode = $listingrec ['TABLECODE'];
		  $listingtablerec = $DBTB->getTable($listingtablecode);

			if ($this->debug) {
				echo 'listing code: $listingcode<br>';
				echo 'query code: $querycode<br>';
				//echo 'table code: $tablecode<br>';
				// echo 'form code: $formcode<br>';
				// echo 'form table code: $formtablecode<br>';
			}

		
		  // Xpaging class
		  $page = new xPaging ();
		  $page->initPagingHistory(CLX_PROP_ID, $querycode . '_LIST');
		  $page->setProp('showhistory', $callingpage ['showhistory']);
		  //MCH 04-12-2018 se creo parametro para indicar el maximo de registros que genera el reporte en excel//
		  //$page->specs ['rpp'] = '65530'; // top of excell
			if ($UILG_format=='xls') {
		  if(!defined('UILG_MAX_EXPORT_REG_EXCEL')) {
			 define('UILG_MAX_EXPORT_REG_EXCEL', '65530');//
		  }
			}
			else if (!defined('UILG_MAX_EXPORT_REG_EXCEL')) {
				define('UILG_MAX_EXPORT_REG_EXCEL', 999999999999);//
				
			}	
			 $page->specs ['rpp'] = UILG_MAX_EXPORT_REG_EXCEL;
		  
		  $page->setProp('plural', dict($listingtablerec ['RECORDNAME_PLURAL']));
		  // Disable efftdate as required on listing
		  if (!empty($listingrec ['HIDE_EFFTDATE'])) {
			 $DBQY->set_efftdate_status(false);
		  } else {
			 $DBQY->set_efftdate_status(!$page->getProp('showhistory'));
		  }
			if ($_SESSION['paging'][$querycode.'_LIST']['showhistory'] != $_SESSION['paging'][$querycode.'_LIST']['oldshowhistory'] ) {
				apc_clear_querycode($querycode);
				$_SESSION['paging'][$querycode.'_LIST']['oldshowhistory']=$_SESSION['paging'][$querycode.'_LIST']['showhistory'];
			}
		
		  $view = $UIVW->loadView($querycode);
			IF (($querycode=='REQUESTS_01' || $querycode=='MY_REQUESTS')) {
				$kk=fopen(kPathToWebSite.'/'.kPathToTemp.'saldebug.txt','a+');
				fwrite($kk, chr(13).chr(10).__METHOD__.' '.__FILE__.':'.__LINE__.' '. chr(13).chr(10));
				fwrite($kk,date('h:i:s').'<br>');
				fwrite($kk, chr(13).chr(10).print_r($querycode,true).' '. chr(13).chr(10). chr(13).chr(10));

				fwrite($kk, chr(13).chr(10).print_r($view,true).' '. chr(13).chr(10));			
				
				fclose($kk);
			}	
			
			$tableWidth = count($view ['COLUMNS']);
		  // Screen Title
		  if ((strlen(dict($listingrec ['TITLE'])) + strlen(dict($listingrec ['CAPTION'])) + strlen($view ['SUBTITLE'])) > CLX_NUM_ZERO) {
			 if (strlen(dict($listingrec ['TITLE'])) > CLX_NUM_ZERO) {
			     $scont=dict($listingrec ['TITLE']) . str_repeat($fbreak, $tableWitdh - CLX_NUM_ONE);
					if ($UILG_format!='json') {$contents .=$scont.$lbreak;}else{	$Aexport[dict($listingrec ['TITLE'])]=array(array($scont));}
			 }
			 if (strlen($view ['SUBTITLE']) > CLX_NUM_ZERO) {
			     $scont = $view ['SUBTITLE'] . str_repeat($fbreak, $tableWitdh - CLX_NUM_ONE) ;
					if ($UILG_format!='json') {$contents .=$scont.$lbreak;}else{	$Aexport[dict($listingrec ['TITLE'])]=array(array($scont));}
			 } elseif (strlen(dict($listingrec ['CAPTION'])) > CLX_NUM_ZERO) {
			     $scont = dict($listingrec ['CAPTION']) . str_repeat($fbreak, $tableWitdh - CLX_NUM_ONE) ;
					if ($UILG_format!='json') {$contents .=$scont.$lbreak;}else{	$Aexport[dict($listingrec ['TITLE'])]=array(array($scont));}
			 }
		  }

		  $fieldinstances = array();
		  reset($view ['COLUMNS']);
			foreach ($view ['COLUMNS'] as  $key=> $fieldrec ) {
				
			 $fieldinstances[$fieldrec ['FIELDTYPE']] = getToolObject('DBFT', $fieldrec ['FIELDTYPE']);
			 if ($fieldrec ['FIELDTYPE'] == 'DECN') {
			     $fieldinstances[$fieldrec ['FIELDTYPE']]->neg_num_parentesis = false;
			 }
			 $fieldinstances[$fieldrec ['FIELDTYPE']]->set_obj_cache($fieldrec);
		  }

		  // The Listing
		  $selectclause = $listingtablecode . '.ID ' . $listingtablecode . '_ID';
		  if (!empty($view ['SELECTCLAUSE'])) {
			 //if (stristr($view ['SELECTCLAUSE'], $selectclause)) {
			 if (strpos($view ['SELECTCLAUSE'], trim($selectclause)) !== false) {
			     $selectclause = $view ['SELECTCLAUSE'];
			 } else {
			     $selectclause .= ', ' . $view ['SELECTCLAUSE'];
			 }
		  }
			
					
						
			IF ($querycode=='REQUESTS_01' || $querycode=='MY_REQUESTS') {
				/*$kk=fopen(kPathToWebSite.'/'.kPathToTemp.'saldebug.txt','a+');
				fwrite($kk, chr(13).chr(10).__METHOD__.' '.__FILE__.':'.__LINE__.' '. chr(13).chr(10));
				fwrite($kk,date('h:i:s').'<br>');
				fwrite($kk, chr(13).chr(10).print_r($querycode,true).' '. chr(13).chr(10). chr(13).chr(10));

				fwrite($kk, chr(13).chr(10).print_r($selectclause,true).' '. chr(13).chr(10));			
				
				fclose($kk);*/
				$replace=array(
					array(array(array('SRRQ.REQSTATUS SRRQ_REQSTATUS',  							'CONCAT(DI4.VALUE, " (", SRRQ.REQSTATUS,")") SRRQ_REQSTATUS')),														'LEFT JOIN CLX_DB_LIST_ITEMS LI4 ON LI4.LISTCODE = "RQST" AND LI4.IDCODE = SRRQ.REQSTATUS  LEFT JOIN CLX_DICTIONARY DI4 ON DI4.ENTRY = LI4.IDNAME AND DI4.LANGUAGE = "es"'),
					array(array(array('SRRQ.BENCHMARK_STAT SRRQ_BENCHMARK_STAT',					'CONCAT(DI2.VALUE, " (", SRRQ.BENCHMARK_STAT,")") SRRQ_BENCHMARK_STAT')),											'LEFT JOIN CLX_DB_LIST_ITEMS LI2 ON LI2.LISTCODE = "BMST" AND LI2.IDCODE = SRRQ.BENCHMARK_STAT  LEFT JOIN CLX_DICTIONARY DI2 ON DI2.ENTRY = LI2.IDNAME AND DI2.LANGUAGE = "es"'),
					array(array(array('SRRQ.AREACODE SRRQ_AREACODE', 					  			'CONCAT(AR.IDCODE,",",AR.IDNAME, " (" ,AR.IDCODE, ")" ) SRRQ_AREACODE')),											'LEFT JOIN SERV_AREAS AR ON AR.IDCODE=SRRQ.AREACODE'),
					array(array(array('SRRQ.PRODTYPE_ATOM SRRQ_PRODTYPE_ATOM',  					'CONCAT(DI5.VALUE, " (", SRRQ.PRODTYPE_ATOM,")") SRRQ_PRODTYPE_ATOM')),												'LEFT JOIN CLX_DB_LIST_ITEMS LI5 ON LI5.LISTCODE = "SB60" AND LI5.IDCODE = SRRQ.PRODTYPE_ATOM LEFT JOIN CLX_DICTIONARY DI5 ON DI5.ENTRY = LI5.IDNAME AND DI5.LANGUAGE = "es"'),
					array(array(array('SRRQ.OWNERCODE SRRQ_OWNERCODE',					   			'CONCAT(US1.LASTNAME, ",", US1.FIRSTNAME, " (", SRRQ.OWNERCODE,")") SRRQ_OWNERCODE')),								'LEFT JOIN CLX_AC_USERS US1 ON US1.USERNAME = SRRQ.OWNERCODE'),		  		
					array(array(array('SRRQ.REQSTATUS SRRQ_REQSTATUS',	   							'CONCAT(US2.LASTNAME ",", US2.FIRSTNAME, " (", SRRQ.ASSIGNEDTO_USER,")") SRRQ_ASSIGNEDTO_USER')),					'LEFT JOIN CLX_AC_USERS US2 ON US2.USERNAME = SRRQ.ASSIGNEDTO_USER'),
					array(array(array('SRRQ.MODIFIEDBY SRRQ_MODIFIEDBY',							'CONCAT(US3.LASTNAME, ",", US3.FIRSTNAME, " (", SRRQ.MODIFIEDBY,")" ) SRRQ_MODIFIEDBY')),							'LEFT JOIN CLX_AC_USERS US3 ON US3.USERNAME = SRRQ.MODIFIEDBY'),
				

					array(array(array('SRRQ.CREATEDBY SRRQ_CREATEDBY',								'CONCAT(US4.LASTNAME, ",", US4.FIRSTNAME, " (", SRRQ.CREATEDBY,")" ) SRRQ_CREATEDBY')),								'LEFT JOIN CLX_AC_USERS US4 ON US4.USERNAME = SRRQ.CREATEDBY'),
					array(array(array('SRRQ.ATTACHMENTS_OWNER SRRQ_ATTACHMENTS_OWNER',				'CONCAT(US5.LASTNAME, ",", US5.FIRSTNAME, " (", SRRQ.ATTACHMENTS_OWNER,")") SRRQ_ATTACHMENTS_OWNER')),				'LEFT JOIN CLX_AC_USERS US5 ON US5.USERNAME = SRRQ.ATTACHMENTS_OWNER'),		  		
					array(array(array('SRRQ.ASSIGNEDTO_USER SRRQ_ASSIGNEDTO_USER',					'CONCAT(US9.LASTNAME, ",", US9.FIRSTNAME, " (", SRRQ.ASSIGNEDTO_USER,")") SRRQ_ASSIGNEDTO_USER')),					'LEFT JOIN CLX_AC_USERS US9 ON US9.USERNAME = SRRQ.ASSIGNEDTO_USER'),

					

					array(array(array('SRRQ.CONTACTID SRRQ_CONTACTID',								'CONCAT(SRRQ.CONTACTID,",", CT1.LASTNAME, ",", CT1.FIRSTNAME, " (", SRRQ.CONTACTID,")" ) SRRQ_CONTACTID')),			'LEFT JOIN CRM_CONTACTS CT1 ON CT1.IDCODE = SRRQ.CONTACTID'),
					
					array(array(array('SRRQ.SERV_REQ_SOURCE SRRQ_SERV_REQ_SOURCE',				  	'CONCAT(DI7.VALUE, " (", SRRQ.SERV_REQ_SOURCE,")") SRRQ_SERV_REQ_SOURCE')),											'LEFT JOIN CLX_DB_LIST_ITEMS LI7 ON LI7.LISTCODE = "LSRC" AND LI7.IDCODE = SRRQ.SERV_REQ_SOURCE  LEFT JOIN CLX_DICTIONARY DI7 ON DI7.ENTRY = LI7.IDNAME AND DI7.LANGUAGE = "es"'),
					
					array(array(array('SRRQ.OPEN_ACTI SRRQ_OPEN_ACTI',				  				'CONCAT(DI8.VALUE, " (", SRRQ.OPEN_ACTI,")") SRRQ_OPEN_ACTI')),														'LEFT JOIN CLX_DB_LIST_ITEMS LI8 ON LI8.LISTCODE = "BINA" AND LI8.IDCODE = SRRQ.OPEN_ACTI LEFT JOIN CLX_DICTIONARY DI8 ON DI8.ENTRY = LI8.IDNAME AND DI8.LANGUAGE = "es"'),
				
					
					array(array(array('SRRQ.PRIORITY SRRQ_PRIORITY',						   		'CONCAT(DI6.VALUE, " (", SRRQ.PRIORITY,")") SRRQ_PRIORITY')),														'LEFT JOIN CLX_DB_LIST_ITEMS LI6 ON LI6.LISTCODE = "ACPR" AND LI6.IDCODE = SRRQ.PRIORITY LEFT JOIN CLX_DICTIONARY DI6 ON DI6.ENTRY = LI6.IDNAME AND DI6.LANGUAGE = "es"'),
					array(array(array('SRRQ.COMPLOC_CODE SRRQ_COMPLOC_CODE',						'CONCAT(COLO.IDCODE,",",COLO.IDNAME, " (",COLO.IDCODE,")" ) SRRQ_COMPLOC_CODE')),									'LEFT JOIN BASE_COMPANY_LOCATIONS COLO ON COLO.IDCODE=SRRQ.COMPLOC_CODE'),
					array(array(array('SRRQ.AREA_APPLICANT SRRQ_AREA_APPLICANT',					'CONCAT(DI1.VALUE, " (", SRRQ.AREA_APPLICANT,")") SRRQ_AREA_APPLICANT')),											'LEFT JOIN CLX_DB_LIST_ITEMS LI1 ON LI1.LISTCODE = "ARSL" AND LI1.IDCODE = SRRQ.AREA_APPLICANT LEFT JOIN CLX_DICTIONARY DI1 ON DI1.ENTRY = LI1.IDNAME AND DI1.LANGUAGE = "es"'),

					array(array(array('SRRQ.ACCOUNTCODE SRRQ_ACCOUNTCODE',							'CONCAT(ACCOX.IDCODE,",",ACCOX.IDNAME, " (",ACCOX.IDCODE,")" ) SRRQ_ACCOUNTCODE')),									'LEFT JOIN CRM_ACCOUNTS ACCOX ON ACCOX.IDCODE=SRRQ.ACCOUNTCODE'),
					array(array(array('SRRQ.COMP_CODE SRRQ_COMP_CODE',								'CONCAT(COMP.IDCODE,",",COMP.IDNAME, " (",COMP.IDCODE,")" ) SRRQ_COMP_CODE')),										'LEFT JOIN BASE_COMPANIES COMP ON COMP.IDCODE=SRRQ.COMP_CODE'),
			
					array(array(//array('SRRQ.SUBJECT SRRQ_SUBJECT',			 						'SRRT1.SUBJECT SRRQ_SUBJECT'), 	
								array('SRRQ.REQTYPE SRRQ_REQTYPE',									'CONCAT(SRRT1.IDNAME, " (" ,SRRT1.IDCODE, ")" ) SRRQ_REQTYPE')),		  											'LEFT JOIN SERV_REQTYPES_TBL SRRT1 ON SRRT1.IDCODE=SRRQ.REQTYPE')
					);
					
//					SRRQ.IDCODE SRRQ_IDCODE,SRRQ.EFFTDATETIME SRRQ_EFFTDATETIME,SRRQ.REQSTATUS SRRQ_REQSTATUS,
//					SRRQ.ASSIGNEDTO_USER SRRQ_ASSIGNEDTO_USER,SRRQ.AREACODE SRRQ_AREACODE,SRRQ.SUBJECT SRRQ_SUBJECT,
//					SRRQ.REQTYPE SRRQ_REQTYPE,SRRQ.DESCRIPTION SRRQ_DESCRIPTION,SRRQ.CLIENT_NAME SRRQ_CLIENT_NAME,
//					SRRQ.IBS_CUSCUN SR////RQ_IBS_CUSCUN,SRRQ.BENCHMARK_STAT SRRQ_BENCHMARK_STAT,
//					(IF(SRRQ.REQSTATUS NOT IN ('CE', 'NP'), DATEDIFF(CURDATE(),SRRQ.EFFTDATETIME), '')) SRRQ_DURATION_DAYS_WRK,
//					(IF(SRRQ.REQSTATUS NOT IN ('CE', 'NP'), (SELECT (CAST(SRAR.BENCHMARK_RED AS SIGNED) - CAST(DATEDIFF(CURDATE(),
//					SRRQ.EFFTDATETIME) AS SIGNED) ) FROM SERV_AREAS SRAR WHERE SRRQ.AREACODE=SRAR.IDCODE AND SRAR.BENCHMARK_RED > 0 LIMIT 1 ), '') ) SRRQ_DAYS_TO_PASTDUE_WRK,S
//					SRRQ.ACTI_PEND SRRQ_ACTI_PEND,SRRQ.ACTI_TOTAL SRRQ_ACTI_TOTAL,SRRQ.MODIFIEDBY SRRQ_MODIFIEDBY,
//					SRRQ.MDATE SRRQ_MDATE,SRRQ.PRODTYPE_ATOM SRRQ_PRODTYPE_ATOM,SRRQ.COMPLOC_CODE SRRQ_COMPLOC_CODE,
//					SRRQ.OWNERCODE SRRQ_OWNERCODE,SRRQ.AREA_APPLICANT SRRQ_AREA_APPLICANT,(SRRQ.ACCOUNTCODE) SRRQ_ACCO_CD_WRK,
//					SRRQ.FAX SRRQ_FAX,SRRQ.TRANS_QTY SRRQ_TRANS_QTY,SRRQ.SERVBOX_CODE SRRQ_SERVBOX_CODE,SRRQ.EMAIL SRRQ_EMAIL,
//					SRRQ.COMPLETE_DATE SRRQ_COMPLETE_DATE

					
					$antselectclausex=$selectclause;
					$selectclausex=$selectclause;
					$joins=' ';


				foreach ($replace as $partreplace) {
					foreach($partreplace[0] as $fields) {
						//foreach($fields as $field) {
							$selectclausex=str_replace($fields[0],$fields[1],$selectclausex);
						//}
					}
					if ($selectclausex<>$antselectclausex) {
						$joins.=$partreplace[1].chr(13).chr(10);
						$antselectclausex=$selectclausex;
					}
				}
				$kk=fopen(kPathToWebSite.'/'.kPathToTemp.'saldebug.txt','a+');
				fwrite($kk,date('h:i:s').'<br>');
				fwrite($kk, chr(13).chr(10).print_r($selectclausex,true).' '. chr(13).chr(10));			
				fwrite($kk, chr(13).chr(10).print_r($joins,true).' '. chr(13).chr(10));			
				
				fclose($kk);
				$selectclause =$selectclausex;
			}
			IF (1==0 &&($querycode=='REQUESTS_01' || $querycode=='MY_REQUESTS')) {
				$kk=fopen(kPathToWebSite.'/'.kPathToTemp.'saldebug.txt','a+');
				fwrite($kk, chr(13).chr(10).__METHOD__.' '.__FILE__.':'.__LINE__.' '. chr(13).chr(10));
				fwrite($kk,date('h:i:s').'<br>');
				fwrite($kk, chr(13).chr(10).print_r($querycode,true).' '. chr(13).chr(10). chr(13).chr(10));

				fwrite($kk, chr(13).chr(10).'paso2'.' '. chr(13).chr(10));			
				
				fclose($kk);
			}
		  // Find parent information if refered to form listing
		  $criteria = '';
		  if (!empty($callingpage ['li']) && !empty($callingpage ['parentrecord'])) {
			 $sql = 'SELECT UIFI.* FROM CLX_UI_LISTINGS UILG, CLX_UI_FORMS UIFM, CLX_UI_FORM_LISTINGS UIFL, CLX_UI_FORM_LISTING_CRITS UIFI WHERE UIFM.IDCODE = UILG.DISPLAY_FORM AND UIFM.IDCODE = UIFL.FORMCODE AND UIFL.ID = UIFI.FORMLISTINGID AND UILG.IDCODE = ' . singleQuote($callingpage ['li']) . ' AND UIFL.LISTINGCODE = ' . singleQuote($listingcode);
				$rs = ClxCommon::$db->executeQuery($sql);
			 $boolean = '';
			 while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
			     if ($r ['VALUE'] !== '') {
			         $criteria .= $boolean . $UIFI->buildOperator($r ['OPERATOR'], $r ['FIELDNAME1'], $r ['VALUE']);
			     } else {
			         $criteria .= $boolean . $UIFI->buildOperator($r ['OPERATOR'], $r ['FIELDNAME1'], $callingpage ['parentrecord'] [$r ['FIELDNAME2']]);
			     }
			     $boolean = ' AND ';
			 }
			 ClxCommon::$db->freeResult($rs);

		  }
		  // end parent record
		  if ($criteria !== '') {
			 if ($view ['WHERECLAUSE'] === '') {
			     $view ['WHERECLAUSE'] .= $criteria;
			 } else {
			     $view ['WHERECLAUSE'] .= ' AND ' . $criteria;
			 }
		  }
		  $fromclause = $view ['FROMCLAUSE'];
		  $whereclause = $view ['WHERECLAUSE'];
		  $sortclause = $view ['SORTCLAUSE'];
		  $groupclause = $view ['GROUPCLAUSE'];
			IF ($querycode=='REQUESTS_01' || $querycode=='MY_REQUESTS') {
				$fromclause.=$joins;
				
			}
			IF (!($querycode=='REQUESTS_01' || $querycode=='MY_REQUESTS')) {
				$selectclause=$this->remove_unwanted_characters_csv($querycode, $selectclause); //aqui
			}
			
		  $page->pageInit($querycode . '_LIST', $here, $selectclause, $fromclause, $whereclause, $sortclause, $groupclause);
		  reset($view ['COLUMNS']);
		  $comma = '';
		  foreach ($view ['COLUMNS'] as $key => $column) {
		  	$scont = dict($column ['FIELDLISTLABEL']) ;
				if ($UILG_format!='json') {
					$contents .=$comma.$scont;
				}
				else{
					$Alexport[$key]=$scont;
				}
			 $comma = $fbreak;
		  }
			if ($UILG_format!='json') {
		  $contents .= $lbreak;
				
		  //$this->echo_contents($contents);
			// 2019-01-15 CR para arreglar el export en los listados...
				$contents=strip_tags2(str_replace($fbreak,$fbreakori,str_replace($fbreakori,' ',str_replace($lbreak,CLX_CHAR_NL,str_replace(chr(13),' ',str_replace(chr(10),' ',$contents))))));
			$this->echo_contents($contents);
			}			
			else{
				$x=count($Aexport);
				$Aexport[$x]=array($Alexport);
				$Alexport=array();
			}
		  // View Summary Accumulator
			 if (intval($listingrec ['VIEW_SUMMARY']) === CLX_NUM_ONE || intval($listingrec ['VIEW_SUMMARY']) === CLX_NUM_THREE) {
			 $view_summary_data = array();
			 reset($view ['COLUMNS']);
			 $formula = $listingrec ['VIEW_SUMMARY_EXP'];
			 foreach ($view ['COLUMNS'] as $key => $fieldrec) {
				
					$formula = str_replace('!' . $key, '$view_summary_data\[\'' . $key . '\'\]\[\'value\'\]', $formula);
                   
			     $view_summary_data [$key] = array('fieldobject' => '', 'value' => '');
			 }
			 $listingrec ['VIEW_SUMMARY_EXP'] = $formula;
			 unset($formula);
		  }

			// Continue with Accumulators
			$i = CLX_NUM_ONE;
		  $bcmath = function_exists('bcadd');
		  // Continue with Accumulators
		  $i = CLX_INIT_ZERO;
			IF (($querycode=='REQUESTS_01' || $querycode=='MY_REQUESTS')) {
				$kk=fopen(kPathToWebSite.'/'.kPathToTemp.'saldebug.txt','a+');
				fwrite($kk, chr(13).chr(10).__METHOD__.' '.__FILE__.':'.__LINE__.' '. chr(13).chr(10));
				fwrite($kk,date('h:i:s').'<br>');
				fwrite($kk, chr(13).chr(10).print_r($querycode,true).' '. chr(13).chr(10). chr(13).chr(10));

				fwrite($kk, chr(13).chr(10).print_r($page->specs['statement'],true).' '. chr(13).chr(10));			
				
				fclose($kk);
			}
		  while ($record = $page->nextRecord()) {
			 // Executing Listing Event per record
			 if ($page->specs['qty'] <= $this->listingFormatLimit) {//renew option
			     $this->loadListingRecord($listingrec ['ID'], $record);
			 }
				$class = ($i % 2 == 0) ? 'data' : 'dataalt';
				reset($view ['COLUMNS']);
			 $comma = '';
			 foreach ($view ['COLUMNS'] as $key => $fieldrec) {
			     $store = ''; // Key record key value to return it after ouput is
					// generated
			     $key_original = $key; // Save de key
			     if (substr($key, strlen($key) - CLX_NUM_FOUR, CLX_NUM_FOUR) === '_WRK') {
			         $fieldkey = substr($key, CLX_NUM_ZERO, - CLX_NUM_FOUR);
			         $store = (array_key_exists($fieldkey, $record)) ? $record [$fieldkey] : '';
			         $record [$fieldkey] = $record [$key];
			         $key = $fieldkey;
			     } else {
			         $store = $record [$key];
			     }
				
					$fieldtype=$fieldrec ['FIELDTYPE'];
					IF ($fieldtype=='FILE'|| $querycode=='REQUESTS_01' || $querycode=='MY_REQUESTS' ) {
						if ($fieldtype=="DATE") {
							if ($record[$key] == '0000-00-00'  || strlen($record[$key]) == 0) {
								$contents .= $comma. ($record[$key] == '0000-00-00')?CLX_CHAR_SINGLE_QUOTE.$key:'';
							} else {
								$contents .=  CLX_CHAR_SINGLE_QUOTE.$comma.xFormat::date($record[$key], $fieldrec['FIELD_FORMAT']);
			       }
						}
						else if ($fieldrec['FIELDTYPE']=='DATI') {
							if ($record[$key] == '0000-00-00 00:00:00' || strlen($record[$key]) == 0) {
								$contents .= $comma;
							} else {
								$contents .=  $comma.CLX_CHAR_SINGLE_QUOTE.xFormat::datetime($record[$key], $fieldrec['FIELD_FORMAT']);
							}
						}
						else if ($fieldrec['FIELDTYPE']=='BOOL') { 
							if (!isset($cambios[$key])) {
								$cambios[$key]=array(dict($fieldrec['FIELD_EXTRA1']),dict($fieldrec['FIELD_EXTRA2']));
							}
							if ($record[$key]==1)
								$contents .=  $comma.$cambios[$key][0];
							else
								$contents .=  $comma.$cambios[$key][1];
						
						}
								
						else if($fieldrec['FIELDTYPE']=='CHAR' ||$fieldrec ['FIELDTYPE']=='LOOK'|| $fieldrec ['FIELDTYPE']=='SOBJ' ||$fieldrec ['FIELDTYPE']=='LIST') {
							$contents .= $comma . (substr($record[$key],0,1)==='0' ?CLX_CHAR_SINGLE_QUOTE :''). $record[$key] ;//. ' ($key,'.$fieldrec ['FIELDTYPE'].')';
						}
						else {
							$contents .= $comma . $record[$key] ;//. ' ($key,'.$fieldrec ['FIELDTYPE'].')';
						}
					}
					else if ($page->specs['qty'] <= $this->listingFormatLimit) {
						$scont = $fieldinstances[$fieldrec ['FIELDTYPE']]->placeListFieldText($key, $fieldrec, $record) ;
						if (($fieldrec ['FIELDTYPE']=='DATI' OR $fieldrec ['FIELDTYPE']=='DATE') and substr($scont,0,1)!=CLX_CHAR_SINGLE_QUOTE)
							$scont=CLX_CHAR_SINGLE_QUOTE.$scont;
						if ($UILG_format!='json') {$contents .=$comma.$scont;}else{$Alexport[$key]=$scont;}
			     } else {
			         if ($fieldrec ['FIELDTYPE'] === 'LIST') {
							$scont = $fieldinstances[$fieldrec ['FIELDTYPE']]->placeListFieldText($key, $fieldrec, $record,1) ;
							if ($UILG_format!='json') {$contents .=$comma.$scont;}else{$Alexport[$key]=$scont;}
			         } else {
							$scont = (($fieldrec ['FIELDTYPE'] == 'LOOK' || $fieldrec ['FIELDTYPE'] == 'CHAR' && substr($record[$key],0,1)=='0') ? '`' : '') . $record[$key] ;
							if ($UILG_format!='json') {$contents .=$comma.$scont;}else{$Alexport[$key]=$scont;}
			         }
			     }
			     $comma = $fbreak;

			     //$this->echo_contents(strip_tags($contents));
			     // View Summary Accumulator
			     if ($page->specs['qty'] <= $this->listingFormatLimit && $listingrec ['VIEW_SUMMARY'] === true) {
			         // Process View Accumulator
			         if ($fieldrec ['FIELDTYPE'] === 'DECN') {
					   $view_summary_data [$key_original] ['fieldobject'] = $fieldinstances[$fieldrec ['FIELDTYPE']];
					   if (!empty($bcmath))
						  $view_summary_data [$key_original] ['value'] = bcadd($view_summary_data [$key_original] ['value'], rounder($record [$key_original], $fieldrec ['FIELD_EXTRA2']));
					   else
						  $view_summary_data [$key_original] ['value'] = $view_summary_data [$key_original] ['value'] + $record [$key_original];
			         } elseif ($fieldrec ['FIELDTYPE'] === 'INTN') {
					   $view_summary_data [$key_original] ['fieldobject'] = $fieldinstances[$fieldrec ['FIELDTYPE']];
					   if (!empty($bcmath))
						  $view_summary_data [$key_original] ['value'] = bcadd($view_summary_data [$key_original] ['value'], $record [$key_original], CLX_NUM_ZERO);
					   else
						  $view_summary_data [$key_original] ['value'] = $view_summary_data [$key_original] ['value'] + $record [$key_original];
			         }
			     }
			     // Return value.
			     $record [$key_original] = $store;
			 }
				if ($UILG_format!='json') {
			 $contents .= $lbreak;
			 // $this->echo_contents(strip_tags($contents));
				// 2019-01-15 CR para arreglar el export en los listados...
	   
					$contents=strip_tags2(str_replace($fbreak,$fbreakori,str_replace($fbreakori,' ',str_replace($lbreak,CLX_CHAR_NL,str_replace(chr(13),' ',str_replace(chr(10),' ',$contents))))));
					
			 $this->echo_contents($contents);
				}
				else {
					$Aexport[]=array($Alexport);$Alexport=array();
				}	
			 $i++;
		  }
		  // Build View Summary Columns
		  if ($page->specs['qty'] <= $this->listingFormatLimit && $listingrec ['VIEW_SUMMARY'] === true) {
			 // Evaluate Formula
			 $listingrec ['VIEW_SUMMARY_EXP'] = '';
			 $comma = '';
			 foreach ($view_summary_data as $key => $view_summary_rec) {
			     if ($view_summary_rec ['value'] !== '') {
						$scont =  $view_summary_rec ['fieldobject']->placeListFieldText($key, $view ['COLUMNS'] [$key], array($key => $view_summary_rec ['value']),1);
						if ($UILG_format!='json') {$contents .=$comma.$scont;}else{$Alexport[$key]=$scont;}
			         $comma = $fbreak;
			     } else {
			 		if ($UILG_format!='json') {$contents .=$comma;}else{$Alexport[$key]='';}
			         $comma = $fbreak;
			     }
			 }
				if ($UILG_format!='json') {
			 $contents .= $lbreak;
			 //$this->echo_contents(strip_tags($contents));
				// 2019-01-15 CR para arreglar el export en los listados...
		   
					$contents=strip_tags2(str_replace($fbreak,$fbreakori,str_replace($fbreakori,' ',str_replace($lbreak,CLX_CHAR_NL,str_replace(chr(13),' ',str_replace(chr(10),' ',$contents))))));
					
			 $this->echo_contents($contents);
			}
				else {
					$Aexport[]=array($Alexport);$Alexport=array();
				}
     		}
        } else {
			warning(dict('GEN_ERR_NO_ACCESS') . ' [UI_LISTING-CAN_READ_2][BUILDLISTINGCSV] ' . $this->getCode(false));
			if ($UILG_format!='json') {
				$contents =$scont; 
			}
			else {
				$Aexport['GEN_ERR_NO_ACCESS']=array($scont);
			}
        }
        if (isset(UserRequest::$request ['u'])) {
		  unset($_SESSION ['ANRJ_history'] [UserRequest::$request ['u']]);
        }
		if ($UILG_format!='json') {
        //$contents = strip_tags($contents);
        //$this->echo_contents($contents);
		// 2019-01-15 CR para arreglar el export en los listados...
			
			
			$contents=strip_tags2(str_replace($fbreak,$fbreakori,str_replace($fbreakori,' ',str_replace($lbreak,CLX_CHAR_NL,str_replace(chr(13),' ',str_replace(chr(10),' ',$contents))))));
			
		$this->echo_contents($contents);
					
		}
		else {
			$contents=json_encode(strip_tags2($Aexport));
			$this->echo_contents($contents);			
		}
				
        return $this->finish_contents($contents);
    }

    function echo_contents(&$contents) {
        switch ($this->s_sw_output_mode) {
		  case 'echo' :
			 echo str_replace(CLX_CHAR_HTML_NBSP, '', $contents);
			 $contents = '';
			 break;
		  case 'file' :
			 $this->create_contents_outputfile();
			 $contents = decode_string($contents);
			 $this->o_outputfile_handle->write(str_replace(CLX_CHAR_HTML_NBSP, '', $contents));
			 $contents = '';
			 break;
		  case 'var' :
			 $contents = str_replace(CLX_CHAR_HTML_NBSP, '', $contents);
			 break;
		  default :
			 break;
        }
    }

    function create_contents_outputfile() {
        if ($this->o_outputfile_handle == null) {
			// 2019-01-15 CR para arreglar el export en los listados...
			if(isset($this->output_to_file['file_name'])) {
				$this->s_output_filename = $this->output_to_file['file_name'];
			} else {
			  $this->s_output_filename = $this->s_output_dir . '/' . $this->s_filename . '_' . str_replace('.', '', microtime(true)) . '.' . $this->s_extension;
			}
		  $this->o_outputfile_handle = new clx_file($this->s_output_filename, 'w');
		  $this->o_outputfile_handle->open();
        }
    }

    function finish_contents($contents) {
        switch ($this->s_sw_output_mode) {
		  case 'echo' :
			 echo str_replace(CLX_CHAR_HTML_NBSP, '', $contents);
			 $contents = '';
			 break;
		  case 'file' :
				// 2019-01-15 CR para arreglar el export en los listados...
				if(isset($this->output_to_file['file_name'])) {
					$contents = '';
				} else {
					$contents = note('<a href="' . $this->s_output_filename . '" download target="_blank" >' . dict('GEN_LBL_DOWNLOAD_THE_FILE') . '</a><br>');
				}
			 $this->create_contents_outputfile();
			 $this->o_outputfile_handle->close();
			 break;
		  case 'var' :
			 $contents = str_replace(CLX_CHAR_HTML_NBSP, '', $contents);
			 break;
		  default :
			 break;
        }
		// 2019-01-15 CR para arreglar el export en los listados...
        //if ($this->getDebugExport()) {
        if ($this->getDebugExport() && ! isset($this->output_to_file['file_name'])) {
		  $contents = '<textarea rows="50" cols="200">' . strip_tags($contents) . '</textarea>';
        }
        return $contents;
    }

    /**
     * remplaza caracteres de escape en operacion que afecta javascript
     * esto ser reversa en lookupnoclose
     *
     * @param string $value
     * @return string
     */
    private function js_add_scape_caracteres($value) {
        $value = str_replace('\"', 'x080', $value);
        $value = str_replace('\\\'', 'x081', $value);
        $value = str_replace(CLX_CHAR_DOUBLE_QUOTE, 'x082', $value);
        $value = str_replace(CLX_CHAR_SINGLE_QUOTE, 'x083', $value);
        return $value;
    }

    /**
     * Initialices temporary tables
     *
     * @param unknown_type $querycode
     */
    private function init_temp_tables($querycode) {
        if (kDatabaseType === 'oci') {
		  return;
        }
        $DBTB = &getTool('DBTB');
        $sql = 'SELECT DISTINCT DBTB.IDCODE FROM CLX_DB_TABLES DBTB, CLX_DB_QUERY_TABLES DBQT WHERE DBQT.TABLECODE = DBTB.IDCODE AND DBQT.QUERYCODE = ' . singleQuote($querycode) . ' AND DBTB.TYPE = ' . singleQuote('TEMP');
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, NUMERIC)) {
		  $DBTB->buildTemporaryTable($r [CLX_NUM_ZERO]);
        }
		ClxCommon::$db->freeResult($rs);


    }
	//remover caracter especial del valor de campo de CSV a exportar
	function remove_unwanted_characters_csv($querycode, $val)
	{
		if ($querycode=='REQUESTS_01'){
			$val=str_replace('SRRQ.NOTES','REPLACE(SRRQ.NOTES,"<","")',$val); //aqui
			$val=str_replace('SRRQ.DESCRIPTION','REPLACE(SRRQ.DESCRIPTION,"<","")',$val); //aqui 
			$val=str_replace('SRRQ.RESOLUTION','REPLACE(SRRQ.RESOLUTION,"<","")',$val); //aqui 

		}
		return $val;
	}

}
