<?php

/**
 * Listing Buttons Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version    	$Rev: 9615 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 */
class ui_listing_buttons extends clx_tool {

    var $type_strings;
    var $access_strings;
    var $window_strings;
    var $execute_strings;

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'UILB';
        $this->name = 'Listing Buttons';
        $this->system = 'Interface';
        $this->version = '0.2.4.2';

        $this->accesskey = 'UILG';


        $this->tablename = 'CLX_UI_LISTING_BUTTONS';
        $this->record_name_singular = 'listing button';
        $this->record_name_plural = 'listing buttons';
        $this->primary_field = 'LISTINGCODE';
        $this->secondary_field = 'NAME';
        $this->seqnum_field = 'SEQNUM';
        $this->primary_key = array('LISTINGCODE', 'SEQNUM');
        $this->highlight_strings = array('ON' => 'GEN_LBL_BUTTON_ALLWAYS_ON', 'ONE' => 'GEN_LBL_BUTTON_SELECT_SINGLE', 'TWO' => 'GEN_LBL_BUTTON_SELECT_TWO', 'MULT' => 'GEN_LBL_BUTTON_SELECT_MULT');
        $this->access_strings = array('NONE' => 'GEN_LBL_UNRESTRICTED', 'canread' => 'GEN_LBL_CANREAD', 'canmodify' => 'GEN_LBL_CANMODIFY', 'candelete' => 'GEN_LBL_CANDELETE', 'DISPLAY' => 'GEN_LBL_CANAPPEND', 'isadmin' => 'GEN_LBL_ISADMIN');
        $this->window_strings = array('SAME' => 'GEN_LBL_SAME_WINDOW',
		  'POP' => 'GEN_LBL_POP_WINDOW',
		  'CUST' => 'GEN_LBL_CUSTOM_WINDOW'
        );
        $this->execute_strings = array('ONE' => 'GEN_LBL_CODE_EXEC_ONE_TIME',
		  'EACH' => 'GEN_LBL_CODE_EXEC_EACH'
        );
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['LISTINGCODE'] = array('type' => 'varchar(32)', 'null' => 'NO', 'properties' => '');
        $fields['NAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '', 'dict' => true);
        $fields['CODELIB'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['CALL'] = array('type' => 'varchar(255)', 'null' => 'NO', 'properties' => '');
        $fields['SEQNUM'] = array('type' => 'int(11) unsigned', 'properties' => '');
        $fields['HIGHLIGHT'] = array('type' => 'varchar(4)', 'properties' => '');
        $fields['EXECUTE'] = array('type' => 'varchar(4)', 'properties' => '');
        $fields['WINDOW'] = array('type' => 'varchar(4)', 'properties' => '');
        $fields['ACCESSES'] = array('type' => 'varchar(16)', 'properties' => '');
        $fields['CONTINUE'] = array('type' => 'tinyint(1) unsigned', 'properties' => '');
        $fields['JUMPLISTCODE'] = array('type' => 'varchar(16)', 'null' => 'YES', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'LISTINGCODE,SEQNUM');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'NAME');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB', 'fields' => $fields, 'indexes' => $indexes);
    }

    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        $record['LISTINGCODE'] = uppercase($record['LISTINGCODE']);
        ClxCommon::$db->createRecord($this->tablename, $record, false);
    }

    function action($ac, $idlist) {
        if ($this->getAccess()->canRead()) {
		  $contents = parent::action($ac, $idlist);
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [UI_LISTING_BUTTONS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makepanel($listingcode = null) {
        if ($this->getAccess()->canRead()) {
		  $t = time();
		  $to = $this->code;
		  $here = 'tool.php?to=' . $to . '&t=' . $t;
		  $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
		  $form->setProp(CLX_PROP_ID, 'UILBPanel');
		  $form->addArgument(CLX_PROP_TO, $this->code);
		  $form->addArgument('listingcode', $listingcode);
		  $form->addArgument('ac', 'NOT_DEFINED');
		  $selectclause = '*';
		  $fromclause = 'CLX_UI_LISTING_BUTTONS';
		  $whereclause = 'LISTINGCODE=' . singleQuote($listingcode);
		  $sortclause = 'SEQNUM';
		  $page = new xPaging();
		  $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
		  $page->setProp('printheaders', false);
		  $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
		  $DICT = getTool('DICT');
		  $table = new htmlTable(CLX_WIDTH_EIGHT);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $page->header($table);
		  $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_NAME'), dict('GEN_LBL_CALL'), dict('GEN_LBL_CODELIB'), dict('GEN_LBL_JUMPLISTCODE'), dict('GEN_LBL_ACCESS'), dict('GEN_LBL_EXECUTE'), dict('GEN_LBL_WINDOW')));
		  $i = CLX_INIT_ZERO;
		  while ($record = $page->nextRecord()) {
			 $recordid = $record['ID'];
			 $select = new htmlCheckbox('selector_' . $recordid, false);
			 $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
			 $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
			 if (!array_key_exists($record['ACCESSES'], $this->access_strings))
			     $this->access_strings[$record['ACCESSES']] = '';
			 $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($DICT->dictFastEdit($record['NAME']), $record['CALL'], $record['CODELIB'], $record['JUMPLISTCODE'], dict($this->access_strings[$record['ACCESSES']]), dict($this->execute_strings[$record['EXECUTE']]), dict($this->window_strings[$record['WINDOW']])));
			 $i++;
		  }
		  $buttons = '';
		  if ($this->getAccess()->canModify()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  }
		  if ($this->getAccess()->canDelete()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  }
		  if ($this->getAccess()->canDisplay()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_BUTTON')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
		  }
		  if (($page->getProp('qty') > CLX_NUM_ONE) && $this->getAccess()->canModify()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_REORDER'), array('action' => 'REORDER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
		  }
		  $form->addContents($page->footers($table, $buttons));
		  return buildPanelInterface(dict('TOOL_UILB_TITLE_LIST'), $listingcode . 'BUTTONS', $form->source(), CLX_NUM_MINUS_1);
        } else {
		  return '';
        }
    }

    function editForm($ac, $idlist, $record, $msg) {
        $modify = (UserRequest::$request['ac'] === 'MODIFY');
        $title = ($modify) ? dict('TOOL_UILB_TITLE_MODIFY') : dict('TOOL_UILB_TITLE_CREATE');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('UILB_id', $record['UILB_id']);
        $form->addArgument('UILB_listingcode', $record['UILB_listingcode']);
        $form->addArgument('UILB_language', 'PHP');
        $form->addArgument('idlist', toString($idlist));
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $DICT = &getTool('DICT');
        $DICT->editEntry($msg, dict('GEN_LBL_NAME') . ':', 'UILB_name', $record, $table, $form);
        reset($this->highlight_strings);
        $pop = new HTMLSelect('UILB_highlight');
        foreach ($this->highlight_strings as $key => $value) {
		  $pop->addItem(dict($value), $key, ($record['UILB_highlight'] == $key));
        }
        $table->addData(dict('GEN_LBL_HIGHLIGHT') . ':', $pop->source());
        reset($this->access_strings);
        $pop = new HTMLSelect('UILB_access');
        foreach ($this->access_strings as $key => $value) {
		  $pop->addItem(dict($value), $key, ($record['UILB_access'] == $key));
        }
        $table->addData(dict('GEN_LBL_ACCESS') . ':', $pop->source());
        $table->addText($msg, dict('GEN_LBL_CALL') . ':', 'UILB_call', $record['UILB_call'], CLX_NUM_FOUR, CLX_NUM_50);
        $CODE = &getTool('CODE');
        $scripts = $CODE->placeChooseCodeLibrary($msg, 'UILB_language', dict('GEN_LBL_LIBRARY') . ':', 'UILB_codelib', $record['UILB_codelib'], $table);
        $UILG = getTool('UILG');
        $UILG->placeChooseListing($msg, dict('GEN_LBL_JUMPLISTCODE'), 'UILB_jumplistcode', $record['UILB_jumplistcode'], $table);
        $table->addCell(dict('TOOL_UILB_EXPLAIN_EXECUTE'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_EIGHT));
        $pop = new HTMLSelect('UILB_execute');
        foreach ($this->execute_strings as $key => $value) {
		  $pop->addItem(dict($value), $key, ($record['UILB_execute'] == $key));
        }
        $table->addData(dict('GEN_LBL_EXECUTE') . ':', $pop->source());
        $table->addCell(dict('TOOL_UILB_EXPLAIN_WINDOW'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_EIGHT));
        $pop = new HTMLSelect('UILB_window');
        foreach ($this->window_strings as $key => $value) {
		  $pop->addItem(dict($value), $key, ($record['UILB_window'] == $key));
        }
        $table->addData(dict('GEN_LBL_WINDOW') . ':', $pop->source());
        $table->addCell(dict('TOOL_UILB_EXPLAIN_CONTINUE'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_EIGHT));
        $pop = new HTMLSelect('UILB_continue');
        $pop->addItem(dict('GEN_LBL_YES'), '1', ($record['UILB_continue'] === '1'));
        $pop->addItem(dict('GEN_LBL_NO'), '0', ($record['UILB_continue'] === '0'));
        $table->addData(dict('GEN_LBL_CONTINUE') . ':', $pop->source());
        $form->addContents($table->source());
        $table = new htmlTable(CLX_WIDTH_EIGHT);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons = '';
        if (!empty($modify)) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        }
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_EIGHT));
        $form->addContents($table->source());
        return $scripts . $form->source();
    }

    function checkForm(&$msg) {
        $msg = array();
        xValidate::required($msg, 'UILB_name_DICT', UserRequest::$post['UILB_name_DICT']);
        $CODE = &getTool('CODE');
        if (!$CODE->codeLibraryExists(UserRequest::$post['UILB_codelib'])) {
		  $msg['UILB_codelib'] = dict('TOOL_UILB_ERR_NOCODELIB');
        }
        if (strlen(UserRequest::$post['UILB_jumplistcode']) > CLX_NUM_ZERO) {
		  $UILG = &getTool('UILG');
		  if (!$UILG->listingExists(UserRequest::$post['UILB_jumplistcode'])) {
			 $msg['UILB_jumplistcode'] = dict('GEN_ERR_INVALID_ENTRY');
		  }
        }
        $DICT = &getTool('DICT');
        $DICT->validateEntry('UILB_name', UserRequest::$post, false, $msg);
        xValidate::required($msg, 'UILB_call', UserRequest::$post['UILB_call']);
        return count($msg) == CLX_NUM_ZERO;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_NAME') . ':', dict($record['UILB_name']));
        $table->addData(dict('GEN_LBL_CALL') . ':', $record['UILB_call']);
    }

    function reorderRecords() {
        $listingcode = UserRequest::$post['listingcode'];
        $records = ClxCommon::$db->getRecords($this->tablename, 'LISTINGCODE=' . singleQuote($listingcode) . ' ORDER BY SEQNUM');
        $list = array();
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
		  $list[$record['ID']] = dict($record['NAME']);
        }
		ClxCommon::$db->freeResult($records);
		

        return $list;
    }

    function reorderAction() {
        parent::reorderAction();
        $callingpage = peekHistory();
        $listingcode = $callingpage['listingcode'];
        $UILG = &getTool('UILG');
        $UILG->versionIncrease('idcode', $listingcode);
    }

    function loadToolRecord($recordid) {
        $DICT = &getTool('DICT');
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID= ' . $recordid);
        $record = array();
        $record['UILB_id'] = $recordid;
        $DICT->loadEntry('UILB_name', $values['NAME'], $record);
        $record['UILB_listingcode'] = $values['LISTINGCODE'];
        $record['UILB_call'] = $values['CALL'];
        $record['UILB_codelib'] = $values['CODELIB'];
        $record['UILB_seqnum'] = $values['SEQNUM'];
        $record['UILB_jumplistcode'] = $values['JUMPLISTCODE'];
        $record['UILB_highlight'] = $values['HIGHLIGHT'];
        $record['UILB_execute'] = $values['EXECUTE'];
        $record['UILB_window'] = $values['WINDOW'];
        $record['UILB_access'] = $values['ACCESSES'];
        $record['UILB_continue'] = $values['CONTINUE'];
        return $record;
    }

    function saveRecord() {
        $DICT = &getTool('DICT');
//	$listingrec = ClxCommon::$db->getRecord('CLX_UI_LISTINGS', 'IDCODE=' . singleQuote(UserRequest::$post['UILB_listingcode']));
        $record = array();
        $record['NAME'] = $DICT->saveEntry('UILB_name', UserRequest::$post);
        $record['CALL'] = clx_html_spchrs_decode(UserRequest::$post['UILB_call']);
        $record['CODELIB'] = UserRequest::$post['UILB_codelib'];
        $record['HIGHLIGHT'] = UserRequest::$post['UILB_highlight'];
        $record['EXECUTE'] = UserRequest::$post['UILB_execute'];
        $record['WINDOW'] = UserRequest::$post['UILB_window'];
        $record['ACCESSES'] = UserRequest::$post['UILB_access'];
        $record['CONTINUE'] = UserRequest::$post['UILB_continue'];
        $record['JUMPLISTCODE'] = (strlen(UserRequest::$post['UILB_jumplistcode']) == CLX_NUM_ZERO) ? 'null' : UserRequest::$post['UILB_jumplistcode'];
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['UILB_id']);
        $UILG = &getTool('UILG');
        $UILG->versionIncrease('IDCODE', UserRequest::$post['UILB_listingcode']);
    }

    function createRecord_() {
        $DICT = &getTool('DICT');
        if (empty(UserRequest::$post['UILB_access'])) {
		  UserRequest::$post['UILB_access'] = 'NONE';
        }
        $record = array();
        $record['LISTINGCODE'] = UserRequest::$post['UILB_listingcode'];
        $record['NAME'] = $DICT->createEntry('UILB_name', UserRequest::$post);
        $record['CALL'] = clx_html_spchrs_decode(UserRequest::$post['UILB_call']);
        $record['CODELIB'] = UserRequest::$post['UILB_codelib'];
        $record['HIGHLIGHT'] = UserRequest::$post['UILB_highlight'];
        $record['EXECUTE'] = UserRequest::$post['UILB_execute'];
        $record['WINDOW'] = UserRequest::$post['UILB_window'];
        $record['ACCESSES'] = UserRequest::$post['UILB_access'];
        $record['CONTINUE'] = UserRequest::$post['UILB_continue'];
        $record['JUMPLISTCODE'] = (strlen(UserRequest::$post['UILB_jumplistcode']) == CLX_NUM_ZERO) ? 'null' : UserRequest::$post['UILB_jumplistcode'];
        $record['SEQNUM'] = ClxCommon::$db->getField($this->tablename, 'SEQNUM', 'LISTINGCODE=' . singleQUote(UserRequest::$post['UILB_listingcode']) . ' ORDER BY SEQNUM DESC') + CLX_NUM_ONE;
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $UILG = &getTool('UILG');
        $UILG->versionIncrease('IDCODE', $record['LISTINGCODE']);
        return $recordid;
    }

    function deleteRecordSk($id) {
        $rec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        $UILG = &getTool('UILG');
        $UILG->versionIncrease('IDCODE', $rec['LISTINGCODE']);
    }

    function defaultRecord() {
        $callingpage = peekHistory();
        $record = array();
        $record['UILB_id'] = '0';
        $record['UILB_name'] = '';
        $record['UILB_name_DICT'] = '';
        $record['UILB_listingcode'] = (!empty($callingpage['listingcode']))?$callingpage['listingcode']:'';
        $record['UILB_call'] = '';
        $record['UILB_codelib'] = '';
        $record['UILB_seqnum'] = '0';
        $record['UILB_highlight'] = 'ON';
        $record['UILB_execute'] = 'ONE';
        $record['UILB_window'] = 'SAME';
        $record['UILB_access'] = 'NONE';
        $record['UILB_jumplistcode'] = '';
        $record['UILB_continue'] = '1';
        return $record;
    }

}
