<?php

/**
 * Pages Rows Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version      $Rev: 9615 $
 *
 * CHANGE LOG
 *  Included label
 */
class ui_page_rows extends clx_tool {

    function __construct() {
        $this->code = 'UIPR';
        $this->name = 'Page Rows';
        $this->system = 'Interface';
        $this->version = '0.2.6.0';
        $this->accesskey = 'UIPG';



        $this->tablename = 'CLX_UI_PAGE_ROWS';
        $this->record_name_singular = 'page row';
        $this->record_name_plural = 'page rows';
        $this->primary_field = 'ID';
        $this->secondary_field = 'NAME';
        $this->seqnum_field = 'SEQNUM';
        $this->yesno = array('GEN_LBL_NO', 'GEN_LBL_YES');
        $this->editform = array(CLX_CLASS_TITLE => 'TOOL_UIPR_TITLE_CREATE',
		  'panels' => array(
			 array('type' => 'normal',
			     'entries' => array(
			         array('label' => 'GEN_LBL_PAGECODE',
					   'fields' => array(
						  array('name' => 'pagecode', 'type' => 'string', 'readonly' => '1'))
			         ), array('label' => 'GEN_LBL_NAME',
					   'fields' => array(
						  array('name' => 'name', 'type' => 'dict', 'required' => '1'))
			         ), array('label' => 'GEN_LBL_ACTIVE',
					   'fields' => array(
						  array('name' => 'active', 'type' => CLX_CLASS_LIST, 'default' => '1', 'options' => $this->yesno))
			         )
			     )
			 )
		  )
        );
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['NAME'] = array('type' => 'varchar(64)', 'null' => 'YES', 'properties' => '', 'dict' => true);
        $fields['PAGECODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['SEQNUM'] = array('type' => 'tinyint(4) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['ACTIVE'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => '1', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'NAME');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'PAGECODE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
		  'fields' => $fields,
		  'indexes' => $indexes);
    }

    function pack($record) {
        $o = $this->packRecord($record);
        $id = $record['ID'];
        $objs = new XMLObject($this->code . '_OBJECTS');
        $this->packsub($objs, 'UIPO', 'SELECT * FROM CLX_UI_PAGE_OBJECTS WHERE IDROW=' . singleQuote($id));
        //Packing Dictionary
        $this->packsub($objs, 'DICT', 'SELECT DICT.* FROM CLX_DICTIONARY DICT, CLX_UI_PAGE_ROWS X WHERE DICT.ENTRY=X.NAME AND X.ID=' . singleQuote($id));
        $o->addObject($objs);
        return $o;
    }

    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null, 'id' => null), $parameters));
        list($package, $formid) = $parameters;
        $record = $this->unpackRecord($package);
        $panelid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        if ($package->hasSubObject($this->code . '_OBJECTS')) {
		  $objs = $package->getObjectXml($this->code . '_OBJECTS');
		  foreach ($objs->objects as $o) {
			 $toolcode = $o->tag;
			 $tool = getTool($toolcode);
			 $tool->unpack(array('package' => $o, 'id' => $panelid));
		  }
        }
    }

    function makepanel($groupcode = null) {
        if ($this->getAccess()->canRead()) {
		  $t = time();
		  $to = $this->code;
		  $here = 'tool.php?to=' . $to . '&t=' . $t;
		  $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
		  $form->setProp(CLX_PROP_ID, 'UIPRPanel');
		  $form->addArgument(CLX_PROP_TO, $this->code);
		  $form->addArgument('pagecode', $groupcode);
		  $form->addArgument('ac', 'NOT_DEFINED');
		  $selectclause = '*';
		  $fromclause = 'CLX_UI_PAGE_ROWS';
		  $whereclause = 'PAGECODE=' . singleQuote($groupcode);
		  $sortclause = 'SEQNUM ASC';
		  $page = new xPaging();
		  $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
		  $page->setProp('printheaders', false);
		  $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
		  $table = new htmlTable(CLX_WIDTH_FOUR);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $page->header($table);
		  $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_NAME'), dict('GEN_LBL_ACTIVE'), dict('GEN_LBL_OBJECTS')));
		  $i = CLX_INIT_ZERO;
		  while ($record = $page->nextRecord()) {
			 $recordid = $record['ID'];
			 $select = new htmlCheckbox('selector_' . $recordid, false);
			 $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
			 $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
			 $objects = ClxCommon::$db->getRecords('CLX_UI_PAGE_OBJECTS', 'IDROW=' . $recordid . ' ORDER BY SEQNUM');
			 $objlist = array();
			 while (($object = ClxCommon::$db->getNextRecord($objects))) {
			     $tool = &getTool($object['TOOLCODE']);
			     if (strlen($object['OBJECTCODE']) > CLX_NUM_ZERO) {
			         $objectrec = ClxCommon::$db->getRecord($tool->tablename, 'IDCODE=' . singleQuote($object['OBJECTCODE']));
			         $objlist[] = $objectrec['IDNAME'] . ' (' . $objectrec['IDCODE'] . ')';
			     } else {
			         $objlist[] = $tool->name;
			     }
			 }
			 ClxCommon::$db->freeResult($objects);
			
			 $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array(dict($record['NAME']), dict($this->yesno[$record['ACTIVE']]), implode('<br>', $objlist)));
			 $i++;
		  }
		  $buttons = '';
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  if ($this->getAccess()->canModify()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  }
		  if ($this->getAccess()->canDelete()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  }
		  if ($this->getAccess()->canDisplay()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_ROW')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
		  }
		  if (($page->getProp('qty') > CLX_NUM_ONE) && $this->getAccess()->canModify()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_REORDER'), array('action' => 'REORDER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
		  }
		  $form->addContents($page->footers($table, $buttons));
		  $modulecode = '';
		  return buildPanelInterface(dict('TOOL_UIPR_TITLE_LIST'), $modulecode . 'TABLES', $form->source(), CLX_NUM_MINUS_1);
        } else {
		  return '';
        }
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_NAME') . ':', dict($record['UIPR_name']));
    }

    function viewFormFill($record) {
        addHistory('idrow', $record['UIPR_id']);
        $contents = '';
        $pagerec = ClxCommon::$db->getRecord('CLX_UI_PAGES', 'IDCODE=' . singleQuote($record['UIPR_pagecode']));
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_UIPR_TITLE_VIEW') . ': ' . dict($record['UIPR_name']), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_PAGE') . ':', $pagerec['IDNAME'] . ' (' . $pagerec['IDCODE'] . ')');
        $table->addData(dict('GEN_LBL_ACTIVE') . ':', dict($this->yesno[$record['UIPR_active']]));
        $contents .= $table->source();
        $UIPO = &getTool('UIPO');
        $contents .= $UIPO->makepanel($record['UIPR_id']);
        return $contents;
    }

    function reorderRecords() {
        $records = ClxCommon::$db->getRecords('CLX_UI_PAGE_ROWS', 'PAGECODE=' . singleQuote(UserRequest::$post['pagecode']) . ' ORDER BY SEQNUM');
        $list = array();
        while (($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE))) {
		  $lbl = dict($record['NAME']);
		  if (trim($lbl) === '')
			 $lbl = '---';
		  $list[$record['ID']] = $lbl . '(' . $record['ID'] . ')';
        }
		ClxCommon::$db->freeResult($records);
		
        return $list;
    }

    function reorderAction() {
        parent::reorderAction();
        $callingpage = peekHistory();
        $pagecode = $callingpage['pagecode'];
        $UIPG = &getTool('UIPG');
        $UIPG->versionIncrease('IDCODE', $pagecode);
    }

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord('CLX_UI_PAGE_ROWS', 'ID= ' . $recordid);
        $record = array();
        $record['UIPR_id'] = $recordid;
        $DICT = &getTool('DICT');
        $DICT->loadEntry('UIPR_name', $values['NAME'], $record);
        $record['UIPR_pagecode'] = $values['PAGECODE'];
        $record['UIPR_seqnum'] = $values['SEQNUM'];
        $record['UIPR_active'] = $values['ACTIVE'];
        return $record;
    }

    function saveRecord() {
        $record = array();
        $DICT = &getTool('DICT');
        $record['NAME'] = $DICT->saveEntry('UIPR_name', UserRequest::$post);
        $record['ACTIVE'] = UserRequest::$post['UIPR_active'];
        ClxCommon::$db->updateRecord('CLX_UI_PAGE_ROWS', $record, 'ID=' . UserRequest::$post['UIPR_id']);
        $UIPG = &getTool('UIPG');
        $UIPG->versionIncrease('IDCODE', UserRequest::$post['UIPR_pagecode']);
    }

    function createRecord_() {
        $record = array();
        $DICT = &getTool('DICT');
        $record['NAME'] = $DICT->createEntry('UIPR_name', UserRequest::$post);
        $record['PAGECODE'] = UserRequest::$post['UIPR_pagecode'];
        $record['ACTIVE'] = UserRequest::$post['UIPR_active'];
        $record['SEQNUM'] = ClxCommon::$db->getField('CLX_UI_PAGE_ROWS', 'SEQNUM', 'PAGECODE=' . singleQuote(UserRequest::$post['UIPR_pagecode']) . ' ORDER BY SEQNUM DESC') + CLX_NUM_ONE;
        $recordid = ClxCommon::$db->createRecord('CLX_UI_PAGE_ROWS', $record, false);
        $UIPG = &getTool('UIPG');
        $UIPG->versionIncrease('IDCODE', UserRequest::$post['UIPR_pagecode']);
        return $recordid;
    }

    /**
     * Delete tool record
     * @param int $id
     */
    function deleteRecordSk($id) {

        $UIPO = &getTool('UIPO');
        $sql = 'SELECT ID FROM ' . $UIPO->tablename . ' WHERE IDROW = ' . $id;
        $rslts = ClxCommon::$db->executeQuery($sql);
        while ($rslt = ClxCommon::$db->getNextRecord($rslts)) {
		  $UIPO->deleteRecordSk($rslt['ID']);
        }
		ClxCommon::$db->freeResult($rslts);	
		$rowrec = ClxCommon::$db->getField('CLX_UI_PAGE_ROWS', 'PAGECODE', 'ID=' . $id);
        ClxCommon::$db->deleteRecords('CLX_UI_PAGE_ROWS', 'ID=' . $id);

        $UIPG = &getTool('UIPG');
        $UIPG->versionIncrease('IDCODE', $rowrec);
    }

    function defaultRecord() {
        $callingpage = peekHistory();
        $record = array();
        $record['UIPR_id'] = CLX_INIT_ZERO;
        $record['UIPR_active'] = CLX_NUM_ONE;
        $record['UIPR_name'] = '';
        $record['UIPR_name_DICT'] = '';
        $record['UIPR_pagecode'] = $callingpage['pagecode'];
        $record['UIPR_seqnum'] = '0';
        return $record;
    }

    function versionIncrease($fieldname, $fieldvalue) {
        $pagecode = ClxCommon::$db->getField($this->tablename, 'PAGECODE', $fieldname . '=' . singleQuote($fieldvalue));
        $UIPG = &getTool('UIPG');
        $UIPG->versionIncrease('IDCODE', $pagecode);
    }

}
